      program ismakef4
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.0    date   08-02-94 (Program extracted from the SEPRAN
c                                        program makef4 version 2.2)
c
c
c
c   copyright (c) 1994  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                         DESCRIPTION
c
c  this program reads the error messages from the input file
c  and puts them in file 4
c ********************************************************************
c
c                      KEYWORDS
c
c   create_file
c   menu
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none

c ********************************************************************
c
c                      COMMON BLOCKS
c
      include 'IScommon/csepch'
      include 'IScommon/cseprd'
      include 'IScommon/cisnas'
c ********************************************************************
c
c                      LOCAL PARAMETERS
c
      integer maxers, i, inext, ios, ifrom, ito, iref, iref95, iref96
      parameter ( maxers=3000 )
      integer istart(maxers)

c     i        General loop variable
c     ifrom    Array ISTART is written from position IFROM to ITO
c     inext    Next record number to which an error must be written
c     ios      Integer giving the value of the i/o status specifier
c     iref     SEPRAN local reference number to which the error file is coupled
c              In SEPRAN this is file 4. The actual reference number may be
c              different (irefer). This number is set by INITMD
c     iref95   The unit containing the input data file
c     iref96   The unit to contain the messages and error-messages
c              reported by this program
c     istart   In this array the starting position of the error messages
c              on the file are stored.
c              istart is stored in the records 2-(maxers+NREC4-1)/NREC4+1
c              of the direct access file
c     ito      Array ISTART is written from position IFROM to ITO
c     maxers   Maximum number of errors plus one that may be stored in the
c              error message file
c ********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISASKNXT  Read next item from the standard input file
c     ISFINISH  Stop SEPRAN
c     ISINIFIL  Open files
c     ISINITMD  Set machine-dependent quantities
c     ISWRITFL  Write to direct access file
c ********************************************************************
c
c                              I/O
c
c     Program ISMAKEF4 requires an input file consisting of records of
c     at most 80 characters (Standard card input)
c
c     The input should have the following structure:
c
c     For each error:
c
c     Error number followed by colon:
c        The error number must start in column 1
c
c     Followed by
c     Text describing the error message
c        The text must start after column 1 (So from column 2) and may not
c        exceed column 79.
c        The number of lines to be used for the error message is arbitrary
c
c
c     Error numbers must be given in the natural sequence, however,
c     gaps between errors are permitted
c
c
c     MAKEF4 produces 2 types of output:
c
c     Output to the standard output file
c
c        This output contains errors in the input file
c        and furthermore produces a list of gaps between error numbers
c
c     Output in the direct access output file
c
c        The direct access output file is a file with record length of 80 bytes
c
c        The first record contains one number: the largest error number
c        that is stored in this file
c
c        The next (maxers+NREC4-1)/NREC4 records contain the starting record
c        numbers of the error messages on the direct access file consequetively.
c        NREC4 defines the number of integers that can be stored in one record
c        of the error message file.
c        If an error is not available, the ending record number of the
c        preceding error plus one is stored.
c        In this way the number of records needed for error number i is
c        equal to istarti+1)-istart(i)
c
c        Finally the last records contain the error messages sequentially
c ********************************************************************
c
c                    ERROR MESSAGES
c
c    1    First column is not zero, but no number in input record
c    2    Error number smaller than preceding one
c    3    Error number larger than MAXMES:increase MAXMES
c    4    Error in the input file
c ********************************************************************
c
c                    PSEUDO CODE
c
c      Initialize common blocks
c      Open error message file
c      Initialise current error number and currect record number
c      While (input not exhausted)
c         Read record from the input file
c         If (first column non-empty) then
c            find error number
c            Check consistency of error number
c            If (error number has wrong value) then
c               Print error message
c               stop
c            Else if (error number has correct value) then
c               Fill information in array istart
c         else if (first column empty) then
c            write record to error file
c            raise current record number
c
c      Fill final positions in array ISTART
c      Write array ISTART to records 2-(maxers+NREC4-1)/NREC4
c      Write first record to error message file
c ======================================================================
c
c     --- SET common blocks (machine-dependent)
c
      call isinitmd
      iref = 4

c     --- Open files (error message file)

      call isinifil(-4)

c     --- Open the input file to be read

      iref95 = 95
      open ( iref95, file='error.dat' )
      iref96 = 96
      open ( iref96, file='error.out' )

c     --- Initialize parameters inext and ierror

      inext  = 2 + (maxers+nrec4-1)/nrec4
      ierror = 0
      istart(1) = inext

c     --- Read errors from the input file until the input is exhausted

200   read ( iref95, 210, iostat = ios ) cards
210   format(a80)
      if ( ios.eq.0 ) then

c     --- ios = 0, so record has been read
c         Check if the first column is blank

         if ( cards(1:1).ne.' ' ) then

c        --- cards(1:1) # blank, so number has been read
c            Interpret this number by subroutine ASKNXT

            icolum = 1
            ityprd = 3
            call isasknxt
            if ( ityprd.ne.2 ) then

c           --- Error 1:  inum=0, hence no number could be found

               write(iref96,220) ierror,(cards(i:i),i=1,79)
220            format(///' *******Error number 1 in program ISMAKEF4'///,
     +                   ' First column of error message is not blank,',
     +                   ' but no error number could be found'//
     +                   ' Preceding error number read is:', i5/
     +                   ' record read is'//1x,79a1)
               stop

            end if
            nerror = nint(realrd)
            if ( nerror.ge.maxers ) then

c           --- Error 3:  nerror >= maxers

               write(iref96,230) nerror,maxers,(cards(i:i),i=1,79)
230            format(///' *******Error number 3 in program MAKEF4'///,
     +                   ' Error number read is larger than maxers'/
     +                   ' Raise maxers'//
     +                   ' Error number read is:', i5/
     +                   ' maxers is:', i5/
     +                   ' record read is'//1x,79a1)
               stop

            else if ( nerror.le.ierror ) then

c           --- Error 2:  nerror <= ierror

               write(iref96,240) nerror,ierror,(cards(i:i),i=1,79)
240            format(///' *******Error number 2 in program MAKEF4'///,
     +                   ' Error number read is not larger than',
     +                   ' preceding error number read'//
     +                   ' Error number read is:', i5/
     +                   ' Preceding error number read is:', i5/
     +                   ' record read is'//1x,79a1)
               stop

            end if
            if ( nerror.gt.ierror+1 ) then

c           --- nerror > ierror+1, i.e. Gap found

               write(iref96,250) ierror+1, nerror-1
250            format(' Message numbers not in use:', i6, ' ...', i6 )

c              --- Fill array ISTART

               do 260 i = ierror+1, nerror-1
                  istart(i) = inext
260            continue

            end if
            ierror = nerror
            istart(ierror) = inext
         else

c        --- First column is blank, hence an error record has been found
c           Write error to file

            write ( irefer, rec = inext ) cards
            inext = inext + 1

         end if
         go to 200

      else if ( ios.lt.0 ) then

c     --- End of file reached
c         Fill rest of array ISTART and write to file 4

         istart(nerror+1) = inext
         inext = 2
         ifrom = 1
         ito   = nerror+1
         call iswritfl ( inext, ifrom, ito, iref, istart )

c        --- write first record to error messages file

         istart(1) = nerror
         ifrom = 1
         ito   = 1
         inext = 1
         call iswritfl ( inext, ifrom, ito, iref, istart )

      else

c     --- Error 4:  Error in input file

         write(iref96,300) ierror,(cards(i:i),i=1,79)
300      format(///' *******Error number 4 in program MAKEF4'///,
     +             ' Error in input file',
     +             ' Preceding error number read is:', i5/
     +             ' record read is'//1x,79a1)

      end if

      close ( iref96 )
      call isfinish(0)
      end
