      program curvduct
c=======================================================================
c
c     programmer      Kees Kassels
c
c     version  1.0    date    25-03-94
c
c   copyright (c) 1994  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c***********************************************************************
c
c                    DESCRIPTION
c
c     ISNaS Host Program
c
c***********************************************************************
c
c                   KEYWORDS
c
c     host_program
c
c***********************************************************************
c
c                   INPUT / OUTPUT PARAMETERS
c
      implicit none
c***********************************************************************
c
c                   COMMON BLOCKS
c
      integer nbuffr
      parameter( nbuffr = 24000000 )
      integer ibuffr
      common ibuffr(nbuffr)
c
c     IBUFFR    Integer buffer array in which all array information
c               with respect to the ISNaS software is stored.
c
c     NBUFFR    Declared length of IBUFFR
c
c
c***********************************************************************
c
c                   LOCAL PARAMETERS
c
c
c***********************************************************************
c
c                   SUBROUTINES CALLED
c
c     ISHMAIN    ISNaS host main subroutine
c
c***********************************************************************
c
c                   I/O
c
c***********************************************************************
c
c                   ERROR MESSAGES
c
c***********************************************************************
c
c                   PSEUDO CODE
c
c          initialize local buffer
c          call ISNaS host main subroutine
c
c=======================================================================

      call ismain( nbuffr )

      end
      function usfunb ( ichoice, x, y, z, t )

c     User written function subroutine. It gives
c     the user the opportunity to define a
c     boundary condition as a function of space
c     and time.

      implicit none      

      double precision usfunb, x, y, z, t
      integer ichoice

c     x        i    x-coordinate 
c     y        i    y-coordinate 
c     t        i    actual time 
c     ichoice  i    choice parameter given by the user input
c     usfunb   o    computed boundary condition

      logical ready
      integer i, j, nz, ny
      double precision H, U0, kappa, cmu, co(2,34), ar(3,34,28),
     +                 z1, z2, x1, x2, u1, u2, u3, u4,
     +                 v1, v2, v3, v4, w1, w2, w3, w4,
     +                 k1, k2, k3, k4, t1, t2, u, v, w, k, eps, 
     +                 omega, lscale, uu, vv, ww, dlta

      open (11, file='mu1in2.dat')
      open (12, file='tu1in2.dat')
      open (13, file='mu1ou2.dat')
      open (14, file='tu1ou2.dat')
      open (15, file='mu1in1.dat')
      open (16, file='tu1in1.dat')
      open (17, file='mu1up1.dat')
      open (18, file='tu1up1.dat')
      open (19, file='mu1ou1.dat')
      open (20, file='tu1ou1.dat')

      dlta = 1d-5
      H = 0.203d0
      U0 = 16d0
      kappa = 4.1d-1
      cmu = 9d-2

      if ( x.le.5d-1*H .and. z.le.2d0*H ) then

c       --- Inlezen domein IN2

c       --- Cartesische snelheden

        nz =  5
        ny = 17

        do j = 1, nz
          co(1,j) = 0d0
          do i = 1, ny+1
            co(2,i)   = 0d0
            ar(1,i,j) = 0d0
            ar(2,i,j) = 0d0
            ar(3,i,j) = 0d0
          end do
        end do

        do j = 1, nz
          read(11,*) co(1,j)
          do i = 2, ny+1
            read(11,*) co(2,i), ar(1,i,j), ar(2,i,j), ar(3,i,j)
          end do
        end do

        ready = .false.
        j = 1
        do while ( .not. ready .and. j .le. nz-1 )

          z1 = (3d0 - co(1,j))*H+dlta
          z2 = (3d0 - co(1,j+1))*H-dlta

          if ( z .ge. z2 .and. z .le. z1 ) then

            i = 1
            do while ( .not. ready .and. i .le. ny )

              x1 = co(2,i)*H-dlta
              x2 = co(2,i+1)*H+dlta

              u1 = ar(1,i,j)*U0
              u2 = ar(1,i+1,j)*U0
              u3 = ar(1,i,j+1)*U0
              u4 = ar(1,i+1,j+1)*U0

              v1 = ar(2,i,j)*U0
              v2 = ar(2,i+1,j)*U0
              v3 = ar(2,i,j+1)*U0
              v4 = ar(2,i+1,j+1)*U0

              w1 = ar(3,i,j)*U0
              w2 = ar(3,i+1,j)*U0
              w3 = ar(3,i,j+1)*U0
              w4 = ar(3,i+1,j+1)*U0

              if ( x .ge. x1 .and. x .le. x2 ) then

                t1 = (x1 - x)/(x1 - x2)
                t2 = (z1 - z)/(z1 - z2)

                u = (1 - t2)*(1 - t1)*u1 + (1 - t2)*t1*u2 +
     +              t2*(1 - t1)*u3 + t2*t1*u4

                v = (1 - t2)*(1 - t1)*v1 + (1 - t2)*t1*v2 +
     +              t2*(1 - t1)*v3 + t2*t1*v4

                w = (1 - t2)*(1 - t1)*w1 + (1 - t2)*t1*w2 +
     +              t2*(1 - t1)*w3 + t2*t1*w4

                ready = .true.

              end if

              i = i + 1

            end do

          end if

          j = j + 1

        end do

c       --- Reynolds stresses;  to deduce turbulent energy

        nz =  5
        ny = 15

        do j = 1, nz
          co(1,j) = 0d0
          do i = 1, ny+1
            co(2,i)   = 0d0
            ar(1,i,j) = 1d-5
          end do
        end do

        do j = 1, nz
          read(12,*) co(1,j)
          do i = 2, ny+1
            read(12,*) co(2,i), uu, vv, ww
            ar(1,i,j) = 5d-1*(uu+vv+ww)
          end do
        end do

        ready = .false.
        j = 1
        do while ( .not. ready .and. j .le. nz-1 )

          z1 = (3d0 - co(1,j))*H+dlta
          z2 = (3d0 - co(1,j+1))*H-dlta

          if ( z .ge. z2 .and. z .le. z1 ) then

            i = 1
            do while ( .not. ready .and. i .le. ny )

              x1 = co(2,i)*H-dlta
              x2 = co(2,i+1)*H+dlta

              k1 = ar(1,i,j)*U0*U0
              k2 = ar(1,i+1,j)*U0*U0
              k3 = ar(1,i,j+1)*U0*U0
              k4 = ar(1,i+1,j+1)*U0*U0

              if ( x .ge. x1 .and. x .le. x2 ) then

                t1 = (x1 - x)/(x1 - x2)
                t2 = (z1 - z)/(z1 - z2)

                k = (1 - t2)*(1 - t1)*k1 + (1 - t2)*t1*k2 +
     +              t2*(1 - t1)*k3 + t2*t1*k4

                ready = .true.

              end if

              i = i + 1

            end do

          end if

          j = j + 1

        end do

      else if ( x.ge.5d-1*H .and. z.le.2d0*H ) then

c       --- Inlezen domein OU2

c       --- Cartesische snelheden

        nz =  5
        ny = 17

        do j = 1, nz
          co(1,j) = 0d0
          do i = 1, ny+1
            co(2,i)   = 0d0
            ar(1,i,j) = 0d0
            ar(2,i,j) = 0d0
            ar(3,i,j) = 0d0
          end do
        end do

        do j = 1, nz
          read(13,*) co(1,j)
          do i = 2, ny+1
            read(13,*) co(2,i), ar(1,i,j), ar(2,i,j), ar(3,i,j)
          end do
        end do

        co(2,1) = 1d0

        ready = .false.
        j = 1
        do while ( .not. ready .and. j .le. nz-1 )

          z1 = (3d0 - co(1,j))*H+dlta
          z2 = (3d0 - co(1,j+1))*H-dlta

          if ( z .ge. z2 .and. z .le. z1 ) then

            i = 1
            do while ( .not. ready .and. i .le. ny )

              x1 = co(2,i)*H+dlta
              x2 = co(2,i+1)*H-dlta

              u1 = ar(1,i,j)*U0
              u2 = ar(1,i+1,j)*U0
              u3 = ar(1,i,j+1)*U0
              u4 = ar(1,i+1,j+1)*U0

              v1 = ar(2,i,j)*U0
              v2 = ar(2,i+1,j)*U0
              v3 = ar(2,i,j+1)*U0
              v4 = ar(2,i+1,j+1)*U0

              w1 = ar(3,i,j)*U0
              w2 = ar(3,i+1,j)*U0
              w3 = ar(3,i,j+1)*U0
              w4 = ar(3,i+1,j+1)*U0

              if ( x .ge. x2 .and. x .le. x1 ) then

                t1 = (x1 - x)/(x1 - x2)
                t2 = (z1 - z)/(z1 - z2)

                u = (1 - t2)*(1 - t1)*u1 + (1 - t2)*t1*u2 +
     +              t2*(1 - t1)*u3 + t2*t1*u4

                v = (1 - t2)*(1 - t1)*v1 + (1 - t2)*t1*v2 +
     +              t2*(1 - t1)*v3 + t2*t1*v4

                w = (1 - t2)*(1 - t1)*w1 + (1 - t2)*t1*w2 +
     +              t2*(1 - t1)*w3 + t2*t1*w4

                ready = .true.

              end if

              i = i + 1

            end do

          end if

          j = j + 1

        end do

c       --- Reynolds stresses;  to deduce turbulent energy

        nz =  5
        ny = 15

        do j = 1, nz
          co(1,j) = 0d0
          do i = 1, ny+1
            co(2,i)   = 0d0
            ar(1,i,j) = 1d-5
          end do
        end do

        do j = 1, nz
          read(14,*) co(1,j)
          do i = 2, ny+1
            read(14,*) co(2,i), uu, vv, ww
            ar(1,i,j) = 5d-1*(uu+vv+ww)
          end do
        end do

        co(2,1) = 1d0

        ready = .false.
        j = 1
        do while ( .not. ready .and. j .le. nz-1 )

          z1 = (3d0 - co(1,j))*H+dlta
          z2 = (3d0 - co(1,j+1))*H-dlta

          if ( z .ge. z2 .and. z .le. z1 ) then

            i = 1
            do while ( .not. ready .and. i .le. ny )

              x1 = co(2,i)*H+dlta
              x2 = co(2,i+1)*H-dlta

              k1 = ar(1,i,j)*U0*U0
              k2 = ar(1,i+1,j)*U0*U0
              k3 = ar(1,i,j+1)*U0*U0
              k4 = ar(1,i+1,j+1)*U0*U0

              if ( x .ge. x2 .and. x .le. x1 ) then

                t1 = (x1 - x)/(x1 - x2)
                t2 = (z1 - z)/(z1 - z2)

                k = (1 - t2)*(1 - t1)*k1 + (1 - t2)*t1*k2 +
     +              t2*(1 - t1)*k3 + t2*t1*k4

                ready = .true.

              end if

              i = i + 1

            end do

          end if

          j = j + 1

        end do

      else if ( x.le.125d-3*H .and. z.ge.2d0*H ) then

c       --- Inlezen domein IN1

c       --- Cartesische snelheden

        nz =  8
        ny = 13

        do j = 1, nz+1
          co(1,j) = 0d0
          do i = 1, ny+1
            co(2,i)   = 0d0
            ar(1,i,j) = 0d0
            ar(2,i,j) = 0d0
            ar(3,i,j) = 0d0
          end do
        end do

        do j = 2, nz+1
          read(15,*) co(1,j)
          do i = 2, ny+1
            read(15,*) co(2,i), ar(1,i,j), ar(2,i,j), ar(3,i,j)
          end do
        end do

        ready = .false.
        j = 1
        do while ( .not. ready .and. j .le. nz )

          z1 = (3d0 - co(1,j))*H+dlta
          z2 = (3d0 - co(1,j+1))*H-dlta

          if ( z .ge. z2 .and. z .le. z1 ) then

            i = 1
            do while ( .not. ready .and. i .le. ny )

              x1 = co(2,i)*H-dlta
              x2 = co(2,i+1)*H+dlta

              u1 = ar(1,i,j)*U0
              u2 = ar(1,i+1,j)*U0
              u3 = ar(1,i,j+1)*U0
              u4 = ar(1,i+1,j+1)*U0

              v1 = ar(2,i,j)*U0
              v2 = ar(2,i+1,j)*U0
              v3 = ar(2,i,j+1)*U0
              v4 = ar(2,i+1,j+1)*U0

              w1 = ar(3,i,j)*U0
              w2 = ar(3,i+1,j)*U0
              w3 = ar(3,i,j+1)*U0
              w4 = ar(3,i+1,j+1)*U0

              if ( x .ge. x1 .and. x .le. x2 ) then

                t1 = (x1 - x)/(x1 - x2)
                t2 = (z1 - z)/(z1 - z2)

                u = (1 - t2)*(1 - t1)*u1 + (1 - t2)*t1*u2 +
     +              t2*(1 - t1)*u3 + t2*t1*u4

                v = (1 - t2)*(1 - t1)*v1 + (1 - t2)*t1*v2 +
     +              t2*(1 - t1)*v3 + t2*t1*v4

                w = (1 - t2)*(1 - t1)*w1 + (1 - t2)*t1*w2 +
     +              t2*(1 - t1)*w3 + t2*t1*w4

                ready = .true.

              end if

              i = i + 1

            end do

          end if

          j = j + 1

        end do

c       --- Reynolds stresses;  to deduce turbulent energy

        nz =  8
        ny = 11

        do j = 1, nz+1
          co(1,j) = 0d0
          do i = 1, ny+1
            co(2,i)   = 0d0
            ar(1,i,j) = 1d-5
          end do
        end do

        do j = 2, nz+1
          read(16,*) co(1,j)
          do i = 2, ny+1
            read(16,*) co(2,i), uu, vv, ww
            ar(1,i,j) = 5d-1*(uu+vv+ww)
          end do
        end do

        ready = .false.
        j = 1
        do while ( .not. ready .and. j .le. nz )

          z1 = (3d0 - co(1,j))*H+dlta
          z2 = (3d0 - co(1,j+1))*H-dlta

          if ( z .ge. z2 .and. z .le. z1 ) then

            i = 1
            do while ( .not. ready .and. i .le. ny )

              x1 = co(2,i)*H-dlta
              x2 = co(2,i+1)*H+dlta

              k1 = ar(1,i,j)*U0*U0
              k2 = ar(1,i+1,j)*U0*U0
              k3 = ar(1,i,j+1)*U0*U0
              k4 = ar(1,i+1,j+1)*U0*U0

              if ( x .ge. x1 .and. x .le. x2 ) then

                t1 = (x1 - x)/(x1 - x2)
                t2 = (z1 - z)/(z1 - z2)

                k = (1 - t2)*(1 - t1)*k1 + (1 - t2)*t1*k2 +
     +              t2*(1 - t1)*k3 + t2*t1*k4

                ready = .true.

              end if

              i = i + 1

            end do

          end if

          j = j + 1

        end do

      else if ( x.ge.125d-3*H .and. x.le. 875d-3*H .and. 
     +          z.ge.2d0*H ) then

c       --- Inlezen domein UP1

c       --- Cartesische snelheden

        nz = 18
        ny = 17

        do j = 1, nz+1
          co(1,j) = 0d0
          do i = 1, ny
            co(2,i)   = 0d0
            ar(1,i,j) = 0d0
            ar(2,i,j) = 0d0
            ar(3,i,j) = 0d0
          end do
        end do

        do j = 2, nz+1
          read(17,*) co(1,j)
          do i = 1, ny
            read(17,*) co(2,i), ar(1,i,j), ar(2,i,j), ar(3,i,j)
          end do
        end do

        ready = .false.
        j = 1
        do while ( .not. ready .and. j .le. nz )

          z1 = (3d0 - co(1,j))*H+dlta
          z2 = (3d0 - co(1,j+1))*H-dlta

          if ( z .ge. z2 .and. z .le. z1 ) then

            i = 1
            do while ( .not. ready .and. i .le. ny-1 )

              x1 = co(2,i)*H-dlta
              x2 = co(2,i+1)*H+dlta

              u1 = ar(1,i,j)*U0
              u2 = ar(1,i+1,j)*U0
              u3 = ar(1,i,j+1)*U0
              u4 = ar(1,i+1,j+1)*U0

              v1 = ar(2,i,j)*U0
              v2 = ar(2,i+1,j)*U0
              v3 = ar(2,i,j+1)*U0
              v4 = ar(2,i+1,j+1)*U0

              w1 = ar(3,i,j)*U0
              w2 = ar(3,i+1,j)*U0
              w3 = ar(3,i,j+1)*U0
              w4 = ar(3,i+1,j+1)*U0

              if ( x .ge. x1 .and. x .le. x2 ) then

                t1 = (x1 - x)/(x1 - x2)
                t2 = (z1 - z)/(z1 - z2)

                u = (1 - t2)*(1 - t1)*u1 + (1 - t2)*t1*u2 +
     +              t2*(1 - t1)*u3 + t2*t1*u4

                v = (1 - t2)*(1 - t1)*v1 + (1 - t2)*t1*v2 +
     +              t2*(1 - t1)*v3 + t2*t1*v4

                w = (1 - t2)*(1 - t1)*w1 + (1 - t2)*t1*w2 +
     +              t2*(1 - t1)*w3 + t2*t1*w4

                ready = .true.

              end if

              i = i + 1

            end do

          end if

          j = j + 1

        end do

c       --- Reynolds stresses;  to deduce turbulent energy

        nz = 27
        ny = 33

        do j = 1, nz+1
          co(1,j) = 0d0
          do i = 1, ny
            co(2,i)   = 0d0
            ar(1,i,j) = 1d-5
          end do
        end do

        do j = 2, nz+1
          read(18,*) co(1,j)
          do i = 1, ny
            read(18,*) co(2,i), uu, vv, ww
            ar(1,i,j) = 5d-1*(uu+vv+ww)
          end do
        end do

        ready = .false.
        j = 1
        do while ( .not. ready .and. j .le. nz )

          z1 = (3d0 - co(1,j))*H+dlta
          z2 = (3d0 - co(1,j+1))*H-dlta

          if ( z .ge. z2 .and. z .le. z1 ) then

            i = 1
            do while ( .not. ready .and. i .le. ny-1 )

              x1 = co(2,i)*H-dlta
              x2 = co(2,i+1)*H+dlta

              k1 = ar(1,i,j)*U0*U0
              k2 = ar(1,i+1,j)*U0*U0
              k3 = ar(1,i,j+1)*U0*U0
              k4 = ar(1,i+1,j+1)*U0*U0

              if ( x .ge. x1 .and. x .le. x2 ) then

                t1 = (x1 - x)/(x1 - x2)
                t2 = (z1 - z)/(z1 - z2)

                k = (1 - t2)*(1 - t1)*k1 + (1 - t2)*t1*k2 +
     +              t2*(1 - t1)*k3 + t2*t1*k4

                ready = .true.

              end if

              i = i + 1

            end do

          end if

          j = j + 1

        end do

      else if ( x.ge.875d-3*H .and. z.ge.2d0*H ) then

c       --- Inlezen domein OU1

c       --- Cartesische snelheden

        nz =  8
        ny = 13

        do j = 1, nz+1
          co(1,j) = 0d0
          do i = 1, ny+1
            co(2,i)   = 0d0
            ar(1,i,j) = 0d0
            ar(2,i,j) = 0d0
            ar(3,i,j) = 0d0
          end do
        end do

        do j = 2, nz+1
          read(19,*) co(1,j)
          do i = 2, ny+1
            read(19,*) co(2,i), ar(1,i,j), ar(2,i,j), ar(3,i,j)
          end do
        end do

        co(2,1) = 1d0

        ready = .false.
        j = 1
        do while ( .not. ready .and. j .le. nz )

          z1 = (3d0 - co(1,j))*H+dlta
          z2 = (3d0 - co(1,j+1))*H-dlta

          if ( z .ge. z2 .and. z .le. z1 ) then

            i = 1
            do while ( .not. ready .and. i .le. ny )

              x1 = co(2,i)*H+dlta
              x2 = co(2,i+1)*H-dlta

              u1 = ar(1,i,j)*U0
              u2 = ar(1,i+1,j)*U0
              u3 = ar(1,i,j+1)*U0
              u4 = ar(1,i+1,j+1)*U0

              v1 = ar(2,i,j)*U0
              v2 = ar(2,i+1,j)*U0
              v3 = ar(2,i,j+1)*U0
              v4 = ar(2,i+1,j+1)*U0

              w1 = ar(3,i,j)*U0
              w2 = ar(3,i+1,j)*U0
              w3 = ar(3,i,j+1)*U0
              w4 = ar(3,i+1,j+1)*U0

              if ( x .ge. x2 .and. x .le. x1 ) then

                t1 = (x1 - x)/(x1 - x2)
                t2 = (z1 - z)/(z1 - z2)

                u = (1 - t2)*(1 - t1)*u1 + (1 - t2)*t1*u2 +
     +              t2*(1 - t1)*u3 + t2*t1*u4

                v = (1 - t2)*(1 - t1)*v1 + (1 - t2)*t1*v2 +
     +              t2*(1 - t1)*v3 + t2*t1*v4

                w = (1 - t2)*(1 - t1)*w1 + (1 - t2)*t1*w2 +
     +              t2*(1 - t1)*w3 + t2*t1*w4

                ready = .true.

              end if

              i = i + 1

            end do

          end if

          j = j + 1

        end do

c       --- Reynolds stresses;  to deduce turbulent energy

        nz =  8
        ny = 11

        do j = 1, nz+1
          co(1,j) = 0d0
          do i = 1, ny+1
            co(2,i)   = 0d0
            ar(1,i,j) = 1d-5
          end do
        end do

        do j = 2, nz+1
          read(20,*) co(1,j)
          do i = 2, ny+1
            read(20,*) co(2,i), uu, vv, ww
            ar(1,i,j) = 5d-1*(uu+vv+ww)
          end do
        end do

        co(2,1) = 1d0

        ready = .false.
        j = 1
        do while ( .not. ready .and. j .le. nz )

          z1 = (3d0 - co(1,j))*H+dlta
          z2 = (3d0 - co(1,j+1))*H-dlta

          if ( z .ge. z2 .and. z .le. z1 ) then

            i = 1
            do while ( .not. ready .and. i .le. ny )

              x1 = co(2,i)*H+dlta
              x2 = co(2,i+1)*H-dlta

              k1 = ar(1,i,j)*U0*U0
              k2 = ar(1,i+1,j)*U0*U0
              k3 = ar(1,i,j+1)*U0*U0
              k4 = ar(1,i+1,j+1)*U0*U0

              if ( x .ge. x2 .and. x .le. x1 ) then

                t1 = (x1 - x)/(x1 - x2)
                t2 = (z1 - z)/(z1 - z2)

                k = (1 - t2)*(1 - t1)*k1 + (1 - t2)*t1*k2 +
     +              t2*(1 - t1)*k3 + t2*t1*k4

                ready = .true.

              end if

              i = i + 1

            end do

          end if

          j = j + 1

        end do

      end if

      lscale = min(kappa*(3*H-z), 0.2*3*H)

      if ( lscale .eq. 0d0 ) then
        eps = 1d4
      else
        eps = (cmu**75d-2)*(k**15d-1)/lscale
      end if

      if ( lscale .eq. 0d0 ) then
        omega = 1d4
      else
        omega = sqrt(k)/lscale
      end if

      if ( ichoice .eq. 1 ) then
        usfunb = v
      else if ( ichoice .eq. 2 ) then
        usfunb =  u
      else if ( ichoice .eq. 3 ) then
        usfunb =  -w
      else if ( ichoice .eq. 4 ) then
        usfunb =  k
      else if ( ichoice .eq. 5 ) then
        usfunb =  eps 
      else if ( ichoice .eq. 6 ) then
        usfunb =  omega
      end if

      close(11)
      close(12)
      close(13)
      close(14)
      close(15)
      close(16)
      close(17)
      close(18)
      close(19)
      close(20)

      end
