      program sepmesh
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.1    date   30-04-1995 Removal of buffer
c        version 2.0    date   11-11-1990 Complete reorganization
c        version 1.0    date   26-03-1988
c
c
c
c   copyright (c) 1988-1995  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c      Program sepmesh reads an input file containing information of the
c      mesh, and generates a SEPRAN mesh
c      The mesh is written to the meshoutput file as indicated in the
c      sepran.env file
c **********************************************************************
c
c                       KEYWORDS
c
c    create
c    mesh
c    plot
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c    SEPMSH  Performs the actual mesh generation
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call sepmsh(0)
      end
      subroutine funccoor ( ichoicechange, ndim, coor, nodes, numnodes)
c ======================================================================
c
      implicit none
      integer ichoicechange, ndim, numnodes, nodes(numnodes)
      double precision coor(ndim,*),coorch(2,105) 
      integer i, nodal_point_number
      integer ni, nj, n11
      double precision alpha, beta, eps


      ni    = 6
      nj    = 14
      n11   = 7
      alpha = .20
      beta  = .24
      eps   = .001
      call chcwl( coorch, ni, nj, n11, alpha, beta, eps)
            
      if ( ichoicechange .eq. 1 ) then
      
         do i = 1, numnodes
	    nodal_point_number = nodes(i)
	    coor(1,nodal_point_number) = coorch(1, i)
	    coor(2,nodal_point_number) = coorch(2, i)
	 end do
	 
      end if
      end
      subroutine chcwl ( coor, ni, nj, n11, alpha, beta, eps )
c
c ********************************************************************
c
c        programmer :    Kees Kassels
c        version    :    1.0    ISNaS/Sepran
c        date       :    20-02-92
c        computer   :    HP 9000/720
c        reference  :
c
c ********************************************************************
c
c     * TUD subroutine to change a SEPRAN mesh. 
c      
c       The mesh is supposed to be created by SEPMESH and stored in
c       the unformatted file meshoutput
c
c ********************************************************************
c
c           INPUT / OUTPUT PARAMETERS
c
      integer  ni, nj, n11
      double precision coor( 1:2, 0:ni, 0:nj ), alpha, 
     .                 beta, eps
c
c
c     COOR     o    Double precision three-dimensional array.
c
c                   COOR(k, i, j) k=1,2  ; i=0(1)ni ; j=0(1)nj 
c
c                   COOR(1, i, j) x-coordinate of vertex (i,j),
c                                 being the left-under
c                                 point of cell(i,j).
c                   COOR(2, i, j) y-coordinate of vertex(i,j).
c
c
c
c
c***********************************************************************
c
c                   COMMON BLOCKS
c
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer ip, jp  
      double precision pi, twpi, 
     .                 xmo, bm, yon, an, 
     .                 x1, x2, x3, fx1, fx2, fx3

c
c     ip, jp     General loop variables
c
c
c ********************************************************************
c
c                         I/O
c
c
c ********************************************************************
c
c                      SUBROUTINES CALLED
c
c
c ********************************************************************
c
c                     ERROR MESSAGES
c
c ********************************************************************
c
      pi = 4d0 * atan(1d0)
      twpi = 2d0 * pi
c
      do 20 jp = 0, nj
      do 20 ip = 0, ni
c
c     * LOWER Block
c
         if ( jp .le. n11 ) then
            yon = 2d0 * jp / n11
            an  = alpha * jp / n11
c
c     * UPPER Block
c
         else
            yon = 2d0 + 2d0*(jp-n11)/(nj-n11)
            an  = alpha * (nj-jp)/(nj-n11)
         end if
c
         xmo = real(ip)/real(ni)
         bm  = 4d0 * beta * (real(ip)/real(ni))*(1d0-real(ip)/real(ni))
c
         x1 = xmo - bm - eps
         x2 = xmo + bm + eps
         x3 = x1
         fx3 = xmo + bm*sin((pi/2d0)*(an*sin(twpi*x3)+yon))-x3
c
10       if((fx3+eps)*(fx3-eps) .ge. 0 ) then
c
            fx1 = xmo + bm*sin((pi/2d0)*(an*sin(twpi*x1)+yon))-x1
            fx2 = xmo + bm*sin((pi/2d0)*(an*sin(twpi*x2)+yon))-x2
            x3 = ( x2*fx1 -x1*fx2 ) / (fx1 - fx2 )
            fx3 = xmo + bm*sin((pi/2d0)*(an*sin(twpi*x3)+yon))-x3
            if ( fx1*fx3 .gt. 0 ) then
               x1 = x3
            else
               x2 = x3
            end if
            goto 10
c
         end if
c
         y3 = an * sin(twpi*x3) + yon
         coor(1,ip,jp) = x3
         coor(2,ip,jp) = y3
c
20    continue 
c
      end
      subroutine chcwl_h ( coor, ni, nj, n11, alpha, beta, eps )
c
c ********************************************************************
c
c        programmer :    Kees Kassels
c        version    :    1.0    ISNaS/Sepran
c        date       :    20-02-92
c        computer   :    HP 9000/720
c        reference  :
c
c ********************************************************************
c
c     * TUD subroutine to change a SEPRAN mesh. 
c      
c       The mesh is supposed to be created by SEPMESH and stored in
c       the unformatted file meshoutput
c
c ********************************************************************
c
c           INPUT / OUTPUT PARAMETERS
c
      integer  ni, nj, n11
      double precision coor( 1:2, 0:ni, 0:nj ), alpha, 
     .                 beta, eps
c
c
c     COOR     o    Double precision three-dimensional array.
c
c                   COOR(k, i, j) k=1,2  ; i=0(1)ni ; j=0(1)nj 
c
c                   COOR(1, i, j) x-coordinate of vertex (i,j),
c                                 being the left-under
c                                 point of cell(i,j).
c                   COOR(2, i, j) y-coordinate of vertex(i,j).
c
c
c
c
c***********************************************************************
c
c                   COMMON BLOCKS
c
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer ip, jp  
      double precision pi, twpi, 
     .                 ymo, bm, xon, an, 
     .                 y1, y2, y3, fy1, fy2, fy3

c
c     ip, jp     General loop variables
c
c
c ********************************************************************
c
c                         I/O
c
c
c ********************************************************************
c
c                      SUBROUTINES CALLED
c
c
c ********************************************************************
c
c                     ERROR MESSAGES
c
c ********************************************************************
c
      pi = 4d0 * atan(1d0)
      twpi = 2d0 * pi
c
      do 20 jp = 0, nj
      do 20 ip = 0, ni
c
c     * LOWER Block
c
         if ( ip .le. n11 ) then
            xon = 2d0 * ip / n11
            an  = alpha * ip / n11
c
c     * UPPER Block
c
         else
            xon = 2d0 + 2d0*(ip-n11)/(ni-n11)
            an  = alpha * (ni-ip)/(ni-n11)
         end if
c
         ymo = real(jp)/real(nj)
         bm  = 4d0 * beta * (real(jp)/real(nj))*(1d0-real(jp)/real(nj))
c
         y1 = ymo - bm - eps
         y2 = ymo + bm + eps
         y3 = y1
         fy3 = ymo + bm*sin((pi/2d0)*(an*sin(twpi*y3)+xon))-y3
c
10       if((fy3+eps)*(fy3-eps) .ge. 0 ) then
c
            fy1 = ymo + bm*sin((pi/2d0)*(an*sin(twpi*y1)+xon))-y1
            fy2 = ymo + bm*sin((pi/2d0)*(an*sin(twpi*y2)+xon))-y2
            y3 = ( y2*fy1 -y1*fy2 ) / (fy1 - fy2 )
            fy3 = ymo + bm*sin((pi/2d0)*(an*sin(twpi*y3)+xon))-y3
            if ( fy1*fy3 .gt. 0 ) then
               y1 = y3
            else
               y2 = y3
            end if
            goto 10
c
         end if
c
         x3 = an * sin(twpi*y3) + xon
         coor(1,ip,jp) = x3
         coor(2,ip,jp) = y3
c
20    continue 
c
      end


