      program tchannel_mb_pari
c=======================================================================
c
c     programmer      Kees Kassels
c
c     version  1.0    date    28-10-94
c
c   copyright (c) 1994  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c***********************************************************************
c
c                    DESCRIPTION
c
c     ISNaS Host Program
c
c***********************************************************************
c
c                   KEYWORDS
c
c     host_program
c
c***********************************************************************
c
c                   INPUT / OUTPUT PARAMETERS
c
      implicit none
c***********************************************************************
c
c                   COMMON BLOCKS
c
      integer nbuffr
      parameter( nbuffr = 4000000 )
      integer ibuffr
      common ibuffr(nbuffr)
c
c     IBUFFR    Integer buffer array in which all array information
c               with respect to the ISNaS software is stored.
c
c     NBUFFR    Declared length of IBUFFR
c
c
c***********************************************************************
c
c                   LOCAL PARAMETERS
c
c
c***********************************************************************
c
c                   SUBROUTINES CALLED
c
c     ISHMAIN    ISNaS host main subroutine
c
c***********************************************************************
c
c                   I/O
c
c***********************************************************************
c
c                   ERROR MESSAGES
c
c***********************************************************************
c
c                   PSEUDO CODE
c
c          initialize local buffer
c          call ISNaS host main subroutine
c
c=======================================================================

      call ismain( nbuffr )

      end
      function usfunb ( ichois, x, y, z, t )
c
c***********************************************************************
c
c     PSEUDO CODE    :
c
c     User written usfunition subroutine. It gives
c     the user the opportunity to define a
c     boundary condition as a usfunition of space
c     and time.
c
c
c***********************************************************************
c
c                   INPUT / OUTPUT PARAMETERS
c
      implicit none      
c
      double precision usfunb, x, y, z, t
      integer ichois
c
c
c     X        i    x-coordinate 
c
c     Y        i    y-coordinate 
c
c     T        i    actual time 
c
c     ICHOIS   i    choice parameter given by the user input
c
c     usfunb   o    computed boundary condition
c
c***********************************************************************
c
c                   LOCAL PARAMETERS
c
c
      double precision u, u0, tke, eps, cmu, delta, kappa, lambda,
     .                 lscale, It, H
c
c     CMU           an empirical constant
c
c     DELTA         boundary layer thickness
c
c     EPS           dissipation rate of turbulent energy  
c
c     H             Distance between to walls 
c
c     It            turbulence intensity 
c
c     KAPPA         Von Karman constant
c
c     LAMBDA        an empirical constant
c
c     LSCALE        length scale
c
c     TKE           turbulent kinetic energy
c    
c     U             tangential velocity
c
c     U0            bulk velocity 
c
c***********************************************************************

      cmu    = 9d-2
      kappa  = 41d-2
      lambda = 9d-2

      delta  = 7.76d-4
      u0     = 1d0
      It     = 1d-2
      H      = 0.1

      if ( y .lt. 0.5*H ) then
        u = min(u0, y*u0/delta)
      else
        u = min(u0, (H-y)*u0/delta)
      end if

      tke    = 15d-1*u*u*It*It
      if ( tke .eq. 0d0 ) tke = 1d-4

      if ( y .lt. 0.5*H ) then
        lscale = min(kappa*y, lambda*delta)
      else
        lscale = min(kappa*(H-y), lambda*delta)
      end if

      if ( lscale .eq. 0d0 ) then
        eps = 1d4
      else
        eps = (cmu**75d-2)*(tke**15d-1)/lscale
      end if

      if ( ichois .eq. 1 ) then
        usfunb = -u
      else if ( ichois .eq. 2 ) then
        usfunb =  tke
      else if ( ichois .eq. 3 ) then
        usfunb =  eps 
      end if

      end


