      subroutine isbndp ( bndcon, coor, ni, nj, nvirtual, i1, i2, iside,
     +                    value, ibinfor, mequat, massfrac )
c ======================================================================
c
c        programmer     Ronald van Nooyen
c        version 4.1    date 02-12-1998 Guus: Set last position
c        version 4.0    date 05-11-1997 Guus: extra parameters
c        version 3.0    date 30-08-1995 Guus: completely rewritten
c        version 2.1    date 30-01-1995 Marcel bndcon redefined
c        version 2.0    date 06-10-1994 new ISNaS Kees K
c        version 1.0    date 17-12-1992
c
c   copyright (c) 1992-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Set inflow profile (parabolic, 2D)
c
c **********************************************************************
c
c                       KEYWORDS
c
c     2d
c     boundary_conditions
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer i1, i2, ni, nj, iside, nvirtual, ibinfor(1-nvirtual:*),
     +        mequat
      double precision bndcon(2,1-nvirtual:*), value,
     +                 massfrac(1-nvirtual:ni+1+nvirtual,
     +                          1-nvirtual:nj+1+nvirtual),
     +                 coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2)

c     bndcon    o    reserved
c     coor      i    reserved
c     i1        i    Lower index number along first direction of a face or edge
c     i2        i    Upper index number along first direction of a face or edge
c     ibinfor   o    Third part of infbnd
c     iside     i    Sequence number of side
c     massfrac  i    Array containing the mass fraction
c     mequat    i    Indicates the type of momentum equation to be solved:
c                    0:  standard equation
c                    1:  multi-phase gas flow
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     value     i    Real number read by isreadfn
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, frsti, frstj, inci, incj, lasti, lastj, j, i
      double precision  x0, y0, dxc, dyc, ll, dl, xij, yij, massmean

c     dl        Length of projection of vector (xij-x0,yij-y0) on
c               the straight line connecting the begin and end
c               vertices of the segment.
c     dxc       Difference in x-coordinates between first and last vertex
c     dyc       Difference in y-coordinates between first and last vertex
c     frsti     First index in i-direction
c     frstj     First index in j-direction
c     i         General loop variable
c     inci      Offset between vertices in i-direction
c     incj      Offset between vertices in j-direction
c     ip        Local counting variable i-direction
c     j         General loop variable
c     jp        Local counting variable j-direction
c     lasti     Last index in i-direction
c     lastj     Last index in j-direction
c     ll        Length of straight line connecting the begin
c               and end vertices of the segment.
c     massmean  Mean value of mass fraction
c     x0        X-coordinate of the first vertex on the segment
c     xij       X-coordinate of the vertex (i,j) on the segment
c     y0        Y-coordinate of the first vertex on the segment
c     yij       Y-coordinate of the vertex (i,j) on the segment
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Determine on which of the four sides of the calculation domain
c     the current segment lies.
c
c        SET PARABOLA:
c
c        Determine the line connecting the first and the last vertex on the
c        segment, project intermediate points onto this line and use the
c        length of this projection to determine a parabolic inflow profile
c        for the true normal component of the velocity.
c
c     The parabola is given by the following formulas:
c
c     un(s) = 4 s (1-s) u_max
c
c     dx = x_end - x_start
c
c     dl = dx . ( x - x_start ) / ||dx||
c
c     s = dl / ||dx|| = dx . ( x - x_start ) / ||dx||^2
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      if ( iside.le.2 ) then
         frstj = i1
         lastj = i2+1
         inci  = 0
         incj  = 1
         if ( iside.eq.1 ) then
            frsti = 1
         else
            frsti = ni+1
         end if
         lasti = frsti
      else
         frsti = i1
         lasti = i2+1
         inci  = 1
         incj  = 0
         if ( iside.eq.3 ) then
            frstj = 1
         else
            frstj = nj+1
         end if
         lastj = frstj
      end if

      x0  = coor(frsti,frstj,1)
      y0  = coor(frsti,frstj,2)
      dxc = coor(lasti,lastj,1) - coor(frsti,frstj,1)
      dyc = coor(lasti,lastj,2) - coor(frsti,frstj,2)
      ll  = dxc**2+dyc**2
      ip  = frsti
      jp  = frstj

c     --- boundary conditions in vertex and midpoint

      do i = i1, i2
         j = 2*i-1
         ibinfor(j) = 12
         xij = coor(ip,jp,1)
         yij = coor(ip,jp,2)
         dl  = (dxc*(xij-x0)+dyc*(yij-y0))/ll
         bndcon(1,j) = -4d0*value*dl*(1d0-dl)
         bndcon(2,j) = 0d0
         if ( mequat.eq.1 ) bndcon(1,j) = bndcon(1,j)*massfrac(ip,jp)
         j = j + 1
         ibinfor(j) = 12
         xij = 0.5d0*(coor(ip,jp,1)+coor(ip+inci,jp+incj,1))
         yij = 0.5d0*(coor(ip,jp,2)+coor(ip+inci,jp+incj,2))
         dl  = (dxc*(xij-x0)+dyc*(yij-y0))/ll
         bndcon(1,j) = -4d0*value*dl*(1d0-dl)
         bndcon(2,j) = 0d0
         if ( mequat.eq.1 ) then
            massmean = 0.5d0*(massfrac(ip,jp)+massfrac(ip+inci,jp+incj))
            bndcon(1,j) = bndcon(1,j)*massmean
         end if
         ip = ip+inci
         jp = jp+incj
      end do
      bndcon(1,j+1) = 0d0
      bndcon(2,j+1) = 0d0
      ibinfor(j+1) = 12

      end
