      subroutine iscoef ( ichois, iincof, rincof, rintur, coefs,
     +                    eddy, solut, nsolut, ndegfd, coor, npointsv,
     +                    ndim, ni, nj, nk, nvirtual, ncoefs, time,
     +                    itopol, iinput, work, veloc, nsubfaces,
     +                    subfaces, mturb )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 4.4    date 06-07-1999 Guus: extra call to USFILSRC
c        version 4.3    date 06-07-1998 Kees V: nusol, nsolut cleaned
c        version 4.2    date 09-06-1998 Charles: nvpoints changed to npointsv
c        version 4.1    date 03-02-1998 Guus: allow 3d space dependent
c                                       coefficients
c        version 4.0    date 18-09-1997 Guus: extra parameter mturb
c        version 3.2    date 04-08-1997 Guus: remove stop
c        version 3.1    date 03-07-1997 Hester:extra call
c        version 3.0    date 10-06-1997 Guus: extra parameters
c        version 2.2    date 29-11-1994 Zijlema: new calls: iscof2 and iscof3
c        version 2.1    date 22-11-1994 Kees K usfil2 and usfil3 changed into
c                                       isflc2 and isflc3
c        version 2.0    date 21-09-1994 Marcel Zijlema: add eddy viscosity to
c                                                     laminar viscosity or to
c                                                     diffusivity coefficient
c        version 1.0    date 20-06-1994
c
c   copyright (c) 1994-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     subroutine to fill the coefficients for the momentum
c     equations and/or transport equations into array coef.
c     The coefficients may be either constant or a function of
c     space, time and preceding solutions.
c **********************************************************************
c
c                       KEYWORDS
c
c     coefficients
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ncoefs, iincof(1:ncoefs), ndegfd, ichois, itopol(*),
     +        nsolut, npointsv, ndim, ni, nj, nk, nvirtual, iinput(*),
     +        nsubfaces(*), subfaces(*), mturb
      double precision rincof(1:ncoefs), coefs(1:npointsv,1:ncoefs),
     +                 solut(1:nsolut), eddy(1:npointsv), work(*),
     +                 coor(*), rintur(*), veloc(*),
     +                 time

c     coefs          o    COEFS(i,k) contains the k-th coefficient in
c                         centroid i. The nodes are ordered in exactly the
c                         same way as the scalar equations in SOLUT.
c                         The coefficients array is extended with virtual
c                         cells in exactly the same way as the locally
c                         copied solution array is.
c     coor           i    Array containing the coordinates for one block
c                         In 2d coor(i,j,.) contains the x resp y-coordinate
c                         of point i,j
c                         in 3d: coor(i,j,k,.) corresponds to (i,j,k)
c     eddy           i    array EDDY contains the values of the
c                         eddy viscosity of that block in the centroids
c                         of the elements. The storage of such
c                         an array EDDY is identical to that of all other
c                         coefficients.
c     ichois         i    General choice parameter
c     iincof         i    Array IINCOF consists of a series of
c                         one-dimensional arrays of length NCOEFS.
c                         For each equation NCOEFS positions are used, in
c                         the case of coupled momentum equations these
c                         equations are counted for one equation only. The
c                         sequence of the storage in IINCOF is the same as
c                         for the unknowns, i.e. first the momentum
c                         equations, then the first transport equation
c                         et-cetera.
c                         For a description see the programmers guide
c     iinput         i    Contains the integer user input for one block.
c                         For a description see the programmers guide
c     itopol         i    The itopol array stores the connections between
c                         the blocks. The basic idea behind the itopol
c                         data-structure is that the blocks communicate
c                         through common sub-faces.
c     mturb          i    Indicates type of turbulence modelling
c     ncoefs         i    Number of coefficients
c     ndegfd         i    Number of degrees of freedom
c     ndim           i    Dimension of space (2 or 3)
c     ni             i    Number of non-virtual cells in the i-direction
c                         (one block)
c     nj             i    Number of non-virtual cells in the j-direction
c                         (one block)
c     nk             i    Number of non-virtual cells in the k-direction
c                         (one block)
c     npointsv       i    Number of points in the mesh with virtual points
c     nsolut         i    Number of unknowns for all degrees of freedom in
c                         one block
c     nsubfaces      i    Array containing the number of subfaces of each face,
c                         see array itopol for an extended description
c     nvirtual       i    Number of rows of virtual cells
c     rincof         i    in rincof real information about the coefficients
c                         is stored. The contents is defined by iincof.
c                         For a description see the programmers guide
c     rintur         i    Array RINTUR is part of array RINPUT and hence is
c                         coupled to one block only. At this moment RINTUR
c                         is filled with fixed constants, however, since in
c                         practice these turbulence constants may depend on
c                         the type of model to be solved, it has been
c                         decided to let RINTUR be part of RINPUT. In this
c                         way it may be possible in the future to adapt the
c                         series of constants in the pre-processing phase.
c                         For a description see the programmers guide
c     solut          i    Solution array
c                         For a description see the programmers guide
c     subfaces       i    The basic idea behind the itopol
c                         data-structure is that the blocks communicate
c                         through common sub-faces. For instance in 2-D,
c                         each block consists of four faces . The
c                         faces are divided into sub faces that are numbered
c                         from west to east and from south to north.
c                         The itopol array stores for each sub
c                         face the neighbouring block ( if the sub face is
c                         external) and the corresponding neighbouring face
c                         and sub face numbers.
c                         For a description see the programmers guide
c     time           i    Actual time of time step
c     veloc          i    Array for the old velocities (used in convection
c                         term calculation)
c     work                Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, choice, iblock, nblocks, nfaces, maxnsubf,
     +        itypepost, idim, npoint, ipiindsc, axisym

c     axisym         In axi-symmetric case axisym=1 else axisym=0
c     choice         General choice parameter
c     i              General loop variable
c     iblock         Block sequence number
c     idim           Loop variable used for dimension
c     ipiindsc       Starting address of array IINDSC in array IINPUT
c     itypepost      Indicates the type of postprocessing required
c                    0: standard Newton
c                    1: linear in computational domain
c     j              General loop variable
c     maxnsubf       Maximum number of subfaces encountered on all subfaces
c     nblocks        Number of blocks
c     nfaces         Number of faces
c     npoint         Number of points for one block
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISCOF1         Fill coefs with constant
c     ISCOF2         add laminar viscosity with turbulent one in 2D case
c     ISCOF3         add laminar viscosity with turbulent one in 3D case
c     ISERCLOS       Resets old name of previous subroutine of higher level
c     ISEROPEN       Produces concatenated name of local subroutine
c     ISERRCHR       Puts values of strings in character error array
c     ISERRINT       Put integer value in error message
c     ISERRSUB       Error messages
c     ISFLC2         fill one coefficient in coefs with a function of space,
c                    time and possible preceding solutions, in 2D case
c     ISFLC3         fill one coefficient in coefs with a function of space,
c                    time and possible preceding solutions, in 3D case
c     ISOUT01        interpolate quantities from centroids to vertices
c     ISOUT02        determine Cartesian velocity components in the centroids, 2
c     USCOMPVISC     User subroutine to fill coefficients
c     USFILC         User subroutine to fill coefficients
c     USFILCSP       User subroutine to fill coefficients
c     USFILSRC       User subroutine to compute the right-hand side of the
c                    energy equation
c **********************************************************************
c
c                       ERROR MESSAGES
c
c       6   iincof(i) out of range
c      25   Value of CHOICE has not yet been implemented
c **********************************************************************
c
c                       PSEUDO CODE
c
c     For i=1(1)NCOEFS do
c
c        CHOICE = IINCOF(i)
c
c        If  CHOICE = -1 then
c
c           Fill COEFS ( for the coefficient i )
c           with constant RINCOF(i) by calling subroutine ISCOF1
c
c        else if CHOICE < -100
c
c           Fill COEFS ( for the coefficient i )
c           with a function by calling the user function USFUNC
c
c     Add eddy viscosity to laminar viscosity or to molecular
c     diffusivity coefficient corresponding to a transport equation
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      call iseropen('iscoef')

c     --- For NCOEFS coefficients do
c         initialize coefs

      do i=1, ncoefs
         do j = 1, npointsv
            coefs(j,i) = 0d0
         end do
      end do

      do i = 1, ncoefs

         choice = iincof(i)

         if ( choice.lt.-1 .and. choice.ge.-100 .or.
     +        choice.ge.200 .and. choice.le.1000 .or.
     +        choice.gt.10002 ) then

c        --- IINCOF(i) wrong value

            call iserrint( i, 1 )
            call iserrint( choice, 2 )
            call iserrchr( 'iscoef',1 )
            call iserrsub ( 6, 2, 0, 1 )

            go to 1000

         else if ( choice.eq.-1 ) then

c        --- Fill COEFS with constant

            call iscof1 ( coefs, ncoefs, npointsv, rincof(i), i )

         else if ( choice.gt.0 .and. choice.lt.200 ) then

c        --- Fill COEFS with a function of space, time and possible
c            preceding solutions

            if ( ndim.eq.2) then

               call isflc2 ( choice, coefs, ncoefs, i, ni, nj, ndim,
     +                       nvirtual, coor, solut , ndegfd, time )

            else

               call isflc3 ( choice, coefs, ncoefs, i, ni, nj, nk,
     +                       nvirtual, coor, solut , ndegfd, time )

            end if

         else if ( choice.eq.10000 ) then

c        --- choice = 10000, special call
c            First fill velocity in array veloc

            if ( ndim.eq.2 ) then

c           --- velocities, 2d
c               determine Cartesian velocity components in the centroids

               call isout02 ( ni, nj, nvirtual, coor, solut, work )
               iblock = iblockrf
               nblocks  = itopol(1)
               nfaces   = itopol(2)
               maxnsubf = itopol(3)
               itypepost = iinput(67)
               npoint = (ni+1)*(nj+1)
               do idim = 1, 2
                  call isout01 ( iblock, ni, nj, nvirtual, nblocks,
     +                           nfaces, maxnsubf, nsubfaces, subfaces,
     +                           coor, work(1+(idim-1)*npointsv),
     +                           veloc(1+(idim-1)*npoint),
     +                           itypepost )
               end do

c              --- interpolation of pressure to vertices

               call isout01 ( iblock, ni, nj, nvirtual, nblocks,
     +                        nfaces, maxnsubf, nsubfaces, subfaces,
     +                        coor, solut(1+2*npointsv),
     +                        veloc(1+2*npoint),
     +                        itypepost )

            else

               call iserrint ( choice, 1 )
               call iserrsub ( 25, 1, 0, 0 )

            end if

            call usfilcsp ( coefs, ncoefs, ni, nj, nk, ndim, nvirtual,
     +                      coor, solut, ndegfd, time, veloc )

         else if ( choice.eq.10001 ) then

c        --- choice = 10001, special call

            ipiindsc = iinput(6)
            axisym = iinput(ipiindsc-1+10)
            call usfilsrc ( coefs(1,i), ni, nj, nk, ndim, nvirtual,
     +                      coor, solut, ndegfd, time, eddy, axisym )

         else if ( choice.eq.10002 ) then

c        --- choice = 10002, special call

            ipiindsc = iinput(6)
            axisym = iinput(ipiindsc-1+10)
            call uscompvisc ( coefs(1,i), eddy, ni, nj, nk, nvirtual,
     +                        ndim )

         else if ( choice.gt.1000 ) then

c        --- choice > 1000, call to usfilc

            call usfilc ( choice-1000, coefs, ncoefs, i, ni, nj, nk,
     +                    ndim, nvirtual, coor, solut, ndegfd, time )

         end if

      end do

      if ( mturb.gt.0 ) then

c     --- Add eddy viscosity to molecular viscosity or to molecular
c         diffusivity coefficient w.r.t. a transport equation

         if ( ndim.eq.2 ) then

            call iscof2 ( ichois, ni, nj, nvirtual,
     +                    ncoefs, rintur, eddy, coefs )

         else if ( ndim.eq.3 ) then

            call iscof3 ( ichois, ni, nj, nk, nvirtual,
     +                    ncoefs, rintur, eddy, coefs )

         end if

      end if

1000  call iserclos('iscoef')

      end
