      subroutine isdist3 ( wdis, coor, iinbc, nvirtual, ni, nj, nk,
     +                     ntrnsp,nturb, rinbc, igeom, icount )
c ======================================================================
c
c     programmer   Frederic Tremblay
c     version      1.1    date    22-08-1997 added array igeom to keep
c                                            track which cell at the boundary
c                                            is the closest to each cell
c                                            in the domain
c     version      1.0    date    18-08-1997
c
c   copyright (c) 1997-1997  "TU-Muenchen"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Muenchen".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     fill array eddy with normal distance to walls in the whole domain
c     constructed from 2D subroutine, modified to improve accuracy
c **********************************************************************
c
c                       KEYWORDS
c
c     3d
c     turbulence
c     wall_distance
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinbc(*),ni,nj,nk,nvirtual,ntrnsp,nturb,icount,
     +                igeom(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual)
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual, 1:3),
     +                 wdis(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual),
     +                 rinbc(*)

c     coor      i    reserved
c     icount    i    Counts the number of positions necessary for the number
c     igeom     i    Integer array that returns the closest cell on a wall
c     iinbc     i    reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     ntrnsp    i    Number of transport equations
c     nturb     i    Number of extra transport equations for turbulence
c     nvirtual  i    Number of rows of virtual cells
c     rinbc     i    reserved
c     wdis      o    Array containing the wall distance
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iside, iarea(2,3), ndir, tdir1,tdir2,
     +        ipstart, nseg, j, ipiinbc, ip, jp,kp, nsign,
     +        ipjp(4,3), i, ipl, jpl,kpl,idim,bctyp(3),iprinbc
      double precision xq(3),xp(3),ndist, bcval(3)

c     bctyp     A given boundary type
c     bcval     A given boundary condition value
c     i         General loop variable
c     iarea     Area of arrays
c     idim      Loop variable used for dimension
c     ip        Local counting variable i-direction
c     ipiinbc   Starting address of array iinbc in array iinput
c     ipjp      Array containing the ip and jp indices of the 4 vertices of
c               the cell
c               ipjp(i,j): i=1,4; j=1,2
c               j=1: ip     j=2: jp
c     ipl       Local ip value
c     iprinbc   Starting address of array rinbc in array iinput
c     ipstart   Starting address
c     iside     Sequence number of side
c     j         General loop variable
c     jp        Local counting variable j-direction
c     jpl       Local jp value
c     kp        Local counting variable k-direction
c     kpl       Local kp value
c     ndir      Direction of normal vector on this side
c     ndist     Distance normal to a solid wall
c     nseg      Number of segments the boundary is divided in
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     tdir1     Direction of first tangent vector on current side
c     tdir2     Direction of second tangent vector on current side
c     xp        Auxiliary array to calculate boundary point number
c     xq        Array containing the co-ordinates of the point Q at the boundary
c **********************************************************************
c
c                       I/O
c
c     none
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('isdist3')

c     --- Initialize wdis and igeom

      do ip = 1, ni
        do jp = 1, nj
           do kp = 1, nk
             wdis(ip,jp,kp) = 1d8
             igeom (ip,jp,kp) = -999
           end do
        end do
      end do

c     --- loop over boundaries

      iprinbc = 0
      icount = 0

      do iside = 1, 6

c     --- Get information on boundary 'iside'

        if ( iside.le.2 ) then
           ndir = 1
           tdir1 = 2
           tdir2 = 3
           if ( iside.eq.1 ) then
              nsign = -1
              iarea(1,1) = 1
           else
              nsign = 1
              iarea(1,1) = ni
           end if
           iarea(2,1) = iarea(1,1)
        else if (iside.le.4) then
           ndir = 2
           tdir1 = 3
           tdir2 = 1
           if ( iside.eq.3 ) then
              nsign = -1
              iarea(1,2) = 1
           else
              nsign = 1
              iarea(1,2) = nj
           end if
           iarea(2,2) = iarea(1,2)
        else
           ndir = 3
           tdir1 = 1
           tdir2 = 2
           if (iside.eq.5) then
              nsign = -1
              iarea(1,3) = 1
           else
              nsign = 1
              iarea(1,3) = nk
           end if
           iarea(2,3) = iarea(1,3)
        end if

        ipstart = iinbc(iside)

        nseg = iinbc(ipstart)

c       --- loop over segments

        do j = 1, nseg

          ipiinbc = ipstart +(j-1) *( 4+(ntrnsp+nturb+2)*4 )

          iarea(1,tdir1) = iinbc( ipiinbc + 1 )
          iarea(2,tdir1) = iinbc( ipiinbc + 3 )
          iarea(1,tdir2) = iinbc( ipiinbc + 2 )
          iarea(2,tdir2) = iinbc( ipiinbc + 4 )

          if ( iinbc(ipiinbc+5).eq.10 ) then

c         --- All velocities are prescribed, hence a no-slip wall is possible
c             Put velocity values in bcval

            do idim = 1,3
               bcval(idim) = 1d20
               bctyp(idim) = iinbc(ipiinbc+5+idim)
               if (bctyp(idim).eq.-1) then
                  bcval(idim) = rinbc(iprinbc+idim)
               end if
            end do

            iprinbc = iprinbc+(ntrnsp+nturb+2)*3

            if (bcval(1).eq.0d0 .and. bcval(2).eq.0d0
     +                          .and. bcval(3).eq.0d0 ) then

c           --- no-slip condition

              do ipl = iarea(1,1), iarea(2,1)
                do jpl = iarea(1,2), iarea(2,2)
                  do kpl = iarea(1,3), iarea(2,3)

                    icount = icount + 1

                    do ip = 1, ni
                      do jp = 1, nj
                        do kp = 1, nk

c                       --- Compute the co-ordinates of P (centroid)

                          do i = 1, 3
                             xp(i) = 125d-3*(coor(ip,jp,kp,i)+
     +                                       coor(ip+1,jp,kp,i)+
     +                                       coor(ip,jp+1,kp,i)+
     +                                       coor(ip,jp,kp+1,i)+
     +                                       coor(ip+1,jp+1,kp,i)+
     +                                       coor(ip+1,jp,kp+1,i)+
     +                                       coor(ip,jp+1,kp+1,i)+
     +                                       coor(ip+1,jp+1,kp+1,i))
                          end do

c                         --- Fill array ipjp as function of ipl, jpl and iside

                          do i = 1, 4
                            ipjp(i,1) = ipl
                            ipjp(i,2) = jpl
                            ipjp(i,3) = kpl
                          end do
                          if ( nsign.eq.1 ) then

c                         --- nsign = 1, sides 2 and 4 and 6

                            ipjp(1,ndir) = ipjp(1,ndir)+1
                            ipjp(2,ndir) = ipjp(2,ndir)+1
                            ipjp(3,ndir) = ipjp(3,ndir)+1
                            ipjp(4,ndir) = ipjp(4,ndir)+1

                          end if

                          ipjp(2,tdir1) = ipjp(2,tdir1) + 1
                          ipjp(3,tdir2) = ipjp(3,tdir2) + 1
                          ipjp(4,tdir1) = ipjp(4,tdir1) + 1
                          ipjp(4,tdir2) = ipjp(4,tdir2) + 1

c                         --- Compute the unit tangential vector and
c                             the co-ordinates of Q

                          do i = 1, 3

                            xq(i) = 25d-2*(
     +                        coor(ipjp(1,1),ipjp(1,2),ipjp(1,3),i)+
     +                        coor(ipjp(2,1),ipjp(2,2),ipjp(2,3),i)+
     +                        coor(ipjp(3,1),ipjp(3,2),ipjp(3,3),i)+
     +                        coor(ipjp(4,1),ipjp(4,2),ipjp(4,3),i) )
                          end do

                          ndist =  sqrt( (xp(1)-xq(1))**2+
     +                                  (xp(2)-xq(2))**2+
     +                                  (xp(3)-xq(3))**2)

                          wdis(ip,jp,kp) = min(ndist,wdis(ip,jp,kp))
                          if (abs(wdis(ip,jp,kp)-ndist).lt.sqreps )
     +                       igeom(ip,jp,kp) = icount

                        end do
                      end do
                    end do

                  end do
                end do
              end do

            end if

          end if

        end do

      end do

      call iserclos('isdist3')

      end
