      subroutine isexcf ( iblock, ni, nj, nk, nvirtual,  ndim,
     +                    ncoefs, nblocks, nfaces, maxnsubf,
     +                    coefs , nsubfaces, subfaces, mequat )
c ======================================================================
c
c        programmer   Kees Kassels
c        version 2.0    date 20-11-1997 Guus: extra parameter mequat
c        version 1.4    date 07-04-1997 Kees K: call to isex03 changed
c        version 1.3    date 04-08-1995 Michiel K: correction of function call
c                                       to ISEX03, because of adaption for
c                                       periodic bc's. This may not be the best
c                                       way, but is sufficient for the moment!
c        version 1.2    date 24-01-1995 Erik: corrected error in the
c                                       call to isex03
c        version 1.1    date 27-12-1994 3D
c        version 1.0    date 29-11-1994
c
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Extrapolate virtual coefficients
c
c **********************************************************************
c
c                       KEYWORDS
c
c    extrapolation
c    topology
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, maxnsubf, nblocks, ncoefs, nfaces, ni, nj, nk,
     +      ndim, nvirtual, mequat
      double precision coefs(*)
      integer nsubfaces(1:nblocks, 1:nfaces)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     coefs     i    reserved
c     iblock    i    Block sequence number
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     mequat    i    Indicates the type of momentum equation to be solved:
c                    0:  standard equation
c                    1:  multi-phase gas flow
c     nblocks   i    Number of blocks
c     ncoefs    i    Number of coefficients
c     ndim      i    Dimension of space (2 or 3)
c     nfaces    i    Number of faces
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     nvirtual  i    Number of rows of virtual cells
c     subfaces  i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, npointsv

c     i         General loop variable
c     npointsv  Number of points in the mesh with virtual points
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISEX03    Extrapolation for 3D
c     ISEXCF02  Compute virtual coefficients not coming from a
c               neighbouring block by extrapolation (2d case)
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      if (ndim.eq.2) then

         call isexcf02 ( iblock , ni, nj, nvirtual, ncoefs,
     +                   nblocks, nfaces, maxnsubf,
     +                   coefs, nsubfaces, subfaces, mequat )

      else if (ndim.eq.3) then

         npointsv=(ni+1+2*nvirtual)*(nj+1+2*nvirtual)*(nk+1+2*nvirtual)

         do i = 1, ncoefs
            call isex03 ( iblock, ndim, ni, nj, nk, nvirtual,
     +                    coefs((i-1)*npointsv+1),4,nvirtual, nblocks,
     +                    nfaces, maxnsubf, nsubfaces, subfaces )
         end do

      end if

      end
