      subroutine isfilc02 ( ichois, iinput, rinput, coefs, solut, coor,
     +                      geom, eddy, work, lwork, itopol, nsubfaces,
     +                      subfaces, igeom, averstr )
c ======================================================================
c
c     programmer   Kees Kassels
c     version 3.2    date 06-07-1998  Kees V: nusol, nsolut cleaned
c     version 3.1    date 18-08-1997  Fred T: extension parameter list
c                                     istuvisc with work,lwork,coor,geom
c                                     and new routine for 3D wall dist.
c     version 3.0    date 11-06-1997  Guus: new parameter list
c     version 2.6    date 23-08-1996  Marcel: extension parameter list
c                                     iscoeftu with maniso
c     version 2.5    date 10-07-1996  Marcel: computation of normal distance
c                                     to walls for low-Re k-eps modeling
c     version 2.4    date 28-02-1995  Guus: new call of iscoeftu
c     version 2.3    date 19-01-1995  Zijlema: extension parameter list
c                                     iscoeftu with iinbc and rinbc
c     version 2.2    date 10-01-1995  Marcel: parameter list iscoeftu changed
c     version 2.1    date 01-11-1994  Kees K : renamed
c     version 2.0    date 21-09-1994  Marcel Zijlema: extension to turbulence
c
c     version 1.01   date 28-07-1994  Zijlema: a small correction
c                                     (ichois -> ichois - 1)
c     version 1.0    date 20-06-1994
c
c   copyright (c) 1994-1997  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     ISNAS subroutine to fill the coefficients for the momentum
c     equations or transport equations into array coefs (including
c     turbulence).
c     The coefficients may be either constant or a function of
c     space, time and preceding solutions.
c **********************************************************************
c
c                       KEYWORDS
c
c     coefficients
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinput(*), ichois, lwork, itopol(*), nsubfaces(*),
     +        subfaces(*), igeom (*)
      double precision rinput(*), coefs(*), solut(*), coor(*),
     +                 geom(*), eddy(*),  work(*), averstr(*)

c     averstr   i    reserved
c     coefs     o    reserved
c     coor      i    reserved
c     eddy      i    reserved
c     geom      i    reserved
c     ichois    i    General choice parameter
c     igeom     i    Integer array that returns the closest cell on a wall
c     iinput    i    reserved
c     itopol    i    reserved
c     lwork     i    Length of array work
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     rinput    i    reserved
c     solut     i    reserved
c     subfaces  i    reserved
c     work           Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ipicof, ipitur, iprcof, iprtim, iprtur, ipidsc, viscorr,
     +        maniso, ni, nj, nk, ndegfd, ndim, mturb, leniindsc,
     +        ncoefs, npoint, nsolut, ipiinbc, iprinbc,
     +        ntimlv, nvirtual, nturb, ntrnsp, ipiintim, itimeact,
     +        icount, nwindowmx
      double precision time, deltat

c     deltat    Product of theta and dt
c     icount    Counts the number of positions necessary for the number
c     ipicof    Starting address of array iincof in array iinput
c     ipidsc    Starting address of array iindsc in array iinput
c     ipiinbc   Starting address of array iinbc in array iinput
c     ipiintim  Starting address of array IINTIM in array IINPUT
c     ipitur    Starting address of array iintur in array iinput
c     iprcof    Starting address of array rincof in array rinput
c     iprinbc   Starting address of array rinbc in array iinput
c     iprtim    Starting address of array rintim in array rinput
c     iprtur    Starting address of array rintur in array rinput
c     itimeact  Actual time step
c     leniindsc Length of first component of array iindsc
c     maniso    Indicates type of nonlinear turbulence (anisotropic)
c               modelling
c     mturb     Indicates type of turbulence modelling
c     ncoefs    Number of coefficients
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npoint    Number of points for one block
c     nsolut    Number of unknowns for all degrees of freedom in one block
c     ntimlv    Number of time levels
c     ntrnsp    Number of transport equations
c     nturb     Number of extra transport equations for turbulence
c     nvirtual  Number of rows of virtual cells
c     nwindowmx Maximum size of the window to be used for time-averaging
c     time      Actual time of time step
c     viscorr   Indicate whether to use viscous corrections or not
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISCOEF    perform the actual filling of the coefficients
c     ISCOEFTU  perform the actual filling of the coefficients
c               w.r.t. the turbulence equations
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISTUVISC  computes turbulent viscosity
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     First, compute turbulent viscosity
c
c     if ICHOIS =  1 then
c
c        fill coefficients for the momentum equation
c
c     else if ICHOIS < 0  then
c
c        fill coefficients for turbulence equations
c
c     else if ICHOIS > 1 then
c
c        fill coefficients for (ichois-1)-th transport equation
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('isfilc02')

      ipicof    = iinput( 1 )
      ipiinbc   = iinput( 2 )
      ipiintim  = iinput( 4 )
      ipidsc    = iinput( 6 )
      ipitur    = iinput( 8 )
      iprcof    = iinput( 26 )
      iprinbc   = iinput( 27 )
      iprtim    = iinput( 29 )
      iprtur    = iinput( 31 )

      ni        = iinput( 51 )
      nj        = iinput( 52 )
      nk        = iinput( 53 )
      nvirtual  = iinput( 54 )
      ndegfd    = iinput( 55 )
      ndim      = iinput( 57 )
      ntrnsp    = iinput( 63 )
      ntimlv    = iinput( 64 )
      ncoefs    = iinput( 66 )
      nturb     = iinput( 70 )
      leniindsc = iinput(106)

      mturb     = iinput( ipitur   -1 + 2 )
      viscorr   = iinput( ipitur   -1 + 3 )
      maniso    = iinput( ipitur   -1 + 4 )
      nwindowmx = iinput( ipitur   -1 + 6 )
      icount    = iinput( ipitur   -1 + 9 )
      itimeact  = iinput( ipiintim -1 + 4 )
      deltat    = rinput( iprtim   -1 + 4 )

      if ( ndim.eq.2 ) then
         npoint = (ni+1+2*nvirtual) * (nj+1+2*nvirtual)
         nsolut = (ni+1+2*nvirtual) * (nj+1+2*nvirtual) * ndegfd *
     +            ntimlv
      else
         npoint = (ni+1+2*nvirtual) * (nj+1+2*nvirtual) *
     +            (nk+1+2*nvirtual)
         nsolut = (ni+1+2*nvirtual) * (nj+1+2*nvirtual) *
     +            (nk+1+2*nvirtual) * ndegfd * ntimlv
      end if

c     --- First, compute eddy viscosity

      if ( mturb.gt.0 )
     +   call istuvisc ( mturb, ndegfd, ntimlv, rinput(iprtur),
     +                   ndim, ni, nj, nk, npoint, nvirtual,
     +                   solut, eddy, rinput(iprcof), viscorr,
     +                   work, lwork, coor, geom, ichois,
     +                   iinput(ipidsc+6 ), igeom, rinput(iprcof+1),
     +                   rinput(iprcof), ntrnsp,nturb,iinput(ipiinbc),
     +                   rinput(iprinbc), itimeact, averstr, icount,
     +                   iinput( ipitur-1+7 ), iinput( ipitur-1+8 ),
     +                   rinput( iprtur-1+25 ), nwindowmx, deltat )

      time   = rinput( iprtim -1 + 1 )

      if ( ichois.eq.1 ) then

c     --- momentum equations

         call iscoef ( ichois, iinput(ipicof), rinput(iprcof),
     +                 rinput(iprtur), coefs, eddy,
     +                 solut, nsolut, ndegfd, coor, npoint, ndim,
     +                 ni, nj, nk, nvirtual, ncoefs, time,
     +                 itopol, iinput, work, work(1+ndim*npoint),
     +                 nsubfaces, subfaces, mturb )

      else if ( ichois.lt.0 ) then

c     --- turbulence equations

         call iscoeftu ( ichois, mturb, ncoefs, npoint, ndegfd,
     +                   ndim, ntimlv, ni, nj, nk, nvirtual, ntrnsp,
     +                   nturb, lwork, iinput(ipicof), rinput(iprcof),
     +                   iinput(ipidsc), coefs, rinput(iprtur), geom,
     +                   solut, eddy, coor, work, iinput(ipiinbc),
     +                   rinput(iprinbc), time, leniindsc, viscorr,
     +                   maniso )

      else if ( ichois.gt.1 ) then

c     --- general transport equation

         ipicof = ipicof + (ichois-1)*ncoefs
         iprcof = iprcof + (ichois-1)*ncoefs

         call iscoef ( ichois, iinput(ipicof), rinput(iprcof),
     +                 rinput(iprtur), coefs, eddy,
     +                 solut, nsolut, ndegfd, coor, npoint, ndim,
     +                 ni, nj, nk, nvirtual, ncoefs, time,
     +                 itopol, iinput, work, work(1+ndim*npoint),
     +                 nsubfaces, subfaces, mturb )

      end if

      call iserclos('isfilc02')

      end
