      subroutine isfilm ( infmm , infmg, infmt, iinput, itypsten )
c ======================================================================
c
c     programmer   Kees Kassels
c     version      2.2   date    26-04-1998 Guus: Extension for 3D WesBeek
c     version      2.1   date    02-10-1996 Guus: Change parameter list
c     version      2.0   date    05-04-1995 Guus: New parameter list,
c                                               new definition of position 1
c     version      1.2   date    20-12-1994 R. van Nooyen wesbeek adaptation
c     version      1.1   date    17-08-1994 R. van Nooyen 3D adaptation
c     version      1.0   date    10-01-1994
c
c   copyright (c) 1993-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     routine which fills the integer information for
c     the matrices in the arrays infmt infmg and infmt
c **********************************************************************
c
c                       KEYWORDS
c
c     matrix_information
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer infmm(*), infmg(*), infmt(*), iinput(*), itypsten

c     iinput    i    reserved
c     infmg     o    Integer information for Pressure-gradient matrix
c     infmm     o    Integer information for Momentum matrix
c     infmt     o    Integer information for Pressure and Transport matrix
c     itypsten  i    Indicates the type of stencil
c                    Possible values
c                    R^2
c                    1:  13-point stencil
c                    2:  21-point stencil
c                    3:  17-point stencil
c                    R^3
c                    1:  51-point stencil
c                    2:  59-point stencil
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ni, nj, nk, ndim, maxdx, iperiod

c     iperiod   Parameter indicating if periodical boundary conditions
c               are present per boundary
c               iperiod is a 3 digit number with the sequence 321
c               Each digit refers to 1 direction
c               The digits have the following meaning
c               0:  no periodical boundary conditions
c               1:  symmetrical periodical boundary conditions
c               2:  anti-symmetrical periodical boundary conditions
c     maxdx     Maximum relative shift between cell indices
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISFILM01  Initialize mct and mctinv
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Extract parameters from iinput
c    Fill first part of the information arrays (general information)
c    Fill second part of the information arrays (addresses)
c    Fill third part of the information arrays (information arrays)
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Extract parameters from iinput

      ni      = iinput(51)
      nj      = iinput(52)
      nk      = iinput(53)
      iperiod = iinput(56)
      ndim    = iinput(57)
      maxdx   = iinput(65)

c     --- FIRST PART , common part

      infmm(1) = iperiod
      infmm(2) = ni
      infmm(3) = nj
      infmm(4) = nk
      infmm(5) = ndim
      if ( ndim.eq.2 ) then
         if ( itypsten.eq.2 ) then
            infmm(6) = 21
         else if ( itypsten.eq.3 ) then
            infmm(6) = 17
         else
            infmm(6) = 13
         end if
      else
         if ( itypsten.eq.2 ) then
            infmm(6) = 59
         else
            infmm(6) = 51
         end if
      end if
      infmm(7) = (ni+1)*(nj+1)*(nk+1)*ndim
      infmm(8) = 1
      infmm(9) = 0
      infmm(10) = 1

      infmg(1) = iperiod
      infmg(2) = ni
      infmg(3) = nj
      infmg(4) = nk
      infmg(5) = ndim
      if ( ndim.eq.2 ) then
         infmg(6) = 6
      else
         infmg(6) = 10
      end if
      infmg(7) = (ni+1)*(nj+1)*(nk+1)*ndim
      infmg(8) = 2
      infmg(9) = 0
      infmg(10) = 1

      infmt(1) = iperiod
      infmt(2) = ni
      infmt(3) = nj
      infmt(4) = nk
      infmt(5) = ndim
      if ( ndim.eq.2 ) then
         infmt(6) = 9
      else
         infmt(6) = 19
      end if
      infmt(7) = (ni+1)*(nj+1)*(nk+1)
      infmt(8) = 3
      infmt(9) = 0
      infmt(10) = 1

c     --- SECOND PART , addresses
c         starting addresses for mct , mctinv and mst

      infmm(11) = 14
      infmm(12) = infmm(11) + ndim*ndim*(maxdx*2+1)*(maxdx*2+1)*
     +   (maxdx*2+1)
      infmm(13) = infmm(12) + 4*ndim*infmm(6)
      infmg(11) = 14
      infmg(12) = infmg(11) + ndim*(maxdx*2+1)*(maxdx*2+1)*
     +   (maxdx*2+1)
      infmg(13) = infmg(12) + 4*ndim*infmg(6)
      infmt(11) = 14
      infmt(12) = infmt(11) + (maxdx*2+1)*(maxdx*2+1)*
     +   (maxdx*2+1)
      infmt(13) = infmt(12) + 4*infmt(6)

c     --- THIRD PART , matrices

      call isfilm01 ( ndim, infmm(6), infmg(6), infmt(6),
     +                infmm(infmm(11)), infmg(infmg(11)),
     +                infmt(infmt(11)), infmm(infmm(12)),
     +                infmg(infmg(12)), infmt(infmt(12)),
     +                infmm(infmm(13)), infmg(infmg(13)),
     +                infmt(infmt(13)),  maxdx, ni, nj, nk )

      end
