      subroutine isfilvso ( iblock , itrans, itimlv, nblocks, isiinput,
     +                      issolut, isitopol, isinfbnd, isbndcon,
     +                      icoupled, usebndcn )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 2.2    date 06-08-1998 Guus: New call to isexvc02
c        version 2.1    date 19-11-1997 Kees K: iperiod removed for 2D
c        version 2.0    date 14-10-1997 Guus: extra parameters
c        version 1.7    date 07-04-1997 Kees K.: call of isex03 changed
c        version 1.6    date 31-08-1995 Kees K.: sequence off calls of
c                                       isex03 and isexvpe3 changed
c        version 1.5    date 26-08-1995 Kees K.: itrans for 3D
c        version 1.4    date 04-08-1995 Michiel: Extrapolation in case
c                                       of periodic bc's added.
c        version 1.3    date 08-05-1995 Michiel: Loop over FACES moved
c                                       into ISFILVSO instead of ISEXS02,
c                                       so that different virtual cells are
c                                       filled per boundary.
c        version 1.2    date 30-03-1995 Michiel: ISEXSO removed
c        version 1.1    date 24-01-1995 Erik: changed idegfd into itrans
c        version 1.0    date 24-11-1994
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     This routine extrapolates the virtual points of the local
c     solution arrays of the subdomains for outer boundaries.
c **********************************************************************
c
c                       KEYWORDS
c
c   extrapolation
c   virtual
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer  iblock, itrans, itimlv, nblocks, issolut,
     +         isiinput, isitopol, isinfbnd, isbndcon, icoupled
      logical usebndcn

c     iblock    i    Block sequence number
c     icoupled  i    If 0 the momentum equations are coupled, if 1 they are not
c     isbndcon  i    Array of sequence numbers (nblocks) of array bndcon
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isinfbnd  i    Array of sequence numbers (nblocks) of array infbnd
c     isitopol  i    Sequence number of array itopol
c     issolut  i/o   Array of sequence numbers (nblocks) of array solut
c     itimlv    i    Loop variable with respect to a given time level
c     itrans    i    reserved
c     nblocks   i    Number of blocks
c     usebndcn  i    If true boundary conditions are used in the extrapolation
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ipsolut, ipiinput, ipitopol, ipnsubf, iperiod,
     +        ipsubf, maxnsubf, ndim, nfaces, ni, nj, nk,
     +        nvirtual, ndegfd, ntimlv, npointsv, iface, ipbndcon,
     +        ipinfbnd

c     iface     Face number
c     ipbndcon  Starting address of array bndcon in buffer
c     iperiod   Parameter indicating if periodical boundary conditions
c               are present per boundary
c               iperiod is a 3 digit number with the sequence 321
c               Each digit refers to 1 direction
c               The digits have the following meaning
c               0:  no periodical boundary conditions
c               1:  symmetrical periodical boundary conditions
c               2:  anti-symmetrical periodical boundary conditions
c     ipiinput  Starting address of array iinput in ibuffr
c     ipinfbnd  Starting address of array infbnd in ibuffr
c     ipitopol  Starting address of array itopol in ibuffr
c     ipnsubf   Starting address of the nsubfaces array
c     ipsolut   Starting address of array solut in buffer
c     ipsubf    Starting address of the subfaces array
c     maxnsubf  Maximum number of subfaces encountered on all subfaces
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     nfaces    Number of faces
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npointsv  Number of points in the mesh with virtual points
c     ntimlv    Number of time levels
c     nvirtual  Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isdpgetp, isingetp

c     ISACTIVE  Activate old permanent array
c     ISDCRPRI  Decrease priorities of all arrays in ibuffr
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISEX03    Extrapolation 3D
c     ISEXSO02  Compute virtual unknowns not coming from a
c               neighbouring block by extrapolation (2d case)
c     ISEXVC02  Compute virtual unknowns in corner
c               points by extrapolation (2d case)
c     ISEXVPE3  Compute virtual unknowns not coming from a
c               neighbouring block by extrapolation (3d periodic case)
c     ISINGETP  get the pointer corresponding to an integer array
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c      fill parameters from array itopol
c
c      fill parameters from array iinput
c      --- actual extrapolation
c      call isexso
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- compute pointers and some parameters for the itopol array

      call isactive(isitopol)
      ipitopol = isingetp(isitopol)
      nfaces   = ibuffr(ipitopol + 2 - 1)
      maxnsubf = ibuffr(ipitopol + 3 - 1)
      ipnsubf  = ibuffr(ipitopol+5-1) + ipitopol - 1
      ipsubf   = ibuffr(ipitopol+6-1) + ipitopol - 1

c     --- fill the  virtual unknowns by extrapolation

      call isactive(isiinput)
      call isactive(issolut)
      ipbndcon = 1
      ipinfbnd = 1
      if ( usebndcn ) then

c     --- Boundary conditions are used for the extrapolation
c         Compute startng addresses of boundary conditions arrays

         call isactive(isinfbnd)
         if ( isbndcon.gt.0 ) call isactive(isbndcon)
         ipinfbnd = isingetp(isinfbnd)
         ipbndcon = isdpgetp(isbndcon)
      end if
      ipiinput = isingetp(isiinput)
      ipsolut  = isdpgetp(issolut)

      ni       = ibuffr(ipiinput + 51 - 1)
      nj       = ibuffr(ipiinput + 52 - 1)
      nk       = ibuffr(ipiinput + 53 - 1)
      nvirtual = ibuffr(ipiinput + 54 - 1)
      ndegfd   = ibuffr(ipiinput + 55 - 1)
      iperiod  = ibuffr(ipiinput + 56 - 1)
      ndim     = ibuffr(ipiinput + 57 - 1)
      ntimlv   = ibuffr(ipiinput + 64 - 1)

      if (ndim.eq.2) then

c     --- handle external (sub)faces

         do iface = 1, nfaces

            call isexso02 ( iblock, iface, itrans, ni, nj, nvirtual,
     +                      ndegfd, itimlv, ntimlv, nblocks, nfaces,
     +                      maxnsubf, buffer(ipsolut), ibuffr(ipnsubf),
     +                      ibuffr(ipsubf), ibuffr(ipinfbnd),
     +                      ibuffr(ipinfbnd+12), buffer(ipbndcon),
     +                      icoupled, usebndcn )

         end do

c        --- handle corner points

         call isexvc02(iblock , itrans, ni, nj, nvirtual, ndegfd,
     +                 itimlv , ntimlv, nblocks, nfaces , maxnsubf,
     +                 buffer(ipsolut), ibuffr(ipnsubf),
     +                 ibuffr(ipsubf) )

      else if (ndim.eq.3) then

         npointsv= (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*
     +             (nk+1+2*nvirtual)

         if ( itrans.eq.1 ) then

            call isex03(iblock,3, ni, nj, nk, nvirtual,
     +               buffer(ipsolut-1+ndegfd*(itimlv-1)*npointsv+1),
     +               1, nvirtual, nblocks, nfaces, maxnsubf,
     +               ibuffr(ipnsubf),ibuffr(ipsubf) )

            call isex03(iblock,3, ni, nj, nk, nvirtual,
     +               buffer(ipsolut-1+ndegfd*(itimlv-1)*npointsv+
     +               npointsv+1), 2, nvirtual, nblocks,
     +               nfaces, maxnsubf, ibuffr(ipnsubf),ibuffr(ipsubf) )

            call isex03(iblock,3, ni, nj, nk, nvirtual,
     +               buffer(ipsolut-1+ndegfd*(itimlv-1)*npointsv+
     +               2*npointsv+1), 3, nvirtual, nblocks,
     +               nfaces, maxnsubf, ibuffr(ipnsubf),ibuffr(ipsubf) )

        else

            call isex03(iblock,3, ni, nj, nk, nvirtual,
     +               buffer(ipsolut-1+ndegfd*(itimlv-1)*npointsv+
     +               (itrans-1)*npointsv+1), 4, nvirtual, nblocks,
     +               nfaces , maxnsubf, ibuffr(ipnsubf),ibuffr(ipsubf) )

        end if

         if ( iperiod.ne.0 ) then

            call isexvpe3(iperiod, itrans, ni, nj,
     +                    nk, ndim, nvirtual, ndegfd, itimlv, ntimlv,
     +                    buffer(ipsolut), iblock, nblocks, nfaces,
     +                    maxnsubf, ibuffr(ipsubf))

         end if

      end if

      call isdcrpri

      end
