      subroutine isincc2 ( solut, ndegfd, nvirtual, ni, nj, eddy,
     +                     mscal, mcav, rincom, work,
     +                     istartup, rincav, iindsc, iincom )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.2    date 23-03-1999    Duncan: extend for Supersonic Scheme
c        version 1.1    date 04-04-1999    Duncan: rincav added to paralist
c        version 1.0    date 19-02-1999
c
c   copyright (c) 1999-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     Initialize array eddy for the first time step and copy to new time step
c     Perform the scaling of the velocity vector
c **********************************************************************
c
c                       KEYWORDS
c
c     initial_conditions
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      integer ni, nj, ndegfd, nvirtual, istartup, mscal, mcav,
     +        iindsc(*), iincom(*)
      double precision rincom(*),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ndegfd,2),
     +                 eddy(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,*),
     +                 work(*), rincav(*)

c     eddy           i    array EDDY contains the values of the
c                         eddy viscosity of that block in the centroids
c                         of the elements. The storage of such
c                         an array EDDY is identical to that of all other
c                         coefficients.
c     iincom         i    Array IINCOM is part of array IINPUT and hence
c                         coupled to one block only. It contains information
c                         with respect to the compressible flow
c                         For a description see the programmers guide
c     iindsc         i    integer information about type of discretization used
c                         IINDSC may be considered as a two-dimensional array of
c                         size 10 x (NTRANS+1): IINDSC(i,j), where j refers to
c                         the equation to be solved.
c                         j=1 refers to the momentum equation, j>1 to the
c                         (j-1)-th transport equation including the
c                         turbulence equations.
c                         For a description see the programmers guide
c     istartup       i    If 1 the cavity calculation is in the startup
c                         procedure, if 0 not
c     mcav           i    If 1 a cavity model is used, if 0 not
c     mscal          i    Indicates use of non-dimensionalisation for
c                         compr. flow:
c                         0:   Equations are non-dimensionalised
c                         1:   Equations are in dimensional formulation
c     ndegfd         i    Number of degrees of freedom
c     ni             i    Number of non-virtual cells in the i-direction
c                         (one block)
c     nj             i    Number of non-virtual cells in the j-direction
c                         (one block)
c     nvirtual       i    Number of rows of virtual cells
c     rincav         i    Array RINCAV is part of array RINPUT and hence is
c                         coupled to one block only.
c                         For a description see the programmers guide
c     rincom         i    Array RINCOM is part of array IINPUT and hence
c                         coupled to one block only. It contains information wit
c                         respect to the compressible flow
c                         For a description see the programmers guide
c     solut         i/o   Solution array
c                         For a description see the programmers guide
c     work                Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c     none
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer  frsti, frstj, lasti, lastj, ip, jp, vartyp, kk,
     +         newtcom, meqstate, menergy, ndim, nk

c     frsti          First index in i-direction
c     frstj          First index in j-direction
c     ip             Local counting variable i-direction
c     jp             Local counting variable j-direction
c     kk             Loop variable
c     lasti          Last index in i-direction
c     lastj          Last index in j-direction
c     menergy        Defines the type of energy equation used.
c                    Possible values:
c                    0: none
c                    1: the primary unknown is the enthalpy
c                    2: the primary unknown is the total enthalpy
c                    3: the primary unknown is the internal energy
c     meqstate       Defines the type of equation of state
c                    See the programmers Guide array IINCOM for the
c                    possible values
c     ndim           Dimension of space (2 or 3)
c     newtcom        If one Newton linearization is used for convective
c                    terms if compressible flow.
c     nk             Number of non-virtual cells in the k-direction
c                    (one block)
c     vartyp         Variable to be interpolated/extrapolated
c                    vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c                    0 for vertex (for ndim=2)
c                    or
c                    vartyp: 1 for V1, 2 for V2 3, for V3 and
c                    4 for pressure, 0 for vertex  (for ndim=3)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     DCOPY          Copy double precision array into other one (BLAS subroutine
c     ISCMRHO2       Computation of density, RHO ,from the equation of state, 2D
c     ISCMVEL2       Adapt velocity components for compressible flow
c     ISERCLOS       Resets old name of previous subroutine of higher level
c     ISEROPEN       Produces concatenated name of local subroutine
c     ISEXTCF2       Extrapolates a coefficient to virtual points
c     USCOMPINTEN    user written subroutine to compute the internal energy
c                    from the temperature
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     compute initial density : eddy(*,*,2)
c
c     extrapolate density to virtual unknowns
c
c     compute initial velocity = initial flux/density
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('isincc2')

      if (iindsc(8).eq.1) then
         newtcom  = 1
      else
         newtcom  = 0
      end if
      meqstate = iincom(9)
      menergy  = iincom(10)
      ndim = 2
      nk = 0

c     --- fill eddy

      frsti = 1-nvirtual
      frstj = 1-nvirtual
      lasti = ni + 1 + nvirtual
      lastj = nj + 1 + nvirtual
      do kk = 1, 6
         do jp = frstj, lastj
            do ip = frsti, lasti
               eddy(ip,jp,kk) = 1d0
            end do
         end do
      end do

c     --- Copy e to T

      call dcopy ( (ni+1+2*nvirtual)*(nj+1+2*nvirtual),
     +             solut(1-nvirtual,1-nvirtual,4,1), 1,
     +             eddy(1-nvirtual,1-nvirtual,7), 1 )

c     --- if compressible then compute initial density and correct
c         initial velocity components for this.

      frsti = 1
      lasti = ni
      frstj = 1
      lastj = nj

      call iscmrho2 ( ni, nj, ndegfd, 2, nvirtual, solut,
     +                rincom, eddy, frsti, lasti, frstj,
     +                lastj, mscal, mcav, 2, istartup, rincav, iincom )

      if ( menergy.eq.3 .and. meqstate.eq.1 ) then

c     --- Use internal energy and user provided equation of state
c         Compute internal energy from the temperature and the pressure

         call uscompinten ( eddy, solut, ni, nj, nk, nvirtual,
     +                      ndim, ndegfd )

      end if
      vartyp = 3

      call isextcf2 ( eddy(1-nvirtual,1-nvirtual,2),
     +                ni, nj, nvirtual, vartyp )

c     --- copy solution computed time level to first time level.

      do ip= 1-nvirtual, ni+1+nvirtual
         do jp= 1-nvirtual, nj+1+nvirtual
            eddy(ip,jp,1) = eddy(ip,jp,2)
         end do
      end do

c     --- store the initial fluxes in eddy(,3) and eddy(,4)

      do ip= 1-nvirtual, ni+1+nvirtual
         do jp= 1-nvirtual, nj+1+nvirtual
            eddy(ip,jp,3) = solut(ip,jp,1,2)
         end do
      end do

      do ip= 1-nvirtual, ni+1+nvirtual
         do jp= 1-nvirtual, nj+1+nvirtual
            eddy(ip,jp,4) = solut(ip,jp,2,2)
         end do
      end do

c     --- if use is made of Newton linearised momentum equation with
c         density predictor step than put initial density in solut(,4)

      if( newtcom.eq.1) then

          do ip= 1-nvirtual, ni+1+nvirtual
             do jp= 1-nvirtual, nj+1+nvirtual
                solut(ip,jp,4,1) = eddy(ip,jp,2)
             end do
          end do

          do ip= 1-nvirtual, ni+1+nvirtual
             do jp= 1-nvirtual, nj+1+nvirtual
                solut(ip,jp,4,2) = eddy(ip,jp,2)
             end do
          end do

      end if

c     --- since the initial conditions are not scaled yet, the
c         initial velocity has to be computed from the initial flux,
c         u=flux/eddy

      call iscmvel2 ( ni, nj, nvirtual,
     +                solut(1-nvirtual,1-nvirtual,1,2),
     +                eddy(1-nvirtual,1-nvirtual,2), work )

c     --- copy solution computed time level to first time level.

      do jp = 1-nvirtual, nj+1+nvirtual
         do ip = 1-nvirtual, ni+1+nvirtual
            solut(ip,jp,1,1) = solut(ip,jp,1,2)
         end do
      end do

      do jp = 1-nvirtual, nj+1+nvirtual
         do ip = 1-nvirtual, ni+1+nvirtual
            solut(ip,jp,2,1) = solut(ip,jp,2,2)
         end do
      end do

      call iserclos('isincc2')

      end
