      subroutine isinccp2 ( coarse, fine, solut, coor, ndegfd,
     +                      nvirtual, ni , nj, ndim, ntimlv)
c ======================================================================
c
c     programmer   Kees Kassels
c     version      1.0    date    19-04-1996
c
c   copyright (c) 1996  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Refined initial condition from coarse grid solution
c
c **********************************************************************
c
c                       KEYWORDS
c
c     initial_conditions
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      integer ni, nj, ndim, ntimlv, ndegfd, nvirtual
      double precision coarse(1:ni/2+1,1:nj/2+1,1:ndegfd),
     +                 fine(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ndegfd),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ndegfd, 1:ntimlv),
     +                 coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual, 1:ndim)

c     coarse    i    Solution on coarse grid
c     coor      i    reserved
c     fine      i    Solution on fine grid
c     ndegfd    i    Number of degrees of freedom
c     ndim      i    Dimension of space (2 or 3)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     ntimlv    i    Number of time levels
c     nvirtual  i    Number of rows of virtual cells
c     solut     i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer  idegfd, frsti, frstj, lasti, lastj,
     +         itimlv, ip, jp, j, p, q
      double precision acov(1:3,1:3), u(1:2)

c     acov      Covariant base vectors.
c               Meaning: a(i,j) gives the value of the i-th
c                        component of the j-th base vector
c     frsti     First index in i-direction
c     frstj     First index in j-direction
c     idegfd    Loop variable with respect to a given unknown
c     ip        Local counting variable i-direction
c     itimlv    Loop variable with respect to a given time level
c     j         General loop variable
c     jp        Local counting variable j-direction
c     lasti     Last index in i-direction
c     lastj     Last index in j-direction
c     p         Index in covariant base vectors
c     q         Index in covariant base vectors
c     u         Cartesian velocities
c **********************************************************************
c
c                       I/O
c
c     none
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c **********************************************************************
c
c                       PSEUDO CODE
c
c      copy solution from coarse grid to fine
c      interpolate missing values
c      interpolate and compute contravariant fluxes in flux points
c      interpolate the other values from vertices to the centre of
c      the grid cells
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('isinccp2')

c     --- copy solution from coarse grid to fine

      do  idegfd = 1, ndegfd
         do jp = 1, nj+1, 2
            do ip = 1, ni+1, 2
               fine( ip, jp, idegfd ) =
     +                        coarse(1+(ip-1)/2,1+(jp-1)/2,idegfd)
            end do
         end do
      end do

c     --- interpolate missing values

      do  idegfd = 1, ndegfd
         do jp = 1, nj+1, 2
            do ip = 2, ni, 2
               fine( ip, jp, idegfd ) = 5d-1 *
     +                                  ( fine((ip-1),jp,idegfd) +
     +                                    fine((ip+1),jp,idegfd) )
            end do
         end do
      end do

      do  idegfd = 1, ndegfd
         do ip = 1, ni+1, 2
            do jp = 2, nj, 2
               fine( ip, jp, idegfd ) = 5d-1 *
     +                                  ( fine(ip,(jp-1),idegfd) +
     +                                    fine(ip,(jp+1),idegfd) )
            end do
         end do
      end do

      do  idegfd = 1, ndegfd
         do jp = 2, nj, 2
            do ip = 2, ni, 2
               fine( ip, jp, idegfd ) = 25d-2 *
     +                              ( fine((ip-1),(jp-1),idegfd) +
     +                                fine((ip-1),(jp+1),idegfd) +
     +                                fine((ip+1),(jp-1),idegfd) +
     +                                fine((ip+1),(jp+1),idegfd) )
           end do
         end do
      end do

      do 10 idegfd = 1, ndegfd

         if ( idegfd.le.ndim ) then

c        --- Compute covariant base vectors needed for computation
c            computation of contravariant fluxes.

            if ( idegfd.eq.1 ) then
               lasti = ni + 1
               lastj = nj
            else if ( idegfd.eq.2 ) then
               lasti = ni
               lastj = nj + 1
            end if

            do 20 jp = 1, lastj
               do 20 ip = 1, lasti

                  do j = 1, ndim
                     acov(j,1) = coor(ip+1,jp,j) - coor(ip,jp,j)
                     acov(j,2) = coor(ip,jp+1,j) - coor(ip,jp,j)
                     acov(j,3) = 0d0
                  end do

                  acov(3,1) = 0d0
                  acov(3,2) = 0d0
                  acov(3,3) = 1d0

                  do  j = 1, ndim
                        u(j) = 5d-1 * ( fine(ip,jp,j) +
     +                               fine(ip+idegfd-1,jp+2-idegfd,j))
                  end do

                  p = mod( idegfd+1,3 )
                  q = mod( idegfd+2,3 )
                  if ( p.eq.0 ) p = 3
                  if ( q.eq.0 ) q = 3

                  solut(ip,jp,idegfd,1) =  ( acov(2,p)*acov(3,q) -
     +                                   acov(3,p)*acov(2,q) ) * u(1)
     +                               + ( acov(3,p)*acov(1,q) -
     +                                   acov(1,p)*acov(3,q) ) * u(2)
20        continue

        else

          lasti = ni
          lastj = nj

          do 30 jp = 1, lastj
             do 30 ip = 1, lasti

                solut(ip,jp,idegfd,1) = 25d-2* ( fine(ip,jp,idegfd) +
     +                                  fine(ip,jp+1,idegfd) +
     +                                  fine(ip+1,jp,idegfd) +
     +                                  fine(ip+1,jp+1,idegfd) )

30          continue

         end if

c        --- copy timelevel 1 to the other timelevels

         frsti = 1-nvirtual
         frstj = 1-nvirtual
         lasti = ni + 1 + nvirtual
         lastj = nj + 1 + nvirtual

         do itimlv = 2, ntimlv
            do  jp = frstj, lastj
               do  ip = frsti, lasti
                  solut( ip, jp, idegfd, itimlv ) =
     +                             solut( ip, jp, idegfd, 1 )
               end do
            end do
         end do

10    continue

      call iserclos('isinccp2')

      end
