      subroutine isincnd ( isiinput, isrinput, isitopol, iadmin,
     +                     iscoor, issolut, iseddy, isgeom )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 3.7    date 21-07-1999 Guus: initialize mcav correctly
c        version 3.6    date 23-03-1999 Kees K: new call iscptl
c        version 3.5    date 03-03-1999 Duncan: new call to iscmrho2
c                                               new call to isincd2
c        version 3.4    date 19-02-1999 Guus: new calls to isincd2/3
c        version 3.3    date 06-08-1998 Guus: new call to isl2t
c        version 3.2    date 15-06-1998 Duncan: new call to iscmrho2
c                                               copy eddy(2) to eddy(1)
c                                               initialize rest of eddy
c                                               new call to isincd2
c        version 3.1    date 09-06-1998 Charles: nvpoints changed
c                                                to npointsv
c        version 3.0    date 21-04-1998 KeesK: wrong declaration of l2t
c        version 2.9    date 02-04-1998 KeesK: l2t added
c        version 2.8    date 05-03-1998 Duncan: extension for cavity
c        version 2.7    date 12-11-1997 Kees K: activate array rinput for
c                                               compressible case
c        version 2.6    date 12-11-1997 Hester: new call to iscmrho2
c        version 2.5    date 05-11-1997 Charles: collocated allowed
c        version 2.4    date 13-10-1997 Charles: uncoupled allowed
c        version 2.3    date 22-08-1997 Fred T: read shear stress
c                                       for smagorinsky model
c        version 2.2    date 20-05-1996 Hester: restart adjusted for
c                                               compr. flow
c        version 2.1    date 19-04-1996 Kees K.: refine option
c        version 2.0    date 07-03-1996 Kees K.: indref replaced
c
c   copyright (c) 1994-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     Set initial conditions
c
c **********************************************************************
c
c                       KEYWORDS
c
c     initial_conditions
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      integer iadmin(*), isiinput(*), isrinput(*), iscoor(*), isitopol,
     +        issolut(*), iseddy(*), isgeom(*)

c     iadmin         i    contains information about
c                         which blocks on which computer are stored.
c     iscoor         i    Array of sequence numbers (nblocks) of array coor
c     iseddy         i    Array of sequence numbers (nblocks) of array eddy
c     isgeom         i    Array of sequence numbers (nblocks) of array geom
c     isiinput       i    Array of sequence numbers (nblocks) of array iinput
c     isitopol       i    Sequence number of array itopol
c     isrinput       i    Array of sequence numbers (nblocks) of array rinput
c     issolut        i    Array of sequence numbers (nblocks) of array solut
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iblock, iequa, ilev, ipcoor, ipiincnd, ipiinput, ipitopol,
     +        iprincnd, iprinput, ipsolut, ipwork,  iswork, ipwork2,
     +        iswork2, ipwork3, iswork3, indref,
     +        isworkr, ipworkr, lworkr,
     +        itimlv, itrans, iunder, lengthc, lentopol, lwork, lwork2,
     +        lwork3, nblocks, maxvpnts,
     +        ndegfd, ndim, neq, ni, nj, nk, ntimlv, nvirtual, ipeddy,
     +        ipiincom, iprincom, npointsv, mcom, refine, vartyp,ipgeom,
     +        ipiintur, mturb, npoint, icoupled, iseqpres, mshtyp,
     +        mscal, mcav, istartup, ipiincav, iprincav, istrt,
     +        iend, jstrt, jend
      logical usebndcn, l2t

c     iblock         Block sequence number
c     icoupled       If 0 the momentum equations are coupled,
c                    if 1 they are not
c     iend           End value for a loop variable
c     iequa          General equation sequence number
c     ilev           Time level of the multi-block transport of information
c     indref         Record number from which inparray has to be read
c     ipcoor         Starting address of array coor in buffer
c     ipeddy         Starting address of array eddy in buffer
c     ipgeom         Starting address of array geom in buffer
c     ipiincav       Starting address of array IINCAV in array IINPUT
c     ipiincnd       Starting address of array iincnd in array iinput
c     ipiincom       Starting address of array IINCOM in array IINPUT
c     ipiinput       Starting address of array iinput in ibuffr
c     ipiintur       Starting address of array IINTUR in array IINPUT
c     ipitopol       Starting address of array itopol in ibuffr
c     iprincav       Starting address of array RINCAV in array RINPUT
c     iprincnd       Starting address of array rincnd in array rinput
c     iprincom       Starting address of array RINCOM in array RINPUT
c     iprinput       Starting address of array rinput in buffer
c     ipsolut        Starting address of array solut in buffer
c     ipwork         Starting address of array work in buffer
c     ipwork2        Starting address of array work2 in buffer
c     ipwork3        Starting address of array work3 in buffer
c     ipworkr        Starting address of array workr in buffer
c     iseqpres       Sequence number of pressure equation
c     istartup       If 1 the cavity calculation is in the startup
c                    procedure, if 0 not
c     istrt          Starting value for a loop variable
c     iswork         Sequence number of array work
c     iswork2        Sequence number of array work2
c     iswork3        Sequence number of array work3
c     isworkr        Sequence number of array workr
c     itimlv         Loop variable with respect to a given time level
c     itrans         indicates the quantities to be transported
c                    for multi block
c     iunder         First element in inparray to be read
c     jend           End value for a loop variable
c     jstrt          Starting value for a loop variable
c     l2t            True if restarting from laminar to turbulent
c     lengthc        Length parameter for issend and isreceiv
c     lentopol       Length of the itopol array
c     lwork          Length of array work
c     lwork2         Length of array work2
c     lwork3         Length of array work3
c     lworkr         Length of array workr
c     maxvpnts       Maximum number of points per block including virtual
c                    ones
c     mcav           If 1 a cavity model is used, if 0 not
c     mcom           If zero the flow is incompressible else (1)
c                    compressible
c     mscal          Indicates use of non-dimensionalisation for
c                    compr. flow:
c                    0:   Equations are non-dimensionalised
c                    1:   Equations are in dimensional formulation
c     mshtyp         Type of grid arrangement
c     mturb          Indicates type of turbulence modelling
c     nblocks        Number of blocks
c     ndegfd         Number of degrees of freedom
c     ndim           Dimension of space (2 or 3)
c     neq            Number of equations
c     ni             Number of non-virtual cells in the i-direction
c                    (one block)
c     nj             Number of non-virtual cells in the j-direction
c                    (one block)
c     nk             Number of non-virtual cells in the k-direction
c                    (one block)
c     npoint         Number of points for one block
c     npointsv       Number of points in the mesh with virtual points
c     ntimlv         Number of time levels
c     nvirtual       Number of rows of virtual cells
c     refine         refine=1 : refine refine=0 : not
c     usebndcn       If true boundary conditions are used in the
c                    extrapolation
c     vartyp         Variable to be interpolated/extrapolated
c                    vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c                    0 for vertex (for ndim=2)
c                    or
c                    vartyp: 1 for V1, 2 for V2 3, for V3 and
c                    4 for pressure, 0 for vertex  (for ndim=3)
c **********************************************************************
c
c                       I/O
c
c     none
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isdpgetp, isingetp, isinleng, isdpleng, isinisgl, isinidbl

c     ISACTIVE  Activate old permanent array
c     ISCMRHO2  Computation of density, RHO ,from the equation of state, 2D
c     ISCPTL    Copy timelevel
c     ISDCREAT  Create permanent space in array IBUFFR (Double precision array)
c     ISDELARR  Delete a permanent array
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISDPLENG  Return with the length of the array in double precisions
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISEXTCF2  Extrapolates a coefficient to virtual points
c     ISFILVSO  Extrapolate solution at outer boundaries
c     ISICREAT  Create permanent space in array IBUFFR (Integer array)
c     ISINCCP2  Refined initial condition from coarse grid solution
c     ISINCD2   Put initial conditions into SOLUT array (2D)
c     ISINCD3   Put initial conditions into SOLUT array (3D)
c     ISINGETP  get the pointer corresponding to an integer array
c     ISINIDBL  Recompute pointer for ibuffr to pointer for buffer
c     ISINIRDF  Startup of reading the restart file
c     ISINISGL  Recompute pointer for buffer to pointer for ibuffr
c     ISINLENG  Return with the length of the array in integers
c     ISL2T     Read initial condition from laminar solution to turbulent
c     ISMBLK    Performs the local transport of multi-block information
c               within a node
c     ISREADFL  Read from direct access file
c     ISSETCON  Make array equal to a constant
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     none
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Important remark:
c
c     At this moment the initial conditions are filled and copied to the other
c     time levels. After that the extrapolation to other blocks is carried
c     out.
c     In fact this is the wrong sequence:
c     - First everything must be filled
c     - Then the extrapolation and copying over block boundaries must be
c       performed
c     - Finally the copying to other time levels must be carried out
c
c     This must be improved in the future
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- determine some general parameters

      call iseropen( 'isincnd' )
      call isactive(isiinput(1))
      ipiinput = isingetp( isiinput(1) )

      nblocks  = ibuffr(ipiinput-1+73)
      nvirtual = ibuffr(ipiinput-1+54)
      ndegfd   = ibuffr(ipiinput-1+55)
      ndim     = ibuffr(ipiinput-1+57)
      ntimlv   = ibuffr(ipiinput-1+64)
      mshtyp   = ibuffr(ipiinput-1+75)
      icoupled = ibuffr(ipiinput-1+76)
      mcav     = ibuffr(ipiinput-1+77)
      if ( icoupled.eq.0 ) then
         iseqpres = 2
      else
         iseqpres = ndim + 1
      end if

      ipiincnd = ibuffr( ipiinput -1 + 3  )

      if ( ibuffr( ipiinput + ipiincnd -1).eq.2 ) then

c     --- read initial conditions from RESTART FILE

         iswork = 0
         lwork = nrec2
         call isactive( isrinput(1) )
         call isactive( isitopol )
         call isicreat(iswork, lwork, 'work' )
         ipwork  = isingetp( iswork )
         iprinput = isdpgetp( isrinput(1) )
         ipitopol = isingetp( isitopol )
         lentopol = isinleng( isitopol )

c        --- initialize reading restart file

         refine = ibuffr( ipiinput + ipiincnd -1 + ndim)

         call isinirdf ( ibuffr(ipiinput), buffer(iprinput),
     +                   ibuffr(ipitopol), lentopol, ibuffr(ipwork),
     +                   lwork, refine )

c        --- read solution from restart file

         if ( refine.eq.1 ) then

            iswork2 = 0
            iswork3 = 0
            maxvpnts  = ibuffr( ipiinput -1 + 69 )
            lwork2 = maxvpnts * ndegfd
            lwork3 = maxvpnts * ndegfd
            call isdcreat( iswork2, lwork2, 'work2')
            call isdcreat( iswork3, lwork3, 'work3')

            do iblock = 1, nblocks

               call isactive(isiinput(iblock))
               ipiinput = isingetp(isiinput(iblock))
               ni       = ibuffr( ipiinput -1 + 51 )
               nj       = ibuffr( ipiinput -1 + 52 )
               nk       = ibuffr( ipiinput -1 + 53 )
               ndegfd   = ibuffr( ipiinput -1 + 55 )
               ndim     = ibuffr( ipiinput -1 + 57 )
               lengthc  = (ni/2+1)*(nj/2+1)*ndegfd
               if ( ndim.eq.3) then
                  lengthc = lengthc*(nk/2+1)
               end if
               iunder = 1
               indref = ifree2
               call isactive( iswork2)
               ipwork2 = isdpgetp(iswork2)
               ipwork2 = isinisgl(ipwork2)
               lengthc = lengthc*intlen
               call isreadfl(indref, iunder, lengthc, iref2,
     +                       ibuffr(ipwork2) )

               ipwork2 = isinidbl(ipwork2)
               call isactive( iswork3)
               ipwork3 = isdpgetp(iswork3)

               call isactive(iscoor(iblock))
               call isactive(issolut(iblock))
               ipcoor  = isdpgetp(iscoor(iblock))
               ipsolut = isdpgetp(issolut(iblock))

                if ( ndim.eq.2 ) then

                   call isinccp2( buffer(ipwork2), buffer(ipwork3),
     +                            buffer(ipsolut), buffer(ipcoor),
     +                            ndegfd, nvirtual, ni, nj,
     +                            ndim, ntimlv )
                end if

c               --- initialize the virtual unknowns of the neighbouring
c                   blocks by transporting them to the neighbouring blocks

                ilev = 1
                do itrans = 1, ndegfd + ( 2 - ndim )*( 1 - icoupled )
                   call ismblk(iblock,   itrans, ilev, isiinput,
     +                         issolut, iadmin, isitopol)
                end do

c               --- extrapolate solution at outer boundaries

                neq      = ndegfd + ( 1 - ndim )*( 1 - icoupled )
                do iequa = 1, neq

                   if ( iequa.lt.iseqpres ) then
                      itrans = 1
                   else
                      itrans = iequa + ( ndim - 1 )*( 1 - icoupled )
                   end if
                   itimlv = 1

                   usebndcn = .false.
                   call isfilvso ( iblock, itrans, itimlv, nblocks,
     +                             isiinput(iblock), issolut(iblock),
     +                             isitopol, isitopol, isitopol,
     +                             icoupled, usebndcn )
                end do

            end do

            call isdelarr(iswork3)
            call isdelarr(iswork2)
            call isdelarr(iswork)

c           --- Copy time level 1 to time level 2

            call iscptl( isiinput, isrinput, issolut, iseddy )

         else

            do iblock = 1, nblocks
               call isactive(isiinput(iblock))
               ipiinput = isingetp(isiinput(iblock))
               ni       = ibuffr( ipiinput -1 + 51 )
               nj       = ibuffr( ipiinput -1 + 52 )
               nk       = ibuffr( ipiinput -1 + 53 )

               call isactive( issolut(iblock) )

               ipsolut  = isdpgetp( issolut(iblock) )
               lengthc  = isdpleng( issolut(iblock) )
               ipsolut  = isinisgl(ipsolut)
               lengthc  = lengthc * intlen
               iunder   = 1
               indref = ifree2

               l2t = refine.eq.2
               if ( l2t ) then

                  lworkr = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*
     +                     (ndegfd-2)*ntimlv
                  call isdcreat( isworkr, lworkr, 'workr')
                  ipworkr = isdpgetp(isworkr)
                  ipworkr = isinisgl(ipworkr)
                  lengthc = lworkr * intlen
                  call isreadfl(indref, iunder, lengthc, iref2,
     +                          ibuffr(ipworkr) )
                  ipworkr = isinidbl(ipworkr)
                  ipsolut = isinidbl(ipsolut)
                  call isl2t( buffer(ipworkr), buffer(ipsolut),
     +                        ndegfd, nvirtual, ni, nj, ntimlv)
                  call isdelarr( isworkr)

               else

                  call isreadfl(indref, iunder, lengthc, iref2,
     +                           ibuffr(ipsolut) )
               end if

               ipiinput = isingetp(isiinput(iblock))
               ipiincom = ibuffr( ipiinput -1 + 9 )
               ipiintur = ibuffr( ipiinput -1 + 8)
               mcom = ibuffr(ipiinput-1+ipiincom)
               mscal = ibuffr(ipiinput-1+4+ipiincom)
               mturb = ibuffr(ipiinput-1+ipiintur+1)

               if (mturb.eq.4) then
                  npoint = (1+2*nvirtual+ni)*(1+2*nvirtual+nj)*
     +                     (1+2*nvirtual+nk)
                  ipgeom = isdpgetp( isgeom(iblock) )
                  ipgeom= ipgeom + 8*2*npoint + npoint
                  ipgeom= isinisgl(ipgeom)
                  indref = ifree2
                  lengthc = (2*ni*nj+2*nj*nk+2*ni*nk+1)*intlen
                  iunder = 1

                  call isreadfl ( indref, iunder, lengthc, iref2,
     +                            ibuffr(ipgeom) )

               end if

c              --- If compressible flow, compute restart value of
c                  density

               if ( mcom.eq.1 ) then

                 call isactive( isrinput(iblock) )

                 iprinput = isdpgetp(isrinput(iblock))
                 iprincom = ibuffr( ipiinput -1 + 34)
                 iprincav = ibuffr( ipiinput -1 + 35)
                 ipsolut  = isdpgetp(issolut(iblock))
                 if ( mcav.eq.1 ) then
                    ipiincav = ibuffr( ipiinput -1 + 11)
                    istartup = ibuffr( ipiinput -1 + ipiincav -1 + 2)
                 else
                    istartup = 0
                 end if

                 call isactive(iseddy(iblock))
                 ipeddy = isdpgetp(iseddy(iblock))

                 ni       = ibuffr( ipiinput -1 + 51 )
                 nj       = ibuffr( ipiinput -1 + 52 )
                 ndegfd   = ibuffr( ipiinput + 55 - 1)
                 ndim     = ibuffr( ipiinput + 57 - 1)
                 if ( ndim.eq.2 ) then
                   npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
                 end if

c                --- fill eddy(..,2)

                 istrt = 1
                 iend  = ni
                 jstrt = 1
                 itimlv = 2
                 jend = nj
                 call iscmrho2 ( ni, nj, ndegfd, ndim, nvirtual,
     +                           buffer(ipsolut),
     +                           buffer(iprinput-1+iprincom),
     +                           buffer(ipeddy), istrt, iend, jstrt,
     +                           jend, mscal, mcav, itimlv, istartup,
     +                           buffer(iprinput-1+iprincav),
     +                           ibuffr(ipiinput-1+ipiincom) )

                 vartyp = 3

c                --- extrapolate density (in eddy(..,2) ),
c                    to virtual points

                 call isextcf2(buffer(ipeddy+npointsv),ni,nj,
     +                   nvirtual,vartyp )

c                --- fill eddy(..,1)

                 istrt = 1
                 iend = ni
                 jstrt = 1
                 jend = nj
                 itimlv = 1

                 call iscmrho2 ( ni, nj, ndegfd, ndim, nvirtual,
     +                           buffer(ipsolut),
     +                           buffer(iprinput-1+iprincom),
     +                           buffer(ipeddy), istrt, iend, jstrt,
     +                           jend, mscal, mcav, itimlv, istartup,
     +                           buffer(iprinput-1+iprincav),
     +                           ibuffr(ipiinput-1+ipiincom) )

c                --- extrapolate density (in eddy(..,1) ),
c                    to virtual points

                 call isextcf2 ( buffer(ipeddy), ni, nj,
     +                           nvirtual, vartyp )

c                --- initialize the momentum
c                     components of the last timestep for
c                      use in the defect correction

                 call issetcon(buffer(ipeddy+2*npointsv),
     +                                   4*npointsv,0d0)

                 call isextcf2(buffer(ipeddy),ni,nj,
     +                   nvirtual,vartyp )

               end if

            end do

            call isdelarr(iswork)

         end if

      else

         do iblock = 1, nblocks

c        --- activate arrays

            call isactive(isiinput(iblock))
            call isactive(isrinput(iblock))
            call isactive(issolut(iblock))
            call isactive(iscoor(iblock))
            ipiinput = isingetp(isiinput(iblock))
            ipiincom = ibuffr( ipiinput -1 + 9 )
            mcom = ibuffr(ipiinput-1+ipiincom)
            if ( mcom.eq.1 ) call isactive(iseddy(iblock))
            if(mcav.eq.1) then
               ipiincav = ibuffr( ipiinput -1 + 11)
               istartup = ibuffr( ipiinput -1 + ipiincav -1 + 2)
            else
               istartup = 0
            end if

c           --- determine some general parameters for initial conditions

            ipiinput = isingetp(isiinput(iblock))
            ni       = ibuffr( ipiinput -1 + 51 )
            nj       = ibuffr( ipiinput -1 + 52 )
            nk       = ibuffr( ipiinput -1 + 53 )
            ndegfd   = ibuffr( ipiinput + 55 - 1)
            ndim     = ibuffr( ipiinput + 57 - 1)
            neq      = ndegfd + ( 1 - ndim )*( 1 - icoupled )

c           --- compute pointers

            ipiinput = isingetp(isiinput(iblock))
            iprinput = isdpgetp(isrinput(iblock))
            ipiincnd = ibuffr( ipiinput -1 +  3 )
            iprincnd = ibuffr( ipiinput -1 + 28 )
            ipcoor   = isdpgetp(iscoor(iblock))
            ipsolut  = isdpgetp(issolut(iblock))

c           --- put initial conditions into solut array

            if ( ndim.eq.2 ) then

               call isincd2 ( ibuffr(ipiinput + ipiincnd -1),
     +                        buffer(iprinput + iprincnd -1),
     +                        buffer(ipsolut) , buffer(ipcoor),
     +                        ndegfd, nvirtual, ni, nj, mshtyp )

            else if ( ndim.eq.3 ) then

              call isincd3 ( ibuffr(ipiinput + ipiincnd -1),
     +                       buffer(iprinput + iprincnd -1),
     +                       buffer(ipsolut) , buffer(ipcoor),
     +                       ndegfd, nvirtual, ni, nj, nk )

            end if

         end do

c        --- initialize the virtual unknowns of the neighbouring blocks
c            by transporting them to the neighbouring blocks

         ilev = 1

         do iblock = 1, nblocks

            do itrans = 1, ndegfd + ( 2 - ndim )*( 1 - icoupled )

               call ismblk(iblock,   itrans, ilev, isiinput, issolut,
     +                     iadmin, isitopol)

            end do

         end do

c        --- extrapolate solution at outer boundaries

         do iblock = 1, nblocks

            do iequa = 1, neq

               if ( iequa.lt.iseqpres ) then
                  itrans = 1
               else
                  itrans = iequa + ( ndim - 1 )*( 1 - icoupled )
               end if

               itimlv = 1

               usebndcn = .false.
               call isfilvso ( iblock, itrans, itimlv, nblocks,
     +                         isiinput(iblock), issolut(iblock),
     +                         isitopol, isitopol, isitopol, icoupled,
     +                         usebndcn )

            end do

         end do

c        --- Copy time level 1 to time level 2

         call iscptl( isiinput, isrinput, issolut, iseddy )

      end if

      call iserclos( 'isincnd' )

      end
