      subroutine isinfa2d (geom, igeom, coor, ni, nj, nvirtual)
c ======================================================================
c
c        programmer   Charles Moulinec
c        version      1.0  date 11-02-1998
c
c   copyright (c) 1998-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Newton Raphson iteration, used for bilinear mapping 2D
c
c **********************************************************************
c
c                       KEYWORDS
c
c   2d
c   inverse_mapping
c   newton
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual,
     +                 igeom(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2,2)
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2),
     +                 geom(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2,2)

c     coor      i    reserved
c     geom      i    reserved
c     igeom     i    reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer p2t, idir, ip,jp, ipl, jpl, ipll, jpll,
     +        ishift, jshift, alphad, betad
      integer maxiter
      double precision coorl2(1:4,1:2), coorp(1:2), epsnr, uci, vci

c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     coorl2    Array of coordinates of 4 surrounding points
c     coorp     Array of coordinates of the surrounded point
c     epsnr     Tolerance for bilinear metric coefficients
c     idir      General loop variable for 1...ndim loop
c     ip        Local counting variable i-direction
c     ipl       Local ip value
c     ipll      Local ip value
c     ishift    Shift in i-direction
c     jp        Local counting variable j-direction
c     jpl       Local jp value
c     jpll      Local jp value
c     jshift    Shift in j-direction
c     maxiter   Maximum number of iterations
c     p2t       Local point number 1 ....4
c     uci       Local coordinate for the interpolation in the i-direction
c     vci       Local coordinate for the interpolation in the j-direction
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISINNR2D  Local bilinear interpolation
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c      determine coordinates of eight surrounding grid points
c      determine the functions of the three coordinates of the
c         center point expressed in the coordinates of the eigth
c         surrounding points and the local coordinates uci, vci,
c         and wci used for interpolation.
c      compute jacobian
c      compute uci,vci and wci with newton raphson iteration
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      maxiter=10
      epsnr=1.d-2

      do alphad= 1, 2

         ishift = alphad - 1
         jshift = 2 - alphad

c        --- calculation in the inner domain

         do jp= 2, nj-jshift
            do ip= 2, ni-ishift

               betad = 0

 1             continue

               ipl = ip + betad * ishift
               jpl = jp + betad * jshift

               ipll = ipl
               jpll = jpl

c              --- coordinates of the center of the cell

               do p2t = 1, 4
                  if( p2t.eq.2 ) then
                     ipl = ipll+1
                  else if ( p2t.eq.3 ) then
                     ipl = ipll
                     jpl = jpll+1
                  else if ( p2t.eq.4 ) then
                     ipl = ipll+1
                  end if
                  do idir= 1, 2
                     coorl2(p2t,idir) = 25d-2 *(coor(ipl-1,jpl-1,idir) +
     +                                          coor(ipl  ,jpl-1,idir) +
     +                                          coor(ipl-1,jpl  ,idir) +
     +                                          coor(ipl  ,jpl  ,idir) )
                  end do
               end do

c              --- coordinates of the center of the face

               do idir= 1, 2
                  coorp(idir) = 5d-1 * (coor(ip,jp,idir) +
     +                                  coor(ip+ishift,jp+jshift,idir) )
               end do

               call isinnr2d ( coorl2, coorp, epsnr, maxiter, uci, vci )

               if ( uci.gt.1.d0+1d-5.or.uci.lt.0.d0+1d-5.or.
     +              vci.gt.1.d0+1d-5.or.vci.lt.0.d0+1d-5 ) then
                  betad = 1
                  goto 1
               end if

               geom(ip,jp,1,alphad) = uci
               geom(ip,jp,2,alphad) = vci
               igeom(ip,jp,1,alphad) = ipl-1
               igeom(ip,jp,2,alphad) = jpl-1

            end do
         end do

c        --- calculation on the faces

         ip= 1
         do jp= 2, nj-jshift

            betad = 0

 2          continue

            ipl = ip + 1 + betad * ishift
            jpl = jp + betad * jshift

            ipll = ipl
            jpll = jpl

c           --- coordinates of the center of the cell

            do p2t = 1, 4
               if( p2t.eq.2 ) then
                  ipl = ipll+1
               else if ( p2t.eq.3 ) then
                  ipl = ipll
                  jpl = jpll+1
               else if ( p2t.eq.4 ) then
                  ipl = ipll+1
               end if
               do idir= 1, 2
                  coorl2(p2t,idir) = 25d-2 * (coor(ipl-1,jpl-1,idir) +
     +                                        coor(ipl  ,jpl-1,idir) +
     +                                        coor(ipl-1,jpl  ,idir) +
     +                                        coor(ipl  ,jpl  ,idir) )
               end do
            end do

c           --- coordinates of the center of the face

            do idir= 1, 2
               coorp(idir) = 5d-1 * (coor(ip,jp,idir) +
     +                               coor(ip+ishift,jp+jshift,idir) )
            end do

            call isinnr2d ( coorl2, coorp, epsnr, maxiter, uci, vci )

            if ( vci.gt.1.d0+1d-5.or.vci.lt.0.d0+1d-5.and.
     +           ishift.eq.0 ) then
               betad = 1
               goto 2
            end if

            geom(ip,jp,1,alphad) = uci
            geom(ip,jp,2,alphad) = vci
            igeom(ip,jp,1,alphad) = ipl-1
            igeom(ip,jp,2,alphad) = jpl-1

         end do

         jp= 1
         do ip= 2, ni-ishift

            betad = 0

 3          continue

            ipl = ip + betad * ishift
            jpl = jp + 1 + betad * jshift

            ipll = ipl
            jpll = jpl

c           --- coordinates of the center of the cell

            do p2t = 1, 4
               if( p2t.eq.2 ) then
                  ipl = ipll+1
               else if ( p2t.eq.3 ) then
                  ipl = ipll
                  jpl = jpll+1
               else if ( p2t.eq.4 ) then
                  ipl = ipll+1
               end if
               do idir= 1, 2
                  coorl2(p2t,idir) = 25d-2 * (coor(ipl-1,jpl-1,idir) +
     +                                        coor(ipl  ,jpl-1,idir) +
     +                                        coor(ipl-1,jpl  ,idir) +
     +                                        coor(ipl  ,jpl  ,idir) )
               end do
            end do

c           --- coordinates of the center of the face

            do idir= 1, 2
               coorp(idir) = 5d-1 * (coor(ip,jp,idir) +
     +                               coor(ip+ishift,jp+jshift,idir) )
            end do

            call isinnr2d ( coorl2, coorp, epsnr, maxiter, uci, vci )

            if ( uci.gt.1.d0+1d-5.or.uci.lt.0.d0+1d-5.and.
     +           jshift.eq.0 ) then
               betad = 1
               goto 3
            end if

            geom(ip,jp,1,alphad) = uci
            geom(ip,jp,2,alphad) = vci
            igeom(ip,jp,1,alphad) = ipl-1
            igeom(ip,jp,2,alphad) = jpl-1

         end do

         ip= ni+1-ishift
         do jp= 2, nj-jshift

            betad = 0

 4          continue

            ipl = ip + ishift - 1 + betad * ishift
            jpl = jp + betad * jshift

            ipll = ipl
            jpll = jpl

c           --- coordinates of the center of the cell

            do p2t = 1, 4
               if( p2t.eq.2 ) then
                  ipl = ipll+1
               else if ( p2t.eq.3 ) then
                  ipl = ipll
                  jpl = jpll+1
               else if ( p2t.eq.4 ) then
                  ipl = ipll+1
               end if
               do idir= 1, 2
                  coorl2(p2t,idir) = 25d-2 * (coor(ipl-1,jpl-1,idir) +
     +                                        coor(ipl  ,jpl-1,idir) +
     +                                        coor(ipl-1,jpl  ,idir) +
     +                                        coor(ipl  ,jpl  ,idir) )
               end do
            end do

c           --- coordinates of the center of the face

            do idir= 1, 2
               coorp(idir) = 5d-1 * (coor(ip,jp,idir) +
     +                               coor(ip+ishift,jp+jshift,idir) )
            end do

            call isinnr2d ( coorl2, coorp, epsnr, maxiter, uci, vci )

            if ( vci.gt.1.d0+1d-5.or.vci.lt.0.d0+1d-5.and.
     +           ishift.eq.0 ) then
               betad = 1
               goto 4
            end if

            geom(ip,jp,1,alphad) = uci
            geom(ip,jp,2,alphad) = vci
            igeom(ip,jp,1,alphad) = ipl-1
            igeom(ip,jp,2,alphad) = jpl-1

         end do

         jp= nj+1-jshift
         do ip= 2, ni-ishift

            betad = 0

 5          continue

            ipl = ip + betad * ishift
            jpl = jp + jshift - 1 + betad * jshift

            ipll = ipl
            jpll = jpl

c           --- coordinates of the center of the cell

            do p2t = 1, 4
               if( p2t.eq.2 ) then
                  ipl = ipll+1
               else if ( p2t.eq.3 ) then
                  ipl = ipll
                  jpl = jpll+1
               else if ( p2t.eq.4 ) then
                  ipl = ipll+1
               end if
               do idir= 1, 2
                  coorl2(p2t,idir) = 25d-2 * (coor(ipl-1,jpl-1,idir) +
     +                                        coor(ipl  ,jpl-1,idir) +
     +                                        coor(ipl-1,jpl  ,idir) +
     +                                        coor(ipl  ,jpl  ,idir) )
               end do
            end do

c           --- coordinates of the center of the face

            do idir= 1, 2
               coorp(idir) = 5d-1 * (coor(ip,jp,idir) +
     +                               coor(ip+ishift,jp+jshift,idir) )
            end do

            call isinnr2d ( coorl2, coorp, epsnr, maxiter, uci, vci )

            if ( uci.gt.1.d0+1d-5.or.uci.lt.0.d0+1d-5.and.
     +           jshift.eq.0 ) then
               betad = 1
               goto 5
            end if

            geom(ip,jp,1,alphad) = uci
            geom(ip,jp,2,alphad) = vci
            igeom(ip,jp,1,alphad) = ipl-1
            igeom(ip,jp,2,alphad) = jpl-1

         end do

c        --- calculation in the corners

         ip= 1
         jp= 1

c        --- coordinates of the center of the cell

         ipl = ip+1
         jpl = jp+1

         ipll = ipl
         jpll = jpl

         do p2t = 1, 4
            if( p2t.eq.2 ) then
               ipl = ipll+1
            else if ( p2t.eq.3 ) then
               ipl = ipll
               jpl = jpll+1
            else if ( p2t.eq.4 ) then
               ipl = ipll+1
            end if
            do idir= 1, 2
               coorl2(p2t,idir) = 25d-2 * (coor(ipl-1,jpl-1,idir) +
     +                                     coor(ipl  ,jpl-1,idir) +
     +                                     coor(ipl-1,jpl  ,idir) +
     +                                     coor(ipl  ,jpl  ,idir) )
            end do
         end do

c        --- coordinates of the center of the face

         do idir= 1, 2
            coorp(idir) = 5d-1 * (coor(ip,jp,idir) +
     +                            coor(ip+ishift,jp+jshift,idir) )
         end do

         call isinnr2d ( coorl2, coorp, epsnr, maxiter, uci, vci )

         geom(ip,jp,1,alphad) = uci
         geom(ip,jp,2,alphad) = vci
         igeom(ip,jp,1,alphad) = ipl-1
         igeom(ip,jp,2,alphad) = jpl-1

         ip= 1
         jp= nj+1-jshift

c        --- coordinates of the center of the cell

         ipl = ip+1
         jpl = jp+jshift-1

         ipll = ipl
         jpll = jpl

         do p2t = 1, 4
            if( p2t.eq.2 ) then
               ipl = ipll+1
            else if ( p2t.eq.3 ) then
               ipl = ipll
               jpl = jpll+1
            else if ( p2t.eq.4 ) then
               ipl = ipll+1
            end if
            do idir= 1, 2
               coorl2(p2t,idir) = 25d-2 * (coor(ipl-1,jpl-1,idir) +
     +                                     coor(ipl  ,jpl-1,idir) +
     +                                     coor(ipl-1,jpl  ,idir) +
     +                                     coor(ipl  ,jpl  ,idir) )
            end do
         end do

c        --- coordinates of the center of the face

         do idir= 1, 2
            coorp(idir) = 5d-1 * (coor(ip,jp,idir) +
     +                            coor(ip+ishift,jp+jshift,idir) )
         end do

         call isinnr2d ( coorl2, coorp, epsnr, maxiter, uci, vci )

         geom(ip,jp,1,alphad) = uci
         geom(ip,jp,2,alphad) = vci
         igeom(ip,jp,1,alphad) = ipl-1
         igeom(ip,jp,2,alphad) = jpl-1

         ip= ni+1-ishift
         jp= 1

c        --- coordinates of the center of the cell

         ipl = ip+ishift-1
         jpl = jp+1

         ipll = ipl
         jpll = jpl

         do p2t = 1, 4
            if( p2t.eq.2 ) then
               ipl = ipll+1
            else if ( p2t.eq.3 ) then
               ipl = ipll
               jpl = jpll+1
            else if ( p2t.eq.4 ) then
               ipl = ipll+1
            end if
            do idir= 1, 2
               coorl2(p2t,idir) = 25d-2 * (coor(ipl-1,jpl-1,idir) +
     +                                     coor(ipl  ,jpl-1,idir) +
     +                                     coor(ipl-1,jpl  ,idir) +
     +                                     coor(ipl  ,jpl  ,idir) )
            end do
         end do

c        --- coordinates of the center of the face

         do idir= 1, 2
            coorp(idir) = 5d-1 * (coor(ip,jp,idir) +
     +                            coor(ip+ishift,jp+jshift,idir) )
         end do

         call isinnr2d ( coorl2, coorp, epsnr, maxiter, uci, vci )

         geom(ip,jp,1,alphad) = uci
         geom(ip,jp,2,alphad) = vci
         igeom(ip,jp,1,alphad) = ipl-1
         igeom(ip,jp,2,alphad) = jpl-1

         ip= ni+1-ishift
         jp= nj+1-jshift

c        --- coordinates of the center of the cell

         ipl = ip+ishift-1
         jpl = jp+jshift-1

         ipll = ipl
         jpll = jpl

         do p2t = 1, 4
            if( p2t.eq.2 ) then
               ipl = ipll+1
            else if ( p2t.eq.3 ) then
               ipl = ipll
               jpl = jpll+1
            else if ( p2t.eq.4 ) then
               ipl = ipll+1
            end if
            do idir= 1, 2
               coorl2(p2t,idir) = 25d-2 * (coor(ipl-1,jpl-1,idir) +
     +                                     coor(ipl  ,jpl-1,idir) +
     +                                     coor(ipl-1,jpl  ,idir) +
     +                                     coor(ipl  ,jpl  ,idir) )
            end do
         end do

c        --- coordinates of the center of the face

         do idir= 1, 2
            coorp(idir) = 5d-1 * (coor(ip,jp,idir) +
     +                            coor(ip+ishift,jp+jshift,idir) )
         end do

         call isinnr2d ( coorl2, coorp, epsnr, maxiter, uci, vci )

         geom(ip,jp,1,alphad) = uci
         geom(ip,jp,2,alphad) = vci
         igeom(ip,jp,1,alphad) = ipl-1
         igeom(ip,jp,2,alphad) = jpl-1

      end do

      end
