      subroutine islinfmt (iinput, linfmm, linfmg, linfmt,
     +                     ni, nj, nk, ndim  )
c=======================================================================
c
c     programmer   Kees Kassels
c
c     version      1.11  date    25-10-94 Kees K. lintmp removed
c     version      1.1   date    05-09-94 RvN infmat lengths adapted
c     version      1.0   date    27-05-94
c
c   copyright (c) 1994  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c***********************************************************************
c
c                  DESCRIPTION
c
c     Compute lenghts of infmat arrays and get ni, nj, nk and ndim
c
c***********************************************************************
c
c                  KEYWORDS
c
c     lengths
c
c***********************************************************************
c

c                   INPUT / OUTPUT PARAMETERS

      implicit none

      integer maxdim
      parameter (maxdim = 3)

      integer linfmm, linfmg, linfmt
      integer iinput(*), ni, nj, nk, ndim

c     iinput    i    reserved
c     linfmg    o    Length of array infmg
c     linfmm    o    Length of array infmm
c     linfmt    o    Length of array infmt
c     maxdim    o    Maximum dimension
c     ndim      o    Dimension of space (2 or 3)
c     ni        o    Number of non-virtual cells in the i-direction (one block)
c     nj        o    Number of non-virtual cells in the j-direction (one block)
c     nk        o    Number of non-virtual cells in the k-direction (one block)
c
c***********************************************************************
c
c                   COMMON BLOCKS
c
c
c***********************************************************************
c
c                   LOCAL PARAMETERS
c
      integer maxcon, maxdx, maxmat
c
c     maxcon    Maximum number of non zero diagonals in a matrix
c     maxdx     Maximum relative shift between cell indices
c     maxmat    Maximum number of different stencils for one matrix
c
c**********************************************************************
c
c                   I/O
c
c**********************************************************************
c
c                   SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c
c**********************************************************************
c
c                   ERROR MESSAGES
c
c***********************************************************************
c
c                   PSEUDO CODE
c
c=======================================================================
c
      call iseropen('islinfmt')
c

      ni        = iinput(51)
      nj        = iinput(52)
      nk        = iinput(53)
      ndim      = iinput(57)
      maxcon    = iinput(60)
      maxmat    = iinput(62)
      maxdx     = iinput(65)

c     -- length of second part of infmat (mct)

      linfmm = ndim*ndim*(maxdx*2+1)*(maxdx*2+1)*(maxdx*2+1)
      linfmg =      ndim*(maxdx*2+1)*(maxdx*2+1)*(maxdx*2+1)
      linfmt =           (maxdx*2+1)*(maxdx*2+1)*(maxdx*2+1)

c     -- length of third part of infmat (mctinv)

      linfmm = linfmm + (maxdim+1)*ndim*maxcon
      linfmg = linfmg + (maxdim+1)*ndim*maxcon
      linfmt = linfmt + (maxdim+1)*maxcon

c     -- length of fourth part of infmat (shifts)

      linfmm = linfmm + ndim*maxcon
      linfmg = linfmg + ndim*maxcon
      linfmt = linfmt + maxcon

c     -- length of infmat

      linfmm = 10+3 + maxmat*linfmm
      linfmg = 10+3 + maxmat*linfmg
      linfmt = 10+3 + maxmat*linfmt


      call iserclos('islinfmt')

      end
