      subroutine ismvgrd ( solut, coor, ndim, ni, nj, nvirtual,
     +                      deltat, velfrsf, relax )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.0    date 31-03-1999
c
c   copyright (c) 1999-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Determine cartesian velocities in the vertices on the upper
c   boundary and the adapt the coordinates for the freesurface flow
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer  ndim, ni, nj, nvirtual
      double precision solut(*), coor(*),  velfrsf(*),
     +                 deltat,  relax

c     coor      i    Array containing the coordinates for one block
c                    In 2d coor(i,j,.) contains the x resp y-coordinate
c                    of point i,j
c                    in 3d: coor(i,j,k,.) corresponds to (i,j,k)
c     deltat    i    Product of theta and dt
c     ndim      i    Dimension of space (2 or 3)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     relax     i    Relaxation parameter
c     solut     i    reserved
c     velfrsf   i    Cartesian velocities at the upper boundary for
c                    the computation of the free surface
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRSUB  Error messages
c     ISMVG01   Adapt coordinates  of freesurface using the kinematic boundary
c               condition dh/dt + u*dh/dx = v
c     ISOUTFRS  Computation of the cartesian velocities in the vertices on the
c               upper boundary for freesurface computation
c **********************************************************************
c
c                       ERROR MESSAGES
c
c       5   Three-dimensional version not yet available
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      call iseropen('ismvgrd')

      if ( ndim.eq.2 ) then

c     --- velocities, 2d
c         determine Cartesian velocity components in the vertices
c         on the upper boundary

          call isoutfrs ( ni, nj, nvirtual, coor, solut, velfrsf )

      else

          call iserrsub ( 5, 1, 0, 0 )

      end if

      call ismvg01 (  coor, deltat, ndim, ni, nj, nvirtual,
     +                 velfrsf(1), relax )

      call iserclos('ismvgrd')

      end
