      subroutine isndt02(iinput, rinput, t, dt, toutst, rstep, istation)
c=======================================================================
c
c     programmer   Michiel Kooper
c
c     version      1.1   date    09-05-95 Kees K. rstep added
c     version      1.0   date    12-01-95
c
c   copyright (c) 1994  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c***********************************************************************
c
c                  DESCRIPTION
c
c     get dt and test on stationary solution
c
c***********************************************************************
c
c                  KEYWORDS
c
c     time integration
c
c***********************************************************************
c
c                   INPUT / OUTPUT PARAMETERS
c
      implicit none
c
      integer iinput(*), istation

      double precision rinput(*), t, dt, toutst, rstep

c     dt        o    Time step
c     iinput    i    reserved
c     istation i/o   Choice whether a stationary solution is needed
c     rinput    i    reserved
c     rstep    i/o   Time step for writing to restart file
c     t         i    Actual time
c     toutst   i/o   Time step for output of results
c
c
c***********************************************************************
c
c                   COMMON BLOCKS
c
c***********************************************************************
c
c                   LOCAL PARAMETERS
c
      integer ipiintim, iprintim, ndt, tvar
      double precision eps, tperiod
c
c     eps       Accuracy with respect to the time parameter tout
c     ipiintim  Starting address of array IINTIM in array IINPUT
c     iprintim  Starting address of array RINTIM in array RINPUT
c     ndt       Number of variable time steps
c     period    Number of actual time period
c     tperiod   End time of actual time period
c     tvar      Choice of variable time steps
c
c**********************************************************************
c
c                   I/O
c
c**********************************************************************
c
c                   SUBROUTINES CALLED
c
c     ISERCLOS      removes name of subroutine from error list
c     ISEROPEN      adds name of subroutine to error list
c
      integer period

      save period
      data period /1/
c
c**********************************************************************
c
c                   ERROR MESSAGES
c
c***********************************************************************
c
c                   PSEUDO CODE
c
c     For istationary:
c        On input:  istation =  0 => No stat.solution wanted
c                            =  1 => Stop if stat.solution
c        On output: istation = -1 => Stat.solution has been
c                                    reached
c     For dt:
c        If ( tvar = 1 .and. t+eps .lt. tperiod ) then
c           dt = tstep ( period + 1 )
c
c=======================================================================

      call iseropen('isndt02')

      iprintim = iinput(29)
      ipiintim = iinput( 4)

      istation = iinput( ipiintim -1 + 10 )
      tvar     = iinput( ipiintim -1 +  7 )

c     --- if DT is not variable or a stationary solution has been
c         reached, this subroutine is skipped

      if ( tvar .ne. 0 .and. istation .ge. 0 ) then

         eps      = 1d-2 * dt
         ndt      = iinput( ipiintim -1 +  8 )
         tperiod  = rinput( iprintim -1 + 50 + period )

         if ( tvar .eq. 1 ) then

c           --- variable time steps DT over fixed periods TPERIOD

            if ( t + eps .gt. tperiod .and. period .lt. ndt ) then
               period = period + 1
            end if

            dt    = rinput( iprintim -1 + 40 + period )

c           --- TOUTST and RSTEP needs to be adapted as well

            if ( toutst .lt. dt ) then

               toutst = rinput( iprintim -1 + 8 )
               toutst = ( int( (dt-eps)/toutst ) + 1d0 ) * toutst

            end if
            if ( rstep .lt. dt ) then

               rstep = rinput( iprintim -1 + 11 )
               rstep = ( int( (dt-eps)/rstep ) + 1d0 ) * toutst

            end if

         end if
      end if

      call iserclos('isndt02')

      end
