      subroutine isnlin3d ( rhside, work, solut, cva, jctg, chsym,
     +                      invjac, rintur, rho, jac, locdx, alphad,
     +                      idiscret, ndegfd, ni, nj, nk, npointsv,
     +                      maniso, mturb, nvirtual, pnttyp, iparea,
     +                      dcell )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 1.0    date 05-09-1996
c
c   copyright (c) 1996-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Update right-hand side vector of the momentum equation with
c     nonlinear stress-strain terms through deferred correction approach
c **********************************************************************
c
c                       KEYWORDS
c
c     3d
c     discretization
c     right_hand_side
c     turbulence
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer locdx(3), alphad, idiscret, ndegfd, ni, nj, nk, mturb,
     +        maniso, npointsv, nvirtual, pnttyp, iparea(2,3), dcell(3)
      double precision work(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,*),
     +                 solut(*), cva(*), jctg(*),
     +                 chsym(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,3,6),
     +                 invjac(*), rintur(*), rho(*),
     +                 rhside(1:ni+1,1:nj+1,1:nk+1),
     +                 jac(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     chsym     i    Christoffel symbols of the second kind in the
c                    point given by pnttyp
c                    (symmetric contravariant part stored as vector
c                    with index encoding given by istwo)
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     dcell     i    reserved
c     idiscret  i    Type of discretization
c     invjac    i    1/sqrt(g) in pnttyp
c     iparea    i    Area of integration points where a contribution to
c                    the matrix is to be calculated
c     jac       i    sqrt(g) in pnttyp
c     jctg      i    Jacobian times symmetric contravariant metric
c                    tensor in point given by pnttyp
c                    (stored as vector index encoding given by istwo)
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     maniso    i    Indicates type of nonlinear turbulence (anisotropic)
c                    modelling
c     mturb     i    Indicates type of turbulence modelling
c     ndegfd    i    Number of degrees of freedom
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     npointsv  i    Number of points in the mesh with virtual points
c     nvirtual  i    Number of rows of virtual cells
c     pnttyp    i    reserved
c     rho       i    Estimate of the asymptotic convergence rate
c     rhside    o    reserved
c     rintur    i    reserved
c     solut     i    reserved
c     work      i    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, kp, betad, gammad, sign, istwo(1:3,1:3),
     +        ishift, jshift, kshift, iarea(2,3)

c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     gammad    Direction index
c     iarea     Area of arrays
c     ip        Local counting variable i-direction
c     ishift    Shift in i-direction
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     jp        Local counting variable j-direction
c     jshift    Shift in j-direction
c     kp        Local counting variable k-direction
c     kshift    Shift in k-direction
c     sign      sign for multiplication (1 or -1)
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISQDTN03  Compute the quadratic products of strain and rotation
c               tensors in R3
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data istwo / 1, 2, 3,
     +             2, 4, 5,
     +             3, 5, 6 /
c ======================================================================
c
c     --- Shift from integration cell centre to cell containing the
c         integration point

      ishift = dcell(1)
      jshift = dcell(2)
      kshift = dcell(3)

      iarea(1,1) = iparea(1,1)+1
      iarea(2,1) = iparea(2,1)-1
      iarea(1,2) = iparea(1,2)+1
      iarea(2,2) = iparea(2,2)-1
      iarea(1,3) = iparea(1,3)+1
      iarea(2,3) = iparea(2,3)-1

      if ( abs(locdx(1))+abs(locdx(2))+abs(locdx(3)).eq.1 ) then

c     --- Integration points (0,1,0), (0,-1,0), (1,0,0), (-1,0,0),
c                            (0,0,1), (0,0,-1)
c         betad in (nonlinear term), betad

         if ( abs(locdx(1)).ne.0 ) betad = 1
         if ( abs(locdx(2)).ne.0 ) betad = 2
         if ( abs(locdx(3)).ne.0 ) betad = 3
         sign = locdx(betad)

         if ( pnttyp.eq.2 .or. pnttyp.eq.3 .or. pnttyp.eq.5 ) then
           iarea(2,betad) = iarea(2,betad) + 1
         else if ( pnttyp.eq.8 ) then
           iarea(1,betad) = iarea(1,betad) - 1
         end if

         call isqdtn03(work, alphad, betad, solut, cva, jctg, chsym,
     +                 mturb, invjac, rintur, rho, idiscret, maniso,
     +                 ndegfd, ni, nj, nk, npointsv, nvirtual, pnttyp,
     +                 iarea)

         do kp = iparea(1,3)+1, iparea(2,3)-1
           do jp = iparea(1,2)+1, iparea(2,2)-1
             do ip = iparea(1,1)+1, iparea(2,1)-1
               rhside(ip,jp,kp) = rhside(ip,jp,kp) + sign*jac(ip,jp,kp)*
     +                            work(ip+ishift,jp+jshift,kp+kshift,1)
             end do
           end do
         end do

      else if ( locdx(1).eq.0 .and. locdx(2).eq.0 .and.
     +          locdx(3).eq.0 ) then

c     --- Centroid point (0,0,0)
c         This concerns the terms involving the Christoffel symbols

        do gammad = 1, 3
          do betad = 1, 3

            call isqdtn03(work, betad, gammad, solut, cva, jctg, chsym,
     +                    mturb, invjac, rintur, rho, idiscret, maniso,
     +                    ndegfd, ni, nj, nk, npointsv, nvirtual,
     +                    pnttyp, iarea)

            do kp = iparea(1,3)+1, iparea(2,3)-1
              do jp = iparea(1,2)+1, iparea(2,2)-1
                do ip = iparea(1,1)+1, iparea(2,1)-1
                  rhside(ip,jp,kp) = rhside(ip,jp,kp) + jac(ip,jp,kp)*
     +                     chsym(ip,jp,kp,alphad,istwo(betad,gammad))*
     +                     work(ip,jp,kp,1)
                end do
              end do
            end do

          end do
        end do

      end if

      end
