      subroutine isqdtn02(work, alphad, betad, solut, cva, jctg, chsym,
     +                    mturb, invjac, rintur, rho, idiscret,
     +                    ndegfd, ni, nj, npointsv, nvirtual, pnttyp,
     +                    iarea                                       )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 2.0    date   03-07-1997 Guus: new call to isintcf2
c        version 1.1    date   28-10-1996 changed loop
c        version 1.0    date   19-08-1996
c
c   copyright (c) 1996-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Compute quadratic products of strain and rotation tensors in R2
c
c **********************************************************************
c
c                       KEYWORDS
c
c     2d
c     turbulence
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer idiscret, ndegfd, ni, nj, npointsv, nvirtual, mturb,
     +        pnttyp, alphad, betad, iarea(2,2)
      double precision cva(*), jctg(*), chsym(*),
     +                 invjac(*), work(*), rintur(*),
     +                 rho(*), solut(1:npointsv,1:ndegfd)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     betad     i    Direction index, usually the direction perpendicular to the
c                    integration direction of the boundary integral.
c                    For example betad in local point (1,0) is 1 and in (0,1): 2
c     chsym     i    Christoffel symbols of the second kind in the
c                    point given by pnttyp
c                    (symmetric contravariant part stored as vector
c                    with index encoding given by istwo)
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     iarea     i    Area of arrays
c     idiscret  i    Type of discretization
c     invjac    i    1/sqrt(g) in pnttyp
c     jctg      i    Jacobian times symmetric contravariant metric
c                    tensor in point given by pnttyp
c                    (stored as vector index encoding given by istwo)
c     mturb     i    Indicates type of turbulence modelling
c     ndegfd    i    Number of degrees of freedom
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     npointsv  i    Number of points in the mesh with virtual points
c     nvirtual  i    Number of rows of virtual cells
c     pnttyp    i    reserved
c     rho       i    Estimate of the asymptotic convergence rate
c     rintur    i    reserved
c     solut     i    reserved
c     work      o    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, idir, jdir, ipnlin, ippderiv, iphightens, ipveloc,
     +        ipturb, nwork, ii
      double precision ctau1, ctau2, ctau3, dummy(1)

c     ctau1     Parameter ctau1 for nonlinear two-equation model
c     ctau2     Parameter ctau2 for nonlinear two-equation model
c     ctau3     Parameter ctau3 for nonlinear two-equation model
c     dummy     Dummy
c     i         General loop variable
c     idir      General loop variable for 1...ndim loop
c     ii        Loop variable
c     iphightens Starting address of array HIGHTENS in array WORK
c     ipnlin    Starting address of array NLIN in array WORK
c     ippderiv  Starting address of array PDERIV in array WORK
c     ipturb    Starting address of array TURB in array WORK
c     ipveloc   Starting address of array VELOC in array WORK
c     j         General loop variable
c     jdir      General loop variable for 1...ndim loop
c     nwork     Second dimension of array work
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISINTCF2  Compute interpolation of a coefficient in an integration point
c     ISQUADTN  Compute the quadratic products of strain and rotation
c               tensors in R2
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('isqdtn02')

c     --- allocate local arrays

      ipnlin     = 1
      iphightens = ipnlin      + npointsv
      ippderiv   = iphightens  + npointsv
      ipveloc    = ippderiv    + 4*npointsv
      ipturb     = ipveloc     + 2*npointsv
      nwork      = ipturb      + 2*npointsv

c     --- initialize the work array's

      do i = 1, nwork
        work( i ) = 0d0
      end do

c                                        idir
c     --- Compute partial derivative of V     in direction jdir in
c         point of type pnttyp

      do idir = 1, 2
        do jdir = 1, 2
          call isintcf2( solut(1,idir),
     +                   work(ippderiv+npointsv*(idir-1+2*(jdir-1))),
     +                   ni, nj, nvirtual, idir, jdir, pnttyp, 0,
     +                   dummy )
        end do
      end do

c     --- Interpolate velocities to pnttyp

      do idir = 1, 2
        call isintcf2( solut(1,idir),
     +                 work(ipveloc+npointsv*(idir-1)),
     +                 ni, nj, nvirtual, idir, 0, pnttyp, 0, dummy )
      end do

c     --- Interpolate turbulence quantities to pnttyp

      do idir = ndegfd-1, ndegfd
        call isintcf2( solut(1,idir),
     +                 work(ipturb+npointsv*(idir-ndegfd+1)),
     +                 ni, nj, nvirtual, 3, 0, pnttyp, 0, dummy )
      end do

c     --- Determine the quadratic products of strain and rotation tensors

      ctau1 = rintur(22)
      ctau2 = rintur(23)
      ctau3 = rintur(24)

      call isquadtn( work(iphightens), alphad, betad, work(ippderiv),
     +               work(ipveloc), cva, invjac, jctg, chsym, ni, nj,
     +               nvirtual, idiscret, ctau1, ctau2, ctau3, iarea )

      if ( mturb.eq.1 .or. mturb.eq.2 .or. mturb.eq.5 ) then

c     --- k-eps type model

        do j = 2-nvirtual, nj+nvirtual-1
          do i = 2-nvirtual, ni+nvirtual-1
            ii = i+nvirtual+(2*nvirtual+ni+1)*(j+nvirtual-1)
            work(ii+ipnlin-1) = -rho(ii)*rintur(3)*
     +                          (work(ii+ipturb-1)**3)*
     +                          work(ii+iphightens-1)/
     +                          ((work(ii+ipturb+npointsv-1)**2)+1d-10)
          end do
        end do
      else if ( mturb.eq.3 ) then

c     --- k-omega model

        do j = 2-nvirtual, nj+nvirtual-1
          do i = 2-nvirtual, ni+nvirtual-1
            ii = i+nvirtual+(2*nvirtual+ni+1)*(j+nvirtual-1)
            work(ii+ipnlin-1) = -rho(ii)*work(ii+ipturb-1)*
     +                          work(ii+iphightens-1)/(rintur(3)*
     +                          ((work(ii+ipturb+npointsv-1)**2)+1d-10))
          end do
        end do
      end if

      call iserclos('isqdtn02')

      end
