      subroutine isqdtn13(cntdef, pderiv, veloc, cva, invjac, jctg,
     +                    chsym, hightens, ni, nj, nk, nvirtual,
     +                    idiscret, ctau1, ctau2, ctau3                )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 1.0    date   05-09-1996
c
c   copyright (c) 1996-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Compute nonlinear effects of the production rate of turbulent
c     energy
c **********************************************************************
c
c                       KEYWORDS
c
c     3d
c     turbulence
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, idiscret
      double precision ctau1, ctau2, ctau3
      double precision cntdef(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual),
     +                 pderiv(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual,
     +                        1:3,1:3),
     +                 veloc(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,
     +                       1:3),
     +                 hightens(1-nvirtual:ni+1+nvirtual,
     +                          1-nvirtual:nj+1+nvirtual,
     +                          1-nvirtual:nk+1+nvirtual)
      double precision cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual,3,3),
     +                 invjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual),
     +                 jctg(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,6),
     +                 chsym(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,3,6)

c     chsym     i    Christoffel symbols of the second kind in the
c                    point given by pnttyp
c                    (symmetric contravariant part stored as vector
c                    with index encoding given by istwo)
c     cntdef    o    Resulted double contraction of deformation rate tensor
c     ctau1     i    Parameter ctau1 for nonlinear two-equation model
c     ctau2     i    Parameter ctau2 for nonlinear two-equation model
c     ctau3     i    Parameter ctau3 for nonlinear two-equation model
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     hightens  i    Higher order tensor constains quadratic products of
c                    strain and rotation tensors
c     idiscret  i    Type of discretization
c     invjac    i    1/sqrt(g) in pnttyp
c     jctg      i    Jacobian times symmetric contravariant metric
c                    tensor in point given by pnttyp
c                    (stored as vector index encoding given by istwo)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     pderiv    i    Partial derivative of flux in point pnttyp
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, kp, alphad, betad, deltal, i, gammal,
     +        istwo(1:3,1:3), iarea(2,3)
      double precision gac, cntdef2

c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     cntdef2   Resulted double contraction of strain or rotation tensor
c     deltal    Loop index
c     gac       Covariant metric tensor
c     gammal    Loop index
c     i         General loop variable
c     iarea     Area of arrays
c     ip        Local counting variable i-direction
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISQUDTN3  Compute the quadratic products of strain and rotation
c               tensors in R3
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data istwo / 1, 2, 3,
     +             2, 4, 5,
     +             3, 5, 6 /
c ======================================================================
c
      iarea(1,1) = 2
      iarea(1,2) = 2
      iarea(1,3) = 2
      iarea(2,1) = ni-1
      iarea(2,2) = nj-1
      iarea(2,3) = nk-1

      if ( idiscret.eq.1 ) then

c     --- standard discretization

        do alphad = 1, 3
          do betad = 1, 3

c         --- Determine the quadratic products of strain and
c             rotation tensors

            call isqudtn3( hightens, alphad, betad, pderiv,
     +                     veloc, cva, invjac, jctg, chsym,
     +                     ni, nj, nk, nvirtual, idiscret,
     +                     ctau1, ctau2, ctau3, iarea )

            do kp = iarea(1,3), iarea(2,3)
              do jp = iarea(1,2), iarea(2,2)
                do ip = iarea(1,1), iarea(2,1)

                  cntdef2 = 0d0

                  do gammal = 1, 3

                    gac = 0d0

                    do i = 1, 3
                      gac = gac + cva(ip,jp,kp,i,alphad) *
     +                            cva(ip,jp,kp,i,gammal)
                    end do

                    cntdef2 = cntdef2 + invjac(ip,jp,kp) *
     +                        gac * pderiv(ip,jp,kp,gammal,betad)

                    do deltal = 1, 3

                      cntdef2 = cntdef2 + invjac(ip,jp,kp) * gac *
     +                    ( chsym(ip,jp,kp,gammal,istwo(deltal,betad))*
     +                      veloc(ip,jp,kp,deltal) -
     +                      chsym(ip,jp,kp,deltal,istwo(betad,deltal))*
     +                      veloc(ip,jp,kp,gammal) )

                    end do

                  end do

                  cntdef(ip,jp,kp) = cntdef(ip,jp,kp) + cntdef2 *
     +                               hightens(ip,jp,kp)

                end do
              end do
            end do

          end do
        end do

      else if ( idiscret.eq.2 ) then

c     --- WesBeek discretization, not yet implemented

      end if

      end
