      subroutine issbstep ( eq, theta, nblocks, isiinput, isrinput,
     +                      iscoor, issolut, iscoefs, iseddy, isinfbnd,
     +                      isbndcon, ismatrix, isgmat, isrhside,
     +                      isgeom, isitopol, iadmin, isigeom,
     +                      isaverstr, isdensup )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 2.2    date 19-01-1999 Kees Vuik: multiblock algorithm
c        version 2.1    date 23-11-1998 Jason: SPMD version
c        version 2.0    date 03-11-1998 Remove ieq
c        version 1.0    date 07-09-1998
c
c   copyright (c) 1998-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     Perform a part of a time step
c
c **********************************************************************
c
c                       KEYWORDS
c
c     time_integration
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iadmin(*), isbndcon(*), iscoor(*), eq,
     +        iseddy(*), isgmat(*), isiinput(*), isinfbnd(*),
     +        isitopol, ismatrix(*), isrhside(*), isrinput(*),
     +        issolut(*), iscoefs(*), isgeom(*), nblocks,
     +        isigeom(*), isaverstr(*), isdensup(*)
      double precision theta

c     eq             i    Counting variable for which equation
c     iadmin        i/o   contains information about
c                         which blocks on which computer are stored.
c     isaverstr     i/o   Array of sequence numbers (nblocks) of array averstr
c     isbndcon      i/o   Array of sequence numbers (nblocks) of array bndcon
c     iscoefs       i/o   Array of sequence numbers (nblocks) of array coefs
c     iscoor        i/o   Array of sequence numbers (nblocks) of array coor
c     isdensup      i/o   Array of sequence numbers (nblocks) of array densup
c     iseddy        i/o   Array of sequence numbers (nblocks) of array eddy
c     isgeom        i/o   Array of sequence numbers (nblocks) of array geom
c     isgmat        i/o   Array of sequence numbers (nblocks) of array gmat
c     isigeom       i/o   Array of sequence numbers (nblocks) of array igeom
c     isiinput      i/o   Array of sequence numbers (nblocks) of array iinput
c     isinfbnd      i/o   Array of sequence numbers (nblocks) of array infbnd
c     isitopol      i/o   Sequence number of array itopol
c     ismatrix      i/o   Array of sequence numbers (nblocks) of array matrix
c     isrhside      i/o   Array of sequence numbers (nblocks) of array rhside
c     isrinput      i/o   Array of sequence numbers (nblocks) of array rinput
c     issolut       i/o   Array of sequence numbers (nblocks) of array solut
c     nblocks        i    Number of blocks
c     theta          i    Parameter theta for theta method
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'
      include 'IScommon/cisblank'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ipiinput, ipiinblk, mcom, ipiincom, ndim, icoupled,
     +        iseqpres, mcav
      logical multibl

c     icoupled       If 0 the momentum equations are coupled,
c                    if 1 they are not
c     ipiinblk       Starting address of array IINBLK in array IINPUT
c     ipiincom       Starting address of array IINCOM in array IINPUT
c     ipiinput       Starting address of array iinput in ibuffr
c     iseqpres       Sequence number of pressure equation
c     mcav           If 1 a cavity model is used, if 0 not
c     mcom           If zero the flow is incompressible else (1)
c                    compressible
c     multibl        If true information about the multi-block process has
c                    been read
c     ndim           Dimension of space (2 or 3)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp

c     ISACTIVE       Activate old permanent array
c     ISBLDMAT       Build matrix
c     ISCOM          Compr. flow: compute density and adjust velocity
c     ISCORN         Correct normal fluxes
c     ISCORR         Correct fluxes, pressure correction
c     ISDD           Solve domain decomposition problem
c     ISDIVMAS       Divide the velocity vector by the mass fraction
c     ISERCLOS       Resets old name of previous subroutine of higher level
c     ISEROPEN       Produces concatenated name of local subroutine
c     ISEXPLI        Calculates the new solution for theta is 0
c     ISFILC         Fill the coefficients arrays
c     ISINGETP       get the pointer corresponding to an integer array
c     ISSDD          Single domain solver
c     ISTEMPERATURE  Compute the temperature from the internal energy
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     For each time step a number of equations will be solved separately
c     (i.e. decoupled)
c     The sequence of the equations may be given by the user
c     If the user did not give this sequence then the standard sequence is
c     as follows:
c     Incompressible case:
c        Solve velocity prediction
c        Solve pressure correction and adapt velocity and pressure
c        Solve transport equations (including turbulence equations)
c     Compressible case:
c        Solve energy equation
c        Solve velocity prediction
c        Solve pressure correction and adapt velocity and pressure
c        Solve transport equations (including turbulence equations)
c
c     Solve an equation consists of the following steps:
c        Compute all coefficients
c        Build system of equations
c        Solve linear system of equations
c
c     With respect to the velocity prediction we have to distinguish between
c     the coupled case and the decoupled case
c     In the coupled case we build one system of equations for the
c     momentum equations
c     The number of equations is in that case equal to 2 + number of transport
c     equations
c     In the decoupled case each velocity component is treated as a
c     separate unknown and therefore there is a separate transport equation
c     for each of the components.
c
c     The method applied for the solution velocity and pressure is
c     pressure-correction.
c
c     This method is based on the following steps:
c
c     -  Predict the velocity using the pressure at the old time level
c     -  Compute the pressure correction in order to satisfy the continuity
c        equation
c     -  Update the velocity and the pressure
c
c     With respect to the muliblock process it may be necessary to update
c     the normal components at the interfaces of the blocks in order to get
c     a uniquely defined velocity. This process will be called adpat fluxes.
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Extract some constants

      call iseropen ( 'issbstep' )
      call isactive ( isiinput(1) )
      ipiinput = isingetp( isiinput(1) )
      ipiincom = ibuffr(ipiinput -1 + 9 )
      mcom     = ibuffr(ipiinput + ipiincom - 1 )
      mcav     = ibuffr(ipiinput-1+77)
      ipiinblk = ibuffr(ipiinput + 7 - 1) + ipiinput - 1
      multibl  = ibuffr(ipiinblk + 4 - 1).gt.0
      ndim     = ibuffr(ipiinput-1+57)
      icoupled = ibuffr(ipiinput-1+76)
      if ( icoupled.eq.0 ) then
         iseqpres = 2
      else
         iseqpres = ndim+1
      end if

c     --- fill coefficients

      if ( eq.ne.iseqpres ) then

c     --- In case of the pressure equation the coefficients
c         of the momentum equation are reused

c         --- fill coefficients

         call isfilc ( eq, nblocks, isiinput, isrinput, iscoefs ,
     +                 issolut, iscoor, isgeom, iseddy, isitopol,
     +                 iadmin, isigeom, isaverstr )

      end if
      if ( nerror.ne.0 ) go to 1000

c     --- build equations, for the choice theta = 0
c         the new solution is calculated for the momentum equation

      if ( eq.ne.1 .or. icoupled.eq.1 .or. theta.gt.0d0 ) then

c     --- Implicit solution of momentum equation or transport equation

         call isbldmat ( eq, nblocks, isiinput, isrinput, iscoor,
     +                   issolut, iscoefs, iseddy, isinfbnd, isbndcon,
     +                   ismatrix, isgmat, isrhside, isgeom,
     +                   isigeom, iadmin, isitopol, isdensup )

      else

c     --- Explicit solution of momentum equation

         call isexpli ( eq, nblocks, isiinput, isrinput, iscoor,
     +                  issolut, iscoefs, iseddy, isinfbnd, isbndcon,
     +                  ismatrix, isgmat, isrhside, isgeom,
     +                  iadmin, isitopol )

      end if

      if ( nerror.ne.0 ) go to 1000

c     --- Solve system of linear equations,
c         If theta = 0 no solution takes place for the momentum equation
c         In case of a domain decomposition the solution means
c         performing a domain decomposition step (isdd)
c         otherwise the system of equations is solved immediately

      if ( eq.ne.1 .or. icoupled.eq.1 .or. theta.gt.0d0 ) then

         if ( nblocks.eq.1 .and. .not. multibl ) then

c        --- Standard single domain algorithm

            call issdd ( eq   , nblocks , isiinput, isrinput,
     +                iscoor  , issolut , ismatrix, isgmat  ,
     +                isrhside, isitopol, iadmin, isinfbnd, isbndcon,
     +                isgeom, isdensup)

         else

c        --- Number of blocks > 1 or special algorithm.
c            The special algorithm means doing the multiblock even
c            in case of a single block
c            A reason for this is the use of periodical boundary conditions
c            by connecting parts of boundaries to each other.

            call isdd ( eq, isiinput, isrinput, iscoor,
     +                  issolut, iscoefs, iseddy, isinfbnd,
     +                  isbndcon, ismatrix, isgmat, isrhside,
     +                  isgeom, isitopol, isdensup, iadmin )

         end if

      end if
      if ( nerror.ne.0 ) go to 1000

c     --- adapt fluxes

      if ( eq.lt.iseqpres ) then
         call iscorn( iadmin, isiinput, issolut, isitopol )

      end if
      if ( nerror.ne.0 ) go to 1000

c     --- pressure correction

      if ( eq.eq.iseqpres ) then

         call iscorr( iadmin, isiinput, isrinput, issolut,
     +                isgmat, isitopol, isinfbnd, isgeom,
     +                isigeom, isbndcon, iscoor, iscoefs,
     +                nblocks )
         if ( nerror.ne.0 ) go to 1000

c        --- Divide the velocity by the mass fraction in case of multiphase
c            gas flow

         call isdivmas ( iadmin, isiinput, isrinput, issolut,
     +                   iscoefs )
         if ( nerror.ne.0 ) go to 1000

         if ( mcom.gt.0 ) then

c        --- Compressible flow, compute the density from the equation of state
c            The pressure has just been computed and the enthalpy has been
c            computed as first equation, hence at this moment the equation
c            of state can be evaluated

            if ( mcav.eq.0 ) then

c           --- No cavity, compute the temperature from the energy and
c               store into eddy

               call istemperature ( nblocks, isiinput, isrinput,
     +                              issolut, iseddy, iadmin )

            end if
            call iscom ( nblocks, isiinput, isrinput, issolut,
     +                   iseddy, iadmin, isitopol )
            if ( nerror.ne.0 ) go to 1000

         end if

      end if

      call iserclos ( 'issbstep' )

1000  end
