      subroutine isshif02(ndim, npointsv, ndegfd, ntimlv, solut,
     +                    mturb, viscorr, turbonly)
c ======================================================================
c
c     programmer   Erik Brakkee
c
c     version      1.4  09-06-1998  Charles: nvpoints changed to npointsv
c     version      1.3  27-03-1998  Kees K: turbonly
c     version      1.2  16-07-1996  Marcel: place lower bound on tke
c                                   in case of low-Reynolds-number model
c     version      1.1  21-07-1994  Erik:  pressure not corrected anymore
c     version      1.0   1-06-1994  Erik:  initial revision
c
c   copyright (c) 1994  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Copy time level n+1 to n (n = 1..ntimlv-1 for a subdomain
c     solut array.
c     For the pressure only time levels n+1 (n = 1..ntimlv-2) are
c     shifted.
c **********************************************************************
c
c                       KEYWORDS
c
c     shift
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ndim, npointsv, ndegfd, ntimlv, mturb, viscorr
      logical turbonly
      double precision solut(1:npointsv, 1:ndegfd, 1:ntimlv)

c     mturb    i    Indicates type of turbulence modelling
c     ndegfd   i    Number of degrees of freedom
c     ndim     i    Dimension of space (2 or 3)
c     npointsv i    Number of points in the mesh with virtual points
c     ntimlv   i    Number of time levels
c     solut   i/o   reserved
c     turbonly i    Shift only for turbulence equations
c     viscorr  i    Indicate whether to use viscous corrections or not
c **********************************************************************
c
c                       COMMON BLOCKS
c
c     None
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer idegfd, itimlv, ipointl

c     idegfd    Loop variable with respect to a given unknown
c     ipointl   Loop variable
c     itimlv    Loop variable with respect to a given time level
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- If low-Reynolds-number k-eps model is employed, a small
c         lower bound is placed on turbulent energy

      idegfd = ndegfd - 1
      if ( mturb.eq.1 .and. viscorr.ne.0 ) then
        do itimlv = 1, ntimlv - 1
          do ipointl = 1, npointsv
            solut(ipointl, idegfd, itimlv+1) =
     +         max(0d0,solut(ipointl, idegfd, itimlv+1)) + 1d-8
          end do
        end do
      end if

      if ( turbonly ) then

c     --- handle only turbulence quantities

         do itimlv = 1, ntimlv - 1
            do idegfd = ndim+2, ndegfd
               do ipointl = 1, npointsv
                  solut(ipointl, idegfd, itimlv) =
     +               solut(ipointl, idegfd, itimlv+1)
               end do
            end do
         end do

      else

c     --- handle all quantities except pressure

         do itimlv = 1, ntimlv - 1
            do idegfd = 1, ndim
               do ipointl = 1, npointsv
                  solut(ipointl, idegfd, itimlv) =
     +               solut(ipointl, idegfd, itimlv+1)
               end do
            end do
            do idegfd = ndim+2, ndegfd
               do ipointl = 1, npointsv
                  solut(ipointl, idegfd, itimlv) =
     +               solut(ipointl, idegfd, itimlv+1)
               end do
            end do
         end do

c        --- handle the pressure

         idegfd = ndim + 1
         do itimlv = 1, ntimlv - 2
            do ipointl = 1, npointsv
               solut(ipointl, idegfd, itimlv) =
     +            solut(ipointl, idegfd, itimlv+1)
            end do
         end do

      end if

      end
