      subroutine issmagor ( eddy, cs, jac, wdis, cntdef, ni, nj,
     +                      nk, nvirtual, igeom, averstr,
     +                      invjac, veloc, jctg, molvis, rho,
     +                      ntrnsp, nturb, iinbc, rinbc, aver,
     +                      lenaver, nwinsta, nwinend, timewind,
     +                      nwindowmx, deltat, itimeact )
c ======================================================================
c
c        programmer     Frederic Tremblay
c        version 1.1    date   24-08-1998 Kees K. aver from local to input
c        version 1.0    date   19-08-1997
c
c
c   copyright (c) 1997-1997  "TU Muenchen"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Muenchen".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Computes eddy viscosity, for the smagorinsky model
c
c **********************************************************************
c
c                       KEYWORDS
c
c     turbulence
c     LES
c     Smagorinsky
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nvirtual, ni, nj, nk,iinbc(*),ntrnsp,
     + nturb, nwinsta,nwindowmx,lenaver,nwinend,itimeact,
     +                igeom(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual)
      double precision cs ,molvis,rho,rinbc(*),timewind,deltat,
     +               cntdef(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual),
     +                 jac (1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual),
     +                 eddy(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual),
     +                 wdis(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual)
      double precision invjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual),
     +                  veloc(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual,3),
     +                averstr(1:lenaver+1,nwindowmx),
     +                 jctg(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual,6),
     +                 aver(lenaver)

c     aver      Averaged wall stress in time window
c     averstr   i    reserved
c     cntdef    i    Resulted double contraction of deformation rate tensor
c     cs        i    Parameter c_s for LES Smagorinsky model
c     deltat    i    Product of theta and dt
c     eddy      o    reserved
c     igeom     i    Integer array that returns the closest cell on a wall
c     iinbc     i    reserved
c     invjac    i    1/sqrt(g) in pnttyp
c     itimeact  i    Actual time step
c     jac       i    sqrt(g) in pnttyp
c     jctg      i    Jacobian times symmetric contravariant metric
c                    tensor in point given by pnttyp
c                    (stored as vector index encoding given by istwo)
c     lenaver   i    Help parameter for the length of array averstr
c     molvis    i    Molecular viscosity
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     ntrnsp    i    Number of transport equations
c     nturb     i    Number of extra transport equations for turbulence
c     nvirtual  i    Number of rows of virtual cells
c     nwindowmx i    Maximum size of the window to be used for time-averaging
c     nwinend   i    End position of time window
c     nwinsta   i    Starting position of time window
c     rho       i    Estimate of the asymptotic convergence rate
c     rinbc     i    reserved
c     timewind  i    Time interval for time window
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c     wdis      i    Array containing the wall distance
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, kp,j,tdir1,tdir2,
     +        pgii1, pgii2, iside, iarea(2,3),
     +        ipstart, nseg, ipiinbc,bctyp(3), i, ncount2,
     +        iprinbc, ncount, idim, ipsta, ipend, ipoint
      double precision one3rd,delta, scale1, scale2, tmpshr, u1,
     +                 u2, yplus, cs0, bcval(3), tsum

c     bctyp     A given boundary type
c     bcval     A given boundary condition value
c     cs0       Non-dimensional distance
c     delta     Help parameter to compute a quantity
c     i         General loop variable
c     iarea     Area of arrays
c     idim      Loop variable used for dimension
c     ip        Local counting variable i-direction
c     ipend     Pointer to end of time window
c     ipiinbc   Starting address of array iinbc in array iinput
c     ipoint    Address in array IBUFFR
c     iprinbc   Starting address of array rinbc in array iinput
c     ipsta     Pointer to begin of time window
c     ipstart   Starting address
c     iside     Sequence number of side
c     j         General loop variable
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c     ncount    Counter
c     ncount2   Counter
c     nseg      Number of segments the boundary is divided in
c     one3rd    1/3
c     pgii1     ?
c     pgii2     ?
c     scale1    ?
c     scale2    ?
c     tdir1     Direction of first tangent vector on current side
c     tdir2     Direction of second tangent vector on current side
c     tmpshr    ?
c     tsum      Flux contributions to the matrix
c     u1        Cartesian velocity component
c     u2        Cartesian velocity component
c     yplus     The dimensionless normal distance to the wall
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERREAL  Put real value in error message
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c    1300   nwindowmx too small for value of timewinderval and time step
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('issmagor')

c     --- loop over the boundaries to calculate averaged
c         friction velocity
c         NOTE : it is assumed that a corner has a negligeable
c               effect (example i=1 and j=1, the tang. dir
c               should be only in k !!! but it involves
c               too much extra if statements for the benefits
c               I think....mmmm

      ipsta = nwinsta
      ipend   = nwinend
      ncount2 = 0
      if (ipend.gt.ipsta) ncount2 = ipend-ipsta
      if (ipend.lt.ipsta) ncount2 = nwindowmx-ipsta+ipend
      if (ipsta.eq.0) ncount2 = ipend
      tsum = 0d0
      do j = 1 , ncount2
         ipoint = ipsta + j
         if (ipoint.gt.nwindowmx) ipoint = ipoint-nwindowmx
         tsum = tsum + averstr(lenaver+1,ipoint)
      end do
      ipend = ipend + 1
      if (ipend.gt.nwindowmx) ipend = 1
      if (tsum+deltat-timewind.gt.sqreps) ipsta = ipsta + 1
      if (ipsta.gt.nwindowmx) ipsta = 1

      if (ipsta.eq.ipend) then
         call iserrint(nwindowmx,1)
         call iserreal(timewind,1)
         call iserrsub ( 1300, 1, 1, 0)
      end if
      if ((itimeact.ne.1).and.(ipsta.eq.0).and.(ipend.eq.1)) then
         call iserrint(nwindowmx,1)
         call iserreal(timewind,1)
         call iserrsub ( 1300, 1, 1, 0)
      end if

      averstr(lenaver+1,ipend) = deltat
      nwinsta = ipsta
      nwinend = ipend
      if (ipend.gt.ipsta) ncount2 = ipend-ipsta
      if (ipend.lt.ipsta) ncount2 = nwindowmx-ipsta+ipend
      if (ipsta.eq.0) ncount2 = ipend
      iprinbc = 0
      ncount = 0
      do iside = 1, 6

c     --- Get information on boundary 'iside'

        if ( iside.le.2 ) then
           tdir1 = 2
           tdir2 = 3
           pgii1 = 4
           pgii2 = 6
           if ( iside.eq.1 ) then
              iarea(1,1) = 1
           else
              iarea(1,1) = ni
           end if
           iarea(2,1) = iarea(1,1)
        else if (iside.le.4) then
           tdir1 = 3
           tdir2 = 1
           pgii1 = 6
           pgii2 = 1
           if ( iside.eq.3 ) then
              iarea(1,2) = 1
           else
              iarea(1,2) = nj
           end if
           iarea(2,2) = iarea(1,2)
        else
           tdir1 = 1
           tdir2 = 2
           pgii1 = 1
           pgii2 = 4
           if (iside.eq.5) then
              iarea(1,3) = 1
           else
              iarea(1,3) = nk
           end if
           iarea(2,3) = iarea(1,3)
        end if

        ipstart = iinbc(iside)

        nseg = iinbc(ipstart)

c       --- loop over segments

        do j = 1, nseg

          ipiinbc = ipstart +(j-1) *( 4+(ntrnsp+nturb+2)*4 )

          iarea(1,tdir1) = iinbc( ipiinbc + 1 )
          iarea(2,tdir1) = iinbc( ipiinbc + 3 )
          iarea(1,tdir2) = iinbc( ipiinbc + 2 )
          iarea(2,tdir2) = iinbc( ipiinbc + 4 )

          if ( iinbc(ipiinbc+5).eq.10 ) then

c         --- All velocities are prescribed, hence a no-slip wall is possible
c             Put velocity values in bcval

          do idim = 1,3
             bcval(idim) = 1d20
             bctyp(idim) = iinbc(ipiinbc+5+idim)
             if (bctyp(idim).eq.-1) then
                bcval(idim) = rinbc(iprinbc+idim)
             end if
          end do

          iprinbc = iprinbc+(ntrnsp+nturb+2)*3

          if ( bcval(1).eq.0d0 .and. bcval(2).eq.0d0
     +                         .and. bcval(3).eq.0d0 ) then

c         --- no-slip condition

          do ip = iarea(1,1), iarea(2,1)
             do jp = iarea(1,2), iarea(2,2)
                do kp = iarea(1,3), iarea(2,3)

                 ncount = ncount + 1

                scale1 = sqrt(jctg(ip,jp,kp,pgii1)*invjac(ip,jp,kp))
                scale2 = sqrt(jctg(ip,jp,kp,pgii2)*invjac(ip,jp,kp))

                u1 = scale1 * veloc(ip,jp,kp,tdir1)
                u2 = scale2 * veloc(ip,jp,kp,tdir2)

                u1 = u1**2 + u2**2
                u1 = sqrt(u1)

                tmpshr = molvis * u1 / wdis(ip,jp,kp)
                averstr(ncount,ipend) = tmpshr

               end do
            end do
         end do

          end if
         end if
        end do

      end do

c     --- Calculate the averaged wall stress in time window
c         first initialize variables

       tsum = 0d0
       do j = 1, lenaver
         aver(j) = 0d0
       end do

       do j = 1, ncount2
         ipoint = ipsta + j
         if (ipoint.gt.nwindowmx) ipoint = ipoint-nwindowmx
         tsum = tsum + averstr(lenaver+1,ipoint)
         do i = 1, lenaver
           aver(i) = aver(i) + averstr(i,ipoint)
     +                       * averstr(lenaver+1,ipoint)
         end do
       end do
       do j = 1, lenaver
         aver(j) = aver(j) / tsum
       end do

c      --- Now, calculate the eddy viscosity with the
c          damping function !!!

      one3rd = 1d0/3d0

      do ip = 1 , ni
         do jp = 1 , nj
            do kp = 1 , nk

c           --- find the closest cell on a wall

               ncount = igeom(ip,jp,kp)

c              --- Calculate the non-dimensional distance

               u1 = sqrt (aver(ncount) / rho )
               yplus = rho*wdis(ip,jp,kp)*u1/molvis

               cs0 = cs*sqrt(1-exp((-yplus/25)**3))

               delta = jac(ip,jp,kp)**one3rd
               eddy(ip,jp,kp)=(cs0*delta)**2*
     +                        sqrt((cntdef(ip,jp,kp)*2))
            end do
         end do
      end do

      call iserclos('issmagor')

      end
