      subroutine isstcr ( nblocks, iadmin, isiinput, isrinput, issolut )
c ======================================================================
c
c     programmer   Erik Brakkee
c     version      1.9 date 05-10-1999 Jason: added request
c     version      1.8 date 29-04-1999 Guus: layout
c     version      1.7 date 23-11-1998 Jason: issenmpi/isrecmpi version
c     version      1.6 date 31-08-1998 Jason: remove check nodeid
c     version      1.5 date 05-07-1997 Guus: new call of isstcr05
c     version      1.4 date 11-02-1997 Kees K: ntype added
c     version      1.3 date 28-02-1995 Guus: new call of isstcr05
c     version      1.2 date 13-01-1995 Michiel: Test for stationary solution
c     version      1.1 date 01-08-1994 Marcel: implementation of excluding
c                                      momentum or transport
c                                      equation for print purposes
c     version      1.0 date 13-07-1994
c
c
c   copyright (c) 1994-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Compute and print the stopping criterium for the stationary
c     solution of the incompressible Navier-Stokes equations.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nblocks
      integer iadmin(1:nblocks)
      integer isiinput(1:nblocks)
      integer isrinput(1:nblocks)
      integer issolut(1:nblocks)

c     iadmin    i    reserved
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isrinput  i    Array of sequence numbers (nblocks) of array rinput
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     nblocks   i    Number of blocks
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ifrac, ipiintim, iprintim, istation, nfrac, nturb,
     +        i, iblock, ifirst, ind,
     +        ipiinput, iprinput, ipnorm, ipiindsc, leniindsc,
     +        ipnormtot, ipoldn, ipsolut, isnorm, isnormtot, isoldn,
     +        lennorm, ncells, ndegfd, ndim, ni, nj, nk, ntimlv, ntrnsp,
     +        nvirtual, ntype, node, ireq
      double precision absaccur, relaccur
      save ifirst, isoldn, ifrac

c     absaccur     Absolute accuracy with respect to residual
c     i            General loop variable
c     iblock       Block sequence number
c     ifirst       reserved
c     ifrac        Actual fractional time step
c     ind          Sequence number of node
c     ipiindsc     Starting address of array IINDSC in array IINPUT
c     ipiinput     Starting address of array iinput in ibuffr
c     ipiintim     Starting address of array IINTIM in array IINPUT
c     ipnorm       Starting address of the norms array for a single block
c     ipnormtot    Starting address of the norms array for several blocks
c     ipoldn       Starting address of the oldn array
c     iprinput     Starting address of array rinput in buffer
c     iprintim     Starting address of array RINTIM in array RINPUT
c     ipsolut      Starting address of array solut in buffer
c     ireq         send-request
c     isnorm       Pointer to a norms array for storing computed norms
c                  for a certain block
c                      norms(ieq,*) contains the norms for equation ieq
c                        j         norms(ieq,j)
c                        1         SUM  (X(new) -X(old))^2
c                             all grid cells
c                        2         SUM  X(new)^2
c                             all grid cells
c                        3         MAX  |X(new) - X(old)|
c                             all grid cells
c     isnormtot    Sequence number of a norms array for storing computed
c                  norms for several blocks.
c     isoldn       Sequence number of an oldn array used for storing the
c                  computed norms at the previous time level
c     istation     Choice whether a stationary solution is needed
c     leniindsc    Length of first component of array iindsc
c     lennorm      Length of the norms array
c     ncells       Total number of grid cells
c     ndegfd       Number of degrees of freedom
c     ndim         Dimension of space (2 or 3)
c     nfrac        Number of fractional time steps
c     ni           Number of non-virtual cells in the i-direction (one block)
c     nj           Number of non-virtual cells in the j-direction (one block)
c     nk           Number of non-virtual cells in the k-direction (one block)
c     node         Node actually received from
c     ntimlv       Number of time levels
c     ntrnsp       Number of transport equations
c     nturb        Number of extra transport equations for turbulence
c     ntype        Type of norm used for stop criterion
c     nvirtual     Number of rows of virtual cells
c     relaccur     Relative accuracy
c **********************************************************************
c
c                       I/O
c
c     None
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp, isdpgetp

c     ISACTIVE       Activate old permanent array
c     ISDCREAT       Create permanent space in array IBUFFR
c                    (Double precision array)
c     ISDELARR       Delete a permanent array
c     ISDPGETP       get the pointer corresponding to a double precision array
c     ISERCLOS       Resets old name of previous subroutine of higher level
c     ISEROPEN       Produces concatenated name of local subroutine
c     ISINGETP       get the pointer corresponding to an integer array
c     ISRECMPI       Receive data from node or host
c     ISSENMPI       Send data to node or host
c     ISSTCR02       compute several norms of the solution for a certain block
c                    2-D
c     ISSTCR03       compute several norms of the solution for a certain block
c                    3-D
c     ISSTCR04       Join computed norms
c     ISSTCR05       Print computed norms
c     ISWAITALL      Wait for all sends to complete
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data ifirst /1/
      data isoldn /0/
      data ifrac  /1/
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c     Compute the norms for all local blocks and put them together
c     into a single norms array of results.
c
c     If ( On the host ) then
c
c        Receive the norms from all nodes not on the host and
c        put them together with the results from the local blocks
c
c     Else
c
c        Send the norms array to the host
c
c     Endif
c
c
c     If ( On the host ) then
c
c        Print results and save some norms for the next call to this
c        routine
c
c     Endif
c
c
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      call iseropen('isstcr')

c     --- compute some general parameters

      call isactive(isiinput(1))
      ipiinput = isingetp(isiinput(1))
      ntimlv   = ibuffr( ipiinput + 64 - 1 )
      ntrnsp   = ibuffr( ipiinput + 63 - 1 )
      nturb    = ibuffr( ipiinput + 70 - 1 )

      ipiindsc = ibuffr( ipiinput - 1 +  6 )
      ipiintim = ibuffr( ipiinput - 1 +  4 )
      iprintim = ibuffr( ipiinput - 1 + 29 )
      leniindsc = ibuffr( ipiinput - 1 +  106 )

      call isactive(isrinput(1))
      iprinput = isdpgetp(isrinput(1))
      nfrac    = ibuffr( ipiinput -  1 + ipiintim - 1 +  6 )
      istation = ibuffr( ipiinput -  1 + ipiintim - 1 + 10 )
      ntype    = ibuffr( ipiinput -  1 + ipiintim - 1 + 15 )
      absaccur = buffer( iprinput -  1 + iprintim - 1 + 29 )
      relaccur = buffer( iprinput -  1 + iprintim - 1 + 30 )

c     --- IFRAC has to be equal to NFRAC for output norm

      if (ifrac.ne.nfrac) then

         ifrac = ifrac + 1
         go to 100

      else

         ifrac = 1

      end if

c     --- lengths of local arrays

      lennorm  = (2 + ntrnsp + nturb)*2

c     --- create local arrays

      isnorm = 0
      call isdcreat(isnorm, lennorm, 'norm' )
      isnormtot = 0
      call isdcreat(isnormtot, lennorm, 'normtot' )

      if ( ifirst.eq.1 .and. inodenr.eq.ISHOST )
     +      call isdcreat ( isoldn, lennorm, 'oldn' )

      ipnormtot = isdpgetp(isnormtot)
      do i = 0, lennorm-1
         buffer(ipnormtot+i) = 0d0
      end do

c     --- compute norms for all local blocks and put them together

      do iblock = 1, nblocks
         if ( iadmin(iblock).eq.inodenr ) then

c        --- activate arrays

            call isactive(isiinput(iblock))
            call isactive(issolut(iblock))

c           --- compute addresses

            ipiinput = isingetp(isiinput(iblock))
            ipsolut  = isdpgetp(issolut(iblock))
            ipnorm   = isdpgetp(isnorm)
            ipnormtot= isdpgetp(isnormtot)

c           --- compute parameters

            ni       = ibuffr(ipiinput + 51 - 1)
            nj       = ibuffr(ipiinput + 52 - 1)
            nk       = ibuffr(ipiinput + 53 - 1)
            ndegfd   = ibuffr(ipiinput + 55 - 1)
            ndim     = ibuffr(ipiinput + 57 - 1)
            ntimlv   = ibuffr(ipiinput + 64 - 1)
            nvirtual = ibuffr(ipiinput + 54 - 1)

c           --- compute norms

            if ( ndim.eq.2 ) then

               call isstcr02 ( ni, nj, ndegfd, ntimlv, nvirtual,
     +                         ntrnsp,  nturb, ntype,
     +                         buffer(ipsolut), buffer(ipnorm) )

            else

               call isstcr03 ( ni, nj, nk, ndim, ndegfd, ntimlv,
     +                         nvirtual, ntrnsp,  nturb, ntype,
     +                         buffer(ipsolut), buffer(ipnorm) )

            end if

c           --- join results with the already computed norms
c               for the other blocks

            call isstcr04 ( ntrnsp, nturb, ntype,
     +                      buffer(ipnormtot), buffer(ipnorm) )

         end if

      end do

c     --- exchange information

      if ( iacnodes.gt.0 .and. inodenr.eq.ISHOST ) then

c     --- compute addresses

         ipnorm   = isdpgetp(isnorm)
         ipnormtot= isdpgetp(isnormtot)

c        --- on the host, so receive information from other nodes

         do ind = 1, iacnodes

c        --- for all nodes not on the host

            node = IS_ANY
            call isrecmpi ( buffer(ipnorm), lennorm, ISDOUBLE, node,
     +                      ICSTCR )

c           --- join results with those from other nodes

            call isstcr04 ( ntrnsp, nturb, ntype,
     +                      buffer(ipnormtot), buffer(ipnorm) )

         end do

      else if ( iacnodes.gt.0  ) then

c     --- not on the host, send norms to the host

         call issenmpi ( buffer(ipnormtot), lennorm, ISDOUBLE, ISHOST,
     +                   ICSTCR, ireq )
      end if

c     --- compute total number of grid cells

      ncells = 0

      if ( inodenr.eq.ISHOST ) then

         do iblock = 1, nblocks

            call isactive(isiinput(iblock))
            ipiinput = isingetp(isiinput(iblock))

            ni = ibuffr(ipiinput + 51 - 1)
            nj = ibuffr(ipiinput + 52 - 1)
            nk = ibuffr(ipiinput + 53 - 1)

            if ( nk.eq.0 ) then

               ncells = ncells + ni*nj

            else

               ncells = ncells + ni*nj*nk

            end if

         end do

      end if

c     --- print results

      if ( inodenr.eq.ISHOST ) then

c     --- only print results if on the host

         call isactive(isnormtot)
         call isactive(isoldn)

         ipnormtot = isdpgetp(isnormtot)
         ipoldn    = isdpgetp(isoldn)

         call isstcr05 ( ntrnsp, nturb, ntype,
     +                   ibuffr( ipiinput + ipiindsc - 1 ),
     +                   buffer(ipoldn), buffer(ipnormtot),
     +                   istation, absaccur, relaccur, leniindsc )

c        --- Test for stationary solution, if requested

         call isactive(isiinput(1))
         ipiinput = isingetp(isiinput(1))
         ipiintim = ibuffr( ipiinput - 1 +  4 )

         if ( ibuffr( ipiinput -  1 + ipiintim - 1 + 10 ).eq.1 ) then

              ibuffr( ipiinput -  1 + ipiintim - 1 + 10 ) = istation

         end if

      else

c     --- make sure that message was sent from node before continuing

         call iswaitall ( 1, ireq )

      end if

      call isdelarr(isnorm)
      call isdelarr(isnormtot)

      ifirst = 0

100   call iserclos('isstcr')

      end
