      subroutine isstcr02(ni,       nj,     ndegfd,  ntimlv,   nvirtual,
     +                    ntrnsp,  nturb, ntype, solut, norms)
c ======================================================================
c
c     programmer   Erik Brakkee
c
c
c     version      1.1  10-02-1997  Kees K:  maxnorm or l2norm
c     version      1.0  13-7-1994  Erik:  initial revision
c
c
c   copyright (c) 1994  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Compute 2-norms and max-norms of the difference between old and new
c     solutions and compute the 2-norm and max-norms of the new solution
c **********************************************************************
c
c                       KEYWORDS
c
c     stop_criterium
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ndegfd, ni, nj, ntimlv, nvirtual, ntrnsp, nturb, ntype
      double precision solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ndegfd, 1:ntimlv)
      double precision norms(2+ntrnsp+nturb, 2)

c     ndegfd     i  Number of degrees of freedom
c     ni         i  Number of non-virtual cells in the i-direction (one block)
c     nj         i  Number of non-virtual cells in the j-direction (one block)
c     norms      o  reserved
c     ntimlv     i  Number of time levels
c     ntrnsp     i  Number of transport equations
c     nturb      i  Number of extra transport equations for turbulence
c     ntype      i  Type of norm used for stopcriterium
c     nvirtual   i  Number of rows of virtual cells
c     solut      i  reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, ieq, j

c     i         General loop variable
c     ieq       Equation number
c     j         General loop variable
c **********************************************************************
c
c                       I/O
c
c     None
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c     Compute norms for the velocity
c
c     Compute norms for the pressure
c
c     Do ieq = 3, 2+ntrnp+nturb
c
c        Compute norms for the transport/turbulence equation
c
c     Enddo
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      do ieq = 1, 2 + ntrnsp + nturb
         norms(ieq,1) = 0d0
         norms(ieq,2) = 0d0
      end do

      if ( ntype.eq.0 ) then

c     --- L2norm

c         --- Compute norms for the velocity

         ieq = 1

c        --- V1 component

         do i = 1, ni
            do j = 1, nj
               norms(ieq,1) = norms(ieq,1) +
     +            5d-1*(solut(i,j,1,ntimlv) -
     +                  solut(i,j,1,ntimlv-1))**2 +
     +            5d-1*(solut(i+1,j,1,ntimlv) -
     +                  solut(i+1,j,1,ntimlv-1))**2
               norms(ieq,2) = norms(ieq,2) +
     +            5d-1*solut(i,j,1,ntimlv)**2 +
     +            5d-1*solut(i+1,j,1,ntimlv)**2
            end do
         end do

c        --- V2 component

         do i = 1, ni
            do j = 1, nj
               norms(ieq,1) = norms(ieq,1) +
     +            5d-1*(solut(i,j,2,ntimlv) -
     +                  solut(i,j,2,ntimlv-1))**2 +
     +            5d-1*(solut(i,j+1,2,ntimlv) -
     +                  solut(i,j+1,2,ntimlv-1))**2
               norms(ieq,2) = norms(ieq,2) +
     +            5d-1*solut(i,j,2,ntimlv)**2 +
     +            5d-1*solut(i,j+1,2,ntimlv)**2
            end do
         end do

c        --- Compute norms for the pressure

         ieq = 2
         do i = 1, ni
            do j = 1, nj
              norms(ieq,1) = norms(ieq,1) + solut(i,j,3,ntimlv)**2

c             --- Note: isstcr is called before isshift and therefore
c                 timelevel ntimlv-1 contains the pressure at the
c                 previous time level and not at the current time level.

              norms(ieq,2) = norms(ieq,2) +
     +            (solut(i,j,3,ntimlv-1) + solut(i,j,3,ntimlv))**2
            end do
         end do

c        --- Compute norms for the transport and turbulence equations

         do ieq = 3, 2 + ntrnsp + nturb

            do i = 1, ni
               do j = 1, nj
                  norms(ieq,1) = norms(ieq,1) +
     +               (solut(i,j,ieq+1,ntimlv) -
     +                solut(i,j,ieq+1,ntimlv-1))**2
                  norms(ieq,2) = norms(ieq,2) +
     +               solut(i,j,ieq+1,ntimlv)**2
               end do
            end do

         end do

      else if ( ntype.eq.1 ) then

c     --- maxnorm

         ieq = 1

c        --- V1 component

         do i = 1, ni
            do j = 1, nj
               norms(ieq,1) = max(norms(ieq,1),
     +            5d-1*abs(solut(i,j,1,ntimlv)
     +                   - solut(i,j,1,ntimlv-1)) +
     +            5d-1*abs(solut(i+1,j,1,ntimlv)
     +                    -solut(i+1,j,1,ntimlv-1)) )
               norms(ieq,2) = max(norms(ieq,2),
     +            5d-1*abs(solut(i,j,1,ntimlv) ) +
     +            5d-1*abs(solut(i+1,j,1,ntimlv)) )
            end do
         end do

c        --- V2 component

         do i = 1, ni
            do j = 1, nj
               norms(ieq,1) = max(norms(ieq,1),
     +            5d-1*abs(solut(i,j,2,ntimlv)-solut(i,j,2,ntimlv-1)) +
     +            5d-1*abs(solut(i,j+1,2,ntimlv)
     +                    -solut(i,j+1,2,ntimlv-1)))
               norms(ieq,2) = max(norms(ieq,2),
     +            5d-1*abs(solut(i,j,2,ntimlv)) +
     +            5d-1*abs(solut(i,j+1,2,ntimlv)) )
            end do
         end do

c        --- Compute norms for the pressure

         ieq = 2
         do i = 1, ni
            do j = 1, nj

c           --- Note: isstcr is called before isshift and therefore
c               timelevel ntimlv-1 contains the pressure at the
c               previous time level and not at the current time level.

               norms(ieq,1) = max(norms(ieq,1),
     +                        abs(solut(i,j,3,ntimlv)) )
               norms(ieq,2) = max(norms(ieq,2),
     +            abs(solut(i,j,3,ntimlv-1) + solut(i,j,3,ntimlv)) )

            end do
         end do

c        --- Compute norms for the transport and turbulence equations

         do ieq = 3, 2 + ntrnsp + nturb

            do i = 1, ni
               do j = 1, nj
                  norms(ieq,1) = max(norms(ieq,1),
     +               abs(solut(i,j,ieq+1,ntimlv) -
     +               solut(i,j,ieq+1,ntimlv-1)))
                  norms(ieq,2) = max(norms(ieq,2),
     +               abs(solut(i,j,ieq+1,ntimlv)) )
               end do
            end do

         end do

      end if
      end
