      function isstdfl(press, istartup, rincav)
c ======================================================================
c
c     programmer     Duncan van der Heul
c     version      1.2    date 03-03-1999     Duncan: rincav added to parlist
c     version      1.1    date 27-07-1998     Duncan: parameter istartup added
c     version      1.0    date 05-03-1998
c
c
c   copyright (c) 1998-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   IsNas compressible function isstdfl calculates the dimensional
c   density out of the dimensional equation of state based on the
c   dimensional pressure
c **********************************************************************
c
c                       KEYWORDS
c
c      compressible, equation_of_state
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer istartup
      double precision press, isstdfl, rincav(*)

c     isstdfl    o  density rho from dimensionfull eos
c     istartup   i  If 1 the cavity calculation is in the startup procedure,
c                   if 0 not
c     press      i  pressure
c     rincav     i  reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      double precision alpha1f, alpha2f, bigfksi, biggksi,
     + c1, c2, c3, c4, c5, delrho, ksi, npoly, pres1tr,
     + pres2tr, rho0, rho2

c     alpha1f   Parameter in F(x)
c     alpha2f   Parameter in F(x)
c     bigfksi   F(x)
c     biggksi   G(x)
c     c1        General help variable
c     c2        General help variable
c     c3        General help variable
c     c4        General help variable
c     c5        General help variable
c     delrho    Density jump across transition
c     ksi       The scaled transition pressure
c     npoly     Degree of the interpolation polynomial
c     pres1tr   Pressure at start of transition
c     pres2tr   Pressure at end of transition
c     rho0      Density for p=0
c     rho2      Density at end of transition
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c            if (P < = P1) then
c               rho is linear function of P
c            else
c               if (P > P1 & P < P2) then
c                   ksi = helpvariable(P)
c                   rho is polynomial of degree npoly of ksi
c               else
c                   rho is linear function of P
c               end if
c            end if
c            isstdfl=rho
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- input variables

      rho0      = 0.1d0
      delrho    = rincav(5)
      pres2tr   = rincav(4)
      pres1tr   = rincav(3)
      c1        = rincav(2)
      c4        = rincav(1)

c     --- write(*,*) 'delrho  ',delrho
c         write(*,*) 'pres2tr ',pres2tr
c         write(*,*) 'pres1tr ',pres1tr
c         write(*,*) 'c1      ',c1
c         write(*,*) 'c4      ',c4
c         --- constants to be computed

      c3    = (pres2tr-pres1tr)*(c4-c1)
      c2    = c1*(pres1tr-pres2tr)+delrho-c3/2d0
      c5    = rho0+c1*pres2tr+c2+c3/2d0
      rho2  = c5

c     --- Actual equation of state

      if( istartup.eq.1) then

c     --- fully wetted flow

         isstdfl=rho2+c4*(press-pres2tr)

      else

c     --- first linear part

         if(press.lt.pres1tr) then
            isstdfl=rho0+c1*press
         else

c        --- second linear part

            if(press.gt.pres2tr) then
               isstdfl=rho2+c4*(press-pres2tr)
            else

c           --- the transition part

c               --- function F(ksi) & function G(ksi)

c               --- variables to be computed

                npoly = 9
                ksi = (press-pres1tr)/(pres2tr-pres1tr)
                alpha1f = 2*(npoly-1d0)/(npoly+1d0)
                alpha2f = 2d0**(-1d0*npoly)*(2d0-alpha1f)
                if (ksi.le.0.5d0) then
                  bigfksi= 1d0/alpha2f*(ksi)**npoly*(1-alpha1f*ksi)
                  biggksi= 1d0/8d0*(2d0*ksi)**(npoly+1d0)*
     +                    (1d0-(npoly-1d0)/(npoly+2d0)*2d0*ksi)
                else
                  bigfksi= 1d0-1d0/alpha2f*(1-ksi)**npoly*
     +                      (alpha1f*ksi+1d0-alpha1f)
                  biggksi= ksi-0.5d0-1d0/8d0*(2d0-2d0*ksi)**(npoly+1d0)*
     +                    ((npoly-1d0)/(npoly+2d0)*(2d0-2d0*ksi)-1d0)

                end if

               isstdfl=rho0+c1*press+c2*bigfksi+c3*biggksi

            end if
         end if
      end if
      end
