      subroutine issysp ( iinput, rinput, matrix, gmat, rhside, coefs,
     +                    solut, coor, infmg, infmt, geom, igeom,
     +                    infbnd, work, eddy, densup, itopol )
c ======================================================================
c
c     programmer   Kees Kassels
c     version      3.1   date    06-12-1999 Duncan: added itopol to parameter
c                                                   list, new call to ispres2d
c     version      3.0   date    02-02-1999 Kees K: axisymmetric
c     version      2.9   date    14-05-1998 Duncan: extension for saving
c                                           coefficients dens upwind
c                                           new call to ispres2d
c     version      2.8   date    04-06-1998 Charles: call to iscont2d changed
c     version      2.7   date    02-12-1997 Guus: new call to iscont2d
c     version      2.6   date    13-10-1997 Charles: collocated allowed
c     version      2.5   date    03-10-1996 Kees K.: newdiscr removed
c     version      2.4   date    26-02-1996 Hester: parameter list ispres2d
c                                                    changed
c     version      2.3   date    06-02-1996 Kees K.: call of ispres3d
c     version      2.2   date    22-11-1995 Kees K.: call of isgmv changed
c     version      2.1   date    10-10-1995 Guus: new compuation of matrix
c                                           and rhs
c     version      2.0   date    19-07-1995 Guus: new parameter list
c     version      1.5   date    05-07-1995 Hester: cleaned up
c     version      1.4   date    30-05-1995 Hester:extended for compress. flow
c     version      1.3   date    22-02-1995 Guus: incorporation prints
c     version      1.2   date    07-10-1994 M. Kooper Theta method implemented
c     version      1.1   date    05-09-1994 R. van Nooyen 2D and 3D version
c     version      1.0   date    16-12-1993
c
c
c   copyright (c) 1993-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     create system of equations i.e. matrix and right-hand side
c     pressure equation
c **********************************************************************
c
c                       KEYWORDS
c
c     pressure_equation
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinput(*), infmg(*), infmt(*), infbnd(*), igeom(*),
     +        itopol(*)
      double precision rinput(*), matrix(*), gmat(*),
     +                 rhside(*), coefs(*), solut(*), coor(*), geom(*),
     +                 work(*), eddy(*), densup(*)

c     coefs   i    reserved
c     coor    i    reserved
c     densup  i    coefficients of density upwind in pressure matrix
c     eddy    i    reserved
c     geom    i    reserved
c     gmat    i    Gradient matrix
c     igeom   i    Integer array that returns the closest cell on a wall
c     iinput  i    reserved
c     infbnd  i    reserved
c     infmg   i    Integer information for Pressure-gradient matrix
c     infmt   i    Integer information for Pressure and Transport matrix
c     itopol  i    reserved
c     matrix  i    reserved
c     rhside  i    reserved
c     rinput  i    reserved
c     solut   i    reserved
c     work         Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ndim,
     +        ni, nj, nk, iprinmt, iprinrhs,
     +        itime1, itime2,
     +        itimeact, itimest, ipiintim, ipiindsc,
     +        ipmctg, ipmctinvg, snconct,  nvirtual,
     +        mshtyp, alphad, npointsv, idiscret,
     +        ipinvjac, ipcva, ipadir, ipbicof, ipibicof, axisym

c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     axisym    In axisymmetric case axisym=1 else axisym=0
c     idiscret  Type of discretization
c     ipadir    Starting address of array ADIR in array WORK
c     ipbicof   Starting address of array bicof in buffer
c     ipcva     Starting address of array CVA in array WORK
c     ipibicof  Starting address of array ibicof in ibuffr
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiintim  Starting address of array IINTIM in array IINPUT
c     ipinvjac  Starting address of array invjac in geom
c     ipmctg    Pointer to mct in infmat
c     ipmctinvg Pointer to mctinv in infmat
c     iprinmt   Indicates whether the matrix must be printed or not
c               Possible values
c               0:  the matrix is not printed
c               1:  the standard matrix is printed
c               2:  the gradient matrix is printed
c               3:  the standard and the gradient matrix are printed
c               4:  the pressure matrix is printed
c               5:  the standard and the pressure matrix are printed
c               6:  the gradient and the pressure matrix are printed
c               7:  the standard, thegradient and the pressure matrix
c                   are printed
c     iprinrhs  Indicates whether the right-hand side must be printed or not
c               Possible values
c               0:  the right-hand side is not printed
c               1:  the standard right-hand side is printed
c               2:  the pressure right-hand side is printed
c               3:  the standard and the pressure  right-hand side are
c                   printed
c     itime1    First time at which the debug prints must be performed
c     itime2    Last time at which the debug prints must be performed
c     itimeact  Actual time step
c     itimest   Time step with respect to debug printing
c     mshtyp    Type of grid arrangement
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npointsv  Number of points in the mesh with virtual points
c     nvirtual  Number of rows of virtual cells
c     snconct   Number of connections in one row of the transport matrix.
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISCONT2D  Compute contravariant base vectors in centroids of
c               alphad cells two-dimensional case
c     ISCOVA2D  Fill covariant base vectors in all integration points in R2
c     ISGMAT2D  Compute gradient matrix in R^2
c     ISPRES2D  Compute pressure matrix in R2
c     ISPRES3D  Compute pressure matrix in R3
c     ISPRIMT1  Print 2D matrix
c     ISPRIMT2  Print 3D matrix
c     ISPRIRH1  Print 2D right-hand side
c     ISPRIRH2  Print 3D right-hand side
c     ISSETCON  Make array equal to a constant
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      ipiintim = iinput(4)
      ipiindsc = iinput(6)
      ni       = iinput( 51 )
      nj       = iinput( 52 )
      nk       = iinput( 53 )
      nvirtual = iinput( 54 )
      ndim     = iinput( 57 )
      mshtyp   = iinput( 75 )
      idiscret = iinput(ipiindsc+1)
      axisym   = iinput(ipiindsc+9)
      iprinmt  = iinput(ipiindsc+10)
      iprinrhs = iinput(ipiindsc+11)
      itime1   = iinput(ipiindsc+12)
      itime2   = iinput(ipiindsc+13)
      itimest  = iinput(ipiindsc+14)
      itimeact = iinput(ipiintim+3)
      snconct  = infmt(6)
      npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
      ipcva    = 1
      ipadir   = ipcva+16*npointsv
      ipinvjac = npointsv*4 + 1
      ipbicof  = ipinvjac + npointsv*(2**ndim)
      ipibicof = 1

      if ( ndim.eq.2  ) then

         if ( mshtyp.eq.1 ) then

c        --- If collocated, building of the G matrix
c            Fill covariant basis vectors in cva for all integration points
c            and contravariant vectors

            call issetcon ( work(ipcva), 16*npointsv, 0d0 )
            call iscova2d ( coor, work(ipcva), ni, nj, nvirtual )

            do alphad = 1, ndim

               call iscont2d ( work(ipadir), geom,
     +                         work(ipcva), ni, nj, nvirtual, alphad )

               call isgmat2d ( alphad, ni, nj, nvirtual, gmat,
     +                         work(ipcva), geom(ipinvjac),
     +                         infbnd(1), infbnd(13), idiscret,
     +                         geom, work(ipadir) )
            end do
         end if

c        --- New discretization, construct matrix and right-hand side
c            in a new way

         call ispres2d ( ni, nj, nvirtual, matrix, work, coefs, gmat,
     +                   rinput, iinput, geom, solut, infbnd, coor,
     +                   rhside, eddy,
     +                   idiscret, geom(ipbicof), igeom(ipibicof),
     +                   densup, axisym, itopol)

      else

         ipmctg      = infmg(11)
         ipmctinvg   = infmg(12)
         call ispres3d ( ni, nj, nk, ndim, nvirtual, matrix, work,
     +                   coefs, gmat, rinput, iinput, geom, solut,
     +                   infbnd, coor, rhside, infmg(ipmctg),
     +                   infmg(ipmctinvg) )

      end if

      if ( iprinmt.ge.4 ) then

c     --- Print pressure matrix

         if ( itimeact.ge.itime1 .and. itimeact.le.itime2 .and.
     +        mod(itimeact-itime1,itimest).eq.0 ) then

            if ( ndim.eq.2 ) then
               call isprimt1 ( 'pressure matrix', matrix, ni, nj,
     +                         snconct, 2 )
            else
               call isprimt2 ( 'pressure matrix', matrix, ni, nj, nk,
     +                         snconct, 2 )
            end if
         end if
      end if

      if ( iprinrhs.gt.1 ) then

c     --- Print pressure right-hand side

         if ( itimeact.ge.itime1 .and. itimeact.le.itime2 .and.
     +        mod(itimeact-itime1,itimest).eq.0 ) then

            if ( ndim.eq.2 ) then
               call isprirh1 ( 'pressure right-hand side', rhside,
     +                          ni, nj, 1 )
            else
               call isprirh2 ( 'pressure right-hand side', rhside,
     +                          ni, nj, nk, 1 )
            end if
         end if
      end if

      end
