      subroutine istimi01 ( iinput, rinput, neq, t0, tend, dt, thetarr,
     +                      toutin, touten, toutst, rinit, rend, rstep,
     +                      methodtim, nfrac, restartw, cartesian,
     +                      nsubtst )
c ======================================================================
c
c     programmer   Kees Kassels
c     version      3.0   date    08-09-1998 Guus: remove ndim, icoupled
c     version      2.1   date    27-03-1998 Kees K: extra parameter nsubtst
c     version      2.0   date    04-09-1997 Guus: extra parameter icoupled, ndim
c     version      1.5   date    22-04-1996 Kees K. restart changed
c     version      1.4   date    22-03-1996 Kees K. refine added
c     version      1.3   date    09-05-1995 Kees K. rinit, rend, rstep added
c     version      1.2   date    11-01-1994 M. Kooper: fractional theta method
c     version      1.1   date    17-10-1994 M. Kooper: theta value implemented
c     version      1.0   date    16-05-1994
c
c   copyright (c) 1994-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     get number of equations and time integration values
c
c **********************************************************************
c
c                       KEYWORDS
c
c     time integration
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinput(*), methodtim, neq, nfrac, restartw, cartesian,
     +        icoupled, nsubtst
      double precision rinput(*), t0, tend, dt, thetarr(*), toutin,
     +                 touten, toutst, rinit, rend, rstep

c     cartesian i    Cartesian=1, write cartesian values in the vertices to
c                    the restart file
c                    cartesian=0, dump contravariant solution vector
c     dt        o    Time step
c     icoupled  o    If 0 the momentum equations are coupled, if 1 they are not
c     iinput    i    reserved
c     methodtim i    Type of time integration
c     ndim      o    Dimension of space (2 or 3)
c     neq       o    Number of equations
c     nfrac     o    Number of fractional time steps
c     nsubtst   i    Number of sub-timesteps
c     rend      i    End time for writing to restart file
c     restartw  o    If restartw > 0, write restart file
c     rinit     i    Start time for writing solutions to the restart file
c     rinput    i    reserved
c     rstep     i    Time step for writing to restart file
c     t0        o    Initial time
c     tend      o    End time
c     thetarr   o    Array of thetas for different theta methods
c     touten    o    End time for output of results
c     toutin    o    Initial time for output of results
c     toutst    o    Time step for output of results
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ifrac, ipiintim, iprintim, ndegfd, ndim, ndt

c     ifrac     Actual fractional time step
c     ipiintim  Starting address of array IINTIM in array IINPUT
c     iprintim  Starting address of array RINTIM in array RINPUT
c     ndegfd    Number of degrees of freedom
c     ndt       Number of variable time steps
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS      removes name of subroutine from error list
c     ISEROPEN      adds name of subroutine to error list
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('istimi01')

      ndegfd   = iinput(55)
      ndim     = iinput(57)
      icoupled = iinput(76)

      if ( icoupled.eq.0 ) then
         neq = ndegfd - ndim + 1
      else
         neq = ndegfd
      end if

      iprintim = iinput(29)
      ipiintim = iinput( 4)

      methodtim = iinput( ipiintim -1 + 1 )
      nfrac     = iinput( ipiintim -1 + 6 )
      ndt       = iinput( ipiintim -1 + 8 )
      restartw  = iinput( ipiintim -1 + 11)
      cartesian = iinput( ipiintim -1 + 12)
      nsubtst   = iinput( ipiintim -1 + 16)

      t0        = rinput( iprintim -1 + 2        )
      tend      = rinput( iprintim -1 + 50 + ndt )
      dt        = rinput( iprintim -1 + 41       )
      toutin    = rinput( iprintim -1 + 6        )
      touten    = rinput( iprintim -1 + 7        )
      toutst    = rinput( iprintim -1 + 8        )
      rinit     = rinput( iprintim -1 + 9        )
      rend      = rinput( iprintim -1 + 10       )
      rstep     = rinput( iprintim -1 + 11       )

      if ( toutin.lt.t0 ) toutin = t0

c     --- The choice of the array THETARR is based on METHOD and NFRAC

      do ifrac = 1, nfrac

        thetarr( 2*ifrac-1 ) = rinput( iprintim -1 + 30 + 2*ifrac-1 )
        thetarr( 2*ifrac   ) = rinput( iprintim -1 + 30 + 2*ifrac   )

      end do

      call iserclos('istimi01')

      end
