      subroutine iswllfn3( iequa , nside , i    , j     , bndcon  ,
     +                     coefs , coor  , eddy , solut , rintur  ,
     +                     ndim  , ndir  , t1dir, t2dir , nsign   ,
     +                     ni    , nj    , nk   , ndegfd, nvirtual,
     +                     nturb , ncoefs, maxb , rough , it1lo   ,
     +                     it2lo , it1hi , it2hi, ieq   , rincof  ,
     +                     wtemp                                   )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 2.0    date   04-06-1996 Marcel: wall law for temperature
c        version 1.1    date   20-02-1996 Marcel: bug fixed
c        version 1.0    date   12-12-1995 (extracted from iswllfnc)
c
c
c   copyright (c) 1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Use wall functions to fill BNDCON for the momentum and
c     dissipation rate equations in R3
c **********************************************************************
c
c                       KEYWORDS
c
c     3d
c     turbulence
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nside, i, j, ndim, nvirtual, ni, nj, nk, iequa,
     +        ndegfd, nturb, ncoefs, maxb, ndir, t1dir,
     +        t2dir, nsign, it1lo, it1hi, it2lo, it2hi, ieq
      double precision coor(*), rintur(*), rough, rincof(*), wtemp,
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,
     +                       1:ndegfd),
     +                 coefs(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,
     +                       1:ncoefs),
     +                 eddy(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual),
     +                 bndcon(1:maxb,it1lo:it1hi,it2lo:it2hi)

c     bndcon    o    reserved
c     coefs     i    reserved
c     coor      i    reserved
c     eddy      i    reserved
c     i         i    General loop variable
c     ieq       i    Equation number
c     iequa     i    General equation sequence number
c     it1hi     i    Upper bound on first tangential index for destination
c     it1lo     i    Lower bound on first tangential index for destination
c     it2hi     i    Upper bound on second tangential index for destination
c     it2lo     i    Lower bound on second tangential index for destination
c     j         i    General loop variable
c     maxb      i    Maximum number of real values for one boundary condition
c     ncoefs    i    Number of coefficients
c     ndegfd    i    Number of degrees of freedom
c     ndim      i    Dimension of space (2 or 3)
c     ndir      i    Direction of normal vector on this side
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nside     i    Number of boundary for which we need the
c                    information
c     nsign     i    Inner product of outward normal in computational domain
c                    with the base vector perpendicular to the face
c     nturb     i    Number of extra transport equations for turbulence
c     nvirtual  i    Number of rows of virtual cells
c     rincof    i    reserved
c     rintur    i    reserved
c     rough     i    The average roughness height
c     solut     i    reserved
c     t1dir     i    Direction of first tangential vector on this side
c     t2dir     i    Direction of second tangential vector on this side
c     wtemp     i    Wall temperature
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, kp
      double precision ndist, physut, tauw, uplus,
     +                 rhow, mu, tke, facs(1:2),
     +                 cp, const, yplus, diffus

c     const     General const
c     cp        Specific heat coefficient at constant pressure
c     diffus    Diffusivity (thermal)
c     facs      Multiplication factor
c     ip        Local counting variable i-direction
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c     mu        The laminar viscosity
c     ndist     Distance normal to a solid wall
c     physut    The physical velocity tangent to the wall
c     rhow      Specific weight
c     tauw      The wall shear stress
c     tke       The turbulent kinetic energy
c     uplus     The dimensionless tangential velocity
c     yplus     The dimensionless normal distance to the wall
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISLOGW        Compute the wall shear stress with log-law based
c                   wall condition depending on smoothness or roughness
c                   of the boundary
c     ISWLLQN3      Determine essential wall quantities
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      if ( nside.eq.1 ) then
        ip = 1
        jp = int((i+1)/2)
        kp = int((j+1)/2)
      else if ( nside.eq.2 ) then
        ip = ni
        jp = int((i+1)/2)
        kp = int((j+1)/2)
      else if ( nside.eq.3 ) then
        ip = int((j+1)/2)
        jp = 1
        kp = int((i+1)/2)
      else if ( nside.eq.4 ) then
        ip = int((j+1)/2)
        jp = nj
        kp = int((i+1)/2)
      else if ( nside.eq.5 ) then
        ip = int((i+1)/2)
        jp = int((j+1)/2)
        kp = 1
      else if ( nside.eq.6 ) then
        ip = int((i+1)/2)
        jp = int((j+1)/2)
        kp = nk
      end if
      if ( ip.gt.ni ) ip = ni
      if ( jp.gt.nj ) jp = nj
      if ( kp.gt.nk ) kp = nk

c     --- First, compute two essential wall quantities: distance
c         perpendicular to the wall and physical tangential velocity

      call iswllqn3( physut, ndist, facs, coor, solut, ni, nj, nk,
     +               nvirtual, ndir, ip, jp, kp, nside, nsign,
     +               t1dir, t2dir )

      if ( iequa.eq.1 ) then

c     --- momentum equations; fill BNDCON using wall functions

        rhow = coefs(ip,jp,kp,1)
        mu = coefs(ip,jp,kp,2)-coefs(ip,jp,kp,1)*eddy(ip,jp,kp)
        tke = solut(ip,jp,kp,ndegfd-1)

        call islogw( tauw, uplus, physut, ndist,
     +               tke, rintur, rhow, mu, rough )

        bndcon(1,i,j) =  0d0
        bndcon(2,i,j) = -tauw*facs(1)
        bndcon(3,i,j) = -tauw*facs(2)

      else if ( iequa.eq.ieq ) then

c     --- temperature equation; fill BNDCON using wall functions

        rhow = rincof(1)
        mu = rincof(2)
        diffus = coefs(ip,jp,kp,4)-
     +           coefs(ip,jp,kp,1)*eddy(ip,jp,kp)/rintur(9)
        cp = coefs(ip,jp,kp,1)/rhow
        tke = solut(ip,jp,kp,ndegfd-1)
        yplus = rhow*( rintur(3)**(25d-2) )*sqrt(tke)*ndist/mu
        const = rintur(9)
        if ( yplus.le.11.3d0 ) const = cp*mu/diffus

        call islogw ( tauw, uplus, physut, ndist,
     +                tke, rintur, rhow, mu, 0d0 )

        bndcon(1,i,j) = 0d0
        bndcon(2,i,j) = 0d0
        if ( tauw.ne.0d0 )
     +    bndcon(2,i,j) = cp*(wtemp-solut(ip,jp,kp,ieq+2))*tauw/
     +                    (const*physut)

      else if (iequa.eq.ndegfd-ndim+1-nturb+2) then

c     --- dissipation rate equation; fill BNDCON using wall functions

        tke = solut(ip,jp,kp,ndegfd-1)

        bndcon(1,i,j)  = ( rintur(3)**(75d-2) )*( tke**(15d-1) )/
     +                   ( rintur(1) * ndist )

      end if

      end
