      subroutine iswllqn2 ( ndist , physut, ni , nj, nvirtual, ip,
     +                      jp, coor, invjac, veloc, ndir, tdir,
     +                      nsign, tsign )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.2    date   12-09-1995 Marcel: definitely correction of
c                                               several directions/signs
c        version 1.1    date   11-09-1995 (Marcel: correction velocity direction
c        version 1.0    date   28-08-1995 (extracted from iswllqnt)
c
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                      DESCRIPTION
c
c     Computes some quantities close to the wall: the normal distance
c     from first computational point to the wall and the velocity
c     tangent to the wall
c
c **********************************************************************
c
c                      KEYWORDS
c
c     turbulence
c
c **********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, ndir, ip, jp, tdir, nsign, tsign
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2),
     +                 veloc(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2),
     +                 invjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual),
     +                 physut, ndist

c     coor      i    reserved
c     invjac    i    1/sqrt(g) in pnttyp
c     ip        i    Local counting variable i-direction
c     jp        i    Local counting variable j-direction
c     ndir      i    Direction of normal vector on this side
c     ndist     o    Distance normal to a solid wall
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nsign     i    Inner product of outward normal in computational domain
c                    with the base vector perpendicular to the face
c     nvirtual  i    Number of rows of virtual cells
c     physut    o    The physical velocity tangent to the wall
c     tdir      i    Direction of tangent vector on this side
c     tsign     i    Inner product of tangential vector defined counter
c                    clockwise with respect to the outward normal and the
c                    base vector along the side
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c **********************************************************************
c
c                      COMMON BLOCKS
c
c **********************************************************************
c
c                      LOCAL PARAMETERS

      integer ipjp(4,2), i
      double precision acov(2,2), xq(2), tvec(2), xp1(2), u(2),
     +                 eucnorm

c     acov      Covariant base vectors.
c               Meaning: a(i,j) gives the value of the i-th
c                        component of the j-th base vector
c     eucnorm   Euclidian norm of a vector
c     i         General loop variable
c     ipjp      Array containing the ip and jp indices of the 4 vertices of
c               the cell
c               ipjp(i,j): i=1,4; j=1,2
c               j=1: ip     j=2: jp
c     tvec      Tangential vector
c     u         Cartesian velocities
c     xp1       Array containing the co-ordinates of the centroid
c     xq        Array containing the co-ordinates of the point Q at the boundary
c **********************************************************************
c
c                      SUBROUTINES CALLED
c
c **********************************************************************
c
c                      I/O
c
c **********************************************************************
c
c                      ERROR MESSAGES
c
c **********************************************************************
c
c                      PSEUDO CODE
c
c    Consider the following domain and cell:
c
c                4
c        ----------------- 3 ----------------------- 4
c        |               |                  |                     |
c        |               |                  |                     |
c        |               |                  |         + P         |
c       1|  ^2           | 2                |                     |
c        |  |            |                  |                     |
c        |  --->1        |                  |          Q          |
c        ----------------- 1 ----------+------------ 2
c               3
c
c    Organization of quantities as function of boundary
c
c    Bnd    ndir  tdir  nsign  tsign      shift with respect to ip,jp
c                                          1      2      3      4
c     1      1     2     -1    -1         0,0    0,1    1,0    1,1
c     2      1     2      1     1         1,0    1,1    0,0    0,1
c     3      2     1     -1     1         0,0    1,0    0,1    1,1
c     4      2     1      1    -1         0,1    1,1    0,0    1,0
c
c    Some remarks are appropriate here regarding the normal and tangent vectors.
c    The normal and tangent directions of the velocity and stress boundary
c    conditions, as given by the user, are defined in counterclockwise manner.
c    This means that the normal vector is defined as the outward normal and
c    the tangent vector is defined counterclockwise.
c
c    However, the internal orientation, i.e. both tangent and normal vectors
c    are defined in `tdir' and `ndir' directions, respectively, will be used.
c
c    Notation: t  and n  are internal tangent and normal vectors, respectively.
c              -I     -I
c    The relationship between the internal and counterclockwise orientation
c    is given in Table (see `nsign' and `tsign').
c
c    Normal and tangent vectors to be used in this routine:
c
c     *
c    t  = x2 - x1 / || x2 - x1 ||
c    -    --   --      --   --
c
c     *
c    n  = ( -t2, t1 )
c    -       --  --
c                                        *
c    This means that the tangent vector t  is always defined in
c                                            *
c    `tdir' direction and the normal vector n  is defined in
c     counterclockwise manner.
c
c                  *                        *
c    We have t  = t  and n  = -nsign.tsign n
c            -I   -      -I                -
c
c
c    Formulae to be used:
c
c
c    dist P to boundary is: |n . PQ|
c                            -   --
c
c    tangential velocity along the wall:  u.t
c                                         - -
c
c                  (alpha)
c    u = a        V          / sqrt(g)
c    -   -(alpha)
c
c    and
c
c    t = tsign t /|| t ||
c    -         -I    -I
c                                                                      *
c ======================================================================
c
c     --- Compute actually two essential wall quantities: distance
c         perpendicular to the wall and physical tangential velocity
c         Fill array ipjp as function of ip, jp and iside

      do i = 1, 4
         ipjp(i,1) = ip
         ipjp(i,2) = jp
      end do
      if ( nsign.eq.1 ) then

c     --- nsign = 1, sides 2 and 4

         ipjp(1,ndir) = ipjp(1,ndir)+1
         ipjp(2,1)    = ipjp(2,1)+1
         ipjp(2,2)    = ipjp(2,2)+1
         ipjp(4,tdir) = ipjp(4,tdir)+1

      else

c     --- nsign = -1, sides 1 and 3

         ipjp(3,ndir) = ipjp(3,ndir)+1
         ipjp(4,1)    = ipjp(4,1)+1
         ipjp(4,2)    = ipjp(4,2)+1
         ipjp(2,tdir) = ipjp(2,tdir)+1

      end if

c     --- Compute the unit tangential vector and the co-ordinates of P and Q
c         as well as the covariant base vector and the Caresian velocity vector
c         in the centroid

      do i = 1, 2
         tvec(i) = coor(ipjp(2,1),ipjp(2,2),i)-
     +             coor(ipjp(1,1),ipjp(1,2),i)
         xp1(i) = 0.25d0*(coor(ipjp(1,1),ipjp(1,2),i)+
     +                    coor(ipjp(2,1),ipjp(2,2),i)+
     +                    coor(ipjp(3,1),ipjp(3,2),i)+
     +                    coor(ipjp(4,1),ipjp(4,2),i))
         xq(i) = 0.5d0*(coor(ipjp(1,1),ipjp(1,2),i)+
     +                  coor(ipjp(2,1),ipjp(2,2),i))
         acov(1,i) = 0.5d0*(coor(ip+1,jp+1,i)-
     +                      coor(ip  ,jp+1,i)+
     +                      coor(ip+1,jp  ,i)-
     +                      coor(ip  ,jp  ,i))
         acov(2,i) = 0.5d0*(coor(ip+1,jp+1,i)-
     +                      coor(ip+1,jp  ,i)+
     +                      coor(ip  ,jp+1,i)-
     +                      coor(ip  ,jp  ,i))
         u(i) = (acov(1,i)*(veloc(ip,jp,1)+veloc(ip+1,jp,1))+
     +           acov(2,i)*(veloc(ip,jp,2)+veloc(ip,jp+1,2)))*
     +          invjac(ip,jp)*0.5d0

      end do
      eucnorm = 1d0/sqrt(tvec(1)**2+tvec(2)**2)
      tvec(1) = tvec(1)*eucnorm
      tvec(2) = tvec(2)*eucnorm
      ndist = abs(-tvec(2)*(xp1(1)-xq(1)) + tvec(1)*(xp1(2)-xq(2)))
      physut = tsign*(u(1)*tvec(1) + u(2)*tvec(2))

      end
