      subroutine iswllqn3 ( physut, ndist, facs    , coor , veloc, ni,
     +                      nj    , nk   , nvirtual, ndir , ip   , jp,
     +                      kp    , nside, nsign   , tdir1, tdir2     )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 1.1    date   05-01-1996 some corrections, comments updated
c        version 1.0    date   13-12-1995 (extracted from iswlqt02)
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                      DESCRIPTION
c
c     Computes some quantities close to the wall in R3: the normal
c     distance from first computational point to the wall and the
c     velocity tangent to the wall
c
c **********************************************************************
c
c                      KEYWORDS
c
c     3d
c     turbulence
c
c **********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, ndir, ip, jp, kp, nside, nsign,
     +        tdir1, tdir2

      double precision physut, facs(2), ndist,
     +                 coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,3),
     +                 veloc(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,3)

c     coor      i    reserved
c     facs      i    Multiplication factor
c     ip        i    Local counting variable i-direction
c     jp        i    Local counting variable j-direction
c     kp        i    Local counting variable k-direction
c     ndir      i    Direction of normal vector on this side
c     ndist     i    Distance normal to a solid wall
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nside     i    Number of boundary for which we need the
c                    information
c     nsign     i    Inner product of outward normal in computational domain
c                    with the base vector perpendicular to the face
c     nvirtual  i    Number of rows of virtual cells
c     physut    i    The physical velocity tangent to the wall
c     tdir1     i    Direction of first tangent vector on current side
c     tdir2     i    Direction of second tangent vector on current side
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c **********************************************************************
c
c                      COMMON BLOCKS
c
c **********************************************************************
c
c                      LOCAL PARAMETERS

      integer i, j, ipjpkp(8,3), p, q, rq, s
      double precision fact, sqrtg, ut(3), acov(3,3),
     +                 ati(3), det, gac(3,3), jcta(3,3),
     +                 jctg(3,3), nvec(3), tvec(3), u(3), xp(3), xq(3)

c     acov      Covariant base vectors.
c               Meaning: a(i,j) gives the value of the i-th
c                        component of the j-th base vector
c     ati       Value of an inner product
c     det       Determinant
c     fact      Multiplication factor
c     gac       Covariant metric tensor
c     i         General loop variable
c     ipjpkp    Array containing the ip, jp and kp indices of the 8
c               vertices of the cell
c               ipjpkp(i,j): i=1,8; j=1,3
c               j=1: ip     j=2: jp     j=3: kp
c     j         General loop variable
c     jcta      Jacobian times contravariant basis vectors
c               in point pnttyp
c               (stored as vector index encoding given by istwo)
c     jctg      Jacobian times symmetric contravariant metric
c               tensor in point given by pnttyp
c               (stored as vector index encoding given by istwo)
c     nvec      Number of vectors to be written to the output file
c     p         Index in covariant base vectors
c     q         Index in covariant base vectors
c     rq        Index in covariant base vectors
c     s         Index in covariant base vectors
c     sqrtg     The Jacobian in the center of a cell
c     tvec      Tangential vector
c     u         Cartesian velocities
c     ut        Tangential velocity (physical)
c     xp        Auxiliary array to calculate boundary point number
c     xq        Array containing the co-ordinates of the point Q at the boundary
c
c **********************************************************************
c
c                      SUBROUTINES CALLED
c
c **********************************************************************
c
c                      I/O
c
c **********************************************************************
c
c                      ERROR MESSAGES
c
c **********************************************************************
c
c                      PSEUDO CODE
c
c    Consider the following domain and cell:
c
c                         ----------------- 5-----------------7
c                        /|      1       /|      /|              /|
c                       / |             / |     / |             / |
c                      /  |    6       /  |    /  |            /  |
c                     /3  |           /4  |   /   |           /   |
c         3          ----------------- | 6-----------------8   |
c       xi           |    |  2       |    |  |    |          |    |
c       |            |    |          |    |  |    |     *P   |    |
c       |            |    |          |    |  |    |     |    |    |
c       |        2   |    -----------|---/   |   1------|----|---/3
c       -------xi    |   /           |  /    |   /      |    |  /
c      /             |  /      5     | /     |  /       *Q   | /
c     / 1            | /             |/      | /             |/
c     xi             ----------------- 2-----------------4
c
c    Organization of quantities as function of boundary
c
c    Bnd    ndir  tdir  nsign  tsign
c     1      1    2,3    -1    -1,1
c     2      1    2,3     1     1,1
c     3      2    3,1    -1    -1,1
c     4      2    3,1     1     1,1
c     5      3    1,2    -1    -1,1
c     6      3    1,2     1     1,1
c
c    Bnd    shift with respect to ip,jp,kp
c
c            1      2      3      4      5      6      7      8
c     1    0,0,0  0,1,0  0,0,1  0,1,1  1,0,0  1,1,0  1,0,1  1,1,1
c     2    1,0,0  1,1,0  1,0,1  1,1,1  0,0,0  0,1,0  0,0,1  0,1,1
c     3    0,0,0  1,0,0  0,0,1  1,0,1  0,1,0  1,1,0  0,1,1  1,1,1
c     4    0,1,0  1,1,0  0,1,1  1,1,1  0,0,0  1,0,0  0,0,1  1,0,1
c     5    0,0,0  1,0,0  0,1,0  1,1,0  0,0,1  1,0,1  0,1,1  1,1,1
c     6    0,0,1  1,0,1  0,1,1  1,1,1  0,0,0  1,0,0  0,1,0  1,1,0
c
c
c    Formulae to be used:
c
c      t
c     u  = u - (u.n) n in centroid
c     -    -    - -  -
c
c                  alpha
c    u = a        V     / sqrt(g)
c    -   -(alpha)
c
c                 n       nn
c    u.n = nsign U /sqrt(g  )
c    - -
c
c            t   alpha                     n  (n)  nn
c    Hence, u = V      a       /sqrt(g) - U  a   /g
c           -          -(alpha)              -
c
c               (n)       nn
c    n = nsign a   /sqrt(g  )
c    -         -
c
c    dist P to boundary is: |n . PQ|
c                            -   --
c
c ======================================================================
c
c     --- Fill array ipjpkp as function of ip, jp, kp and iside

      do i = 1, 8
         ipjpkp(i,1) = ip
         ipjpkp(i,2) = jp
         ipjpkp(i,3) = kp
      end do

      if ( nsign.eq.1 ) then

c     --- nsign = 1, i.e. sides 2, 4 and 6

         ipjpkp(1,ndir) = ipjpkp(1,ndir) + 1

         ipjpkp(6,tdir1) = ipjpkp(6,tdir1) + 1

         ipjpkp(7,tdir2) = ipjpkp(7,tdir2) + 1

         ipjpkp(2,ndir) = ipjpkp(2,ndir) + 1
         ipjpkp(2,tdir1) = ipjpkp(2,tdir1) + 1

         ipjpkp(3,ndir) = ipjpkp(3,ndir) + 1
         ipjpkp(3,tdir2) = ipjpkp(3,tdir2) + 1

         if ( nside.eq.4 ) then
           ipjpkp(6,tdir2) = ipjpkp(6,tdir2) + 1
           ipjpkp(7,tdir1) = ipjpkp(7,tdir1) + 1
           ipjpkp(2,tdir2) = ipjpkp(2,tdir2) + 1
           ipjpkp(3,tdir1) = ipjpkp(3,tdir1) + 1
         end if

         ipjpkp(8,tdir1) = ipjpkp(8,tdir1) + 1
         ipjpkp(8,tdir2) = ipjpkp(8,tdir2) + 1

         ipjpkp(4,1) = ipjpkp(4,1) + 1
         ipjpkp(4,2) = ipjpkp(4,2) + 1
         ipjpkp(4,3) = ipjpkp(4,3) + 1

      else

c     --- nsign = -1, i.e. sides 1, 3 and 5

         ipjpkp(2,tdir1) = ipjpkp(2,tdir1) + 1

         ipjpkp(3,tdir2) = ipjpkp(3,tdir2) + 1

         ipjpkp(5,ndir) = ipjpkp(5,ndir) + 1

         ipjpkp(4,tdir1) = ipjpkp(4,tdir1) + 1
         ipjpkp(4,tdir2) = ipjpkp(4,tdir2) + 1

         ipjpkp(6,ndir) = ipjpkp(6,ndir) + 1
         ipjpkp(6,tdir1) = ipjpkp(6,tdir1) + 1

         ipjpkp(7,ndir) = ipjpkp(7,ndir) + 1
         ipjpkp(7,tdir2) = ipjpkp(7,tdir2) + 1

         if ( nside.eq.3 ) then
           ipjpkp(2,tdir2) = ipjpkp(2,tdir2) + 1
           ipjpkp(3,tdir1) = ipjpkp(3,tdir1) + 1
           ipjpkp(6,tdir2) = ipjpkp(6,tdir2) + 1
           ipjpkp(7,tdir1) = ipjpkp(7,tdir1) + 1
         end if

         ipjpkp(8,1) = ipjpkp(8,1) + 1
         ipjpkp(8,2) = ipjpkp(8,2) + 1
         ipjpkp(8,3) = ipjpkp(8,3) + 1

      end if

c     --- Compute the co-ordinates of P and Q, the covariant base vector
c         and the Caresian velocity vector times Jacobian in the centroid

      do i = 1, 3

         xp(i) = 125d-3*(coor(ipjpkp(1,1),ipjpkp(1,2),ipjpkp(1,3),i)+
     +                   coor(ipjpkp(2,1),ipjpkp(2,2),ipjpkp(2,3),i)+
     +                   coor(ipjpkp(3,1),ipjpkp(3,2),ipjpkp(3,3),i)+
     +                   coor(ipjpkp(4,1),ipjpkp(4,2),ipjpkp(4,3),i)+
     +                   coor(ipjpkp(5,1),ipjpkp(5,2),ipjpkp(5,3),i)+
     +                   coor(ipjpkp(6,1),ipjpkp(6,2),ipjpkp(6,3),i)+
     +                   coor(ipjpkp(7,1),ipjpkp(7,2),ipjpkp(7,3),i)+
     +                   coor(ipjpkp(8,1),ipjpkp(8,2),ipjpkp(8,3),i))
         xq(i) = 25d-2*(coor(ipjpkp(1,1),ipjpkp(1,2),ipjpkp(1,3),i)+
     +                  coor(ipjpkp(2,1),ipjpkp(2,2),ipjpkp(2,3),i)+
     +                  coor(ipjpkp(3,1),ipjpkp(3,2),ipjpkp(3,3),i)+
     +                  coor(ipjpkp(4,1),ipjpkp(4,2),ipjpkp(4,3),i))

         acov(i,1) = 25d-2*(coor(ip+1,jp,kp,i)-coor(ip,jp,kp,i)+
     +                      coor(ip+1,jp+1,kp,i)-coor(ip,jp+1,kp,i)+
     +                      coor(ip+1,jp,kp+1,i)-coor(ip,jp,kp+1,i)+
     +                      coor(ip+1,jp+1,kp+1,i)-coor(ip,jp+1,kp+1,i))
         acov(i,2) = 25d-2*(coor(ip,jp+1,kp,i)-coor(ip,jp,kp,i)+
     +                      coor(ip+1,jp+1,kp,i)-coor(ip+1,jp,kp,i)+
     +                      coor(ip,jp+1,kp+1,i)-coor(ip,jp,kp+1,i)+
     +                      coor(ip+1,jp+1,kp+1,i)-coor(ip+1,jp,kp+1,i))
         acov(i,3) = 25d-2*(coor(ip,jp,kp+1,i)-coor(ip,jp,kp,i)+
     +                      coor(ip+1,jp,kp+1,i)-coor(ip+1,jp,kp,i)+
     +                      coor(ip,jp+1,kp+1,i)-coor(ip,jp+1,kp,i)+
     +                      coor(ip+1,jp+1,kp+1,i)-coor(ip+1,jp+1,kp,i))

         u(i) = (acov(i,1)*(veloc(ip,jp,kp,1)+veloc(ip+1,jp,kp,1))+
     +           acov(i,2)*(veloc(ip,jp,kp,2)+veloc(ip,jp+1,kp,2))+
     +           acov(i,3)*(veloc(ip,jp,kp,3)+veloc(ip,jp,kp+1,3)))
     +          *5d-1

      end do

c     --- Determine contravariant base vector times Jacobian

      do i = 1, 3
         rq = mod( i+1,3 )
         s = mod( i+2,3 )
         if ( rq.eq.0 ) rq = 3
         if ( s.eq.0 ) s = 3
         do j = 1, 3
           p = mod( j+1,3 )
           q = mod( j+2,3 )
           if ( p.eq.0 ) p = 3
           if ( q.eq.0 ) q = 3
             jcta(i,j) = acov(rq,p)*acov(s,q)-acov(s,p)*acov(rq,q)
         end do
      end do

c     --- Determine Jacobian

      sqrtg = acov(1,1)*(acov(2,2)*acov(3,3)-acov(3,2)*acov(2,3)) +
     +        acov(2,1)*(acov(3,2)*acov(1,3)-acov(1,2)*acov(3,3)) +
     +        acov(3,1)*(acov(1,2)*acov(2,3)-acov(2,2)*acov(1,3))

c     --- Determine contravariant metric tensor times g

      do i = 1, 3
        do j = 1, 3
          jctg(i,j) = jcta(1,i)*jcta(1,j)+jcta(2,i)*jcta(2,j)+
     +                jcta(3,i)*jcta(3,j)
        end do
      end do

c     --- Compute the tangential velocity along the plane wall

      if ( ndir.eq.1 ) then
        fact = 5d-1*(veloc(ip,jp,kp,1)+veloc(ip+1,jp,kp,1))/jctg(1,1)
      else if ( ndir.eq.2 ) then
        fact = 5d-1*(veloc(ip,jp,kp,2)+veloc(ip,jp+1,kp,2))/jctg(2,2)
      else if ( ndir.eq.3 ) then
        fact = 5d-1*(veloc(ip,jp,kp,3)+veloc(ip,jp,kp+1,3))/jctg(3,3)
      end if

      do i = 1, 3
        ut(i) = u(i)/sqrtg - fact*jcta(i,ndir)
      end do

      physut = sqrt(ut(1)**2+ut(2)**2+ut(3)**2)

c     --- Compute unit tangential vector

      do i = 1, 3
        tvec(i) = ut(i)/(physut+1d-10)
      end do

c     --- Compute some factors according to the Mathematical manual, p. 32

      do i = 1, 3
        ati(i) = tvec(1)*acov(1,i)+tvec(2)*acov(2,i)+tvec(3)*acov(3,i)
        do j = 1, 3
          gac(i,j) = acov(1,i)*acov(1,j)+acov(2,i)*acov(2,j)+
     +               acov(3,i)*acov(3,j)
        end do
      end do

      det = gac(tdir1,tdir1)*gac(tdir2,tdir2) -
     +      gac(tdir2,tdir1)*gac(tdir1,tdir2)

      facs(1) = ( ati(tdir1)*gac(tdir2,tdir2) -
     +            ati(tdir2)*gac(tdir1,tdir2) )/det

      facs(2) = ( gac(tdir1,tdir1)*ati(tdir2) -
     +            gac(tdir2,tdir1)*ati(tdir1) )/det

c     --- Compute the unit normal vector at the wall

      if ( ndir.eq.1 ) then
        do i = 1, 3
          acov(i,2) = 5d-1*(coor(ip,jp+1,kp,i)-coor(ip,jp,kp,i)+
     +                      coor(ip,jp+1,kp+1,i)-coor(ip,jp,kp+1,i))
          acov(i,3) = 5d-1*(coor(ip,jp,kp+1,i)-coor(ip,jp,kp,i)+
     +                      coor(ip,jp+1,kp+1,i)-coor(ip,jp+1,kp,i))
        end do
        do i = 1, 3
         rq = mod( i+1,3 )
         s = mod( i+2,3 )
         if ( rq.eq.0 ) rq = 3
         if ( s.eq.0 ) s = 3
         jcta(i,1) = acov(rq,2)*acov(s,3)-acov(s,2)*acov(rq,3)
        end do
        do i = 1, 3
          nvec(i) = jcta(i,1)/sqrt(jcta(1,1)**2+jcta(2,1)**2+
     +                             jcta(3,1)**2)
        end do
      else if ( ndir.eq.2 ) then
        do i = 1, 3
          acov(i,3) = 5d-1*(coor(ip,jp,kp+1,i)-coor(ip,jp,kp,i)+
     +                      coor(ip+1,jp,kp+1,i)-coor(ip+1,jp,kp,i))
          acov(i,1) = 5d-1*(coor(ip+1,jp,kp,i)-coor(ip,jp,kp,i)+
     +                      coor(ip+1,jp,kp+1,i)-coor(ip,jp,kp+1,i))
        end do
        do i = 1, 3
         rq = mod( i+1,3 )
         s = mod( i+2,3 )
         if ( rq.eq.0 ) rq = 3
         if ( s.eq.0 ) s = 3
         jcta(i,2) = acov(rq,3)*acov(s,1)-acov(s,3)*acov(rq,1)
        end do
        do i = 1, 3
          nvec(i) = jcta(i,2)/sqrt(jcta(1,2)**2+jcta(2,2)**2+
     +                             jcta(3,2)**2)
        end do
      else if ( ndir.eq.3 ) then
        do i = 1, 3
          acov(i,1) = 5d-1*(coor(ip+1,jp,kp,i)-coor(ip,jp,kp,i)+
     +                      coor(ip+1,jp+1,kp,i)-coor(ip,jp+1,kp,i))
          acov(i,2) = 5d-1*(coor(ip,jp+1,kp,i)-coor(ip,jp,kp,i)+
     +                      coor(ip+1,jp+1,kp,i)-coor(ip+1,jp,kp,i))
        end do
        do i = 1, 3
         rq = mod( i+1,3 )
         s = mod( i+2,3 )
         if ( rq.eq.0 ) rq = 3
         if ( s.eq.0 ) s = 3
         jcta(i,3) = acov(rq,1)*acov(s,2)-acov(s,1)*acov(rq,2)
        end do
        do i = 1, 3
          nvec(i) = jcta(i,3)/sqrt(jcta(1,3)**2+jcta(2,3)**2+
     +                             jcta(3,3)**2)
        end do
      end if

c     --- Compute the normal distance to the wall

      ndist = abs((xp(1)-xq(1))*nvec(1) + (xp(2)-xq(2))*nvec(2) +
     +            (xp(3)-xq(3))*nvec(3))

      end
