      subroutine isxeu2d ( iinput, rhside, solut, coefs, work,
     +                     rinput, npointsv,
     +                     vsum, invjac, coor, mct, bndcon,
     +                     ibipoint, ibinfor, jac, eddy, iblock ,
     +                     itrans, itimlv, nblocks, isiinput, issolut,
     +                     isitopol, gmat, infmg )
c ======================================================================
c
c        programmer     Duncan van der Heul
c        version 1.3    date 27-08-1999 Duncan: move invjac to distribution
c                                       routines
c        version 1.2    date 33-03-1999 Duncan: new call to ismomccv2
c                                                           isbmomcc2
c        version 1.1    date 32-03-1999 Guus: correction conservative upwind
c                                             for multiblock
c        version 1.0    date 29-10-1998 subroutine based on ideas of isexm2d
c
c   copyright (c) 1995-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Solve Momentum equation explicitly in 2d with either Heun's method
c    or explicit Euler
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinput(*), npointsv, infmg(*),
     +        mct(2,2,-2:2,-2:2,-2:2), ibipoint(*), ibinfor(*),
     +        iblock , itrans, itimlv, nblocks, isiinput, issolut,
     +        isitopol
      double precision rhside(*), work(*), coor(*),
     +                 solut(npointsv,*), coefs(npointsv,*), rinput(*),
     +                 vsum(npointsv,*),  invjac(npointsv,*),
     +                 eddy(*), gmat(*), bndcon(2,*), jac(*)

c     bndcon    i    reserved
c     coefs     i    reserved
c     coor      i    reserved
c     eddy      i    reserved
c     gmat      i    Gradient matrix
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     iblock    i    Block sequence number
c     iinput    i    reserved
c     infmg     i    Integer information for Pressure-gradient matrix
c     invjac    i    1/sqrt(g) in pnttyp
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isitopol  i    Sequence number of array itopol
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     itimlv    i    Loop variable with respect to a given time level
c     itrans    i    reserved
c     jac       i    sqrt(g) in pnttyp
c     mct       i    Integer matrix information
c     nblocks   i    Number of blocks
c     npointsv  i    Number of points in the mesh with virtual points
c     rhside    o    reserved
c     rinput    i    reserved
c     solut     i    reserved
c     vsum           Flux contributions to the matrix
c     work           Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c      include 'IScommon/cisnas'
c      include 'IScommon/cisparal'
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iprho, alphad, iprintim, pnttyp, acttyp,
     +        vartyp, ni, nj, nvirtual, ipmu, mequat,
     +        iarea(2,2), locdx(2,5), dcell(2,5),
     +        ipcva, ipjcta, ipjctg,
     +        ipchsym , ipveloc, iptemp, ipgeo,
     +        i, idir, ipiindsc, mcom, ipiincom,
     +        ndegfd, ntimlv, nintpol(2,5), dij(2,4,2,5),
     +        ipadir, idiscret, ipgradu, ipprems, pnttyps(5),
     +        ichoisdis, iconserv, iconv, iprindsc, iupwind,
     +        ipiintim, iprhside, heunstep
      double precision dt
      logical reuse

c     acttyp    Type of action to be taken
c               0      : interpolation for point value
c               1..ndim: finite difference for partial derivative
c                        in the direction given by value of acttyp
c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     dcell     reserved
c     dij       Array containing the distances of the interpolation
c               points to the cell centra in global co-ordinates
c               dij(alpha,i,j) denotes the distance of the i-th
c               interpolation point, component j and with respect to
c               the V-alpha unknown (distribution)
c     dt        Time step
c     heunstep  Stepnumber in Heun's method
c     i         General loop variable
c     iarea     Area of arrays
c     ichoisdis Choice parameter with respect to the convection scheme
c               for discretization of convective term of transport
c               equation(s) including turbulence.
c               Meaning of ichoisdis:
c                 ichoisdis = 0: 2nd order central scheme
c                 ichoisdis = 1: 1st order standard upwind scheme
c                 ichoisdis = 2: Hybrid central/upwind scheme
c                 ichoisdis = 3: Linear kappa-scheme
c                 ichoisdis = 4: Sweby Phi-limiter
c                 ichoisdis = 5: R-kappa limiter
c                 ichoisdis = 6: USR-kappa limiter
c                 ichoisdis = 7: PL-kappa limiter
c                 ichoisdis = 8: USPL-kappa limiter
c                 ichoisdis = 9: Symmetric PL-kappa limiter
c                 ichoisdis = 10: MPL2-kappa limiter
c                 ichoisdis = 11: Symmetric PL-kappa limiter
c     iconserv  If 1 the conservative scheme is used, otherwise (0) the
c               non-conservative one
c     iconv     If 1 the timestep is iterated till converged, otherwise (0)
c               apply only one iterative step
c     idir      General loop variable for 1...ndim loop
c     idiscret  Type of discretization
c     ipadir    Starting address of array ADIR in array WORK
c     ipchsym   Starting address of array CHSYM in array WORK
c     ipcva     Starting address of array CVA in array WORK
c     ipgeo     Starting address of array GEO in array WORK
c     ipgradu   Starting address of array GRADU in array WORK
c     ipiincom  Starting address of array IINCOM in array IINPUT
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiintim  Starting address of array IINTIM in array IINPUT
c     ipjcta    Starting address of array JCTA in array WORK
c     ipjctg    Starting address of array JCTG in array WORK
c     ipmu      Starting address of array MU in array WORK
c     ipprems   Starting address of array PREMS in array WORK
c     iprho     Starting address of array rho either in BUFFER or WORK
c     iprhside  Starting address of array rhside in buffer
c     iprindsc  Starting address of array RINDSC in array RINPUT
c     iprintim  Starting address of array RINTIM in array RINPUT
c     iptemp    Starting address of temporary array in IBUFFR
c     ipveloc   Starting address of array VELOC in array WORK
c     iupwind   Type of upwinding
c     locdx     Position of the integration point with respect to
c               the centre of the integration cell
c               The cell sizes are assumed to be 2 times 2, hence the
c               distance of a midside point to a cell centre is always 1
c     mcom      If zero the flow is incompressible else (1) compressible
c     mequat    Indicates the type of momentum equation to be solved:
c               0:  standard equation
c               1:  multi-phase gas flow
c     ndegfd    Number of degrees of freedom
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nintpol   Number of interpolation points in the distribution
c     nj        Number of non-virtual cells in the j-direction (one block)
c     ntimlv    Number of time levels
c     nvirtual  Number of rows of virtual cells
c     pnttyp    reserved
c     pnttyps   Contains the value of pnttype for all 5 integration points
c     reuse     If true the computed coefficients may be reused again
c     vartyp    Variable to be interpolated/extrapolated
c               vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c               0 for vertex (for ndim=2)
c               or
c               vartyp: 1 for V1, 2 for V2 3, for V3 and
c               4 for pressure, 0 for vertex  (for ndim=3)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     DCOPY     Copy double precision array into other one (BLAS subroutine)
c     DSCAL     BLAS routine to scale vector
c     ISBGEO2D  Fill geometrical quantities along boundary in R2
c     ISBMOMCC2 Fill normal half cells for convection in R2
c     ISBMOMP2  Fill normal half cells for pressure bc for Euler eqs. in R2
c     ISBUPWVEL Update the multiplication vector which indicates from which
c               side upwind must be taken for half cells where we have an
c               outflow boundary condition
c     ISCHRYS2  Fill chrystoffel symbols in one integration point in R2
c     ISCOVA2D  Fill covariant base vectors in all integration points in R2
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISEXBO21  Fills virtual points at outer boundaries for theta is 0
c     ISEXBOU2  Fills the boundary conditions for theta = 0
c     ISEXDIC2
c     ISEXDICC2
c     ISEXDICU2
c     ISEXTIM2  Calculates the solution at the new time level
c               for theta=0
c     ISGMAT2D  Compute gradient matrix in R^2
c     ISINTCF2  Compute interpolation of a coefficient in an integration point
c     ISJCTA2D  Fill Jacobian times contravariant base vectors
c               in one integration point in R2
c     ISJCTG2D  Fill Jacobian times contravariant metric tensors
c               in one integration point in R2
c     ISMAKMPC2 Make a list of integration points for the momentum equations
c               in R2
c     ISMOMCCV2 Fill convective terms for one integration point in R2
c     ISMTVC23  Gradient matrix times pressure in 2D
c     ISSETCON  Make array equal to a constant
c     ISUPWVEL2 Compute sign vectors corresponding to velocity for upwind
c     ISURHSC2D Update right-hand side vector with higher order upwind
c               scheme (2D)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c    Compute constants
c    Clear array matrix
c    Clear array rhside
c    Make list of integration points
c    Interpolate coefficients to integration points
c    For idim := 1 (1) 2 do
c       Build time-derivative part of the matrix and rhside
c       Make list of integration points
c       For all integration points do
c          Fill coefficients for convective terms
c          Fill multiplication factors for convective terms
c          distribute convective terms
c          Fill coefficients for stress terms
c          Fill multiplication factors for stress terms
c          Correct boundary cells for stress terms
c          distribute stress terms
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('isexm2d')

c     --- Compute constants

      ipiindsc = iinput(6)
      iprindsc = iinput(33)
      iprintim = iinput(29)
      ipiintim = iinput(4)
      ni       = iinput(51)
      nj       = iinput(52)
      nvirtual = iinput(54)
      ndegfd   = iinput(55)
      ipiincom = iinput(9)
      mcom     = iinput(ipiincom)
      mequat   = iinput(ipiincom+2)
      dt       = rinput(iprintim -1 +4)
      npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
      ipcva    = 1
      ipjcta   = ipcva+16*npointsv
      ipjctg   = ipjcta+4*npointsv
      ipchsym  = ipjctg+3*npointsv
      iprho    = ipchsym+6*npointsv
      ipmu     = iprho+npointsv
      ipveloc  = ipmu+npointsv

      ipadir   = ipjctg
      ipgradu  = ipveloc+2*npointsv
      ipprems  = ipgradu+8*npointsv
      ipgeo    = ipprems+16*npointsv
      idiscret = iinput(ipiindsc+1)

      iconserv = iinput(ipiindsc+8)
      iconv    = iinput(ipiintim -1 + 17)
      iupwind  = iinput(ipiindsc+4)

      do heunstep = 1, 2

c     --- Clear array rhside

      call issetcon ( rhside, (ni+1)*(nj+1)*2, 0d0 )

c     --- Fill covariant basis vectors in cva for all integration points

      if( heunstep.eq.1) then

      call issetcon ( work(ipcva), 16*npointsv, 0d0 )
      call iscova2d ( coor, work(ipcva), ni, nj, nvirtual )

c     --- Fill array bcgeo and put into array work

      call isbgeo2d ( work(ipgeo), ni, nj, nvirtual, work(ipcva),
     +                ibipoint, invjac, jac )

      end if

      do alphad = 1, 2

         iarea(1,1) = 1
         iarea(1,2) = 1
         iarea(2,1) = ni
         iarea(2,2) = nj
         if ( alphad.eq.1 ) then
            iarea(2,1) = ni+1
         else
            iarea(2,2) = nj+1
         end if
         vartyp = 3
         iprhside = 1+(alphad-1)*(ni+1)*(nj+1)

c        --- Compute local co-ordinates, distances to cell center in
c            global co-ordinates, point type, number of interpolation
c            points and interpolation formula

         call ismakmpc2 ( alphad, locdx, pnttyps, dcell,
     +                    nintpol, dij )
         pnttyp = pnttyps(1)

         if ( mcom.eq.0 ) then

c        --- incompressible case, interpolate rho from coefs

            acttyp = 0
            call isintcf2 ( coefs(1,1), work(iprho), ni, nj, nvirtual,
     +                      vartyp, acttyp, pnttyp, 0, coor )

         else if( mcom.eq.1) then

            call issetcon ( work(iprho), npointsv, 1d0 )

         end if

c        --- Compute inverse of Jacobian, contravariant basis vectors
c            and christoffel symbols

         call isjcta2d ( work(ipcva+4*npointsv*(pnttyp-1)),
     +                   work(ipjcta), ni, nj, nvirtual )

c        --- The metric tensor times sqrt(g)

         call isjctg2d ( invjac(1,pnttyp), work(ipjcta),
     +                   work(ipjctg), ni, nj, nvirtual )

c        --- The christoffel symbols of the second kind

         call ischrys2 ( work(ipcva), work(ipjcta),
     +                   work(ipchsym), invjac(1,pnttyp), ni,
     +                   nj, nvirtual, pnttyp )

c        --- Next the convective terms

         if ( iinput(ipiindsc).eq.1 ) then

c        --- iindsc(1)=1, use convection term
c            Loop over integration points

            do i = 0, 2

c           --- First clear array vsum

               reuse = i.eq.0 .or. iconserv.eq.1 .and. i.eq.1 .or.
     +                 iupwind.gt.0 .and. iconserv.eq.1

               call issetcon ( vsum, 2*npointsv, 0d0 )

               if ( .not. reuse ) pnttyp = pnttyps(2*i+1)

c              --- Fill some coefficients

               if ( .not. reuse ) then

c              --- Fill rho except for first integration point
c                  In the case of a central conservative scheme we use also
c                  Centroid points for the second integration point

                  vartyp = 3
                  acttyp = 0
                  call isintcf2 ( coefs(1,1), work(iprho), ni, nj,
     +                            nvirtual, vartyp, acttyp, pnttyp, 0,
     +                            coor )
               end if

c              --- Fill velocity componentwise

               if ( iconserv.eq.0 .or. .not. reuse .or. i.eq.0 ) then

c              --- The velocity is required in all integration points,
c                  but in a central conservative scheme we use also
c                  Centroid points for the second integration point

c                  use second timelevel of velocity for iterative pressure
c                  correction

                  acttyp = 0

                    if ( iconv.eq.0) then

                      do idir = 1, 2
                        vartyp = idir
                        iptemp = ipveloc+(idir-1)*npointsv

                        call isintcf2 ( solut(1,idir),
     +                                  work(iptemp), ni,
     +                                  nj, nvirtual, vartyp, acttyp,
     +                                  pnttyp, 0, coor )
                      end do

                    else if( iconv.eq.1) then

                      do idir = 1, 2
                        vartyp = idir
                        iptemp = ipveloc+(idir-1)*npointsv
                        call isintcf2 ( solut(1,idir+ndegfd),
     +                                  work(iptemp), ni,
     +                                  nj, nvirtual, vartyp, acttyp,
     +                                  pnttyp, 0, coor )

                      end do

                    end if
               end if

c              --- Next fill vsum with contribution of convective terms to
c                  integration points

               call ismomccv2 ( ni, nj, nvirtual, work(iprho),
     +                          work(ipchsym),
     +                          work(ipveloc), vsum, alphad, i,
     +                          iinput(ipiindsc), mcom )

c              --- Correct contribution of convective terms for half cells

               call isbmomcc2 ( ni, nj, nvirtual, work(ipveloc), vsum,
     +                          alphad, work(iprho),
     +                          i, ibinfor, ibipoint,
     +                          iinput(ipiindsc), mcom  )

               if ( iconserv.eq.0 ) then

c              --- Non-conservative scheme

c                  --- Distribute computed values over the positions
c                     corresponding to the unknowns

                  call isexdic2( ni, nj, nvirtual, rhside, vsum,
     +                             alphad, iarea, iinput(ipiindsc),
     +                             nintpol, dij, mct, i, ndegfd,
     +                             eddy(npointsv+1), invjac(1,pnttyp))

               else

c              --- Conservative scheme

c                  --- Distribute computed values over the positions
c                     corresponding to the unknowns

                  if ( iupwind.eq.0 ) then

c                 --- central scheme

                     call isexdicc2( ni, nj, nvirtual, rhside, vsum,
     +                             alphad, iarea, iinput(ipiindsc),
     +                             nintpol, dij, mct, i, ndegfd,
     +                             eddy(npointsv+1), invjac(1,pnttyp))
                  else

c                 --- upwind scheme
c                     First store terms like (1-sign(v^alpha)) in array
c                     work from position ipprems

                     if ( i.ne.0 ) then

                     call isupwvel2 ( ni, nj, nvirtual, solut,
     +                                work(ipprems), alphad, i )

c                    --- Correct for half cells

                        call isbupwvel ( ni, nj, nvirtual, alphad, i,
     +                                   ibinfor, ibipoint,
     +                                   work(ipprems) )
                     end if
                     call isexdicu2 ( ni, nj, nvirtual, rhside, vsum,
     +                                alphad, iarea, iinput(ipiindsc),
     +                                nintpol, dij, mct, i, ndegfd,
     +                                work(ipprems), eddy(npointsv+1),
     +                                invjac(1,pnttyp))

                  end if

               end if

               ichoisdis = iinput(ipiindsc-1+5)

               if ( ichoisdis.gt.2 .and. i.gt.0 ) then

c              --- Higher order upwind scheme

                  if (mcom.eq.0) then

                     call isurhsc2d ( alphad, ni, nj, nvirtual,
     +                                rhside(iprhside), vsum,
     +                                work(ipveloc), solut(1,alphad),
     +                                iarea, i,
     +                                ichoisdis, rinput(iprindsc-1+1),
     +                                rinput(iprindsc-1+2),
     +                                rinput(iprindsc-1+3),
     +                                rinput(iprindsc-1+4),
     +                                invjac(1,pnttyp))
                  else

                     call isurhsc2d ( alphad, ni, nj, nvirtual,
     +                                rhside(iprhside), vsum,
     +                                work(ipveloc),
     +                                eddy((alphad)*npointsv+1),
     +                                iarea, i,
     +                                ichoisdis, rinput(iprindsc-1+1),
     +                                rinput(iprindsc-1+2),
     +                                rinput(iprindsc-1+3),
     +                                rinput(iprindsc-1+4),
     +                                invjac(1,pnttyp) )

                  end if

               end if

            end do

         end if

c        --- For Euler boundary condition for pressure
c            Compute contribution rhside of pressure boundary
c            condition for Euler eqs., sigmann given.

         call isbmomp2 ( ni, nj, nvirtual, alphad,
     +                work(ipjctg), work(ipchsym),
     +                ibinfor, ibipoint, bndcon,
     +                rhside )

c        --- Compute the g-matrix

         if( heunstep.eq.1) then

         call isgmat2d ( alphad, ni, nj, nvirtual, gmat, work(ipcva),
     +                   invjac, ibipoint, ibinfor, idiscret, jac,
     +                   work(ipadir) )
         end if

      end do

c     --- Update the right-hand side vector with -Gp^n
c         In two-phase flow: -eps Gp^n**

      call ismtvc23 ( gmat, solut(1,3), rhside, ni, nj, nvirtual,
     +                infmg(infmg(12)), coefs(1,6), mequat )

      if ( mcom.eq.0 ) then

c     --- incompressible case, use rho from coefs

         call isextim2 ( solut, rhside, coefs(1,1), dt, ni, nj,
     +                   nvirtual, ndegfd, mcom, heunstep)

      else

c     --- compressible case, use rho from eddy

         call isextim2 ( solut, rhside, eddy, dt, ni, nj,
     +                   nvirtual, ndegfd, mcom, heunstep)

      end if

      if( heunstep.eq.2) then
         call dscal(npointsv*2, 0.5d0, solut(1,ndegfd+1), 1)
      end if

      ntimlv = 2

      call isexbou2 ( ni, nj, nvirtual, ndegfd, ntimlv, solut,
     +                ibinfor, ibipoint, work(ipgeo), bndcon )

      call isexbo21 ( iblock, itrans, itimlv, nblocks,
     +                isiinput, issolut, isitopol,
     +                ibinfor, ibipoint, work(ipgeo), bndcon )

      if( heunstep.eq.1) then
         call dcopy(npointsv*2, solut(1,ndegfd+1),
     +                              1, eddy(npointsv+1),1)
      end if
      end do

      call iserclos('isexm2d')
      end
