      subroutine isaxpupw ( ni, nj, nvirtual, alphad, kdir, vecy,
     +                      vecup2, intn, k, facs, vecx, dcell, iarea )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.0    date 16-08-1998 copied from isaxpy2d of
c                                       Guus Segal
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Perform a vector update for ISNaS over a 2-dimensional region
c    First order upwind scheme.
c    The update looks as follows:
c    y(i,j) = y(i,j) + a * x(i,j)
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, alphad, kdir,
     +         iarea(1:2,1:2), k, intn, dcell(2)
      double precision vecy(ni+1,nj+1), facs,
     +                 vecx(1-nvirtual:ni+nvirtual+1,
     +                      1-nvirtual:nj+nvirtual+1),
     +                 vecup2(1-nvirtual:ni+nvirtual+1,
     +                        1-nvirtual:nj+nvirtual+1,2)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     dcell     i    reserved
c     facs      i    Multiplication factor
c     iarea     i    Area of arrays
c     intn      i    Integration point number
c     k         i    General loop variable
c     kdir      i    General loop variable for 1...ndim loop
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     vecup2    i    Upwind help vector 2, (1+sign V)/2 and (1-sign V)/2 in
c                    integration point
c     vecx      i    Vector to be added
c     vecy      i    Vector to be updated
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j

c     i         General loop variable
c     j         General loop variable
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

c     --- first order or higher order upwind scheme

      if ( intn.ne.1 ) then

c     --- integration points 2, 3, 4, 5

         if ( kdir.eq.alphad  ) then

            do j = iarea(1,2), iarea(2,2)
               do i = iarea(1,1), iarea(2,1)
                  vecy(i,j) = vecy(i,j) + facs*
     +                        vecup2(i+dcell(1),j+dcell(2),k)*
     +                        vecx(i,j)
               end do
            end do

         else

            if ( k.lt.3 ) then

               do j = iarea(1,2), iarea(2,2)
                  do i = iarea(1,1), iarea(2,1)
                     vecy(i,j) = vecy(i,j) + facs*
     +                           vecup2(i+dcell(1),j+dcell(2),1)*
     +                           vecx(i,j)
                  end do
               end do

            else if ( k.gt.4 ) then

               do j = iarea(1,2), iarea(2,2)
                  do i = iarea(1,1), iarea(2,1)
                     vecy(i,j) = vecy(i,j) + facs*
     +                           vecup2(i+dcell(1),j+dcell(2),2)*
     +                           vecx(i,j)
                  end do
               end do

            else

               do j = iarea(1,2), iarea(2,2)
                  do i = iarea(1,1), iarea(2,1)
                     vecy(i,j) = vecy(i,j)+facs*vecx(i,j)
                  end do
               end do

            end if

         end if

      end if

      end
