      subroutine isbcc2 ( alphad, ni, nj, nvirtual, vsum, rhside,
     +                    locdx, dcellv1, dcellv2, ibinfor, ibipoint,
     +                    bndcon , bcgeo)
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.4    date 02-12-1998 Guus: iprec removed
c        version 1.3    date 12-10-1998 dcellv added
c        version 1.2    date 09-12-1997 change for periodic b.c.
c        version 1.1    date 15-10-1996 only for dirichlet-dirichlet
c        version 1.1    date 15-10-1996 only for dirichlet-dirichlet
c        version 1.0    date 03-09-1996
c
c   copyright (c) 1996-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Dirichlet-dirichlet boundary condition convection term for
c   tangential cells 2D, Wesbeek discretization.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, ibipoint(*), ibinfor(*), alphad,
     +        locdx(2), dcellv1, dcellv2
      double precision rhside(ni+1,nj+1,2),
     +                 bndcon(2,*), bcgeo(8,*),
     +                 vsum(1-nvirtual+dcellv1:ni+1+nvirtual+dcellv1,
     +                      1-nvirtual+dcellv2:nj+1+nvirtual+dcellv2,2)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     bcgeo     i    reserved
c     bndcon    i    reserved
c     dcellv1   i    Help variable replacing dcell
c     dcellv2   i    Help variable replacing dcell
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside   i/o   reserved
c     vsum      i    Flux contributions to the matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer unut, uxuy, parabolic
      parameter( unut = 4, uxuy = 10, parabolic= 12)
      integer ipibinf, ipoint, ip, jp, xp(2), iside, iparea(2,2),tdir,
     +        ndir, nsign, tsign, istwo(2,2), mip, mjp, btype
      double precision  gttdivsg, gntdivsg, gtt, gnn, gnt,
     +                  un, ut
      double precision u(1:2)

c     btype     Type of boundary condition
c     gnn       Contravariant metric tensor
c     gnt       Contravariant metric tensor in nt direction
c     gntdivsg  Contravariant metric tensor in nt direction divided by sqrt(g)
c     gtt       Contravariant metric tensor in tt direction
c     gttdivsg  Contravariant metric tensor in tt direction divided by sqrt(g)
c     ip        Local counting variable i-direction
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     ipibinf   Starting address of array infor for present side minus one
c     ipoint    Address in array IBUFFR
c     iside     Sequence number of side
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     jp        Local counting variable j-direction
c     mip       Integration cell indices
c     mjp       Integration cell indices
c     ndir      Direction of normal vector on this side
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     parabolic Parabolic boundary conditions
c     tdir      Direction of tangent vector on this side
c     tsign     Inner product of tangential vector defined counter
c               clockwise with respect to the outward normal and the
c               base vector along the side
c     u         Cartesian velocities
c     un        Normal velocity (physical)
c     unut      U_n and u_t prescribed
c     ut        Tangential velocity (physical)
c     uxuy      U_x and u_y prescribed
c     xp        Auxiliary array to calculate boundary point number
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c      loop over tangential cells
c      if boundary condition is dirichlet-dirichlet then
c         put known convection term into right-hand side
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data istwo / 1, 2,
     +             2, 3 /
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- loop over tangential cells at the boundary
c         Compute loop bounds

      iparea(1,1) = 1
      iparea(2,1) = -1
      iparea(1,2) = 1
      iparea(2,2) = -1

      if ( alphad.eq.1 ) then

c     --- iside 3 and 4

         do iside = 3, 4

            tdir = 1
            ndir = 2
            if ( iside.eq.3) then
               nsign = -1
               tsign = 1
               iparea(1,1) = 2
               iparea(2,1) = ni
               iparea(1,2) = 1
               iparea(2,2) = 1
               mjp = 1
            else
               nsign = 1
               tsign = -1
               iparea(1,1) = 2
               iparea(2,1) = ni
               iparea(1,2) = nj
               iparea(2,2) = nj
               mjp = nj+1
            end if

            if ( (iside.eq.3 .and. locdx(2).eq.-1) .or.
     +           (iside.eq.4 .and. locdx(2).eq. 1) ) then

               ipibinf = ibipoint(iside)+nvirtual-1

               do  jp = iparea(1,2), iparea(2,2)
                  do  ip = iparea(1,1), iparea(2,1)

                     xp(1) = ip
                     xp(2) = jp
                     ipoint = ipibinf+2*xp(tdir) - 1

                     btype = ibinfor(ipoint)

                     if ( btype.eq.uxuy .or. btype.eq.unut .or.
     +                    btype.eq.parabolic .or. btype.eq.88 ) then

                        if ( btype.ne.uxuy ) then

c                       --- un and ut given

                           un = bndcon(1,ipoint)*nsign
                           ut = bndcon(2,ipoint)*tsign
                           gttdivsg = bcgeo(istwo(ndir,ndir),ipoint)
                           gntdivsg =-bcgeo(istwo(ndir,tdir),ipoint)
                           gnn = gttdivsg/bcgeo(8,ipoint)
                           gtt = bcgeo(8,ipoint)*gttdivsg
                           gnt = bcgeo(8,ipoint)*gntdivsg
                           un = sqrt(gnn)*un
                           ut = (sqrt(gtt)*ut-gnt*un)/gtt
                           u(ndir) = bcgeo(8,ipoint)*un
                           u(tdir) = bcgeo(8,ipoint)*ut

                        else

c                       --- ux and uy given

                           u(ndir) = bcgeo(2+2*ndir,ipoint)*
     +                               bndcon(1,ipoint)+
     +                               bcgeo(3+2*ndir,ipoint)*
     +                               bndcon(2,ipoint)

                           u(tdir) = bcgeo(2+2*tdir,ipoint)*
     +                               bndcon(1,ipoint)+
     +                               bcgeo(3+2*tdir,ipoint)*
     +                               bndcon(2,ipoint)
                        end if

                       rhside(ip,jp,alphad ) = rhside(ip,jp,alphad ) -
     +                 (  vsum(ip,mjp,1) * u(1) +
     +                    vsum(ip,mjp,2) * u(2) )

                          vsum(ip,mjp,1) = 0d0
                          vsum(ip,mjp,2) = 0d0

                     end if

                     btype = ibinfor(ipoint)

                  end do
               end do
           end if

         end do

      else

c     --- alphad =  2

         do iside = 1, 2

            tdir = 2
            ndir = 1
            if ( iside.eq.1) then
               nsign = -1
               tsign = -1
               iparea(1,1) = 1
               iparea(2,1) = 1
               iparea(1,2) = 2
               iparea(2,2) = nj
               mip = 1
            else
               nsign = 1
               tsign = 1
               iparea(1,1) = ni
               iparea(2,1) = ni
               iparea(1,2) = 2
               iparea(2,2) = nj
               mip =ni+1
            end if

            if ( (iside.eq.1 .and. locdx(1).eq.-1) .or.
     +           (iside.eq.2 .and. locdx(1).eq. 1) ) then

               ipibinf = ibipoint(iside)+nvirtual-1

               do  jp = iparea(1,2), iparea(2,2)
                  do  ip = iparea(1,1), iparea(2,1)

                     xp(1) = ip
                     xp(2) = jp
                     ipoint = ipibinf+2*xp(tdir) - 1

                     btype = ibinfor(ipoint)

                     if ( btype.eq.uxuy .or. btype.eq.unut .or.
     +                    btype.eq.parabolic .or. btype.eq.88 ) then

                        if ( btype.ne.uxuy ) then

c                       --- un and ut given

                           un = bndcon(1,ipoint)*nsign
                           ut = bndcon(2,ipoint)*tsign
                           gttdivsg = bcgeo(istwo(ndir,ndir),ipoint)
                           gntdivsg =-bcgeo(istwo(ndir,tdir),ipoint)
                           gnn = gttdivsg/bcgeo(8,ipoint)
                           gtt = bcgeo(8,ipoint)*gttdivsg
                           gnt = bcgeo(8,ipoint)*gntdivsg
                           un = sqrt(gnn)*un
                           ut = (sqrt(gtt)*ut-gnt*un)/gtt
                           u(ndir) = bcgeo(8,ipoint)*un
                           u(tdir) = bcgeo(8,ipoint)*ut

                        else

c                       --- ux and uy given

                           u(ndir) = bcgeo(2+2*ndir,ipoint)*
     +                               bndcon(1,ipoint)+
     +                               bcgeo(3+2*ndir,ipoint)*
     +                               bndcon(2,ipoint)

                           u(tdir) = bcgeo(2+2*tdir,ipoint)*
     +                               bndcon(1,ipoint)+
     +                               bcgeo(3+2*tdir,ipoint)*
     +                               bndcon(2,ipoint)
                        end if

                        rhside(ip,jp,alphad ) = rhside(ip,jp,alphad )
     +                  - (  vsum(mip,jp,1) * u(1) +
     +                       vsum(mip,jp,2) * u(2) )

                        vsum(mip,jp,1) = 0d0
                        vsum(mip,jp,2) = 0d0

                     end if

                     btype = ibinfor(ipoint)

                  end do
              end do

           end if

         end do

      end if

      end
