      subroutine isbmomcc2 ( ni, nj, nvirtual, veloc, vsum, alphad,
     +                       densrho, intp, ibinfor, ibipoint, iindsc,
     +                       mcom )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.2    date 27-08-1999  Duncan: put invjac in distribution
c                                                routines
c        version 1.1    date 23-03-1999  Duncan: extended for compressible
c                                               flow with Newton linearisation
c        version 1.0    date 17-02-1998
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c    determines the coefficients of the velocity components in a given
c    integration point within a boundary flux cell for the convection term in R2
c **********************************************************************
c
c                       KEYWORDS
c
c    boundary_conditions
c    convection
c    matrix
c    right_hand_side
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, iindsc(*), intp,
     +         alphad, ibipoint(*), ibinfor(*), mcom
      double precision densrho(1-nvirtual:ni+1+nvirtual,
     +                          1-nvirtual:nj+1+nvirtual),
     +                  vsum(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2),
     +                  veloc(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,2)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     densrho   i    Array containing the density rho. The size and structure is
c                    the same as for the scalar unknowns
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     iindsc    i    reserved
c     intp      i    Interpolated values of variable
c     mcom      i    If zero the flow is incompressible else (1) compressible
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c     vsum      o    Flux contributions to the matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer snnsnt, snnut
      parameter( snnsnt = 7, snnut = 6 )
      integer xp(2), ip, jp, btype, iconserv, iupwind,
     +        iside, tdir, ndir, nsign,
     +        mlin, ipibinf, ilow, jlow, iupp, jupp
      double precision rmult1

c     btype     Type of boundary condition
c     iconserv  If 1 the conservative scheme is used, otherwise (0) the
c               non-conservative one
c     ilow      Lower index of subface
c     ip        Local counting variable i-direction
c     ipibinf   Starting address of array infor for present side minus one
c     iside     Sequence number of side
c     iupp      Upper i-index of subface
c     iupwind   Type of upwinding
c     jlow      Lower j-index for a subface
c     jp        Local counting variable j-direction
c     jupp      Upper j-index of subface
c     mlin      Type of linearization (1=newton, 2=picard)
c     ndir      Direction of normal vector on this side
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     rmult1    Multiplication factor
c     snnsnt    Sigma_nn and sigma_nt prescribed
c     snnut     Sigma_nn and u_t prescribed
c     tdir      Direction of tangent vector on this side
c     xp        Auxiliary array to calculate boundary point number
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c       Relevant formula for an internal cell
c
c
c    The convective terms in case of the classical discretization are given by:
c
c
c    rho/ sqrt(g) V^a V^1 |^(i+1/2,j)_(i-1/2,j)  +
c    rho/ sqrt(g) V^a V^2 |^(i,j+1/2)_(i-1/2,j)  +
c    rho/ sqrt(g) CHRYS(a,b,c) V^b V^c |(i,j)
c
c   Application of Newton linearization:
c
c  V^a V^b =  W^a V^b   + V^a W^b   -  W^a W^b, with W = V^n
c
c   results in the following contribution of the V^1 component to the matrix:
c
c    rho/ sqrt(g) V^1 |^(i+1/2,j)_(i-1/2,j)  +
c    rho/ sqrt(g) V^2 |^(i,j+1/2)_(i-1/2,j)  +
c    rho/ sqrt(g) V^a |^(i+1/2,j)_(i-1/2,j)  +
c  2 rho/ sqrt(g) CHRYS(a,b,c) V^b |(i,j)
c
c  and of course a similar expression for the V^2 component
c
c  Application of the Picard linearization (instead of the Newton lin.) gives
c  for the b-th momentum equation:
c
c  V^a V^b =  W^a V^b  and  V^b V^b =  W^b V^b
c
c
c   Consider a normal half cell at the lower boundary, i.e.
c       ntdir=(2,1)
c       ntsig=(-1,1)
c
c
c         ------------------------------------------
c         |              |            |            |
c         |              |            |            |
c         |              |            |            |
c         |              |-----+------|            |
c         |              |            |            |
c         |              |            |            |
c         |              |            |            |
c         ---------------+-----*------+-------------
c
c   The cell centre is indicated by *, the integration points by +
c
c
c   This cell is only active if the normal stress is given in the boundary point
c
c   The following things must be updated:
c
c   Point (i,j): multiply volume by 1/2
c   Integration points at boundary: multiply contribution by 1/2
c   Point outside the region: set contribution equal to zero and add
c                             to point (0,0)
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      if( mcom.eq.0) then
         mlin     = iindsc(8)
      else if(mcom.eq.1) then
         mlin     = 2
      end if

      iconserv = iindsc(9)
      iupwind  = iindsc(5)

c     --- Determine multiplication factors due
c         to linearisation type

      if ( mlin.eq.1 ) then
         rmult1 = 2d0
      else
         rmult1 = 1d0
      end if

      if ( iconserv.eq.1 .and. iupwind.le.0 ) rmult1 = 0.5d0*rmult1

c     --- Loop over all sides

      do iside = 2*alphad-1, 2*alphad

         ilow = 1
         iupp = 1
         jlow = 1
         jupp = 1
         ipibinf = ibipoint(iside)-1+nvirtual
         if ( iside.le.2 ) then

c        --- iside = 1, 2:  left- and right-hand side

            jupp = nj
            tdir = 2
            ndir = 1
            if ( iside.eq.1 ) then

c           --- iside = 1:  left-hand side

               nsign = -1

            else

c           --- iside = 2:  right-hand side

               nsign = 1
               ilow = ni+1
               iupp = ni+1

            end if

         else

c        --- iside = 3, 4:  lower- and upper side

            iupp = ni
            tdir = 1
            ndir = 2
            if ( iside.eq.3 ) then

c           --- iside = 3:  lower side

               nsign = -1

            else

c           --- iside = 4:  upper side

               nsign = 1
               jlow = nj+1
               jupp = nj+1

            end if

         end if
         if ( intp.eq.0 ) then

c        --- Points (0,0) at boundary

            do jp = jlow, jupp
               do ip =  ilow, iupp
                  xp(1) = ip
                  xp(2) = jp
                  btype = ibinfor(ipibinf+2*xp(tdir))
                  if ( btype.eq.snnut .or. btype.eq.snnsnt ) then

c                 --- Half cell with given normal stress component
c                     First multiply present contribution by 0.5

                     vsum(ip,jp,1) = 0.5d0*vsum(ip,jp,1)
                     vsum(ip,jp,2) = 0.5d0*vsum(ip,jp,2)

c                    --- Next add the contribution of the point outside

                     vsum(ip,jp,ndir) = vsum(ip,jp,ndir) +
     +                                nsign*rmult1*densrho(ip,jp)*

c                    --- +
c                        --- invjac(ip,jp)*veloc(ip,jp,ndir)
     +                                veloc(ip,jp,ndir)

                  end if

               end do

            end do

         else if ( intp.eq.1 ) then

c        --- Points (-1,0) and (1,0) at boundaries 1 and 2
c            Points (0,-1) and (0,1) at boundaries 3 and 4
c            Point outside the region, set vsum = 0

            if ( iside.eq.1 ) then
               ilow = 0
               iupp = 0
            else if ( iside.eq.3 ) then
               jlow = 0
               jupp = 0
            end if
            if ( iconserv.eq.1 ) then

c           --- Conservative discretization, point outside region is
c               one further then usual

               if ( iside.eq.2 ) then
                  ilow = ni+2
                  iupp = ni+2
               else if ( iside.eq.4 ) then
                  jlow = nj+2
                  jupp = nj+2
               end if

            end if

            do jp = jlow, jupp
               do ip =  ilow, iupp
                  xp(1) = ip
                  xp(2) = jp
                  btype = ibinfor(ipibinf+2*xp(tdir))
                  if ( btype.eq.snnut .or. btype.eq.snnsnt ) then

c                 --- Half cell with given normal stress component
c                     Point outside the region, set vsum = 0

                     vsum(ip,jp,1) = 0d0
                     vsum(ip,jp,2) = 0d0

                  end if

               end do

            end do

         else if ( iconserv.eq.0 .or. iupwind.le.0 ) then

c        --- Points (0,-1) and (0,1) at boundaries 1 and 2
c            Points (-1,0) and (1,0) at boundaries 3 and 4
c            Point at boundary, multiply by 1/2

            if ( iside.le.2 ) then
               jupp = nj+1
            else
               iupp = ni+1
            end if
            do jp = jlow, jupp
               do ip =  ilow, iupp
                  xp(1) = ip
                  xp(2) = jp
                  btype = ibinfor(ipibinf+2*xp(tdir))
                  if ( btype.eq.snnut .or. btype.eq.snnsnt ) then

c                 --- Half cell with given normal stress component
c                     Point at boundary, multiply by 1/2

                     vsum(ip,jp,1) = 0.5d0*vsum(ip,jp,1)
                     vsum(ip,jp,2) = 0.5d0*vsum(ip,jp,2)

                  else

c                 --- Check if previous cell has given normal stress component

                     btype = ibinfor(ipibinf+2*xp(tdir)-2)
                     if ( btype.eq.snnut .or. btype.eq.snnsnt ) then

c                    --- Half cell with given normal stress component
c                        Point at boundary, multiply by 1/2

                        vsum(ip,jp,1) = 0.5d0*vsum(ip,jp,1)
                        vsum(ip,jp,2) = 0.5d0*vsum(ip,jp,2)
                     end if

                  end if

               end do

            end do

         end if

      end do

      end
