      subroutine isbtrad2 ( iside, ni, nj, nvirtual, dtsum, cva, matrix,
     +                      ibinfor, ibipoint, bndcon, rhside, jcta,
     +                      invjac, idiscret, mct )
c ======================================================================
c
c        programmer  Guus Segal
c        version 1.1    date 18-05-1998 Charles Bilinear discretization
c        version 1.0    date 06-11-1995
c
c   copyright (c) 1995-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c   Correct dtsum, matrix and right-hand side for Robbins type boundary
c   conditions for the transport equation in R2
c **********************************************************************
c
c                       KEYWORDS
c
c     2d
c     boundary_conditions
c     matrix
c     diffusion
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, ibinfor(*), ibipoint(*), iside,
     +        idiscret, mct(-2:2,-2:2,-2:2)
      double precision bndcon(2,*), matrix(ni+1,nj+1,*),
     +                 rhside(ni+1,nj+1),
     +                 dtsum(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2),
     +                 cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2),
     +                 jcta(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2,2),
     +                 invjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual)

c     bndcon    i    reserved
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     dtsum     i    Flux derivative contributions to the matrix
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     idiscret  i    Type of discretization
c     invjac    i    1/sqrt(g) in pnttyp
c     iside     i    Sequence number of side
c     jcta      i    Jacobian times contravariant basis vectors
c                    in point pnttyp
c                    (stored as vector index encoding given by istwo)
c     matrix   i/o   reserved
c     mct       i    Integer matrix information
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside   i/o   reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer robb
      parameter ( robb = 2 )
      integer iarea(2,2), ip, jp, mip, mjp, mxp(2), ishift,
     +        jshift, tdir, ndir, nsign, tsign, ofstnr, ipibinf, ict1
      double precision gnn, ljcta

c     gnn       Contravariant metric tensor
c     iarea     Area of arrays
c     ict1      Pointer in ibinfor with respect to boundary point
c     ip        Local counting variable i-direction
c     ipibinf   Starting address of array infor for present side minus one
c     ishift    Shift in i-direction
c     jp        Local counting variable j-direction
c     jshift    Shift in j-direction
c     ljcta     Length of a contravariant base vector times Jacobian
c     mip       Integration cell indices
c     mjp       Integration cell indices
c     mxp       Auxiliary array to calculate boundary point number
c     ndir      Direction of normal vector on this side
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     ofstnr    Offset diagonal number of matrix element that
c               corresponds to the current point of the interpolation
c               or extrapolation formula
c     robb      Robbins boundary condition, i.e. aT + (k grad T).n given
c     tdir      Direction of tangent vector on this side
c     tsign     Inner product of tangential vector defined counter
c               clockwise with respect to the outward normal and the
c               base vector along the side
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISORIENT2 Computes ndir, nsign, tdir, tsign for a 2-dimensional
c               problem
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   Consider a boundary cell at the lower boundary:
c
c
c
c                -------|-------
c                |             |
c                |             |
c                |             |
c                -      +      -
c                |             |
c                |             |
c                |             |
c           ------------|------------------
c
c
c   For the internal elements we have a contribution for all four
c   integration points.
c
c   For the classical discretization the contribution to the lower
c   boundary is:
c
c              ab        |
c   - sqrt(g) K     T,   |              (a=1)
c                     b  | (0,-1)
c
c   Because of the Robbins boundary condition this term is replaced by:
c
c       / brhs - sigma T,
c   boundary
c
c   where brhs is stored in bndcon(2,.) and sigma in bndcon(1,.)
c
c   The integral is equal to
c
c   g_tt (brhs - sigma T) = sqrt(g) sqrt(g^nn) (brhs - sigma T)
c
c   Hence the dtsum contribution for the boundary point should be made zero,
c   and the contribution to matrix and right-hand side are given above
c
c   The term T should be distributed over the points (0,0) and (0,-1),
c   with a factor 1/2, since (0,-1/2) does not contain a transport unknown
c
c   However, since (0,-1) is a virtual point, this point is folded
c   inwardly in the following way:
c
c   T(0,-1) = 2T(0,0) - T(0,1)
c
c   Hence the final distribution becomes:
c
c   T(0,-1/2) = 3/2 T(0,0) - 1/2T(0,1)
c
c   In the Wesbeek we follow the same approach
c **********************************************************************
c
c
c
c
c
c                      PSEUDO CODE
c
c   Compute constants depending on the side sequence number
c   For all mid side points at the boundary iside do
c     if ( boundary condition is Robbns condition) then
c        set dtsum equal to zero
c        update matrix and right-hand side
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      iarea(1,1) = 1
      iarea(2,1) = ni
      iarea(1,2) = 1
      iarea(2,2) = nj
      ishift = 0
      jshift = 0
      if ( iside.eq.1 ) then
         iarea(2,1) = 1
      else if ( iside.eq.2 ) then
         iarea(1,1) = ni
         ishift = 1
      else if ( iside.eq.3 ) then
         iarea(2,2) = 1
      else
         iarea(1,2) = nj
         jshift = 1
      end if
      ipibinf = ibipoint(iside)-1+nvirtual

c     --- Compute tdir and ndir

      call isorient2 ( iside, tdir, ndir, tsign, nsign )
      ofstnr = mct(nsign*ishift,nsign*jshift,0)

      if ( idiscret.eq.1 ) then

c     --- Standard ISNaS discretization
c         loop over boundary cells along side iside

         do mjp = iarea(1,2), iarea(2,2)
            do mip = iarea(1,1), iarea(2,1)

c           --- Get integration cell index

               mxp(1) = mip
               mxp(2) = mjp
               ict1   = ipibinf+2*mxp(tdir)

c              --- Get integration cell position for ibinfor

               if ( ibinfor(ict1).eq.robb ) then

c              --- Robbins boundary conditions found
c                  First remove contribution to dtsum

                  ip = mip+ishift
                  jp = mjp+jshift
                  dtsum(ip,jp,1) = 0d0
                  dtsum(ip,jp,2) = 0d0
                  gnn = invjac(ip,jp)**2 * ( cva(ip,jp,1,tdir)**2 +
     +                                       cva(ip,jp,2,tdir)**2 )

                  rhside(mip,mjp) = rhside(mip,mjp) + sqrt(gnn) *
     +                              bndcon(2,ict1) / invjac(ip,jp)

                  matrix(mip,mjp,1) = matrix(mip,mjp,1) + 1.5d0 *
     +                                sqrt(gnn) * bndcon(1,ict1) /
     +                                invjac(ip,jp)
                  matrix(mip,mjp,ofstnr) = matrix(mip,mjp,ofstnr) -
     +                                0.5d0 * sqrt(gnn) *
     +                                bndcon(1,ict1) / invjac(ip,jp)
               end if

            end do
         end do

      else if (idiscret.eq.2.or.idiscret.eq.4) then

c     --- WesBeek discretization
c         loop over boundary cells along side iside

         do mjp = iarea(1,2), iarea(2,2)
            do mip = iarea(1,1), iarea(2,1)

c           --- Get integration cell index

               mxp(1) = mip
               mxp(2) = mjp
               ict1   = ipibinf+2*mxp(tdir)

c              --- Get integration cell position for ibinfor

               if ( ibinfor(ict1).eq.robb ) then

c              --- Robbins boundary conditions found
c                  First remove contribution to dtsum

                  ip = mip+ishift
                  jp = mjp+jshift
                  dtsum(ip,jp,1) = 0d0
                  dtsum(ip,jp,2) = 0d0
                  ljcta = sqrt(jcta(ip,jp,1,ndir)**2 +
     +                         jcta(ip,jp,2,ndir)**2) * nsign

                  rhside(mip,mjp) = rhside(mip,mjp) + ljcta *
     +                              bndcon(2,ict1)

                  matrix(mip,mjp,1) = matrix(mip,mjp,1) + 1.5d0 *
     +                                ljcta * bndcon(1,ict1)
                  matrix(mip,mjp,ofstnr) = matrix(mip,mjp,ofstnr) -
     +                                0.5d0 * ljcta * bndcon(1,ict1)
               end if

            end do
         end do

      end if

      end
