      subroutine iscmcv2d ( prescorr, ustar, ni, nj, nvirtual, coefs,
     +                      dt, coor, cva, cvc, ctc, jac, rhside,
     +                      gmat, gmctinv, invjac, ibipoint, ibinfor,
     +                      idiscret, adir, geom, igeom )
c ======================================================================
c
c        programmer     Charles Moulinec
c        version 1.3    date   28-05-1997 Charles: iii removed
c        version 1.2    date   15-12-1997 Charles: new parameter iii
c        version 1.1    date   29-10-1997 Charles: corrections
c        version 1.0    date   13-10-1997 Charles: collocated allowed
c
c
c   copyright (c) 1997-1997  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     This subroutine performs the of the matrix vector multiplication
c     f := f - G p in 2D
c     where p is the pressure including virtual points,
c           g is the gradient matrix (no virtual points) and
c           f is the right-hand side (no virtual points) to be updated
c **********************************************************************
c
c                       KEYWORDS
c
c      multiplication
c      matrix
c      vector
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, gmctinv(0:3,2,6), ibipoint(*),
     +        ibinfor(*), idiscret
      integer igeom(ni+1,nj+1,2,2)
      double precision gmat(ni+1,nj+1,2,2),
     +                 ustar(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2),
     +                 prescorr(1-nvirtual:ni+1+nvirtual,
     +                          1-nvirtual:nj+1+nvirtual),
     +                 coefs(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,*),
     +                 coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2),
     +                 cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2,4),
     +                 cvc(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2),
     +                 ctc(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2),
     +                 jac(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,4),
     +                 rhside(ni+1,nj+1),
     +                 adir(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2),
     +                 invjac((ni+1+2*nvirtual)*(nj+1+2*nvirtual),4)
      double precision dt
      double precision geom(ni+1,nj+1,2,6)

c     adir      i    Contravariant base vector in integration cell center
c     coefs    i/o   reserved
c     coor      i    reserved
c     ctc       i    Work array containing the contravariant WESBEEK vectors
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     cvc       i    Work array containing the covariant WESBEEK vectors
c     dt        o    Time step
c     geom      i    reserved
c     gmat      i    Gradient matrix
c     gmctinv   i    Connection+row degree of freedom to column indices
c                    for gradient matrix
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     idiscret  i    Type of discretization
c     igeom     i    reserved
c     invjac    i    1/sqrt(g) in pnttyp
c     jac       i    sqrt(g) in pnttyp
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     prescorr  i    Array containing the pressure correction
c     rhside    o    reserved
c     ustar    i/o   The intermediate velocity vector u*
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, alphad, npointsv

c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     i         General loop variable
c     j         General loop variable
c     npointsv  Number of points in the mesh with virtual points
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISCOVA2D  Fill covariant base vectors in all integration points in R2
c     ISCPGMO2D Give contribution of the pressure for collocated in R2
c     ISSETCON  Make array equal to a constant
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      do j = 1-nvirtual,nj+1+nvirtual
         do i = 1-nvirtual,ni+1+nvirtual
            prescorr(i,j) = prescorr(i,j) * dt / coefs(i,j,1)
         end do
      end do

      npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
      call issetcon ( cva, 16*npointsv, 0d0 )
      call iscova2d ( coor, cva, ni, nj, nvirtual )

      do alphad = 1, 2

         call issetcon ( rhside, (ni+1)*(nj+1), 0d0 )
         call iscpgmo2d ( ni, nj, nvirtual, rhside, jac,
     +                    alphad, cva, cvc, ctc, prescorr,
     +                    gmat, gmctinv, invjac, ibipoint, ibinfor,
     +                    idiscret, adir,
     +                    geom, igeom )

         do j = 1, nj
            do i = 1, ni
               ustar(i,j,alphad) = ustar(i,j,alphad) + rhside(i,j)
            end do
         end do

      end do

      do j = 1-nvirtual,nj+1+nvirtual
         do i = 1-nvirtual,ni+1+nvirtual
            prescorr(i,j) = prescorr(i,j) / dt * coefs(i,j,1)
         end do
      end do

      end
