      subroutine iscom01 ( isiinput, isrinput, issolut, iseddy )
c ======================================================================
c
c     programmer  Hester Bijl
c     version      1.8    date 03-03-1999 Duncan: new call to iscmrho2
c     version      1.7    date 15-02-1999 Duncan: remove call to isextcf2
c     version      1.6    date 22-07-1998 Duncan: new call to iscmrho2
c     version      1.5    date 09-06-1998 Charles: nvpoints changed to npointsv
c     version      1.4    date 05-03-1998 Duncan: extension for cavity
c     version      1.3    date 12-11-1997 Hester: new call to iscmrho2
c     version      1.2    date 10-10-1995 Guus: new call to iscmvel2
c     version      1.1    date 05-07-1995 Hester: cleaned up
c     version      1.0    date 30-05-1995
c
c   copyright (c) 1998-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     ISnas COMpressible routine 1, 2-dimensional.
c
c **********************************************************************
c
c                       KEYWORDS
c
c     compressible
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer isiinput, isrinput, issolut, iseddy

c     iseddy         i    Array of sequence numbers (nblocks) of array eddy
c     isiinput       i    Array of sequence numbers (nblocks) of array iinput
c     isrinput       i    Array of sequence numbers (nblocks) of array rinput
c     issolut        i    Array of sequence numbers (nblocks) of array solut
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ipiinput,iprinput, ipiincom, iprincom, ipsolut, mscal,
     +        ipeddy, ni, nj, ndegfd, ndim, nvirtual, npointsv,
     +        frsti, frstj, lasti,lastj, mcom, iswork, ipwork,
     +        mcav, istartup, ipiincav, iprincav

c     frsti          First index in i-direction
c     frstj          First index in j-direction
c     ipeddy         Starting address of array eddy in buffer
c     ipiincav       Starting address of array IINCAV in array IINPUT
c     ipiincom       Starting address of array IINCOM in array IINPUT
c     ipiinput       Starting address of array iinput in ibuffr
c     iprincav       Starting address of array RINCAV in array RINPUT
c     iprincom       Starting address of array RINCOM in array RINPUT
c     iprinput       Starting address of array rinput in buffer
c     ipsolut        Starting address of array solut in buffer
c     ipwork         Starting address of array work in buffer
c     istartup       If 1 the cavity calculation is in the startup
c                    procedure, if 0 not
c     iswork         Sequence number of array work
c     lasti          Last index in i-direction
c     lastj          Last index in j-direction
c     mcav           If 1 a cavity model is used, if 0 not
c     mcom           If zero the flow is incompressible else (1)
c                    compressible
c     mscal          Indicates use of non-dimensionalisation for
c                    compr. flow:
c                    0:   Equations are non-dimensionalised
c                    1:   Equations are in dimensional formulation
c     ndegfd         Number of degrees of freedom
c     ndim           Dimension of space (2 or 3)
c     ni             Number of non-virtual cells in the i-direction
c                    (one block)
c     nj             Number of non-virtual cells in the j-direction
c                    (one block)
c     npointsv       Number of points in the mesh with virtual points
c     nvirtual       Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c
c     none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp, isdpgetp

c     ISACTIVE       Activate old permanent array
c     ISCMRHO2       Computation of density, RHO ,from the equation of state, 2D
c     ISCMVEL2       Adapt velocity components for compressible flow
c     ISDCREAT       Create permanent space in array IBUFFR (Double precision ar
c     ISDELARR       Delete a permanent array
c     ISDPGETP       get the pointer corresponding to a double precision array
c     ISERCLOS       Resets old name of previous subroutine of higher level
c     ISEROPEN       Produces concatenated name of local subroutine
c     ISINGETP       get the pointer corresponding to an integer array
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     none
c **********************************************************************
c
c                       PSEUDO CODE
c
c     If compressible then
c         - compute density from equation of state
c         - extrapolate the density to the virtual points
c         - compute velocity dividing the momentum components by
c             the density
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('iscom01')

      call isactive(isiinput)
      call isactive(isrinput)
      call isactive(issolut)
      call isactive(iseddy)

      ipiinput = isingetp(isiinput)

      ni       = ibuffr(ipiinput + 51 - 1)
      nj       = ibuffr(ipiinput + 52 - 1)
      nvirtual = ibuffr(ipiinput + 54 - 1)
      npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)

      iswork   = 0
      call isdcreat ( iswork, npointsv, 'work' )
      ipiinput = isingetp(isiinput)
      iprinput = isdpgetp(isrinput)
      ipsolut  = isdpgetp(issolut )
      ipeddy   = isdpgetp(iseddy  )
      ipwork   = isdpgetp(iswork  )
      ipiincom = ibuffr(ipiinput +  9 - 1)
      ipiincav = ibuffr(ipiinput + 11 - 1)
      iprincom = ibuffr(ipiinput + 34 - 1)
      iprincav = ibuffr(ipiinput + 35 - 1)

      ndegfd   = ibuffr(ipiinput + 55 - 1)
      ndim     = ibuffr(ipiinput + 57 - 1)
      mcom     = ibuffr(ipiinput + ipiincom - 1)
      mscal    = ibuffr(ipiinput + ipiincom + 4- 1)
      mcav     = ibuffr(ipiinput + 77 - 1)
      if( mcav.eq.1) then
         istartup = ibuffr(ipiinput - 1+ ipiincav -1 + 2)
      else
         istartup = 0
      end if

      if ( ndim.eq.2 ) then

        frsti = 1-nvirtual
        lasti = ni+nvirtual
        frstj = 1-nvirtual
        lastj = nj+nvirtual

c       --- compute density from equation of state

        if ( mcom.eq.1 ) then

           call iscmrho2 ( ni, nj, ndegfd, ndim, nvirtual,
     +                     buffer(ipsolut),
     +                     buffer(iprinput-1+iprincom),
     +                     buffer(ipeddy),frsti, lasti,frstj,lastj,
     +                     mscal, mcav, 2 , istartup,
     +                     buffer(iprinput-1+iprincav),
     +                     ibuffr(ipiinput-1+ipiincom) )

         end if

c        --- compute velocity components from the previously computed fluxes
c            and density by u = flux/density

         call iscmvel2 ( ni, nj, nvirtual,
     +                   buffer(ipsolut+ndegfd*npointsv),
     +                   buffer(ipeddy+npointsv), buffer(ipwork) )

      end if

      call isdelarr ( iswork )

      call iserclos('iscom01')

      end
