      subroutine iscomsup (ni, nj, ndegfd, nvirtual, solut, eddy,
     +                     invjac, cva, rincom, switch, shiftr,
     +                     mscal, mcav, ichoicedb, istartup, rincav,
     +                     shrstarfo, shrstarho, rhotild, nblocks,
     +                     nfaces, maxnsubf, nsubfaces, subfaces )
c ======================================================================
c
c        programmer     Duncan van der Heul
c        version 1.3    date 12-12-1999     Duncan: revision of Supersonic
c                                                   with defect correction
c        version 1.2    date 04-04-1999     Duncan: extended with Supersonic
c                                                   with defect correction
c        version 1.1    date 23-03-1999     Duncan: extended with Supersonic
c                                                   Cranck-Nicholson
c        version 1.0    date 11-12-1998
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   DeFT compressible subroutine to calculate the switches for the (local)
c   switch to the supersonic scheme when Mach is above 0.9
c **********************************************************************
c
c                       KEYWORDS
c
c     compressible
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, ndegfd, nvirtual, mscal, mcav, istartup,
     +                 ichoicedb, nblocks, nfaces, maxnsubf,
     +                 nsubfaces(*), subfaces(*)
      double precision invjac(1-nvirtual:ni+1+nvirtual,
     +                 1-nvirtual:nj+1+nvirtual,4),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                 1-nvirtual:nj+1+nvirtual,ndegfd,*),
     +                 rincom(6),
     +                 switch(ni+1,nj+1,2),
     +                 cva(1-nvirtual:ni+1+nvirtual,
     +                 1-nvirtual:nj+1+nvirtual,2,2,4),
     +                 rincav(*), shiftr(ni+1,nj+1,*),
     +                 eddy(1-nvirtual:ni+1+nvirtual,
     +                 1-nvirtual:nj+1+nvirtual,2),
     +                 shrstarfo(ni+1,nj+1,*),
     +                 shrstarho(ni+1,nj+1,*),
     +                 rhotild(ni+1,nj+1,*)

c     cva            i    Array containing the covariant basis vectors
c                         in 2D we have
c                                                      l
c                         cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                      (alpha)     point type p
c                         in 3D it is cva(i,j,k,l,alpha,p) referring to
c                         cell i,j,k
c     eddy           i    array EDDY contains the values of the
c                         eddy viscosity of that block in the centroids
c                         of the elements. The storage of such
c                         an array EDDY is identical to that of all other
c                         coefficients.
c     ichoicedb      i    if 1 a gradient based, if 2 a mach based,
c                         if 3 an unconditional density bias is applied
c     invjac         i    1/sqrt(g) in pnttyp
c     istartup       i    If 1 the cavity calculation is in the startup
c                         procedure, if 0 not
c     maxnsubf       i    Maximum number of subfaces encountered on all subfaces
c     mcav           i    If 1 a cavity model is used, if 0 not
c     mscal          i    Indicates use of non-dimensionalisation for
c                         compr. flow:
c                         0:   Equations are non-dimensionalised
c                         1:   Equations are in dimensional formulation
c     nblocks        i    Number of blocks
c     ndegfd         i    Number of degrees of freedom
c     nfaces         i    Number of faces
c     ni             i    Number of non-virtual cells in the i-direction
c                         (one block)
c     nj             i    Number of non-virtual cells in the j-direction
c                         (one block)
c     nsubfaces      i    Array containing the number of subfaces of each face,
c                         see array itopol for an extended description
c     nvirtual       i    Number of rows of virtual cells
c     rhotild        i    ?
c     rincav         i    Array RINCAV is part of array RINPUT and hence is
c                         coupled to one block only.
c                         For a description see the programmers guide
c     rincom         i    Array RINCOM is part of array IINPUT and hence
c                         coupled to one block only. It contains information
c                         with respect to the compressible flow
c                         For a description see the programmers guide
c     shiftr         i    Shift variable used in compressible density upwinding.
c     shrstarfo      i    Shift variable based on tentative density field
c                         rhostar, first order
c     shrstarho      i    Shift variable based on tentative density field
c                         rhostar, higher order
c     solut          i    Solution array
c                         For a description see the programmers guide
c     subfaces       i    This array contains a description of all subfaces
c                         subfaces(iblock,iface,isubface,.) refers to block
c                         iblock, face iface and subface isubface
c                         Contents of the last index
c                         Pos. 1  i1
c                         Pos. 2  i2
c                         Pos. 3  j1
c                         Pos. 4  j2
c                         Pos. 5  block number neighbour block
c                         Pos. 6  face number neighbour face
c                         Pos. 7  subface number neighbour subface
c                         Pos. 8  iorient
c                         Pos. 9  Reserved for preprocessor only
c                         Pos. 10 Reserved for preprocessor only
c     switch         i    Continuous compressible density upwind switch
c                           switch = 0 for Mach numbers < 1-ls
c                           switch=0.5 for Mach numbers > 1+ls
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, k
      double precision gamcm, mach2, cons, cons1, ls,
     +                 locm, uvel, vvel, speed2, csqrdinv
      logical periodic

c     cons           Constant used in equation of state for compressible
c                    computations
c     cons1          Constant used in equation of state for compressible
c                    computations
c     csqrdinv       Inverse of speed of sound squared
c     gamcm          Gas constant gamma (for compressible comput.)
c     i              General loop variable
c     j              General loop variable
c     k              General loop variable
c     locm           Local Mach number.
c     ls             Length of switch interval for compr. density
c                    upwind switch
c                      switch=0   for Mach numbers < 1-ls
c                      switch=0.5 for Mach numbers > 1+ls
c     mach2          Square of the mach number (for compressible comput.)
c     periodic       Periodic boundary conditions
c     speed2         Speed squared
c     uvel           Velocity in i direction
c     vvel           Velocity in j direction
c **********************************************************************
c
c                       I/O
c
c     none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      double precision isdstfll, isdstlss, isdlimt

c     ISCHECKPERI Check for periodic boundary conditions
c     ISCHECKPERJ Check for periodic boundary conditions
c     ISDLIMT     limiter function for the upwind density bias
c     ISDSTFLL    Derivative of dimensional equation of state
c     ISDSTLSS    Derivative of dimensionless equation of state
c     ISSETCON    Make array equal to a constant
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     none
c **********************************************************************
c
c                       PSEUDO CODE
c
c      Mach based density bias
c
c          -if nondimensional computation
c              -if speed greater than 0
c                    - compute local Ma nr
c                             |U|     a_alpha U^alpha
c                        M = ------ = ----------------
c                              a             a
c                     a = nondimensional speed of sound
c                    - compute switch function for density shift i direction
c                    - compute switch function for density shift j direction
c              -end if
c          -else if dimensional computation
c              -if speed greater than 0
c                    - compute local Ma nr
c                             |U|     a_alpha U^alpha
c                        M = ------ = ----------------
c                              a             a
c                     a = dimensional speed of sound
c                       = sqrt((gamma-1)|h|)
c                    - compute switch function for density shift i direction
c                    - compute switch function for density shift j direction
c              -end if
c          -end if
c          - compute shift ratio of density equal to shifted density divided
c              by non shifted density i direction
c          - compute shift ratio of density equal to shifted density divided
c              by non shifted density j direction
c            a) For switch == 0   no upwinding, so shift = 1
c            b) For switch == 1 full upwinding, so
c
c                          upwinded density
c                  shift= ------------------
c                           central density
c            c) For switch between 0 and 0.5 shift between value of a) and b)
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- set both switch and ratio of shifted/central density to noshift value

      call issetcon(switch,(ni+1)*(nj+1)*2,0.5d0)

c     --- ratio of shifted/central density for eddy at previous timelevel
c              and the tentative density field rhostar

      call issetcon(shiftr,(ni+1)*(nj+1)*2,1d0)
      call issetcon(shrstarho,(ni+1)*(nj+1)*2,1d0)
      call issetcon(shrstarfo,(ni+1)*(nj+1)*2,1d0)

      if(ichoicedb.eq.4) then

         gamcm = rincom(1)
         mach2 = rincom(2)**2d0

         cons = 1d0 + ((gamcm - 1d0)*mach2)/2d0

         cons1 = mach2 / cons

         ls=0.4d0

         do k= 1,2
            do j = 1, nj+1
               do i = 1, ni+1

                  switch(i,j,k) = 0d0

               end do
            end do
         end do

         if (mscal.eq.0) then

c        --- compute the local mach number in the center of the cells,
c            and compute the continuous switch function, nondimensional.

           do j = 1, nj+1
              do i = 1, ni+1

                 speed2=( solut(i,j,1,1)*cva(i,j,1,1,2 )   +
     +                         solut(i,j,2,1)*cva(i,j,1,2,2 )  )**2
     +                                      +
     +                       ( solut(i,j,1,1)*cva(i,j,2,1,2 ) +
     +                         solut(i,j,2,1)*cva(i,j,2,2,2 )  )**2

                 if(mcav.eq.1)  then

                     locm =  invjac(i,j,3)*sqrt(speed2 *
     +                        isdstlss(solut(i,j,3,1), rincom,
     +                                          istartup, rincav))
                 else

                     locm =  invjac(i,j,3)*sqrt(speed2 * cons1 /
     +                   abs(solut(i,j,4,1)) )

                 end if

                 uvel=abs(solut(i,j,1,1)*invjac(i,j,3) )
                 vvel=abs(solut(i,j,2,1)*invjac(i,j,3) )
                 if( uvel.gt.1d-15) then

                    switch(i,j,1) = min(1d0,max(0d0,0.25d0*
     +                           (((locm-1d0)/ls)+1d0)))

                 else

                    switch(i,j,1) = 0d0
                 end if
                 if( mcav.eq.1) then
                    if( vvel.gt.1d-15) then

                       switch(i,j,2) = min(1d0,max(0d0,0.25d0*
     +                           (((locm-1d0)/ls)+1d0)))

                    else

                       switch(i,j,2) = 0d0

                    end if
                 else

                    switch(i,j,2) = 0d0

                 end if

             end do
           end do

         else if (mscal.eq.1) then

c        --- unscaled, dimensional computation

            do j = 1, nj+1
               do i = 1, ni+1

                 speed2=( solut(i,j,1,1)*cva(i,j,1,1,2 )   +
     +                         solut(i,j,2,1)*cva(i,j,1,2,2 )  )**2
     +                                      +
     +                       ( solut(i,j,1,1)*cva(i,j,2,1,2 ) +
     +                         solut(i,j,2,1)*cva(i,j,2,2,2 )  )**2
                 speed2=( solut(i,j,1,1))**2

                 if(mcav.eq.1)  then

                     locm =  invjac(i,j,3)*sqrt(speed2 *
     +                       isdstfll(solut(i,j,3,1),
     +                                        istartup, rincav))

                 else

                     locm =  invjac(i,j,3)*sqrt(speed2  /
     +                   ((gamcm-1d0)*abs(solut(i,j,4,1))) )

                 end if

                 uvel=abs(solut(i,j,1,1)*invjac(i,j,3) )
                 vvel=abs(solut(i,j,2,1)*invjac(i,j,3) )
                 if( uvel.gt.1d-15) then

                    switch(i,j,1) = min(1d0,max(0d0,0.25d0*
     +                        (((locm-1d0)/ls)+1d0)))

                 else
                    switch(i,j,1) = 0d0

                 end if

                 if( mcav.eq.1) then
                    if( vvel.gt.1d-15) then

                       switch(i,j,2) = min(1d0,max(0d0,0.25d0*
     +                        (((locm-1d0)/ls)+1d0)))

                     else

                       switch(i,j,2) = 0d0

                    end if
                 else

                    switch(i,j,2) = 0d0

                 end if

               end do
            end do

         end if

      else if( ichoicedb.eq.5) then

c     --- fix both switches to 0.5d0 for Supersonic Cranck-Nicholson

         do j=1, nj+1
            do i=1, 2
               switch(i,j,1) =0d0
            end do
            do i=3,ni-1
               switch(i,j,1) =0.5d0
            end do
            do i=ni-1, ni+1
               switch(i,j,1) =0d0
            end do
         end do

         do j=1, 2
            do i=1, ni+1
               switch(i,j,2) =0d0
            end do
         end do
         do j=3, nj-1
            do i=1, ni+1
               switch(i,j,2) =0.5d0
            end do
         end do
         do j=nj, nj+1
            do i=1, ni+1
               switch(i,j,2) =0d0
            end do
         end do

c        --- compute shifted density Sro/ro in points (i-1/2,j)

         do i = 2, ni+1
            do j = 1, nj+1
               uvel=solut(i,j,1,1)

               shiftr(i,j,1) =
     +                        ((1d0-sign(1d0,uvel))*
     +                                             eddy(i,j,2)
     +                        +(1d0+sign(1d0,uvel))*
     +                                             eddy(i-1,j,2))/
     +                  (eddy(i,j,2)+eddy(i-1,j,2))

            end do
         end do

         if( mcav.eq.1) then

            do i = 1, ni+1
               do j = 1, nj+1
                  vvel=solut(i,j,2,1)

                  shiftr(i,j,2) =
     +                           ((1d0-sign(1d0,vvel))*
     +                                             eddy(i,j,2)
     +                           +(1d0+sign(1d0,vvel))*
     +                                             eddy(i,j-1,2))/
     +                      (eddy(i,j,2)+eddy(i,j-1,2))

               end do
            end do
         else

            call issetcon(shiftr,(ni+1)*(nj+1),1d0)

         end if

      else if( ichoicedb.eq.6) then

         do j=1, nj+1
            do i=1, 1
               switch(i,j,1) =0d0
            end do
            do i=2,ni-1

c           --- define square of speed of sound at the face

               if(mcav.eq.1)  then

                  csqrdinv= (isdstfll(solut(i-1,j,3,1),
     +                                      istartup, rincav)+
     +                       isdstfll(solut(i  ,j,3,1),
     +                                      istartup, rincav))/2d0
               else
                  csqrdinv= 2d0*cons1 /
     +                         (abs(solut(i-1,j,4,1))+
     +                                 abs(solut(i,j,4,1)))
               end if

c              --- define normal component of cartesian velocity squared

               speed2=   solut(i,j,1,1)**2/
     +                     ( cva(i,j,2,1,3 )**2+cva(i,j,2,2,3 )**2)

               locm =    sqrt(speed2*csqrdinv)

               if ( locm.lt.0.5d0) then
                  switch(i,j,1) = 0d0
               else if(locm.ge.0.5d0 .and. locm .lt.4d0) then
                  switch(i,j,1) = 0.5d0
               else if(locm.ge.4d0 .and. locm.lt.8d0) then
                  switch(i,j,1) = 0.5d0
               else if(locm.ge.8d0) then
                  switch(i,j,1) = 0.875d0
               end if
            end do
            do i=ni-1, ni+1
               switch(i,j,1) =0d0
            end do
         end do

         do j=1, 1
            do i=1, ni+1
               switch(i,j,2) =0d0
            end do
         end do

         do j=2, nj+1
            do i=1, ni+1

c           --- define square of inverse of  speed of sound at the face

               if(mcav.eq.1)  then

                  csqrdinv= (isdstfll(solut(i,j-1,3,1),
     +                                      istartup, rincav)+
     +                       isdstfll(solut(i,j  ,3,1),
     +                                      istartup, rincav))
     +                                                     /2d0
               else
                  csqrdinv= 2d0*cons1 /
     +                         (abs(solut(i,j-1,4,1))+
     +                                 abs(solut(i,j,4,1)))
               end if

c              --- define normal component of cartesian velocity squared

               speed2=   solut(i,j,2,1)**2/
     +                     ( cva(i,j,1,1,2 )**2+
     +                                      cva(i,j,2,1,2 )**2)

               locm =    sqrt(speed2*csqrdinv)

               if ( locm.lt.0.5d0) then
                  switch(i,j,2) = 0d0
               else if(locm.ge.0.5d0 .and. locm .lt.4d0) then
                  switch(i,j,2) = 0.5d0
               else if(locm.ge.4d0 .and. locm.lt.8d0) then
                  switch(i,j,2) = 0.5d0
               else if(locm.ge.8d0) then
                  switch(i,j,2) = 0.875d0
               end if

            end do
         end do

c        --- compute shifted density Sro/ro in points (i-1/2,j) and
c            (i,j-1/2)

         i=1
         do j = 1, nj+1

            periodic = .FALSE.

            call ischeckperi(iblockrf, ni, nj, nblocks,
     +                      nfaces, maxnsubf, nsubfaces,
     +                      subfaces, 1, j, periodic)
            if( periodic) then
               uvel=solut(i,j,1,1)

               shiftr(i,j,1) =
     +                        ((1d0-sign(1d0,uvel))*
     +                                             eddy(i,j,2)
     +                        +(1d0+sign(1d0,uvel))*
     +                                             eddy(i-1,j,2))/
     +                  (eddy(i,j,2)+eddy(i-1,j,2))

            end if

         end do

         do j = 1, nj+1
            do i = 2, ni

               uvel=solut(i,j,1,1)

               shiftr(i,j,1) =
     +                        ((1d0-sign(1d0,uvel))*
     +                                             eddy(i,j,2)
     +                        +(1d0+sign(1d0,uvel))*
     +                                             eddy(i-1,j,2))/
     +                  (eddy(i,j,2)+eddy(i-1,j,2))

            end do
         end do

         i=ni+1
         do j = 1, nj+1

            periodic = .FALSE.

            call ischeckperi(iblockrf, ni, nj, nblocks,
     +                      nfaces, maxnsubf, nsubfaces,
     +                      subfaces, ni, j, periodic)
            if( periodic) then
               uvel=solut(i,j,1,1)

               shiftr(i,j,1) =
     +                        ((1d0-sign(1d0,uvel))*
     +                                             eddy(i,j,2)
     +                        +(1d0+sign(1d0,uvel))*
     +                                             eddy(i-1,j,2))/
     +                  (eddy(i,j,2)+eddy(i-1,j,2))

            end if

         end do

         if( mcav.eq.1) then

            j=1
            do i = 1, ni+1

               periodic = .FALSE.

               call ischeckperj(iblockrf, ni, nj, nblocks,
     +                         nfaces, maxnsubf, nsubfaces,
     +                         subfaces, i, 1, periodic)
               if( periodic) then

                  vvel=solut(i,j,2,1)

                  shiftr(i,j,2) =
     +                           ((1d0-sign(1d0,vvel))*
     +                                             eddy(i,j,2)
     +                           +(1d0+sign(1d0,vvel))*
     +                                             eddy(i,j-1,2))/
     +                      (eddy(i,j,2)+eddy(i,j-1,2))
               end if

            end do

            do j = 2, nj
               do i = 1, ni+1

                  vvel=solut(i,j,2,1)

                  shiftr(i,j,2) =
     +                           ((1d0-sign(1d0,vvel))*
     +                                             eddy(i,j,2)
     +                           +(1d0+sign(1d0,vvel))*
     +                                             eddy(i,j-1,2))/
     +                      (eddy(i,j,2)+eddy(i,j-1,2))

               end do
            end do

            j = nj+1
            do i = 1, ni+1

               periodic = .FALSE.

               call ischeckperj(iblockrf, ni, nj, nblocks,
     +                         nfaces, maxnsubf, nsubfaces,
     +                         subfaces, i, nj, periodic)
               if( periodic) then

                  vvel=solut(i,j,2,1)

                  shiftr(i,j,2) =
     +                           ((1d0-sign(1d0,vvel))*
     +                                             eddy(i,j,2)
     +                           +(1d0+sign(1d0,vvel))*
     +                                             eddy(i,j-1,2))/
     +                      (eddy(i,j,2)+eddy(i,j-1,2))
               end if

            end do

         else

            call issetcon(shiftr,(ni+1)*(nj+1),1d0)

         end if

c        --- first order upwind shift of the tentative density field rhostar

c            in points (i-1/2,j)

         do j = 1, nj+1
            do i = 1, ni+1

                  uvel=sign(1d0,solut(i,j,1,1))
                  shrstarfo(i,j,1) =(0.5d0*(1d0+uvel)*
     +                              (eddy(i-1,j,2))+
     +                            0.5d0*(1d0-uvel)*
     +                              (eddy(i,j,2)))/
     +                               (0.5d0*(eddy(i-1,j,2)+
     +                                              eddy(i,j,2)))
            end do
         end do

c        --- in points (i,j-1/2)

         do j = 1, nj+1
            do i = 1, ni+1
                  vvel=sign(1d0,solut(i,j,2,1))
                  shrstarfo(i,j,2) =(0.5d0*(1d0+vvel)*
     +                              (eddy(i,j-1,2))+
     +                            0.5d0*(1d0-vvel)*
     +                              (eddy(i,j,2)))/
     +                               (0.5d0*(eddy(i,j-1,2)+
     +                                              eddy(i,j,2)))
            end do
         end do

c        --- higher order upwind shift of the tentative density field rhostar

c            reset rhotild, argument of limiter function isdlimt

         call issetcon(rhotild,2*(ni+1)*(nj+1),0d0)

c        --- computation of the value of rhotild in i direction

         do j=1,nj+1
            do i=1,ni+1

               if( abs(eddy(i+1,j,2)-eddy(i,j,2)).gt.1d-12) then
                  rhotild(i,j,1)=(eddy(i,j,2)-eddy(i-1,j,2))/
     +                   (eddy(i+1,j,2)-eddy(i,j,2))
               else
                  rhotild(i,j,1)=(eddy(i,j,2)-eddy(i-1,j,2))*
     +                    sign(1d12,(eddy(i+1,j,2)-eddy(i,j,2)))
               end if

               if( abs(rhotild(i,j,1)).lt.1d-12)
     +              rhotild(i,j,1)=rhotild(i,j,1)+1d-12
            end do
         end do

c        --- computation of the value of rhotild in j direction

         do j=1,nj+1
            do i=1,ni+1

               if( abs(eddy(i,j+1,2)-eddy(i,j,2)).gt.1d-12) then
                  rhotild(i,j,2)=(eddy(i,j,2)-eddy(i,j-1,2))/
     +                   (eddy(i,j+1,2)-eddy(i,j,2))
               else
                  rhotild(i,j,2)=(eddy(i,j,2)-eddy(i,j-1,2))*
     +                    sign(1d12,(eddy(i,j+1,2)-eddy(i,j,2)))
               end if

               if( abs(rhotild(i,j,2)).lt.1d-12)
     +              rhotild(i,j,2)=rhotild(i,j,2)+1d-12
            end do
         end do

c        --- computation of higher order shifted density Sro/ro
c                                              in points (i-1/2,j)

         do j = 1, nj+1
            do i = 1, ni+1
                  uvel=sign(1d0,solut(i,j,1,1))
                  shrstarho(i,j,1) =(0.5d0*(1d0+uvel)*
     +                     (eddy(i-1,j,2)+
     +                       0.5d0*isdlimt(rhotild(i-1,j,1))*
     +                          (eddy(i,j,2)-eddy(i-1,j,2)))+
     +                   0.5d0*(1d0-uvel)*
     +                     (eddy(i,j,2)+
     +                       0.5d0*isdlimt(1d0/rhotild(i,j,1))*
     +                          (eddy(i-1,j,2)-eddy(i,j,2))))/
     +                      (0.5d0*(eddy(i-1,j,2)+eddy(i,j,2)))

                 if( uvel.gt.1d-8) then
                    if( isdlimt(rhotild(i-1,j,1)).lt.1d-6 .and.
     +                    switch(i,j,1).gt.1d-4 ) then
                       switch(i,j,1)=0.5d0
                    end if
                 else if( uvel.lt.-1d-8) then
                    if( isdlimt(1d0/rhotild(i,j,1)).lt.1d-6.and.
     +                    switch(i,j,1).gt.1d-4 ) then
                       switch(i,j,1)=0.5d0
                    end if
                 end if
            end do
         end do

c        --- computation of higher order shifted density Sro/ro
c                                              in points (i,j-1/2)

         do j = 1, nj+1
            do i = 1, ni+1
                  vvel=sign(1d0,solut(i,j,2,1))
                  shrstarho(i,j,2) =(0.5d0*(1d0+vvel)*
     +                     (eddy(i,j-1,2)+
     +                       0.5d0*isdlimt(rhotild(i,j-1,2))*
     +                          (eddy(i,j,2)-eddy(i,j-1,2)))+
     +                   0.5d0*(1d0-vvel)*
     +                     (eddy(i,j,2)+
     +                   0.5d0*isdlimt(1d0/rhotild(i,j,2))*
     +                          (eddy(i,j-1,2)-eddy(i,j,2))))/
     +                      (0.5d0*(eddy(i,j-1,2)+eddy(i,j,2)))
                 if( vvel.gt.1d-8) then
                    if( isdlimt(rhotild(i,j-1,2)).lt.1d-6 .and.
     +                  switch(i,j,2).gt.1d-4) then
                       switch(i,j,2)=0.5d0
                    end if
                 else if( vvel.lt.-1d-8) then
                    if( isdlimt(1d0/rhotild(i,j,2)).lt.1d-6.and.
     +                  switch(i,j,2).gt.1d-4) then
                       switch(i,j,2)=0.5d0
                    end if
                 end if
            end do
         end do

          do j=1, nj+1
             do i=2, ni-1
               shiftr(i,j,1)=shrstarho(i,j,1)
             end do
          end do

          do j=2, nj-1
             do i=1, ni+1
               shiftr(i,j,2)=shrstarho(i,j,2)
             end do
          end do

      end if

      end
