      subroutine iscova3d ( coor, cva , ni, nj, nk, nvirtual,
     +                      npf , rspv, cv )
c ======================================================================
c
c
c        programmer     Kees Kassels
c        version 1.0    date 16-05-95 (as iscova2d for 3D)
c
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                      DESCRIPTION
c
c    Compute covariant base vectors in all points of the grid
c    three-dimensional case
c ********************************************************************
c
c                      KEYWORDS
c
c    base_vectors
c    interpolation
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,3)
      double precision cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual,3,3,8)
      integer npf(0:3,0:4,1:8), rspv(1:3,1:8,0:3,0:4,1:8)
      double precision cv(1:8,0:3,0:4,1:8)

c     coor      i    reserved
c     cv        i    reserved
c     cva       o    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     npf       i    reserved
c     nvirtual  i    Number of rows of virtual cells
c     rspv      i    reserved
c ********************************************************************
c
c                      COMMON BLOCKS
c
c ********************************************************************
c
c                      LOCAL PARAMETERS

      integer i, j, k, vartyp, pnttyp, lnpf, lrs(3,8), alphad, betad
      double precision lcv(8)

c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     i         General loop variable
c     j         General loop variable
c     k         General loop variable
c     lcv       Cv for the given acttyp, vartyp and pnttyp
c     lnpf      Npf for the given acttyp, vartyp and pnttyp
c     lrs       Rspv for the given acttyp, vartyp and pnttyp
c     pnttyp    reserved
c     vartyp    Variable to be interpolated/extrapolated
c               vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c               0 for vertex (for ndim=2)
c               or :
c               vartyp: 1 for V1, 2 for V2 3, for V3 and
c               4 for pressure, 0 for vertex  (for ndim=3)
c ********************************************************************
c
c                      SUBROUTINES CALLED
c
c ********************************************************************
c
c                      I/O
c
c ********************************************************************
c
c                      ERROR MESSAGES
c
c ********************************************************************
c
c                      PSEUDO CODE
c
c     The covariant basis vectors are given by:
c      alpha                    beta
c     a       = dx       / d xsi       = cva ( i, j, k, alpha, beta, n )
c      (beta)     alpha
c
c     i,j,k refers to the cell index and n (pnttyp) to the position of the point
c     in a cell
c
c     To evaluate the base vectors a central difference formula is applied
c     This formula is implicitly stored in the array npf, cv and rspv
c     according to:
c
c      npf(beta,vartyp,pnttyp)
c     sum        cv(m) x(alpha+dalpha(m))
c      m=1
c
c     where vartyp = 0 (coordinates are given in the vertices) and
c     dalpha(m) = rspv(alpha, m, beta, vartyp, pnttyp)
c     The parameter acttype is equal to beta (difference in beta direction)
c
c     The integration points for all internal cells as well as one row of
c     virtual cells are filled.
c     In order to compute the Christoffel symbols, which require derivatives of
c     covariant base functions some points in the second row of virtual cells
c     must be filled.
c
c ======================================================================
c
      vartyp = 0
      do pnttyp = 1, 8

         do betad = 1, 3

c        --- Introduce abbreviations for interpolation formula components

            lnpf = npf(betad,vartyp,pnttyp)
            do j = 1, lnpf
               do i = 1, 3
                  lrs(i,j) = rspv(i,j,betad,vartyp,pnttyp)
               end do
               lcv(j) = cv(j,betad,vartyp,pnttyp)
            end do
            do alphad = 1, 3

c           --- for higher efficiency we deal with each possible lnpf
c               separately (maximize calculation vs. memory access of cva)

               if ( lnpf.eq.2 ) then

c              --- two-point difference

                  do k = 2-nvirtual, nk+nvirtual
                     do j = 2-nvirtual, nj+nvirtual
                        do i = 2-nvirtual, ni+nvirtual
                           cva(i,j,k,alphad,betad,pnttyp) =
     +                      lcv(1)*coor(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1)
     +                                  ,alphad)+
     +                      lcv(2)*coor(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)
     +                                  ,alphad)
                        end do
                     end do
                  end do

c                 --- Boundary cells (planes )

                  if ( mod(pnttyp,2).eq.0 ) then

c                 --- point types 2, 4, 6 and 8

                     i = 1-nvirtual
                     do k = 2-nvirtual, nk+nvirtual
                        do j = 2-nvirtual, nj+nvirtual
                           cva(i,j,k,alphad,betad,pnttyp) =
     +                      lcv(1)*coor(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1)
     +                                  ,alphad)+
     +                      lcv(2)*coor(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)
     +                                  ,alphad)
                        end do
                     end do

                  end if

                  if ( pnttyp.eq.3 .or. pnttyp.eq.4 .or.
     +                 pnttyp.eq.7 .or. pnttyp.eq.8) then

c                 --- point types 3, 4, 7 and 8

                     j = 1-nvirtual
                     do k = 2-nvirtual, nk+nvirtual
                        do i = 2-nvirtual, ni+nvirtual
                           cva(i,j,k,alphad,betad,pnttyp) =
     +                      lcv(1)*coor(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1)
     +                                  ,alphad)+
     +                      lcv(2)*coor(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)
     +                                  ,alphad)
                        end do
                     end do

                  end if

                  if ( pnttyp.ge.5 ) then

c                 --- point types 5, 6, 7 and 8

                     k = 1-nvirtual
                     do j = 2-nvirtual, nj+nvirtual
                        do i = 2-nvirtual, ni+nvirtual
                           cva(i,j,k,alphad,betad,pnttyp) =
     +                      lcv(1)*coor(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1)
     +                                  ,alphad)+
     +                      lcv(2)*coor(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)
     +                                  ,alphad)
                        end do
                     end do

                  end if


c                 --- Boundary cells ( edges )

                  if ( pnttyp.ge.7 ) then

c                 --- point types  7 and 8

                     k = 1-nvirtual
                     j = 1-nvirtual
                        do i = 2-nvirtual, ni+nvirtual
                           cva(i,j,k,alphad,betad,pnttyp) =
     +                      lcv(1)*coor(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1)
     +                                  ,alphad)+
     +                      lcv(2)*coor(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)
     +                                  ,alphad)
                        end do

                  end if

                  if ( pnttyp.eq.6 .or. pnttyp.eq.8 ) then

c                 --- point types  6 and 8

                     k = 1-nvirtual
                     i = 1-nvirtual
                        do j = 2-nvirtual, nj+nvirtual
                           cva(i,j,k,alphad,betad,pnttyp) =
     +                      lcv(1)*coor(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1)
     +                                  ,alphad)+
     +                      lcv(2)*coor(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)
     +                                  ,alphad)
                        end do

                  end if


                  if ( pnttyp.eq.4 .or. pnttyp.eq.8 ) then

c                 --- point types  4 and 8

                     i = 1-nvirtual
                     j = 1-nvirtual
                        do k = 2-nvirtual, nk+nvirtual
                           cva(i,j,k,alphad,betad,pnttyp) =
     +                      lcv(1)*coor(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1)
     +                                  ,alphad)+
     +                      lcv(2)*coor(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)
     +                                  ,alphad)
                        end do

                  end if

                  if ( pnttyp.eq.8  ) then

c                 --- point type  8

                     i = 1-nvirtual
                     j = 1-nvirtual
                     k = 1-nvirtual
                           cva(i,j,k,alphad,betad,pnttyp) =
     +                      lcv(1)*coor(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1)
     +                                  ,alphad)+
     +                      lcv(2)*coor(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)
     +                                  ,alphad)

                  end if

               else if ( lnpf.eq.4 ) then

c              --- four-point difference

                  do k = 2-nvirtual, nk+nvirtual
                     do j = 2-nvirtual, nj+nvirtual
                        do i = 2-nvirtual, ni+nvirtual
                        cva(i,j,k,alphad,betad,pnttyp) =
     +                      lcv(1)*coor(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1)
     +                                  ,alphad)+
     +                      lcv(2)*coor(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)
     +                                  ,alphad)+
     +                      lcv(3)*coor(i+lrs(1,3),j+lrs(2,3),k+lrs(3,3)
     +                                  ,alphad)+
     +                      lcv(4)*coor(i+lrs(1,4),j+lrs(2,4),k+lrs(3,4)
     +                                  ,alphad)
                        end do
                     end do
                  end do


c                 --- Boundary cells (planes )

                  if ( mod(pnttyp,2).eq.0 ) then

c                 --- point types 2, 4, 6 and 8

                     i = 1-nvirtual
                     do k = 2-nvirtual, nk+nvirtual
                        do j = 2-nvirtual, nj+nvirtual
                           cva(i,j,k,alphad,betad,pnttyp) =
     +                      lcv(1)*coor(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1)
     +                                  ,alphad)+
     +                      lcv(2)*coor(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)
     +                                  ,alphad)+
     +                      lcv(3)*coor(i+lrs(1,3),j+lrs(2,3),k+lrs(3,3)
     +                                  ,alphad)+
     +                      lcv(4)*coor(i+lrs(1,4),j+lrs(2,4),k+lrs(3,4)
     +                                  ,alphad)
                        end do
                     end do

                  end if

                  if ( pnttyp.eq.3 .or. pnttyp.eq.4 .or.
     +                 pnttyp.eq.7 .or. pnttyp.eq.8) then

c                 --- point types 3, 4, 7 and 8

                     j = 1-nvirtual
                     do k = 2-nvirtual, nk+nvirtual
                        do i = 2-nvirtual, ni+nvirtual
                           cva(i,j,k,alphad,betad,pnttyp) =
     +                      lcv(1)*coor(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1)
     +                                  ,alphad)+
     +                      lcv(2)*coor(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)
     +                                  ,alphad)+
     +                      lcv(3)*coor(i+lrs(1,3),j+lrs(2,3),k+lrs(3,3)
     +                                  ,alphad)+
     +                      lcv(4)*coor(i+lrs(1,4),j+lrs(2,4),k+lrs(3,4)
     +                                  ,alphad)

                        end do
                     end do

                  end if

                  if ( pnttyp.ge.5 ) then

c                 --- point types 5, 6, 7 and 8

                     k = 1-nvirtual
                     do j = 2-nvirtual, nj+nvirtual
                        do i = 2-nvirtual, ni+nvirtual
                           cva(i,j,k,alphad,betad,pnttyp) =
     +                      lcv(1)*coor(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1)
     +                                  ,alphad)+
     +                      lcv(2)*coor(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)
     +                                  ,alphad)+
     +                      lcv(3)*coor(i+lrs(1,3),j+lrs(2,3),k+lrs(3,3)
     +                                  ,alphad)+
     +                      lcv(4)*coor(i+lrs(1,4),j+lrs(2,4),k+lrs(3,4)
     +                                  ,alphad)

                        end do
                     end do

                  end if


c                 --- Boundary cells ( edges )

                  if ( pnttyp.ge.7 ) then

c                 --- point types  7 and 8

                     k = 1-nvirtual
                     j = 1-nvirtual
                        do i = 2-nvirtual, ni+nvirtual
                           cva(i,j,k,alphad,betad,pnttyp) =
     +                      lcv(1)*coor(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1)
     +                                  ,alphad)+
     +                      lcv(2)*coor(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)
     +                                  ,alphad)+
     +                      lcv(3)*coor(i+lrs(1,3),j+lrs(2,3),k+lrs(3,3)
     +                                  ,alphad)+
     +                      lcv(4)*coor(i+lrs(1,4),j+lrs(2,4),k+lrs(3,4)
     +                                  ,alphad)

                        end do

                  end if

                  if ( pnttyp.eq.6 .or. pnttyp.eq.8 ) then

c                 --- point types  6 and 8

                     k = 1-nvirtual
                     i = 1-nvirtual
                        do j = 2-nvirtual, nj+nvirtual
                           cva(i,j,k,alphad,betad,pnttyp) =
     +                      lcv(1)*coor(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1)
     +                                  ,alphad)+
     +                      lcv(2)*coor(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)
     +                                  ,alphad)+
     +                      lcv(3)*coor(i+lrs(1,3),j+lrs(2,3),k+lrs(3,3)
     +                                  ,alphad)+
     +                      lcv(4)*coor(i+lrs(1,4),j+lrs(2,4),k+lrs(3,4)
     +                                  ,alphad)

                        end do

                  end if


                  if ( pnttyp.eq.4 .or. pnttyp.eq.8 ) then

c                 --- point types  4 and 8

                     i = 1-nvirtual
                     j = 1-nvirtual
                        do k = 2-nvirtual, nk+nvirtual
                           cva(i,j,k,alphad,betad,pnttyp) =
     +                      lcv(1)*coor(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1)
     +                                  ,alphad)+
     +                      lcv(2)*coor(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)
     +                                  ,alphad)+
     +                      lcv(3)*coor(i+lrs(1,3),j+lrs(2,3),k+lrs(3,3)
     +                                  ,alphad)+
     +                      lcv(4)*coor(i+lrs(1,4),j+lrs(2,4),k+lrs(3,4)
     +                                  ,alphad)

                        end do

                  end if

                  if ( pnttyp.eq.8  ) then

c                 --- point type  8

                     i = 1-nvirtual
                     j = 1-nvirtual
                     k = 1-nvirtual
                           cva(i,j,k,alphad,betad,pnttyp) =
     +                      lcv(1)*coor(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1)
     +                                  ,alphad)+
     +                      lcv(2)*coor(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)
     +                                  ,alphad)+
     +                      lcv(3)*coor(i+lrs(1,3),j+lrs(2,3),k+lrs(3,3)
     +                                  ,alphad)+
     +                      lcv(4)*coor(i+lrs(1,4),j+lrs(2,4),k+lrs(3,4)
     +                                  ,alphad)
                  end if

               else if ( lnpf.eq.8 ) then

c              --- four-point difference

                  do k = 2-nvirtual, nk+nvirtual
                     do j = 2-nvirtual, nj+nvirtual
                        do i = 2-nvirtual, ni+nvirtual
                        cva(i,j,k,alphad,betad,pnttyp) =
     +                      lcv(1)*coor(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1)
     +                                  ,alphad)+
     +                      lcv(2)*coor(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)
     +                                  ,alphad)+
     +                      lcv(3)*coor(i+lrs(1,3),j+lrs(2,3),k+lrs(3,3)
     +                                  ,alphad)+
     +                      lcv(4)*coor(i+lrs(1,4),j+lrs(2,4),k+lrs(3,4)
     +                                  ,alphad)+
     +                      lcv(5)*coor(i+lrs(1,5),j+lrs(2,5),k+lrs(3,5)
     +                                  ,alphad)+
     +                      lcv(6)*coor(i+lrs(1,6),j+lrs(2,6),k+lrs(3,6)
     +                                  ,alphad)+
     +                      lcv(7)*coor(i+lrs(1,7),j+lrs(2,7),k+lrs(3,7)
     +                                  ,alphad)+
     +                      lcv(8)*coor(i+lrs(1,8),j+lrs(2,8),k+lrs(3,8)
     +                                  ,alphad)
                        end do
                     end do
                  end do


c                 --- Boundary cells (planes )

                  if ( mod(pnttyp,2).eq.0 ) then

c                 --- point types 2, 4, 6 and 8

                     i = 1-nvirtual
                     do k = 2-nvirtual, nk+nvirtual
                        do j = 2-nvirtual, nj+nvirtual
                           cva(i,j,k,alphad,betad,pnttyp) =
     +                      lcv(1)*coor(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1)
     +                                  ,alphad)+
     +                      lcv(2)*coor(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)
     +                                  ,alphad)+
     +                      lcv(3)*coor(i+lrs(1,3),j+lrs(2,3),k+lrs(3,3)
     +                                  ,alphad)+
     +                      lcv(4)*coor(i+lrs(1,4),j+lrs(2,4),k+lrs(3,4)
     +                                  ,alphad)+
     +                      lcv(5)*coor(i+lrs(1,5),j+lrs(2,5),k+lrs(3,5)
     +                                  ,alphad)+
     +                      lcv(6)*coor(i+lrs(1,6),j+lrs(2,6),k+lrs(3,6)
     +                                  ,alphad)+
     +                      lcv(7)*coor(i+lrs(1,7),j+lrs(2,7),k+lrs(3,7)
     +                                  ,alphad)+
     +                      lcv(8)*coor(i+lrs(1,8),j+lrs(2,8),k+lrs(3,8)
     +                                  ,alphad)

                        end do
                     end do

                  end if

                  if ( pnttyp.eq.3 .or. pnttyp.eq.4 .or.
     +                  pnttyp.eq.7 .or. pnttyp.eq.8) then

c                 --- point types 3, 4, 7 and 8

                     j = 1-nvirtual
                     do k = 2-nvirtual, nk+nvirtual
                        do i = 2-nvirtual, ni+nvirtual
                           cva(i,j,k,alphad,betad,pnttyp) =
     +                      lcv(1)*coor(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1)
     +                                  ,alphad)+
     +                      lcv(2)*coor(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)
     +                                  ,alphad)+
     +                      lcv(3)*coor(i+lrs(1,3),j+lrs(2,3),k+lrs(3,3)
     +                                  ,alphad)+
     +                      lcv(4)*coor(i+lrs(1,4),j+lrs(2,4),k+lrs(3,4)
     +                                  ,alphad)+
     +                      lcv(5)*coor(i+lrs(1,5),j+lrs(2,5),k+lrs(3,5)
     +                                  ,alphad)+
     +                      lcv(6)*coor(i+lrs(1,6),j+lrs(2,6),k+lrs(3,6)
     +                                  ,alphad)+
     +                      lcv(7)*coor(i+lrs(1,7),j+lrs(2,7),k+lrs(3,7)
     +                                  ,alphad)+
     +                      lcv(8)*coor(i+lrs(1,8),j+lrs(2,8),k+lrs(3,8)
     +                                  ,alphad)

                        end do
                     end do

                  end if

                  if ( pnttyp.ge.5 ) then

c                 --- point types 5, 6, 7 and 8

                     k = 1-nvirtual
                     do j = 2-nvirtual, nj+nvirtual
                        do i = 2-nvirtual, ni+nvirtual
                           cva(i,j,k,alphad,betad,pnttyp) =
     +                      lcv(1)*coor(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1)
     +                                  ,alphad)+
     +                      lcv(2)*coor(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)
     +                                  ,alphad)+
     +                      lcv(3)*coor(i+lrs(1,3),j+lrs(2,3),k+lrs(3,3)
     +                                  ,alphad)+
     +                      lcv(4)*coor(i+lrs(1,4),j+lrs(2,4),k+lrs(3,4)
     +                                  ,alphad)+
     +                      lcv(5)*coor(i+lrs(1,5),j+lrs(2,5),k+lrs(3,5)
     +                                  ,alphad)+
     +                      lcv(6)*coor(i+lrs(1,6),j+lrs(2,6),k+lrs(3,6)
     +                                  ,alphad)+
     +                      lcv(7)*coor(i+lrs(1,7),j+lrs(2,7),k+lrs(3,7)
     +                                  ,alphad)+
     +                      lcv(8)*coor(i+lrs(1,8),j+lrs(2,8),k+lrs(3,8)
     +                                  ,alphad)

                        end do
                     end do

                  end if


c                 --- Boundary cells ( edges )

                  if ( pnttyp.ge.7 ) then

c                 --- point types  7 and 8

                     k = 1-nvirtual
                     j = 1-nvirtual
                        do i = 2-nvirtual, ni+nvirtual
                           cva(i,j,k,alphad,betad,pnttyp) =
     +                      lcv(1)*coor(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1)
     +                                  ,alphad)+
     +                      lcv(2)*coor(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)
     +                                  ,alphad)+
     +                      lcv(3)*coor(i+lrs(1,3),j+lrs(2,3),k+lrs(3,3)
     +                                  ,alphad)+
     +                      lcv(4)*coor(i+lrs(1,4),j+lrs(2,4),k+lrs(3,4)
     +                                  ,alphad)+
     +                      lcv(5)*coor(i+lrs(1,5),j+lrs(2,5),k+lrs(3,5)
     +                                  ,alphad)+
     +                      lcv(6)*coor(i+lrs(1,6),j+lrs(2,6),k+lrs(3,6)
     +                                  ,alphad)+
     +                      lcv(7)*coor(i+lrs(1,7),j+lrs(2,7),k+lrs(3,7)
     +                                  ,alphad)+
     +                      lcv(8)*coor(i+lrs(1,8),j+lrs(2,8),k+lrs(3,8)
     +                                  ,alphad)

                        end do

                  end if

                  if ( pnttyp.eq.6 .or. pnttyp.eq.8 ) then

c                 --- point types  6 and 8

                     k = 1-nvirtual
                     i = 1-nvirtual
                        do j = 2-nvirtual, nj+nvirtual
                           cva(i,j,k,alphad,betad,pnttyp) =
     +                      lcv(1)*coor(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1)
     +                                  ,alphad)+
     +                      lcv(2)*coor(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)
     +                                  ,alphad)+
     +                      lcv(3)*coor(i+lrs(1,3),j+lrs(2,3),k+lrs(3,3)
     +                                  ,alphad)+
     +                      lcv(4)*coor(i+lrs(1,4),j+lrs(2,4),k+lrs(3,4)
     +                                  ,alphad)+
     +                      lcv(5)*coor(i+lrs(1,5),j+lrs(2,5),k+lrs(3,5)
     +                                  ,alphad)+
     +                      lcv(6)*coor(i+lrs(1,6),j+lrs(2,6),k+lrs(3,6)
     +                                  ,alphad)+
     +                      lcv(7)*coor(i+lrs(1,7),j+lrs(2,7),k+lrs(3,7)
     +                                  ,alphad)+
     +                      lcv(8)*coor(i+lrs(1,8),j+lrs(2,8),k+lrs(3,8)
     +                                  ,alphad)

                        end do

                  end if


                  if ( pnttyp.eq.4 .or. pnttyp.eq.8 ) then

c                 --- point types  4 and 8

                     i = 1-nvirtual
                     j = 1-nvirtual
                        do k = 2-nvirtual, nk+nvirtual
                           cva(i,j,k,alphad,betad,pnttyp) =
     +                      lcv(1)*coor(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1)
     +                                  ,alphad)+
     +                      lcv(2)*coor(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)
     +                                  ,alphad)+
     +                      lcv(3)*coor(i+lrs(1,3),j+lrs(2,3),k+lrs(3,3)
     +                                  ,alphad)+
     +                      lcv(4)*coor(i+lrs(1,4),j+lrs(2,4),k+lrs(3,4)
     +                                  ,alphad)+
     +                      lcv(5)*coor(i+lrs(1,5),j+lrs(2,5),k+lrs(3,5)
     +                                  ,alphad)+
     +                      lcv(6)*coor(i+lrs(1,6),j+lrs(2,6),k+lrs(3,6)
     +                                  ,alphad)+
     +                      lcv(7)*coor(i+lrs(1,7),j+lrs(2,7),k+lrs(3,7)
     +                                  ,alphad)+
     +                      lcv(8)*coor(i+lrs(1,8),j+lrs(2,8),k+lrs(3,8)
     +                                  ,alphad)

                        end do

                  end if

                  if ( pnttyp.eq.8  ) then

c                 --- point type  8

                     i = 1-nvirtual
                     j = 1-nvirtual
                     k = 1-nvirtual
                           cva(i,j,k,alphad,betad,pnttyp) =
     +                      lcv(1)*coor(i+lrs(1,1),j+lrs(2,1),k+lrs(3,1)
     +                                  ,alphad)+
     +                      lcv(2)*coor(i+lrs(1,2),j+lrs(2,2),k+lrs(3,2)
     +                                  ,alphad)+
     +                      lcv(3)*coor(i+lrs(1,3),j+lrs(2,3),k+lrs(3,3)
     +                                  ,alphad)+
     +                      lcv(4)*coor(i+lrs(1,4),j+lrs(2,4),k+lrs(3,4)
     +                                  ,alphad)+
     +                      lcv(5)*coor(i+lrs(1,5),j+lrs(2,5),k+lrs(3,5)
     +                                  ,alphad)+
     +                      lcv(6)*coor(i+lrs(1,6),j+lrs(2,6),k+lrs(3,6)
     +                                  ,alphad)+
     +                      lcv(7)*coor(i+lrs(1,7),j+lrs(2,7),k+lrs(3,7)
     +                                  ,alphad)+
     +                      lcv(8)*coor(i+lrs(1,8),j+lrs(2,8),k+lrs(3,8)
     +                                  ,alphad)
                  end if


               end if

            end do

         end do

      end do


      end
