      subroutine iscrhsd ( ni, nj, nvirtual, rhside, vsum,
     +                      veloc, dcell, iarea )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.0    date 04-11-1996 Extracted from subroutine isdstndf
c                                       of Ronald van Nooyen
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Righthandside part of newton linearized convection term
c
c **********************************************************************
c
c                       KEYWORDS
c
c     discretization
c     right_hand_side
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, dcell(2), iarea(2,2)
      double precision rhside(*),
     +                 vsum((ni+1+2*nvirtual)*(nj+1+2*nvirtual),*),
     +                 veloc((ni+1+2*nvirtual)*(nj+1+2*nvirtual),*)

c     dcell     i    reserved
c     iarea     i    Area of arrays
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside   i/o   reserved
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c     vsum      i    Flux contributions to the matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer  kdir

c     kdir      General loop variable for 1...ndim loop
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISAXYZ2D  Perform a vector*vector update for ISNaS over a
c               2-dimensional region
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    For each velocity component in the integration point do
c          f := f + vsum * u / 2
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      do kdir = 1, 2

c     --- Newton linearization
c         The right-hand side is updated by a factor
c         update_matrix * velocity / 2
c         The right-hand side vector is situated in the
c         centres of the integration cells
c         The update matrix is positioned in the
c         integration point. So the multiplication must
c         be caried out in the integration points
c         As a consequence both the velocity vactor and
c         the matrix must be shifted with dcell

         call isaxyz2d ( ni, nj, nvirtual, dcell(1), dcell(2),
     +                   dcell(1), dcell(2), rhside, 0.5d0,
     +                   vsum(1,kdir), veloc(1,kdir), iarea )

      end do

      end
