      subroutine isdrhodp ( ndim, ni, nj, nk, nvirtual, ndegfd, rinput,
     +                      iinput, hold, press, drhodp, dpold,
     +                      eddy, solut )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 29-06-1999
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Compute drhodp and store in array drhodp
c    drhodp is used in the pressure correction algorithm
c **********************************************************************
c
c                       KEYWORDS
c
c    2d
c    matrix
c    pressure_equation
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ndim, ni, nj, nk, nvirtual, ndegfd, iinput(*)
      double precision hold(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual),
     +                 rinput(*), solut(*),
     +                 press(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual),
     +                 eddy(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual),
     +                 drhodp(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual),
     +                 dpold(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual)

c     dpold          i    pressure correction at previous time step
c     drhodp         o    Contains drho/dp in all centres of the cells
c     eddy           i    array EDDY contains the values of the
c                         eddy viscosity of that block in the centroids
c                         of the elements. The storage of such
c                         an array EDDY is identical to that of all other
c                         coefficients.
c     hold           i    Enthalpy at previous time step
c     iinput         i    Contains the integer user input for one block.
c                         For a description see the programmers guide
c     ndegfd         i    Number of degrees of freedom
c     ndim           i    Dimension of space (2 or 3)
c     ni             i    Number of non-virtual cells in the i-direction
c                         (one block)
c     nj             i    Number of non-virtual cells in the j-direction
c                         (one block)
c     nk             i    Number of non-virtual cells in the k-direction
c                         (one block)
c     nvirtual       i    Number of rows of virtual cells
c     press          i    pressure
c     rinput         i    The real user input is stored in the array rinput
c                         for each block
c                         For a description see the programmers guide
c     solut          i    Solution array
c                         For a description see the programmers guide
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, iprincom, mscal, mcav, ipiincav, ipiincom,
     +        istartup, npointsv, igauss, iprincav, meqstate
      double precision gamcm, mach2, cons, cons1,
     +                 epssc

c     cons           Constant used in equation of state for compressible
c                    computations
c     cons1          Constant used in equation of state for compressible
c                    computations
c     epssc          Parameter in the secant type estimate for the drhodp
c                    derivative
c     gamcm          Gas constant gamma (for compressible comput.)
c     i              General loop variable
c     igauss         if igauss=1 non linear Gauss Seidel is applied
c                    else not
c     ipiincav       Starting address of array IINCAV in array IINPUT
c     ipiincom       Starting address of array IINCOM in array IINPUT
c     iprincav       Starting address of array RINCAV in array RINPUT
c     iprincom       Starting address of array RINCOM in array RINPUT
c     istartup       If 1 the cavity calculation is in the startup
c                    procedure, if 0 not
c     j              General loop variable
c     mach2          Square of the mach number (for compressible comput.)
c     mcav           If 1 a cavity model is used, if 0 not
c     meqstate       Defines the type of equation of state
c                    See the programmers Guide array IINCOM for the
c                    possible values
c     mscal          Indicates use of non-dimensionalisation for
c                    compr. flow:
c                    0:   Equations are non-dimensionalised
c                    1:   Equations are in dimensional formulation
c     npointsv       Number of points in the mesh with virtual points
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      double precision isstdfl, isstdls

c     DAXPY          Computes a vector + a constant * other vector. This
c                    is a BLAS routine.
c     ISDSTFL2       Derivative of dimensional equation of state, vector form
c     ISDSTLS2       Derivative of dimensionless equation of state, vector form
c     ISSTDFL        dimensionfull equation of state
c     ISSTDFL2       dimensionfull equation of state vector form
c     ISSTDLS        dimensionless equation of state
c     ISSTDLS2       dimensionless equation of state vector form
c     USDRHODP       User subroutine to compute drho/dp
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    drho/dp follows immediately from the equation of state
c    For a perfect gas we have:
c
c    rho = gamma/(gamma-1) p/h, hence drho/dp = gamma/((gamma-1)h)
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      mcav      = iinput(77)
      ipiincom  = iinput(9)
      mscal     = iinput(ipiincom+5 - 1)
      meqstate  = iinput(ipiincom+9 - 1)
      npointsv  = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)

      if(mcav.eq.1) then
        ipiincav =iinput(11)
        igauss   =iinput(ipiincav - 1 + 1)
        istartup =iinput(ipiincav - 1 + 2)
      else
        igauss   = 0
        istartup = 0
      end if

      iprincom = iinput(34)-1
      iprincav = iinput(35)

      if ( meqstate.eq.0 ) then

         gamcm = rinput(iprincom+1)
         mach2 = rinput(iprincom+2)**2d0
         epssc   = 1d-4

         cons = 1d0 + ((gamcm - 1d0)*mach2)/2d0

      end if
      if ( mscal.eq.0 ) then

c     --- mscal = 0, scaled equations (hence dimensionless)

         cons1 = (gamcm * mach2) / cons

      else if ( meqstate.eq.0 ) then

c     --- mscal = 0, no scaling, dimensionfull

         cons1 = gamcm/(gamcm-1d0)

      end if

      if ( mcav.eq.1 ) then

c     --- mcav = 1, standard algorithm for cavity

         if( igauss.eq.0 ) then

c        --- igauss = 0, standard method

            if ( mscal.eq.0 ) then

c           --- mscal = 0, the equations are scaled in order to get a
c                          dimensionless form

               call isdstls2 ( press, rinput(iprincom), istartup,
     +                         npointsv, drhodp, rinput(iprincav) )

            else if ( mscal.eq.1 ) then

c           --- mscal = 1, no scaling applied, hence we have the
c                          dimension full form

               call isdstfl2 ( press, npointsv, istartup, drhodp,
     +                         rinput(iprincav) )

            end if

         else if( igauss.eq.1) then

c        --- igauss = 1, non linear Gauss-Seidel algorithm, update matrix
c            with compressible part only used in the multiblock proces

            call daxpy(npointsv,1d0, dpold, 1, press, 1)

            if ( mscal.eq.0 ) then

c           --- mscal = 0, the equations are scaled in order to get a
c                          dimensionless form

               call isstdls2 ( press, rinput(iprincom), istartup,
     +                         drhodp, npointsv, rinput(iprincav) )

            else if( mscal.eq.1) then

c           --- mscal = 1, no scaling applied, hence we have the
c                          dimension full form

               call isstdfl2 ( press, istartup, drhodp, npointsv,
     +                         rinput(iprincav) )

            end if

            call daxpy(npointsv,-1d0, dpold, 1, press, 1)
            call daxpy(npointsv,-1d0, eddy, 1, drhodp, 1)

            if ( mscal.eq.0 ) then

c           --- dimensionless computation

               do j = 1-nvirtual, nj+1+nvirtual
                  do i = 1-nvirtual, ni+1+nvirtual

                     if ( abs(dpold(i,j)).gt.epssc ) then

                        drhodp(i,j)= drhodp(i,j)/dpold(i,j)

                     else

                        drhodp(i,j)=
     +                         (isstdls(press(i,j)+epssc,
     +                             rinput(iprincom), istartup,
     +                                        rinput(iprincav))-
     +                             eddy(i,j))/(epssc)
                     end if
                  end do
               end do

            else if ( mscal.eq.1 ) then

c           --- dimensional computation

               do j = 1-nvirtual, nj+1+nvirtual
                  do i = 1-nvirtual, ni+1+nvirtual

                     if ( abs(dpold(i,j)).gt.epssc ) then

                        drhodp(i,j)= drhodp(i,j)/dpold(i,j)

                     else

                        drhodp(i,j)=
     +                         (isstdfl(press(i,j)+epssc,
     +                             istartup, rinput(iprincav))-
     +                             eddy(i,j))/
     +                           (epssc)
                     end if
                 end do
               end do

            end if

         end if

      else

c     --- mcav = 0, no cavity

         if ( meqstate.eq.0 ) then

c        --- meqstate = 0, perfect gas: rho = gamma/(gamma-1) / p

            do j = 1-nvirtual, nj+1+nvirtual
               do i = 1-nvirtual, ni+1+nvirtual

                  drhodp(i,j)= cons1/hold(i,j)

              end do
            end do

         else

c        --- meqstate = 1, user defined equation of state
c            Call subroutine to compute drho/dp

            call usdrhodp ( eddy, solut, ni, nj, nk, nvirtual, ndim,
     +                      ndegfd, drhodp )

         end if

      end if

      end
