      subroutine isexbo22(iblock, iface, itrans, ni, nj, nvirtual,
     +                    ndegfd, itimlv, ntimlv, nblocks, nfaces,
     +                    maxnsubf, solut, nsubfaces, subfaces,
     +                    ibinfor, ibipoint, bcgeo, bndcon )
c ======================================================================
c
c        programmer   Kees Vuik
c
c        version 1.1  date 29-10-1998  Duncan: minor layout changes
c        version 1.0  date 16-10-1995
c
c   copyright (c) 1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Initialize virtual unknowns in 2D for theta = 0. Some boundary
c     conditions are taken into account.
c **********************************************************************
c
c                       KEYWORDS
c
c   mapping_array
c   topology
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, itrans, maxnsubf, nblocks, ndegfd, nfaces,
     +        ni, nj, nvirtual, itimlv, ntimlv, iface,
     +        ibipoint(*), ibinfor(*)
      double precision solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ndegfd, 1:ntimlv),
     +                       bcgeo(8,*), bndcon(2,*)
      integer nsubfaces(1:nblocks, 1:nfaces)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     bcgeo     i    reserved
c     bndcon    i    reserved
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     iblock    i    Block sequence number
c     iface     i    Face number
c     itimlv    i    Loop variable with respect to a given time level
c     itrans    i    reserved
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     ndegfd    i    Number of degrees of freedom
c     nfaces    i    Number of faces
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     ntimlv    i    Number of time levels
c     nvirtual  i    Number of rows of virtual cells
c     solut     i    reserved
c     subfaces  i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer          i, ihigh, ilow, isubface, j, k,
     +                 ifi, jfi, klow, khigh, ndir, nsign,
     +                 tdir, tsign, ipibinf, ishift, jshift

c     i         General loop variable
c     ifi       Starting value of general loop variable i
c     ihigh     Upper index of subface
c     ilow      Lower index of subface
c     ipibinf   Starting address of array infor for present side minus one
c     ishift    Shift in i-direction
c     isubface  Subface number
c     j         General loop variable
c     jfi       Starting value of general loop variable j
c     jshift    Shift in j-direction
c     k         General loop variable
c     khigh     Upper index of unknowns
c     klow      Lower index of unknowns
c     ndir      Direction of normal vector on this side
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     tdir      Direction of tangent vector on this side
c     tsign     Inner product of tangential vector defined counter
c               clockwise with respect to the outward normal and the
c               base vector along the side
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISEXBO23  Computes the tangential velocities and extrapolates them
c               to the virtual points
c     ISORIENT2 Computes ndir, nsign, tdir, tsign for a 2-dimensional
c               problem
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c        If (subface isubface is external ) then
c
c          Determine index range [ilow,ihigh] along the subface at which
c          to compute virtual coordinates by extrapolation.
c
c          Do i = ilow, ihigh
c
c             extrapolate nvirtual rows of virtual coordinates at
c             index i
c
c          Enddo
c
c        Endif
c
c     Endfor
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- determine index range for unknowns

      if ( itrans.eq.1 ) then

c     --- assume solut, tangential fluxes across the boundary,
c         followed by normal fluxes across the boundary

         klow = 1
         khigh = 2

      else

         klow = itrans
         khigh = itrans

      end if

c     --- Compute tdir and ndir

      call isorient2 ( iface, tdir, ndir, tsign, nsign )

      ipibinf = ibipoint(iface)+nvirtual-1

c     --- handle external subfaces

      do isubface = 1, nsubfaces(iblock, iface)
         if ( subfaces(iblock, iface, isubface, 5).eq.0 ) then

               do k = klow, khigh

c              --- determine index range of uninitialized unknowns

                  ilow = subfaces(iblock,iface,isubface,1)
                  ihigh = subfaces(iblock,iface,isubface,2)

                  if ( k.eq.tdir ) then

c                 --- tangential flux component

                     ihigh = ihigh + 1

c                    --- if the previous subface was internal then exclude
c                        index ilow

                     if ( isubface.gt.1 ) then
                        if ( subfaces(iblock,iface,isubface-1,5) .ne.
     +                        0 )
     +                     ilow = ilow + 1
                     end if

c                    --- if the next subface is internal then exclude
c                        index ihigh

                     if ( isubface.lt.nsubfaces(iblock,iface) ) then
                        if ( subfaces(iblock,iface,isubface+1,5) .ne.
     +                        0 )
     +                     ihigh = ihigh - 1
                     end if

                  end if

                  if ( iface.eq.ileft) then

                     do i = 0, 1-nvirtual, -1
                        do j = ilow, ihigh

                           if ( i.eq.0 .and. j.ne.1 .and.
     +                          j.ne.nj+1 .and. k.eq.tdir ) then

c                          --- adaptation for tangential velocities

                              ishift = 1
                              jshift = 0
                              call isexbo23 ( ni, nj, nvirtual, ndegfd,
     +                             itimlv, solut, ibinfor,
     +                             bcgeo, bndcon, iface, i, j, ishift,
     +                             jshift, ipibinf, tdir, ndir, tsign,
     +                             nsign )
                           else
                              solut(i,j,k,itimlv) =
     +                                  2d0*solut(i+1,j,k,itimlv) -
     +                                      solut(i+2,j,k,itimlv)
                           end if
                        end do
                     end do

                  else if ( iface.eq.iright) then

                     if ( k.eq.1 ) then
                        ifi = ni+2
                     else
                        ifi = ni+1
                     end if
                     do i = ifi, ni+1+nvirtual
                        do j = ilow, ihigh

                           if ( i.eq.ni+1 .and. j.ne.1 .and.
     +                          j.ne.nj+1 .and. k.eq.tdir ) then

c                          --- adaptation for tangential velocities

                              ishift = -1
                              jshift = 0
                              call isexbo23 ( ni, nj, nvirtual, ndegfd,
     +                             itimlv, solut, ibinfor,
     +                             bcgeo, bndcon, iface, i, j, ishift,
     +                             jshift, ipibinf, tdir, ndir, tsign,
     +                             nsign )
                           else
                              solut(i,j,k,itimlv) =
     +                                  2d0*solut(i-1,j,k,itimlv) -
     +                                      solut(i-2,j,k,itimlv)
                           end if
                        end do
                     end do

                  else if ( iface.eq.idown ) then

                     do j = 0, 1-nvirtual, -1
                        do i = ilow, ihigh

                           if ( j.eq.0 .and. i.ne.1 .and.
     +                          i.ne.ni+1 .and. k.eq.tdir ) then

c                          --- adaptation for tangential velocities

                              ishift = 0
                              jshift = 1
                              call isexbo23 ( ni, nj, nvirtual, ndegfd,
     +                             itimlv, solut, ibinfor,
     +                             bcgeo, bndcon, iface, i, j, ishift,
     +                             jshift, ipibinf, tdir, ndir, tsign,
     +                             nsign )
                           else
                              solut(i,j,k,itimlv) =
     +                                  2d0*solut(i,j+1,k,itimlv) -
     +                                      solut(i,j+2,k,itimlv)
                           end if
                        end do
                     end do

                  else

                     if ( k.eq.2 ) then
                        jfi = nj+2
                     else
                        jfi = nj+1
                     end if
                     do j = jfi, nj+1+nvirtual
                        do i = ilow, ihigh

                           if ( j.eq.nj+1 .and. i.ne.1 .and.
     +                          i.ne.ni+1 .and. k.eq.tdir ) then

c                          --- adaptation for tangential velocities

                              ishift = 0
                              jshift = -1
                              call isexbo23 ( ni, nj, nvirtual, ndegfd,
     +                             itimlv, solut, ibinfor,
     +                             bcgeo, bndcon, iface, i, j, ishift,
     +                             jshift, ipibinf, tdir, ndir, tsign,
     +                             nsign )
                           else
                              solut(i,j,k,itimlv) =
     +                                  2d0*solut(i,j-1,k,itimlv) -
     +                                      solut(i,j-2,k,itimlv)
                           end if
                        end do
                     end do

                  end if

               end do

         end if
      end do

      end
