      subroutine isexmom ( iinput, rinput, matrix, gmat ,rhside, coefs,
     +                     solut , coor  , infmm , infmg, geom , infbnd,
     +                     bndcon, eddy  , work  ,
     +                     iblock , itrans, itimlv, nblocks,
     +                     isiinput, issolut, isitopol )
c ======================================================================
c
c     programmer   Kees Vuik
c
c     version      1.5   date    29-10-1998 Duncan : introduction of
c                                                       isxeu2d
c     version      1.4   date    09-06-1998 Charles: nvpoints changed
c                                                    to npointsv
c     version      1.3   date    03-10-1996 (Kees K.: newdiscr changed )
c     version      1.2   date    12-02-1996 (Kees K.: calls to ismcc and
c                                          isfrhs removed)
c     version      1.1   date    30-10-1995 (Guus: new call to isexm2d)
c     version      1.0   date    14-09-1995
c
c
c   copyright (c) 1993-1995  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c
c
c **********************************************************************
c
c                       KEYWORDS
c
c     momentum_equation
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinput(*),
     +        infmm(*), infmg(*), infbnd(*),
     +        iblock, itrans, itimlv, nblocks, issolut,
     +        isiinput, isitopol
      double precision rinput(*), bndcon(*), matrix(*), gmat(*),
     +        rhside(*), coefs(*), solut(*), coor(*), geom(*),
     +        work(*), eddy(*)

c     bndcon    i    reserved
c     coefs     i    reserved
c     coor      i    reserved
c     eddy      i    reserved
c     geom      i    reserved
c     gmat      i    Gradient matrix
c     iblock    i    Block sequence number
c     iinput    i    reserved
c     infbnd    i    reserved
c     infmg     i    Integer information for Pressure-gradient matrix
c     infmm     i    Integer information for Momentum matrix
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isitopol  i    Sequence number of array itopol
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     itimlv    i    Loop variable with respect to a given time level
c     itrans    i    reserved
c     matrix    i    reserved
c     nblocks   i    Number of blocks
c     rhside    i    reserved
c     rinput    i    reserved
c     solut     i    reserved
c     work      i    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ipiindsc, ipiintim, iprinmt,
     +        itime1, itime2, itimeact, itimest,
     +        ndim, ni, nj, nk, nvirtual, npointsv,
     +        ipvsum, ipdvsum, ipwork, ipinvjac, ipmct
      logical newdiscr

c     ipdvsum   Starting address of array DVSUM in array WORK
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiintim  Starting address of array IINTIM in array IINPUT
c     ipinvjac  Starting address of array invjac in geom
c     ipmct     Pointer to mct in infmat
c     iprinmt   Indicates whether the matrix must be printed or not
c               Possible values
c               0:  the matrix is not printed
c               1:  the standard matrix is printed
c               2:  the gradient matrix is printed
c               3:  the standard and the gradient matrix are printed
c               4:  the pressure matrix is printed
c               5:  the standard and the pressure matrix are printed
c               6:  the gradient and the pressure matrix are printed
c               7:  the standard, thegradient and the pressure matrix
c                   are printed
c     ipvsum    Starting address of array VSUM in array WORK
c     ipwork    Starting address of array work in buffer
c     itime1    First time at which the debug prints must be performed
c     itime2    Last time at which the debug prints must be performed
c     itimeact  Actual time step
c     itimest   Time step with respect to debug printing
c     ndim      Dimension of space (2 or 3)
c     newdiscr  If true the new type of discretization is used
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npointsv  Number of points in the mesh with virtual points
c     nvirtual  Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRSUB  Error messages
c     ISEXM2D   Calculate explicit viscous momentum solution in 2D
c     ISPRIMT1  Print 2D matrix
c     ISPRIMT2  Print 3D matrix
c     ISXEU2D   Calculate explicit inviscid momentum solution in 2D
c **********************************************************************
c
c                       ERROR MESSAGES
c
c      26   Explicit solution of transport has not yet been implemented
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('isexmom')

      ni       = iinput(51)
      nj       = iinput(52)
      nk       = iinput(53)
      nvirtual = iinput(54)

      ndim     = iinput(57)
      ipiintim = iinput(4)

      ipiindsc = iinput(6)
      iprinmt  = iinput(ipiindsc+10)
      itime1   = iinput(ipiindsc+12)
      itime2   = iinput(ipiindsc+13)
      itimest  = iinput(ipiindsc+14)

      itimeact = iinput(ipiintim+3)

      npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
      if (ndim.eq.3)  npointsv = npointsv*(nk+1+2*nvirtual)

      ipiindsc=iinput(6)
      newdiscr = ndim.eq.2
      if ( newdiscr ) then

c     --- At this moment only the 2d time derivative part is programmed here

         ni       = iinput(51)
         nj       = iinput(52)
         nvirtual = iinput(54)
         npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
         ipvsum  = 1
         ipdvsum = ipvsum+2*npointsv
         ipwork  = ipdvsum+4*npointsv
         ipinvjac = npointsv*(2**ndim) + 1
         ipmct    = infmm(11)

         if ( iinput(ipiindsc-1+3).eq.0 ) then

c        --- Navier-Stokes for theta=0

            call isexm2d ( iinput, matrix, rhside, solut, coefs,
     +                  work(ipwork), rinput, npointsv,
     +                  infmm, work(ipvsum), work(ipdvsum),
     +                  geom(ipinvjac), coor, infmm(ipmct),
     +                  bndcon, infbnd(1),
     +                  infbnd(2*ndim*(1+2*(ndim-1))+1), geom,
     +                  eddy(npointsv+1), iblock , itrans, itimlv,
     +                  nblocks, isiinput, issolut, isitopol,
     +                  gmat, infmg )

         else

c        --- Euler for theta=0

            call isxeu2d ( iinput, rhside, solut, coefs,
     +                  work(ipwork), rinput, npointsv,
     +                  work(ipvsum),
     +                  geom(ipinvjac), coor, infmm(ipmct),
     +                  bndcon, infbnd(1),
     +                  infbnd(2*ndim*(1+2*(ndim-1))+1), geom,
     +                  eddy(npointsv+1), iblock , itrans, itimlv,
     +                  nblocks, isiinput, issolut, isitopol,
     +                  gmat, infmg )
        end if

      end if
      if ( nerror.ne.0 ) go to 1000

      if ( .not. newdiscr ) then
         call iserrsub ( 26, 0, 0, 0 )
         go to 1000
      end if

      if ( mod(iprinmt,4).ge.2 ) then

c     --- Print g matrix

         if ( itimeact.ge.itime1 .and. itimeact.le.itime2 .and.
     +        mod(itimeact-itime1,itimest).eq.0 ) then

            if ( ndim.eq.2 ) then
               call isprimt1 ( 'gradient matrix', gmat, ni, nj,
     +                         infmg(6), 3 )
            else
               call isprimt2 ( 'gradient matrix', gmat, ni, nj, nk,
     +                         infmg(6), 3 )
            end if
         end if
      end if

1000  call iserclos('isexmom')

      end
