      subroutine isgmat02 ( ni, nj, nvirtual, gmat, cva, invjac,
     +                      alphad )
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.0    date 20-09-95
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c ********************************************************************
c
c                         DESCRIPTION
c
c    Compute internal part of gradient matrix in R^2 for one direction
c    At this moment the discretization is restricted to standard ISNaS
c ********************************************************************
c
c                      KEYWORDS
c
c    grad_matrix
c    matrix
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, alphad
      double precision gmat(ni+1,nj+1,2,6),
     +                 cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2),
     +                 invjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     gmat      o    Gradient matrix
c     invjac    i    1/sqrt(g) in pnttyp
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c ********************************************************************
c
c                    COMMON BLOCKS
c
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer ip, jp, betad, i, iupploop, jupploop
      double precision fact

c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     fact      Multiplication factor
c     i         General loop variable
c     ip        Local counting variable i-direction
c     iupploop  Upper bound for loop
c     jp        Local counting variable j-direction
c     jupploop  Upper bound for loop
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c ********************************************************************
c
c                              I/O
c
c ********************************************************************
c
c                    ERROR MESSAGES
c
c ********************************************************************
c
c                    METHOD
c
c   The following discretization is used in the standard ISNaS case
c
c            alpha 1 |         |(1,0)                alpha 2 |         |(0,1)
c   sqrt(g) g        |        p|        +   sqrt(g) g        |        p|
c                    |(0,0)    |(-1,0)                       |(0,0)    |(0,-1)
c
c   The pressure is considered as a derivative in the 1 and 2 direction,
c   hence the coefficients may be evaluated in the cell centre
c
c                       alpha gamma
c   To compute sqrt(g) g             we use only the covariant base vectors:
c
c              1    2        2    1
c   sqrt(g) = a    a     -  a    a            g   = a    .  a
c              (1)  (2)      (1)  (2)          bc    (b)     (c)
c
c            alpha alpha      1       1       1         2       2
c   sqrt(g) g            = ------- ( a       a       + a       a      )
c                          sqrt(g)    (beta)  (beta)    (beta)  (beta)
c
c            alpha beta      -1       1        1         2        2
c   sqrt(g) g            = ------- ( a        a       + a        a      )
c                          sqrt(g)    (alpha)  (beta)    (alpha)  (beta)
c
c
c
c                    PSEUDO CODE
c
c
c    First clear part corresponding to virtual unknowns
c    Loop over Va unknown  (a=alpha)
c       Compute factor before (1,0) and (-1,0) (a=1) or (0,1) and (0,-1)
c       Store factor in gmat(a,4) and -gmat(a,3)
c       Compute factor before (0,1) and (0,-1) (a=1) or (1,0) and (-1,0)
c       and multiply by .25
c       Store factor in gmat(a,5), gmat(a,6) and -gmat(a,1), -gmat(a,2)
c ********************************************************************
c
c                    DATA STATEMENTS
c
c ======================================================================
c
c     --- First clear part corresponding to virtual unknowns

      if ( alphad.eq.1 ) then

c     --- V1-cell

         do ip = 1, ni+1
            do i = 1, 6
               gmat(ip,nj+1,alphad,i) = 0d0
            end do
         end do
         iupploop = ni+1
         jupploop = nj
         betad = 2

      else

c     --- V2-cell

         do jp = 1, nj+1
            do i = 1, 6
               gmat(ni+1,jp,alphad,i) = 0d0
            end do
         end do
         iupploop = ni
         jupploop = nj+1
         betad = 1

      end if

c     Actually filling of gmat

      do ip = 1, iupploop
         do jp = 1, jupploop
            fact = invjac(ip,jp) * (cva(ip,jp,1,betad)**2+
     +                              cva(ip,jp,2,betad)**2)
            gmat(ip,jp,alphad,3) = -fact
            gmat(ip,jp,alphad,4) =  fact
            fact = -invjac(ip,jp) * (cva(ip,jp,1,alphad)*
     +                               cva(ip,jp,1,betad)+
     +                               cva(ip,jp,2,alphad)*
     +                               cva(ip,jp,2,betad) )*0.25d0
            gmat(ip,jp,alphad,1) = -fact
            gmat(ip,jp,alphad,2) = -fact
            gmat(ip,jp,alphad,5) =  fact
            gmat(ip,jp,alphad,6) =  fact

         end do

      end do

      end
