      subroutine isgmat03 ( ni, nj, nk, nvirtual, gmat, cva, invjac,
     +                      alphad )
c ======================================================================
c
c
c        programmer     Kees Kassels
c        version 1.0    date 11-01-96
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c ********************************************************************
c
c                         DESCRIPTION
c
c    Compute internal part of gradient matrix in R^3 for one direction
c    At this moment the discretization is restricted to standard ISNaS
c ********************************************************************
c
c                      KEYWORDS
c
c    grad_matrix
c    matrix
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, alphad
      double precision gmat(ni+1,nj+1,nk+1,3,10),
     +                 cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,
     +                     1-nvirtual:nk+1+nvirtual,3,3),
     +                 invjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     gmat      o    Gradient matrix
c     invjac    i    1/sqrt(g) in pnttyp
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c ********************************************************************
c
c                    COMMON BLOCKS
c
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer ip, jp, kp, betad, gammad, i, iupploop, jupploop,
     +         kupploop, alphal, betal, valpha(3), vbeta(3)
      double precision ctva(3,3), fact

c     alphal    Loop index
c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     betal     Loop index
c     ctva      Contravariant base vectors times jacobian
c     fact      Multiplication factor
c     gammad    Direction index
c     i         General loop variable
c     ip        Local counting variable i-direction
c     iupploop  Upper bound for loop
c     jp        Local counting variable j-direction
c     jupploop  Upper bound for loop
c     kp        Local counting variable k-direction
c     kupploop  Upper bound for loop
c     valpha    Array for cyclic changing ordering
c               alphal = 1       alphal = 2   alphal = 3
c               valpha(1) =1 ---> valpha(1)=2  valpha(1)=3
c               valpha(2) =2 ---> valpha(2)=3  valpha(2)=1
c               valpha(3) =3 ---> valpha(3)=1  valpha(3)=2
c     vbeta     Array for cyclic changing ordering
c               betal = 1        betal = 2    betal = 3
c               vbeta(1) =1 ---> vbeta(1)=2  vbeta(1)=3
c               vbeta(2) =2 ---> vbeta(2)=3  vbeta(2)=1
c               vbeta(3) =3 ---> vbeta(3)=1  vbeta(3)=2
c
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c ********************************************************************
c
c                              I/O
c
c ********************************************************************
c
c                    ERROR MESSAGES
c
c ********************************************************************
c
c                    METHOD
c
c   The following discretization is used in the standard ISNaS case
c
c            alpha 1 |         |(1,0,0)              alpha 2 |         |(0,1,0)
c   sqrt(g) g        |        p|        +   sqrt(g) g        |        p|
c                    |(0,0,0)  |(-1,0,0)                     |(0,0,0)  |(0,-1,0)
c
c             alpha 3 |         |(0,0,1)
c  + sqrt(g) g        |        p|
c                     |(0,0,0)  |(0,0,-1)
c
c   The pressure is considered as a derivative in the 1,2 and 3 direction,
c   hence the coefficients may be evaluated in the cell centre
c
c                       alpha gamma
c   To compute sqrt(g) g             we use only the covariant base vectors:
c
c    (alpha)    1
c    a       = ----  .   a      ^ a          for alpha, beta, gamma
c             sqrt(g)     (beta)    (gamma)       cyclic
c
c    so :
c
c             (alpha)
c    sqrt(g) a        =    a      ^ a          for alpha, beta, gamma
c                           (beta)    (gamma)       cyclic
c
c
c               alpha,beta     1                 alpha             beta
c    sqrt(g) x g           = ------  ( sqrt(g)  a      . sqrt(g) a      )
c                            sqrt(g)
c
c
c ********************************************************************
c
c                    DATA STATEMENTS
c
c ======================================================================
c
c
c     --- initialize gmat

      do kp = 1,nk+1
         do jp = 1, nj+1
            do ip = 1, ni+1
               do i=1,10
                  gmat(ip,jp,kp,alphad,i) = 0d0
               end do
            end do
         end do
      end do

      if ( alphad.eq.1 ) then

c     --- V1-cell

         iupploop = ni+1
         jupploop = nj
         kupploop = nk
         betad  = 2
         gammad = 3

      else if ( alphad.eq.2 ) then

c     --- V2-cell

         iupploop = ni
         jupploop = nj+1
         kupploop = nk

         betad  = 3
         gammad = 1

      else

c     --- V3-cell


         iupploop = ni
         jupploop = nj
         kupploop = nk+1

         betad  = 1
         gammad = 2

      end if

c     Actually filling of gmat

      do ip = 1, iupploop
         do jp = 1, jupploop
            do kp = 1, kupploop

               do alphal = 1, 3
                  do betal = 1, 3
c
c                    --- cyclic
c
c                    alphal = 1       alphal = 2   alpha = 3
c
c                    valpha(1) =1 ---> valpha(1)=2  valpha(1)=3
c                    valpha(2) =2 ---> valpha(2)=3  valpha(2)=1
c                    valpha(3) =3 ---> valpha(3)=1  valpha(3)=2

                     valpha(1) = alphal
                     valpha(2) = mod(alphal  ,3) + 1
                     valpha(3) = mod(alphal+1,3) + 1

                     vbeta(1)  = betal
                     vbeta(2)  = mod(betal  ,3) + 1
                     vbeta(3)  = mod(betal+1,3) + 1

                     ctva(alphal,betal) =
     +                  cva(ip,jp,kp,vbeta(2),valpha(2))*
     +                  cva(ip,jp,kp,vbeta(3),valpha(3))-
     +                  cva(ip,jp,kp,vbeta(3),valpha(2))*
     +                  cva(ip,jp,kp,vbeta(2),valpha(3))

                  end do
               end do

               fact = -25d-2 *
     +                invjac(ip,jp,kp)*(ctva(alphad,1)*ctva(betad,1) +
     +                                  ctva(alphad,2)*ctva(betad,2) +
     +                                  ctva(alphad,3)*ctva(betad,3))

               gmat(ip,jp,kp,alphad,1) = fact
               gmat(ip,jp,kp,alphad,2) = fact
               gmat(ip,jp,kp,alphad,5) = -fact
               gmat(ip,jp,kp,alphad,6) = -fact

               fact =-invjac(ip,jp,kp)*(ctva(alphad,1)*ctva(alphad,1) +
     +                                  ctva(alphad,2)*ctva(alphad,2) +
     +                                  ctva(alphad,3)*ctva(alphad,3))


               gmat(ip,jp,kp,alphad,3) = fact
               gmat(ip,jp,kp,alphad,4) = -fact

               fact = -25d-2 *
     +                invjac(ip,jp,kp)*(ctva(alphad,1)*ctva(gammad,1) +
     +                                  ctva(alphad,2)*ctva(gammad,2) +
     +                                  ctva(alphad,3)*ctva(gammad,3))

               gmat(ip,jp,kp,alphad,7)  = fact
               gmat(ip,jp,kp,alphad,8)  = fact
               gmat(ip,jp,kp,alphad,9)  = -fact
               gmat(ip,jp,kp,alphad,10) = -fact

            end do
         end do
      end do

      end
