      subroutine isgmat22 ( ni, nj, nvirtual, gmat, ibipoint, ibinfor )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.4    date 03-12-1998 Guus: remove iprec
c        version 1.3    date 19-11-1997 Kees K: change for periodic
c        version 1.2    date 20-03-1996 Guus: allow btype=0
c        version 1.1    date 16-11-1995 Guus: correction for sides, which
c                                       are partly multiblock
c        version 1.0    date 20-09-1995
c
c   copyright (c) 1995-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Folding subroutine for the gradient matrix in 2D
c
c **********************************************************************
c
c                       KEYWORDS
c
c    grad_matrix
c    matrix
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, ibipoint(*), ibinfor(*)
      double precision gmat(ni+1,nj+1,2,6)

c     gmat     i/o   Gradient matrix
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer snnut
      parameter ( snnut = 6 )
      integer ip, jp, iparea(2,2), mxp(2), iside, ipibinf,
     +        nsign, btype, tdir, ndir, inside, btypenb,
     +        i1shift, i2shift, j1shift, j2shift, ih
      logical skip
      double precision fact

c     btype     Type of boundary condition
c     btypenb   Type of boundary condition of neighbour side
c     fact      Multiplication factor
c     i1shift   Shift from tangential velocity to normal velocity in
c               i-direction in case of outside boundary followed by
c               multiblock boundary
c     i2shift   Shift from tangential velocity to normal velocity in
c               i-direction in case of multiblock boundary followed by
c               outside boundary
c     ih        Help variable to store a constant temporarily
c     inside    Position of connection corresponding to point inside the inner
c               region opposite the boundary point
c     ip        Local counting variable i-direction
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     ipibinf   Starting address of array infor for present side minus one
c     iside     Sequence number of side
c     j1shift   Shift from tangential velocity to normal velocity in
c               j-direction in case of outside boundary followed by
c               multiblock boundary
c     j2shift   Shift from tangential velocity to normal velocity in
c               j-direction in case of multiblock boundary followed by
c               outside boundary
c     jp        Local counting variable j-direction
c     mxp       Auxiliary array to calculate boundary point number
c     ndir      Direction of normal vector on this side
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     skip      If true boundary condition must be skipped
c     snnut     Sigma_nn and u_t prescribed
c     tdir      Direction of tangent vector on this side
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   The parts of the stencil outside the region are folded inwards.
c   Since all normal velocity half cells have been treated in isgmat12
c   it is sufficient to restrict ourselves to the tangential
c   boundary cells
c   Periodical and multiblock boundaries are skipped
c   Virtual pressures are folded insidely in subroutine isgmatf2
c
c   Consider a tangential cell at the lower boundary, i.e.
c       ntdir=(2,1)
c       ntsig=(-1,1)
c
c
c                +             +
c
c         ---------------+--------------------------
c         |              |            |            |
c         |      |       |     |      |            |
c         |              |            |            |
c         |      +       *     +      |            |
c         |              |            |            |
c         |      |       |     |      |            |
c         |              |            |            |
c         ---------------+--------------------------
c
c                +             +
c
c
c    The two outside pressures must be folded inwardly by linear extrapolation
c
c   Consider a normal cell at the lower boundary, i.e.
c       ntdir=(2,1)
c       ntsig=(-1,1)
c
c
c         |
c         |
c         ---------------+--------------------------
c         |              |            |            |
c         |              |            |            |
c         |              |            |            |
c   +     |      +       |     +      |            |
c         |              |            |            |
c         |              |            |            |
c         |              |            |            |
c         -------*-------+--------------------------
c         |
c         |
c
c   +            +             +
c
c
c    The three outside pressures below the boundary have already been removed
c    in isgmat12
c    At the corners of the region, however, the outside pressure must
c    be folded inwardly
c    It concerns the pressure above the boundary (in the pricture)
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Loop over all sides

      do iside = 1, 4

         ipibinf = ibipoint(iside)-1+nvirtual

         iparea(1,1) = 1
         iparea(2,1) = 1
         iparea(1,2) = 1
         iparea(2,2) = 1
         i1shift = 0
         j1shift = 0
         i2shift = 0
         j2shift = 0
         if ( iside.le.2 ) then

c        --- iside = 1, 2:  left- and right-hand side

            iparea(1,2) = 2
            iparea(2,2) = nj
            tdir = 2
            ndir = 1
            if ( iside.eq.1 ) then

c           --- iside = 1:  left-hand side

               nsign = -1
               inside = 4
               j2shift = -1

            else

c           --- iside = 2:  right-hand side

               nsign = 1
               inside = 3
               iparea(1,1) = ni
               iparea(2,1) = ni
               i1shift = 1
               i2shift = 1
               j2shift = -1

            end if

         else

c        --- iside = 3, 4:  lower- and upper side

            iparea(1,1) = 2
            iparea(2,1) = ni
            tdir = 1
            ndir = 2
            if ( iside.eq.3 ) then

c           --- iside = 3:  lower side

               nsign = -1
               inside = 4
               i2shift = -1

            else

c           --- iside = 4:  upper side

               nsign = 1
               inside = 3
               iparea(1,2) = nj
               iparea(2,2) = nj
               j1shift = 1
               j2shift = 1
               i2shift = -1

            end if

         end if

         do jp = iparea(1,2), iparea(2,2)
            do ip = iparea(1,1), iparea(2,1)

c           --- Get integration cell index

               mxp(1) = ip
               mxp(2) = jp

c              --- Get integration cell position for ibinfor

               btype = ibinfor(ipibinf+2*mxp(tdir)-1)
               skip = btype.ge.88 .and. btype.le.100
               if ( .not. skip ) then

c              --- Standard boundary
c                  Remove pressure from point (0,-1), i.e. subtract outside
c                  contributions from inner ones and set equal to zero
c                  Outside points: sides 1 and 3: 1 and 2
c                                  sides 2 and 4: 5 and 6
c                  Central points:  3 and 4
c                  Inside points: sides 1 and 3: 5 and 6
c                                 sides 2 and 4: 1 and 2
c                  Hence outside points: inside+2*nsign
c                         inside points: inside-2*nsign

                  fact = gmat(ip,jp,tdir,3+2*nsign)
                  gmat(ip,jp,tdir,3-2*nsign) =
     +                gmat(ip,jp,tdir,3-2*nsign) - fact
                  gmat(ip,jp,tdir,3) =
     +                gmat(ip,jp,tdir,3) + 2d0*fact
                  gmat(ip,jp,tdir,3+2*nsign) = 0d0
                  fact = gmat(ip,jp,tdir,4+2*nsign)
                  gmat(ip,jp,tdir,4-2*nsign) =
     +                gmat(ip,jp,tdir,4-2*nsign) - fact
                  gmat(ip,jp,tdir,4) =
     +                gmat(ip,jp,tdir,4) + 2d0*fact
                  gmat(ip,jp,tdir,4+2*nsign) = 0d0

               else if ( btype.lt.90 ) then

c              --- Transition from multiblock to other boundary
c                  Check if the previous or the next boundary point is
c                  periodical

                  btypenb = ibinfor(ipibinf+2*mxp(tdir)-2)
                  if ( btypenb.ge.90 ) then

c                 --- previous point is multiblock
c                     Eliminate point 2

                     fact = gmat(ip,jp,tdir,4+2*nsign)
                     gmat(ip,jp,tdir,4-2*nsign) =
     +                  gmat(ip,jp,tdir,4-2*nsign) - fact
                     gmat(ip,jp,tdir,4) =
     +                  gmat(ip,jp,tdir,4) + 2d0*fact
                     gmat(ip,jp,tdir,4+2*nsign) = 0d0

c                    --- Eliminate point 5 in normal cell

                     if ( iside.eq.1 .or. iside.eq.3 ) then
                        ih = 5
                     else
                        ih = 6
                     end if
                     fact = gmat(ip+i2shift,jp+j2shift,ndir,ih)
                     gmat(ip+i2shift,jp+j2shift,ndir,ih-4) =
     +                  gmat(ip+i2shift,jp+j2shift,ndir,ih-4) - fact
                     gmat(ip+i2shift,jp+j2shift,ndir,ih-2) =
     +                  gmat(ip+i2shift,jp+j2shift,ndir,ih-2) + 2d0*fact
                     gmat(ip+i2shift,jp+j2shift,ndir,ih) = 0d0

                  else

c                 --- next point is multiblock
c                     Eliminate point 1

                     fact = gmat(ip,jp,tdir,3+2*nsign)
                     gmat(ip,jp,tdir,3-2*nsign) =
     +                   gmat(ip,jp,tdir,3-2*nsign) - fact
                     gmat(ip,jp,tdir,3) =
     +                   gmat(ip,jp,tdir,3) + 2d0*fact
                     gmat(ip,jp,tdir,3+2*nsign) = 0d0

c                    --- Eliminate point 1 in normal cell

                     if ( iside.eq.1 .or. iside.eq.3 ) then
                        ih = 1
                     else
                        ih = 2
                     end if
                     fact = gmat(ip+i1shift,jp+j1shift,ndir,ih)
                     gmat(ip+i1shift,jp+j1shift,ndir,ih+4) =
     +                  gmat(ip+i1shift,jp+j1shift,ndir,ih+4) - fact
                     gmat(ip+i1shift,jp+j1shift,ndir,ih+2) =
     +                  gmat(ip+i1shift,jp+j1shift,ndir,ih+2) + 2d0*fact
                     gmat(ip+i1shift,jp+j1shift,ndir,ih) = 0d0

                  end if

               end if

            end do

         end do

c        --- Fold virtual points for normal velocity cells at corners
c            First "lower" part

         if ( iside.eq.2 ) then
            ip = ni+1
         else
            ip = 1
         end if
         if ( iside.eq.4 ) then
            jp = nj+1
         else
            jp = 1
         end if
         btype = ibinfor(ipibinf+2)
         if ( btype.eq.snnut .or. btype.ge.88 .and. btype.le.100 ) then

c        --- sigma^nn and u_t given, or periodical type of bc's,
c            only in this case points have to be eliminated
c            Find boundary type of neighbour

            if ( iside.eq.1 ) then
               btypenb = ibinfor(ibipoint(3)-1+nvirtual+2)
            else if ( iside.eq.2 ) then
               btypenb = ibinfor(ibipoint(3)-1+nvirtual+2*ni)
            else if ( iside.eq.3 ) then
               btypenb = ibinfor(ibipoint(1)-1+nvirtual+2)
            else
               btypenb = ibinfor(ibipoint(1)-1+nvirtual+2*nj)
            end if

            skip = btypenb.ge.88 .and. btypenb.le.100
            if ( .not. skip ) then
               fact = gmat(ip,jp,ndir,inside-2)
               gmat(ip,jp,ndir,inside+2) =
     +             gmat(ip,jp,ndir,inside+2) - fact
               gmat(ip,jp,ndir,inside) =
     +             gmat(ip,jp,ndir,inside) + 2d0*fact
               gmat(ip,jp,ndir,inside-2) = 0d0
            end if
            if ( btype.ne.snnut ) then

c           --- periodical case, eliminate most extreme corner point

               fact = gmat(ip,jp,ndir,inside-2+nsign)
               gmat(ip,jp,ndir,inside+2+nsign) =
     +             gmat(ip,jp,ndir,inside+2+nsign) - fact
               gmat(ip,jp,ndir,inside+nsign) =
     +             gmat(ip,jp,ndir,inside+nsign) + 2d0*fact
               gmat(ip,jp,ndir,inside-2+nsign) = 0d0

            end if

         end if

c        --- Next "upper" part

         if ( iside.eq.2 ) then
            ip = ni+1
         else
            ip = iparea(2,1)
         end if
         if ( iside.eq.4 ) then
            jp = nj+1
         else
            jp = iparea(2,2)
         end if
         if ( iside.le.2 ) then
            btype = ibinfor(ipibinf+2*nj)
         else
            btype = ibinfor(ipibinf+2*ni)
         end if
         if ( btype.eq.snnut .or. btype.ge.88 .and. btype.le.100 ) then

c        --- sigma^nn and u_t given, or periodical type of bc's,
c            only in this case points have to be eliminated
c            Find boundary type of neighbour

            if ( iside.eq.1 ) then
               btypenb = ibinfor(ibipoint(4)-1+nvirtual+2)
            else if ( iside.eq.2 ) then
               btypenb = ibinfor(ibipoint(4)-1+nvirtual+2*ni)
            else if ( iside.eq.3 ) then
               btypenb = ibinfor(ibipoint(2)-1+nvirtual+2)
            else
               btypenb = ibinfor(ibipoint(2)-1+nvirtual+2*nj)
            end if

            skip = btypenb.ge.88 .and. btypenb.le.100
            if ( .not. skip ) then
               fact = gmat(ip,jp,ndir,inside+2)
               gmat(ip,jp,ndir,inside-2) =
     +             gmat(ip,jp,ndir,inside-2) - fact
               gmat(ip,jp,ndir,inside) =
     +             gmat(ip,jp,ndir,inside) + 2d0*fact
               gmat(ip,jp,ndir,inside+2) = 0d0
            end if
            if ( btype.ne.snnut ) then

c           --- periodical case, eliminate most extreme corner point

               fact = gmat(ip,jp,ndir,inside+2+nsign)
               gmat(ip,jp,ndir,inside-2+nsign) =
     +             gmat(ip,jp,ndir,inside-2+nsign) - fact
               gmat(ip,jp,ndir,inside+nsign) =
     +             gmat(ip,jp,ndir,inside+nsign) + 2d0*fact
               gmat(ip,jp,ndir,inside+2+nsign) = 0d0

            end if

         end if

      end do

      end
