      subroutine isgmat2d ( alphad, ni, nj, nvirtual, gmat, cva, invjac,
     +                      ibipoint, ibinfor, idiscret, jac, adir )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.3    date 22-06-1998 Charles Change in comments
c        version 2.2    date 18-05-1998 Charles Bilinear discretization
c        version 2.1    date 19-12-1996 Kees K.: call isgmat04 changed
c        version 2.0    date 01-10-1996 Wesbeek discretization
c        version 1.0    date 20-09-1995
c
c   copyright (c) 1995-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Compute gradient matrix in R^2
c
c **********************************************************************
c
c                       KEYWORDS
c
c    grad_matrix
c    matrix
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, ibipoint(*), ibinfor(*), idiscret,
     +        alphad
      double precision gmat(*),
     +                 cva((ni+1+2*nvirtual)*(nj+1+2*nvirtual),2,2,4),
     +                 invjac((ni+1+2*nvirtual)*(nj+1+2*nvirtual),4),
     +                 adir(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2),
     +                 jac((ni+1+2*nvirtual)*(nj+1+2*nvirtual),4)

c     adir      i    Contravariant base vector in integration cell center
c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     gmat      o    Gradient matrix
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     idiscret  i    Type of discretization
c     invjac    i    1/sqrt(g) in pnttyp
c     jac       i    sqrt(g) in pnttyp
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISGMAT02  Compute internal part of gradient matrix in R^2 for
c               one direction
c     ISGMAT04  Compute internal part of gradient matrix in R^2 for
c               one direction Wesbeek discretization
c     ISGMAT12  Correct the gradient matrix for boundary cells in 2D
c     ISGMAT22  Folding subroutine for the gradient matrix in 2D
c     ISGMWB12  Correct the gradient matrix for boundary cells in 2D
c               Wesbeek discretization
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Fill gmat for all V1-unknowns
c    Fill gmat for all V2-unknowns
c    Correct gmat for boundary cells as function of the boundary conditions
c    Fold all virtual pressures inside the region
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      if ( idiscret.eq.1 ) then

c     --- Classical discretization

         if ( alphad.eq.1 ) then

c        --- Fill gmat for all V1-unknowns

               call isgmat02 ( ni, nj, nvirtual, gmat, cva(1,1,1,3),
     +                         invjac(1,3), 1 )
         else

c        --- Fill gmat for all V2-unknowns

               call isgmat02 ( ni, nj, nvirtual, gmat, cva(1,1,1,2),
     +                         invjac(1,2), 2 )

         end if

      else if ( idiscret.eq.2.or.idiscret.eq.4 ) then

c     --- Wesbeek discretization

         if ( alphad.eq.1 ) then

c        --- Fill gmat for all V1-unknowns

            call isgmat04 ( ni, nj, nvirtual, gmat, cva, jac(1,3), 1,
     +                      adir, ibipoint, ibinfor )

         else

c        --- Fill gmat for all V2-unknowns

            call isgmat04 ( ni, nj, nvirtual, gmat, cva, jac(1,2), 2,
     +                      adir, ibipoint, ibinfor )

         end if

      end if

      if ( alphad.eq.2 ) then

c     --- alphad=2, which means that gmat has been filled for both
c         components. Now correct boundary cells and fold virtual pressures
c         Correct gmat for boundary cells as function
c         of the boundary conditions

         if ( idiscret.eq.1 ) then

c        --- Classical discretization

            call isgmat12 ( ni, nj, nvirtual, gmat, cva, invjac,
     +                      ibipoint, ibinfor )

         else if ( idiscret.eq.2.or.idiscret.eq.4 ) then

c        --- Wesbeek discretization

            call isgmwb12 ( ni, nj, nvirtual, gmat, ibipoint, ibinfor )

         end if

c        --- Fold all virtual pressures inside the region

         call isgmat22 ( ni, nj, nvirtual, gmat, ibipoint, ibinfor )

      end if

      end
