      subroutine isgmtbw3d ( iface, ni, nj, nk, gmat,
     +                       ilow1, ihigh1, ilow2, ihigh2, ibinfor )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 09-04-1998
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Adapt the g-matrix for boundary conditions of the type un given
c
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, ilow1, ihigh1,ilow2, ihigh2,
     +        ibinfor(ilow1:ihigh1,ilow2:ihigh2), iface
      double precision gmat(ni+1,nj+1,nk+1,3,10)

c     gmat     i/o   Gradient matrix
c     ibinfor   i    Third part of infbnd
c     iface     i    Face number
c     ihigh1    i    First upper index of an array
c     ihigh2    i    Second upper index of an array
c     ilow1     i    First lower index of an array
c     ilow2     i    Second lower index of an array
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer unut,  uxuy, unsnt, parabolic
      parameter( unut = 4, uxuy = 10, unsnt = 5, parabolic = 12)
      logical  ungiven
      integer  ndir, tdir(2), iarea(2,3), nijk(3),
     +         xp(3), ict1, ict2, btype, ip, jp, kp, i

c     btype     Type of boundary condition
c     i         General loop variable
c     iarea     Area of arrays
c     ict1      Pointer in ibinfor with respect to boundary point
c     ict2      Pointer in ibinfor with respect to boundary point
c     ip        Local counting variable i-direction
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c     ndir      Direction of normal vector on this side
c     nijk      Vector of number of points in the i,j and k-directions
c     parabolic Parabolic boundary conditions
c     tdir      Direction of tangent vector on this side
c     ungiven   If true un is given
c     unsnt     U_n and sigma_nt prescribed
c     unut      U_n and u_t prescribed
c     uxuy      U_x and u_y prescribed
c     xp        Auxiliary array to calculate boundary point number
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    For each point at the boundary where we have un given, the corresponding
c    row in gmat is set equal to zero
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Fill ni, nj and nk in the array nijk

      nijk(1) = ni
      nijk(2) = nj
      nijk(3) = nk

c     --- Compute tdir and ndir

      if ( iface.le.2 ) then

c     --- iface = 1, 2:  back and front plane

         ndir    = 1
         tdir(1) = 2
         tdir(2) = 3

      else if ( iface.gt.2  .and. iface.lt.5) then

c     --- iface = 3, 4:  left- and right plane

         ndir = 2
         tdir(1) = 3
         tdir(2) = 1

      else

c     --- iface = 5, 6:  bottom and top

         ndir = 3
         tdir(1) = 1
         tdir(2) = 2

      end if

c     --- loop over normal half cells
c         Compute loop bounds

      if ( mod(iface,2).eq.0 ) then
         iarea(1,ndir) = nijk(ndir)+1
         iarea(2,ndir) = nijk(ndir)+1
      else
         iarea(1,ndir) = 1
         iarea(2,ndir) = 1
      end if

      iarea(1,tdir(1)) = 1
      iarea(2,tdir(1)) = nijk(tdir(1))
      iarea(1,tdir(2)) = 1
      iarea(2,tdir(2)) = nijk(tdir(2))

      do kp = iarea(1,3), iarea(2,3)
         do jp = iarea(1,2), iarea(2,2)
            do ip = iarea(1,1), iarea(2,1)

c           --- Store integration cell indices in vector

               xp(1) = ip
               xp(2) = jp
               xp(3) = kp

c              --- Get corresponding index into ibinfor
c                  Get boundary information

               ict1 = 2*xp(tdir(1))
               ict2 = 2*xp(tdir(2))
               btype = ibinfor(ict1,ict2)

               ungiven = (btype.eq.uxuy .or.
     +                    btype.eq.unut .or. btype.eq.parabolic .or.
     +                    btype.eq.unsnt  )

               if ( ungiven ) then

c              --- Dirichlet half cell, set contribution equal to zero

                  do i = 1, 10
                     gmat(ip,jp,kp,ndir,i) = 0d0
                  end do

              end if

            end do
         end do
      end do

      end
