      subroutine ismom2dwb ( iinput, matrix, rhside, solut, coefs, work,
     +                       rinput, npointsv, infmm, vsum, dvsum,
     +                       invjac, coor, mct, bndcon, ibipoint,
     +                       ibinfor, jac, eddy, gmat, infmg )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.13   date 09-12-1999 Guus: new calls to isbwbp isbwbs2
c        version 1.12   date 29-09-1999 Guus: new call to ismomrh2
c        version 1.11   date 22-03-1999 KeesK: axisymmetric compressible
c        version 1.11   date 01-03-1999 KeesK: routine iscconv added
c        version 1.10   date 12-02-1999 KeesK: in compressible case make rho=1
c                                       independent of input for convection
c                                       terms
c        version 1.9    date 02-02-1999 Kees K: axisymmetric
c        version 1.8    date 20-01-1999 Kees K: call of isvecup1 changed
c        version 1.7    date 09-12-1998 Kees K: non conservative upwind
c        version 1.6    date 14-08-1998 Kees K: call of isbcc2 changed
c                                               for gnu compiler
c        version 1.5    date 14-08-1998 Kees K: conservative first order
c                                               upwind
c        version 1.4    date 27-07-1998 Duncan: extension to iterate timestep
c                                               get the whole eddy 1-4 not 2-4
c                                               apply underrelaxation
c        version 1.3    date 12-04-1998 Duncan: pressure bc for euler
c                                               added, sim.to isbmomp2
c        version 1.2    date 22-06-1998 Charles:argument changed in iscont2d
c        version 1.1    date 19-05-1998 Kees K.:distribution of stress
c                                       term changed.
c        version 1.0    date 16-02-1998
c
c   copyright (c) 1998-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Build momentum matrix and right-hand side in R2
c    WesBeek discretization
c **********************************************************************
c
c                       KEYWORDS
c
c    2d
c    matrix
c    momentum_equation
c    right_hand_side
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinput(*), npointsv, infmm(*), infmg(*),
     +        mct(2,2,-2:2,-2:2,-2:2), ibipoint(*), ibinfor(*)
      double precision rhside(*), work(*), matrix(*), coor(*),
     +                 solut(npointsv,*), coefs(npointsv,*), rinput(*),
     +                 vsum(npointsv,*),  invjac(npointsv,*),
     +                 dvsum(npointsv,2,2), bndcon(2,*),jac(npointsv,*),
     +                 eddy(6*npointsv), gmat(*)

c     bndcon    i    reserved
c     coefs     i    reserved
c     coor      i    reserved
c     dvsum     i    Flux derivative contributions to the matrix
c     eddy      i    reserved
c     gmat      i    Gradient matrix
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     iinput    i    reserved
c     infmg     i    Integer information for Pressure-gradient matrix
c     infmm     i    Integer information for Momentum matrix
c     invjac    i    1/sqrt(g) in pnttyp
c     jac       i    sqrt(g) in pnttyp
c     matrix    o    reserved
c     mct       i    Integer matrix information
c     npointsv  i    Number of points in the mesh with virtual points
c     rhside    o    reserved
c     rinput    i    reserved
c     solut     i    reserved
c     vsum      i    Flux contributions to the matrix
c     work      i    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iprho, nconct, alphad, iprintim, pnttyp, vartyp, ni, nj,
     +        nvirtual, ipmatrix, iprhside, ipmu, iarea(2,2), locdx(2),
     +        dcell(2), iparea(2,2), ipcva, ipjcta, ipjctg, ipchsym,
     +        ipveloc, ipgeo, i, iside, ipiindsc, mcom,
     +        ipiincom, nintpol(2), dij(2,4,2), acttyp, iprindsc,
     +        ipiintur, ichoisdis, iprintur, ndegfd, ipwork, mturb,
     +        maniso, iprhoinv, mequat, ipgradu,
     +        ipadir, idiscret, ipadiri, ipadirii, ipmassfr, ipprems,
     +        iconv, ipiintim, ipvecup1, ipvecup2, iupwind, typp,
     +        dcellv(2), iconserv, axisym
      double precision deltat
      logical reuse, tangcell

c     acttyp    Type of action to be taken
c               0      : interpolation for point value
c               1..ndim: finite difference for partial derivative
c                        in the direction given by value of acttyp
c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     axisym    In axisymmetric case axisym=1 else axisym=0
c     dcell     reserved
c     dcellv    Help variable replacing dcell
c     deltat    Product of theta and dt
c     dij       Array containing the distances of the interpolation
c               points to the cell centra in global co-ordinates
c               dij(alpha,i,j) denotes the distance of the i-th
c               interpolation point, component j and with respect to
c               the V-alpha unknown (distribution)
c     i         General loop variable
c     iarea     Area of arrays
c     ichoisdis Choice parameter with respect to the convection scheme
c               for discretization of convective term of transport
c               equation(s) including turbulence.
c               Meaning of ichoisdis:
c                 ichoisdis = 0: 2nd order central scheme
c                 ichoisdis = 1: 1st order standard upwind scheme
c                 ichoisdis = 2: Hybrid central/upwind scheme
c                 ichoisdis = 3: Linear kappa-scheme
c                 ichoisdis = 4: Sweby Phi-limiter
c                 ichoisdis = 5: R-kappa limiter
c                 ichoisdis = 6: USR-kappa limiter
c                 ichoisdis = 7: PL-kappa limiter
c                 ichoisdis = 8: USPL-kappa limiter
c                 ichoisdis = 9: Symmetric PL-kappa limiter
c                 ichoisdis = 10: MPL2-kappa limiter
c                 ichoisdis = 11: Symmetric PL-kappa limiter
c     iconserv  If 1 the conservative scheme is used, otherwise (0) the
c               non-conservative one
c     iconv     If 1 the timestep is iterated till converged, otherwise (0)
c               apply only one iterative step
c     idiscret  Type of discretization
c     ipadir    Starting address of array ADIR in array WORK
c     ipadiri   Starting address of array ADIRI in array WORK
c     ipadirii  Starting address of array ADIRII in array WORK
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     ipchsym   Starting address of array CHSYM in array WORK
c     ipcva     Starting address of array CVA in array WORK
c     ipgeo     Starting address of array GEO in array WORK
c     ipgradu   Starting address of array GRADU in array WORK
c     ipiincom  Starting address of array IINCOM in array IINPUT
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiintim  Starting address of array IINTIM in array IINPUT
c     ipiintur  Starting address of array IINTUR in array IINPUT
c     ipjcta    Starting address of array JCTA in array WORK
c     ipjctg    Starting address of array JCTG in array WORK
c     ipmassfr  Starting address of array MASSFRAC in array WORK
c     ipmatrix  Starting address of array matrix in buffer
c     ipmu      Starting address of array MU in array WORK
c     ipprems   Starting address of array PREMS in array WORK
c     iprho     Starting address of array rho either in BUFFER or WORK
c     iprhoinv  Starting address of array rhoinv in work
c     iprhside  Starting address of array rhside in buffer
c     iprindsc  Starting address of array RINDSC in array RINPUT
c     iprintim  Starting address of array RINTIM in array RINPUT
c     iprintur  Starting address of array rintur in array RINPUT
c     ipvecup1  Starting address of array VECUP1 in BUFFER
c     ipvecup2  Starting address of array VECUP2 in BUFFER
c     ipveloc   Starting address of array VELOC in array WORK
c     ipwork    Starting address of array work in buffer
c     iside     Sequence number of side
c     iupwind   Type of upwinding
c     locdx     Position of the integration point with respect to
c               the centre of the integration cell
c               The cell sizes are assumed to be 2 times 2, hence the
c               distance of a midside point to a cell centre is always 1
c     maniso    Indicates type of nonlinear turbulence (anisotropic)
c               modelling
c     mcom      If zero the flow is incompressible else (1) compressible
c     mequat    Indicates the type of momentum equation to be solved:
c               0:  standard equation
c               1:  multi-phase gas flow
c     mturb     Indicates type of turbulence modelling
c     nconct    Number of connections in a row of the matrix
c     ndegfd    Number of degrees of freedom
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nintpol   Number of interpolation points in the distribution
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nvirtual  Number of rows of virtual cells
c     pnttyp    reserved
c     reuse     If true the computed coefficients may be reused again
c     tangcell  If true a tangential cell has been found with integration point
c               at the boundary
c     typp      Help variable to store pnttype in case of upwind
c     vartyp    Variable to be interpolated/extrapolated
c               vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c               0 for vertex (for ndim=2)
c               or
c               vartyp: 1 for V1, 2 for V2 3, for V3 and
c               4 for pressure, 0 for vertex  (for ndim=3)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISAXISM   Computes the extra term  2 * mu * Ur /r   in
c               the axisymmetric case
c     ISBCC2    Dirichlet-dirichlet boundary condition convection term for
c               tangential cells 2D, Wesbeek discretization.
c     ISBCS2    Dirichlet-dirichlet boundary condition stress term for
c               tangential cells 2D, Wesbeek discretization.
c     ISBGEO2D  Fill geometrical quantities along boundary in R2
c     ISBMCUPW  Adapt normal half cells in upwind case for WesBeek
c               discretization
c     ISBMOMC2  Fill normal half cells for convection in R2
c     ISBMOMT2  Correct the time derivative part of the matrix diagonal
c               with respect to the boundary cells where sigma^nn is given
c     ISBWBP2   Fill normal half cells for pressure bc for Euler eqs. in R2
c     ISBWBS1   ISnas Momentum Matrix Stress at the Boundary in 2 dimensions
c               Tangential cells only, Wesbeek discretization
c     ISBWBS2   Compute stress contribution to matrix and right-hand side for
c               "normal" half cells, Wesbeek discretization.
c     ISCCONV   Fill help arrays vecup1 and vecup2 for convective terms
c     ISCONT2D  Compute contravariant base vectors in centroids of
c               alphad cells two-dimensional case
c     ISCONTG   Compute integration path coefficients two-dimensional case
c     ISCONTI   Compute contravariant base vectors times jacobian
c               two-dimensional case
c     ISCOVA2D  Fill covariant base vectors in all integration points in R2
c     ISCRHSD   Righthandside part of newton linearized convection term
c     ISDISC2D  Distribution of convective terms
c     ISDISCUP  Distribution of interpolation or finite difference formulas
c               over integration point with respect to the convection term
c               Wesbeek discretization first order upwind
c     ISDSWB2D  Distribution of the intermediate matrix 'prems' into
c               the  matrix, Wesbeek discretization.
c     ISGMAT2D  Compute gradient matrix in R^2
c     ISGMATRR  Multiply gradient matrix by r ( axisymmetric )
c     ISINTCF2  Compute interpolation of a coefficient in an integration point
c     ISINTCFA  Compute interpolation of a coefficient in an integration point
c               in the axisymmetric case
c     ISMAKMP2  Make a list of integration points for the momentum equations
c               in R2
c     ISMCVUWB  determines the coefficients of the velocity components
c               in a flux cell for the convection term in R2, first order
c               upwind, wesbeek discretization
c     ISMOMCV2  Fill convective terms for one integration point in R2
c     ISMOMF21  All parts of the matrix outside the region are folded to
c               internal domain for the momentum matrix in R2
c               This subroutine is meant for a 23-point stencil, wesbeek
c               discretization.
c     ISMOMRH2  Build the right-hand side source term in R2
c     ISMOMTD2  Adds the time derivative to the idagonal of the matrix and
c               the right-hand side
c     ISMOMUR   Apply underrelaxation with regard to the previous step in
c               the momentum equation
c     ISMST2WB  Components of the stress tensor in the integration points.
c               Two-dimensional 21-point Wesbeek discretization
c     ISMTVC23  Gradient matrix times pressure in 2D
c     ISNLIN2D  Update right-hand side vector with nonlinear effects of
c               stress-strain relationship in case of nonlinear modeling
c     ISPREMS   Computation of stress terms before distribution
c               Wesbeek discretization
c     ISSETCON  Make array equal to a constant
c     ISURHS2D  Update right-hand side vector with higher order upwind
c               scheme (2D)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c    Compute constants
c    Clear array matrix
c    Clear array rhside
c    Make list of integration points
c    Interpolate coefficients to integration points
c    For idim := 1 (1) 2 do
c       Build time-derivative part of the matrix and rhside
c       Make list of integration points
c       For all integration points do
c          Fill coefficients for convective terms
c          Fill multiplication factors for convective terms
c          distribute convective terms
c          Fill coefficients for stress terms
c          Fill multiplication factors for stress terms
c          Correct boundary cells for stress terms
c          distribute stress terms
c    Compute gradient matrix
c    Correct right-hand-side term with minus gradient matrix times p^n
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Compute constants

      ipiindsc = iinput(6)
      ipiintur = iinput(8)
      ipiintim = iinput(4)
      iprintim = iinput(29)
      iprintur = iinput(31)
      iprindsc = iinput(33)
      ni       = iinput(51)
      nj       = iinput(52)
      nvirtual = iinput(54)
      ndegfd   = iinput(55)
      ipiincom = iinput(9)
      mcom     = iinput(ipiincom)
      mequat   = iinput(ipiincom+2)
      mturb    = iinput(ipiintur-1+2)
      maniso   = iinput(ipiintur-1+4)
      nconct   = infmm(6)
      deltat   = rinput(iprintim -1 +4)
      ipcva    = 1
      ipjcta   = ipcva+16*npointsv
      ipjctg   = ipjcta+4*npointsv
      ipchsym  = ipjctg+3*npointsv
      iprho    = ipchsym+6*npointsv
      ipmu     = iprho+npointsv
      ipveloc  = ipmu+npointsv
      iprhoinv = ipveloc+2*npointsv
      ipadir   = ipjctg
      ipadiri  = ipadir+2*npointsv
      ipadirii = ipadiri+2*npointsv
      ipgradu  = iprhoinv+npointsv
      ipmassfr = ipgradu+npointsv
      ipprems  = ipmassfr+8*npointsv
      ipgeo    = ipprems+16*npointsv
      ipvecup1 = ipgeo+8*npointsv
      ipvecup2 = ipvecup1+npointsv
      idiscret = iinput(ipiindsc+1)
      iupwind  = iinput(ipiindsc-1+5)
      iconserv = iinput(ipiindsc-1+9)
      axisym   = iinput(ipiindsc-1+10)
      iconv    = iinput(ipiintim -1 + 17)

c     --- Clear array matrix
c         Clear array rhside

      call issetcon ( matrix, (ni+1)*(nj+1)*nconct*2, 0d0 )
      call issetcon ( rhside, (ni+1)*(nj+1)*2, 0d0 )

c     --- Fill covariant basis vectors in cva for all integration points

      call issetcon ( work(ipcva), 16*npointsv, 0d0 )
      call iscova2d ( coor, work(ipcva), ni, nj, nvirtual )

c     --- Fill array bcgeo and put into array work

      call isbgeo2d ( work(ipgeo), ni, nj, nvirtual, work(ipcva),
     +                ibipoint, invjac, jac )

      do alphad = 1, 2

c     --- Build time-derivative part of the matrix and rhside
c         Interpolate rho to integration point (Cell centre)

         iarea(1,1) = 1
         iarea(1,2) = 1
         iarea(2,1) = ni
         iarea(2,2) = nj
         if ( alphad.eq.1 ) then
            iarea(2,1) = ni+1
         else
            iarea(2,2) = nj+1
         end if

         pnttyp = 4-alphad
         vartyp = 3
         ipmatrix = 1+(alphad-1)*(ni+1)*(nj+1)
         iprhside = ipmatrix
         if ( mcom.eq.0 ) then

c        --- incompressible case, interpolate rho from coefs

            acttyp = 0
            if (axisym.eq.1) then
               call isintcfa ( coefs(1,1), work(iprho), ni, nj,
     +                        nvirtual,vartyp, acttyp, pnttyp, 0, coor)
            else
               call isintcf2 ( coefs(1,1), work(iprho), ni, nj,
     +                        nvirtual,vartyp, acttyp, pnttyp, 0, coor)
            end if

         else

c        --- compressible case, interpolate rho from eddy

            acttyp = 0
            if (axisym.eq.1) then
               call isintcfa ( eddy, work(iprho), ni, nj, nvirtual,
     +                          vartyp, acttyp, pnttyp, 0, coor )
            else
               call isintcf2 ( eddy, work(iprho), ni, nj, nvirtual,
     +                          vartyp, acttyp, pnttyp, 0, coor )
            end if

         end if

c        --- compute the contravariant components

         call iscont2d ( work(ipadir), jac, work(ipcva), ni,
     +                   nj, nvirtual, alphad )

         call ismomtd2 ( alphad, ni, nj, nvirtual, work(iprho),
     +                   solut(1,alphad), matrix(ipmatrix),
     +                   rhside(iprhside), coor, deltat, iarea, mcom,
     +                   work(ipmassfr), mequat, work(ipveloc), axisym )

         if ( mcom.eq.1 ) then

c        --- for the compressible case rho = 1 in the routines
c            for the convection terms because the unknown is
c            then rho*V
c            for the axisymmetric case we use the rhoinv part
c            as help array to make rho = r, rhoinv is later used
c            in the stress term, but then initialized again

            call issetcon ( work(iprho), npointsv, 1d0 )
            call issetcon ( work(iprhoinv), npointsv, 1d0 )

         end if

c        --- Correct for half cells

         call isbmomt2 ( ni, nj, nvirtual, matrix(ipmatrix),
     +                   rhside(iprhside), alphad, ibinfor, ibipoint )

c        --- Loop over integration points

         do i = 1, 5

c        --- Compute local co-ordinates, distances to cell center in global
c            co-ordinates, point type, number of interpolation points
c            and interpolatoin formula

            call ismakmp2 ( i, alphad, locdx, pnttyp, dcell,
     +                      nintpol, dij )

            reuse = locdx(1).eq.1 .or. locdx(2).eq.1

c           --- List of loop bounds for all integration points
c               iparea gives the bounds for the cell indices of the integration
c               points
c               These are the indices of the centres of the integration cells
c               but shifted over a distance dcell, since the actual
c               integration points may be in another cell

            iparea(1,1) = iarea(1,1)+dcell(1)
            iparea(2,1) = iarea(2,1)+dcell(1)
            iparea(1,2) = iarea(1,2)+dcell(2)
            iparea(2,2) = iarea(2,2)+dcell(2)

            if ( iinput(ipiindsc).eq.1 ) then

c           --- iindsc(1)=1, use convection term
c               First clear array vsum

               call issetcon ( vsum, 2*npointsv, 0d0 )

c              --- Fill some coefficients

               if ( (i.ne.1 .and. .not. reuse) .or.
     +               (axisym.eq.1) ) then

c              --- Fill rho except for first integration point
c                  if 'central' then rho in integration point
c                  if 'upwind' and 'conservative'  then rho in cell centre

                  typp = pnttyp
                  if ( iupwind.eq.1 .and. iconserv.eq.1) then
                     typp = 4 - alphad
                  end if

                  vartyp = 3
                  acttyp = 0
                  if ( mcom.eq.0) then
                     if( axisym.eq.1 ) then

                        call isintcfa ( coefs(1,1), work(iprho), ni, nj,
     +                            nvirtual, vartyp, acttyp, typp, 0,
     +                            coor )
                     else
                        call isintcf2 ( coefs(1,1), work(iprho), ni, nj,
     +                            nvirtual, vartyp, acttyp, typp, 0,
     +                            coor )
                     end if
                  else

c                 --- compressible, rho = 1 and in the axisymmetric
c                     case rho = r

                     if( axisym.eq.1 ) then
                        call issetcon ( work(iprhoinv), npointsv, 1d0 )
                        call isintcfa ( work(iprhoinv), work(iprho), ni,
     +                            nj, nvirtual, vartyp, acttyp, typp, 0,
     +                            coor )
                     end if

                  end if

               end if

c              --- Fill help array vecup1 with the coefficient rho*V
c                  for the convective term, depending
c                  on upwind and conservative or not conservative.
c                  Fill help array vecup2 with the factor which
c                  determines the direction of upwind.

               call iscconv ( alphad, iinput, solut, coor, work,
     +                         ipveloc, iprho, ipvecup1, ipvecup2,
     +                         pnttyp, locdx, npointsv, ibinfor,
     +                         ibipoint )

c              --- Next fill vsum with contribution of convective terms to
c                  integration points

               if ( iupwind.eq.1 ) then

c              --- cva and invjac in the cell centre

                  typp = 4 - alphad

                  call ismcvuwb ( ni, nj, nvirtual, work(ipvecup1),
     +                            work(ipvecup2), vsum, locdx,
     +                            dcell, iarea, work(ipadir),
     +                            work(ipcva+4*npointsv*(typp-1)),
     +                            invjac(1,typp), iconserv )

               else

                  call ismomcv2 ( ni, nj, nvirtual, work(iprho),
     +                            work(ipchsym), invjac(1,pnttyp),
     +                            work(ipveloc), vsum, alphad, locdx,
     +                            iinput(ipiindsc), iparea,
     +                            work(ipadir),
     +                            work(ipcva+4*npointsv*(pnttyp-1)) )
               end if

c              --- Correct contribution of convective terms for half cells

               if ( iupwind.eq.1 ) then

                  call isbmcupw ( ni, nj, nvirtual, vsum, alphad,
     +                            locdx, ibinfor, ibipoint )
               else
                  call isbmomc2 ( ni, nj, nvirtual, work(ipveloc), vsum,
     +                            alphad, work(iprho), invjac(1,pnttyp),
     +                            work(ipchsym), locdx, dcell, ibinfor,
     +                            ibipoint, iinput(ipiindsc),
     +                            work(ipadir),
     +                            work(ipcva+4*npointsv*(pnttyp-1))  )
               end if

               if ( iinput(ipiindsc-1+8).eq.1) then

c              --- Newton linearization of convective terms,
c                  right-hand-side part

                  call iscrhsd ( ni, nj, nvirtual, rhside(iprhside),
     +                           vsum, work(ipveloc), dcell,  iarea )

               end if

c              --- in the upwind case the indices of
c                  vsum are the cell-centre indices
c                  in the central case these are the
c                  integration point cell indices
c                  for subroutine isbcc2 this is adapted
c                  with dcellv for the moment

               if ( iupwind.eq.1 ) then
                  dcellv(1) = dcell(1)
                  dcellv(2) = dcell(2)
               else
                  dcellv(1) = 0
                  dcellv(2) = 0
               end if

c              --- Boundary condition wesbeek discretization

               call isbcc2 ( alphad, ni, nj, nvirtual,  vsum, rhside,
     +                       locdx, dcellv(1), dcellv(2), ibinfor,
     +                       ibipoint, bndcon , work(ipgeo) )

c              --- Distribute computed values over the positions
c                  corresponding to the unknowns

               if ( iupwind.eq.1 ) then

                  call isdiscup ( ni, nj, nvirtual, matrix,
     +                            vsum, work(ipvecup2), alphad,
     +                            dcell, iarea, mct, i )
               else

                  call isdisc2d ( ni, nj, nvirtual, matrix,
     +                            vsum, dcell, alphad,
     +                            iarea, iinput(ipiindsc), nintpol,
     +                            dij, mct, i )
               end if

               ichoisdis = iinput(ipiindsc-1+5)
               if ( ichoisdis.gt.2 ) then

c              --- Higher order upwind scheme
c                  Since defect correction is used this implies updating the
c                  right-hand side

                  if (mcom.eq.0) then

                     call isurhs2d ( alphad, ni, nj, nvirtual,
     +                               rhside(iprhside), vsum,
     +                               work(ipveloc), solut(1,alphad),
     +                               iarea, dcell, locdx,
     +                               ichoisdis, rinput(iprindsc-1+1),
     +                               rinput(iprindsc-1+2),
     +                               rinput(iprindsc-1+3),
     +                               rinput(iprindsc-1+4) )

                  else

                     call isurhs2d ( alphad, ni, nj, nvirtual,
     +                               rhside(iprhside), vsum,
     +                               work(ipveloc),
     +                               eddy((1+alphad)*npointsv+1),
     +                               iarea, dcell, locdx,
     +                               ichoisdis, rinput(iprindsc-1+1),
     +                               rinput(iprindsc-1+2),
     +                               rinput(iprindsc-1+3),
     +                               rinput(iprindsc-1+4) )
                  end if

               end if

            end if

            if ( iinput(ipiindsc-1+3).eq.0 ) then

c           --- Stress terms
c               Clear arrays vsum and dvsum

               call issetcon ( vsum, 2*npointsv, 0d0 )
               call issetcon ( dvsum, 4*npointsv, 0d0 )
               vartyp = 3
               acttyp = 0
               if ( .not. reuse ) then

                  if ( axisym.eq.1) then
                     call isintcfa ( coefs(1,2), work(ipmu), ni, nj,
     +                            nvirtual, vartyp, acttyp, pnttyp, 0,
     +                            coor )
                  else
                     call isintcf2 ( coefs(1,2), work(ipmu), ni, nj,
     +                            nvirtual, vartyp, acttyp, pnttyp, 0,
     +                            coor )
                  end if

               end if

               if ( i.eq.1 .and. axisym.eq.1 ) then
                 call isaxism( ni, nj, nvirtual, matrix, coor,
     +                           work(ipmu), alphad, work(ipadir),
     +                           work(ipcva+4*npointsv*(pnttyp-1)) )
               end if

               call isconti ( work(ipadiri),  work(ipcva), ni, nj,
     +                        nvirtual, locdx, alphad )

               call iscontg ( work(ipadirii), invjac,
     +                        work(ipcva), ni, nj, nvirtual,
     +                        locdx, alphad )

               call ismst2wb ( ni, nj, nvirtual, work(ipmu),
     +                         dvsum, alphad,locdx, iarea,
     +                         work(ipadir), work(ipadiri) )

c              --- Correction for tangential boundary cells

               if ( i.gt.1 ) then

c              --- Only the integration point at the boundary is important

                  tangcell = .false.
                  if ( alphad.eq.1 ) then
                     if ( locdx(2).eq.-1 ) then
                        iside = 3
                        tangcell = .true.
                     else if ( locdx(2).eq.1 ) then
                        iside = 4
                        tangcell = .true.
                     end if
                  else
                     if ( locdx(1).eq.-1 ) then
                        iside = 1
                        tangcell = .true.
                     else if ( locdx(1).eq.1 ) then
                        iside = 2
                        tangcell = .true.
                     end if
                  end if

c                 --- Get information on boundary

                  if ( tangcell ) then

                     call isbwbs1 ( ni, nj, nvirtual, iside, dvsum,
     +                              alphad, locdx, work(ipmu), dcell,
     +                              ibinfor, ibipoint, bndcon,
     +                              rhside, work(ipadir),
     +                              work(ipadiri),
     +                              work(ipadirii), work(ipcva) )

                  end if

               end if

c              --- Correct contribution of stress terms for half cells

               call isbwbs2 ( ni, nj, nvirtual, dvsum, alphad,
     +                        work(ipmu), locdx, dcell, ibinfor,
     +                        ibipoint, bndcon, rhside, work(ipcva),
     +                        work(ipgeo), work(ipadir),
     +                        work(ipadiri), coor, axisym )

c              --- Distribution of matrix

               if ( i.gt.1 ) then

                  call isprems ( ni, nj, nvirtual, locdx, work(ipprems),
     +                           dvsum, work(ipadirii), alphad, iarea,
     +                           mcom)

                  call isbcs2 ( alphad, ni, nj, nvirtual, work(ipprems),
     +                          rhside, locdx, ibinfor, ibipoint,
     +                          bndcon, work(ipgeo) )

                  call isdswb2d ( alphad, ni, nj, nvirtual, locdx,
     +                            iarea, work(ipprems), matrix,
     +                            work(ipcva), invjac, mct ,
     +                            work(iprhoinv), mcom,
     +                            eddy(npointsv+1) )

               end if

               if ( maniso.gt.0 ) then

c              --- Adding nonlinear effects of stress-strain relationship
c                  Since defect correction is used this implies updating the
c                  right-hand side

                  ichoisdis = iinput(ipiindsc-1+1)
                  ipwork  = ipveloc+2*npointsv
                  if ( iinput(ipiindsc).ne.1 .and. i.ne.1 .and.
     +                 .not. reuse ) then

c                 --- In the case there is no convection,
c                     rho must computed in integration points

                     vartyp = 3
                     acttyp = 0
                     call isintcf2 ( coefs(1,1), work(iprho), ni, nj,
     +                               nvirtual, vartyp, acttyp, pnttyp,
     +                               0, coor )
                  end if
                  call isnlin2d ( rhside(iprhside), work(ipwork), solut,
     +                            work(ipcva+4*npointsv*(pnttyp-1)),
     +                            work(ipjctg), work(ipchsym),
     +                            invjac(1,pnttyp), rinput(iprintur),
     +                            work(iprho), jac, locdx, alphad,
     +                            ichoisdis, ndegfd, ni, nj, npointsv,
     +                            mturb, nvirtual, pnttyp, iarea,
     +                            dcell )

               end if

            else

c           --- Compute contribution rhside of pressure boundary
c               condition for Euler eqs., sigmann given.

              if ( i.eq.1 ) then

                  call isconti ( work(ipadiri),  work(ipcva), ni, nj,
     +                           nvirtual, locdx, alphad )
                  call isbwbp2 ( ni, nj, nvirtual, alphad,
     +                           ibinfor, ibipoint, bndcon, rhside,
     +                           work(ipgeo), work(ipadir),
     +                           work(ipadiri), coor, axisym )
              end if

            end if

         end do

c        --- Compute the g-matrix

         call isgmat2d ( alphad, ni, nj, nvirtual, gmat, work(ipcva),
     +                   invjac, ibipoint, ibinfor, idiscret, jac,
     +                   work(ipadir) )

      end do

      if ( axisym.eq.1) then
         call isgmatrr( ni, nj, nvirtual, gmat, coor )
      end if

c     --- Finally the folding subroutine
c         All unknowns outside the region are folded insidewards

      call ismomf21 ( ni, nj, nvirtual, mct, matrix, rhside,
     +                ibinfor, ibipoint, work(ipgeo), bndcon )

c     --- Add the rest of the right-hand-side vector

      if ( mcom.eq.0 ) then

c     --- incompressible case, use rho from coefs

         call ismomrh2 ( coefs(1,3), coefs(1,1), coor, work, gmat,
     +                   infmg(infmg(12)), rhside,
     +                   work(1+2*npointsv), ni, nj, nvirtual,
     +                   ibipoint, ibinfor, work(ipcva) )

      else

c     --- compressible case, use rho from eddy

         call ismomrh2 ( coefs(1,3), eddy(npointsv+1), coor, work,
     +                   gmat, infmg(infmg(12)), rhside,
     +                   work(1+2*npointsv), ni, nj, nvirtual,
     +                   ibipoint, ibinfor, work(ipcva) )

      end if

c     --- Update the right-hand side with -Gp^n
c         In two-phase flow: -eps Gp^n**

      call ismtvc23 ( gmat, solut(1,3), rhside, ni, nj, nvirtual,
     +             infmg(infmg(12)), coefs(1,6), mequat )

      if( iconv.eq.1 ) then

         call ismomur ( matrix, rhside, eddy(2*npointsv+1),
     +                  ni, nj, nconct, nvirtual)

      end if
      end
