      subroutine ismom3d ( iinput, matrix, rhside, solut, coefs, work,
     +                     npf, rspv, cv, rinput, npointsv, infmat,
     +                     vsum, dvsum, jac, invjac, coor, mct,
     +                     bndcon, ibipoint, ibindex, ibinfor, gmat,
     +                     mctg, mctinvg )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 3.4    date 22-04-1998 Guus: correction stress
c        version 3.3    date 17-02-1997 Marcel: parameter list of
c                                               isurhs3d changed
c        version 3.2    date 03-10-1996 keesk: idiscret
c        version 3.1    date 05-09-1996 Marcel: new call to subroutine
c                                               isnlin3d
c        version 3.0    date 22-07-1996 Guus: computation of gmat and rhside
c                                       implemented, infmg removed
c        version 2.1    date 26-05-1996 Kees K: new call to subroutine
c                                               isbms13d
c        version 2.0    date 26-03-1996 Marcel: new call to subroutine
c                                               isurhs3d
c        version 1.3    date 22-03-1996 Marcel: parameter list of isdisc3d
c                                               changed
c        version 1.2    date 06-02-1995 compute g-matrix
c        version 1.1    date 13-12-1995 reuse for locdx(3)
c        version 1.0    date 01-06-1995
c
c   copyright (c) 1995-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Build momentum matrix in R3
c
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer npf(*), iinput(*), rspv(*), npointsv, infmat(*),
     +        mct(3,3,-2:2,-2:2,-2:2), ibipoint(*), ibinfor(*),
     +        ibindex(2,2,*), mctg(*), mctinvg(*)
      double precision rhside(*), work(*), matrix(*), cv(*), coor(*),
     +                 solut(npointsv,*), coefs(npointsv,*), rinput(*),
     +                 vsum(npointsv,*), jac(npointsv,*),
     +                 invjac(npointsv,*),dvsum(npointsv,3,3),
     +                 bndcon(3,*), gmat(*)

c     bndcon    i    reserved
c     coefs     i    reserved
c     coor      i    reserved
c     cv        i    reserved
c     dvsum     i    Flux derivative contributions to the matrix
c     gmat      i    Gradient matrix
c     ibindex   i    ibindex(1,1,side)
c                                    first point index where information
c                                    is available on this side in
c                                    direction tdir (2D) or tdir(1) (3D)
c                    ibindex(2,1,side) last point index where information
c                                    is available on this side in
c                                    direction tdir (2D) or tdir(1) (3D
c                    ibindex(1,2,side)
c                                    first point index where information
c                                    is available on this side in
c                                    direction tdir or tdir(2) (3D)
c                    ibindex(2,2,side)
c                                    last point index where information
c                                    is available on this side in
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     iinput    i    reserved
c     infmat    i    Integer information for the matrix
c     invjac    i    1/sqrt(g) in pnttyp
c     jac       i    sqrt(g) in pnttyp
c     matrix    i    reserved
c     mct       i    Integer matrix information
c     mctg      i    Integer matrix information
c     mctinvg   i    Matrix information
c     npf       i    reserved
c     npointsv  i    Number of points in the mesh with virtual points
c     rhside    i    reserved
c     rinput    i    reserved
c     rspv      i    reserved
c     solut     i    reserved
c     vsum      i    Flux contributions to the matrix
c     work      i    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer maxpts, ndim
      parameter ( maxpts=7, ndim=3 )
      integer iprho, nconct, acttyp, alphad, iprintim, pnttyp,
     +        vartyp, ni, nj, nk, nvirtual, ipmatrix, iprhside, ipmu,
     +        iarea(2,3), locdx(3), codelist(maxpts), dcell(3),
     +        vtlist(3,maxpts), iparea(2,3), ipcva, ipjcta, ipjctg,
     +        ipchsym , ipveloc, iptemp, kdir, jdir,
     +        ptsused, i, idir, ipiindsc, maxb, iprindsc, ichoisdis,
     +        lptlist(3,maxpts), iface,
     +        ilow1, ihigh1, ilow2, ihigh2, ipgeo,
     +        ipiintur, iprintur, ndegfd, ipwork, mturb,
     +        maniso
      double precision deltat
      logical nzvsum(1:3,1:3), nzdvsum(1:3,1:3), reuse, tangcell

c     acttyp    Type of action to be taken
c               0      : interpolation for point value
c               1..ndim: finite difference for partial derivative
c                        in the direction given by value of acttyp
c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     codelist  List of point types (pnttyp) for the integration points
c     dcell     reserved
c     deltat    Product of theta and dt
c     i         General loop variable
c     iarea     Area of arrays
c     ichoisdis Choice parameter with respect to the convection scheme
c               for discretization of convective term of transport
c               equation(s) including turbulence.
c               Meaning of ichoisdis:
c                 ichoisdis = 0: 2nd order central scheme
c                 ichoisdis = 1: 1st order standard upwind scheme
c                 ichoisdis = 2: Hybrid central/upwind scheme
c                 ichoisdis = 3: Linear kappa-scheme
c                 ichoisdis = 4: Sweby Phi-limiter
c                 ichoisdis = 5: R-kappa limiter
c                 ichoisdis = 6: USR-kappa limiter
c                 ichoisdis = 7: PL-kappa limiter
c                 ichoisdis = 8: USPL-kappa limiter
c                 ichoisdis = 9: Symmetric PL-kappa limiter
c                 ichoisdis = 10: MPL2-kappa limiter
c                 ichoisdis = 11: Symmetric PL-kappa limiter
c     idir      General loop variable for 1...ndim loop
c     iface     Face number
c     ihigh1    First upper index of an array
c     ihigh2    Second upper index of an array
c     ilow1     First lower index of an array
c     ilow2     Second lower index of an array
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     ipchsym   Starting address of array CHSYM in array WORK
c     ipcva     Starting address of array CVA in array WORK
c     ipgeo     Starting address of array GEO in array WORK
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiintur  Starting address of array IINTUR in array IINPUT
c     ipjcta    Starting address of array JCTA in array WORK
c     ipjctg    Starting address of array JCTG in array WORK
c     ipmatrix  Starting address of array matrix in buffr
c     ipmu      Starting address of array MU in array WORK
c     iprho     Starting address of array rho either in BUFFER or WORK
c     iprhside  Starting address of array rhside in buffr
c     iprindsc  Starting address of array RINDSC in array RINPUT
c     iprintim  Starting address of array RINTIM in array RINPUT
c     iprintur  Starting address of array rintur in array RINPUT
c     iptemp    Starting address of temporary array in IBUFFR
c     ipveloc   Starting address of array VELOC in array WORK
c     ipwork    Starting address of array work in buffr
c     jdir      General loop variable for 1...ndim loop
c     kdir      General loop variable for 1...ndim loop
c     locdx     Position of the integration point with respect to
c               the centre of the integration cell
c               The cell sizes are assumed to be 2 times 2, hence the
c               distance of a midside point to a cell centre is always 1
c     lptlist   Contains the position of the integration points relative to
c               the integration cell centre measured in local co-ordinates
c     maniso    Indicates type of nonlinear turbulence (anisotropic)
c               modelling
c     maxb      Maximum number of real values for one boundary condition
c     maxpts    Maximum number of integration points per cell
c     mturb     Indicates type of turbulence modelling
c     nconct    Number of connections in a row of the matrix
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     nvirtual  Number of rows of virtual cells
c     nzdvsum   Indicates whether or not a partial derivative in a
c               certain direction occurs in the stencil.
c               For example the derivative dU2/dksi1 in the point (0,1)
c               or the V1-stencil of the 13-point molecule does not exist
c               nzdvsum should therefore be false in that point
c     nzvsum    Indicates whether or not a velocity in a
c               certain direction has a contribution in the stencil.
c     pnttyp    reserved
c     ptsused   Number of points in use
c     reuse     If true the computed coefficients may be reused again
c     tangcell  If true a tangential cell has been found with integration point
c               at the boundary
c     vartyp    Variable to be interpolated/extrapolated
c               vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c               0 for vertex (for ndim=2)
c               or
c               vartyp: 1 for V1, 2 for V2 3, for V3 and
c               4 for pressure, 0 for vertex  (for ndim=3)
c     vtlist    Contains for each integration point the shift of the cell
c               index of this integration point with respect to the cell
c               containing the centre of the integration cell.
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISBGEO3D  Compute geometrical quantities at boundaries of 2D region
c               and storeinto array bcgeo. Three-dimensional case
c     ISBMOMC3  Determines the coefficients of the velocity components in
c               a given integration point within a boundary flux cell for
c               for the convection term in R3
c     ISBMOMT3  Correct the time derivative part of the matrix diagonal
c               with respect to the boundary cells where sigma^nn is given
c               R^3 only
c     ISBMS13D  ISnas Momentum Matrix Stress at the Boundary in 3 dimensions
c               Tangential cells only
c     ISBMS23D  Compute stress contribution to matrix and right-hand side
c               for "normal" half cells
c     ISCHRYS3  Compute the chrystoffel symbols in integration point
c               three-dimensional case
c     ISCOVA3D  Compute covariant base vectors in all points of the grid
c               three-dimensional case
c     ISDISC3D  Distribution of interpolation or finite difference formulas
c               over integration point with respect to the convection term 3D
c     ISDISS3D  Distribution of interpolation or finite difference formulas
c               over integration point with respect to the stress term 3D
c     ISGMAT3D  Compute gradient matrix in R^3
c     ISINTCF3  Compute interpolation of a coefficient in one integration
c               point in R^3
c     ISJCTA3D  Compute sqrt(g) x contra variant base vectors in integration
c               point three-dimensional case
c     ISJCTG3D  Compute sqrt(g) x contra variant metric tensors in integration
c               point three-dimensional case
c     ISMAKMP3  Give a list of integration points for a given momentum
c               integration cell. (3D)
c     ISMOMCV3  Determines the coefficients of the velocity components in
c               a given integration point within a flux cell for
c               the convection term in R3
c     ISMOMF3D  All parts of the matrix outside the region are folded to
c               internal domain for the momentum matrix in R3
c     ISMOMRH3  Build the right-hand side source term in R3
c     ISMOMST3  Determines the coefficients of the flux components and
c               the partial  derivatives of the flux components in a given
c               integration point within a flux cell for the stress terms
c               Three-dimensional classical ISNaS discretization
c     ISMOMTD3  Adds the time derivative to the idagonal of the matrix and
c               the right-hand side R^3 only
c     ISMTVC33  Gradient matrix times pressure in 3D
c     ISNLIN3D  Update right-hand side vector with nonlinear effects of
c               stress-strain relationship in case of nonlinear modeling
c     ISSETCON  Make array equal to a constant
c     ISURHS3D  Update right-hand side vector with higher order upwind
c               scheme (3D)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c    Compute constants
c    Clear array matrix
c    Clear array rhside
c    Fill covariant basis vectors in cva for all integration points
c
c    For idim := 1 (1) 3 do
c
c       Build time-derivative part of the matrix and rhside
c       Interpolate rho to integration point (Cell centre)
c       Correct for half cells
c
c       Make list of integration points
c
c       For all integration points do
c
c          Fill coefficients for convective terms
c          Fill multiplication factors for convective terms
c          Correct contribution of convective terms for half cells
c          distribute convective terms
c
c          Fill coefficients for stress terms
c          Fill multiplication factors for stress terms
c          Correct contribution of stress terms for tangential boundary cells
c          Correct contribution of stress terms for half cells
c          distribute stress terms
c
c    Finally the folding subroutine
c    All unknowns outside the region are folded insidewards
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Compute constants

      ipiindsc = iinput(6)
      ipiintur = iinput(8)
      iprintim = iinput(29)
      iprintur = iinput(31)
      iprindsc = iinput(33)
      ni       = iinput(51)
      nj       = iinput(52)
      nk       = iinput(53)
      nvirtual = iinput(54)
      ndegfd   = iinput(55)
      maxb     = iinput(61)
      mturb    = iinput(ipiintur-1+2)
      maniso   = iinput(ipiintur-1+4)
      nconct   = infmat(6)
      deltat   = rinput(iprintim -1 +4)
      npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*(nk+1+2*nvirtual)
      ipcva    = 1
      ipjcta   = ipcva+72*npointsv
      ipjctg   = ipjcta+9*npointsv
      ipchsym  = ipjctg+6*npointsv
      iprho    = ipchsym+18*npointsv
      ipmu     = iprho+npointsv
      ipveloc  = ipmu+npointsv
      ipgeo    = ipjcta
      acttyp   = 0

c     --- Clear array matrix
c         Clear array rhside

      call issetcon ( matrix, (ni+1)*(nj+1)*(nk+1)*nconct*3, 0d0 )
      call issetcon ( rhside, (ni+1)*(nj+1)*(nk+1)*3, 0d0 )

c     --- Fill covariant basis vectors in cva for all integration points

      call issetcon ( work(ipcva), 72*npointsv, 0d0 )
      call iscova3d ( coor, work(ipcva), ni, nj, nk, nvirtual, npf,
     +                rspv, cv )

      do alphad = 1, 3

c     --- Build time-derivative part of the matrix and rhside
c         Interpolate rho to integration point (Cell centre)

         iarea(1,1) = 1
         iarea(1,2) = 1
         iarea(1,3) = 1
         iarea(2,1) = ni
         iarea(2,2) = nj
         iarea(2,3) = nk
         if ( alphad.eq.1 ) then
            iarea(2,1) = ni+1
            pnttyp     = 7
         else if ( alphad.eq.2 ) then
            iarea(2,2) = nj+1
            pnttyp     = 6
         else
            iarea(2,3) = nk+1
            pnttyp     = 4
         end if
         vartyp = 4
         ipmatrix = 1+(alphad-1)*(ni+1)*(nj+1)*(nk+1)
         iprhside = ipmatrix
         call isintcf3 ( coefs(1,1), work(iprho), ni, nj, nk, nvirtual,
     +                   npf, rspv, cv, acttyp, vartyp, pnttyp )
         call ismomtd3 ( ni, nj, nk, nvirtual, work(iprho),
     +                   solut(1,alphad), matrix(ipmatrix),
     +                   rhside(iprhside), deltat, iarea )

c        --- Correct for half cells

         do iface = 1, 6
            ilow1  = ibindex(1,1,iface)
            ihigh1 = ibindex(2,1,iface)
            ilow2  = ibindex(1,2,iface)
            ihigh2 = ibindex(2,2,iface)
            call isbmomt3 ( ni, nj, nk, matrix(ipmatrix),
     +                      rhside(iprhside), alphad,
     +                      ibinfor(ibipoint(iface)), ilow1,
     +                      ihigh1,  ilow2, ihigh2, iface )
         end do

c        --- Make list of integration points, meant for internal cells only

         vartyp = alphad
         call ismakmp3 ( vartyp, ptsused, vtlist, lptlist, codelist )

c        --- Loop over integration points

         do 300 i = 1, ptsused

c        --- Compute some general constants and arrays with respect to
c            integration point

            pnttyp = codelist(i)

c           --- Shift with respect to integration cell center

            locdx(1) = lptlist(1,i)
            locdx(2) = lptlist(2,i)
            locdx(3) = lptlist(3,i)
            reuse = locdx(1).eq.1 .or. locdx(2).eq.1 .or. locdx(3).eq.1

c           --- Shift to compute cell index of integration point
c               relative to cell center

            dcell(1)  = vtlist(1,i)
            dcell(2)  = vtlist(2,i)
            dcell(3)  = vtlist(3,i)

            if ( .not. reuse ) then

c           --- Compute inverse of Jacobian, contravariant basis vectors
c               and christoffel symbols

               call isjcta3d ( work(ipcva+9*npointsv*(pnttyp-1)),
     +                         work(ipjcta), ni, nj, nk, nvirtual )

c              --- The metric tensor times sqrt(g)

               call isjctg3d ( invjac(1,pnttyp), work(ipjcta),
     +                         work(ipjctg), ni, nj, nk, nvirtual )

c              --- The christoffel symbols of the second kind

               call ischrys3 ( work(ipcva), work(ipjcta), work(ipchsym),
     +                         invjac(1,pnttyp), ni, nj, nk, nvirtual,
     +                         pnttyp )

            end if

c           --- List of loop bounds for all integration points
c               iparea gives the bounds for the cell indices of the integration
c               points
c               These are the indices of the centres of the integration cells
c               but shifted over a distance dcell, since the actual
c               integration points may be in another cell

            iparea(1,1) = iarea(1,1)+dcell(1)
            iparea(2,1) = iarea(2,1)+dcell(1)
            iparea(1,2) = iarea(1,2)+dcell(2)
            iparea(2,2) = iarea(2,2)+dcell(2)
            iparea(1,3) = iarea(1,3)+dcell(3)
            iparea(2,3) = iarea(2,3)+dcell(3)

            if ( iinput(ipiindsc).eq.1 ) then

c           --- iindsc(1)=1, use convection term
c               First clear array vsum

               call issetcon ( vsum, 3*npointsv, 0d0 )

c              --- Fill some coefficients

               if ( i.ne.1 .and. .not. reuse ) then

c              --- Fill rho except for first integration point

                  vartyp = 4
                  call isintcf3 ( coefs(1,1), work(iprho), ni, nj, nk,
     +                            nvirtual, npf, rspv, cv, acttyp,
     +                            vartyp, pnttyp )
               end if

               if ( .not. reuse ) then

c              --- Fill velocity componentwise

                  do idir = 1, 3
                     vartyp = idir
                     iptemp = ipveloc+(idir-1)*npointsv
                     call isintcf3 ( solut(1,idir), work(iptemp), ni,
     +                               nj, nk, nvirtual, npf, rspv, cv,
     +                               acttyp, vartyp, pnttyp )
                  end do

               end if

c              --- Next fill vsum with contribution of convective terms to
c                  integration points

               call ismomcv3 ( ni, nj, nk, nvirtual, work(iprho),
     +                         work(ipchsym), invjac(1,pnttyp),
     +                         work(ipveloc), vsum, alphad, locdx,
     +                         iinput(ipiindsc), iparea )

c              --- Correct contribution of convective terms for half cells

               do iface = 1, 6
                  ilow1  = ibindex(1,1,iface)
                  ihigh1 = ibindex(2,1,iface)
                  ilow2  = ibindex(1,2,iface)
                  ihigh2 = ibindex(2,2,iface)
                  call isbmomc3 ( ni, nj, nk, nvirtual, ilow1, ihigh1,
     +                         ilow2, ihigh2, work(ipveloc),  vsum,
     +                         alphad, work(iprho), invjac(1,pnttyp),
     +                         locdx, dcell, ibinfor(ibipoint(iface)),
     +                         iinput(ipiindsc), iface )
               end do

c              --- Distribute computed values over the positions
c                  corresponding to the unknowns

               call isdisc3d ( ni, nj, nk, nvirtual, matrix,
     +                         rhside(iprhside), vsum, work(ipveloc),
     +                         mct, dcell, npf, rspv, cv, alphad,
     +                         pnttyp, iarea, iinput(ipiindsc), i )

               ichoisdis = iinput(ipiindsc-1+5)
               if ( ichoisdis.gt.2 ) then

c              --- Higher order upwind scheme
c                  Since defect correction is used this implies updating the
c                  right-hand side

                 call isurhs3d ( alphad, ni, nj, nk, nvirtual,
     +                           rhside(iprhside), vsum, work(ipveloc),
     +                           solut(1,alphad), iarea, dcell, locdx,
     +                           ichoisdis, rinput(iprindsc-1+1),
     +                           rinput(iprindsc-1+2),
     +                           rinput(iprindsc-1+3),
     +                           rinput(iprindsc-1+4) )

               end if

            end if

            if ( iinput(ipiindsc-1+3).eq.0 ) then

c           --- Stress terms according to standard ISNaS

               do kdir = 1, ndim
                  do jdir = 1, ndim
                     nzdvsum(jdir,kdir) = .false.
                  end do
               end do

c              --- Clear arrays vsum and dvsum

               call issetcon ( vsum, 3*npointsv, 0d0 )
               call issetcon ( dvsum, 9*npointsv, 0d0 )
               vartyp = 4
               call isintcf3 ( coefs(1,1), work(iprho), ni, nj, nk,
     +                         nvirtual, npf, rspv, cv, acttyp,
     +                         vartyp, pnttyp )
               call isintcf3 ( coefs(1,2), work(ipmu), ni, nj, nk,
     +                         nvirtual, npf, rspv, cv, acttyp,
     +                         vartyp, pnttyp )

               call ismomst3 ( ni, nj, nk, nvirtual, work(ipmu),
     +                         work(ipjctg), work(ipchsym), vsum,
     +                         dvsum, alphad, locdx, nzdvsum, iparea )

c              --- Correction for tangential boundary cells

               if ( i.gt.1 ) then

c              --- Only the integration point at the boundary is important

                  tangcell = .false.
                  if ( alphad.eq.1 ) then
                     if ( locdx(2).eq.-1 ) then
                        iface = 3
                        tangcell = .true.
                     else if ( locdx(2).eq.1 ) then
                        iface = 4
                        tangcell = .true.
                     else if ( locdx(3).eq.-1 ) then
                        iface = 5
                        tangcell = .true.
                     else if ( locdx(3).eq.1 ) then
                        iface = 6
                        tangcell = .true.
                     end if
                  else if ( alphad.eq.2 ) then
                     if ( locdx(1).eq.-1 ) then
                        iface = 1
                        tangcell = .true.
                     else if ( locdx(1).eq.1 ) then
                        iface = 2
                        tangcell = .true.
                     else if ( locdx(3).eq.-1 ) then
                        iface = 5
                        tangcell = .true.
                     else if ( locdx(3).eq.1 ) then
                        iface = 6
                        tangcell = .true.
                     end if
                  else if ( alphad.eq.3 ) then
                     if ( locdx(1).eq.-1 ) then
                        iface = 1
                        tangcell = .true.
                     else if ( locdx(1).eq.1 ) then
                        iface = 2
                        tangcell = .true.
                     else if ( locdx(2).eq.-1 ) then
                        iface = 3
                        tangcell = .true.
                     else if ( locdx(2).eq.1 ) then
                        iface = 4
                        tangcell = .true.
                     end if
                  end if

                  if ( tangcell ) then
                     ilow1  = ibindex(1,1,iface)
                     ihigh1 = ibindex(2,1,iface)
                     ilow2  = ibindex(1,2,iface)
                     ihigh2 = ibindex(2,2,iface)
                     call isbms13d ( ni, nj, nk, nvirtual,ilow1,
     +                               ihigh1, ilow2, ihigh2, vsum,
     +                               dvsum, alphad, work(ipmu),
     +                               nzvsum, nzdvsum , jac(1,pnttyp),
     +                               invjac(1,pnttyp), work(ipjcta),
     +                               work(ipjctg), work(ipchsym),
     +                               work(ipcva+9*npointsv*(pnttyp-1)),
     +                               locdx, dcell,
     +                               ibinfor(ibipoint(iface)),
     +                               bndcon(1,ibipoint(iface)),
     +                               maxb, rhside, iface )
                  end if

               end if

c              --- Correct contribution of stress terms for half cells

               do iface = 1, 6
                  ilow1  = ibindex(1,1,iface)
                  ihigh1 = ibindex(2,1,iface)
                  ilow2  = ibindex(1,2,iface)
                  ihigh2 = ibindex(2,2,iface)
               call isbms23d ( ni, nj, nk, nvirtual, ilow1, ihigh1,
     +                         ilow2, ihigh2, vsum, dvsum, alphad,
     +                         work(ipmu), nzvsum, nzdvsum,
     +                         jac(1,pnttyp), work(ipjctg),
     +                         work(ipchsym), locdx, dcell,
     +                         ibinfor(ibipoint(iface)),
     +                         rhside, iface )
               end do

c              --- Distribution of matrix

               call isdiss3d ( ni, nj, nk, nvirtual, matrix, vsum,
     +                         dvsum, invjac, mct, dcell, npf, rspv, cv,
     +                         alphad, pnttyp, iarea, nzdvsum )

               if ( maniso.gt.0 ) then

c              --- Adding nonlinear effects of stress-strain relationship
c                  Since defect correction is used this implies updating the
c                  right-hand side

                 ichoisdis = iinput(ipiindsc-1+2)
                 ipwork  = ipveloc+3*npointsv
                 call isnlin3d (rhside(iprhside), work(ipwork), solut,
     +                          work(ipcva+9*npointsv*(pnttyp-1)),
     +                          work(ipjctg), work(ipchsym),
     +                          invjac(1,pnttyp), rinput(iprintur),
     +                          work(iprho), jac(1,pnttyp), locdx,
     +                          alphad, ichoisdis, ndegfd, ni, nj, nk,
     +                          npointsv, maniso, mturb, nvirtual,
     +                          pnttyp, iarea, dcell )
               end if

            end if

300      continue

      end do

c     --- Finally the folding subroutine
c         All unknowns outside the region are folded insidewards

      do iface = 1, 6

         ilow1  = ibindex(1,1,iface)
         ihigh1 = ibindex(2,1,iface)
         ilow2  = ibindex(1,2,iface)
         ihigh2 = ibindex(2,2,iface)

         call isbgeo3d ( iface, ni, nj, nk, nvirtual, ilow1, ihigh1,
     +                   ilow2, ihigh2, work(ipcva), work(ipgeo) )

         call ismomf3d ( iface, ni, nj, nk, ilow1, ihigh1,
     +                   ilow2, ihigh2, mct, matrix, rhside,
     +                   ibinfor(ibipoint(iface)), work(ipgeo),
     +                   bndcon(1,ibipoint(iface)) )

      end do

c     --- Compute the g-matrix

      call isgmat3d (ni, nj, nk, nvirtual, gmat, work(ipcva), invjac,
     +               ibipoint, ibindex, ibinfor, mctg, mctinvg )

c     --- Update the right-hand side with -Gp^n

      call ismtvc33 ( gmat, solut(1,4), rhside, ni, nj, nk, nvirtual,
     +                mctinvg )

c     --- Add the rest of the right-hand-side vector

      call ismomrh3 ( coefs(1,3), coefs(1,1), coor, work, gmat,
     +                mctinvg, rhside,
     +                work(1+3*npointsv), ni, nj, nk, nvirtual )

      end
