      subroutine ismom3dwb ( iinput, matrix, rhside, solut, coefs, work,
     +                       npf, rspv, cv, rinput, npointsv, infmat,
     +                       vsum, jac, invjac, coor, mct,
     +                       bndcon, ibipoint, ibindex, ibinfor, gmat,
     +                       mctinvg )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 11-02-1998
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Build momentum matrix in R3 using the WesBeek discretization
c
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer npf(*), iinput(*), rspv(*), npointsv, infmat(*),
     +        mct(3,3,-2:2,-2:2,-2:2), ibipoint(*), ibinfor(*),
     +        ibindex(2,2,*), mctinvg(*)
      double precision rhside(*), work(*), matrix(*), cv(*), coor(*),
     +                 solut(npointsv,*), coefs(npointsv,*), rinput(*),
     +                 vsum(npointsv,*), jac(npointsv,*),
     +                 invjac(npointsv,*),
     +                 bndcon(3,*), gmat(*)

c     bndcon    i    reserved
c     coefs     i    reserved
c     coor      i    reserved
c     cv        i    reserved
c     gmat      i    Gradient matrix
c     ibindex   i    ibindex(1,1,side)
c                                    first point index where information
c                                    is available on this side in
c                                    direction tdir (2D) or tdir(1) (3D)
c                    ibindex(2,1,side) last point index where information
c                                    is available on this side in
c                                    direction tdir (2D) or tdir(1) (3D
c                    ibindex(1,2,side)
c                                    first point index where information
c                                    is available on this side in
c                                    direction tdir or tdir(2) (3D)
c                    ibindex(2,2,side)
c                                    last point index where information
c                                    is available on this side in
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     iinput    i    reserved
c     infmat    i    Integer information for the matrix
c     invjac    i    1/sqrt(g) in pnttyp
c     jac       i    sqrt(g) in pnttyp
c     matrix    i    reserved
c     mct       i    Integer matrix information
c     mctinvg   i    Matrix information
c     npf       i    reserved
c     npointsv  i    Number of points in the mesh with virtual points
c     rhside    i    reserved
c     rinput    i    reserved
c     rspv      i    reserved
c     solut     i    reserved
c     vsum      i    Flux contributions to the matrix
c     work      i    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer maxpts
      parameter ( maxpts=7 )
      integer iprho, nconct, acttyp, alphad, iprintim, pnttyp,
     +        vartyp, ni, nj, nk, nvirtual, ipmatrix, iprhside, ipmu,
     +        iarea(2,3), codelist(maxpts), dcell(3), iperm(5),
     +        vtlist(3,maxpts), ipcva, pnttypcn, locdx(3),
     +        ipveloc, iptemp, ipadir, ipadiri, ipadirii, ipprems,
     +        ptsused, i, idir, ipiindsc,
     +        lptlist(3,maxpts), iface, iprhovel, ipxcentr,
     +        ilow1, ihigh1, ilow2, ihigh2, ipgeo
      double precision deltat
      logical reuse

c     acttyp    Type of action to be taken
c               0      : interpolation for point value
c               1..ndim: finite difference for partial derivative
c                        in the direction given by value of acttyp
c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     codelist  List of point types (pnttyp) for the integration points
c     dcell     reserved
c     deltat    Product of theta and dt
c     i         General loop variable
c     iarea     Area of arrays
c     idir      General loop variable for 1...ndim loop
c     iface     Face number
c     ihigh1    First upper index of an array
c     ihigh2    Second upper index of an array
c     ilow1     First lower index of an array
c     ilow2     Second lower index of an array
c     ipadir    Starting address of array ADIR in array WORK
c     ipadiri   Starting address of array ADIRI in array WORK
c     ipadirii  Starting address of array ADIRII in array WORK
c     ipcva     Starting address of array CVA in array WORK
c     iperm     Array of permutations of the numbers 1, 2 and 3
c               Contents: 1, 2, 3, 1, 2
c     ipgeo     Starting address of array GEO in array WORK
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipmatrix  Starting address of array matrix in buffer
c     ipmu      Starting address of array MU in array WORK
c     ipprems   Starting address of array PREMS in array WORK
c     iprho     Starting address of array rho either in BUFFER or WORK
c     iprhovel  Starting address of array rho*u in BUFFER or WORK
c     iprhside  Starting address of array rhside in buffer
c     iprintim  Starting address of array RINTIM in array RINPUT
c     iptemp    Starting address of temporary array in IBUFFR
c     ipveloc   Starting address of array VELOC in array WORK
c     ipxcentr  Starting address of array XCENTR in BUFFER or WORK
c     locdx     Position of the integration point with respect to
c               the centre of the integration cell
c               The cell sizes are assumed to be 2 times 2, hence the
c               distance of a midside point to a cell centre is always 1
c     lptlist   Contains the position of the integration points relative to
c               the integration cell centre measured in local co-ordinates
c     maxpts    Maximum number of integration points per cell
c     nconct    Number of connections in a row of the matrix
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     nvirtual  Number of rows of virtual cells
c     pnttyp    reserved
c     pnttypcn  point type of centroid of integration cell
c     ptsused   Number of points in use
c     reuse     If true the computed coefficients may be reused again
c     vartyp    Variable to be interpolated/extrapolated
c               vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c               0 for vertex (for ndim=2)
c               or
c               vartyp: 1 for V1, 2 for V2 3, for V3 and
c               4 for pressure, 0 for vertex  (for ndim=3)
c     vtlist    Contains for each integration point the shift of the cell
c               index of this integration point with respect to the cell
c               containing the centre of the integration cell.
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISAXYZ1   Perform the multiplication y = rho u/sqrt(g)
c     ISBCS3    Fill prescribed velocities in the term prems, which is used to
c               build the stress matrix. 3D WesBeek version.
c     ISBGEO3D  Compute geometrical quantities at boundaries of 2D region
c               and storeinto array bcgeo. Three-dimensional case
c     ISCONT3D  Compute contravariant base vectors in centroids of alphad cells
c               three-dimensional case
c     ISCONTG3  Compute integration path coefficients for the velocity gradient
c               three-dimensional case
c     ISCONTI3  Compute contravariant base vectors times jacobian
c               three-dimensional case in the integration points
c     ISCOVA3D  Compute covariant base vectors in all points of the grid
c               three-dimensional case
c     ISDISC3D  Distribution of interpolation or finite difference formulas
c               over integration point with respect to the convection term 3D
c     ISDSWB3D  Distribution of the intermediate matrix 'prems' into
c               the  matrix, Wesbeek discretization 3D.
c     ISGMATW3D Compute gradient matrix in R^3
c               WesBeek discretization
c     ISGMTBW3D Adapt the g-matrix for boundary conditions of the type un given
c     ISINTCF3  Compute interpolation of a coefficient in one integration
c               point in R^3
c     ISMAKMP3  Give a list of integration points for a given momentum
c               integration cell. (3D)
c     ISMF3DWB  All parts of the matrix outside the region are folded to
c               internal domain for the momentum matrix in R3,
c               WesBeek version.
c     ISMOMCW3  Compute the coefficient in the integration point
c               within a flux cell for the convection term in R3 (WesBeek)
c     ISMOMRH3  Build the right-hand side source term in R3
c     ISMOMTD3  Adds the time derivative to the idagonal of the matrix and
c               the right-hand side R^3 only
c     ISMST3WB  Components of the stress tensor in the integration points.
c               Three-dimensional 59-point Wesbeek discretization
c     ISMTVC33  Gradient matrix times pressure in 3D
c     ISPREMS3  Computation of stress terms before distribution, Wesbeek
c               discretization 3D.
c     ISSETCON  Make array equal to a constant
c     ISXCENTR3 Compute coordinates of centroids of elements and store in xcentr
c               three-dimensional case
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c    Compute constants
c    Clear array matrix
c    Clear array rhside
c    Fill covariant basis vectors in cva for all integration points
c
c    For idim := 1 (1) 3 do
c
c       Build time-derivative part of the matrix and rhside
c       Interpolate rho to integration point (Cell centre)
c       Correct for half cells
c
c       Make list of integration points
c
c       For all integration points do
c
c          Fill coefficients for convective terms
c          Fill multiplication factors for convective terms
c          Correct contribution of convective terms for half cells
c          distribute convective terms
c
c          Fill coefficients for stress terms
c          Fill multiplication factors for stress terms
c          Correct contribution of stress terms for tangential boundary cells
c          Correct contribution of stress terms for half cells
c          distribute stress terms
c
c    Finally the folding subroutine
c    All unknowns outside the region are folded insidewards
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data iperm/1,2,3,1,2/
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Compute constants

      ipiindsc = iinput(6)
      iprintim = iinput(29)
      ni       = iinput(51)
      nj       = iinput(52)
      nk       = iinput(53)
      nvirtual = iinput(54)
      nconct   = infmat(6)
      deltat   = rinput(iprintim -1 +4)
      npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*(nk+1+2*nvirtual)

c     --- Compute starting addresses in array work
c         The present length is equal to 72+6+3+2*3+9+18=114 * npointsv

      ipcva    = 1
      ipadir   = ipcva+72*npointsv
      ipxcentr = ipadir+3*npointsv
      iprho    = ipxcentr+3*npointsv
      iprhovel = iprho+npointsv
      ipmu     = iprhovel+npointsv
      ipveloc  = ipmu+npointsv
      ipadiri  = ipveloc+3*npointsv
      ipadirii = ipadiri+3*npointsv
      ipprems  = ipadirii+9*npointsv
      ipgeo    = ipxcentr
      acttyp   = 0

c     --- Clear array matrix
c         Clear array rhside

      call issetcon ( matrix, (ni+1)*(nj+1)*(nk+1)*nconct*3, 0d0 )
      call issetcon ( rhside, (ni+1)*(nj+1)*(nk+1)*3, 0d0 )
      call issetcon ( gmat, (ni+1)*(nj+1)*(nk+1)*30, 0d0 )

      call issetcon ( work(ipxcentr), 3*npointsv, 0d0 )
      call issetcon ( work(ipadir), 3*npointsv, 0d0 )
      call issetcon ( work(iprho), npointsv, 0d0 )
      call issetcon ( work(ipveloc), 3*npointsv, 0d0 )
      call issetcon ( work(iprhovel), npointsv, 0d0 )
      call issetcon ( work(ipadiri), 3*npointsv, 0d0 )
      call issetcon ( work(ipadirii), 9*npointsv, 0d0 )
      call issetcon ( work(ipprems), 18*npointsv, 0d0 )

c     --- Fill covariant basis vectors in cva for all integration points

      call issetcon ( work(ipcva), 72*npointsv, 0d0 )
      call iscova3d ( coor, work(ipcva), ni, nj, nk, nvirtual, npf,
     +                rspv, cv )

c     --- Fill coordinates of centroid in work from position ipxcentr

      call isxcentr3 ( coor, work(ipxcentr), ni, nj, nk, nvirtual )

      do alphad = 1, 3

c     --- Build time-derivative part of the matrix and rhside
c         Interpolate rho to integration point (Cell centre)

         iarea(1,1) = 1
         iarea(1,2) = 1
         iarea(1,3) = 1
         iarea(2,1) = ni
         iarea(2,2) = nj
         iarea(2,3) = nk
         if ( alphad.eq.1 ) then
            iarea(2,1) = ni+1
            pnttypcn     = 7
         else if ( alphad.eq.2 ) then
            iarea(2,2) = nj+1
            pnttypcn     = 6
         else
            iarea(2,3) = nk+1
            pnttypcn     = 4
         end if
         vartyp = 4
         ipmatrix = 1+(alphad-1)*(ni+1)*(nj+1)*(nk+1)
         iprhside = ipmatrix

c        --- Interpolate rho to V^alpha point

         call isintcf3 ( coefs(1,1), work(iprho), ni, nj, nk, nvirtual,
     +                   npf, rspv, cv, acttyp, vartyp, pnttypcn )

c        --- Store a^alpha vector in work from position adir

         call iscont3d ( work(ipadir), jac, work(ipcva), ni,
     +                   nj, nk, nvirtual, alphad )
         call ismomtd3 ( ni, nj, nk, nvirtual, work(iprho),
     +                   solut(1,alphad), matrix(ipmatrix),
     +                   rhside(iprhside), deltat, iarea )

c        --- Make list of integration points, meant for internal cells only

         vartyp = alphad
         call ismakmp3 ( vartyp, ptsused, vtlist, lptlist, codelist )

c        --- Loop over integration points

         do i = 2, 7

c        --- Compute some general constants and arrays with respect to
c            integration point

            pnttyp = codelist(i)

c           --- Shift with respect to integration cell center

            reuse = mod(i,2).eq.1

c           --- Shift to compute cell index of integration point
c               relative to cell center

            dcell(1)  = vtlist(1,i)
            dcell(2)  = vtlist(2,i)
            dcell(3)  = vtlist(3,i)

            locdx(1) = lptlist(1,i)
            locdx(2) = lptlist(2,i)
            locdx(3) = lptlist(3,i)

            if ( iinput(ipiindsc).eq.1 ) then

c           --- iindsc(1)=1, use convection term
c               First clear array vsum

               call issetcon ( vsum, 3*npointsv, 0d0 )

c              --- Fill some coefficients

               if ( .not. reuse ) then

c              --- rho

                  vartyp = 4
                  call isintcf3 ( coefs(1,1), work(iprho), ni, nj, nk,
     +                            nvirtual, npf, rspv, cv, acttyp,
     +                            vartyp, pnttyp )

c                 --- Fill velocity componentwise (only Picard)

                  idir = iperm(i/2+alphad-1)
                  vartyp = idir
                  iptemp = ipveloc+(idir-1)*npointsv
                  call isintcf3 ( solut(1,idir), work(iptemp), ni,
     +                            nj, nk, nvirtual, npf, rspv, cv,
     +                            acttyp, vartyp, pnttyp )

c                 --- Compute rho V^i/sqrt(g) in integration points
c                     Store in work from position iprhovel

                  call isaxyz1 ( ni, nj, nk, nvirtual, work(iprho),
     +                           invjac(1,pnttyp), work(iptemp),
     +                           work(iprhovel) )
               end if

c              --- Next fill vsum with contribution of convective terms to
c                  integration points

               call ismomcw3 ( ni, nj, nk, nvirtual, work(iprhovel),
     +                         work(ipadir),
     +                         work(ipcva+9*npointsv*(pnttyp-1)),
     +                         vsum, alphad, i, dcell )

c              --- Distribute computed values over the positions
c                  corresponding to the unknowns

               call isdisc3d ( ni, nj, nk, nvirtual, matrix,
     +                         rhside(iprhside), vsum, work(ipveloc),
     +                         mct, dcell, npf, rspv, cv, alphad,
     +                         pnttyp, iarea, iinput(ipiindsc), i )

            end if

            if ( iinput(ipiindsc+3-1).eq.0 ) then

c           --- Stress terms according to WesBeek
c               First clear array vsum

               call issetcon ( vsum, 9*npointsv, 0d0 )
               vartyp = 3
               acttyp = 0
               if ( .not. reuse ) then

c              --- The following quantities are only computed for the even
c                  integration points (2,4,6)
c                  They are reused for the odd points (3,5,7)
c                  First the viscosity is stored in array work from
c                  position ipmu

                  call isintcf3 ( coefs(1,2), work(ipmu), ni, nj, nk,
     +                            nvirtual, npf, rspv, cv, acttyp,
     +                            vartyp, pnttyp )

c                 --- Compute a^gamma, where gamma corresponds to the
c                     integration point (i=2,3: gamma=1;i=4,5: gamma=2;
c                                        i=6,7: gamma=3)

                  call isconti3 ( work(ipadiri), work(ipcva), ni, nj,
     +                            nk, nvirtual, locdx, alphad )

               end if

c              --- Compute c^1, c^2 and c^3
c                  c is used to express the gradient of the velocity in the
c                  integration point in terms of surrounding velocity vectors
c                  The vectors C^beta are stored in work from position ipadrii

               call iscontg3 ( work(ipadirii), work(ipcva), ni, nj,
     +                         nk, nvirtual, locdx, alphad )

c              --- Store the product mu sqrt(g) a^alpha a_gamma^beta
c                  in array vsum(gamma,.), where beta refers to the present
c                  integration point, gamma =1,2,3 and . = 1,2,3

               call ismst3wb ( ni, nj, nk, nvirtual, work(ipmu),
     +                         vsum, alphad, locdx, iarea,
     +                         work(ipadir), work(ipadiri) )

c              --- Store the coefficients of the Cartesian velocity in the
c                  6 points surrounding the integration point in array
c                  prems(gamma,theta), where gamma=1,2,3 refers to the
c                  velocity component and theta = 1...6 refers to the 6 points

               call isprems3 ( ni, nj, nk, nvirtual, locdx,
     +                         work(ipprems), vsum, work(ipadirii),
     +                         alphad, iarea )

c              --- The effect of prescribed velocties is substituted in prems
c                  the right-hand side is updated

               call isbcs3 ( alphad, ni, nj, nk,
     +                   work(ipprems), rhside, locdx, ibipoint,
     +                   ibindex, ibinfor, bndcon )

c              --- Distribution of matrix

               call isdswb3d ( alphad, ni, nj, nk, nvirtual, locdx,
     +                         iarea, work(ipprems), matrix,
     +                         work(ipcva), invjac, mct )
            end if

         end do

c        --- Compute g-matrix for internal unknowns

         call isgmatw3d ( ni, nj, nk, nvirtual, gmat, work(ipadir),
     +                    jac(1,pnttypcn), ibipoint, ibindex,
     +                    ibinfor, alphad, work(ipxcentr) )

      end do

c     --- Update the g-matrix for essential boundary conditions

      do iface = 1, 6

         ilow1  = ibindex(1,1,iface)
         ihigh1 = ibindex(2,1,iface)
         ilow2  = ibindex(1,2,iface)
         ihigh2 = ibindex(2,2,iface)
         call isgmtbw3d ( iface, ni, nj, nk, gmat, ilow1, ihigh1,
     +                    ilow2, ihigh2, ibinfor(ibipoint(iface)) )

      end do

c     --- Finally the folding subroutine
c         All unknowns outside the region are folded insidewards

      do iface = 1, 6

         ilow1  = ibindex(1,1,iface)
         ihigh1 = ibindex(2,1,iface)
         ilow2  = ibindex(1,2,iface)
         ihigh2 = ibindex(2,2,iface)

         call isbgeo3d ( iface, ni, nj, nk, nvirtual, ilow1, ihigh1,
     +                    ilow2, ihigh2, work(ipcva), work(ipgeo) )

         call ismf3dwb ( iface, ni, nj, nk, nconct, ilow1, ihigh1,
     +                    ilow2, ihigh2, mct, matrix, rhside,
     +                    ibinfor(ibipoint(iface)), work(ipgeo),
     +                    bndcon(1,ibipoint(iface)) )

      end do

c     --- Update the right-hand side with -Gp^n

      call ismtvc33 ( gmat, solut(1,4), rhside, ni, nj, nk, nvirtual,
     +                mctinvg )

c     --- Add the rest of the right-hand-side vector

      call ismomrh3 ( coefs(1,3), coefs(1,1), coor, work, gmat,
     +                mctinvg, rhside,
     +                work(1+3*npointsv), ni, nj, nk, nvirtual )

      end
