      subroutine ismomf2d ( ni, nj, nvirtual, mct, matrix, rhside,
     +                      ibinfor, ibipoint, bcgeo, bndcon )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.6    date 24-12-1998 Guus: adaptation for transition
c                                       multiblock fixed boundary
c        version 1.5    date 20-03-1996 Guus: Remove masspar
c        version 1.4    date 12-02-1996 Guus: Correction for corner
c                                       where two tangential velocs are given
c        version 1.3    date 15-11-1995 Guus: correction non-zero ut
c        version 1.2    date 28-09-1995 Kees V. use of subroutine isorient2
c        version 1.1    date 18-07-1995 Reduction of code for corners
c        version 1.0    date 06-03-1995 subroutine based on ideas of ismmbf
c
c   copyright (c) 1995-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    All parts of the matrix outside the region are folded to internal
c    domain for the momentum matrix in R2
c    The folding process depends on the types of boundary conditions
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, ibipoint(*), ibinfor(*),
     +        mct(2,2,-2:2,-2:2,-2:2)
      double precision bcgeo(8,*), rhside(ni+1,nj+1,2),
     +                 matrix(ni+1,nj+1,2,*), bndcon(2,*)

c     bcgeo     i    reserved
c     bndcon    i    reserved
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     matrix   i/o   reserved
c     mct       i    Integer matrix information
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside   i/o   reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer unut, uxuy, unsnt, snnut, walllaw, parabolic
      parameter( unut = 4, uxuy = 10, unsnt = 5, snnut = 6,
     +           walllaw = 9, parabolic= 12)
      logical ungiven, skip, utgiven, eliminate
      integer btype, ipibinf, ipoint, alphad, iconct, tsign,
     +        iconcto, iconctc, iconcti, iconctr, iconctl, k,
     +        ip, jp, xp(2), ndir, istep, jstep, icbtype,
     +        tdir, nsign, iside, istwo(2,2), iparea(2,2), nsignside
      double precision fact, gtt, gttdivsg, gntdivsg, un, ut, gnn, gnt

c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     btype     Type of boundary condition
c     eliminate If true the point must be eliminated
c     fact      Multiplication factor
c     gnn       Contravariant metric tensor
c     gnt       Contravariant metric tensor in nt direction
c     gntdivsg  Contravariant metric tensor in nt direction divided by sqrt(g)
c     gtt       Contravariant metric tensor in tt direction
c     gttdivsg  Contravariant metric tensor in tt direction divided by sqrt(g)
c     icbtype   Type of boundary condition with respect to centre of integration
c               cell
c     iconct    Loop variable for loop over all connections in the matrx
c     iconctc   Connection in matrix for central point
c     iconcti   Connection in matrix for internal point
c     iconctl   Connection in matrix for "left" point
c     iconcto   Connection in matrix for outside point
c     iconctr   Connection in matrix for "right" point
c     ip        Local counting variable i-direction
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     ipibinf   Starting address of array infor for present side minus one
c     ipoint    Address in array IBUFFR
c     iside     Sequence number of side
c     istep     Step in the i-direction
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     jp        Local counting variable j-direction
c     jstep     Step in the j-direction
c     k         General loop variable
c     ndir      Direction of normal vector on this side
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     nsignside nsign for adjacent side
c     parabolic Parabolic boundary conditions
c     skip      If true boundary condition must be skipped
c     snnut     Sigma_nn and u_t prescribed
c     tdir      Direction of tangent vector on this side
c     tsign     Inner product of tangential vector defined counter
c               clockwise with respect to the outward normal and the
c               base vector along the side
c     un        Normal velocity (physical)
c     ungiven   If true un is given
c     unsnt     U_n and sigma_nt prescribed
c     unut      U_n and u_t prescribed
c     ut        Tangential velocity (physical)
c     utgiven   If true U_tis  given in point
c     uxuy      U_x and u_y prescribed
c     walllaw   Law of the wall boundary condition
c     xp        Auxiliary array to calculate boundary point number
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISORIENT2 Computes ndir, nsign, tdir, tsign for a 2-dimensional
c               problem
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    The folding process will be carried out per side.
c    Some rows corresponding to unknowns in the neighbourhood of the rectangle
c    require special attention
c
c    With respect to the boundary conditions the following combinations
c    are important:
c
c    U^n and U^t given   (U^n = sqrt(g^nn) u_n,                     (1)
c                         U^t=(sqrt(g_tt) u_t - g_nt U^n)/g_tt)     (2)
c
c    u_n given, u_t not
c    u_t given, u_n not
c    u_t nor u_n given
c    multiblock
c    pericodical boundary conditions
c
c    For illustration purposes we limit ourselves to the lower side and
c    in case of a corner to the combination of lower side and left-hand side
c    With respect to the tangential cells we limit ourselves to the cells
c    2 until ni, hence all tangential velocities not at other boundaries
c
c    For those cells we have the following situation:
c
c          --(-2,2)     --(0,2)      --(2,2)
c
c                |(-1,1)       |(1,1)
c
c          --(-2,0)     --(0,0)      --(2,0)
c                 (-1,-1)       (1,-1)
c    - - - - - - | - - - - - - | - - - - - - - - - -  lower boundary
c
c          --(-2,-2)    --(0,-2)     --(2,-2)
c
c    The tangential unknowns (.,-2) are outside the region and must be
c    folded. The normal components (.,-1) may be known, in which case
c    the known values will be substituted.
c
c    The folding process is as follows:
c
c    a. Periodical and multiblock boundaries: no action should be taken
c    b. U^n and U^t given at the boundary (.,-1):
c            U^t(.,-2) = 2U^t(.,-1) - U^t(.,0)                         (3)
c    c. U^t not given:
c            U^t(.,-2) = 2U^t(.,0) - U^t(.,2)                          (4)
c    d. U^t given and U^n not:
c            At the points (-2,-2) and (2,-2) use formula (4)
c            At the point (0,-2) use a combination of (2) and (3):
c            U^t(0,-2) = 2U^t(0,-1) - U^t(0,0)                         (5)
c            U^t(0,-1) = (sqrt(g_tt) u_t - g_nt (U^n(1,-1)+U^n(-1,-1))/2)/g_tt)
c
c
c    With respect to the normal cells we first consider the cells
c    2 until ni-1, hence all normal velocities with a distance of at least
c    1.5 cell to other boundaries
c
c    For those cells we have the following situation:
c
c
c                |(-2,2)       |(0,2)        |(2,2)
c
c                       --(-1,1)     --(1,1)
c                 (-2,0)        (0,0)         (2,0)
c    - - - - - - | - - - - - - | - - - - - - | - - -   lower boundary
c
c                       --(-1,-1)    --(1,-1)
c
c                |(-2,-2)      |(0,-2)       |(2,-2)
c
c
c     Both the tangential unknowns U^t(.,-1) as the normal unknowns U^n(.,-2)
c     should be folded
c
c    The folding process is as follows:
c
c    a. Periodical and multiblock boundaries: no action should be taken
c    b. U^n given at the boundary: diagonal element 1, off-diagonal elements 0
c       Value in right-hand side
c    c. In all other situations we do:
c       - U^n(.,-2) = 2U^n(.,0) - U^n(.,2)                              (6)
c       If U^t given: Use formula (5) with adapted indices
c       else
c          at this moment use U^t(.,-1) = U^t(.,1) (See 91-09, page 34)  (7)
c
c    Finally we consider a cell at a corner
c
c                        |left boundary
c
c                |(-2,2) |     |(0,2)        |(2,2)
c
c                       --(-1,1)     --(1,1)
c                 (-2,0) |      (0,0)         (2,0)
c                |       - - - | - - - - - - | - - -   lower boundary
c
c                       --(-1,-1)    --(1,-1)
c
c                |(-2,-2)      |(0,-2)       |(2,-2)
c
c
c    The folding process is as follows:
c
c    a. If point (-2,0) does not correspond to periodical or multiblock bc's
c       nor U^n given then
c         - U^n(-2,-2) = ( 2U^n(0,-2)-U^n(2,-2) + 2U^n(-2,0)-U^n(-2,2) )/2   (8)
c         if ( ut given ) then
c            eliminate (-1,-1) using formula (5)
c            In the special case that along the neighbour boundary also only ut
c            is given we may not follow this approach since then U(-2,0) is
c            changed, but this a tangential velocity at the other side
c            which itself changes (-1,-1)
c
c            In this special case we have at both ut given and compute
c            un in the following way:
c
c            u = ut t + un n       = ut t + un n , where 1 refers to
c                  1 1    1 1          2 2    2 2
c            lower side and 2 to the left side
c
c            hence un  = ( ut  - ut (t ,t ) ) / (n ,t )
c                    1       2     1  1  2        1  2
c
c                  un  = ( ut  - ut (t ,t ) ) / (n ,t )
c                    2       1     2  2  1        2  1
c
c                  t  = a    / ||a   ||  t  = a    / ||a   ||
c                   1    (1)      (1)     2    (2)      (2)
c
c                        (2)      (2)          (1)      (1)
c                  n  = a   / ||a    ||  n  = a   / ||a    ||
c                   1                     2
c
c            This special possibility has not yet been programmed since it
c            is not at all clear if this situation ever occurs.
c            In the input subroutine this situation is recognized and
c            an error message is given.
c    b. All other points are eliminated at their respective boundaries
c
c
c                    PSEUDO CODE
c
c      For iside := 1 (1) 4 do
c         For 2 corner normal cells do
c            eliminate the point "(-2,-2)" if necessary
c         For tangential cells do
c            Perform the folding process over all outside connections
c         For normal cells do
c            Perform the folding process over all outside connections
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data istwo / 1, 2,
     +             2, 3 /
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Array mct is updated temporarily in order to get local co-ordinates

      mct(1,2,-1,-1,0 ) = mct(1,2,-1, 0,0 )
      mct(1,2, 1,-1,0 ) = mct(1,2, 0, 0,0 )
      mct(1,2, 1, 1,0 ) = mct(1,2, 0, 1,0 )
      mct(2,1,-1,-1,0 ) = mct(2,1, 0,-1,0 )
      mct(2,1,-1, 1,0 ) = mct(2,1, 0, 0,0 )
      mct(2,1, 1, 1,0 ) = mct(2,1, 1, 0,0 )

c     --- loop over faces

      do iside = 1, 4

c     --- Compute tdir and ndir

         call isorient2 ( iside, tdir, ndir, tsign, nsign )

         ipibinf = ibipoint(iside)+nvirtual-1

c        --- loop over normal first and last half cells
c            Compute loop bounds

         iparea(1,1) = 1
         iparea(1,2) = 1
         iparea(2,1) = 1
         iparea(2,2) = 1
         if ( iside.eq.2 ) then

c        --- iside = 2:  right-hand side

            iparea(1,1) = ni+1
            iparea(2,1) = ni+1

         else if ( iside.eq.4 ) then

c        --- iside = 4:  upper side

            iparea(1,2) = nj+1
            iparea(2,2) = nj+1

         end if
         if ( tdir.eq.1 ) then
            iparea(2,1) = ni
            istep = ni-1
            jstep = 1
         else
            iparea(2,2) = nj
            istep = 1
            jstep = nj-1
         end if
         alphad = ndir

c        --- loop over cells

         nsignside = 1
         do jp = iparea(1,2), iparea(2,2), jstep
            do ip = iparea(1,1), iparea(2,1), istep
               nsignside = -nsignside
               xp(1) = ip
               xp(2) = jp
               ipoint = ipibinf+2*xp(tdir)
               btype = ibinfor(ipoint)
               ungiven = btype.eq.parabolic .or. btype.eq.88 .or.
     +                   btype.eq.unut .or. btype.eq.uxuy .or.
     +                   btype.eq.unsnt .or. btype.eq.walllaw
               skip = btype.ge.89 .and. btype.le.99
               eliminate = .not. ungiven .and. .not. skip
               if ( skip .and. .not. eliminate ) then

c              --- Periodical or multiblock boundary
c                  Find type of boundary condition in virtual part of boundary

                  btype = ibinfor(ipibinf+2*(xp(tdir)+nsignside))
                  skip = btype.ge.89 .and. btype.le.99
                  eliminate = .not. skip

               end if
               if ( eliminate ) then

c              --- No periodical nor multiblock boundary
c                  The point u^n(-2,-2) is eliminated

                  if ( tdir.eq.1 ) then
                     iconct = mct(alphad,ndir,nsignside,nsign,0)
                     fact = matrix(ip,jp,alphad,iconct)
                     matrix(ip,jp,alphad,iconct) = 0d0
                     iconct = mct(alphad,ndir,nsignside,0,0)
                     matrix(ip,jp,alphad,iconct) =
     +                  matrix(ip,jp,alphad,iconct) + fact
                     iconct = mct(alphad,ndir,nsignside,-nsign,0)
                     matrix(ip,jp,alphad,iconct) =
     +                  matrix(ip,jp,alphad,iconct) - fact*0.5d0
                     iconct = mct(alphad,ndir,0,nsign,0)
                     matrix(ip,jp,alphad,iconct) =
     +                  matrix(ip,jp,alphad,iconct) + fact
                     iconct = mct(alphad,ndir,-nsignside,nsign,0)
                     matrix(ip,jp,alphad,iconct) =
     +                  matrix(ip,jp,alphad,iconct) - fact*0.5d0
                  else
                     iconct = mct(alphad,ndir,nsign,nsignside,0)
                     fact = matrix(ip,jp,alphad,iconct)
                     matrix(ip,jp,alphad,iconct) = 0d0
                     iconct = mct(alphad,ndir,0,nsignside,0)
                     matrix(ip,jp,alphad,iconct) =
     +                  matrix(ip,jp,alphad,iconct) + fact
                     iconct = mct(alphad,ndir,-nsign,nsignside,0)
                     matrix(ip,jp,alphad,iconct) =
     +                  matrix(ip,jp,alphad,iconct) - fact*0.5d0
                     iconct = mct(alphad,ndir,nsign,0,0)
                     matrix(ip,jp,alphad,iconct) =
     +                  matrix(ip,jp,alphad,iconct) + fact
                     iconct = mct(alphad,ndir,nsign,-nsignside,0)
                     matrix(ip,jp,alphad,iconct) =
     +                  matrix(ip,jp,alphad,iconct) - fact*0.5d0
                  end if
                  ipoint = ipibinf+2*xp(tdir)+nsignside
                  btype = ibinfor(ipoint)
                  if ( btype.eq.snnut ) then

c                 --- ut is given use formula (5) except if at the other
c                     boundary also only ut is given
c                     Find type of boundary condition in virtual part
c                     of boundary (not yet programmed)
c                     Standard situation use formula (5)

                     k = nsignside
                     if ( tdir.eq.1 ) then
                        iconcto = mct ( alphad, tdir, k, nsign, 0 )
                        iconcti = mct ( alphad, tdir, k,-nsign, 0 )
                        iconctl = mct ( alphad, ndir, (k-1)/2, 0,0 )
                        iconctr = mct ( alphad, ndir, (k+1)/2, 0,0 )
                     else
                        iconcto = mct ( alphad, tdir, nsign, k, 0 )
                        iconcti = mct ( alphad, tdir,-nsign, k, 0 )
                        iconctl = mct ( alphad, ndir, 0, (k-1)/2, 0)
                        iconctr = mct ( alphad, ndir, 0, (k+1)/2, 0)
                     end if
                     fact = matrix(ip,jp,alphad,iconcto)
                     matrix(ip,jp,alphad,iconcto) = 0d0
                     gttdivsg = bcgeo(istwo(ndir,ndir),ipoint)
                     gntdivsg =-bcgeo(istwo(ndir,tdir),ipoint)
                     gtt = bcgeo(8,ipoint)*gttdivsg
                     ut = bcgeo(8,ipoint)*bndcon(2,ipoint)*tsign
                     matrix(ip,jp,alphad,iconcti) =
     +                  matrix(ip,jp,alphad,iconcti)-fact
                     rhside(ip,jp,alphad) = rhside(ip,jp,alphad) -
     +                  2d0*fact/sqrt(gtt)*ut
                     fact = fact*gntdivsg/gttdivsg
                     matrix(ip,jp,alphad,iconctl) =
     +                  matrix(ip,jp,alphad,iconctl)-fact
                     matrix(ip,jp,alphad,iconctr) =
     +                  matrix(ip,jp,alphad,iconctr)-fact

                  end if

               end if

            end do

         end do

      end do

      do iside = 1, 4

c     --- Compute tdir and ndir

         call isorient2 ( iside, tdir, ndir, tsign, nsign )

         ipibinf = ibipoint(iside)+nvirtual-1

c        --- loop over tangential cells
c            Compute loop bounds

         iparea(1,1) = 1
         iparea(1,2) = 1
         iparea(2,1) = 1
         iparea(2,2) = 1

         if ( iside.le.2 ) then

c        --- iside = 1, 2:  left- and right-hand side

            iparea(2,2) = nj+1
            if ( iside.eq.2 ) then

c           --- iside = 2:  right-hand side

               iparea(1,1) = ni
               iparea(2,1) = ni

            end if

         else

c        --- iside = 3, 4:  lower- and upper side

            iparea(2,1) = ni+1
            if ( iside.eq.4 ) then

c           --- iside = 4:  upper side

               iparea(1,2) = nj
               iparea(2,2) = nj

            end if

         end if

c        --- Loop over three tangential unknowns outside the region

         alphad = tdir
         if ( tdir.eq.1 ) then
            iconctl = mct ( alphad, ndir, -1, nsign, 0 )
            iconctr = mct ( alphad, ndir,  1, nsign, 0 )
         else
            iconctl = mct ( alphad, ndir, nsign, -1, 0 )
            iconctr = mct ( alphad, ndir, nsign,  1, 0 )
         end if
         do k = -1, 1

            if ( tdir.eq.1 ) then
               iconcto = mct ( alphad, tdir, k, nsign, 0 )
               iconctc = mct ( alphad, tdir, k,     0, 0 )
               iconcti = mct ( alphad, tdir, k,-nsign, 0 )
            else
               iconcto = mct ( alphad, tdir, nsign, k, 0 )
               iconctc = mct ( alphad, tdir,     0, k, 0 )
               iconcti = mct ( alphad, tdir,-nsign, k, 0 )
            end if

c           --- loop over cells

            do 200 jp = iparea(1,2), iparea(2,2)
               do 150 ip = iparea(1,1), iparea(2,1)
                  xp(1) = ip
                  xp(2) = jp
                  ipoint = ipibinf+2*(xp(tdir)+k)-1
                  btype = ibinfor(ipoint)
                  skip = btype.ge.89 .and. btype.le.99
                  if ( .not. skip .and. btype.eq.88 .and. k.ne.0 ) then

c                 --- Transition point found, not below center of cell
c                     Act as cell center

                     icbtype =ibinfor(ipibinf+2*xp(tdir)-1)
                     skip = icbtype.ge.89 .and. icbtype.le.99

                  end if
                  if ( .not. skip ) then
                     if ( btype.eq.parabolic .or. btype.eq.88 ) then
                        btype = unut
                     else if ( btype.eq.walllaw ) then
                        btype = unsnt
                     end if
                     utgiven = btype.eq.snnut .or. btype.eq.unut .or.
     +                         btype.eq.uxuy
                     fact = matrix(ip,jp,alphad,iconcto)
                     matrix(ip,jp,alphad,iconcto) = 0d0
                     if ( .not. utgiven .or. btype.eq.snnut .and.
     +                    k.ne.0 )then

c                    --- U_t has not been given use formula (4) or
c                        U_t has been given and U_n not in points k = -1, 1

                        matrix(ip,jp,alphad,iconctc) =
     +                     matrix(ip,jp,alphad,iconctc)+2d0*fact
                        matrix(ip,jp,alphad,iconcti) =
     +                     matrix(ip,jp,alphad,iconcti)-fact

                     else if ( btype.eq.snnut ) then

c                    --- U_t has been given and U_n not
c                        use formula (5) at the points k = 0

                        gttdivsg = bcgeo(istwo(ndir,ndir),ipoint)
                        gntdivsg =-bcgeo(istwo(ndir,tdir),ipoint)
                        gtt = bcgeo(8,ipoint)*gttdivsg
                        ut = bndcon(2,ipoint)*tsign*bcgeo(8,ipoint)
                        matrix(ip,jp,tdir,iconctc) =
     +                     matrix(ip,jp,alphad,iconctc)-fact
                        rhside(ip,jp,alphad) = rhside(ip,jp,alphad) -
     +                     2d0*fact/sqrt(gtt)*ut
                        fact = fact*gntdivsg/gttdivsg
                        matrix(ip,jp,tdir,iconctl) =
     +                     matrix(ip,jp,alphad,iconctl)-fact
                        matrix(ip,jp,tdir,iconctr) =
     +                     matrix(ip,jp,alphad,iconctr)-fact
                     else
                        if ( btype.eq.unut ) then

c                       --- un and ut given

                           un = bndcon(1,ipoint)*nsign
                           ut = bndcon(2,ipoint)*tsign
                           gttdivsg = bcgeo(istwo(ndir,ndir),ipoint)
                           gntdivsg =-bcgeo(istwo(ndir,tdir),ipoint)
                           gnn = gttdivsg/bcgeo(8,ipoint)
                           gtt = bcgeo(8,ipoint)*gttdivsg
                           gnt = bcgeo(8,ipoint)*gntdivsg
                           un = sqrt(gnn)*un
                           ut = (sqrt(gtt)*ut-gnt*un)/gtt
                           un = bcgeo(8,ipoint)*un
                           ut = bcgeo(8,ipoint)*ut

                        else

c                       --- ux and uy given

                           ut = bcgeo(2+2*tdir,ipoint)*bndcon(1,ipoint)+
     +                          bcgeo(3+2*tdir,ipoint)*bndcon(2,ipoint)

                        end if

                        matrix(ip,jp,alphad,iconctc) =
     +                     matrix(ip,jp,alphad,iconctc)-fact
                        rhside(ip,jp,alphad) = rhside(ip,jp,alphad) -
     +                     2d0*fact*ut

                     end if

                  end if

c                 --- Only if all tangential velocities are treated we may
c                     substitute the values for the normal velocities if
c                     given, since the matrix elements may have been updated
c                     during elimination

                  if ( k.eq.1 ) then

c                 --- k = 1, eliminate normal velocities at the boundary
c                     if given (All tangential velocities corresponding to
c                     this cell are treated.
c                     "Left" point:

                     ipoint = ipibinf+2*xp(tdir)-2
                     btype = ibinfor(ipoint)
                     ungiven = .false.
                     if ( btype.eq.unut .or. btype.eq.parabolic .or.
     +                    btype.eq.88 .or. btype.eq.unsnt .or.
     +                    btype.eq.walllaw ) then

                        un = bndcon(1,ipoint)*nsign
                        gttdivsg = bcgeo(istwo(ndir,ndir),ipoint)
                        gnn = gttdivsg*bcgeo(8,ipoint)
                        un = sqrt(gnn)*un
                        ungiven = .true.
                     else if ( btype.eq.uxuy ) then
                        un = bcgeo(2+2*ndir,ipoint)*bndcon(1,ipoint)+
     +                       bcgeo(3+2*ndir,ipoint)*bndcon(2,ipoint)
                        ungiven = .true.
                     end if
                     if ( ungiven ) then

c                    --- un is given

                        rhside(ip,jp,alphad) = rhside(ip,jp,alphad) -
     +                         matrix(ip,jp,alphad,iconctl)*un
                        matrix(ip,jp,alphad,iconctl) = 0d0

                     end if

c                    --- "Right" point:

                     ipoint = ipibinf+2*xp(tdir)
                     btype = ibinfor(ipoint)
                     ungiven = .false.
                     if ( btype.eq.unut .or. btype.eq.parabolic .or.
     +                    btype.eq.88 .or. btype.eq.unsnt .or.
     +                    btype.eq.walllaw ) then

                        un = bndcon(1,ipoint)*nsign
                        gttdivsg = bcgeo(istwo(ndir,ndir),ipoint)
                        gnn = gttdivsg*bcgeo(8,ipoint)
                        un = sqrt(gnn)*un
                        ungiven = .true.
                     else if ( btype.eq.uxuy ) then
                        un = bcgeo(2+2*ndir,ipoint)*bndcon(1,ipoint)+
     +                       bcgeo(3+2*ndir,ipoint)*bndcon(2,ipoint)
                        ungiven = .true.
                     end if
                     if ( ungiven ) then

c                    --- un is given

                        rhside(ip,jp,alphad) = rhside(ip,jp,alphad) -
     +                         matrix(ip,jp,alphad,iconctr)*un
                        matrix(ip,jp,alphad,iconctr) = 0d0

                     end if

                  end if

150            continue

200         continue

         end do

c        --- loop over normal half cells
c            Compute loop bounds

         iparea(1,1) = 1
         iparea(1,2) = 1
         iparea(2,1) = 1
         iparea(2,2) = 1
         if ( iside.le.2 ) then

c        --- iside = 1, 2:  left- and right-hand side

            iparea(2,2) = nj
            if ( iside.eq.2 ) then

c           --- iside = 2:  right-hand side

               iparea(1,1) = ni+1
               iparea(2,1) = ni+1

            end if

         else

c        --- iside = 3, 4:  lower- and upper side

            iparea(2,1) = ni
            if ( iside.eq.4 ) then

c           --- iside = 4:  upper side

               iparea(1,2) = nj+1
               iparea(2,2) = nj+1

            end if

         end if
         alphad = ndir

c        --- loop over cells

         do 400 jp = iparea(1,2), iparea(2,2)
            do 350 ip = iparea(1,1), iparea(2,1)
               xp(1) = ip
               xp(2) = jp
               ipoint = ipibinf+2*xp(tdir)
               btype = ibinfor(ipoint)
               ungiven = btype.eq.parabolic .or. btype.eq.88 .or.
     +                   btype.eq.unut .or. btype.eq.uxuy .or.
     +                   btype.eq.unsnt .or. btype.eq.walllaw
               skip = btype.ge.89 .and. btype.le.99
               if ( ungiven ) then

c              --- Normal velocity given, set diagonal to 1 and
c                  off-diagonal to 0

                  matrix(ip,jp,alphad,1 ) = 1d0
                  do k = 2, 13
                     matrix(ip,jp,alphad,k ) = 0d0
                  end do
                  if ( btype.eq.uxuy ) then
                     un = bcgeo(2+2*ndir,ipoint)*bndcon(1,ipoint)+
     +                    bcgeo(3+2*ndir,ipoint)*bndcon(2,ipoint)
                  else
                     un = bndcon(1,ipoint)*nsign
                     gttdivsg = bcgeo(istwo(ndir,ndir),ipoint)
                     gnn = gttdivsg*bcgeo(8,ipoint)
                     un = sqrt(gnn)*un
                  end if
                  rhside(ip,jp,alphad ) = un

               else

c              --- Normal velocity not given

                  if ( .not. skip ) then

c                 --- Loop over two tangential unknowns outside the region

                     do k = -1, 1, 2
                        if ( tdir.eq.1 ) then
                           iconcto = mct ( alphad, tdir, k, nsign, 0 )
                           iconcti = mct ( alphad, tdir, k,-nsign, 0 )
                           iconctl = mct ( alphad, ndir, (k-1)/2, 0,0 )
                           iconctr = mct ( alphad, ndir, (k+1)/2, 0,0 )
                        else
                           iconcto = mct ( alphad, tdir, nsign, k, 0 )
                           iconcti = mct ( alphad, tdir,-nsign, k, 0 )
                           iconctl = mct ( alphad, ndir, 0, (k-1)/2, 0)
                           iconctr = mct ( alphad, ndir, 0, (k+1)/2, 0)
                        end if
                        ipoint = ipibinf+2*xp(tdir)+k
                        btype = ibinfor(ipoint)
                        utgiven = btype.eq.snnut .or. btype.eq.unut.or.
     +                            btype.eq.uxuy .or. btype.eq.88 .or.
     +                            btype.eq.parabolic .or.
     +                            btype.eq.walllaw
                        fact = matrix(ip,jp,alphad,iconcto)
                        matrix(ip,jp,alphad,iconcto) = 0d0
                        if ( utgiven ) then

c                       --- ut is given use formula (5)

                           gttdivsg = bcgeo(istwo(ndir,ndir),ipoint)
                           gntdivsg =-bcgeo(istwo(ndir,tdir),ipoint)
                           gtt = bcgeo(8,ipoint)*gttdivsg
                           ut = bcgeo(8,ipoint)*bndcon(2,ipoint)*tsign
                           matrix(ip,jp,alphad,iconcti) =
     +                        matrix(ip,jp,alphad,iconcti)-fact
                           rhside(ip,jp,alphad) = rhside(ip,jp,alphad) -
     +                        2d0*fact/sqrt(gtt)*ut
                           fact = fact*gntdivsg/gttdivsg
                           matrix(ip,jp,alphad,iconctl) =
     +                        matrix(ip,jp,alphad,iconctl)-fact
                           matrix(ip,jp,alphad,iconctr) =
     +                        matrix(ip,jp,alphad,iconctr)-fact

                        else

c                       --- ut not given use formula (7)

                           matrix(ip,jp,alphad,iconcti) =
     +                        matrix(ip,jp,alphad,iconcti)+fact

                        end if

                     end do

                  end if

c                 --- Loop over three normal unknowns outside the region

                  do k = -1, 1
                     ipoint = ipibinf+2*(xp(tdir)+k)
                     btype = ibinfor(ipoint)
                     skip = btype.ge.89 .and. btype.le.99
                     if ( .not. skip ) then
                        if ( tdir.eq.1 ) then
                           iconcto = mct ( alphad, ndir, k, nsign, 0 )
                           iconcti = mct ( alphad, ndir, k, -nsign, 0 )
                           iconctc = mct ( alphad, ndir, k, 0, 0 )
                        else
                           iconcto = mct ( alphad, ndir, nsign, k, 0 )
                           iconcti = mct ( alphad, ndir, -nsign, k, 0 )
                           iconctc = mct ( alphad, ndir, 0, k, 0 )
                        end if
                        fact = matrix(ip,jp,alphad,iconcto)
                        matrix(ip,jp,alphad,iconcto) = 0d0
                        matrix(ip,jp,alphad,iconctc) =
     +                     matrix(ip,jp,alphad,iconctc)+2d0*fact
                        matrix(ip,jp,alphad,iconcti) =
     +                     matrix(ip,jp,alphad,iconcti)-fact

                      end if

                   end do

                end if

350         continue

400      continue

      end do

c     --- Reset array mct

      mct(1,2,-1,-1,0 ) = -1
      mct(1,2, 1,-1,0 ) = -1
      mct(1,2, 1, 1,0 ) = -1
      mct(2,1,-1,-1,0 ) = -1
      mct(2,1,-1, 1,0 ) = -1
      mct(2,1, 1, 1,0 ) = -1

      end
