      subroutine ispmat3d( gmat, dgmat, ni, nj, nk, ndim )
c=======================================================================
c
c        programmer     Kees Kassels
c        version 1.0    date 06-02-96
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c***********************************************************************
c
c                         DESCRIPTION
c     is3dgm is a subroutine which computes the matrix  Dm*{- (R inv)*G}
c
c***********************************************************************
c
c                         INPUT / OUTPUT PARAMETERS
c
      integer           ni, nj, nk, ndim
      double precision  gmat( ni+1, nj+1, nk+1, ndim, 10 ),
     .                  dgmat( ni+1, nj+1, nk+1, 19 )

c
c     dgmat     i    Divergence of gradient
c     gmat      i    Gradient matrix
c     ndim      i    Dimension of space (2 or 3)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c
c***********************************************************************
c
c                        LOCAL PARAMETERS
c
      integer ip, jp, kp, i
c
c     i         General loop variable
c     ip        Local counting variable i-direction
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c
c
c***********************************************************************
c
c     * initialise dgmat
c
      do 30 ip = 1,ni+1
         do 30 jp = 1,nj+1
            do 30 kp = 1,nk+1
              do 30 i = 1, 19
                 dgmat(ip,jp,kp,i) = 0d0
30    continue

      do 20 ip = 1, ni
      do 20 jp = 1, nj
      do 20 kp = 1, nk

         dgmat(ip,jp,kp, 1) =   gmat(ip+1,jp  ,kp  ,1, 3)
     +                        + gmat(ip  ,jp+1,kp  ,2, 3)
     +                        + gmat(ip  ,jp  ,kp+1,3, 3)
     +                        - gmat(ip  ,jp  ,kp  ,1, 4)
     +                        - gmat(ip  ,jp  ,kp  ,2, 4)
     +                        - gmat(ip  ,jp  ,kp  ,3, 4)

         dgmat(ip,jp,kp, 2) = - gmat(ip  ,jp  ,kp  ,1, 1)
     +                        - gmat(ip  ,jp  ,kp  ,2, 7)

         dgmat(ip,jp,kp, 3) = + gmat(ip+1,jp  ,kp  ,1, 1)
     +                        + gmat(ip  ,jp  ,kp+1,3, 7)
     +                        - gmat(ip  ,jp  ,kp  ,1, 2)
     +                        - gmat(ip  ,jp  ,kp  ,2, 3)
     +                        - gmat(ip  ,jp  ,kp  ,3, 8)

         dgmat(ip,jp,kp, 4) =   gmat(ip+1,jp  ,kp  ,1, 2)
     +                        - gmat(ip  ,jp  ,kp  ,2, 9)

         dgmat(ip,jp,kp, 5) =   gmat(ip  ,jp+1,kp  ,2, 7)
     +                        + gmat(ip  ,jp  ,kp+1,3, 1)
     +                        - gmat(ip  ,jp  ,kp  ,1, 3)
     +                        - gmat(ip  ,jp  ,kp  ,2, 8)
     +                        - gmat(ip  ,jp  ,kp  ,3, 2)

         dgmat(ip,jp,kp, 6) =   gmat(ip+1,jp  ,kp  ,1, 4)
     +                        + gmat(ip  ,jp+1,kp  ,2, 9)
     +                        + gmat(ip  ,jp  ,kp+1,3, 5)
     +                        - gmat(ip  ,jp  ,kp  ,2,10)
     +                        - gmat(ip  ,jp  ,kp  ,3, 6)

         dgmat(ip,jp,kp, 7) =   gmat(ip  ,jp+1,kp  ,2, 8)
     +                        - gmat(ip  ,jp  ,kp  ,1, 5)

         dgmat(ip,jp,kp, 8) =   gmat(ip+1,jp  ,kp  ,1, 5)
     +                        + gmat(ip  ,jp+1,kp  ,2, 4)
     +                        + gmat(ip  ,jp  ,kp+1,3, 9)
     +                        - gmat(ip  ,jp  ,kp  ,1, 6)
     +                        - gmat(ip  ,jp  ,kp  ,3,10)

         dgmat(ip,jp,kp, 9) =   gmat(ip+1,jp  ,kp  ,1, 6)
     +                        + gmat(ip  ,jp+1,kp  ,2,10)

         dgmat(ip,jp,kp,10) = - gmat(ip  ,jp  ,kp  ,2, 1)
     +                        - gmat(ip  ,jp  ,kp  ,3, 7)

         dgmat(ip,jp,kp,11) = - gmat(ip  ,jp  ,kp  ,1, 7)
     +                        - gmat(ip  ,jp  ,kp  ,3, 1)

         dgmat(ip,jp,kp,12) =   gmat(ip+1,jp  ,kp  ,1, 7)
     +                        + gmat(ip  ,jp+1,kp  ,2, 1)
     +                        - gmat(ip  ,jp  ,kp  ,1, 8)
     +                        - gmat(ip  ,jp  ,kp  ,2, 2)
     +                        - gmat(ip  ,jp  ,kp  ,3, 3)

         dgmat(ip,jp,kp,13) =   gmat(ip+1,jp  ,kp  ,1, 8)
     +                        - gmat(ip  ,jp  ,kp  ,3, 5)

         dgmat(ip,jp,kp,14) =   gmat(ip  ,jp+1,kp  ,2, 2)
     +                        - gmat(ip  ,jp  ,kp  ,3, 9)

         dgmat(ip,jp,kp,15) =   gmat(ip  ,jp  ,kp+1,3, 8)
     +                        - gmat(ip  ,jp  ,kp  ,2, 5)

         dgmat(ip,jp,kp,16) =   gmat(ip  ,jp  ,kp+1,3, 2)
     +                        - gmat(ip  ,jp  ,kp  ,1, 9)

         dgmat(ip,jp,kp,17) =   gmat(ip+1,jp  ,kp  ,1, 9)
     +                        + gmat(ip  ,jp+1,kp  ,2, 5)
     +                        + gmat(ip  ,jp  ,kp+1,3, 4)
     +                        - gmat(ip  ,jp  ,kp  ,1,10)
     +                        - gmat(ip  ,jp  ,kp  ,2, 6)

         dgmat(ip,jp,kp,18) =   gmat(ip+1,jp  ,kp  ,1,10)
     +                        + gmat(ip  ,jp  ,kp+1,3, 6)

         dgmat(ip,jp,kp,19) =   gmat(ip  ,jp+1,kp  ,2, 6)
     +                        + gmat(ip  ,jp  ,kp+1,3,10)

20    continue

      end
