      subroutine isprems ( ni, nj, nvirtual, locdx, prems,  dvsum,
     +                     adirii, alphad, iarea , mcom)
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.2    date 04-03-1999 bug fixed compressible case
c        version 1.1    date 16-02-1999 extension for compressible case
c                                       extra term -2/3*divu
c        version 1.0    date 12-07-1996
c
c   copyright (c) 1996-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Computation of stress terms before distribution, Wesbeek
c    discretization.
c **********************************************************************
c
c                       KEYWORDS
c
c     distribution
c     discretization
c     matrix
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, locdx(2), alphad, iarea(1:2,2), mcom
      double precision prems(1:ni+1,1:nj+1,2,4),
     +                 dvsum(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2,2),
     +                 adirii(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2,2)

c     adirii    i    Contravariant base vector in integration point
c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     dvsum     i    Flux derivative contributions to the matrix
c     iarea     i    Area of arrays
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     mcom      i    If zero the flow is incompressible else (1) compressible
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     prems     i    Intermediate matrix for building the stress term according
c                    to the Wesbeek discretization.
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, ip, jp, gammad, deltad, idir, signg, ish, jsh

c     deltad    Direction index
c     gammad    Direction index
c     i         General loop variable
c     idir      General loop variable for 1...ndim loop
c     ip        Local counting variable i-direction
c     ish       Shift in i-direction
c     jp        Local counting variable j-direction
c     jsh       Shift in j-direction
c     signg     Integration sign
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    In case of the Wesbeek discretization we have the following expressions
c    for the stress terms (alpha cell):
c
c
c         (a)               (1)  b |i+1/2,j              (2)  b |i,j+1/2
c        a    . (  sqrt(g) a    T  |         +   sqrt(g) a   T  |        )
c                            b     |i-1/2,j                b    |i,j-1/2
c          (i,j)
c
c
c      in dvsum(i,j,gamma,1:2) the vector for the inproduct of
c
c
c         (a)               (1)     gamma
c        a    .    sqrt(g) a       T       is stored
c                            gamma
c          (i,j)
c
c
c                                              (1)     (a)
c        so : dvsum(i,j,gamma,j) = mu sqrt(g) a        a
c                                              gamma   j
c
c
c    In this routine the inproduct is computed of dvsum and the stress
c    tensor.
c                      (gamma)
c     prems = dvsum . T
c
c
c    for the stress tensor we fill in
c
c
c                    1              gamma
c                  du             du
c                  --- +     ---
c                    gamma          1
c       (gamma)    dx             dx
c     T         =
c
c                    2              gamma
c                  du             du
c                  --- +     ---
c                    gamma          2
c                  dx             dx
c
c
c
c    The derivatives of the cartesian velocities will be computed
c    according to next formula
c                                       (0,0)                  (-1,1)
c                                      |                      |
c            alpha      (1)      alpha |       (2)      alpha |
c      grad u       =  a        u      |  +   a        u      |
c            (-1,0)     (-1,0)         |       (-1,0)         |
c                                      |                      |
c                                       (-2,0)                 (-1,-1)
c
c    In this way we get per integration point a contribution to the
c    cartesian velocity components in four points surrounding the
c    integration point.
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      do i = 1, 4

         do gammad = 1, 2

            do deltad = 1, 2

               do jp = iarea(1,2), iarea(2,2)
                  do ip = iarea(1,1), iarea(2,1)

                     prems(ip,jp,deltad,i) = 0d0

                  end do
               end do

            end do

         end do

      end do

      if ( alphad.eq.1 ) then
         ish = (locdx(1)-1)/2
         jsh = (locdx(2)+1)/2
      else
         ish = (locdx(1)+1)/2
         jsh = (locdx(2)-1)/2
      end if

      do i = 1, 4

c     --- sign = -1, 1, -1, 1

         signg = (-1)**i

         if ( alphad.eq.1 ) then

c        --- idir = 1, 1, 2, 2

            if ( i.lt.3 ) then
               idir = 1
            else
               idir = 2
            end if

         else

c        --- idir = 2, 2, 1, 1

            if ( i.lt.3 ) then
               idir = 2
            else
               idir = 1
            end if

         end if

         if ( mcom.eq.1) then

c        --- compressible case

            do gammad = 1, 2

               do deltad = 1, 2

                  do jp = iarea(1,2), iarea(2,2)
                     do ip = iarea(1,1), iarea(2,1)

                        prems(ip,jp,deltad,i) =
     +                     prems(ip,jp,deltad,i) +
     +                     dvsum(ip,jp,gammad,deltad) *
     +                     signg * adirii(ip+ish,jp+jsh,idir,gammad)

                        prems(ip,jp,gammad,i) =
     +                     prems(ip,jp,gammad,i) +
     +                     dvsum(ip,jp,gammad,deltad) *
     +                     signg * adirii(ip+ish,jp+jsh,idir,deltad)

                       if ( gammad.eq.deltad ) then

                          prems(ip,jp,1,i) =
     +                        prems(ip,jp,1,i) - (2d0/3d0) *
     +                        dvsum(ip,jp,gammad,deltad) *
     +                        signg * adirii(ip+ish,jp+jsh,idir,deltad)
                          prems(ip,jp,2,i) =
     +                        prems(ip,jp,2,i) - (2d0/3d0) *
     +                        dvsum(ip,jp,gammad,deltad) *
     +                        signg * adirii(ip+ish,jp+jsh,idir,deltad)

                       end if

                    end do
                  end do

               end do

            end do

         else

            do gammad = 1, 2

               do deltad = 1, 2

                  do jp = iarea(1,2), iarea(2,2)
                     do ip = iarea(1,1), iarea(2,1)

                        prems(ip,jp,deltad,i) =
     +                     prems(ip,jp,deltad,i) +
     +                     dvsum(ip,jp,gammad,deltad) *
     +                     signg * adirii(ip+ish,jp+jsh,idir,gammad)

                        prems(ip,jp,gammad,i) =
     +                     prems(ip,jp,gammad,i) +
     +                     dvsum(ip,jp,gammad,deltad) *
     +                     signg * adirii(ip+ish,jp+jsh,idir,deltad)

                    end do
                  end do

               end do

            end do

         end if

      end do

      end
