      subroutine istran3d ( iinput, matrix, rhside, solut, coefs, work,
     +                      invjac, rinput, npointsv, infmt, tsum,
     +                      dtsum, coor, mct, bndcon, ibipoint, ibindex,
     +                      ibinfor, jac, iequa, ndegfd, nwork )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 1.3    date 10-12-1997 Guus: make definition of iequa
c                                       consistent
c        version 1.2    date 17-02-1997 Marcel: parameter list of
c                                               isurhdc3 changed
c        version 1.1    date 17-10-1996 New definition of iindsc
c        version 1.0    date 29-04-1996
c
c   copyright (c) 1996-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Build transport matrix and right-hand side in R3
c
c **********************************************************************
c
c                       KEYWORDS
c
c    3d
c    matrix
c    transport_matrix
c    right_hand_side
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinput(*), npointsv, infmt(*), iequa, ndegfd,
     +        mct(-2:2,-2:2,-2:2), ibipoint(6), ibinfor(*),
     +        ibindex(2,2,6)
      double precision rhside(*), work(*), matrix(*), coor(*),
     +                 solut(npointsv,ndegfd,*), coefs(npointsv,*),
     +                 rinput(*), tsum(npointsv),  invjac(npointsv,*),
     +                 dtsum(npointsv,3), bndcon(3,*),
     +                 jac(npointsv,*)

c     bndcon    i    reserved
c     coefs     i    reserved
c     coor      i    reserved
c     dtsum     i    Flux derivative contributions to the matrix
c     ibindex   i    ibindex(1,1,side)
c                                    first point index where information
c                                    is available on this side in
c                                    direction tdir (2D) or tdir(1) (3D)
c                    ibindex(2,1,side) last point index where information
c                                    is available on this side in
c                                    direction tdir (2D) or tdir(1) (3D
c                    ibindex(1,2,side)
c                                    first point index where information
c                                    is available on this side in
c                                    direction tdir or tdir(2) (3D)
c                    ibindex(2,2,side)
c                                    last point index where information
c                                    is available on this side in
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     iequa     i    General equation sequence number
c     iinput    i    reserved
c     infmt     i    Integer information for Pressure and Transport matrix
c     invjac    i    1/sqrt(g) in pnttyp
c     jac       i    sqrt(g) in pnttyp
c     matrix    o    reserved
c     mct       i    Integer matrix information
c     ndegfd    i    Number of degrees of freedom
c     npointsv  i    Number of points in the mesh with virtual points
c     nwork     i    Second dimension of array work
c     rhside    o    reserved
c     rinput    i    reserved
c     solut     i    reserved
c     tsum      i    Flux contributions to the matrix
c     work      i    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer nconct, alphad, iprintim, pnttyp, vartyp, ni, nj, nk,
     +        nvirtual, iarea(2,3), locdx(3), iprindsc, nwork,
     +        dcell(3), iparea(2,3), ipcva, ipjcta, acttyp, ncoefs,
     +        i, ipiindsc, ipcvc, ipctc, idegfd,
     +        leniindsc, ipcvac,
     +        ipcstar, ipfstar, ipzot, ipkdif1, ipkdif2, ipkdif3,
     +        ipkdif4, ipkdif5, ipkdif6, ichoisdis, idiscret
      double precision deltat
      logical reuse
      integer rspv(1:3,1:8,0:3,0:4,1:8), npf(0:3,0:4,1:8)
      double precision cv(1:8,0:3,0:4,1:8)

c     acttyp    Type of action to be taken
c               0      : interpolation for point value
c               1..ndim: finite difference for partial derivative
c                        in the direction given by value of acttyp
c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     cv        reserved
c     dcell     reserved
c     deltat    Product of theta and dt
c     i         General loop variable
c     iarea     Area of arrays
c     ichoisdis Choice parameter with respect to the convection scheme
c               for discretization of convective term of transport
c               equation(s) including turbulence.
c                 ichoisdis = 0: 2nd order central scheme
c                 ichoisdis = 1: 1st order standard upwind scheme
c                 ichoisdis = 2: Hybrid central/upwind scheme
c                 ichoisdis = 3: Linear kappa-scheme
c                 ichoisdis = 4: Sweby Phi-limiter
c                 ichoisdis = 5: R-kappa limiter
c                 ichoisdis = 6: USR-kappa limiter
c                 ichoisdis = 7: PL-kappa limiter
c                 ichoisdis = 8: USPL-kappa limiter
c                 ichoisdis = 9: Symmetric PL-kappa limiter
c                 ichoisdis = 10: MPL2-kappa limiter
c                 ichoisdis = 11: Symmetric PL-kappa limiter
c     idegfd    Loop variable with respect to a given unknown
c     idiscret  Type of discretization
c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     ipcstar   Starting address of array CSTAR in array WORK
c     ipctc     Starting address of array CTC in array WORK
c     ipcva     Starting address of array CVA in array WORK
c     ipcvac    Starting address of array CVA with respect to centre point
c               in array WORK
c     ipcvc     Starting address of array CVC in array WORK
c     ipfstar   Starting address of array FSTAR in array WORK
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipjcta    Starting address of array JCTA in array WORK
c     ipkdif1   Starting address of array KDIF1 in array WORK
c     ipkdif2   Starting address of array KDIF2 in array WORK
c     ipkdif3   Starting address of array KDIF3 in array WORK
c     ipkdif4   Starting address of array KDIF4 in array WORK
c     ipkdif5   Starting address of array KDIF5 in array WORK
c     ipkdif6   Starting address of array KDIF6 in array WORK
c     iprindsc  Starting address of array RINDSC in array RINPUT
c     iprintim  Starting address of array RINTIM in array RINPUT
c     ipzot     Starting address of array ZOT in array WORK
c     leniindsc Length of first component of array iindsc
c     locdx     Position of the integration point with respect to
c               the centre of the integration cell
c               The cell sizes are assumed to be 2 times 2, hence the
c               distance of a midside point to a cell centre is always 1
c     ncoefs    Number of coefficients
c     nconct    Number of connections in a row of the matrix
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npf       reserved
c     nvirtual  Number of rows of virtual cells
c     pnttyp    reserved
c     reuse     If true the computed coefficients may be reused again
c     rspv      reserved
c     vartyp    Variable to be interpolated/extrapolated
c               vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c               0 for vertex (for ndim=2)
c               or
c               vartyp: 1 for V1, 2 for V2 3, for V3 and
c               4 for pressure, 0 for vertex  (for ndim=3)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISBTRA3D  Correct dtsum, matrix and right-hand side for Robbins type
c               boundary conditions for the transport equation in R3
c     ISCOVA3D  Compute covariant base vectors in all points of the grid
c               three-dimensional case
c     ISDISCT3  Distribute convective terms for transport equations over the
c               matrix in R3
c     ISDISDT3  Distribute diffusion term for tranport equation in R3
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISINTCF3  Compute interpolation of a coefficient in one integration
c               point in R^3
c     ISJCTA3D  Compute sqrt(g) x contra variant base vectors in integration
c               point three-dimensional case
c     ISPNDF    Make tables for interpolation and finite difference formulas
c     ISSETCON  Make array equal to a constant
c     ISTRAC3D  Compute contribution of convection term in R3 for the
c               transport equations
c     ISTRAD3D  Compute contribution of diffusion term in R3 for the
c               transport equations
c     ISTRAF3D  All parts of the matrix outside the region are folded to
c               internal domain for the transport matrix in R3
c     ISTRAT3D  adds the time derivative to the diagonal of the matrix and
c               the right-hand side for the 3D transport equation
c     ISTRFM3D  Transform the cartesian diffusion coefficients
c               into contravariant coefficients in R3
c     ISURHDC3  Update right-hand side vector with higher order upwind scheme
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c      17   internal error: array size too large
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Compute constants
c    Clear array matrix
c    Clear array rhside
c    Fill covariant basis vectors in cva for all integration points
c    Transform the cartesian diffusion coefficients into
c    contravariant coefficients
c    Interpolate C^*, f^* and D to integration point (Cell centre)
c    Build time-derivative part and constant part of the matrix and rhside
c    For all the integration points, except the centre do
c       Interpolate coefficients to integration points
c       Compute local co-ordinates, distances to cell center in global
c       co-ordinates, point type
c       Interpolate coefficients to integration points
c       Compute contribution of convection term in integration point
c       Distribute contribution of convection term in integration point
c       Correct right-hand side in case of higher order upwind scheme
c       (defect correction)
c       Compute contribution of diffusion term in integration point
c       Compute effect of boundary conditions
c       Distribute contribution of diffusion term in integration point
c    Fold all points in the matrix outside the region inwardly
c
c    Remark:  Array cvac is the last part of array cva
c             In fact the last 9 arrays of cva are reused as cvac
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

c-----------------TEMPORARY---------------------------------------------
      call ispndf(npf,rspv,cv,3)

c-----------------TEMPORARY---------------------------------------------

c     --- Compute constants

      leniindsc = iinput(106)
      ipiindsc = iinput(6)+leniindsc*(iequa-2)
      iprintim = iinput(29)
      iprindsc = iinput(33)+10*(iequa-2)
      idegfd   = iequa+2
      ni       = iinput(51)
      nj       = iinput(52)
      nk       = iinput(53)
      nvirtual = iinput(54)
      ncoefs   = iinput(66)
      nconct   = infmt(6)
      deltat   = rinput(iprintim -1 +4)
      npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*(nk+1+2*nvirtual)
      ipcva    = 1
      ipcvac   = ipcva+63*npointsv
      ipjcta   = ipcva+72*npointsv
      ipcstar  = ipjcta+9*npointsv
      ipzot    = ipcstar+npointsv
      ipfstar  = ipzot+npointsv
      ipkdif1  = ipfstar+npointsv
      ipkdif2  = ipkdif1+npointsv
      ipkdif3  = ipkdif2+npointsv
      ipkdif4  = ipkdif3+npointsv
      ipkdif5  = ipkdif4+npointsv
      ipkdif6  = ipkdif5+npointsv
      ipcvc    = ipkdif6+npointsv
      ipctc    = ipcvc+9*npointsv
      if ( ipctc+9*npointsv.gt.nwork ) then
        call iseropen( 'istran3d' )
        call iserrchr( 'work',1 )
        call iserrint( ipctc+9*npointsv, 1 )
        call iserrint( nwork, 2 )
        call iserrsub ( 17, 2, 0, 1 )
        call iserclos( 'istran3d' )
      end if

c     --- Clear array matrix
c         Clear array rhside

      call issetcon ( matrix, (ni+1)*(nj+1)*(nk+1)*nconct, 0d0 )
      call issetcon ( rhside, (ni+1)*(nj+1)*(nk+1), 0d0 )

c     --- Fill covariant basis vectors in cva for all integration points

      call issetcon ( work(ipcva), 72*npointsv, 0d0 )
      call iscova3d ( coor, work(ipcva), ni, nj, nk, nvirtual, npf,
     +                rspv, cv )

c     --- Transform the cartesian diffusion coefficients into
c         contravariant coefficients

      idiscret = iinput(ipiindsc+1)
      if ( idiscret.eq.1 ) then

c     --- classical discretization
c         First, fill contravariant basis vectors in cell centre

        pnttyp = 8
        call isjcta3d ( work(ipcva+9*npointsv*(pnttyp-1)),
     +                  work(ipjcta), ni, nj, nk, nvirtual )
        call istrfm3d ( coefs, invjac(1,pnttyp), work(ipjcta),
     +                  ncoefs, ni, nj, nk, nvirtual )

      end if

c     --- Build time-derivative part and constant part of the matrix and rhside
c         Interpolate C^*, f^* and D to integration point (Cell centre)

      iarea(1,1) = 1
      iarea(1,2) = 1
      iarea(1,3) = 1
      iarea(2,1) = ni
      iarea(2,2) = nj
      iarea(2,3) = nk
      pnttyp = 8
      vartyp = 4
      acttyp = 0
      call isintcf3 ( coefs(1,1), work(ipcstar), ni, nj, nk, nvirtual,
     +                npf, rspv, cv, acttyp, vartyp, pnttyp )
      call isintcf3 ( coefs(1,2), work(ipzot), ni, nj, nk, nvirtual,
     +                npf, rspv, cv, acttyp, vartyp, pnttyp )
      call isintcf3 ( coefs(1,3), work(ipfstar), ni, nj, nk, nvirtual,
     +                npf, rspv, cv, acttyp, vartyp, pnttyp )

      call istrat3d ( ni, nj, nk, nvirtual, work(ipcstar),
     +                solut(1,idegfd,1), matrix, rhside, deltat, iarea,
     +                jac(1,pnttyp), work(ipfstar), work(ipzot) )

c     --- Loop over integration points

      do 300 i = 2, 7

c     --- Compute local co-ordinates, distances to cell center in global
c         co-ordinates, point type

         dcell(1) = 0
         dcell(2) = 0
         dcell(3) = 0
         if ( i.eq.2 ) then
            locdx(1) = -1
            locdx(2) =  0
            locdx(3) =  0
            alphad   = 1
         else if ( i.eq.3 ) then
            locdx(1) = 1
            locdx(2) = 0
            locdx(3) = 0
            alphad   = 1
            dcell(1) = 1
         else if ( i.eq.4 ) then
            locdx(1) =  0
            locdx(2) = -1
            locdx(3) =  0
            alphad   = 2
         else if ( i.eq.5 ) then
            locdx(1) = 0
            locdx(2) = 1
            locdx(3) = 0
            alphad   = 2
            dcell(2) = 1
         else if ( i.eq.6 ) then
            locdx(1) =  0
            locdx(2) =  0
            locdx(3) = -1
            alphad   = 3
         else
            locdx(1) = 0
            locdx(2) = 0
            locdx(3) = 1
            alphad   = 3
            dcell(3) = 1
         end if
         pnttyp = 8-2**(alphad-1)
         reuse = locdx(1).eq.1 .or. locdx(2).eq.1 .or. locdx(3).eq.1

         if ( .not. reuse ) then

c        --- Compute cstar and kdif in integration point

            vartyp = 4
            acttyp = 0
            if ( iinput(ipiindsc).eq.1 ) then

c           --- Convection

               call isintcf3 ( coefs(1,1), work(ipcstar), ni, nj, nk,
     +                         nvirtual, npf, rspv, cv, acttyp, vartyp,
     +                         pnttyp )
            end if

            if ( iinput(ipiindsc-1+3).eq.0 ) then

c           --- Diffusion

               call isintcf3 ( coefs(1,4), work(ipkdif1), ni, nj, nk,
     +                         nvirtual, npf, rspv, cv, acttyp, vartyp,
     +                         pnttyp )
               call isintcf3 ( coefs(1,5), work(ipkdif2), ni, nj, nk,
     +                         nvirtual, npf, rspv, cv, acttyp, vartyp,
     +                         pnttyp )
               call isintcf3 ( coefs(1,6), work(ipkdif3), ni, nj, nk,
     +                         nvirtual, npf, rspv, cv, acttyp, vartyp,
     +                         pnttyp )
               call isintcf3 ( coefs(1,7), work(ipkdif4), ni, nj, nk,
     +                         nvirtual, npf, rspv, cv, acttyp, vartyp,
     +                         pnttyp )
               call isintcf3 ( coefs(1,8), work(ipkdif5), ni, nj, nk,
     +                         nvirtual, npf, rspv, cv, acttyp, vartyp,
     +                         pnttyp )
               call isintcf3 ( coefs(1,9), work(ipkdif6), ni, nj, nk,
     +                         nvirtual, npf, rspv, cv, acttyp, vartyp,
     +                         pnttyp )

            end if

         end if

c        --- List of loop bounds for all integration points
c            iparea gives the bounds for the cell indices of the integration
c            points
c            These are the indices of the centres of the integration cells
c            but shifted over a distance dcell, since the actual
c            integration points may be in another cell

         iparea(1,1) = iarea(1,1)+dcell(1)
         iparea(2,1) = iarea(2,1)+dcell(1)
         iparea(1,2) = iarea(1,2)+dcell(2)
         iparea(2,2) = iarea(2,2)+dcell(2)
         iparea(1,3) = iarea(1,3)+dcell(3)
         iparea(2,3) = iarea(2,3)+dcell(3)

         if ( iinput(ipiindsc).eq.1 ) then

c        --- convection term
c            First clear array tsum

            call issetcon ( tsum, npointsv, 0d0 )

c           --- Next fill tsum with contribution of convective terms to
c               integration points

            call istrac3d ( ni, nj, nk, nvirtual, work(ipcstar),
     +                      solut(1,alphad,2), tsum, iparea,
     +                      locdx(alphad) )

c           --- Distribute computed values over the positions
c               corresponding to the unknowns

            ichoisdis = iinput(ipiindsc-1+5)
            call isdisct3 ( alphad, ni, nj, nk, nvirtual, matrix, tsum,
     +                      solut(1,alphad,2), mct, iarea, dcell,
     +                      work(ipcstar), work(ipkdif1),
     +                      jac(1,pnttyp), ichoisdis,
     +                      iinput(ipiindsc-1+3) )

            if ( ichoisdis.gt.2 ) then

c           --- Higher order upwind scheme
c               Since defect correction is used this implies updating the
c               right-hand side

              call isurhdc3 (alphad, ni, nj, nk, nvirtual, rhside,
     +                       tsum, solut(1,alphad,2),
     +                       solut(1,idegfd,1), iarea, dcell,
     +                       ichoisdis, rinput(iprindsc-1+1),
     +                       rinput(iprindsc-1+2), rinput(iprindsc-1+3),
     +                       rinput(iprindsc-1+4) )

            end if

         end if

         if ( iinput(ipiindsc-1+3).eq.0 ) then

c        --- Diffusion term
c            First clear array dtsum

            call issetcon ( dtsum, 3*npointsv, 0d0 )
            ichoisdis = iinput(ipiindsc-1+6)
            if ( idiscret.gt.1 ) then

c           --- Wesbeek discretization, fill contravariant basis vectors

               call isjcta3d ( work(ipcva+9*npointsv*(pnttyp-1)),
     +                         work(ipjcta), ni, nj, nk, nvirtual )

            end if

            call istrad3d ( ni, nj, nk, nvirtual, work(ipkdif1),
     +                      jac(1,pnttyp), dtsum, iparea, locdx(alphad),
     +                      alphad, coefs(1,2*(alphad+1)+int(alphad/3)),
     +                      work(ipcvac), work(ipjcta), work(ipcvc),
     +                      work(ipctc), idiscret )

c           --- Compute effect of boundary conditions

            call isbtra3d ( i-1, ni, nj, nk, nvirtual, dtsum,
     +                      ibindex(1,1,i-1),ibindex(2,1,i-1),
     +                      ibindex(1,2,i-1),ibindex(2,2,i-1),
     +                      matrix, ibinfor(ibipoint(i-1)),
     +                      bndcon(1,ibipoint(i-1)), rhside,
     +                      work(ipjcta), invjac(1,pnttyp), idiscret,
     +                      mct )

c           --- Distribution of matrix

            call isdisdt3 ( alphad, ni, nj, nk, nvirtual, matrix, dtsum,
     +                      mct, iarea, dcell, ichoisdis )

         end if

300   continue

c     --- Finally the folding subroutine
c         All unknowns outside the region are folded insidewards

      call istraf3d ( ni, nj, nk, nvirtual, mct, matrix, rhside,
     +                ibinfor, ibipoint, bndcon )

      end
