      subroutine istratd2 ( ni, nj, nvirtual, axisym, cstar, solut,
     +                       matrix, rhside, deltat, iparea, jac,
     +                       fstar, zot, coor )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date 24-02-1999 Kees K. axisymmetric
c        version 1.0    date 07-03-1995 (Subroutine extracted from ismmt2)
c
c   copyright (c) 1995-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     adds the time derivative to the diagonal of the matrix and
c     the right-hand side
c     R^2 only
c **********************************************************************
c
c                       KEYWORDS
c
c     2d
c     matrix
c     right_hand_side
c     time_derivative
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, iparea(2,2), axisym
      double precision cstar(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual),
     +                 fstar(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual),
     +                 zot(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual),
     +                 matrix(ni+1,nj+1), rhside(ni+1,nj+1), deltat,
     +                 jac(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual),
     +                 coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,1:2)

c     axisym    i    In axisymmetric case axisym=1 else axisym=0
c     coor      i    reserved
c     cstar     i    The capacity in a given integration point
c     deltat    i    Product of theta and dt
c     fstar     i    The source term in a given integration point
c     iparea    i    Area of integration points where a contribution to
c                    the matrix is to be calculated
c     jac       i    sqrt(g) in pnttyp
c     matrix    o    reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rhside    o    reserved
c     solut     i    reserved
c     zot       i    The zeroth order term in a given integration point
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp
      double precision deltatin

c     deltatin  Inverse of deltat, i.e. 1/deltat
c     ip        Local counting variable i-direction
c     jp        Local counting variable j-direction
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c      It is assumed that the matrix and right-hand side have been initialized
c      before.
c      Furthermore this subroutine may only be called for centroids of
c      transport cells.
c
c        The element for the time-derivative in the matrix diagonal is
c
c          cstar     T      / deltat
c              (0,0) (0,0)
c
c
c        The element for the time-derivative in the right-hand side is
c
c          cstar     T      / deltat
c              (0,0) (0,0)
c
c        The element for the source part in the right-hand side is
c
c              *
c             f      sqrt(g)
c              (0,0)        (0,0)
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- loop over cells

      deltatin = 1d0/deltat

      if ( axisym.eq.1) then

c     --- axisymmetric case, contribution multiplied by r

         do jp = iparea(1,2), iparea(2,2)
            do ip = iparea(1,1), iparea(2,1)
               matrix(ip,jp) = jac(ip,jp)*
     +                      ( cstar(ip,jp)*deltatin+zot(ip,jp) ) *
     +                    25d-2 * ( coor(ip,jp,1) + coor(ip+1,jp,1) +
     +                              coor(ip,jp+1,1) + coor(ip+1,jp+1,1))
            end do
         end do

         do jp = iparea(1,2), iparea(2,2)
            do ip = iparea(1,1), iparea(2,1)
               rhside(ip,jp) = jac(ip,jp)*
     +                      ( cstar(ip,jp)*solut(ip,jp)*deltatin+
     +                        fstar(ip,jp) ) *
     +                    25d-2 * ( coor(ip,jp,1) + coor(ip+1,jp,1) +
     +                              coor(ip,jp+1,1) + coor(ip+1,jp+1,1))
            end do
         end do

      else

         do jp = iparea(1,2), iparea(2,2)
            do ip = iparea(1,1), iparea(2,1)
               matrix(ip,jp) = jac(ip,jp)*
     +                      ( cstar(ip,jp)*deltatin+zot(ip,jp) )
            end do
         end do

         do jp = iparea(1,2), iparea(2,2)
            do ip = iparea(1,1), iparea(2,1)
               rhside(ip,jp) = jac(ip,jp)*
     +                      ( cstar(ip,jp)*solut(ip,jp)*deltatin+
     +                        fstar(ip,jp) )
            end do
         end do

      end if

      end
