      subroutine isvsvvu(ndim,usez,vecy,facs,vecz,vecx,
     +                   vyfi,vyli,vyfj,vylj,vyfk,vylk,
     +                   vzfi,vzli,vzfj,vzlj,vzfk,vzlk,
     +                   vxfi,vxli,vxfj,vxlj,vxfk,vxlk,
     +                   area)
c
c======================================================================
c
c     programmer   Ronald van Nooyen
c
c     version     2.1  ISNAS     date    02-09-1994 RvN Changed
c                                to conform to new ISNaS standard
c     version     2.0  ISNAS     date    06-10-1993
c                 Name changed from is0xpy to isvsvvu
c                 and comments vecxd
c
c     version     1.1  ISNAS     date    14-04-93
c                                 same code for 2-d and 3-d version
c
c     version     1.0  ISNAS     date    23-03-93
c
c     computer    HP - 9000/720
c
c
c     Copyright (c) 1994  "TU-Delft"
c
c     Permission to copy or distribute this software or documentation
c     in hard copy or soft copy granted only by written license
c     obtained from "TU-Delft".
c
c     All rights reserved. No part of this publication may be reproduced,
c     stored in a retrieval system ( e.g., in memory, disk, or core)
c     or be transmitted by any means, electronic, mechanical, photocopy,
c     recording, or otherwise, without written permission from the
c     publisher.
c
c**********************************************************************
c
c                 DESCRIPTION
c
c     if (usez) then y(i) = y + facs * vecz(i) * vecx(i)
c               else y(i) = y + facs * vecx(i)
c
c     ISnas Vector Scalar Vector Vector vecx
c     ^^    ^      ^      ^      ^      ^
c     does a vector update by adding a scalar times a vector or
c     component wise vector vector product to a vector
c
c     See the pseudo code for a detailed description
c
c**********************************************************************
c
cimp      implicit none
c
c
c                 NAMED CONSTANTS
c
      integer maxdim
      parameter (maxdim = 3)
c
c     maxdim:     highest physical space dimension for which ISNaS
c                 can work. Used to determine array sizes here to
c                 accomondate 2D and 3D case with same code.
c
c                 INPUT / OUTPUT PARAMETERS
c
      logical usez
      integer  ndim,
     +     vyfi,vyli,vyfj,vylj,vyfk,vylk,
     +     vzfi,vzli,vzfj,vzlj,vzfk,vzlk,
     +     vxfi,vxli,vxfj,vxlj,vxfk,vxlk
      integer area(1:2,1:maxdim)
      double precision facs
      double precision
     +                 vecy(vyfi:vyli,vyfj:vylj,vyfk:vylk),
     +                 vecz(vzfi:vzli,vzfj:vzlj,vzfk:vzlk),
     +                 vecx(vxfi:vxli,vxfj:vxlj,vxfk:vxlk)
c
c     area      i    index bounds on the subdomain where
c                    the vector VECY is to be changed
c     facs      i    scalar multiplication factor
c     ndim      i    reserved
c     usez      i    determines whether vecz is used or not
c     vecx      i    update for vecy
c     vecy     i/o   vector to be changed
c     vecz      i    vector of component multiplication factors
c     vxfi      i    lower vecx array bound for i
c     vxfj      i    lower vecx array bound for j
c     vxfk      i    lower vecx array bound for k
c     vxli      i    upper vecx array bound for i
c     vxlj      i    upper vecx array bound for j
c     vxlk      i    upper vecx array bound for k
c     vyfi      i    lower vecy array bound for i
c     vyfj      i    lower vecY array bound for j
c     vyfk      i    lower vecy array bound for k
c     vyli      i    upper vecy array bound for i
c     vylj      i    upper vecy array bound for j
c     vylk      i    upper vecy array bound for k
c     vzfi      i    lower vecz array bound for i
c     vzfj      i    lower vecz array bound for j
c     vzfk      i    lower vecz array bound for k
c     vzli      i    upper vecz array bound for i
c     vzlj      i    upper vecz array bound for j
c     vzlk      i    upper vecz array bound for k
c
c***********************************************************************
c
c                 COMMON BLOCKS
c
c                 ---
c
c**********************************************************************
c
c                 LOCAL PARAMETERS
c
      integer i,j,k
c
c     i         General loop variable
c     j         General loop variable
c     k         General loop variable
c
c**********************************************************************
c
c                 I/O
c
c                 ---
c
c**********************************************************************
c
c                 SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c
c**********************************************************************
c
c                 ERROR MESSAGES
c
c                 ---
c
c***********************************************************************
c
c     PSEUDO CODE    :
c
c     for k:=area(1,3) to area(2,3) do
c        for j:=area(1,2) to area(2,2) do
c           for i:=area(1,1) to area(2,1) do
c              if (usez) then (in the code: in loop 1000)
c
c            vecy(i,j,k)=vecy(i,j,k)+facs*vecz(i,j,k)*vecx(i,j,k)
c
c              else if (not usez) then (in the code: in loop 2000)
c
c            vecy(i,j,k)=vecy(i,j,k)+facs*vecz(i,j,k)*vecx(i,j,k)
c
c              end if
c           end do
c        end do
c     end do
c
c***********************************************************************
c
c                 DATA
c
c                 ---
c
c=======================================================================

      call iseropen('isvsvvu')

      if (usez) then

         do 1000 k=area(1,3),area(2,3)
            do 1100 j=area(1,2),area(2,2)
               do 1110 i=area(1,1),area(2,1)

                  vecy(i,j,k) = vecy(i,j,k)+facs*vecz(i,j,k)*vecx(i,j,k)

 1110          continue
 1100       continue
 1000    continue

      else

         do 2000 k=area(1,3),area(2,3)
            do 2100 j=area(1,2),area(2,2)
               do 2110 i=area(1,1),area(2,1)

                  vecy(i,j,k) = vecy(i,j,k)+facs*vecx(i,j,k)

 2110          continue
 2100       continue
 2000    continue

      end if

      call iserclos('isvsvvu')

      end


