      subroutine isrdgrid ( maxblocki, ndim, nblocks, isiinput, iscoor,
     +                      isitopol, indorien )
c ======================================================================
c
c        programmer     Erik Brakkee
c        version 3.4    date 15-12-1999 Guus: clear array itopol
c        version 3.3    date 27-11-1999 Guus: extra check consistency of
c                                             topology
c        version 3.2    date 07-09-1999 Guus: check consistency of topology
c        version 3.1    date 25-06-1996 Guus: extra parameter indorien
c        version 3.0    date 25-06-1996 Guus: extra parameter indorien
c        version 2.0    date 07-03-1996 Guus: remove mapar
c        version 1.6    date 13-09-1995 Guus: allow for unformatted files
c        version 1.5    date 28-08-1995 Guus: extend max ni,nj,nk to 10000
c        version 1.4    date 28-02-1995 Guus: extend iinput
c        version 1.3    date 28-12-1994 Kees K. giving nptot a value for
c                                       3D, only one block 3D
c        version 1.2    date 28-07-1994 Guus: removal of open and close file
c        version 1.1    date 21-07-1994 Erik: the relative block
c                                       orientation is now computed
c        version 1.0    date 23-05-1994
c
c
c   copyright (c) 1994-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Read the grid input file describing completely the subdomain grids and
c     block topology. The grid input file will be opened and closed
c     After the call to this routine, the iinput arrays will have been
c     allocated and the following fields of this array will have been
c     filled:
c             ni, nj, nk, ndim, vlen, nblocks, max_points, max_cells,
c             npoint_global
c     Warning: This subroutine uses memory management subroutines and hence
c              should be treated as a memory management subroutine itself
c **********************************************************************
c
c                       KEYWORDS
c
c   read_mesh
c   topology
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer maxblocki
      integer iscoor(1:maxblocki), isiinput(1:maxblocki),
     +        isitopol, nblocks, ndim, indorien(*)

c     indorien  i    In this array it is stored if the block orientation is
c                    positive (indorien(iblock)=1) or
c                    negative(indorien(iblock)=-1)
c     iscoor    o    Array of sequence numbers (nblocks) of array coor
c     isiinput  o    Array of sequence numbers (nblocks) of array iinput
c     isitopol  o    Sequence number of array itopol
c     maxblocki i    Maximum number of blocks
c     nblocks   o    Number of blocks
c     ndim      o    Dimension of space (2 or 3)
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, ipabsor, ipcoor, ipiinput, ipitopol, isabsor, inum,
     +        leniinp, lensfread, lensubfac, ipnsubf, length, iorien,
     +        lentopol, maxcells, maxnijk, maxnsubf, maxpoints,
     +        nfaces, ni, nj, nk, npoint, vlen, ipsubf, ipwork
      double precision dnumbr(3)
      parameter ( leniinp = 150 )
      parameter ( lensubfac = 10 )

c     dnumbr    Work array to store a series numbers that have been read
c     i         General loop variable
c     inum      Indicates the number of numbers that have been read
c     iorien    If 1 the sign of the Jacobian is positive and the
c               directions are kept, otherwise -1 the directions are
c               interchanged
c     ipabsor   Starting address of an array 1..nblocks containing the
c               absolute orientation of the subdomains. (1 -> sqrt(g) >0
c               and -1 -> sqrt(g) < 0)
c     ipcoor    Starting address of array coor in buffr
c     ipiinput  Starting address of array iinput in ibuffr
c     ipitopol  Starting address of array itopol in ibuffr
c     ipnsubf   Starting address of the nsubfaces array
c     ipsubf    Starting address of the subfaces array
c     ipwork    Starting address of array work in buffr
c     isabsor   Pointer to an array 1..nblocks containing the
c               absolute orientation of the subdomains. (1 -> sqrt(g) >0
c               and -1 -> sqrt(g) < 0)
c     length    Length of the array in words
c     leniinp   Length of array iinput
c     lensfread Number of integers to be read for each subface
c     lensubfac Last dimension of the itopol array
c     lentopol  Length of the itopol array
c     maxcells  Maximum number of cells in a subdomain
c     maxnijk   Max(ni,nj,nk) over all blocks
c     maxnsubf  Maximum number of subfaces encountered on all subfaces
c     maxpoints Maximum number of points in a block
c     nfaces    Number of faces
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npoint    Number of points for one block
c     vlen      Number of virtual tangential velocities needed from
c               neighbouring blocks
c **********************************************************************
c
c                       I/O
c
c      none
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isblockor, isddvlen, isdpgetp, isingetp

c     ISACTIVE       Activate old permanent array
c     ISBLOCKOR      Compute the absolute orientation of a subdomain
c     ISCHECKTOP     Check the consistency of the topology
c     ISCHECKTOP1    Check the consistency of the topology
c     ISCHTOP        Change topology because of negative orientation
c     ISDCREAT       Create permanent space in array IBUFFR
c                    (Double precision array)
c     ISDCRPRI       Decrease priorities of all arrays in ibuffr
c     ISDDVLEN       Compute the last dimension of a subdomain virt array and
c                    initialize the vstart values of itopol
c     ISDELARR       Delete a permanent array
c     ISDPGETP       get the pointer corresponding to a double precision array
c     ISERCLOS       Resets old name of previous subroutine of higher level
c     ISEROPEN       Produces concatenated name of local subroutine
c     ISERRCHR       Puts values of strings in character error array
c     ISERRINT       Put integer value in error message
c     ISERRSUB       Error messages
c     ISICREAT       Create permanent space in array IBUFFR (Integer array)
c     ISINGETP       get the pointer corresponding to an integer array
c     ISIOR          Compute iorient fields of the itopol array
c     ISNUMSF        Compute a unique numbering of subfaces
c     ISPOSOR        Compute the block orientation and if necessary change
c                    directions
c     ISRDTOP        Read block topology from a file
c     ISREADRL       read a series of numbers which may be at several records
c                    skip texts and other symbols
c     ISTMSPAC       Create space for temporary array
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     522   Parameter in finvol.new incorrect
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Read ndim, nblocks
c
c     Allocate and clear isiinput(1:nblocks)
c
c     Read and store the sizes of the subdomains (ni x nj in 2-D and
c     ni x nj x nk in 3-D)
c
c     Read dimensioning parameters for the itopol array.
c
c     Allocate the subdomain coor arrays (without virtual points)
c     Allocate the itopol array
c
c     Compute the absolute orientation of subdomains (clockwise or
c     anti-clockwise grid or equivalently sqrt(g) >0 or sqrt(g) < 0)
c
c     Initialize the iorient fields of the itopol array
c
c     Read the itopol array
c
c     Read the coordinates
c
c     Compute a unique numbering of subfaces for
c     postprocessing purposes (computation of the stream function)
c
c     Initialize the vstart fields of the itopol array (isddvlen)
c     and the vlen fields of the iinput arrays
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('isrdgrid')

c     --- read ndim, nblocks

      if ( iref10.gt.0 ) then
         call isreadrl ( dnumbr, 2, inum )
         ndim    = nint(dnumbr(1))
         nblocks = nint(dnumbr(2))
      else
         read ( irefre ) ndim, nblocks
      end if
      if ( ndim.lt.2 .or. ndim.gt.3 ) then
         call iserrint ( ndim, 1 )
         call iserrint ( 2, 2 )
         call iserrint ( 3, 3 )
         call iserrchr ( 'ndim', 1 )
         call iserrsub ( 522, 3, 0, 1 )
      end if
      if ( nblocks.lt.1 .or. nblocks.gt.10000 ) then
         call iserrint ( nblocks, 1 )
         call iserrint ( 1, 2 )
         call iserrint ( 10000, 3 )
         call iserrchr ( 'nblocks', 1 )
         call iserrsub ( 522, 3, 0, 1 )
      end if

c     --- allocate isiinput(1:nblocks)

      do iblockrf = 1, nblocks

         isiinput(iblockrf) = 0
         call isicreat(isiinput(iblockrf), leniinp, 'iinput' )
         ipiinput = isingetp(isiinput(iblockrf))
         do i = 1, leniinp
            ibuffr(ipiinput+i-1) = 0
         end do

      end do

c     --- read and store the sizes of the blocks

      maxpoints = 0
      maxcells  = 0
      nk        = 0
      maxnijk   = 0

      do iblockrf = 1, nblocks

         call isactive(isiinput(iblockrf))
         ipiinput = isingetp(isiinput(iblockrf))

         if ( iref10.gt.0 ) then
            call isreadrl ( dnumbr, ndim, inum )
            ni = nint(dnumbr(1))
            nj = nint(dnumbr(2))
            if ( ndim.eq.3 ) nk = nint(dnumbr(3))
         else
            if ( ndim.eq.2 ) then
               read ( irefre ) ni, nj
            else
               read ( irefre ) ni, nj, nk
            end if
         end if
         if ( ni.lt.1 .or. ni.gt.10000 ) then
            call iserrint ( ni, 1 )
            call iserrint ( 1, 2 )
            call iserrint ( 10000, 3 )
            call iserrchr ( 'ni', 1 )
            call iserrsub ( 522, 3, 0, 1 )
         end if
         if ( nj.lt.1 .or. nj.gt.10000 ) then
            call iserrint ( nj, 1 )
            call iserrint ( 1, 2 )
            call iserrint ( 10000, 3 )
            call iserrchr ( 'nj', 1 )
            call iserrsub ( 522, 3, 0, 1 )
         end if
         if ( ndim.eq.3 ) then
            if ( nk.lt.1 .or. nk.gt.10000 ) then
               call iserrint ( nk, 1 )
               call iserrint ( 1, 2 )
               call iserrint ( 10000, 3 )
               call iserrchr ( 'nk', 1 )
               call iserrsub ( 522, 3, 0, 1 )
            end if
         end if

         ibuffr(ipiinput + 51 - 1) = ni
         ibuffr(ipiinput + 52 - 1) = nj
         ibuffr(ipiinput + 53 - 1) = nk
         ibuffr(ipiinput + 57 - 1) = ndim
         ibuffr(ipiinput + 73 - 1) = nblocks

         maxnijk = max(maxnijk, ni)
         maxnijk = max(maxnijk, nj)
         maxnijk = max(maxnijk, nk)

         maxpoints = MAX(maxpoints, (ni+1)*(nj+1)*(nk+1))
         if ( ndim.eq.2 ) then

            maxcells = MAX(maxcells, ni*nj)

         else

            maxcells = MAX(maxcells, ni*nj*nk)

         end if

      end do

c     --- fill maxpoints, maxcells of the iinput arrays

      do iblockrf = 1, nblocks

         call isactive(isiinput(iblockrf))
         ipiinput = isingetp(isiinput(iblockrf))
         ibuffr(ipiinput + 71 - 1) = maxpoints
         ibuffr(ipiinput + 72 - 1) = maxcells

      end do

c     --- read dimensioning parameters for the itopol array

      if ( iref10.gt.0 ) then
         call isreadrl ( dnumbr, 2, inum )
         maxnsubf = nint(dnumbr(1))
         lensfread    = nint(dnumbr(2))
      else
         read ( irefre ) maxnsubf
         read ( irefre ) lensfread
      end if
      if ( maxnsubf.lt.1 .or. maxnsubf.gt.maxnijk) then
         call iserrint ( maxnsubf, 1 )
         call iserrint ( 1, 2 )
         call iserrint ( maxnijk, 3 )
         call iserrchr ( 'maxnsubf', 1 )
         call iserrsub ( 522, 3, 0, 1 )
      end if
      if ( lensfread.lt.7 .or. lensfread.gt.7 ) then
         call iserrint ( lensfread, 1 )
         call iserrint ( 7, 2 )
         call iserrint ( 7, 3 )
         call iserrchr ( 'lensubfac', 1 )
         call iserrsub ( 522, 3, 0, 1 )
      end if

c     --- allocate coor and itopol

      do iblockrf = 1, nblocks

         call isactive(isiinput(iblockrf))
         ipiinput = isingetp(isiinput(iblockrf))
         ni       = ibuffr(ipiinput + 51 - 1)
         nj       = ibuffr(ipiinput + 52 - 1)
         nk       = ibuffr(ipiinput + 53 - 1)

         iscoor(iblockrf) = 0
         length = (ni+1)*(nj+1)*(nk+1)*ndim
         call isdcreat(iscoor(iblockrf), length, 'coor' )

      end do

      nfaces = 2*ndim
      lentopol = 6 + nblocks*nfaces +
     +           nblocks*nfaces*maxnsubf*lensubfac
      call isicreat(isitopol, lentopol, 'itopol' )
      ipitopol = isingetp(isitopol)

c     --- Clear array itopol

      do i = 0, lentopol-1
         ibuffr(ipitopol+i) = 0
      end do
      ibuffr(ipitopol + 1 - 1) = nblocks
      ibuffr(ipitopol + 2 - 1) = nfaces
      ibuffr(ipitopol + 3 - 1) = maxnsubf
      ibuffr(ipitopol + 4 - 1) = lensubfac
      ibuffr(ipitopol + 5 - 1) = 7
      ibuffr(ipitopol + 6 - 1) = 7 + nblocks*nfaces

c     --- read the itopol array

      ipnsubf  = ibuffr(ipitopol+5-1) + ipitopol - 1
      ipsubf   = ibuffr(ipitopol+6-1) + ipitopol - 1
      call isrdtop ( lensfread, nblocks, nfaces, maxnsubf,
     +               ibuffr(ipnsubf), ibuffr(ipsubf) )

c     --- Check the topol array

      call ischecktop ( nblocks, nfaces, maxnsubf, ibuffr(ipnsubf),
     +                  ibuffr(ipsubf) )
      do iblockrf = 1, nblocks

         call isactive(isiinput(iblockrf))
         ipiinput = isingetp(isiinput(iblockrf))

         call ischecktop1 ( iblockrf, nblocks, nfaces, maxnsubf,
     +                      ibuffr(ipnsubf), ibuffr(ipsubf),
     +                      ibuffr(ipiinput) )

      end do

c     --- read the subdomain coordinate arrays

      do iblockrf = 1, nblocks

         call isactive(isiinput(iblockrf))
         call isactive(iscoor(iblockrf))

         ipcoor = isdpgetp(iscoor(iblockrf))
         ipiinput = isingetp(isiinput(iblockrf))

         ni = ibuffr(ipiinput + 51 - 1)
         nj = ibuffr(ipiinput + 52 - 1)
         nk = ibuffr(ipiinput + 53 - 1)

         npoint = (ni+1)*(nj+1)*(nk+1)*ndim

         if ( iref10.gt.0 ) then
            call isreadrl ( buffer(ipcoor), npoint, inum )
         else
            read ( irefre ) (buffer(ipcoor+i-1), i=1,npoint)
         end if

      end do

c     --- Check block orientation, if necessary reverse

      if ( ndim.eq.2 ) then
         do iblockrf = 1, nblocks

            call isactive(isiinput(iblockrf))
            ipiinput = isingetp(isiinput(iblockrf))
            ni       = ibuffr(ipiinput + 51 - 1)
            nj       = ibuffr(ipiinput + 52 - 1)
            call istmspac((ni+1)*(nj+1)*2*intlen,ipwork)
            call isactive(iscoor(iblockrf))
            ipcoor = isdpgetp(iscoor(iblockrf))
            call isposor ( buffer(ipcoor), buffer(ipwork), ni, nj,
     +                     iorien )
            indorien(iblockrf) = iorien
            if ( indorien(iblockrf).eq.-1 ) then

c           --- orientation is reversed, interchange ni and nj

               ibuffr(ipiinput + 51 - 1) = nj
               ibuffr(ipiinput + 52 - 1) = ni

            end if

         end do
      end if

c     --- change the itopol array if necessary

      ipitopol = isingetp(isitopol)
      ipnsubf  = ibuffr(ipitopol+5-1) + ipitopol - 1
      ipsubf   = ibuffr(ipitopol+6-1) + ipitopol - 1
      call ischtop ( lensfread, nblocks, nfaces, maxnsubf,
     +               ibuffr(ipnsubf), ibuffr(ipsubf), indorien )

c     --- compute the absolute orientation of the subdomains

      isabsor = 0
      call isicreat(isabsor, nblocks, 'absor' )
      ipabsor = isingetp(isabsor)

      do iblockrf = 1, nblocks

         call isactive(isiinput(iblockrf))
         call isactive(iscoor(iblockrf))

         ipcoor = isdpgetp(iscoor(iblockrf))
         ipiinput = isingetp(isiinput(iblockrf))

         ni = ibuffr(ipiinput + 51 - 1)
         nj = ibuffr(ipiinput + 52 - 1)
         nk = ibuffr(ipiinput + 53 - 1)

         if ( nk.eq.0 ) then
            ibuffr(ipabsor + iblockrf - 1) =
     +         isblockor(ni, nj, ndim, buffer(ipcoor))
         else
            ibuffr(ipabsor + iblockrf - 1) = 1
         end if

      end do

c     --- compute the iorient fields of the itopol array

      ipabsor      = isingetp(isabsor)
      ipitopol     = isingetp(isitopol)
      ipnsubf  = ibuffr(ipitopol+5-1) + ipitopol - 1
      ipsubf   = ibuffr(ipitopol+6-1) + ipitopol - 1
      call isior ( nblocks, nfaces, maxnsubf,
     +             ibuffr(ipnsubf), ibuffr(ipsubf),
     +             ibuffr(ipabsor))
      call isdelarr(isabsor)

c     --- compute a unique numbering of subfaces

      if ( lensubfac.ge.10 ) call isnumsf(isitopol)

c     --- compute the vstart fields of the itopol array
c         and store the vlen fields

      do iblockrf = 1, nblocks

         vlen = isddvlen(iblockrf, isitopol)

         call isactive(isiinput(iblockrf))
         ipiinput = isingetp(isiinput(iblockrf))
         ibuffr(ipiinput + 58 - 1) = vlen

      end do

      call isdcrpri

      call iserclos('isrdgrid')

      end
