      subroutine isrdint ( maxblocki, maxneq, nblocks, isiinput,
     +                     isrinput, iscoor, issolut, iscoefs, isitopol,
     +                     iseddy, isgeom, iadmin, nmachines, iscotmp,
     +                     indorien)
c ======================================================================
c
c        programmer     Guus Segal
c        version 4.0    date 07-10-1998 Guus renamed to isrdint because of
c                                       HP F90 compiler
c        version 3.3    date 30-10-1997 Kees K: periodic b.c.
c        version 3.2    date 13-10-1997 Charles: uncoupled allowed
c        version 3.1    date 06-08-1996 Guus: extra check on finvol.new
c        version 3.0    date 25-06-1996 Guus: extra parameter indorien
c        version 2.0    date 07-03-1996 Guus: Remove mapar
c        version 1.6    date 31-01-1996 Guus: Set nblockrf
c        version 1.5    date 13-09-1995 Guus: Allow for unformatted files
c        version 1.4    date 16-01-1995 Erik: check neq <= maxneq
c                                       and check if saveprec as
c                                       stored in iinblk has the same
c                                       value as stored in iinsol.
c        version 1.3    date 31-10-1994 Kees K. array coefs added
c        version 1.2    date 19-08-1994 RvN 3D adaptation
c        version 1.1    date 28-07-1994 Guus: Open and close grid file
c        version 1.0    date 26-05-1994
c
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read the input files for the host program
c    Initialize some arrays
c    The following tasks are performed by this subroutine:
c    - Read grid information, create istopol
c    - Read intermediate file, create isiinput and isrinput
c    - Create iscoor and extend with virtual points
c    - Create extended solution arrays issolut
c **********************************************************************
c
c                       KEYWORDS
c
c     mapping_array
c     read
c     topology
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iadmin(*), iscoor(*), issolut(*), isiinput(*),
     +        isitopol, isrinput(*), nblocks, maxblocki,
     +        maxneq, nmachines, iscotmp(*), iscoefs(*), indorien(*),
     +        iseddy(*), isgeom(*)

c     iadmin    i    reserved
c     indorien  i    In this array it is stored if the block orientation is
c                    positive (indorien(iblock)=1) or
c                    negative(indorien(iblock)=-1)
c     iscoefs   i    Array of sequence numbers (nblocks) of array coefs
c     iscoor    i    Array of sequence numbers (nblocks) of array coor
c     iscotmp   i    Array of sequence numbers (nblocks) of array cotmp
c     iseddy    i    Array of sequence numbers (nblocks) of array eddy
c     isgeom    i    Array of sequence numbers (nblocks) of array geom
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isitopol  i    Sequence number of array itopol
c     isrinput  i    Array of sequence numbers (nblocks) of array rinput
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     maxblocki i    Maximum number of blocks
c     maxneq    i    Maximum number of equations allowed
c     nblocks   i    Number of blocks
c     nmachines i    Number of nodes that are used
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'
      include 'IScommon/cisnasch'
      include 'IScommon/csepch'
      include 'IScommon/cisblank'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, ipcoefs, ipiinblk, ipiinput, ipiinsol, ipsolut, irefsv,
     +      length, ncoefs, ndegfd, ndim, neq, ni, nj, nk, ntimlv, ios,
     +      nvirtual, saveprec, icoupled, ipiinbc, ipitopol, nfaces,
     +      maxnsubf, ipsubf, ntrnsp, ipiintur, ipiincom, nturb, ipcoor,
     +      ipnsubf, mturb, leddy, maxvpnts, mcom, ipeddy, lgeom,
     +      idiscret, ipgeom, ipiindsc

c     i         General loop variable
c     icoupled  If 0 the momentum equations are coupled, if 1 they are not
c     idiscret  Type of discretization
c     ios       Iostatus variable
c     ipcoefs   Starting address of array coefs in buffer
c     ipcoor    Starting address of array coor in buffer
c     ipeddy    Starting address of array eddy in buffer
c     ipgeom    Starting address of array geom in buffer
c     ipiinbc   Starting address of array iinbc in array iinput
c     ipiinblk  Starting address of array IINBLK in array IINPUT
c     ipiincom  Starting address of array IINCOM in array IINPUT
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipiinsol  Starting address of array IINSOL in array IINPUT
c     ipiintur  Starting address of array IINTUR in array IINPUT
c     ipitopol  Starting address of array itopol in ibuffr
c     ipnsubf   Starting address of the nsubfaces array
c     ipsolut   Starting address of array solut in buffer
c     ipsubf    Starting address of the subfaces array
c     irefsv    Help variable to save the value of a reference number
c     leddy     Length of array eddy
c     length    Length of the array in words
c     lgeom     Length of array geom
c     maxnsubf  Maximum number of subfaces encountered on all subfaces
c     maxvpnts  Maximum number of points per block including virtual ones
c     mcom      If zero the flow is incompressible else (1) compressible
c     mturb     Indicates type of turbulence modelling
c     ncoefs    Number of coefficients
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     neq       Number of equations
c     nfaces    Number of faces
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     ntimlv    Number of time levels
c     ntrnsp    Number of transport equations
c     nturb     Number of extra transport equations for turbulence
c     nvirtual  Number of rows of virtual cells
c     saveprec  Save preconditioner across domain decomposition
c               iterations (=1) or not (=0)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isdpgetp, isingetp

c     ISACTIVE  Activate old permanent array
c     ISCHSUBF  Change array subfaces for priodic boundary conditions
c     ISDCREAT  Create permanent space in array IBUFFR (Double precision array)
c     ISDELARR  Delete a permanent array
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISFILI    Fill the inner regions of the coordinate arrays
c     ISFILV    Fill virtual coordinates of the local coordinate
c               arrays of all blocks
c     ISINGETP  get the pointer corresponding to an integer array
c     ISINIFIL  Open files
c     ISMAKADM  Ad-hoc subroutine to fill array iadmin
c     ISRDGRID  Read the grid input file describing completely the subdomain
c               grids and block topology
c     ISREADIF  Read intermediate file
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c      20   Error in reading of file
c     500   End of file found
c     597   number of equations exceeds the maximum number
c     598   SAVEPREC has different values in IINBLK and IINSOL
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Read the grid input file and create arrays istopol
c    Store the co-ordinates in the temporary array iscotmp
c    Read the intermediate file and create the arrays isiinput and isrinput
c    Create array iadmin
c    Compute some parameters with respect to isiinput
c    Create the co-ordinates arrays
c    Fill array iscoor
c    Delete temporary array iscotmp
c
c    Warning: most of the subroutines called in this subroutine use memory
c             management and hence should be handled with care
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isrdint' )

c     --- Read the grid input file and create arrays istopol
c         Store the co-ordinates in the temporary array iscotmp
c         open grid input file

      call isinifil ( -10 )
      if ( nerror.ne.0 ) go to 1000
      irefsv = irefre
      irefre = abs(iref10)
      ios = 0
      if ( iref10.lt.0 ) read(irefre, iostat=ios) string(1:6)
      if ( ios.lt.0 ) then

c     --- end of file found

         call iserrint ( abs(iref10), 1 )
         call iserrchr ( name10, 1 )
         call iserrsub ( 500, 1, 0, 1 )

         go to 1000

      else if ( ios.gt.0 ) then

c     --- error in reading of file

         call iserrint ( abs(iref10), 1 )
         call iserrint ( ios, 2 )
         call iserrchr ( name10, 1 )
         call iserrsub ( 20, 2, 0, 1 )
         go to 1000

      end if
      call isrdgrid ( maxblocki, ndim, nblocks, isiinput, iscotmp,
     +                isitopol, indorien )
      if ( nerror.ne.0 ) go to 1000
      nblockrf = nblocks

c     --- close grid input file

      close(irefre)
      irefre = irefsv

c     --- Read the intermediate file and create the arrays isiinput and
c         isrinput

      if ( nerror.ne.0 ) go to 1000
      call isreadif ( isiinput, isrinput, nblocks, nmachines, ndim )
      if ( nerror.ne.0 ) go to 1000

c     --- Check if neq > maxneq

      call isactive(isiinput(1))
      ipiinput = isingetp(isiinput(1))
      ndegfd   = ibuffr(ipiinput + 55 - 1)
      ndim     = ibuffr(ipiinput + 57 - 1)
      icoupled = ibuffr(ipiinput + 76 - 1)
      neq      = ndegfd + ( 1 - ndim )*( 1 - icoupled )

      if ( neq.gt.maxneq ) then

         call iserrint ( neq,    1 )
         call iserrint ( maxneq, 2 )
         call iserrsub ( 597, 2, 0, 0 )

      end if

      ipiindsc  = ibuffr(ipiinput + 6 - 1)
      ipiintur  = ibuffr(ipiinput + 8 - 1) + ipiinput - 1
      ipiincom  = ibuffr(ipiinput + 9 - 1) + ipiinput - 1
      mturb     = ibuffr(ipiintur + 2 - 1)
      mcom      = ibuffr(ipiincom + 1 - 1)
      idiscret  = ibuffr(ipiinput-1+ipiindsc+1)

c     --- Check if saveprec as stored in both iinblk and iinsol is
c         the same.

      call isactive(isiinput(1))
      ipiinput = isingetp(isiinput(1))
      ipiinblk = ibuffr(ipiinput + 7 - 1) + ipiinput - 1
      saveprec = ibuffr(ipiinblk + 3 - 1)
      if ( nerror.ne.0 ) go to 1000
      do i = 1, neq
        ipiinsol = ibuffr(ipiinput + 5 - 1) + ipiinput - 1

        if ( saveprec.ne.ibuffr(ipiinsol + 15 + 25*(i-1) - 1) ) then
           call iserrint ( saveprec, 1 )
           call iserrint ( ibuffr(ipiinsol + 15 + 25*(i-1) - 1), 2 )
           call iserrint ( i, 3)
           call iserrsub ( 598, 3, 0, 0 )
        end if

      end do
      if ( nerror.ne.0 ) go to 1000

c     --- Create array iadmin

      call ismakadm ( nmachines, nblocks, iadmin )
      if ( nerror.ne.0 ) go to 1000

c     --- Create the co-ordinates arrays

      do iblockrf = 1, nblocks

         call isactive ( isiinput(iblockrf) )
         ipiinput = isingetp(isiinput(iblockrf))

         ni       = ibuffr(ipiinput + 50)
         nj       = ibuffr(ipiinput + 51)
         nk       = ibuffr(ipiinput + 52)
         nvirtual = ibuffr(ipiinput + 53)
         ndegfd   = ibuffr(ipiinput + 54)
         ndim     = ibuffr(ipiinput + 56)
         ntimlv   = ibuffr(ipiinput + 63)
         ncoefs   = ibuffr(ipiinput + 65)
         maxvpnts = ibuffr(ipiinput + 68)

c        --- extended coordinate arrays

         length = (ni+1+2*nvirtual)*
     +            (nj+1+2*nvirtual)*ndim
         if ( ndim.eq.3 ) length = length*(nk+1+2*nvirtual)

         iscoor(iblockrf) = 0
         call isdcreat(iscoor(iblockrf), length, 'coor' )

c        --- extended coefficient arrays

         length = (ni+1+2*nvirtual)*
     +            (nj+1+2*nvirtual)*ncoefs
         if ( ndim.eq.3 ) length = length*(nk+1+2*nvirtual)

         iscoefs(iblockrf) = 0
         call isdcreat(iscoefs(iblockrf), length, 'coefs' )
         if ( nerror.ne.0 ) go to 1000

c        --- initialize coefficient arrays

         ipcoefs = isdpgetp(iscoefs(iblockrf))

         do i = 0, length -1
            buffer(ipcoefs+i) = 0d0
         end do

c        --- subdomain solut arrays

         length = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*ndegfd*ntimlv
         if ( ndim.eq.3) length = length*(nk+1+2*nvirtual)
         issolut(iblockrf) = 0
         call isdcreat ( issolut(iblockrf), length , 'solut' )
         if ( nerror.ne.0 ) go to 1000

c        --- initialize solut arrays

         ipsolut = isdpgetp(issolut(iblockrf))

         do i = 0, length -1
            buffer(ipsolut+i) = 0d0
         end do

c        --- subdomain eddy arrays, necessary for output to
c            create these arrays here

         if ( mturb.gt.0 ) then
            leddy = maxvpnts
         else if ( mcom.gt.0 ) then
            leddy = maxvpnts * 7
         else
            leddy = 0
         end if
         if ( leddy.gt.0 ) then
           iseddy(iblockrf) = 0
           call isdcreat ( iseddy(iblockrf), leddy, 'eddy' )
         end if

c        --- initialize eddy arrays

         if ( leddy.gt.0 ) then
            ipeddy = isdpgetp(iseddy(iblockrf))
            do i = 0, leddy -1
               buffer(ipeddy+i) = 0d0
            end do
	 end if

c        --- subdomain geom arrays, necessary for output to
c            create these arrays here

         lgeom   = maxvpnts * (2**ndim) * 2
         if ( mturb.gt.0 ) then
            lgeom   = lgeom + maxvpnts
         end if

         if ( idiscret.eq.4 ) then
            lgeom   = lgeom + maxvpnts * 2**ndim
         end if

         call isdcreat( isgeom(iblockrf), lgeom  , 'geom' )

c        --- initialize eddy arrays

         ipgeom = isdpgetp(isgeom(iblockrf))
         do i = 0, lgeom -1
            buffer(ipgeom+i) = 0d0
         end do

      end do

c     --- Change array subfaces for periodic boundary conditions

      call isactive(isiinput(1))
      call isactive(iscotmp(1))
      call isactive(isitopol)
      ipiinput = isingetp(isiinput(1))
      ipitopol = isingetp(isitopol)

c     --- attention for this purpose the coordinate array without
c         virtuals has to be used : see function isblockor in
c         subroutine ischsubf.

      ipcoor   = isdpgetp(iscotmp(1))

      ipiinbc  = ibuffr(ipiinput-1+2)
      ntrnsp   = ibuffr(ipiinput-1+63)
      ipiintur = ibuffr(ipiinput-1+8)
      ni = ibuffr(ipiinput + 50)
      nj = ibuffr(ipiinput + 51)
      nk = ibuffr(ipiinput + 52)

      nturb    = ibuffr(ipiinput+ipiintur-1)
      nblocks  = ibuffr(ipitopol+1-1)
      nfaces   = ibuffr(ipitopol+2-1)
      maxnsubf = ibuffr(ipitopol+3-1)
      ipsubf   = ibuffr(ipitopol+6-1) + ipitopol -1
      ipnsubf  = ibuffr(ipitopol+5-1) + ipitopol -1
      call ischsubf ( ibuffr(ipiinput), ni, nj, nk, ndim,
     +                ntrnsp, nturb, nblocks, nfaces, maxnsubf,
     +                ibuffr(ipiinput+ipiinbc-1), ibuffr(ipnsubf),
     +                ibuffr(ipsubf), buffer(ipcoor), isitopol  )

c     --- Fill array iscoor

      if ( nerror.ne.0 ) go to 1000
      call isfili ( nblocks, isiinput, iscotmp, iscoor )
      if ( nerror.ne.0 ) go to 1000

      call isfilv ( nblocks, isiinput, iscoor, isitopol )
      if ( nerror.ne.0 ) go to 1000

c     --- Delete temporary array iscotmp

      do iblockrf = 1, nblocks
         call isdelarr ( iscotmp(iblockrf) )
      end do

1000  call iserclos ( 'isrdint' )
      end
