      subroutine isreadcm ( iincom, rincom, dnumbr )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.10   date 30-06-1999 Guus:extension iincom
c        version 1.9    date 11-08-1998 Guus:extension iincom
c        version 1.8    date 28-07-1998 Guus:extension iincom
c        version 1.7    date 02-03-1998 Guus:extension rincom
c        version 1.6    date 11-12-1997 Guus:parameter no_enthalpy
c        version 1.5    date 01-12-1997 Hester:extension other compr.
c                                              press. corr. method
c        version 1.4    date 12-11-1997 Hester:extension no scaling
c        version 1.3    date 03-07-1997 Hester:correction compr. density upw.
c        version 1.2    date 10-06-1997 Extension with multi-phase gas flow
c        version 1.1    date 19-02-1996 Extension with upwind
c        version 1.0    date 07-02-1995
c
c
c   copyright (c) 1995-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read all input concerning the compressibility modelling from the
c    intermediate file and store in arrays iincom and rincom for one block
c **********************************************************************
c
c                       KEYWORDS
c
c    compressible
c    read_cards
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iincom(*)
      double precision dnumbr(*), rincom(*)

c     dnumbr              Work array to store a series numbers that have been
c                         read
c     iincom         o    Array IINCOM is part of array IINPUT and hence
c                         coupled to one block only. It contains information
c                         with respect to the compressible flow
c                         For a description see the programmers guide
c     rincom         o    Array RINCOM is part of array IINPUT and hence
c                         coupled to one block only. It contains information
c                         with respect to the compressible flow
c                         For a description see the programmers guide
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer inum, mcom, mcomupw, mscal, mprcor, noenthal
      double precision param

c     inum           Indicates the number of numbers that have been read
c     mcom           If zero the flow is incompressible else (1)
c                    compressible
c     mcomupw        If zero the rho equation for compressible flow is
c                    solved by a central scheme,
c                    otherwise by a upwind scheme
c     mprcor         Indicates choice of compressible pressure correction
c                    method:
c                    0:   Original formulation
c                    1:   Pressure corr. equation better for contact
c                         discontinuities, eq. obtained from
c                         energy+momentum eq.
c     mscal          Indicates use of non-dimensionalisation for
c                    compr. flow:
c                    0:   Equations are non-dimensionalised
c                    1:   Equations are in dimensional formulation
c     noenthal       If 1 the enthalpy is not taken into account,
c                    if 0 it is
c     param          Arbitrary parameter read
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERREAL  Put real value in error message
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISREADRL  read a series of numbers which may be at several records
c               skip texts and other symbols
c **********************************************************************
c
c                       I/O
c
c     Input is read from the intermediate file
c     The following input is expected
c     100 reals with respect to iincom
c     100 reals with respect to rincom
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     502   Parameter in input file incorrrect
c     512   rincom(i) in input file incorrrect
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen ( 'isreadcm' )

c     --- Read iincom

      call isreadrl ( dnumbr, 100, inum )
      mcom = nint(dnumbr(1))
      if ( mcom.lt.0 .or. mcom.gt.1 ) then
         call iserrint ( mcom, 1 )
         call iserrchr ( 'mcom', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      iincom(1) = mcom
      mcomupw = nint(dnumbr(2))
      if ( mcomupw.lt.0 .or. mcomupw.gt.1 ) then
         call iserrint ( mcomupw, 1 )
         call iserrchr ( 'mcomupw', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      iincom(2) = mcomupw
      iincom(3) = nint(dnumbr(3))
      noenthal = nint(dnumbr(4))
      if ( noenthal.lt.0 .or. noenthal.gt.1 ) then
         call iserrint ( noenthal, 1 )
         call iserrchr ( 'noenthal', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      iincom(4) = noenthal
      mscal = nint(dnumbr(5))
      if ( mscal.lt.0 .or. mscal.gt.1 ) then
         call iserrint ( mscal, 1 )
         call iserrchr ( 'mscal', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      iincom(5) = mscal
      mprcor = nint(dnumbr(6))
      if ( mprcor.lt.0 .or. mprcor.gt.1 ) then
         call iserrint ( mprcor, 1 )
         call iserrchr ( 'mprcor', 1 )
         call iserrsub ( 502, 1, 0, 1 )
      end if
      iincom(6) = mprcor
      iincom(7) = nint(dnumbr(7))
      iincom(8) = nint(dnumbr(8))
      iincom(9) = nint(dnumbr(9))
      iincom(10) = nint(dnumbr(10))

c     --- Read rincom

      call isreadrl ( rincom, 9, inum )
      if ( mcom.gt.0 ) then

c     --- Compressible flow, test parameters

         param = rincom(1)
         if ( param.lt.1d0 .or. param.gt.2d0 ) then
            call iserrint ( 1, 1 )
            call iserreal ( param, 1 )
            call iserrchr ( 'gamma', 1 )
            call iserrsub ( 512, 1, 1, 1 )
         end if
         param = rincom(2)
         if ( param.lt.0d0 .or. param.gt.10d0 ) then
            call iserrint ( 1, 1 )
            call iserreal ( param, 1 )
            call iserrchr ( 'mach', 1 )
            call iserrsub ( 512, 1, 1, 1 )
         end if
         param = rincom(3)
         if ( param.lt.0d0 .or. param.gt.6d3 ) then
            call iserrint ( 1, 1 )
            call iserreal ( param, 1 )
            call iserrchr ( 'vel_sound', 1 )
            call iserrsub ( 512, 1, 1, 1 )
         end if
         param = rincom(4)
         if ( param.lt.-90d0 .or. param.gt.90d0 ) then
            call iserrint ( 1, 1 )
            call iserreal ( param, 1 )
            call iserrchr ( 'alpha', 1 )
            call iserrsub ( 512, 1, 1, 1 )
         end if
         param = rincom(5)
         if ( param.lt.0d0 .or. param.gt.1d0 ) then
            call iserrint ( 1, 1 )
            call iserreal ( param, 1 )
            call iserrchr ( 'p_v', 1 )
            call iserrsub ( 512, 1, 1, 1 )
         end if
         param = rincom(6)
         if ( param.lt.0d0 .or. param.gt.3d7 ) then
            call iserrint ( 1, 1 )
            call iserreal ( param, 1 )
            call iserrchr ( 'p_out', 1 )
            call iserrsub ( 512, 1, 1, 1 )
         end if
         param = rincom(7)
         if ( param.lt.0d0 .or. param.gt.1d7 ) then
            call iserrint ( 1, 1 )
            call iserreal ( param, 1 )
            call iserrchr ( 'p_ref', 1 )
            call iserrsub ( 512, 1, 1, 1 )
         end if
         param = rincom(8)
         if ( param.lt.1d-4 .or. param.gt.1d4 ) then
            call iserrint ( 1, 1 )
            call iserreal ( param, 1 )
            call iserrchr ( 'rho_ref', 1 )
            call iserrsub ( 512, 1, 1, 1 )
         end if
         param = rincom(9)
         if ( param.lt.1d-4 .or. param.gt.1d4 ) then
            call iserrint ( 1, 1 )
            call iserreal ( param, 1 )
            call iserrchr ( 'vel_ref', 1 )
            call iserrsub ( 512, 1, 1, 1 )
         end if

      end if

      call isreadrl ( dnumbr, 91, inum )

      call iserclos ( 'isreadcm' )
      end
