      subroutine isreadds ( iindsc, rindsc, ntrans, dnumbr, ndim,
     +                      icoupled, mcav )
c ======================================================================
c
c        programmer     Guus Segal
c        version 3.1    date 30-03-1999 Duncan: extension for newton
c                                               linearisation compressible flow
c        version 3.0    date 18-05-1998 Charles Bilinear discretization
c        version 2.9    date 26-03-1998 Guus: extension with BILINEAR_INTERPOL.
c        version 2.1    date 02-03-1998 Guus: extra keyword conservative
c        version 2.0    date 09-10-1997 Guus: Extra parameters ndim, icoupled
c        version 1.7    date 17-02-1997 Marcel: add new limiters, part II
c        version 1.6    date 09-12-1996 Marcel: add new limiters
c        version 1.5    date 10-10-1996 Guus: new check on iindsc(3,1)
c        version 1.4    date 04-06-1996 Marcel: error 605
c        version 1.3    date 18-10-1995 Marcel: Modified flux-limiting
c                                               parameters
c        version 1.2    date 22-02-1995 Enlarge iindsc
c        version 1.1    date 07-02-1995 Extension with type of linearization
c        version 1.0    date 16-08-1994
c
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read all input concerning the discretization from the intermediate
c    file and store in the arrays iindsc and rindsc for one block
c **********************************************************************
c
c                       KEYWORDS
c
c    discretization
c    read_cards
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ntrans, iindsc(20,*), icoupled, ndim, mcav
      double precision dnumbr(*), rindsc(10,*)

c     dnumbr         Work array to store a series numbers that have been read
c     icoupled  i    If 0 the momentum equations are coupled, if 1 they are not
c     iindsc    o    reserved
c     mcav      i    If 1 a cavity model is used, if 0 not
c     ndim      i    Dimension of space (2 or 3)
c     ntrans    i    Number of transport equations plus number of turbulence
c                    equations
c     rindsc    o    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, idiscret, imonoton, inum, iterm, iupwind, j, mlin

c     i         General loop variable
c     idiscret  Type of discretization
c     imonoton  Indicates type of scheme for diffusion with respect to monotony
c     inum      Indicates the number of numbers that have been read
c     iterm     Indicates which terms are taken into account
c     iupwind   Type of upwinding
c     j         General loop variable
c     mlin      Type of linearization (1=newton, 2=picard)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERREAL  Put real value in error message
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISREADRL  read a series of numbers which may be at several records
c               skip texts and other symbols
c **********************************************************************
c
c                       I/O
c
c     Input is read from the intermediate file
c     The following input is expected
c     For i := 1(1)ntrans+1
c        iindsc(j), j = 1, 100
c     For i := 1(1)ntrans+1
c        rindsc(j), j = 1, 100
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     509   iindsc(i) in input file incorrrect
c     525   rindsc(i,j) read incorrect
c     605   combination of iindsc(8,1)<>2 and iindsc(5,1)<>0 incorrrect
c **********************************************************************
c
c                       PSEUDO CODE
c
c     For i := 1 (1) ntrans+1
c        Read iindsc
c        Check iindsc
c     For i := 1 (1) ntrans+1
c        Read rindsc
c        Check rindsc
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isreadds' )

      do i = 1, ntrans+1+icoupled*(ndim-1)

c     --- Read iindsc

         call isreadrl ( dnumbr, 100, inum )
         do j = 1, 20
            iindsc(j,i) = nint(dnumbr(j))
         end do
         iterm = iindsc(1,i)
         if ( iterm.lt.0 .or. iterm.gt.2 ) then
            call iserrint ( 1, 1 )
            call iserrint ( i, 2 )
            call iserrint ( iterm, 3 )
            call iserrchr ( 'iterm', 1 )
            call iserrsub ( 509, 3, 0, 1 )
         end if
         idiscret = iindsc(2,i)

c        --- if ( idiscret.lt.1 .or. idiscret.gt.4 ) then

         if ( idiscret.lt.1 .or. idiscret.gt.5 ) then
            call iserrint ( 2, 1 )
            call iserrint ( i, 2 )
            call iserrint ( idiscret, 3 )
            call iserrchr ( 'idiscret_method', 1 )
            call iserrsub ( 509, 3, 0, 1 )
         end if
         idiscret = iindsc(3,i)
         if ( idiscret.lt.0 .or. idiscret.gt.1 ) then
            call iserrint ( 3, 1 )
            call iserrint ( i, 2 )
            call iserrint ( idiscret, 3 )
            call iserrchr ( 'idiscret_viscosity', 1 )
            call iserrsub ( 509, 3, 0, 1 )
         end if
         idiscret = iindsc(4,i)
         if ( idiscret.lt.1 .or. idiscret.gt.4 ) then
            call iserrint ( 4, 1 )
            call iserrint ( i, 2 )
            call iserrint ( idiscret, 3 )
            call iserrchr ( 'idiscret_pressure', 1 )
            call iserrsub ( 509, 3, 0, 1 )
         end if
         iupwind = iindsc(5,i)
         if ( iupwind.lt.0 .or. iupwind.gt.11 ) then
            call iserrint ( 5, 1 )
            call iserrint ( i, 2 )
            call iserrint ( iupwind, 3 )
            call iserrchr ( 'iupwind', 1 )
            call iserrsub ( 509, 3, 0, 1 )
         end if
         imonoton = iindsc(6,i)
         if ( imonoton.lt.0 .or. imonoton.gt.2 ) then
            call iserrint ( 6, 1 )
            call iserrint ( i, 2 )
            call iserrint ( imonoton, 3 )
            call iserrchr ( 'imonoton', 1 )
            call iserrsub ( 509, 3, 0, 1 )
         end if
         idiscret = iindsc(7,i)
         if ( idiscret.lt.0 .or. idiscret.gt.1 ) then
            call iserrint ( 7, 1 )
            call iserrint ( i, 2 )
            call iserrint ( idiscret, 3 )
            call iserrchr ( 'coupled/decoupled', 1 )
            call iserrsub ( 509, 3, 0, 1 )
         end if
         mlin = iindsc(8,i)
         if ( mlin.lt.1 .or. mlin.gt.2 ) then
            call iserrint ( 8, 1 )
            call iserrint ( i, 2 )
            call iserrint ( mlin, 3 )
            call iserrchr ( 'lin_convection', 1 )
            call iserrsub ( 509, 3, 0, 1 )
         end if
         idiscret = iindsc(9,i)
         if ( idiscret.lt.0 .or. idiscret.gt.1 ) then
            call iserrint ( 9, 1 )
            call iserrint ( i, 2 )
            call iserrint ( idiscret, 3 )
            call iserrchr ( 'conservative', 1 )
            call iserrsub ( 509, 3, 0, 1 )
         end if

      end do

      if ( iindsc(8,1).ne.2 .and. iindsc(5,1).ne.0
     +                              .and. mcav.eq.0) then
        call iserrchr ( 'linearization', 1 )
        call iserrchr ( '8', 2 )
        call iserrint ( iindsc(8,1) , 1 )
        call iserrchr ( 'iupwind', 3 )
        call iserrchr ( '5', 4 )
        call iserrint ( iindsc(5,1) , 2 )
        call iserrsub ( 605, 2, 0, 4 )
      end if

      do i = 1, ntrans+1+icoupled*(ndim-1)

c     --- Read rindsc

         call isreadrl ( rindsc(1,i), 4, inum )
         iupwind = iindsc(5,i)
         if ( iupwind.le.2 ) then
             do j = 1, 3
               if ( abs(rindsc(j,i)).gt.1d-15 ) then
                  call iserrint ( j, 1 )
                  call iserrint ( i, 2 )
                  call iserreal ( rindsc(j,i), 1 )
                  call iserrsub ( 525, 2, 1, 0 )
               end if
            end do
         else if ( iupwind.ne.4 .and. iupwind.ne.7 ) then
            if ( rindsc(1,i).lt.-1d0 .or. rindsc(1,i).gt.1d0 ) then
               call iserrint ( 1, 1 )
               call iserrint ( i, 2 )
               call iserreal ( rindsc(1,i), 1 )
               call iserrsub ( 525, 2, 1, 0 )
            end if
         else if ( iupwind.ne.7 ) then
            if ( rindsc(2,i).lt.1d0 .or. rindsc(2,i).gt.2d0 ) then
              call iserrint ( 2, 1 )
              call iserrint ( i, 2 )
              call iserreal ( rindsc(2,i), 1 )
              call iserrsub ( 525, 2, 1, 0 )
            end if
         else
            if ( rindsc(3,i).lt.1d0 .or. rindsc(3,i).gt.1.5d0 ) then
              call iserrint ( 3, 1 )
              call iserrint ( i, 2 )
              call iserreal ( rindsc(3,i), 1 )
              call iserrsub ( 525, 2, 1, 0 )
            end if
         end if
         if ( iupwind.ge.8 .and. iupwind.ne.11 ) then
            if ( rindsc(3,i).lt.1d0 ) then
              call iserrint ( 3, 1 )
              call iserrint ( i, 2 )
              call iserreal ( rindsc(3,i), 1 )
              call iserrsub ( 525, 2, 1, 0 )
            end if
         else if ( iupwind.eq.11 ) then
            if ( rindsc(3,i).lt.1d0 .or. rindsc(3,i).gt.2d0 ) then
              call iserrint ( 3, 1 )
              call iserrint ( i, 2 )
              call iserreal ( rindsc(3,i), 1 )
              call iserrsub ( 525, 2, 1, 0 )
            end if
         end if
         if ( iupwind.ge.8 .and. iupwind.le.10 ) then
            if ( rindsc(4,i).lt.-1d0 .or. rindsc(4,i).gt.0d0 ) then
              call iserrint ( 4, 1 )
              call iserrint ( i, 2 )
              call iserreal ( rindsc(4,i), 1 )
              call iserrsub ( 525, 2, 1, 0 )
            end if
         end if
         if ( iupwind.eq.10 ) then
            if ( rindsc(4,i).gt.-rindsc(1,i) ) then
              call iserrint ( 4, 1 )
              call iserrint ( i, 2 )
              call iserreal ( rindsc(4,i), 1 )
              call iserrsub ( 525, 2, 1, 0 )
            end if
         end if
         call isreadrl ( dnumbr, 96, inum )

      end do

      call iserclos ( 'isreadds' )
      end
