      subroutine isreadstr ( iglobstr, rglobstr, ilocstr, rlocstr,
     +                       icluster, dnumbr, nclusters, leniclust  )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 21-04-1999
c
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read all input concerning the structure of the main program from the
c    intermediate file and store in arrays iglobstr, rglobstr, ilocstr, rlocstr
c    and icluster for one block
c **********************************************************************
c
c                       KEYWORDS
c
c    structure
c    read_cards
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nclusters, leniclust, iglobstr(*), ilocstr(nclusters,*),
     +        icluster(*)
      double precision dnumbr(*),  rglobstr(*), rlocstr(nclusters,*)

c     dnumbr         Work array to store a series numbers that have been read
c     icluster  o    Array in which information about the clusters is stored
c                    icluster(i), i=1(1)nclusters contains the starting
c                    position of information about cluster i
c                    In this starting position the number of equations is
c                    stored, followed by the equation sequence numbers
c     iglobstr  o    Contains information of the global structure of the program
c                    See the programmers Guide for a description
c     ilocstr   o    Contains information of the structure of the main program
c                    for each cluster
c                    See the programmers Guide for a description
c     leniclust i    Length of array icluster
c     nclusters i    Number of clusters
c     rglobstr  o    Contains information of the global structure of the program
c                    See the programmers Guide for a description
c     rlocstr   o    Contains information of the structure of the main program
c                    for each cluster
c                    See the programmers Guide for a description
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer inum, ipcluster, i, j, ih, irest, numloop

c     i         General loop variable
c     ih        Help variable to store a constant temporarily
c     inum      Indicates the number of numbers that have been read
c     ipcluster Starting address of cluster in array icluster
c     irest     Rest after subtraction
c     j         General loop variable
c     numloop   Number of times a loop must be carried out
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISREADRL  read a series of numbers which may be at several records
c               skip texts and other symbols
c **********************************************************************
c
c                       I/O
c
c     Input is read from the intermediate file
c     The following input is expected
c     100 reals with respect to iinfrsur
c     100 reals with respect to rinfrsur
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen ( 'isreadstr' )

c     --- Read iglobstr

      call isreadrl ( dnumbr, 25, inum )
      do i = 1, 25
         iglobstr(i) = nint(dnumbr(i))
         iglobstr(i+25) = 0
      end do

c     --- Read rglobstr

      call isreadrl ( rglobstr, 10, inum )

c     --- Read icluster

      numloop = (leniclust-1)/1000+1
      irest = leniclust
      ipcluster = 0
      do j = 1, numloop
         ih = min(1000,irest)
         call isreadrl ( dnumbr, ih, inum )
         do i = 1, ih
            icluster(ipcluster+i) = nint(dnumbr(i))
         end do
         irest = irest-1000
         ipcluster = ipcluster+1000
      end do

c     --- Read ilocstr

      do j = 1, nclusters
         call isreadrl ( dnumbr, 20, inum )
         do i = 1, 20
            ilocstr(j,i) = nint(dnumbr(i))
         end do
      end do

c     --- Read rlocstr

      do j = 1, nclusters
         call isreadrl ( dnumbr, 10, inum )
         do i = 1, 10
            rlocstr(j,i) = dnumbr(i)
         end do
      end do

      call iserclos ( 'isreadstr' )
      end
