      subroutine isreadtm ( iintim, rintim, dnumbr )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.7    date 18-02-1999 KeesK: test changed for error message
c        version 1.6    date 11-08-1998 Guus: Improve error message
c        version 1.5    date 27-05-1998 Guus: extension iintim/rintim
c        version 1.4    date 27-03-1998 Kees K:  nsubsteps added
c        version 1.3    date 22-04-1996 Kees K:  error messages added for
c                                       force_deletion, restart , outputw,
c                                       write_restart, length iintim=15
c        version 1.2    date 23-11-1995 Guus: remove error message concerning
c                                             toutend
c        version 1.1    date 12-01-1995 M. Kooper expansion of RINTIM
c        version 1.0    date 19-05-1994
c
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read all input concerning the time integration from the intermediate
c    file and store in the arrays iintim and rintim for one block
c **********************************************************************
c
c                       KEYWORDS
c
c    read_cards
c    time_integration
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iintim(20)
      double precision dnumbr(*), rintim(60)

c     dnumbr    i    Work array to store a series numbers that have been read
c     iintim    i    reserved
c     rintim    i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer inum, istation, j, methodtim, nfrac, tvar, ndt,
     +        restart, forcedel, outputw, nsubsteps
      double precision absaccur, relaccur, t0, toutend, tstepend(10),
     +      toutinit, toutstep, tstep(10),
     +      testfrac1, testfrac2, testfrac3

c     absaccur  Absolute accuracy with respect to residual
c     forcedel  Overwrite allowed forcedel=1 else forcedel=0
c     inum      Indicates the number of numbers that have been read
c     istation  Choice whether a stationary solution is needed
c     j         General loop variable
c     methodtim Type of time integration
c     ndt       Number of variable time steps
c     nfrac     Number of fractional time steps
c     nsubsteps Number of sub-timesteps
c     outputw   Outputw = 1 : write restart file for refining purposes,
c               cartesian unkowns in the vertices are written.
c               Outputw = 0 : the contravariant solution vector is
c               written.
c     relaccur  Relative accuracy
c     restart   Restart = -1 : no restart file will be written
c               Restart = 0 : overwrite   restart = 1 : append
c     t0        Initial time
c     testfrac1 Counter to test consistency in the choice of the thetas
c               when METHOD.eq.4
c     testfrac2 Counter to test consistency in the choice of the thetas
c               when METHOD.eq.4
c     testfrac3 Counter to test consistency in the choice of the thetas
c               when METHOD.eq.4
c     toutend   End time for output of results
c     toutinit  Initial time for output of results
c     toutstep  Frequency for output of results
c     tstep     Array of ndt time steps
c     tstepend  Array of ndt time periods for variable tstep
c     tvar      Choice of variable time steps
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERREAL  Put real value in error message
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISREADRL  read a series of numbers which may be at several records
c               skip texts and other symbols
c     ISWARSUB  Warnings
c **********************************************************************
c
c                       I/O
c
c     Input is read from the intermediate file
c     The following input is expected
c     100 integers with respect to iintim
c     100 reals with respect to rintim
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     506   iintim(i) in input file incorrrect
c     507   rintim(i) in input file incorrrect
c     612   stationary solution not combined with theta=1
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Read iintim
c     Check iintim
c     Read rintim
c     Check rintim
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isreadtm' )

c     --- Read iintim

      call isreadrl ( dnumbr, 100, inum )

      do j = 1, 20
         iintim(j) = nint(dnumbr(j))
      end do

      methodtim = iintim(1)
      if ( methodtim.lt.1 .or. methodtim.gt.4 ) then
         call iserrint ( 1, 1 )
         call iserrint ( methodtim, 2 )
         call iserrchr ( 'time integration method', 1 )
         call iserrsub ( 506, 2, 0, 1 )
      end if
      forcedel = iintim(13)
      if ( forcedel.lt.0 .or. forcedel.gt.1 ) then
         call iserrint ( 2, 1 )
         call iserrint ( forcedel, 2 )
         call iserrchr ( 'force_deletion', 1 )
         call iserrsub ( 506, 2, 0, 1 )
      end if
      restart = iintim(11)
      if ( restart.lt.-1 .or. restart.gt.1 ) then
         call iserrint ( 3, 1 )
         call iserrint ( restart, 2 )
         call iserrchr ( 'restart', 1 )
         call iserrsub ( 506, 2, 0, 1 )
      end if
      outputw = iintim(12)
      if ( outputw.lt.0 .or. outputw.gt.1 ) then
         call iserrint ( 3, 1 )
         call iserrint ( outputw, 2 )
         call iserrchr ( 'outputw', 1 )
         call iserrsub ( 506, 2, 0, 1 )
      end if
      nfrac = iintim(6)
      if ( methodtim.eq.4 .and. nfrac.lt.1   .or.
     +     methodtim.eq.4 .and. nfrac.gt.5 ) then
         call iserrint ( 6, 1 )
         call iserrint ( nfrac, 2 )
         call iserrchr ( 'number of fractional time steps', 1 )
         call iserrsub ( 506, 2, 0, 1 )
      end if
      tvar = iintim(7)
      if ( tvar.lt.0 .or. tvar.gt.3 ) then
         call iserrint ( 7, 1 )
         call iserrint ( tvar, 2 )
         call iserrchr ( 'choice for variable time steps method', 1 )
         call iserrsub ( 506, 2, 0, 1 )
      end if
      ndt = iintim(8)
      if ( tvar.eq.1 .and. ndt.lt.1    .or.
     +     tvar.eq.1 .and. ndt.gt.10 ) then
         call iserrint ( 8, 1 )
         call iserrint ( ndt, 2 )
         call iserrchr ( 'number of different time steps', 1 )
         call iserrsub ( 506, 2, 0, 1 )
      end if
      istation = iintim(10)
      if ( istation.lt.0 .or. istation.gt.1 ) then
         call iserrint ( 10, 1 )
         call iserrint ( istation, 2 )
         call iserrchr ( 'istationary', 1 )
         call iserrsub ( 506, 2, 0, 1 )
      end if
      nsubsteps = iintim(16)
      if ( nsubsteps.lt.0  ) then
         call iserrint ( 16, 1 )
         call iserrint ( nsubsteps, 2 )
         call iserrchr ( 'nsubsteps', 1 )
         call iserrsub ( 506, 2, 0, 1 )
      end if

c     --- Read rintim

      call isreadrl ( rintim, 60, inum )
      call isreadrl ( dnumbr, 40, inum )
      t0 = rintim(2)
      if ( t0.lt.-1d3 .or. t0.gt.1d3 ) then
         call iserrint ( 2, 1 )
         call iserreal ( t0, 1 )
         call iserrchr ( 't0', 1 )
         call iserrsub ( 507, 1, 1, 1 )
      end if
      absaccur = rintim(21)
      if ( absaccur.lt.1d-15 .or. absaccur.gt.1d3 ) then
         call iserrint ( 21, 1 )
         call iserreal ( absaccur, 1 )
         call iserrchr ( 'absaccur momentum', 1 )
         call iserrsub ( 507, 1, 1, 1 )
      end if
      relaccur = rintim(22)
      if ( relaccur.lt.1d-15 .or. relaccur.gt.1d-1 ) then
         call iserrint ( 22, 1 )
         call iserreal ( relaccur, 1 )
         call iserrchr ( 'relaccur momentum', 1 )
         call iserrsub ( 507, 1, 1, 1 )
      end if
      absaccur = rintim(23)
      if ( absaccur.lt.1d-15 .or. absaccur.gt.1d3 ) then
         call iserrint ( 23, 1 )
         call iserreal ( absaccur, 1 )
         call iserrchr ( 'absaccur pressure', 1 )
         call iserrsub ( 507, 1, 1, 1 )
      end if
      relaccur = rintim(24)
      if ( relaccur.lt.1d-15 .or. relaccur.gt.1d-1 ) then
         call iserrint ( 24, 1 )
         call iserreal ( relaccur, 1 )
         call iserrchr ( 'relaccur pressure', 1 )
         call iserrsub ( 507, 1, 1, 1 )
      end if
      absaccur = rintim(25)
      if ( absaccur.lt.1d-15 .or. absaccur.gt.1d3 ) then
         call iserrint ( 25, 1 )
         call iserreal ( absaccur, 1 )
         call iserrchr ( 'absaccur transport', 1 )
         call iserrsub ( 507, 1, 1, 1 )
      end if
      relaccur = rintim(26)
      if ( relaccur.lt.1d-15 .or. relaccur.gt.1d-1 ) then
         call iserrint ( 26, 1 )
         call iserreal ( relaccur, 1 )
         call iserrchr ( 'relaccur transport', 1 )
         call iserrsub ( 507, 1, 1, 1 )
      end if
      absaccur = rintim(27)
      if ( absaccur.lt.1d-15 .or. absaccur.gt.1d3 ) then
         call iserrint ( 27, 1 )
         call iserreal ( absaccur, 1 )
         call iserrchr ( 'absaccur turbulence', 1 )
         call iserrsub ( 507, 1, 1, 1 )
      end if
      relaccur = rintim(28)
      if ( relaccur.lt.1d-15 .or. relaccur.gt.1d-1 ) then
         call iserrint ( 28, 1 )
         call iserreal ( relaccur, 1 )
         call iserrchr ( 'relaccur turbulence', 1 )
         call iserrsub ( 507, 1, 1, 1 )
      end if
      absaccur = rintim(29)
      if ( absaccur.lt.0d0 .or. absaccur.gt.1d3 ) then
         call iserrint ( 29, 1 )
         call iserreal ( absaccur, 1 )
         call iserrchr ( 'absaccur stationary solution', 1 )
         call iserrsub ( 507, 1, 1, 1 )
      end if
      relaccur = rintim(30)
      if ( relaccur.lt.0d0 .or. relaccur.gt.1d-1 ) then
         call iserrint ( 30, 1 )
         call iserreal ( relaccur, 1 )
         call iserrchr ( 'relaccur stationary solution', 1 )
         call iserrsub ( 507, 1, 1, 1 )
      end if
      if ( methodtim.eq.4 ) then
         testfrac1 = 0d0
         testfrac2 = 0d0
         testfrac3 = 0d0

c        --- Test consistency in choice of the thetas when METHOD.eq.4
c            Sum theta_i = 1
c            Sum ( theta_2i )**2 = Sum ( theta_(2i-1) )**2

         do j=1, nfrac
            testfrac1    = testfrac1 + rintim(30+2*j-1)
     +                               + rintim(30+2*j  )
            testfrac2    = testfrac2 + rintim(30+2*j  )**2
            testfrac3    = testfrac3 + rintim(30+2*j-1)**2
         end do
         if ( testfrac1.lt.(1d0-1d-6)             .or.
     +        testfrac1.gt.(1d0+1d-6)             .or.
     +        testfrac2.lt.testfrac3*(1d0-1d-6)   .or.
     +        testfrac2.gt.testfrac3*(1d0+1d-6) ) then
            call iserrint ( 30+2*nfrac, 1 )
            call iserreal ( rintim(30+2*nfrac), 1 )
            call iserrchr ( 'choice of thetas', 1 )
            call iserrsub ( 507, 1, 1, 1 )
         end if
      end if
      do j=1, ndt
         tstep(j)    = rintim(40+j)
         tstepend(j) = rintim(50+j)
         if ( j.eq.   1 .and. tstepend(j).lt.t0 .or.
     +         j.eq. ndt .and. tstepend(j).gt.1d6     ) then
            call iserrint ( 50+j, 1 )
            call iserreal ( tstepend(j), 1 )
            call iserrchr ( 'tstepend', 1 )
            call iserrsub ( 507, 1, 1, 1 )
         end if
	 
c        --- test with j-1 only if j greater than 1

	 if ( j .gt. 1 ) then
            if ( j.lt. ndt         .and. 
     +            tstepend(j).le.tstepend(j-1)      ) then  
               call iserrint ( 50+j, 1 )
               call iserreal ( tstepend(j), 1 )
               call iserrchr ( 'tstepend', 1 )
               call iserrsub ( 507, 1, 1, 1 )
            end if
         end if
	 
	 
	 
         if ( tstep(j).le.0d0                                 .or.
     +        j.eq.1 .and.  tstep(j) .gt.
     +                     (tstepend(j)-t0)*(1d0+1d-6))       then 
            call iserrint ( 40+j, 1 )
            call iserreal ( tstep(j), 1 )
            call iserrchr ( 'tstep', 1 )
            call iserrsub ( 507, 1, 1, 1 )
         end if
	 
c        --- test with j-1 only if j greater than 1

	 if ( j .gt. 1 ) then	 
             if ( tstep(j) .gt.
     +           (tstepend(j)-tstepend(j-1))*(1d0+1d-6) )     then
               call iserrint ( 40+j, 1 )
               call iserreal ( tstep(j), 1 )
               call iserrchr ( 'tstep', 1 )
               call iserrsub ( 507, 1, 1, 1 )
            end if
         end if
      end do
      toutinit = rintim(6)
      if ( toutinit.lt.t0 .or. toutinit.gt.tstepend(ndt) ) then
         call iserrint ( 6, 1 )
         call iserreal ( toutinit, 1 )
         call iserrchr ( 'toutinit', 1 )
         call iserrsub ( 507, 1, 1, 1 )
      end if
      toutend = rintim(7)
      if ( toutend.lt.toutinit ) then
         call iserrint ( 7, 1 )
         call iserreal ( toutend, 1 )
         call iserrchr ( 'toutend', 1 )
         call iswarsub ( 507, 1, 1, 1 )
      end if
      toutstep = rintim(8)
      if ( toutstep.lt.tstep(1)                .or.
     +     toutstep.gt.tstepend(ndt)-t0+1d-6 ) then
         call iserrint ( 8, 1 )
         call iserreal ( toutstep, 1 )
         call iserrchr ( 'toutstep', 1 )
         call iserrsub ( 507, 1, 1, 1 )
      end if

c     --- Test if istation is used correctly (only for theta = 1)

      if ( istation.eq.1 .and. rintim(32).ne.1d0 ) then
         call iserrsub ( 612, 0, 0, 0 )
      end if

      call iserclos ( 'isreadtm' )
      end
