      subroutine istrco(iblock, ni, nj, nk, nvirtual, ndim,
     +                  nblocks, nfaces, maxnsubf,
     +                  iscoor, isiinput, coor,
     +                  nsubfaces, subfaces, vco)
c ======================================================================
c
c        programmer   Erik Brakkee
c        version 1.2    date 04-04-1997 Kees K: 3D
c        version 1.1    date 11-03-1997 Kees K: calls to isco2get and
c                                       isco2put i.s.o iscoget and iscoput
c        version 1.0    date 23-05-1994
c
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Transport all inner coordinates of a block that are virtual to its
c     neighbouring blocks to its neighbouring blocks.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, maxnsubf, nblocks, ndim, nfaces, ni,
     +      nj, nk, nvirtual
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1:ndim)
      integer iscoor(1:nblocks), isiinput(1:nblocks)
      integer nsubfaces(1:nblocks, 1:nfaces)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)
      double precision vco(*)

c     coor      i    reserved
c     iblock    i    Block sequence number
c     iscoor    i    Array of sequence numbers (nblocks) of array coor
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     ndim      i    Dimension of space (2 or 3)
c     nfaces    i    Number of faces
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     nvirtual  i    Number of rows of virtual cells
c     subfaces  i    reserved
c     vco      i/o   Work array in which the co-ordinates to be copied are
c                    stored
c                    vco has dimension: vco(1:nvirtual, 1:ndim, 1:nijk+1)
c                    with nijk max(ni,nj,nk)
c                    Array vco(i,m,n) contains the differences between points
c                    at a distance from the boundary and the corresponding
c                    points on the boundary.
c                    i:  the distance normal to the subface
c                    m:  refers tot the x (m=1), y (m=2), or z (m=3)
c                        co-ordinate
c                    n:  the distance along the subface
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ibto, iface, ifto, ipcoornb, ipiinput, isfto,
     +        isubface, nito, njto, nkto
      double precision corner1(1:3)

c     corner1   First corner of neighbouring block
c     ibto      Target block
c     iface     Face number
c     ifto      Target face
c     ipcoornb  Starting address of the coor array of the neighbour block
c     ipiinput  Starting address of array iinput in ibuffr
c     isfto     Target subface
c     isubface  Subface number
c     nito      I-dimension of neighbour
c     njto      J-dimension of neighbour
c     nkto      K-dimension of neighbour
c **********************************************************************
c
c                       I/O
c
c      none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isdpgetp, isingetp

c     ISCO2GET  Get virtual coordinates from a certain block in (2D)
c     ISCO2PUT  Put virtual coordinates from a certain block in (2D)
c     ISCO3GET  Get virtual coordinates from a certain block (3D)
c     ISCO3PUT  Put virtual coordinates from a certain block (3D)
c     ISDPGETP  Get the pointer corresponding to a double precision array
c     ISINGETP  Get the pointer corresponding to an integer array
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     None
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      do iface = 1, 2*ndim
         do isubface = 1, nsubfaces(iblock, iface)
            ibto = subfaces(iblock, iface, isubface, 5)
            if ( ibto.ne.0 ) then

               if ( ndim.eq.2 ) then
                  call isco2get(iblock, iface, isubface,
     +                          ni, nj, nvirtual, nblocks, nfaces,
     +                          maxnsubf, subfaces, coor, vco)
               else
                  call isco3get(iblock, iface, isubface,
     +                          ni, nj, nk, nvirtual, nblocks, nfaces,
     +                          maxnsubf, subfaces, coor, vco,
     +                          max(ni,nj,nk) , corner1)
               end if

               ifto = subfaces(iblock, iface, isubface, 6)
               isfto = subfaces(iblock, iface, isubface, 7)
               ipiinput = isingetp(isiinput(ibto))
               nito = ibuffr(ipiinput+51-1)
               njto = ibuffr(ipiinput+52-1)
               nkto = ibuffr(ipiinput+53-1)
               ipcoornb = isdpgetp(iscoor(ibto))

               if ( ndim.eq.2 ) then
                  call isco2put(ibto, ifto, isfto,
     +                          nito, njto, nvirtual, nblocks, nfaces,
     +                          maxnsubf, nsubfaces, subfaces,
     +                          buffer(ipcoornb), vco)
               else
                  call isco3put(ibto, ifto, isfto,
     +                          nito, njto, nkto, nvirtual, nblocks,
     +                          nfaces, maxnsubf,  nsubfaces, subfaces,
     +                          buffer(ipcoornb), vco,
     +                          max(ni,nj,nk), corner1 )
               end if

            end if
         end do
      end do

      end
