      subroutine isblkgcr(nmbsol  , itmax   , irestart, itrunc  ,
     +                    eps2    , ioutp   , rnorm   ,
     +                    rhsd    , usol    , res     ,
     +                    searchsp, svalsp  , searchvc, svalvec ,
     +                    betas   , phis    ,
     +                    alphalocal, alphaglobal, alphasum,
     +                    resid   , iequa   , isiinput,
     +                    isrinput, iscoor  , issolut , iscoefs ,
     +                    iseddy  , isinfbnd, isbndcon, ismatrix,
     +                    isgmat  , isrhside, isgeom  , isitopol,
     +                    isdensup, iadmin  , nreuse  , ireusev )
c ======================================================================
c
c     programmer   Kees Vuik
c     version      6.2  20-09-1999 Jason: output only on HOST
c     version      6.1  20-07-1999 Jason: classical Gram-Schmidt
c     version      6.0  14-07-1999 Kees V.: general revision
c     version      5.9  04-03-1999 Kees V.: removed array usolinit
c     version      5.8  16-02-1999 Kees K.: minor bug line 300
c     version      5.7  20-01-1999 Kees V.: remove nkrylin and krylovin
c     version      5.6  05-01-1999 Kees V.: error message 1611
c     version      5.5  23-12-1998 Kees V.: new call to ishgmr02
c     version      5.4  23-11-1998 Jason: removed array iaction
c     version      5.3  15-04-1998 Duncan:  extension for saving
c                                           coefficients dens upwind
c                                           new call to ishgmr02
c                                           new call to ishgmr04
c     version      5.2  02-04-1997 Kees K. literal constants in
c                                  double precision for f90
c     version      5.1  19-01-1997 Kees V.: correct orthogonalization
c     version      5.0  19-01-1997 Erik: extended for reusing search
c                                  directions
c     version      4.1  18-07-1995 Erik: extended error message with
c                                  equation number
c     version      4.0  27-02-1995 Erik: restarted GCR now used optimized
c                                  implementation. (See report 93-10
c                                  Kees Vuik)
c     version      3.1  06-02-1995 Michiel K:array raction removed
c     version      3.0  05-01-1995 Erik: adapted for inaccurate solution
c                                  of subdomains + general cleanup
c     version      2.1  03-111994  Kees K coefs and geom added
c     version      2.0  19-5-1994  Erik: adapted for multi-block acceleration
c     version      1.2  xx-1-1990  Cees Vuik: initial revision
c
c   copyright (c) 1990-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     This is the GMRESR algorithm which used GMRES in the innerloop
c     for optimal polynomial preconditioning.
c     IMPORTANT!!!!!
c       All double precision arrays in this subroutine must not be
c       governed by the memory management subroutines. The required
c       space should be allocated by routine isendspc
c     reference
c        C. Vuik
c        Further experiences with GMRESR
c        Supercomputer 1993(55), pp. 13-27
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer          nmbsol, itmax, irestart, ireusev, itrunc, ioutp,
     +                 nreuse, numorth
      double precision eps2,rnorm
      double precision res(nmbsol),      rhsd(nmbsol),
     +                 searchvc(nmbsol), svalvec(nmbsol),
     +                 searchsp(nmbsol,itrunc),
     +                 svalsp(nmbsol,itrunc),
     +                 usol(nmbsol)
      double precision resid(itmax)
      double precision betas(itrunc,itrunc), phis(itrunc)
      double precision alphalocal(itrunc), alphaglobal(itrunc),
     +               alphasum(itrunc)
      integer iequa      , iadmin(*)  , isbndcon(*), iscoor(*)  ,
     +        iseddy(*)  , isgmat(*)  , isiinput(*), isinfbnd(*),
     +        isitopol   , ismatrix(*), isrhside(*), isrinput(*),
     +        issolut(*) , iscoefs(*) , isgeom(*), isdensup(*)

c     alphaglobal    i    workspace for local innerproducts
c     alphalocal     i    workspace for global innerproducts
c     alphasum       i    workspace for global innerproducts
c     betas          i    Representation of the modified search vectors
c                         in terms of original search directions in optimized
c                         restarted GCR.
c                         u^(m)_k = sum_{i = 1,k} betas(k,i) u_i
c     eps2           i    Determine the accuracy of the final approximation.
c                         The termination criterion is based upon
c                         ||b-Ax || <  eps1 + eps2 * (b-Ax ) .
c                               k                         0
c     iadmin         i    contains information about
c                         which blocks on which computer are stored.
c     iequa          i    General equation sequence number
c     ioutp          i    Defines amount of output
c     irestart       i    Indicates after how many steps the method restarts
c     ireusev        i    Reuse search values v= A*s (=1) or not (=0)
c     isbndcon       i    Array of sequence numbers (nblocks) of array bndcon
c     iscoefs        i    Array of sequence numbers (nblocks) of array coefs
c     iscoor         i    Array of sequence numbers (nblocks) of array coor
c     isdensup       i    Array of sequence numbers (nblocks) of array densup
c     iseddy         i    Array of sequence numbers (nblocks) of array eddy
c     isgeom         i    Array of sequence numbers (nblocks) of array geom
c     isgmat         i    Array of sequence numbers (nblocks) of array gmat
c     isiinput       i    Array of sequence numbers (nblocks) of array iinput
c     isinfbnd       i    Array of sequence numbers (nblocks) of array infbnd
c     isitopol       i    Sequence number of array itopol
c     ismatrix       i    Array of sequence numbers (nblocks) of array matrix
c     isrhside       i    Array of sequence numbers (nblocks) of array rhside
c     isrinput       i    Array of sequence numbers (nblocks) of array rinput
c     issolut        i    Array of sequence numbers (nblocks) of array solut
c     itmax          i    The maximum number of iterations to be performed in
c                         the linear solver.
c     itrunc         i    Indicates after how many steps the method truncates
c     nmbsol         i    Length of the multi-block solution vector
c     nreuse         i    Number of search directions to reuse.
c     phis           i    Representation of the solution vector in terms
c                         of the original search directions
c                         x = x0 + phi(1)*searchvc(1) + ... + phi(k)*searchvc(k)
c     res            i    Array containing the residual vector.
c     resid          i    Array containing the norm of the residuals
c     rhsd           i    Vector containing the right-hand side vector of the
c                         system of equations.
c     rnorm          i    Norm of the residual vector.
c     searchsp       o    Two-dimensional array containing the search space
c     searchvc       o    Vector containing a search direction
c     svalsp         o    Two-dimensional array containing the results of
c                         the search directions in the search space
c     svalvec        o    Vector containing the result of a search direction
c     usol           i    Solution vector of length n. On input the array should
c                         contain a starting vector. At output the array
c                         contains the last iterate, which is an approximation
c                         to the solution of the system.
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, ioverwrt, imin, ipiinblk, ipiinput, isol,
     +        iter, itrestart, j, nblocks, ipiintim, itimeact,
     +        ndim, icoupled, iseqpres, icgs2
      double precision betainpr, epslin, r0norm, rdummy, dlocal, dnorm
      logical rstgcr

c     betainpr       Value of an inner product
c     dlocal         local part of a global sum
c     dnorm          norm of some distributed vector
c     epslin         Required accuracy (including relative and absolute
c                    error) in the linear solver.
c     i              General loop variable
c     icgs2          If 1, use classical Gram-Schmidt with reorth, else use
c                    modified
c     icoupled       If 0 the momentum equations are coupled,
c                    if 1 they are not
c     imin           Minimum position in table
c     ioverwrt       Number of the vector to be overwritten
c     ipiinblk       Starting address of array IINBLK in array IINPUT
c     ipiinput       Starting address of array iinput in ibuffr
c     ipiintim       Starting address of array IINTIM in array IINPUT
c     iseqpres       Sequence number of pressure equation
c     isol           Accuracy of subdomain solution in relation with
c                    domain decomposition.
c                      0 = accurate enough
c                      1 = inaccurate with respect to multi-block accuracy
c                      2 = inaccurate using ilu approximation
c     iter           The current iteration number.
c     itimeact       Actual time step
c     itrestart      Number of iterations since the last restart
c     j              General loop variable
c     nblocks        Number of blocks
c     ndim           Dimension of space (2 or 3)
c     numorth        Number of vectors to be orthogonalized
c     r0norm         This is the norm of the initial residual, and it
c                    is used in the calculation of the residual with
c                    vector updates.
c     rdummy         Real dummy
c     rstgcr         Logical indicating whether or not an optimized
c                    implementation of restart is used or not
c **********************************************************************
c
c                       I/O
c
c      None
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp
      double precision ddot, dnrm2

c     DAXPY          Computes a vector + a constant * other vector. This
c                    is a BLAS routine.
c     DCOPY          Copy double precision array into other one (BLAS subroutine
c     DDOT           Computes an inner product. This is a BLAS routine.
c     DNRM2          Calculates the 2-norm of a given vector. This is
c                    a BLAS routine.
c     ISACTIVE       Activate old permanent array
c     ISBLKMAT       Matrix vector product for multi-block solution
c     ISBLKPRE       Preconditioner vector product for multi-block solution
c     ISERCLOS       Resets old name of previous subroutine of higher level
c     ISEROPEN       Produces concatenated name of local subroutine
c     ISERREAL       Put real value in error message
c     ISERRINT       Put integer value in error message
c     ISERRSUB       Error messages
c     ISGLOMA        Global matrix-vector product over all subdomains
c     ISGLSUM        performs a global sum of a quantitiy or vector across
c                    all processes
c     ISINGETP       get the pointer corresponding to an integer array
c     ISWARSUB       Print warnings
c **********************************************************************
c
c                       ERROR MESSAGES
c
c    1605   no convergence in the GMRESR accelerated multi-block algorithm
c    1607   No convergence in the GCR accelerated domain decomposition
c    1611   The exact residual violates the termination criterion.
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      call iseropen('isblkgcr')

      rstgcr = ( irestart.eq.itrunc )

c     --- what type of algorithm is used ?

      call isactive(isiinput(1))
      ipiinput = isingetp(isiinput(1))
      ipiinblk = ibuffr(ipiinput + 7 - 1) + ipiinput - 1
      isol     = ibuffr(ipiinblk + 2 - 1)
      ndim     = ibuffr(ipiinput + 57 - 1)
      nblocks  = ibuffr(ipiinput + 73 - 1)
      icoupled = ibuffr(ipiinput + 76 - 1)
      ipiintim = ibuffr( ipiinput -1 + 4 )
      itimeact = ibuffr ( ipiinput - 1 + ipiintim -1 + 4 )
      icgs2    = ibuffr( ipiinblk + 10*iequa + 9 - 1 )

c     --- make an error message if isol < 0 or isol > 2

      if ( icoupled.eq.0 ) then
         iseqpres = 2
      else
         iseqpres = ndim + 1
      end if

c     --- print welcome message

      if ( nmbsol.ne.0 .and. ioutp.ge.1 .and. inodenr.eq.ISHOST ) then
         write(*,*) ' '
         if ( isol.eq.0 ) then
            write(*,*)
     +      'THE GMRESR ACCELERATED MULTI-BLOCK ALGORITHM FOR ',
     +      'EQUATION ', iequa
         else
            write(*,*)
     +      'THE GCR ACCELERATED MULTI-BLOCK ALGORITHM FOR EQUATION ',
     +      iequa
            write(*,*) 'WHICH PERMITS INACCURATE SOLUTION OF',
     +                 ' SUBDOMAINS'
         end if
         if ( ioutp.ge.2 ) write(*,*) ' '
      end if

c     --- initialisation

      if ( iacnodes.gt.0 .and. isol.ne.0 ) then
         dlocal = ddot ( nmbsol,usol,1,usol,1 )
         call isglsum ( dlocal, dnorm, 1, ISDOUBLE )
         dnorm = sqrt(dnorm)
      else
         dnorm = dnrm2(nmbsol,usol,1)
      end if
      if ( dnorm.le.1.0d-30 ) then
        if ( ioutp.ge.2 .and. inodenr.eq.ISHOST) 
     +        write(*,*) "Initial estimate = 0-vector"
        do i = 1,nmbsol
          usol(i) = 0.0d0
          res(i) = rhsd(i)
        end do
      else

         call isblkmat ( nmbsol, rhsd, usol, res, iequa , isol,
     +                   isiinput, isrinput, iscoor, issolut, iscoefs,
     +                   iseddy, isinfbnd, isbndcon, ismatrix, isgmat,
     +                   isrhside, isgeom, isitopol, isdensup, iadmin,
     +                   nblocks )

         do i = 1, nmbsol
            res(i) = rhsd(i)-res(i)
         end do
      end if

      if  ( iacnodes.gt.0 .and. isol.ne.0 ) then
         dlocal = ddot( nmbsol, res, 1, res, 1 )
         call isglsum ( dlocal, r0norm, 1, ISDOUBLE )
         r0norm = sqrt ( r0norm )
      else
         r0norm = dnrm2 ( nmbsol, res, 1 )
      end if
      epslin   = eps2*r0norm

c     --- Because the right-hand-side is only incompatible up to machine
c         accuracy, eps1 must be limited
c         Below is a temporary solution, in the future
c         this minimal absolute accuracy should be based
c         on the norm of the velocity field and the machine accuracy.

      if ( epslin.lt.epsmac .and. inodenr.eq.ISHOST) then
         write(*,*) '*********************************************'
         write(*,*) 'WARNING: isddgmresr: eps2 too small'
         write(*,*) 'using absolute accuracy of 1d-15 instead'
         write(*,*) '*********************************************'
         epslin = epsmac
      end if

      if ( ioutp.ge.1 .and.inodenr.eq.ISHOST)
     +   write(*,1005) nmbsol,nmbsol,epslin,itmax,irestart,itrunc,
     +                 nreuse

      iter  = 0
      itrestart   = 0
      rnorm = r0norm

      do while ( rnorm.gt.epslin .and. iter.lt.itmax )

c     --- When we start with a zero solution for the momentum
c         equations in the first timestep, it is possible that
c         the solution is more or less zero. In such a case it
c         is difficult to satisfy the termination criterion when
c         accurate subdomain solution is used. For this reason
c         epslin is adapted in such a way that only one iteration
c         is done.

         if ( iter.eq. 0 .and. isol.eq.0 .and. iequa.lt.iseqpres .and.
     +        itimeact.eq.1 ) then

            dnorm = dnrm2(nmbsol,usol,1)

            if ( dnorm.le.1.0d-30 ) then
               epslin = r0norm*1d20
            end if
         end if

         if ( itrestart.eq.irestart) then

c        --- in case of optimized GCR, construct the solution

            if ( rstgcr ) then

c           --- The solution is:
c                  usol + sum_(j=1,...,itrestart)*searchsp(1,j)

               do j = 1, itrestart
                  call daxpy(nmbsol, phis(j), searchsp(1,j), 1,
     +                                        usol, 1)
               end do

            end if

            itrestart = 0

c           --- calculates the startresidual

            if (iter.ne.0) then

               call isblkmat ( nmbsol, rhsd, usol, res, iequa , isol,
     +                         isiinput, isrinput, iscoor, issolut,
     +                         iscoefs, iseddy, isinfbnd, isbndcon,
     +                         ismatrix, isgmat, isrhside, isgeom,
     +                         isitopol, isdensup, iadmin, nblocks )

              do i = 1, nmbsol
                 res(i) = rhsd(i)-res(i)
              end do
            end if
         end if

         iter = iter+1
         itrestart = itrestart+1

         if ( rstgcr ) then

c        --- optimized restarted GCR

            phis(itrestart) = 0d0
            do j = 1, itrestart-1
               betas(itrestart,j) = 0d0
            end do

         end if

c        --- calculate search direction searchvc and result
c            svalvec = A*searchvc

         if ( iter.le.nreuse
     +        .and.
     +        ( isol.eq.0 .or. ireusev.eq.1 )
     +      ) then

c        --- re-use a search direction vector and value vector

c            write(*,*) 'ishgmr01: reusing s + As'

            call dcopy(nmbsol, searchsp(1,iter), 1, searchvc, 1)
            call dcopy(nmbsol, svalsp(1,iter), 1, svalvec, 1)

         else if ( iter.le.nreuse ) then

c        --- re-use only a search direction and compute the
c            corresponding value

c            write(*,*) 'ishgmr01: reusing s, computing As'

            call dcopy(nmbsol, searchsp(1,iter), 1, searchvc, 1)

            call isgloma(nblocks, iequa, isiinput, issolut,
     +                   ismatrix, isitopol, iadmin,
     +                   searchvc, svalvec)

         else

c        --- compute a new search direction: searchvc

            call isblkpre ( res, searchvc,
     +              nmbsol  , iequa,
     +              isiinput, isrinput, iscoor  , issolut ,
     +              isinfbnd, isbndcon, ismatrix, isgmat ,
     +              isrhside, isgeom  , isitopol, isdensup, iadmin )

c           --- compute svalvec = A * searchvc

            call isblkmat ( nmbsol, rhsd, searchvc, svalvec, iequa ,
     +                      isol,
     +                      isiinput, isrinput, iscoor, issolut,
     +                      iscoefs, iseddy, isinfbnd, isbndcon,
     +                      ismatrix, isgmat, isrhside, isgeom,
     +                      isitopol, isdensup, iadmin, nblocks )

         end if

         if ( rstgcr ) then

c        --- optimized restarted GCR

            do j = 1, itrestart-1
               betas(j, itrestart) = 0d0
            end do
            betas(itrestart,itrestart) = 1d0

         end if

         if ( icgs2.eq.1 ) then

c        --- Reorthogonalized classical Gram-Schmidt

            rdummy = 1d0
            imin = 1
            if ( iter.gt.nreuse .or. ireusev.eq.0 ) then

               numorth = min(itrestart-1,itrunc)

c              --- Orthogonalize

               do j = 1, numorth
                  alphalocal(j) = -ddot(nmbsol,svalsp(1,j),1,
     +                                   svalvec, 1)
               end do

               if ( iacnodes.gt.0 .and. isol.ne.0 ) then
                  call isglsum ( alphalocal, alphaglobal, numorth,
     +                           ISDOUBLE )
               else
                  do j = 1,numorth
                     alphaglobal(j) = alphalocal(j)
                  end do
               end if

               do j = 1, numorth

                  call daxpy(nmbsol,alphaglobal(j),svalsp(1,j),1,
     +                        svalvec,1)
                  alphasum(j) = alphaglobal(j)

               end do

c              --- Reorthogonalize

               do j = 1, numorth
                  alphalocal(j) = -ddot(nmbsol,svalsp(1,j),1,
     +                                   svalvec,1)
               end do

               if ( iacnodes.gt.0 .and. isol.ne.0 ) then
                  call isglsum ( alphalocal, alphaglobal, numorth,
     +                           ISDOUBLE )
               else
                  do j = 1,numorth
                     alphaglobal(j) = alphalocal(j)
                  end do
               end if

               do j = 1, numorth
                  call daxpy(nmbsol,alphaglobal(j),svalsp(1,j),1,
     +                     svalvec,1)
                  alphasum(j) = alphasum(j) + alphaglobal(j)

                  if(abs(alphasum(j)).le.rdummy.or.j.eq.1) then
                     rdummy = abs(alphasum(j))
                     imin = j
                  end if

                  if ( rstgcr ) then

c                 --- optimized restarted GCR

                     call daxpy(itrestart, alphasum(j), betas(1,j),
     +                                  1, betas(1,itrestart), 1)

                  else

c                 --- GCR with Jackson & Robinson truncation

                     call daxpy(nmbsol, alphasum(j),searchsp(1,j),1,
     +                                       searchvc,1)

                  end if

               end do

            end if

         else

c        --- modified Gram-Schmidt

            rdummy = 1d0
            imin = 1
            if ( iter.gt.nreuse .or. ireusev.eq.0 ) then

               do j = 1, min(itrestart-1,itrunc)

                  if ( iacnodes.gt.0 .and. isol.ne.0 ) then
                     dlocal = -ddot(nmbsol,svalsp(1,j),1,svalvec,1)
                     call isglsum ( dlocal, betainpr, 1, ISDOUBLE )
                  else
                     betainpr = -ddot(nmbsol,svalsp(1,j),1,svalvec,1)
                  end if

                  if(abs(betainpr).le.rdummy.or.j.eq.1) then
                     rdummy = abs(betainpr)
                     imin = j
                  end if
                  call daxpy(nmbsol,betainpr,svalsp(1,j),1,svalvec,1)
                  if ( rstgcr ) then

c                 --- optimized restarted GCR

                     call daxpy(itrestart, betainpr, betas(1,j), 1,
     +                                  betas(1,itrestart), 1)

                  else

c                 --- GCR with Jackson & Robinson truncation

                     call daxpy(nmbsol,betainpr,searchsp(1,j),1,
     +                                       searchvc,1)

                  end if
               end do

            end if

         end if

c        --- Below a choice in the truncation strategy is possible.
c            The choice is made by removing the c* in the first column,
c            and adding a c* before the non-wanted strategies.
c            reference
c            ---------
c            C. Vuik
c            Further experiences with GMRESR
c            Technical University Report 92-12, Delft, 1992.

c            --- This is the trunclast strategy
c            ioverwrt = mod(itn,ntrunk)
c            if(ioverwrt.eq.0) ioverwrt = ntrunk

c            --- This is the truncfirst strategy
c            ioverwrt = min(itn,ntrunk)

c            --- This is the Jackson & Robinson strategy

         if(itrestart.gt.itrunc) then
            ioverwrt = imin
         else
            ioverwrt = itrestart
         end if

c        --- End of truncation strategies

         if ( iacnodes.gt.0 .and. isol.ne.0 ) then
            dlocal = ddot ( nmbsol, svalvec, 1, svalvec, 1 )
            call isglsum ( dlocal, dnorm, 1, ISDOUBLE )
            dnorm = sqrt( dnorm )
         else
            dnorm = dnrm2(nmbsol,svalvec,1)
         end if

         rdummy = 1.0d0/dnorm
         if ( rstgcr ) then

c        --- optimized restarted GCR

            do i = 1, itrestart
               betas(i,itrestart)  = betas(i,itrestart)*rdummy
            end do
            call dcopy(nmbsol, searchvc, 1, searchsp(1,itrestart), 1)

         else

c        --- GCR with truncation

            do i = 1, nmbsol
               searchsp(i,ioverwrt) = searchvc(i)*rdummy
            end do

         end if

         do i = 1, nmbsol
            svalsp(i,ioverwrt) = svalvec(i)*rdummy
         end do

c        --- update x and r

         if ( iacnodes.gt.0 .and. isol.ne.0 ) then
            dlocal = ddot(nmbsol,svalsp(1,ioverwrt),1,res,1)
            call isglsum ( dlocal, rdummy, 1, ISDOUBLE )
         else
            rdummy = ddot(nmbsol,svalsp(1,ioverwrt),1,res,1)
         end if

         call daxpy(nmbsol,-rdummy,svalsp(1,ioverwrt),1,res,1)

         if ( rstgcr ) then

c        --- optimized restarted GCR

            phis(itrestart) = 0d0
            call daxpy(itrestart, rdummy, betas(1,itrestart),1,
     +                                    phis, 1)

         else

            call daxpy(nmbsol,rdummy,searchsp(1,ioverwrt),1,usol,1)

         end if

         if ( iacnodes.gt.0 .and. isol.ne.0 ) then
            dlocal = ddot ( nmbsol, res, 1 , res, 1 )
            call isglsum ( dlocal, rnorm, 1, ISDOUBLE )
            rnorm = sqrt(rnorm)
         else
            rnorm = dnrm2(nmbsol,res,1)
         end if

         resid(iter) = rnorm

c        --- print some results

         if ( ioutp.ge.2 .and.inodenr.eq.ISHOST)
     +      write(*,1010) iter-nreuse,usol(1),rnorm

c        --- end while ( rnorm.gt.eps1 .and. iter.lt.itmax )

      end do

      nreuse = min(iter,itrunc)

c     --- investigate the reason to stop

      if ( rnorm.gt.epslin ) then

c     --- no convergence

         call iseropen('ishgmr01')

         if ( isol.eq.0 ) then

            call iserrint(iequa, 1)
            call iserrint(itrunc, 2)
            call iserrint(1, 3)
            call iserrint(itmax, 4)
            call iserreal(eps2, 1)
            call iswarsub(1605, 4, 1, 0)

         else

            call iserrint(iequa, 1)
            call iserrint(itrunc, 2)
            call iserrint(itmax, 3)
            call iserreal(eps2, 1)
            call iswarsub(1607, 3, 1, 0)

         end if

         call iserclos('ishgmr01')

      else

         if ( ioutp.ge.2 .and. inodenr.eq.ISHOST) then

            write(*,*) ' '
            write(*,*) 'CONVERGED FOR EQUATION ', iequa

         end if

      end if

c     --- calculate the exact residual.

      if ( ioutp.ge.1.and.inodenr.eq.ISHOST ) write(*,1020) iter,rnorm

c     --- in case of optimized GCR, construct the solution

      if ( rstgcr ) then

c     --- The solution is:
c            usol + sum_(j=1,...,itrestart)*searchsp(1,j)

         do j = 1, itrestart
            call daxpy(nmbsol, phis(j), searchsp(1,j), 1,
     +                                  usol, 1)
         end do

      end if

c     --- Note: computation of the exact residual is essential
c         with accurate solution of subdomains.

      if ( ioutp.ge.1 .or. isol.eq.0 ) then

         call isblkmat ( nmbsol, rhsd, usol, res, iequa , isol,
     +                   isiinput, isrinput, iscoor, issolut, iscoefs,
     +                   iseddy, isinfbnd, isbndcon, ismatrix, isgmat,
     +                   isrhside, isgeom, isitopol, isdensup, iadmin,
     +                   nblocks )

         do i = 1, nmbsol
            res(i) = rhsd(i)-res(i)
         end do
         if ( iacnodes.gt.0 .and. isol.ne.0 ) then
            dlocal = ddot ( nmbsol, res, 1 , res, 1 )
            call isglsum ( dlocal, rnorm, 1, ISDOUBLE )
            rnorm = sqrt(rnorm)
         else
            rnorm = dnrm2(nmbsol,res,1)
         end if

      end if

      if ( ioutp.ge.1 .and.inodenr.eq.ISHOST) write(*,1030) rnorm

c     --- Test if the exact norm of the residual satisfies the stop
c         criterion

      if ( rnorm.gt.epslin ) then

         if ( rnorm.le.25*epslin ) then

            if ( ioutp.ge.1 ) call iswarsub ( 1611, 0, 0, 0 )

         else

            call iserrsub ( 1611, 0, 0, 0 )

         end if

      end if

      if ( nerror.ne.0 ) go to 1000

1000  call iserclos('isblkgcr')

1005  format(' Solution of a linear system calculated with '
     +       ,' GCR ',/,
     +       ' the matrix A is',i6,' by',i6,/,
     +       ' required accuracy is',ss,1pe13.4e2,' itmax is ',i4,/,
     +       ' The restart value is ',i4,/,
     +       ' There are only ',i4,
     +       ' vektors searchsp and svalsp used.',/,
     +       ' Number of reused search directions is ', i4)
1010  format(i6,ss,1pe13.4e2,1pe13.4e2)
1020  format(' Number of iterations is',i4,/,
     +          ' The estimated norm of the residual is ',ss,1pe13.4e2)
1030  format(' The exact norm of the residual is    ',ss,1pe13.4e2)

      return
      end
