      subroutine isco3put ( iblock, iface, isubface, ni, nj, nk,
     +                      nvirtual, nblocks, nfaces, maxnsubf,
     +                      nsubfaces, subfaces, coor, vco, maxnijk,
     +                      corner1 )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date 24-06-1997 Kees K. :extension with corners
c                                       and orientation
c        version 1.0    date 14-02-1996 3d extension of isco3put
c
c
c   copyright (c) 1996-1996  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Put the virtual coordinates from an intermediate representation vco into
c     the virtual region of a certain subface of  a certain block.
c     These coordinates are numbered in the increasing i and j directions.
c     Coordinates are numbered in the tangential direction first, then
c     in a direction normal to the interface. The x coordinates occur
c     first then the y coordinates.
c     The intermediate representation may be treated as a multi-dimensional
c     array as follows:
c     vco( 1:nvirtual, 1:ndim, 1:number_of_coordinates_in_tangential_dirs)
c     The orientation of the neighbouring faces w.r.t. each other must
c     be known, which is also determined in this subroutine.
c     The restriction at this moment is that the blocks must be coupled
c     through complete faces, no subfaces are taken into account.
c **********************************************************************
c
c                       KEYWORDS
c
c   mapping_array
c   topology
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, iface, isubface, maxnsubf, nblocks, maxnijk,
     +        nfaces, ni, nj, nk, nvirtual, nsubfaces(nblocks,nfaces),
     +        subfaces(nblocks,nfaces,maxnsubf,*)
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,3),
     +                 vco(nvirtual,3,maxnijk+1,maxnijk+1),
     +                 corner1(1:3)

c     coor     i/o   reserved
c     corner1   i    First corner of neighbouring block
c     iblock    i    Block sequence number
c     iface     i    Face number
c     isubface  i    Subface number
c     maxnijk   i    Max(ni,nj,nk) over all blocks
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     nfaces    i    Number of faces
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     nvirtual  i    Number of rows of virtual cells
c     subfaces  i    reserved
c     vco       i    Work array in which the co-ordinates to be copied are
c                    stored
c                    vco has dimension: vco(1:nvirtual, 1:ndim, 1:nijk+1)
c                    with nijk max(ni,nj,nk)
c                    Array vco(i,m,n) contains the differences between points
c                    at a distance from the boundary and the corresponding
c                    points on the boundary.
c                    i:  the distance normal to the subface
c                    m:  refers tot the x (m=1), y (m=2), or z (m=3)
c                        co-ordinate
c                    n:  the distance along the subface
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i1, i2, idim, ihigh, ii, ilow, inorm, j1,
     +        j2, jhigh, jj, jlow, ishift, k1, k2, kk
      integer kface(6), i, ipos, iorient, kndface, ifto, ibto,
     +        isfto, istartp
      integer facenrs(6,2)
      double precision cornersn(1:4,1:3), diffc, mindiffc

c     cornersn  Array with coordinates of the corners of the face
c     diffc     Differenc between coordinates
c     facenrs   array with two times six facenumbers for kind_of_face=1
c               and kind_of_face=2, used for kndface
c     i         General loop variable
c     i1        Lower index number along first direction of a face or edge
c     i2        Upper index number along first direction of a face or edge
c     ibto      Target block
c     idim      Loop variable used for dimension
c     ifto      Target face
c     ihigh     Upper index of subface
c     ii        Loop variable
c     ilow      Lower index of subface
c     inorm     Loop variable
c     iorient   Relative orientation of subfaces, see itopol
c     ipos      Position in array
c     isfto     Target subface
c     ishift    Shift in i-direction
c     istartp   indicating if two neighbouring faces are rotated
c               w.r.t. each other( counter clockwise).
c               istartp = 1  : no rotation
c               istartp = 2  : rotation of pi * 1/2
c               istartp = 3  : rotation of pi
c               istartp = 4  : rotation of pi * 3/2
c     j1        Lower index number along second direction of a face
c     j2        Upper index number along second direction of a face
c     jhigh     Upper j-index for a subface
c     jj        Loop variable
c     jlow      Lower j-index for a subface
c     k1        Lower index number along third direction of a face or edge
c     k2        Upper index number along third direction of a face or edge
c     kface     array with kind of face numbers
c     kk        Loop variable
c     kndface   we have to kinds of faces w.r.t. the direction of
c               the tangential velocities; if the orientation is the same
c               it means that the tangential directions can be made the same
c               by rotation of one of the faces.
c               a face has the same orientation
c               with the faces 1,4,5 or with the faces 2,3,6
c               kind of face = 1 :  1,4,5,  2,3,6  ; same with  1,4,5,
c               kind of face = 2 :  2,3,6,  1,4,5  ; same with  2,3,6,
c     mindiffc  Minimal differenc between coordinates
c **********************************************************************
c
c                       I/O
c
c      none
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c **********************************************************************
c
c                       ERROR MESSAGES
c
c    1600   iblock out of range   (iblock,nblocks)
c    1601   iface out of range    (iface,nfaces)
c    1602   isubface out of range (isubface,nsubfaces)
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c     perform some checks
c     Compute the i and j bounds for the co-ordinates at the boundary
c     Compute the co-ordinates at boundary from vco
c
c     See subroutine isco3put for an explanation
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data kface / 2,1,1,2,2,1 /
      data facenrs / 1,4,5, 2,3,6,
     +               2,3,6, 1,4,5 /
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- first perform some checks

      if ( iblock.lt.1 .or. iblock.gt.nblocks ) then

         call iseropen('isco3put')
         call iserrint(iblock, 1)
         call iserrint(nblocks, 2)
         call iserrsub ( 1600, 2, 0, 0)
         call iserclos('isco3put')

      end if

      if ( iface.lt.1 . or. iface.gt.nfaces ) then

         call iseropen('isco3put')
         call iserrint(iface, 1)
         call iserrint(nfaces, 2)
         call iserrsub ( 1601, 2, 0, 0)
         call iserclos('isco3put')

      end if

      if ( isubface.lt.1 .or.
     +     isubface.gt.nsubfaces(iblock,iface) ) then

         call iseropen('isco3put')
         call iserrint(isubface, 1)
         call iserrint(iblock,   2)
         call iserrint(iface,    3)
         call iserrint(nsubfaces(iblock, iface), 4)
         call iserrsub ( 1602, 4, 0, 0)
         call iserclos('isco3put')

      end if

c     --- we have to kinds of faces w.r.t. the direction of
c         the tangential velocities; if the orientation is the same
c         it means that the tangential directions can be made the same
c         by rotation of one of the faces.
c         a face has the same orientation
c         with the faces 1,4,5 or with the faces 2,3,6
c         kind of face = 1 :  1,4,5,  2,3,6  ; same with  1,4,5,
c         kind of face = 2 :  2,3,6,  1,4,5  ; same with  2,3,6,

      ifto = subfaces(iblock, iface, isubface, 6)
      kndface = kface(ifto)

      do i =1, 6
         if ( iface.eq.facenrs(i,kndface) ) ipos=i
      end do
      if ( ipos.le.3) then

c     --- orientation is the same

         iorient = 1

      else

c     --- orientation is not the same

         iorient = 0

      end if

c     --- Compute the i, j and k bounds for the co-ordinates at the boundary

      ilow = subfaces(iblock, iface, isubface, 1)
      ihigh = subfaces(iblock, iface, isubface, 2) + 1
      jlow = subfaces(iblock, iface, isubface, 3)
      jhigh = subfaces(iblock, iface, isubface, 4) + 1

      if ( iface.eq.1 ) then

         ii = 1
         j1 = ilow
         j2 = ihigh
         k1 = jlow
         k2 = jhigh

      else if ( iface.eq.2 ) then

         ii = ni+1
         j1 = ilow
         j2 = ihigh
         k1 = jlow
         k2 = jhigh

      else if ( iface.eq.3 ) then

         jj = 1
         i1 = jlow
         i2 = jhigh
         k1 = ilow
         k2 = ihigh

      else if ( iface.eq.4 ) then

         jj = nj+1
         i1 = jlow
         i2 = jhigh
         k1 = ilow
         k2 = ihigh

      else if ( iface.eq.5 ) then

         kk = 1
         i1 = ilow
         i2 = ihigh
         j1 = jlow
         j2 = jhigh

      else

         kk = nk+1
         i1 = ilow
         i2 = ihigh
         j1 = jlow
         j2 = jhigh

      end if

c     --- Compute coordinates by adding the differences stored in vco

      do inorm = 1, nvirtual

         if ( mod(iface,2).eq.1 ) then

            ishift = -inorm

         else

            ishift = inorm

         end if

         if ( iorient.eq.1 ) then

c        --- orientation is the same

            if ( iface.le.2 ) then

c           --- faces 1 and 2, loop over j and k

c               --- computing how much the face must be rotated
c                  for matching with the neighbouring face, this
c                  is done by looking for which corner point has
c                  the same coordinates as the first corner point
c                  of the neighbouring face

               do idim = 1, 3
                  cornersn(1,idim)=coor(ii,j1,k1,idim)
                  cornersn(2,idim)=coor(ii,j2,k1,idim)
                  cornersn(3,idim)=coor(ii,j2,k2,idim)
                  cornersn(4,idim)=coor(ii,j1,k2,idim)
               end do

               istartp  = 1
               mindiffc = 1d10
               do i = 1, 4
                  diffc = abs(corner1(1) - cornersn(i,1)) +
     +                    abs(corner1(2) - cornersn(i,2)) +
     +                    abs(corner1(3) - cornersn(i,3))
                  if ( diffc.lt.mindiffc ) then
                     mindiffc = diffc
                     istartp = i
                  end if
               end do

               if (istartp.eq.1) then

c              --- if startpoint=1 no order change

                  do idim = 1, 3
                     do jj = j1, j2
                        do kk = k1, k2
                           coor(ii+ishift,jj,kk,idim) =
     +                        vco(inorm,idim,jj-j1+1,kk-k1+1) +
     +                        coor(ii,jj,kk,idim)
                        end do
                     end do
                  end do

               else if (istartp.eq.2) then

c              --- exchange jj and kk and take jj in reverse order

                  do idim = 1, 3
                     do jj = j2, j1,-1
                        do kk = k1, k2
                           coor(ii+ishift,kk,jj,idim) =
     +                        vco(inorm,idim,jj-j1+1,kk-k1+1) +
     +                        coor(ii,kk,jj,idim)
                        end do
                     end do
                  end do

               else if (istartp.eq.3) then

c              --- take jj and kk in reverse order

                  do idim = 1, 3
                     do jj = j2, j1,-1
                        do kk = k2, k1,-1
                           coor(ii+ishift,jj,kk,idim) =
     +                        vco(inorm,idim,jj-j1+1,kk-k1+1) +
     +                        coor(ii,jj,kk,idim)
                        end do
                     end do
                  end do

               else if (istartp.eq.4) then

c              --- exchange jj and kk and take kk in reverse order

                  do idim = 1, 3
                     do jj = j1, j2
                        do kk = k2, k1,-1
                           coor(ii+ishift,kk,jj,idim) =
     +                        vco(inorm,idim,jj-j1+1,kk-k1+1) +
     +                        coor(ii,kk,jj,idim)
                        end do
                     end do
                  end do

               end if

            else if ( iface.le.4 ) then

c           --- faces 3 and 4, loop over i and k

               do idim = 1, 3
                  cornersn(1,idim)=coor(i1,jj,k1,idim)
                  cornersn(2,idim)=coor(i2,jj,k1,idim)
                  cornersn(3,idim)=coor(i2,jj,k2,idim)
                  cornersn(4,idim)=coor(i1,jj,k2,idim)
               end do

               istartp  = 1
               mindiffc = 1d10
               do i = 1, 4
                  diffc = abs(corner1(1) - cornersn(i,1)) +
     +                    abs(corner1(2) - cornersn(i,2)) +
     +                    abs(corner1(3) - cornersn(i,3))
                  if ( diffc.lt.mindiffc ) then
                     mindiffc = diffc
                     istartp  = i
                  end if
               end do

               if ( istartp.eq. 1 ) then

c              --- if startpoint=1 no order change

                  do idim = 1, 3
                     do ii = i1, i2
                        do kk = k1, k2
                           coor(ii,jj+ishift,kk,idim) =
     +                        vco(inorm,idim,ii-i1+1,kk-k1+1) +
     +                        coor(ii,jj,kk,idim)
                        end do
                     end do
                  end do

               else if ( istartp.eq. 2 ) then

c              --- exchange ii and kk and take ii in reverse order

                  do idim = 1, 3
                     do ii = i2, i1, -1
                        do kk = k1, k2
                           coor(kk,jj+ishift,ii,idim) =
     +                        vco(inorm,idim,ii-i1+1,kk-k1+1) +
     +                        coor(kk,jj,ii,idim)
                        end do
                     end do
                  end do

               else if (istartp.eq.3) then

c              --- take ii and kk in reverse order

                  do idim = 1, 3
                     do ii = i2, i1,-1
                        do kk = k2, k1,-1
                           coor(ii,jj+ishift,kk,idim) =
     +                        vco(inorm,idim,ii-i1+1,kk-k1+1) +
     +                        coor(ii,jj,kk,idim)
                        end do
                     end do
                  end do

               else if (istartp.eq.4) then

c              --- exchange ii and kk and take kk in reverse order

                  do idim = 1, 3
                     do ii = i1, i2
                        do kk = k2, k1,-1
                           coor(kk,jj+ishift,ii,idim) =
     +                        vco(inorm,idim,ii-i1+1,kk-k1+1) +
     +                        coor(kk,jj,ii,idim)
                        end do
                     end do
                  end do

               end if

            else

c           --- faces 5 and 6, loop over i and j

               do idim = 1, 3
                  cornersn(1,idim)=coor(i1,j1,kk,idim)
                  cornersn(2,idim)=coor(i2,j1,kk,idim)
                  cornersn(3,idim)=coor(i2,j2,kk,idim)
                  cornersn(4,idim)=coor(i1,j2,kk,idim)
               end do

               istartp  = 1
               mindiffc = 1d10
               do i = 1, 4
                  diffc = abs(corner1(1) - cornersn(i,1)) +
     +                    abs(corner1(2) - cornersn(i,2)) +
     +                    abs(corner1(3) - cornersn(i,3))
                  if ( diffc.lt.mindiffc ) then
                     mindiffc = diffc
                     istartp = i
                  end if
               end do

               if ( istartp.eq. 1 ) then

c              --- if startpoint=1 no order change

                  do idim = 1, 3
                     do ii = i1, i2
                        do jj = j1, j2
                           coor(ii,jj,kk+ishift,idim) =
     +                        vco(inorm,idim,ii-i1+1,jj-j1+1) +
     +                        coor(ii,jj,kk,idim)
                        end do
                     end do
                  end do

               else if ( istartp.eq. 2 ) then

c              --- exchange ii and jj and take ii in reverse order

                  do idim = 1, 3
                     do ii = i2, i1,-1
                        do jj = j1, j2
                           coor(jj,ii,kk+ishift,idim) =
     +                        vco(inorm,idim,ii-i1+1,jj-j1+1) +
     +                        coor(jj,ii,kk,idim)
                        end do
                     end do
                  end do

               else if (istartp.eq.3) then

c              --- take ii and jj in reverse order

                  do idim = 1, 3
                     do ii = i2, i1,-1
                        do jj = j2, j1,-1
                           coor(ii,jj,kk+ishift,idim) =
     +                        vco(inorm,idim,ii-i1+1,jj-j1+1) +
     +                        coor(ii,jj,kk,idim)
                        end do
                     end do
                  end do

               else if (istartp.eq.4) then

c              --- exchange ii and jj and take jj in reverse order

                  do idim = 1, 3
                     do ii = i1, i2
                        do jj = j2, j1,-1
                           coor(jj,ii,kk+ishift,idim) =
     +                        vco(inorm,idim,ii-i1+1,jj-j1+1) +
     +                        coor(jj,ii,kk,idim)
                        end do
                     end do
                  end do

               end if

            end if

         else if ( iorient.eq.0 ) then

c        --- orientation is not the same

            if ( iface.le.2 ) then

c           --- faces 1 and 2, loop over j and k

               do idim = 1, 3
                  cornersn(1,idim)=coor(ii,j1,k1,idim)
                  cornersn(2,idim)=coor(ii,j2,k1,idim)
                  cornersn(3,idim)=coor(ii,j2,k2,idim)
                  cornersn(4,idim)=coor(ii,j1,k2,idim)
               end do

               istartp  = 1
               mindiffc = 1d10
               do i = 1, 4
                  diffc = abs(corner1(1) - cornersn(i,1)) +
     +                    abs(corner1(2) - cornersn(i,2)) +
     +                    abs(corner1(3) - cornersn(i,3))
                  if ( diffc.lt.mindiffc ) then
                     mindiffc = diffc
                     istartp = i
                  end if
               end do

               if (istartp.eq.1) then

c              --- if startpoint=1 exchange jj and kk

                  do idim = 1, 3
                     do jj = j1, j2
                        do kk = k1, k2
                           coor(ii+ishift,kk,jj,idim) =
     +                        vco(inorm,idim,jj-j1+1,kk-k1+1) +
     +                        coor(ii,kk,jj,idim)
                        end do
                     end do
                  end do

               else if (istartp.eq.2) then

c              --- take jj in reverse order

                  do idim = 1, 3
                     do jj = j2, j1,-1
                        do kk = k1, k2
                           coor(ii+ishift,jj,kk,idim) =
     +                        vco(inorm,idim,jj-j1+1,kk-k1+1) +
     +                        coor(ii,jj,kk,idim)
                        end do
                     end do
                  end do

               else if (istartp.eq.3) then

c              --- exchange jj and kk and take jj and kk in reverse order

                  do idim = 1, 3
                     do jj = j2, j1,-1
                        do kk = k2, k1,-1
                           coor(ii+ishift,kk,jj,idim) =
     +                        vco(inorm,idim,jj-j1+1,kk-k1+1) +
     +                        coor(ii,kk,jj,idim)
                        end do
                     end do
                  end do

               else if (istartp.eq.4) then

c              --- take kk in reverse order

                  do idim = 1, 3
                     do jj = j1, j2
                        do kk = k2, k1,-1
                           coor(ii+ishift,jj,kk,idim) =
     +                        vco(inorm,idim,jj-j1+1,kk-k1+1) +
     +                        coor(ii,jj,kk,idim)
                        end do
                     end do
                  end do

               end if

            else if ( iface.le.4 ) then

c           --- faces 3 and 4, loop over i and k

               do idim = 1, 3
                  cornersn(1,idim)=coor(i1,jj,k1,idim)
                  cornersn(2,idim)=coor(i2,jj,k1,idim)
                  cornersn(3,idim)=coor(i2,jj,k2,idim)
                  cornersn(4,idim)=coor(i1,jj,k2,idim)
               end do

               istartp  = 1
               mindiffc = 1d10
               do i = 1, 4
                  diffc = abs(corner1(1) - cornersn(i,1)) +
     +                    abs(corner1(2) - cornersn(i,2)) +
     +                    abs(corner1(3) - cornersn(i,3))

                  if ( diffc.lt.mindiffc ) then
                     mindiffc = diffc
                     istartp = i
                  end if
               end do

               if ( istartp.eq. 1 ) then

c              --- exchange ii and kk

                  do idim = 1, 3
                     do ii = i1, i2
                        do kk = k1, k2
                           coor(kk,jj+ishift,ii,idim) =
     +                        vco(inorm,idim,ii-i1+1,kk-k1+1) +
     +                        coor(kk,jj,ii,idim)
                        end do
                     end do
                  end do

               else if ( istartp.eq. 2 ) then

c              --- take ii in reverse order

                  do idim = 1, 3
                     do ii = i2, i1, -1
                        do kk = k1, k2
                           coor(ii,jj+ishift,kk,idim) =
     +                        vco(inorm,idim,ii-i1+1,kk-k1+1) +
     +                        coor(ii,jj,kk,idim)
                        end do
                     end do
                  end do

               else if (istartp.eq.3) then

c              --- exchange ii and kk andtake ii and kk in reverse order

                  do idim = 1, 3
                     do ii = i2, i1,-1
                        do kk = k2, k1,-1
                           coor(kk,jj+ishift,ii,idim) =
     +                        vco(inorm,idim,ii-i1+1,kk-k1+1) +
     +                        coor(kk,jj,ii,idim)
                        end do
                     end do
                  end do

               else if (istartp.eq.4) then

c              --- take kk in reverse order

                  do idim = 1, 3
                     do ii = i1, i2
                        do kk = k2, k1,-1
                           coor(ii,jj+ishift,kk,idim) =
     +                        vco(inorm,idim,ii-i1+1,kk-k1+1) +
     +                        coor(ii,jj,kk,idim)
                        end do
                     end do
                  end do

               end if

            else

c           --- faces 5 and 6, loop over i and j

               do idim = 1, 3
                  cornersn(1,idim)=coor(i1,j1,kk,idim)
                  cornersn(2,idim)=coor(i2,j1,kk,idim)
                  cornersn(3,idim)=coor(i2,j2,kk,idim)
                  cornersn(4,idim)=coor(i1,j2,kk,idim)
               end do

               istartp  = 1
               mindiffc = 1d10
               do i = 1, 4
                  diffc = abs(corner1(1) - cornersn(i,1)) +
     +                    abs(corner1(2) - cornersn(i,2)) +
     +                    abs(corner1(3) - cornersn(i,3))
                  if ( diffc.lt.mindiffc ) then
                     mindiffc = diffc
                     istartp = i
                  end if
               end do

               if ( istartp.eq. 1 ) then

c              --- exchange ii and jj

                  do idim = 1, 3
                     do ii = i1, i2
                        do jj = j1, j2
                           coor(jj,ii,kk+ishift,idim) =
     +                        vco(inorm,idim,ii-i1+1,jj-j1+1) +
     +                        coor(jj,ii,kk,idim)
                        end do
                     end do
                  end do

               else if ( istartp.eq. 2 ) then

c              --- take ii in reverse order

                  do idim = 1, 3
                     do ii = i2, i1,-1
                        do jj = j1, j2
                           coor(ii,jj,kk+ishift,idim) =
     +                        vco(inorm,idim,ii-i1+1,jj-j1+1) +
     +                        coor(ii,jj,kk,idim)
                        end do
                     end do
                  end do

               else if (istartp.eq.3) then

c              --- exchange ii and jj andtake ii and jj in reverse order

                  do idim = 1, 3
                     do ii = i2, i1,-1
                        do jj = j2, j1,-1
                           coor(jj,ii,kk+ishift,idim) =
     +                        vco(inorm,idim,ii-i1+1,jj-j1+1) +
     +                        coor(jj,ii,kk,idim)
                        end do
                     end do
                  end do

               else if (istartp.eq.4) then

c              --- take jj in reverse order

                  do idim = 1, 3
                     do ii = i1, i2
                        do jj = j2, j1,-1
                           coor(ii,jj,kk+ishift,idim) =
     +                        vco(inorm,idim,ii-i1+1,jj-j1+1) +
     +                        coor(ii,jj,kk,idim)
                        end do
                     end do
                  end do

               end if

            end if

         end if

      end do

c     --- put istartp in subfaces array pos. 8

      ibto  = subfaces(iblock, iface, isubface, 5)
      ifto  = subfaces(iblock, iface, isubface, 6)
      isfto = subfaces(iblock, iface, isubface, 7)
      subfaces(ibto  , ifto , isfto   , 8) = istartp

      end
