      subroutine isdd ( ieq, isiinput, isrinput, iscoor, issolut,
     +                  iscoefs, iseddy, isinfbnd, isbndcon, ismatrix,
     +                  isgmat, isrhside, isgeom, isitopol, isdensup,
     +                  iadmin)
c ======================================================================
c
c     programmer   Erik Brakkee
c     version      2.5 date 21-09-1999 Guus: new call to ismblk
c     version      2.4 date 14-07-1999 Kees V: ishgcr replaced by isblksol
c     version      2.3 date 26-02-1999 Kees V: (in)accurate combined
c     version      2.2 date 23-11-1998 Jason: SPMD version
c     version      2.1 date 31-08-1998 Jason: remove check nodeid
c     version      2.0 date 14-05-1998 Duncan: extension for saving
c                                              coefficients dens upwind
c                                              new call to ishgmr
c                                              new call to ishplain
c                                              new call to ishgcr
c     version      1.9 date 13-10-1997 Charles: uncoupled allowed
c     version      1.8 date 06-08-1997 Guus: return in case of error
c     version      1.3 date 28-04-1997 Kees K:change for 3D
c     version      1.2 date 06-02-1995 Michiel K:array raction removed
c     version      1.1 date 03-11-1994 Kees K coefs and geom added
c     version      1.0 date 24-05-1994 Erik: initial revision
c
c   copyright (c) 1994-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     This routine solves a domain decomposition problem for a certain
c     type of unknown using either the unaccelerated or GMRESR accelerated
c     algorithm.
c     In case of a single block problem, it reduces to the linear solver
c **********************************************************************
c
c                       KEYWORDS
c
c     domain_decompostion
c     linear_solver
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ieq , iadmin(*)  , isbndcon(*), iscoor(*)  ,
     +        iseddy(*)  , isgmat(*)  , isiinput(*), isinfbnd(*),
     +        isitopol   , ismatrix(*), isrhside(*), isrinput(*),
     +        issolut(*) , iscoefs(*)   , isgeom(*), isdensup(*)

c     iadmin   i    reserved
c     ieq      i    Equation number
c     isbndcon i    Array of sequence numbers (nblocks) of array bndcon
c     iscoefs  i    Array of sequence numbers (nblocks) of array coefs
c     iscoor   i    Array of sequence numbers (nblocks) of array coor
c     isdensup i    Array of sequence numbers (nblocks) of array densup
c     iseddy   i    Array of sequence numbers (nblocks) of array eddy
c     isgeom   i    Array of sequence numbers (nblocks) of array geom
c     isgmat   i    Array of sequence numbers (nblocks) of array gmat
c     isiinput i    Array of sequence numbers (nblocks) of array iinput
c     isinfbnd i    Array of sequence numbers (nblocks) of array infbnd
c     isitopol i    Sequence number of array itopol
c     ismatrix i    Array of sequence numbers (nblocks) of array matrix
c     isrhside i    Array of sequence numbers (nblocks) of array rhside
c     isrinput i    Array of sequence numbers (nblocks) of array rinput
c     issolut  i    Array of sequence numbers (nblocks) of array solut
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ialg, ilev, ind, ipiinblk, ipiinput, itrans, nblocks,
     +        nvirt, vlen, iblock, isol, ndim, icoupled, iseqpres
      logical done

c     done       indicates node domain decomposition should stop
c     ialg       Type of multi-block algorithm to use
c                0 = accelerated
c                1 = unaccelerated
c     iblock     Block sequence number
c     icoupled   If 0 the momentum equations are coupled, if 1 they are not
c     ilev       Time level of the multi-block transport of information
c     ind        Sequence number of node
c     ipiinblk   Starting address of array IINBLK in array IINPUT
c     ipiinput   Starting address of array iinput in ibuffr
c     iseqpres   Sequence number of pressure equation
c     isol       Accuracy of subdomain solution in relation with
c                domain decomposition.
c                  0 = accurate enough
c                  1 = inaccurate with respect to multi-block accuracy
c                  2 = inaccurate using ilu approximation
c     itrans     reserved
c     nblocks    Number of blocks
c     ndim       Dimension of space (2 or 3)
c     nvirt      Number of virtual unknowns
c     vlen       Number of virtual tangential velocities needed from
c                neighbouring blocks
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isingetp

c     ISACTIVE  Activate old permanent array
c     ISBLKSOL  Solves multi-block problem using accelerated algorithm
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISHPLAIN  Solve multi-block problem using unaccelerated algorithm
c     ISINGETP  get the pointer corresponding to an integer array
c     ISNDD     Node domain decomposition function
c     ISSNDDHS  Copy the arrays of virtual unknowns for all
c               blocks to the local copies at the nodes, using PVM
c               if necessary.
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('isdd')

c     --- first determine which algorithm to use

      call isactive(isiinput(1))
      ipiinput = isingetp(isiinput(1))
      nblocks  = ibuffr(ipiinput + 73 - 1)
      ndim     = ibuffr(ipiinput + 57 - 1)
      icoupled = ibuffr(ipiinput + 76 - 1)
      if ( icoupled.eq.0 ) then
         iseqpres = 2
      else
         iseqpres = ndim + 1
      end if
      ipiinblk = ibuffr(ipiinput + 7 - 1) + ipiinput - 1
      ialg     = ibuffr(ipiinblk + 1 + 10*ieq - 1)
      isol     = ibuffr(ipiinblk + 2 - 1)

c     --- compute the dimension of the problem for GMRESR

      nvirt = 0
      do iblock = 1, nblocks

c     --- set iblockrf for error messages

         iblockrf = iblock

c        --- compute vlen for block iblock

         call isactive(isiinput(iblock))
         ipiinput = isingetp(isiinput(iblock))
         vlen     = ibuffr(ipiinput + 58 - 1)
         nvirt    = nvirt + vlen

      end do
      if ( nerror.ne.0 ) go to 1000

c     --- twice as many virtual unknowns for the momentum equation
c         In case of a single block nvirt = 0 and vlen = 0

      if ( ieq.eq.1 ) nvirt = 2*nvirt

      if ( isol.eq.0 ) then

c     --- this is the parallel case

         if (inodenr.eq.ISHOST) then

            if ( ialg.eq.0 .and. nvirt.ne.0 ) then

c           --- accelerated algorithm

            call isblksol ( ieq, isiinput, isrinput, iscoor  , issolut ,
     +                 iscoefs, iseddy  , isinfbnd, isbndcon, ismatrix,
     +                 isgmat , isrhside, isgeom  , isitopol, isdensup,
     +                 iadmin, isol, nblocks )
            else

c           --- unaccelerated algorithm
c               This is the standard algorithm used in case of single block

               call ishplain ( ieq, isiinput, isrinput, iscoor,
     +                      issolut, iscoefs, iseddy, isinfbnd,
     +                      isbndcon, ismatrix, isgmat, isrhside,
     +                      isgeom, isitopol, isdensup, iadmin )
            end if

c           --- final transport of virtual unknowns (multi-block only)

            if ( ieq.lt.iseqpres ) then
               itrans = 1
            else
               itrans = ieq + ( ndim - 1 )*( 1 - icoupled )
            end if

            ilev = 2
            done = .true.

            do ind = 1, iacnodes

c           --- for all nodes not on the host

               call issnddhs( nblocks, itrans, ilev, ind, isiinput,
     +                        issolut, isitopol, iadmin, done)

            end do

         else

            call isndd ( ieq, nblocks, isiinput, isrinput, iscoor,
     +                   issolut, iscoefs, iseddy, isinfbnd, isbndcon,
     +                   ismatrix, isgmat, isrhside, isgeom, isitopol,
     +                   isdensup, iadmin )

         end if

      else

c     --- GCR based algorithm used with inaccurate solution
c         of subdomains

         call isblksol ( ieq, isiinput, isrinput, iscoor, issolut ,
     +                   iscoefs, iseddy, isinfbnd, isbndcon, ismatrix,
     +                   isgmat, isrhside, isgeom, isitopol, isdensup,
     +                   iadmin, isol, nblocks )

      end if
      if ( nerror.ne.0 ) go to 1000

1000  call iserclos('isdd')

      end
