      function isddpu04 ( iblock, iface, isubface, ilev, ni,
     +                    nj, ntraq1, ntraq2, nvirtual, nblocks,
     +                    nfaces, maxnsubf, idiscret, isolv,
     +                    traq, subfaces, smsg )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 08-12-1998 Copied from isddpu02
c
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Copy an intermediate representation of virtual unknowns for a certain
c     subface to the traq array of a certain block.
c     The intermediate representation is a one dimensional array which is
c     equal to the double precision data in the multi-block messages from
c     node to host.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, iface, ilev, isddpu04, isubface,
     +      maxnsubf, nblocks, ntraq1, nfaces, ni, nj, ntraq2,
     +      nvirtual, idiscret, isolv
      double precision smsg(*)
      double precision traq(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1:ntraq1, 1:ntraq2)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     iblock    i    Block sequence number
c     idiscret  i    Type of discretization
c     iface     i    Face number
c     ilev      i    Time level of the multi-block transport of information
c     isddpu04  o    number of virtual unknowns copied.
c     isolv     i    Called by solver for periodic b.c. isolv=1 else isolv=0
c     isubface  i    Subface number
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     nfaces    i    Number of faces
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     ntraq1    i    Dimensioning parameter of the TRAQ array.
c                    (ntraq1 = ndegfd for solut, ncoefs for coefs)
c     ntraq2    i    Dimensioning parameter of the TRAQ array.
c                    (ntraq2 = ntimlv for solut, 1 for coefs)
c     nvirtual  i    Number of rows of virtual cells
c     smsg      o    reserved
c     subfaces  i    reserved
c     traq     i/o   reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ibto, ifto, ihigh, ii, ilow, iorient, isfto, ismsg,
     +        iunktraq, jj, nsignmb, relblckor, tsignmb,
     +        irow, nrow
      double precision usign
      logical reverse

c     ibto      Target block
c     ifto      Target face
c     ihigh     Upper index of subface
c     ii        Loop variable
c     ilow      Lower index of subface
c     iorient   Relative orientation of subfaces, see itopol
c     irow      Loop variable for rows
c     isfto     Target subface
c     ismsg     Counter for smsg
c     iunktraq  Number of the unknown iunk(i) in traq
c     jj        Loop variable
c     nrow      Number of rows
c     nsignmb   Nsignmb = 0 if the normal direction of the subface
c               coincides with that of its neighbouring block, otherwise
c               nsignmb = 1
c     relblckor Relative block orientation. If the neighbouring
c               block has sqrt(g) with opposite sign to this block
c               then relblckor = 1 otherwise it is 0
c     reverse   Reverse the order of copying unknowns (true) or not (false)
c     tsignmb   If the tangential direction of the subface coincides with
c               that of its neighbouring block tsignmb = 0, otherwise
c               tsignmb = 1
c     usign     Sign to multiply the unknowns with when copying from traq
c               to the intermediate representation
c **********************************************************************
c
c                       I/O
c
c      none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     None
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     None
c **********************************************************************
c
c                       PSEUDO CODE
c
c        iunk(1) = normal fluxes on the block interfaces
c
c     ismsg = 1
c     FOR unknowns to be transported
c     DO
c        determine index range [i1,i2] X [j1,j2]
c
c        determine order and sign (usign) of copying
c
c        FOR I = I1, I2, ISTEP
c        DO
c           FOR J = J1, J2, JSTEP
c           DO
c              traq(I,J) = usign*smsg(ismsg)
c              ismsg = ismsg + 1
c           DONE
c        DONE
c     DONE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      ismsg = 1
      nrow = 1

      iunktraq = 2

      if ( iface.eq.ileft .or. iface.eq.iright ) then
         iunktraq = 3 - iunktraq

         if ( iunktraq.eq.2 ) then
            if ( idiscret.eq.2 .or. idiscret.eq.4 ) nrow = 2
         end if

      end if

c     --- in case of periodic b.c. this routine will be called in
c         the linear solver, we have then a restricted solution vector

      if ( iunktraq.gt.2 .and. isolv.eq.1) then
         iunktraq = 1
      end if

c     --- put the unknown iunk(i) into smsg
c          determine the index range

      ilow = subfaces(iblock, iface, isubface, 1)
      ihigh = subfaces(iblock, iface, isubface, 2)

c     --- relative orientation

      iorient    = subfaces(iblock,iface,isubface,8)
      tsignmb    = mod(iorient,2)
      nsignmb    = mod(iorient/2,2)
      relblckor  = iorient/4
      ibto       = subfaces(iblock,iface,isubface,5)
      ifto       = subfaces(iblock,iface,isubface,6)
      isfto      = subfaces(iblock,iface,isubface,7)

      reverse = .false.
      usign   = 1d0

      if ( (iblock*nfaces + iface)*maxnsubf + isubface .ge.
     +     (ibto*nfaces + ifto)*maxnsubf + isfto ) then

c     --- the representation is determined by the neighbouring block

         if ( relblckor.eq.1 ) usign = -usign

         if ( tsignmb.eq.1 ) then

            reverse = .true.

         end if

         if ( nsignmb.eq.1 ) then

            usign = -usign

         end if

      end if

      do irow = 1, nrow

         if ( iface.eq.ileft ) then

            ii = 2 -irow

            if ( reverse ) then

               do jj = ilow, ihigh
                  traq(ii,jj,iunktraq,ilev) = 0.5d0*
     +               (usign*smsg(ismsg+ihigh-jj)+
     +                traq(ii,jj,iunktraq,ilev))
                  smsg(ismsg+ihigh-jj) = usign*
     +                traq(ii,jj,iunktraq,ilev)
               end do

            else

               do jj = ilow, ihigh
                  traq(ii,jj,iunktraq,ilev) = 0.5d0*
     +               (usign*smsg(ismsg+jj-ilow)+
     +                traq(ii,jj,iunktraq,ilev))
                  smsg(ismsg+jj-ilow) = usign*
     +                traq(ii,jj,iunktraq,ilev)
               end do

            end if

            ismsg = ismsg + ihigh - ilow + 1

         else if ( iface.eq.iright ) then

            ii = ni + irow

            if ( reverse ) then

               do jj = ilow, ihigh
                  traq(ii,jj,iunktraq,ilev) = 0.5d0*
     +               (usign*smsg(ismsg+ihigh-jj)+
     +                traq(ii,jj,iunktraq,ilev))
                  smsg(ismsg+ihigh-jj) = usign*
     +                traq(ii,jj,iunktraq,ilev)
               end do

            else

               do jj = ilow, ihigh
                  traq(ii,jj,iunktraq,ilev) = 0.5d0*
     +               (usign*smsg(ismsg+jj-ilow)+
     +                 traq(ii,jj,iunktraq,ilev))
                  smsg(ismsg+jj-ilow)  = usign*
     +                 traq(ii,jj,iunktraq,ilev)
               end do

            end if

            ismsg = ismsg + ihigh - ilow + 1

         else if ( iface.eq.idown ) then

            jj = 2 - irow

            if ( reverse ) then

               do ii = ilow, ihigh
                  traq(ii,jj,iunktraq,ilev) = 0.5d0*
     +               (usign*smsg(ismsg + ihigh - ii)+
     +                traq(ii,jj,iunktraq,ilev))
                  smsg(ismsg + ihigh - ii)  = usign*
     +                traq(ii,jj,iunktraq,ilev)
               end do

            else

               do ii = ilow, ihigh
                  traq(ii,jj,iunktraq,ilev) = 0.5d0*
     +               (usign*smsg(ismsg + ii - ilow)+
     +                traq(ii,jj,iunktraq,ilev))
                  smsg(ismsg + ii - ilow)  = usign*
     +                traq(ii,jj,iunktraq,ilev)
               end do

            end if

            ismsg = ismsg + ihigh - ilow + 1

         else

            jj = nj + irow

            if ( reverse ) then

               do ii = ilow, ihigh
                  traq(ii,jj,iunktraq,ilev) = 0.5d0*
     +               (usign*smsg(ismsg + ihigh - ii)+
     +                traq(ii,jj,iunktraq,ilev))
                  smsg(ismsg + ihigh - ii)  = usign*
     +                traq(ii,jj,iunktraq,ilev)
               end do

            else

               do ii = ilow, ihigh
                  traq(ii,jj,iunktraq,ilev) = 0.5d0*
     +               (usign*smsg(ismsg + ii - ilow)+
     +                traq(ii,jj,iunktraq,ilev))
                  smsg(ismsg + ii - ilow)  = usign*
     +                traq(ii,jj,iunktraq,ilev)
               end do

            end if

            ismsg = ismsg + ihigh - ilow + 1

         end if

      end do

      isddpu04 = ismsg - 1

      end
