      subroutine isddrel ( itrans, ilev, isiinput, istraq, isitopol,
     +                     relvar )
c ======================================================================
c
c     programmer   Erik Brakkee
c     version      1.6  18-05-1998 Charles Bilinear discretization
c     version      1.5  27-02-1997 Kees K: extension 3D
c     version      1.4  11-12-1996 Kees K: idiscret
c     version      1.3  13-03-1996 Guus: New call to isddre02
c     version      1.2  22-11-1994 Erik: extended for coefs.
c                                      changed solut into traq.
c     version      1.1  24-198-1994  Erik: internal representation of virt
c                                      is not used anymore.
c     version      1.0  18-4-1994
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Copy all virtual unknowns of a certain type (itrans) at a certain
c     time level (ilev) to a single one-dimensional array of relevant variables
c     (relvar)
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ilev, isiinput(*), istraq(*), isitopol, itrans
      double precision relvar(*)

c     ilev     i  Time level of the multi-block transport of information
c     isiinput i  Array of sequence numbers (nblocks) of array iinput
c     isitopol i  Sequence number of array itopol
c     istraq   i  Array of sequence numbers (nblocks) of array traq
c     itrans   i  reserved
c     relvar   o  Array containing copied virtual unknowns
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, iblock, ipiinput, ipitopol, ipnsubf, ipsubf, iptraq,
     +        istrtrel, maxnsubf, nblocks, ndegfd, nfaces, ni,
     +        nj, nk, nrelvar, ntimlv, nundegfs, nvirtual, vlen,
     +        ipiindsc, idiscret, ndim

c     i         General loop variable
c     iblock    Block sequence number
c     idiscret  Type of discretization
c     ipiindsc  Starting address of array IINDSC in array IINPUT
c     ipiinput  Starting address of array iinput in ibuffr
c     ipitopol  Starting address of array itopol in ibuffr
c     ipnsubf   Starting address of the nsubfaces array
c     ipsubf    Starting address of the subfaces array
c     iptraq    Starting address of array traq in buffer
c     istrtrel  First free position in relvar
c     maxnsubf  Maximum number of subfaces encountered on all subfaces
c     nblocks   Number of blocks
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     nfaces    Number of faces
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     nrelvar   Total number of relevant variables
c     ntimlv    Number of time levels
c     nundegfs  Number of different types degrees of freedom to be gotten from
c               the traq arrays
c     nvirtual  Number of rows of virtual cells
c     vlen      Number of virtual tangential velocities needed from
c               neighbouring blocks
c **********************************************************************
c
c                       I/O
c
c      none
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isddre02, isdpgetp, isingetp

c     ISACTIVE  Activate old permanent array
c     ISDDRE02  Get all virtual unknowns of a certain type for a certain
c               block
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISINGETP  get the pointer corresponding to an integer array
c **********************************************************************
c
c                       ERROR MESSAGES
c
c        -
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c     do iblock = 1, nblocks
c
c       get all virtual unknowns of certain type itrans and certain
c       time level from traq using isddre02(...)
c
c       place in relvar array directly after the unknowns of the preceding
c       block
c
c     end do
c
c     Fill the unused position at the end of relvar with zeroes.
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      call iseropen('isddrel')

c     --- activate arrays

      call isactive(isitopol)
      ipitopol = isingetp(isitopol)
      nblocks  = ibuffr(ipitopol + 1 - 1)
      do iblock = 1, nblocks
         call isactive(isiinput(iblock))
         call isactive(istraq(iblock))
      end do

c     --- compute pointers

      ipitopol     = isingetp(isitopol)
      ipnsubf  = ibuffr(ipitopol+5-1) + ipitopol - 1
      ipsubf   = ibuffr(ipitopol+6-1) + ipitopol - 1

c     --- compute some general parameters

      nblocks      = ibuffr(ipitopol + 1 - 1)
      nfaces       = ibuffr(ipitopol + 2 - 1)
      maxnsubf     = ibuffr(ipitopol + 3 - 1)
      ipiinput     = isingetp(isiinput(1))
      ndegfd       = ibuffr(ipiinput + 55 - 1)
      ndim         = ibuffr(ipiinput + 57 - 1)
      ntimlv       = ibuffr(ipiinput + 64 - 1)
      nvirtual     = ibuffr(ipiinput + 54 - 1)
      ipiindsc     = ibuffr(ipiinput-1+6)
      idiscret     = ibuffr(ipiinput-1+ipiindsc+1)

c     --- compute nundegfs

      nundegfs = 1
      if ( itrans.eq.1 ) then
         if( ndim.eq.2) then
            if ( idiscret.eq.2 .or. idiscret.eq.4 ) then
               nundegfs = nundegfs + 2
            else
               nundegfs = nundegfs + 1
            end if
         else
            nundegfs = nundegfs + 2
         end if
      end if

c     --- main loop

      nrelvar = 0
      istrtrel = 1
      do iblock = 1, nblocks

         iblockrf = iblock
         ipiinput = isingetp(isiinput(iblock))
         ni       = ibuffr(ipiinput + 51 - 1)
         nj       = ibuffr(ipiinput + 52 - 1)
         nk       = ibuffr(ipiinput + 53 - 1)
         vlen     = ibuffr(ipiinput + 58 - 1)

         iptraq   = isdpgetp(istraq(iblock))

         nrelvar = nrelvar + vlen

         istrtrel = istrtrel +
     +      isddre02 ( iblock, itrans, ilev, ni, nj, nk, ndegfd, ntimlv,
     +                 nvirtual, nblocks, nfaces, maxnsubf, idiscret,
     +                 buffer(iptraq), ibuffr(ipnsubf), ibuffr(ipsubf),
     +                 relvar(istrtrel) )

      end do

c     --- fill the remaining positions of the array of relevant variables

      nrelvar = nundegfs*nrelvar

      do i = istrtrel, nrelvar
         relvar(i) = 0d0
      end do

      call iserclos('isddrel')

      end
