      function isddvlen(iblock, isitopol)
c ======================================================================
c
c        programmer   Erik Brakkee
c        version 1.1    date 23-04-1997 Kees K.: 3D
c        version 1.0    date 23-05-1994
c
c
c   copyright (c) 1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Compute and return vlen, the number of virtual tangential velocity
c     unknowns of a certain block.
c **********************************************************************
c
c                       KEYWORDS
c
c   mapping_array
c   topology
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, isddvlen, isitopol, nblocks

c     iblock       i   Block sequence number
c     isddvlen     o   Vlen parameter of block iblock
c     isitopol     i   Sequence number of array itopol
c     nblocks      i   Number of blocks
c     ndim         i    Dimension of space (2 or 3)
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ipitopol, ipnsubf, ipsubf, maxnsubf, nfaces, ndim

c     ipitopol  Starting address of array itopol in ibuffr
c     ipnsubf   Starting address of the nsubfaces array
c     ipsubf    Starting address of the subfaces array
c     maxnsubf  Maximum number of subfaces encountered on all subfaces
c     nfaces    Number of faces
c **********************************************************************
c
c                       I/O
c
c      None
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
       integer isddvl02, isingetp

c      ISACTIVE     Activate a permanent array
c      ISDDVL02     Does all the work for ISDDVLEN
c      ISINGETP     Get address of a permanent integer array
c **********************************************************************
c
c                       ERROR MESSAGES
c
c         None
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      call isactive(isitopol)
      ipitopol     = isingetp(isitopol)
      nblocks      = ibuffr(ipitopol + 1 - 1)
      nfaces       = ibuffr(ipitopol + 2 - 1)
      maxnsubf = ibuffr(ipitopol + 3 - 1)
      ipnsubf  = ibuffr(ipitopol+5-1) + ipitopol - 1
      ipsubf   = ibuffr(ipitopol+6-1) + ipitopol - 1

      if ( nfaces.eq.4 ) then
         ndim = 2
      else
         ndim = 3
      end if

      isddvlen = isddvl02 ( iblock, nblocks, nfaces, maxnsubf,
     +                      ibuffr(ipnsubf), ibuffr(ipsubf), ndim )

      end
