      subroutine isnetflo( isiinput, issolut, flout )
c ======================================================================
c
c     programmer  Ronald van Nooyen
c     (net outflow as in iscorr03 by Erik Brakkee)
c
c     version     1.1  date 09-06-1998 Charles: nvpoints changed to npointsv
c     version     1.0  ISNAS     date  05-09-1994
c
c     computer    HP - 9000/720
c
c     Copyright (c) 1994  "TU-Delft"
c
c     Permission to copy or distribute this software or documentation
c     in hard copy or soft copy granted only by written license
c     obtained from "TU-Delft".
c
c     All rights reserved. No part of this publication may be reproduced,
c     stored in a retrieval system ( e.g., in memory, disk, or core)
c     or be transmitted by any means, electronic, mechanical, photocopy,
c     recording, or otherwise, without written permission from the
c     publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c                  Calculate net mass outflow
c
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c
      implicit none
c
c                  NAMED CONSTANTS
c
      integer maxdim
      parameter (maxdim = 3)
c
c     maxdim: highest physical space dimension for which ISNaS can work
c
c                  INPUT / OUTPUT PARAMETERS
c
c
      integer isiinput, issolut
      double precision flout
c
c     flout     o    net outward flux for this block
c     isiinput  i    reserved
c     issolut   i    reserved
c
c **********************************************************************
c
c                  COMMON BLOCKS
c
      include 'IScommon/cisblank'
c
c **********************************************************************
c
c                  LOCAL PARAMETERS
c
      integer area(1:2,1:maxdim), areap(1:2,1:maxdim), ndegfd, ntimlv,
     +        varea(1:2,1:maxdim), auxdim, nvirtual, npointsv,
     +        ipnewflux, ndim, isingetp, isdpgetp, ipiinput,
     +        ipsolut
      double precision partflux
c
c     area           bounds of summation area
c     areap          bounds of pressure area
c     auxdim         loop counter for space dimension
c     ipiinput       starting address of array iinput in ibuffr
c     ipnewflux      starting address of new flux in buffer
c     ipsolut        starting address of array solut in buffer
c     ndim           reserved
c     npointsv       reserved
c     ntimlv         reserved
c     nvirtual       reserved
c     partflux       partial flux sum
c     varea          bounds of array with virtual points
c
c **********************************************************************
c
c                  SAVE
c
c                  ---
c
c **********************************************************************
c
c                  DATA
c
c                  ---
c
c **********************************************************************
c
c                  I/O
c
c                  ---
c
c **********************************************************************
c
c                  SUBROUTINES CALLED
c
c     ISASUM      sum over given area
c     ISERCLOS    removes name of subroutine from error list
c     ISEROPEN    adds name of subroutine to error list
c     ISDPGETP
c     ISINGETP
c
c **********************************************************************
c
c                  ERROR MESSAGES
c
c                  ---
c
c **********************************************************************
c
c                  PSEUDO CODE
c
c                  ---
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('isnetflo')

      call isactive(isiinput)
      call isactive(issolut)

      ipiinput = isingetp(isiinput)
      ipsolut  = isdpgetp(issolut )

      do 100 auxdim=1,maxdim

c     --- lower area bound is one

         areap(1,auxdim) = 1

c        --- upper area bound is ni, nj or max(1,nk)

         areap(2,auxdim) = max(1,ibuffr(ipiinput-1+50+auxdim))
         area(1,auxdim) = areap(1,auxdim)
         area(2,auxdim) = areap(2,auxdim)
 100  continue

      ndim     = ibuffr(ipiinput-1+57)
      ntimlv   = ibuffr(ipiinput-1+64)
      ndegfd   = ibuffr(ipiinput-1+55)
      nvirtual = ibuffr(ipiinput-1+54)

      varea(1,3) = 1
      varea(2,3) = 1
      do 200 auxdim=1,ndim
         varea(1,auxdim) = areap(1,auxdim)-nvirtual
         varea(2,auxdim) = areap(2,auxdim)+1+nvirtual
 200  continue

      npointsv = (varea(2,1)-varea(1,1)+1)*(varea(2,2)-varea(1,2)+1)
      if (ndim.eq.3) npointsv = npointsv*(varea(2,3)-varea(1,3)+1)

      ipnewflux = ipsolut+(ntimlv-1)*ndegfd*npointsv

      flout = 0d0

      do 300 auxdim=1,ndim

         area(2,auxdim) = areap(1,auxdim)
         call isasum(varea(1,1),varea(2,1),varea(1,2),varea(2,2),
     +               varea(1,3),varea(2,3),area,
     +               buffer(ipnewflux+(auxdim-1)*npointsv),
     +               partflux)
         area(2,auxdim) = areap(2,auxdim)

         flout = flout - partflux

         area(1,auxdim) = areap(2,auxdim)+1
         area(2,auxdim) = areap(2,auxdim)+1
         call isasum(varea(1,1),varea(2,1),varea(1,2),varea(2,2),
     +               varea(1,3),varea(2,3),area,
     +               buffer(ipnewflux+(auxdim-1)*npointsv),
     +               partflux)
         area(1,auxdim) = areap(1,auxdim)
         area(2,auxdim) = areap(2,auxdim)

         flout = flout + partflux

 300  continue

      call iserclos('isnetflo')

      end
