      function isredd02(iblock, itrans,  ilev,
     +                  ni, nj, nk,
     +                  ntraq1, ntraq2, nvirtual,
     +                  nblocks, nfaces, maxnsubf, idiscret,
     +                  traq,   nsubfaces, subfaces, relvar)
c ======================================================================
c
c        programmer     Erik Brakkee
c        version 1.2    date 21-10-1997 Kees K. call of isddput with isolv
c        version 1.1    date 12-12-1996 Kees K. idiscret added
c        version 1.0    date 07-04-1994
c
c
c   copyright (c) 1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     This routine copies a one-dimensional array of virtual unknowns of
c     certain type of a certain block to traq.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, ilev, isredd02, itrans,
     +      maxnsubf, idiscret,
     +      nblocks, ntraq1, nfaces, ni, nj, nk, ntraq2, nvirtual
      integer nsubfaces(1:nblocks, 1:nfaces)
      double precision relvar(*)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)
      double precision traq(*)

c     iblock    i    Block sequence number
c     idiscret  i    Type of discretization
c     ilev      i    Time level of the multi-block transport of information
c     isredd02  o    Number of unknowns copied
c     itrans    i    reserved
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     nfaces    i    Number of faces
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     ntraq1    i    Dimensioning parameter of the TRAQ array.
c                    (ntraq1 = ndegfd for solut, ncoefs for coefs)
c     ntraq2    i    Dimensioning parameter of the TRAQ array.
c                    (ntraq2 = ntimlv for solut, 1 for coefs)
c     nvirtual  i    Number of rows of virtual cells
c     relvar    i    Array containing copied virtual unknowns
c     subfaces  i    reserved
c     traq      i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iface, istrtrel, isubface, isolv

c     iface     Face number
c     isolv     Called by solver for periodic b.c. isolv=1 else isolv=0
c     istrtrel  First free position in relvar
c     isubface  Subface number
c **********************************************************************
c
c                       I/O
c
c      None
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isddput

c     ISDDPUT   Put virtual unknowns to the arrays of virtual unknowns
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      istrtrel = 1
      isolv = 0

      do iface = 1, nfaces
         do isubface = 1, nsubfaces(iblock,iface)
         if ( subfaces(iblock,iface,isubface,5).ne.0 ) then

            istrtrel = istrtrel +
     +         isddput(iblock, iface,  isubface, itrans,  ilev,
     +                 ni, nj, nk,
     +                 ntraq1, ntraq2, nvirtual,
     +                 nblocks, nfaces, maxnsubf, idiscret, isolv,
     +                 traq,   nsubfaces, subfaces,
     +                 relvar(istrtrel))

         end if
         end do
      end do

      isredd02 = istrtrel - 1

      end
