      subroutine RAPx (finemat, coarsmat, nxf, nyf, sl1, sl2, sl3, sl4)
c ======================================================================
c
c        programmer     Jos van Kan
c
c       version 1.2    15 05 97
c         new cell centered R & P
c         nxc = nxf/2 + 1 instead of (nxf+1)/2
c       version 1.1    10-04-1997
c         utilizes module mgpreset instead of common block cmgconst
c       version 1.0    27-03-1997
c
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c **********************************************************************
c
c         MODULES USED
c
      use mgpreset
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Calculate RAP matrix with semicoarsening in x-direction
c   centering shall be decided by the parity of nxf.
c **********************************************************************
c
c                       KEYWORDS
c
c   multi_grid
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer sl1, sl2, sl3, sl4
      integer nxf, nyf
      double precision
     +finemat(1-sl1:nxf+sl2,1-sl3:nyf+sl4,9),
     +coarsmat (nxf/2+1, nyf, 9)

c     coarsmat  i    Matrix on coarse level
c     finemat   i    Matrix on fine level
c     nxf       i    Number of grid x points on fine grid
c     nyf       i    Number of grid y points on fine grid
c     sl1       i    Slack variable, x lower bound
c     sl2       i    Slack variable, x upper bound
c     sl3       i    Slack variable, y lower bound
c     sl4       i    Slack variable, y upper bound
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ixf, iyf, ixc, nxc, centring
      double precision aa0, aa1

c     aa0       Interpolation weight in restriction/prolongation
c     aa1       Interpolation weight in restriction/prolongation
c     centring  0 = cellcentering, 1 = vertex centering
c     ixc       Counter to number of grid x points on coarse grid
c     ixf       Counter to number of grid x points on fine grid
c     iyf       Counter to number of grid y points on fine grid
c     nxc       Number of grid x points on coarse grid
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      nxc = nxf/2 + 1
      centring = mod (nxf, 2)
      if (centring.eq.CELLC) then

c     --- cell centered

      aa0 = 0.25d0
      aa1 = 0.75d0
      do iyf = 1, nyf

c     --- stencil left column

        coarsmat (1, iyf, p00) = 0d0
        coarsmat (1, iyf, p01) = 0d0
        coarsmat (1, iyf, p02) = 0d0

        do ixc = 2, nxc - 1
          ixf = 2 * ixc - 2
          coarsmat (ixc , iyf, p00) = aa1 * finemat (ixf , iyf, p00) +
     +       aa0 * (finemat (ixf, iyf, p10) + finemat (ixf+1, iyf, p00))
          coarsmat (ixc , iyf, p01) = aa1 * finemat (ixf , iyf, p01) +
     +       aa0 * (finemat (ixf, iyf, p11) + finemat (ixf+1, iyf, p01))
          coarsmat (ixc , iyf, p02) = aa1 * finemat (ixf , iyf, p02) +
     +       aa0 * (finemat (ixf, iyf, p12) + finemat (ixf+1, iyf, p02))
        end do
        
        coarsmat (nxc, iyf, p00) = aa1 * finemat (nxf, iyf, p00) +
     +       aa0 * finemat (nxf, iyf, p10)
        coarsmat (nxc, iyf, p01) = aa1 * finemat (nxf, iyf, p01) +
     +       aa0 * finemat (nxf, iyf, p11)
        coarsmat (nxc, iyf, p02) = aa1 * finemat (nxf, iyf, p02) +
     +       aa0 * finemat (nxf, iyf, p12)
c       --- end stencil left column

c           --- stencil center column

          coarsmat (1 , iyf, p10) = 
     +      + aa0 * finemat (1 , iyf, p20) + aa1 * finemat (1, iyf, p10)
          coarsmat (1 , iyf, p11) = 
     +      + aa0 * finemat (1 , iyf, p21) + aa1 * finemat (1, iyf, p11)
          coarsmat (1 , iyf, p12) = 
     +      + aa0 * finemat (1 , iyf, p22) + aa1 * finemat (1, iyf, p12)

        do ixc = 2, nxc-1
          ixf = 2 * ixc - 2
          coarsmat (ixc , iyf, p10) = aa0 * finemat (ixf , iyf, p00) +
     +       aa1 * (finemat (ixf, iyf, p10) + finemat (ixf+1, iyf, p00))
     +     + aa0 * finemat (ixf+1 , iyf, p20) +
     +       aa1 * (finemat (ixf, iyf, p20) + finemat (ixf+1, iyf, p10))
          coarsmat (ixc , iyf, p11) = aa0 * finemat (ixf , iyf, p01) +
     +       aa1 * (finemat (ixf, iyf, p11) + finemat (ixf+1, iyf, p01))
     +     + aa0 * finemat (ixf+1 , iyf, p21) +
     +       aa1 * (finemat (ixf, iyf, p21) + finemat (ixf+1, iyf, p11))
          coarsmat (ixc , iyf, p12) = aa0 * finemat (ixf , iyf, p02) +
     +       aa1 * (finemat (ixf, iyf, p12) + finemat (ixf+1, iyf, p02))
     +     + aa0 * finemat (ixf+1 , iyf, p22) +
     +       aa1 * (finemat (ixf, iyf, p22) + finemat (ixf+1, iyf, p12))
        end do

          coarsmat (nxc , iyf, p10) = aa0 * finemat (nxf , iyf, p00) +
     +       aa1 * finemat (nxf, iyf, p10) 
          coarsmat (nxc , iyf, p11) = aa0 * finemat (nxf , iyf, p01) +
     +       aa1 * finemat (nxf, iyf, p11) 
          coarsmat (nxc , iyf, p12) = aa0 * finemat (nxf , iyf, p02) +
     +       aa1 * finemat (nxf, iyf, p12) 

c       --- end stencil center column

c           --- stencil right column

          coarsmat (1, iyf, p20) = aa1 * finemat (1, iyf, p20) +
     +       aa0 * finemat (1, iyf, p10)
          coarsmat (1, iyf, p21) = aa1 * finemat (1, iyf, p21) +
     +       aa0 * finemat (1, iyf, p11)
          coarsmat (1, iyf, p22) = aa1 * finemat (1, iyf, p22) +
     +       aa0 * finemat (1, iyf, p12)

        do ixc = 2, nxc - 1
          ixf = 2 * ixc - 2
          coarsmat (ixc , iyf, p20) = aa1 * finemat (ixf+1 , iyf, p20) +
     +       aa0 * (finemat (ixf, iyf, p20) + finemat (ixf+1, iyf, p10))
          coarsmat (ixc , iyf, p21) = aa1 * finemat (ixf+1 , iyf, p21) +
     +       aa0 * (finemat (ixf, iyf, p21) + finemat (ixf+1, iyf, p11))
          coarsmat (ixc , iyf, p22) = aa1 * finemat (ixf+1 , iyf, p22) +
     +       aa0 * (finemat (ixf, iyf, p22) + finemat (ixf+1, iyf, p12))
        end do
        coarsmat (nxc, iyf, p20) = 0d0
        coarsmat (nxc, iyf, p21) = 0d0
        coarsmat (nxc, iyf, p22) = 0d0

c       --- end stencil right column

      end do

c     --- end case cell centered

      else

c     --- begin case vertex centered

      aa0 = 0.25d0
      aa1 = 0.5d0

c     --- first all lines except highest and lowest
c         --- begin inner lines

      do iyf = 2, nyf - 1

c--- stencil center row

c     --- region left boundary

        coarsmat (1, iyf, p01) = 0d0
        coarsmat (1, iyf, p21) = aa1 * (finemat (1 + 1, iyf, p21)
     +   + finemat (1, iyf, p21))
     +   + aa0*finemat (1+1, iyf, p11)
        coarsmat (1, iyf, p11) = finemat (1, iyf, p11) +
     +   aa1*(finemat (1, iyf, p21) + finemat (1+1, iyf, p01))+
     +   aa0*finemat (1+1, iyf, p11)
        do ixc = 2, nxc-1
          ixf = 2*ixc - 1
          coarsmat (ixc, iyf, p01) = aa1 * (finemat (ixf - 1, iyf, p01)
     +     + finemat (ixf, iyf, p01)) +
     +      aa0*finemat (ixf-1, iyf, p11)
          coarsmat (ixc, iyf, p21) = aa1 * (finemat (ixf + 1, iyf, p21)
     +     + finemat (ixf, iyf, p21)) +
     +      aa0*finemat (ixf+1, iyf, p11)
          coarsmat (ixc, iyf, p11) = finemat (ixf, iyf, p11) +
     +     aa1*(finemat (ixf, iyf, p01) + finemat (ixf-1, iyf, p21))+
     +     aa1*(finemat (ixf, iyf, p21) + finemat (ixf+1, iyf, p01))+
     +     aa0*finemat (ixf-1, iyf, p11) +
     +     aa0*finemat (ixf+1, iyf, p11)
        end do

c       --- region right boundary

        nxf = 2*nxc - 1
        coarsmat (nxc, iyf, p01) = aa1 * (finemat (nxf - 1, iyf, p01)
     +   + finemat (nxf, iyf, p01)) +
     +    aa0*finemat (nxf-1, iyf, p11)
        coarsmat (nxc, iyf, p21) = 0d0
        coarsmat (nxc, iyf, p11) = finemat (nxf, iyf, p11) +
     +   aa1*(finemat (nxf, iyf, p01) + finemat (nxf-1, iyf, p21))+
     +   aa0*finemat (nxf-1, iyf, p11)

c       --- end stencil center row

c           --- stencil upper row
c           --- region left boundary

        coarsmat (1, iyf, p02) = 0d0
        coarsmat (1, iyf, p22) = aa1 * (finemat (1 + 1, iyf, p22)
     +   + finemat (1, iyf, p22)) +
     +    aa0*finemat (1+1, iyf, p12)
        coarsmat (1, iyf, p12) = finemat (1, iyf, p12) +
     +   aa1*(finemat (1, iyf, p22) + finemat (1+1, iyf, p02))+
     +   aa0*finemat (1+1, iyf, p12)

c       --- region inner points

        do ixc = 2, nxc-1
          ixf = 2*ixc - 1
          coarsmat (ixc, iyf, p02) = aa1 * (finemat (ixf - 1, iyf, p02)
     +    + finemat (ixf, iyf, p02)) +
     +     aa0*finemat (ixf-1, iyf, p12)
          coarsmat (ixc, iyf, p22) = aa1 * (finemat (ixf + 1, iyf, p22)
     +    + finemat (ixf, iyf, p22)) +
     +     aa0*finemat (ixf+1, iyf, p12)
          coarsmat (ixc, iyf, p12) = finemat (ixf, iyf, p12) +
     +    aa1*(finemat (ixf, iyf, p02) + finemat (ixf-1, iyf, p22))+
     +    aa1*(finemat (ixf, iyf, p22) + finemat (ixf+1, iyf, p02))+
     +    aa0*finemat (ixf-1, iyf, p12) +
     +    aa0*finemat (ixf+1, iyf, p12)
        end do

c       --- region right boundary

        nxf = 2*nxc - 1
        coarsmat (nxc, iyf, p02) = aa1 * (finemat (nxf - 1, iyf, p02)
     +   + finemat (nxf, iyf, p02)) +
     +    aa0*finemat (nxf-1, iyf, p12)
        coarsmat (nxc, iyf, p22) = 0d0
        coarsmat (nxc, iyf, p12) = finemat (nxf, iyf, p12) +
     +   aa1*(finemat (nxf, iyf, p02) + finemat (nxf-1, iyf, p22))+
     +   aa0*finemat (nxf-1, iyf, p12)

c       --- end stencil upper row

c           --- stencil lower row
c           --- region left boundary

        coarsmat (1, iyf, p00) = 0d0
        coarsmat (1, iyf, p20) = aa1 * (finemat (1 + 1, iyf, p20)
     +   + finemat (1, iyf, p20)) +
     +   aa0*finemat (1+1, iyf, p10)
        coarsmat (1, iyf, p10) = finemat (1, iyf, p10) +
     +   aa1*(finemat (1, iyf, p20) + finemat (1+1, iyf, p00))+
     +   aa0*finemat (1+1, iyf, p10)

c       --- region inner points

        do ixc = 2, nxc-1
          ixf = 2*ixc - 1
          coarsmat (ixc, iyf, p00) = aa1 * (finemat (ixf - 1, iyf, p00)
     +     + finemat (ixf, iyf, p00)) +
     +     aa0*finemat (ixf-1, iyf, p10)
          coarsmat (ixc, iyf, p20) = aa1 * (finemat (ixf + 1, iyf, p20)
     +     + finemat (ixf, iyf, p20)) +
     +     aa0*finemat (ixf+1, iyf, p10)
          coarsmat (ixc, iyf, p10) = finemat (ixf, iyf, p10) +
     +     aa1*(finemat (ixf, iyf, p00) + finemat (ixf-1, iyf, p20))+
     +     aa1*(finemat (ixf, iyf, p20) + finemat (ixf+1, iyf, p00))+
     +     aa0*finemat (ixf-1, iyf, p10) +
     +     aa0*finemat (ixf+1, iyf, p10)
        end do

c       --- region right boundary

        nxf = 2*nxc - 1
        coarsmat (nxc, iyf, p00) = aa1 * (finemat (nxf - 1, iyf, p00)
     +   + finemat (nxf, iyf, p00)) +
     +   aa0*finemat (nxf-1, iyf, p10)
        coarsmat (nxc, iyf, p20) = 0d0
        coarsmat (nxc, iyf, p10) = finemat (nxf, iyf, p10) +
     +   aa1*(finemat (nxf, iyf, p00) + finemat (nxf-1, iyf, p20))+
     +   aa0*finemat (nxf-1, iyf, p10)

c--- end stencil lower row
      end do

c--- end inner lines

c--- and now for two loose ends: the low and high region boundaries

c     --- begin region lower boundary
c          cm (ixc, 1, pi0) = 0
c         --- stencil center row
c         --- lower left corner

        coarsmat (1, 1, p01) = 0d0
        coarsmat (1, 1, p21) = aa1 * (finemat (1 + 1, 1, p21)
     +   + finemat (1, 1, p21)) +
     +   aa0*finemat (1+1, 1, p11)
        coarsmat (1, 1, p11) = finemat (1, 1, p11) +
     +   aa1*(finemat (1, 1, p21) + finemat (1+1, 1, p01))+
     +   aa0*finemat (1+1, 1, p11)

c       --- region lower boundary internal points

        do ixc = 2, nxc-1
          ixf = 2*ixc - 1
          coarsmat (ixc, 1, p01) = aa1 * (finemat (ixf - 1, 1, p01)
     +     + finemat (ixf, 1, p01)) +
     +     aa0*finemat (ixf-1, 1, p11)
          coarsmat (ixc, 1, p21) = aa1 * (finemat (ixf + 1, 1, p21)
     +     + finemat (ixf, 1, p21)) +
     +     aa0*finemat (ixf+1, 1, p11)
          coarsmat (ixc, 1, p11) = finemat (ixf, 1, p11) +
     +     aa1*(finemat (ixf, 1, p01) + finemat (ixf-1, 1, p21))+
     +     aa1*(finemat (ixf, 1, p21) + finemat (ixf+1, 1, p01))+
     +     aa0*finemat (ixf-1, 1, p11) +
     +     aa0*finemat (ixf+1, 1, p11)
        end do

c--- lower right hand corner
        nxf = 2*nxc - 1
        coarsmat (nxc, 1, p01) = aa1 * (finemat (nxf - 1, 1, p01)
     +   + finemat (nxf, 1, p01)) +
     +   aa0*finemat (nxf-1, 1, p11)
        coarsmat (nxc, 1, p21) = 0d0
        coarsmat (nxc, 1, p11) = finemat (nxf, 1, p11) +
     +   aa1*(finemat (nxf, 1, p01) + finemat (nxf-1, 1, p21))+
     +   aa0*finemat (nxf-1, 1, p11)

c       --- end stencil center row

c           --- stencil upper row
c           --- lower left hand corner

        coarsmat (1, 1, p02) = 0d0
        coarsmat (1, 1, p22) = aa1 * (finemat (1 + 1, 1, p22)
     +   + finemat (1, 1, p22)) +
     +   aa0*finemat (1+1, 1, p12)
        coarsmat (1, 1, p12) = finemat (1, 1, p12) +
     +   aa1*(finemat (1, 1, p22) + finemat (1+1, 1, p02))+
     +   aa0*finemat (1+1, 1, p12)

c       --- region lower boundary internal points

        do ixc = 2, nxc-1
          ixf = 2*ixc - 1
          coarsmat (ixc, 1, p02) = aa1 * (finemat (ixf - 1, 1, p02)
     +     + finemat (ixf, 1, p02)) +
     +     aa0*finemat (ixf-1, 1, p12)
          coarsmat (ixc, 1, p22) = aa1 * (finemat (ixf + 1, 1, p22)
     +     + finemat (ixf, 1, p22)) +
     +     aa0*finemat (ixf+1, 1, p12)
          coarsmat (ixc, 1, p12) = finemat (ixf, 1, p12) +
     +     aa1*(finemat (ixf, 1, p02) + finemat (ixf-1, 1, p22))+
     +     aa1*(finemat (ixf, 1, p22) + finemat (ixf+1, 1, p02))+
     +     aa0*finemat (ixf-1, 1, p12) +
     +     aa0*finemat (ixf+1, 1, p12)
        end do

c       --- lower right hand corner

        nxf = 2*nxc - 1
        coarsmat (nxc, 1, p02) = aa1 * (finemat (nxf - 1, 1, p02)
     +   + finemat (nxf, 1, p02)) +
     +   aa0*finemat (nxf-1, 1, p12)
        coarsmat (nxc, 1, p22) = 0d0
        coarsmat (nxc, 1, p12) = finemat (nxf, 1, p12) +
     +   aa1*(finemat (nxf, 1, p02) + finemat (nxf-1, 1, p22))+
     +   aa0*finemat (nxf-1, 1, p12)

c       --- end of stencil upper row

c           --- stencil lower row that's easy.

        do ixc = 1, nxc
          coarsmat (ixc, 1, p00) = 0d0
          coarsmat (ixc, 1, p20) = 0d0
          coarsmat (ixc, 1, p10) = 0d0
        end do

c       --- end of region lower boundary

c           --- begin region upper boundary
c           cm (ixc, nyf, pi2) = 0d0
c           --- begin stencil center row
c           --- upper left hand corner

        coarsmat (1, nyf, p01) = 0d0
        coarsmat (1, nyf, p21) = aa1 * (finemat (1 + 1, nyf, p21)
     +   + finemat (1, nyf, p21)) +
     +   aa0*finemat (1+1, nyf, p11)
        coarsmat (1, nyf, p11) = finemat (1, nyf, p11) +
     +   aa1*(finemat (1, nyf, p21) + finemat (1+1, nyf, p01))+
     +   aa0*finemat (1+1, nyf, p11)

c       --- region upper boundary, internal points

        do ixc = 2, nxc-1
          ixf = 2*ixc - 1
          coarsmat (ixc, nyf, p01) = aa1 * (finemat (ixf - 1, nyf, p01)
     +     + finemat (ixf, nyf, p01)) +
     +     aa0*finemat (ixf-1, nyf, p11)
          coarsmat (ixc, nyf, p21) = aa1 * (finemat (ixf + 1, nyf, p21)
     +     + finemat (ixf, nyf, p21)) +
     +     aa0*finemat (ixf+1, nyf, p11)
          coarsmat (ixc, nyf, p11) = finemat (ixf, nyf, p11) +
     +     aa1*(finemat (ixf, nyf, p01) + finemat (ixf-1, nyf, p21))+
     +     aa1*(finemat (ixf, nyf, p21) + finemat (ixf+1, nyf, p01))+
     +     aa0*finemat (ixf-1, nyf, p11) +
     +     aa0*finemat (ixf+1, nyf, p11)
        end do

c       --- upper right hand corner

        nxf = 2*nxc - 1
        coarsmat (nxc, nyf, p01) = aa1 * (finemat (nxf - 1, nyf, p01)
     +   + finemat (nxf, nyf, p01)) +
     +   aa0*finemat (nxf-1, nyf, p11)
        coarsmat (nxc, nyf, p21) = 0d0
        coarsmat (nxc, nyf, p11) = finemat (nxf, nyf, p11) +
     +   aa1*(finemat (nxf, nyf, p01) + finemat (nxf-1, nyf, p21))+
     +   aa0*finemat (nxf-1, nyf, p11)

c       --- end stencil center row

c           --- begin stencil upper row
c            thats easy

        do ixc = 1, nxc
          coarsmat (ixc, nyf, p02) = 0d0
          coarsmat (ixc, nyf, p22) = 0d0
          coarsmat (ixc, nyf, p12) = 0d0
        end do

c       --- end stencil upper row

c           --- begin stencil lower row
c           --- upper left hand corner

        coarsmat (1, nyf, p00) = 0d0
        coarsmat (1, nyf, p20) = aa1 * (finemat (1 + 1, nyf, p20)
     +   + finemat (1, nyf, p20)) +
     +   aa0*finemat (1+1, nyf, p10)
        coarsmat (1, nyf, p10) = finemat (1, nyf, p10) +
     +   aa1*(finemat (1, nyf, p20) + finemat (1+1, nyf, p00))+
     +   aa0*finemat (1+1, nyf, p10)

c       --- region upper boundary, internal points

        do ixc = 2, nxc-1
          ixf = 2*ixc - 1
          coarsmat (ixc, nyf, p00) = aa1 * (finemat (ixf - 1, nyf, p00)
     +     + finemat (ixf, nyf, p00)) +
     +     aa0*finemat (ixf-1, nyf, p10)
          coarsmat (ixc, nyf, p20) = aa1 * (finemat (ixf + 1, nyf, p20)
     +     + finemat (ixf, nyf, p20)) +
     +     aa0*finemat (ixf+1, nyf, p10)
          coarsmat (ixc, nyf, p10) = finemat (ixf, nyf, p10) +
     +     aa1*(finemat (ixf, nyf, p00) + finemat (ixf-1, nyf, p20))+
     +     aa1*(finemat (ixf, nyf, p20) + finemat (ixf+1, nyf, p00))+
     +     aa0*finemat (ixf-1, nyf, p10) +
     +     aa0*finemat (ixf+1, nyf, p10)
        end do

c       --- upper right hand corner

        nxf = 2*nxc - 1
        coarsmat (nxc, nyf, p00) = aa1 * (finemat (nxf - 1, nyf, p00)
     +   + finemat (nxf, nyf, p00)) +
     +   aa0*finemat (nxf-1, nyf, p10)
        coarsmat (nxc, nyf, p20) = 0d0
        coarsmat (nxc, nyf, p10) = finemat (nxf, nyf, p10) +
     +   aa1*(finemat (nxf, nyf, p00) + finemat (nxf-1, nyf, p20))+
     +   aa0*finemat (nxf-1, nyf, p10)

c       --- end stencil lower row
c           --- end region upper boundary
c           --- end Case vertex centered

      end if
      end
