      subroutine RAPy1P3 ( fmcoef0, fmcoef1, fmcoef2,
     +    cmcoef0, cmcoef1, cmcoef2, nxf, nyf)
c ======================================================================
c
c        programmer     Jos van Kan
c        version 1.2    date   01-07-1997
c           correction of inter grid operators cell centered
c        version 1.1    date   04-06-1997
c           isnazification of all variables and subroutine calls
c           all truncated to max 8 characters.
c        version 1.0    date   11-03-1997
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c   pilotcode 3d mgpackage
c
c **********************************************************************
c
c                       MODULES USED
c
      use mgpreset
c **********************************************************************
c
c                       DESCRIPTION
c
c   Calculate RAP matrix with semicoarsening in y-direction
c   centring shall be decided by the parity of nyf.
c   only one plane at a time, and only ony stencil line
c   saving bufferspace.
c   This subroutine must be called 9 times:
c     with arguments (cm is scratch space)
c   fm (1, 1, 0), fm (1, 1, 3), fm (1, 1, 6),
c   cm (1, 1, p000), cm (1, 1, p010), cm (1, 1, p020),
c   fm (1, 1, 1), fm (1, 1, 4), fm (1, 1, 7),
c   cm (1, 1, p100), cm (1, 1, p110), cm (1, 1, p120),
c     .
c     .
c     .
c     .
c   fm (1, 1, 2), fm (1, 1, 5), fm (1, 1, 8),
c   cm (1, 1, p202), cm (1, 1, p212), cm (1, 1, p222),
c **********************************************************************
c
c                       KEYWORDS
c
c    multi_grid
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nxf, nyf
      double precision
     +fmcoef0 (nxf,nyf), fmcoef1 (nxf,nyf), fmcoef2 (nxf,nyf),
     +cmcoef0 (nxf, nyf/2+1), cmcoef1 (nxf, nyf/2+1),
     +cmcoef2 (nxf, nyf/2+1)

c     cmcoef0   o    Auxiliary array containing coefficients of coarse matrix
c     cmcoef1   o    Auxiliary array containing coefficients of coarse matrix
c     cmcoef2   o    Auxiliary array containing coefficients of coarse matrix
c     fmcoef0   i    Auxiliary array containing coefficients of fine matrix
c     fmcoef1   i    Auxiliary array containing coefficients of fine matrix
c     fmcoef2   i    Auxiliary array containing coefficients of fine matrix
c     nxf       i    Number of grid x points on fine grid
c     nyf       i    Number of grid y points on fine grid
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ixf, iyf, nyc, iyc, centring
      double precision aa0, aa1

c     aa0       Interpolation weight in restriction/prolongation
c     aa1       Interpolation weight in restriction/prolongation
c     cellc     Cell centered (module mgpreset)
c     centring  0 = cellcentering, 1 = vertex centering
c     ixf       Counter to number of grid x points on fine grid
c     iyc       Counter to number of grid y points on coarse grid
c     iyf       Counter to number of grid y points on fine grid
c     nyc       Number of grid y points on coarse grid
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      nyc = nyf/2 + 1
      centring = mod (nyf, 2)
      if (centring.eq.CELLC) then

c     --- cell centered

      aa0 = 0.25d0
      aa1 = 0.75d0

c     --- stencil lower coefficient

      do ixf = 1, nxf
        cmcoef0 (ixf, 1) = 0d0
      end do

      do iyc = 2, nyc - 1
        iyf = 2 * iyc - 2
        do ixf = 1, nxf
          cmcoef0 (ixf ,iyc) = aa1*fmcoef0 (ixf ,iyf) +
     +    aa0*(fmcoef1 (ixf,iyf) + fmcoef0 (ixf,iyf+1))
        end do
      end do

      do ixf = 1, nxf
        cmcoef0 (ixf ,nyc) = aa1*fmcoef0 (ixf ,nyf) +
     +  aa0*fmcoef1 (ixf,nyf)
      end do

c     --- end stencil lower coefficient

c         --- stencil center coefficient

      do ixf = 1, nxf
        cmcoef1 (ixf ,1) =
     +                aa1*fmcoef0 (ixf,1)
     +                + aa0*fmcoef2 (ixf,1)
     +                + aa1*fmcoef1 (ixf,1)
      end do
      do iyc = 2, nyc-1
        iyf = 2 * iyc - 2
        do ixf = 1, nxf
          cmcoef1 (ixf ,iyc) = aa0*fmcoef0 (ixf ,iyf) +
     +                  aa1*(fmcoef1 (ixf,iyf) + fmcoef0 (ixf,iyf+1))
     +                       + aa0*fmcoef2 (ixf,iyf+1) +
     +                  aa1*(fmcoef2 (ixf,iyf) + fmcoef1 (ixf,iyf+1))
        end do
      end do
      do ixf = 1, nxf
        cmcoef1 (ixf ,nyc) = aa0*fmcoef0 (ixf ,nyf) +
     +                aa1*fmcoef1 (ixf,nyf)
     +                +aa1*fmcoef2 (ixf,nyf)
      end do

c     --- end stencil center coefficient

c         --- stencil upper coefficient

      do ixf = 1, nxf
        cmcoef2 (ixf,1) = aa1*fmcoef2 (ixf,1) +
     +               aa0*fmcoef1 (ixf,1)
      end do
      do iyc = 2, nyc - 1
        iyf = 2 * iyc - 2
        do ixf = 1, nxf
          cmcoef2 (ixf,iyc) = aa1*fmcoef2 (ixf,iyf+1) +
     +                 aa0*(fmcoef2 (ixf,iyf) + fmcoef1 (ixf,iyf+1))
        end do
      end do
      do ixf = 1, nxf
        cmcoef2 (ixf, nyc) = 0d0
      end do

c     --- end stencil upper coefficient

c         --- end case cell centered

      else

c     --- begin case vertex centered

      aa0 = 0.25d0
      aa1 = 0.5d0

      do ixf = 1, nxf

c     --- region lower boundary

        cmcoef0 (ixf, 1) = 0d0
        cmcoef1 (ixf, 1) = fmcoef1 (ixf,1) +
     +             aa1*(fmcoef2 (ixf,1) +fmcoef0 (ixf,2))+
     +             aa0*fmcoef1 (ixf,2)
        cmcoef2 (ixf, 1) = aa1* (fmcoef2 (ixf,2) +fmcoef2 (ixf,2)) +
     +             aa0*fmcoef1 (ixf,2)
      end do

c--- region center

      do iyc = 2,nyc-1
        iyf = 2*iyc - 1
        do ixf = 1, nxf
          cmcoef0 (ixf,iyc)= aa1*(fmcoef0 (ixf,iyf-1)
     +           +fmcoef0 (ixf,iyf)) + aa0*fmcoef1 (ixf,iyf-1)
          cmcoef1 (ixf,iyc) = fmcoef1 (ixf,iyf) +
     +            aa1*(fmcoef0 (ixf,iyf) +fmcoef2 (ixf,iyf-1))+
     +            aa1*(fmcoef2 (ixf,iyf) +fmcoef0 (ixf,iyf+1))+
     +            aa0*fmcoef1 (ixf,iyf-1) + aa0*fmcoef1 (ixf,iyf+1)
          cmcoef2 (ixf,iyc) = aa1* (fmcoef2 (ixf,iyf+1)
     +            +fmcoef2 (ixf,iyf)) +aa0*fmcoef1 (ixf,iyf+1)
        end do
      end do

c     --- region upper boundary

      nyf = 2*nyc - 1
      do ixf = 1, nxf
        cmcoef0(ixf, nyc)=aa1*(fmcoef0(ixf,nyf-1)
     +         +fmcoef0(ixf,nyf))+ aa0*fmcoef1(ixf,nyf-1)
        cmcoef1(ixf, nyc)=fmcoef1(ixf,nyf)+
     +         aa1*(fmcoef0(ixf,nyf)+fmcoef2(ixf,nyf-1))+
     +         aa0*fmcoef1(ixf,nyf-1)
        cmcoef2(ixf, nyc)=0d0
      end do

c--- end case vertex centered
      end if

      end
