      subroutine autosize ( nx, ny, acorsnin, ahilevel,
     +        OUTRBUF,
     +        sl1, sl2, sl3, sl4)
c ======================================================================
c
c        programmer     Jos van Kan
c
c       version 1.1    10-04-1997
c         utilizes module mgpreset instead of common block cmgconst
c       version 1.0    27-03-1997
c
c
c   copyright (c) 1996
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c  autosize calculates the required dimension of rbuf to accomodate all
c  coarse grid info and scratch space.
c  Output in OUTRBUF. This should (at least) be the declared
c  dimension of rbuf in the calling program to the mg - package.
c **********************************************************************
c
c                       KEYWORDS
c
c   multi_grid
c **********************************************************************
c
c         MODULES USED
c
      use mgpreset
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer OUTRBUF
      integer nx, ny, acorsnin, ahilevel,
     +       sl1, sl2, sl3, sl4

c     acorsnin  i    0=full coarsening 1=semicoarsening  (formal parameter)
c     ahilevel  i    Maximum level of coarse grids (level 0 is finest)
c                    (formal parameter)
c     nx        i    Number of grid x points on finest grid
c     ny        i    Number of grid y points on finest grid
c     outrbuf   o    calculated size for rbuf
c     sl1       i    Slack variable, x lower bound
c     sl2       i    Slack variable, x upper bound
c     sl3       i    Slack variable, y lower bound
c     sl4       i    Slack variable, y upper bound
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ibuf (300)

c     ibuf      Integer array of dimension 300 that contains the
c               administration of this particular multi grid block.
c **********************************************************************
c
c
c     ERRMSG    Reports and counts errors (MG)
c     MKLVLINF  Reserves space for all matrices and arrays (MG)
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ERRMSG    Reports and counts errors (MG)
c     MKLVLINF  Reserves space for all matrices and arrays (MG)
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- sanity check

      if (ahilevel.gt.MAXLEVEL) then
         call errmsg ( "autosize", "hilevel exceeds MAXLEVEL", .FALSE.,
     +    ibuf)
      end if
      ibuf (MAXRBUFi) = 64000000
      ibuf (offrbufi) = 1
      ibuf (offscri) = ibuf (MAXRBUFi) + 1
      ibuf (hileveli) = ahilevel
      ibuf (corsnini) = acorsnin
      ibuf (sizexi) = nx
      ibuf (sizeyi) = ny
      ibuf (slacki) = sl1
      ibuf (slacki + 1) = sl2
      ibuf (slacki + 2) = sl3
      ibuf (slacki + 3) = sl4
      call mklvlinf (ibuf)
      OUTRBUF = ibuf (offrbufi)
      if (ibuf (corsnini).eq.FULLCORS) then
         OUTRBUF = OUTRBUF + ibuf (sizexi + 1) * ny * 9
      else
         OUTRBUF = OUTRBUF + (nx + sl1 + sl2)*
     +   (ny + sl3 + sl4)
      end if
      end
