      subroutine makematv ( matveci, origin, stncilsz, ibuf)
c ======================================================================
c
c        programmer     Jos van Kan
c
c       version 1.1    10-04-1997
c         utilizes module mgpreset instead of common block cmgconst
c       version 1.0    27-03-1997
c
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Reserves bufferspace and creates pointers to a matrix or
c   vector on all levels.
c   If space must be also reserved on the finest level then origin is 0
c   and in this case the same slack space is generated as the original
c   input quantities had. Otherwise origin = 1.
c   On the coarser level there is no slack space.
c   For a matrix, stencil size is 9, for a diagonal block (like the
c   matrices for line jacobi) stencil size is 3 and for a vector
c   stencil size is 1.
c **********************************************************************
c
c                       KEYWORDS
c
c   multi_grid
c **********************************************************************
c
c         MODULES USED
c
      use mgpreset
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ibuf (*)
      integer origin, matveci,  stncilsz

c     ibuf      i    Integer array of dimension 300 that contains the
c                    administration of this particular multi grid block.
c     matveci   i    Indices of a matrix/vector in ibuf
c     origin    i    0 or 1 depending on whether this quantity is known
c                    on the finest grid or not.
c     stncilsz  i    9 for matrix, 3 for LUdec, 1 for vector
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ilevel

c     ilevel    Levelcounter (multi grid levels)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     GETRMEM   Reserves space in real buffer and returns pointer (MG)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- the following call reserves space in the buffer
c         and returns a pointer

      if (origin.eq.0) then

c     --- on finest level the same slack space is reserved as in the
c         input quantities mat, sol and rhs
c         also for LUx and LUy, just to be consistent

        call getrmem ( ibuf (matveci),
     +   stncilsz * (ibuf (sizexi) + ibuf (slacki) + ibuf (slacki+1))
     +   * (ibuf (sizeyi) + ibuf (slacki+2) + ibuf (slacki+3)), ibuf)
      end if

      do ilevel = 1, ibuf (hileveli)
        call getrmem ( ibuf (matveci + ilevel),
     +    stncilsz * ibuf (sizexi + ilevel) *
     +    ibuf (sizeyi + ilevel), ibuf)
      end do
      end
