      subroutine iscpper2 ( iinput, iinbc, output, ni, nj, iblock )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 28-01-1999
c
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    This subroutine is only used in case of periodical boundary conditions
c    The values on corresponding faces on one block are averaged and both values
c    are replaced by the average value
c **********************************************************************
c
c                       KEYWORDS
c
c     periodic
c     average
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinput(*), iinbc(*), ni, nj, iblock
      double precision output(ni+1,nj+1)

c     iblock    i    Block sequence number
c     iinbc     i    reserved
c     iinput    i    reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     output   i/o   Cartesian components of the flow field in the vertices
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, i1, i2, iblocknb, icoupled, iface, ifacenb,
     +        ipstart, isubfacnb, j, j1, jpstart, jshift, k, ndegfd,
     +        ndegfdlc, nseg, btype

c     btype     Type of boundary condition
c     i         General loop variable
c     i1        Lower index number along first direction of a face or edge
c     i2        Upper index number along first direction of a face or edge
c     iblocknb  Block number of neighbour
c     icoupled  If 0 the momentum equations are coupled, if 1 they are not
c     iface     Face number
c     ifacenb   Face number of neighbour
c     ipstart   Starting address
c     isubfacnb Subface number of neighbour
c     j         General loop variable
c     j1        Lower index number along second direction of a face
c     jpstart   Starting address
c     jshift    Shift in j-direction
c     k         General loop variable
c     ndegfd    Number of degrees of freedom
c     ndegfdlc  Local number of degrees of freedom
c     nseg      Number of segments the boundary is divided in
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c    1105   Periodical boundary conditions on different faces
c **********************************************************************
c
c                       PSEUDO CODE
c
c    For iface := 1 (1) 4 do
c       nseg := number of segments along face iface
c       For k := 1 (1) nseg do
c          btype := type of boundary condition along segment k
c          if ( btype periodical ) then
c             iblocknb := neighbouring block
c             if ( iblocknb = iblock ) then
c                ifacenb := neighbouring face
c                isubfacnb := neigbouring subface
c
c                Store bounds along iface subface k into i1 and i2
c                Store bounds along ifacenb subface isubfacnb into j1 and j2
c                Compute mean values and put into output
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Compute ndegfdlc

      ndegfd   = iinput(55)
      icoupled = iinput(76)
      if ( icoupled.eq.0 ) then

c     --- Coupled approach

         ndegfdlc = ndegfd-1

      else

c     --- Decoupled approach

         ndegfdlc = ndegfd

      end if

c     --- Loop over faces

      do iface = 1, 4

        nseg = iinbc(iinbc(iface))

c       --- Loop over subfaces

        do k = 1, nseg

c       --- btype := type of boundary condition along segment k

           ipstart = iinbc(iface)+(k-1)*(2+ndegfdlc*3)
           btype = iinbc(ipstart+3)

           if ( btype.eq.98 ) then

c          --- btype = periodical
c              iblocknb := neighbouring block

              iblocknb = iinbc(ipstart+4)/10+1
              if ( iblocknb.eq.iblock ) then

c             --- iblocknb = iblock
c                 ifacenb := neighbouring face
c                 isubfacnb := neigbouring subface

                 ifacenb = iinbc(ipstart+4)-10*(iblocknb-1)
                 isubfacnb = iinbc(ipstart+5)

c                --- Store bounds along iface subface k into i1 and i2
c                    Compute mean values and put into output

                 i1 = iinbc(ipstart+1)
                 i2 = iinbc(ipstart+2)+1

c                --- Store bounds along ifacenb subface isubfacnb into jarea

                 jpstart = iinbc(ifacenb)+(isubfacnb-1)*(2+ndegfdlc*3)

                 j1 = iinbc(jpstart+1)
                 if ( iface.le.2 .and. ifacenb.gt.2 .or.
     +                iface.gt.2 .and. ifacenb.le.2 ) then

c                --- Error: periodical boundary conditions on different faces

                    call iserrint ( iface, 1 )
                    call iserrint ( ifacenb, 2 )
                    call iserrint ( iblock, 3 )
                    call iserrsub ( 1105, 3, 0, 0 )

                 end if
                 if ( iface.le.2 ) then

c                --- Periodical boundary conditions along faces 1 and 2

                    if ( iface.eq.1 ) then
                       i = 1
                    else
                       i = ni+1
                    end if
                    if ( iface.eq.ifacenb ) then

c                   --- face and neighbour face are the same, reverse direction

                       jshift = j1+i2
                       do j = i1, i2
                          output(i,j) = 0.5d0*
     +                            (output(i,j)+output(i,jshift-j))
                          output(i,jshift-j) = output(i,j)
                       end do

                    else

c                   --- Opposite faces

                       jshift = j1-i1
                       do j = i1, i2
                          output(i,j) = 0.5d0*
     +                            (output(i,j)+output(i,jshift+j))
                          output(i,jshift+j) = output(i,j)
                       end do

                    end if

                 else

c                --- Periodical boundary conditions along faces 3 and 4

                    if ( iface.eq.3 ) then
                       j = 1
                    else
                       j = nj+1
                    end if
                    if ( iface.eq.ifacenb ) then

c                   --- face and neighbour face are the same, reverse direction

                       jshift = j1+i2
                       do i = i1, i2
                          output(i,j) = 0.5d0*
     +                            (output(i,j)+output(jshift-i,j))
                          output(jshift-i,j) = output(i,j)
                       end do

                    else

c                   --- Opposite faces

                       jshift = j1-i1
                       do i = i1, i2
                          output(i,j) = 0.5d0*
     +                            (output(i,j)+output(i+jshift,j))
                          output(i+jshift,j) = output(i,j)
                       end do

                    end if

                 end if

               end if

            end if

         end do

      end do
      end
