      subroutine isgenpr ( mesnum, nints, intarr, nreal, realarr, nchar,
     +                     chararr )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.3    date   05-04-1998 Duncan: ispri109 resurrected
c        version 1.2    date   23-01-1997 Kees V. ispri109 deleted
c        version 1.1    date   22-04-1996 Extenson with print of time
c        version 1.0    date   19-04-1994
c
c
c     copyright (c) 1994-1996  "TU-Delft"
c     permission to copy or distribute this software or documentation
c     in hard copy or soft copy granted only by written license
c     obtained from "TU-Delft".
c     all rights reserved. no part of this publication may be reproduced,
c     stored in a retrieval system ( e.g., in memory, disk, or core)
c     or be transmitted by any means, electronic, mechanical, photocopy,
c     recording, or otherwise, without written permission from the
c     publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Actual print subroutine. Prints reals integers and strings in a
c     special format depending on the value of the parameter mesnum
c **********************************************************************
c
c                       KEYWORDS
c
c    print
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer  mesnum, nchar, nints, nreal, intarr(*)
      double precision realarr(*)
      character *(*) chararr(*)

c     chararr   i    Array of character strings containing the strings that
c                    must be substituted in the print format.
c     intarr    i    Array of integers containing the integers that
c                    must be substituted in the print format.
c     mesnum    i    Sequence number of the type of format
c                    If 1 <= mesnum <= 100 isprint calls the user print
c                    subroutine isuserpr, in which the user may define
c                    his own output formats.
c                    For mesnum > 100 the standard ISNaS print subroutine
c                    isgenpr is called, which prints according to predefined
c                    formats. The relation between mesnum and predefined
c                    format is given in isgenpr
c     nchar     i    Number of strings in a message
c     nints     i    Number of integers in a message
c     nreal     i    Number of reals in a message
c     realarr   i    Array of reals containing the reals that
c                    must be substituted in the print format.
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/ciscom'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
c **********************************************************************
c
c                       I/O
c
c     The output is printed to standard output
c **********************************************************************
c
c                       ERROR MESSAGES
c
c       2   value of mesnum not yet implemented
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERRINT  Put integer value in error message file
c     ISERRSUB  Error messages
c     ISPRI101  Print arrays iinput and rinput
c     ISPRI102  Print array coor
c     ISPRI103  Print array solut
c     ISPRI104  Print array geom
c     ISPRI105  Print array amat
c     ISPRI106  Print array rhside
c     ISPRI107  Print array coefs
c     ISPRI108  Prints information from the linear solver
c     ISPRI200  Prints CPU time used
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Check parameters

      if ( mesnum.eq.200 ) then

c     --- mesnum = 200, print CPU time

         call ispri200 ( realarr, chararr )

      else if ( mesnum.gt.109 ) then

c     --- mesnum > 108

         call iserrint ( mesnum, 1 )
         call iserrsub ( 2, 1, 0, 0 )

      else

         go to ( 101, 102, 103, 104, 105, 106, 107, 108, 109 ),
     +           mesnum-100

c        --- mesnum = 101, print arrays iinput and rinput

101      call ispri101 ( intarr, realarr, chararr )
         go to 1000

c        --- mesnum = 102, print array coor

102      call ispri102 ( intarr, realarr, chararr )
         go to 1000

c        --- mesnum = 103, print array solut

103      call ispri103 ( intarr, realarr, chararr )
         go to 1000

c        --- mesnum = 104, print array geom

104      call ispri104 ( intarr, realarr, chararr )
         go to 1000

c        --- mesnum = 105, print array matrix

105      call ispri105 ( intarr, realarr, chararr )
         go to 1000

c        --- mesnum = 106, print array rhside

106      call ispri106 ( intarr, realarr, chararr )
         go to 1000

c        --- mesnum = 107, print array coefs

107      call ispri107 ( intarr, realarr, chararr )
         go to 1000

c        --- mesnum = 108, print information and residuals from
c                          the linear solver

108      call ispri108 ( intarr, realarr )
         go to 1000

c        --- mesnum = 109, print number of iterations and residual
c                          from the non-linear Gauss-Seidel algorithm

109      call ispri109 ( intarr, realarr )
         go to 1000
      end if

1000  continue
      end
