      subroutine isnt3d ( ni, nj, nk, nvirtual, coor,
     +                    ip, jp, kp, ip0, jp0, kp0, uci, vci, wci )
c ======================================================================
c
c        programmer   Kees Kassels
c        version      2.1  date 14-10-1997 Fred T. change converg. criteria
c        version      2.0  date 06-03-1996 Remove ndim
c        version      1.1  date 26-09-1995 Kees K. bug fixed
c        version      1.0  date 29-12-1994
c
c   copyright (c) 1994-1996  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Newton Raphson iteration, used for trilinear mapping 3D
c
c **********************************************************************
c
c                       KEYWORDS
c
c   3d
c   inverse_mapping
c   newton
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, ip, jp, kp, ip0 ,jp0 ,kp0
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,3)
      double precision uci, vci, wci

c     coor      i    reserved
c     ip        i    Local counting variable i-direction
c     ip0       i    Ip of point where interpolated value is needed
c     jp        i    Local counting variable j-direction
c     jp0       i    Jp of point where interpolated value is needed
c     kp        i    Local counting variable k-direction
c     kp0       i    Kp of point where interpolated value is needed
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     uci       o    Local coordinate for the interpolation in the i-direction
c     vci       o    Local coordinate for the interpolation in the j-direction
c     wci       o    Local coordinate for the interpolation in the k-direction
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer maxiter, iter, pnt, idir, ipl, jpl, kpl
      double precision coorl(1:8,1:3), fuvw(1:3), dfuvw(1:3,1:3),
     +                 duci, dvci, dwci, detjac
      logical conv

c     conv      Logical variable indicating acquired convergence.
c     coorl     Array of coordinates of 8 surrounding points
c     detjac    Determinant jacobian
c     dfuvw     Jacobian of function fuvw
c     duci      Finite difference of uci
c     dvci      Finite difference of vci
c     dwci      Finite difference of wci
c     fuvw      Function of the three local coordinates u, v and w
c     idir      General loop variable for 1...ndim loop
c     ipl       Local ip value
c     iter      The current iteration number.
c     jpl       Local jp value
c     kpl       Local kp value
c     maxiter   Maximum number of iterations
c     pnt       Local point number 1 ....8
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value into error message
c     ISWARSUB  Print warnings
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     1006     no convergence in computation of bilinear mapping
c **********************************************************************
c
c                       PSEUDO CODE
c
c      determine coordinates of eight surrounding grid points
c      determine the functions of the three coordinates of the
c         center point expressed in the coordinates of the eigth
c         surrounding points and the local coordinates uci, vci,
c         and wci used for interpolation.
c      compute jacobian
c      compute uci,vci and wci with newton raphson iteration
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      ipl = ip
      jpl = jp
      kpl = kp

c     --- coordinates of the eight surrounding grid points

      do pnt = 1, 8
         if( pnt.eq.2 ) then
            ipl = ip+1
         else if ( pnt.eq.3 ) then
            ipl = ip
            jpl = jp+1
         else if ( pnt.eq.4 ) then
            ipl = ip+1
         else if ( pnt.eq.5 ) then
            ipl = ip
            jpl = jp
            kpl = kp+1
         else if ( pnt.eq.6 ) then
            ipl = ip+1
         else if ( pnt.eq.7 ) then
            ipl = ip
            jpl = jp+1
         else if ( pnt.eq.8 ) then
            ipl = ip+1
         end if
         do idir= 1, 3
            coorl(pnt,idir) = 125d-3 * (
     +                        coor(ipl-1,jpl-1,kpl-1,idir) +
     +                        coor(ipl  ,jpl-1,kpl-1,idir) +
     +                        coor(ipl-1,jpl  ,kpl-1,idir) +
     +                        coor(ipl  ,jpl  ,kpl-1,idir) +
     +                        coor(ipl-1,jpl-1,kpl  ,idir) +
     +                        coor(ipl  ,jpl-1,kpl  ,idir) +
     +                        coor(ipl-1,jpl  ,kpl  ,idir) +
     +                        coor(ipl  ,jpl  ,kpl  ,idir)  )
         end do
      end do

      uci = 0.5d0
      vci = 0.5d0
      wci = 0.5d0

c     --- set ksi and eta

      conv = .false.
      maxiter = 10
      iter = 1
100   if ( iter.le.maxiter .and. .not. conv ) then

         do idir = 1, 3

c        --- the three functions

            fuvw(idir) =
     +         (-coorl(1,idir)+coorl(2,idir)+coorl(3,idir)-coorl(4,idir)
     +         +coorl(5,idir)-coorl(6,idir)-coorl(7,idir)+coorl(8,idir))
     +         *uci*vci*wci +
     +         (coorl(1,idir)-coorl(2,idir)-coorl(3,idir)+coorl(4,idir))
     +         *uci*vci   +
     +         (coorl(1,idir)-coorl(2,idir)-coorl(5,idir)+coorl(6,idir))
     +         *uci*wci   +
     +         (coorl(1,idir)-coorl(3,idir)-coorl(5,idir)+coorl(7,idir))
     +         *vci*wci   +
     +         (-coorl(1,idir)+coorl(2,idir))
     +         *uci       +
     +         (-coorl(1,idir)+coorl(3,idir))
     +         *vci       +
     +         (-coorl(1,idir)+coorl(5,idir))
     +         *wci       +
     +         coorl(1,idir) - coor(ip0,jp0,kp0,idir)

c           --- Jacobian

            dfuvw(idir,1) =
     +         (-coorl(1,idir)+coorl(2,idir)+coorl(3,idir)-coorl(4,idir)
     +         +coorl(5,idir)-coorl(6,idir)-coorl(7,idir)+coorl(8,idir))
     +         *vci*wci +
     +         (coorl(1,idir)-coorl(2,idir)-coorl(3,idir)+coorl(4,idir))
     +         *vci   +
     +         (coorl(1,idir)-coorl(2,idir)-coorl(5,idir)+coorl(6,idir))
     +         *wci   +
     +         (-coorl(1,idir)+coorl(2,idir))
            dfuvw(idir,2) =
     +         (-coorl(1,idir)+coorl(2,idir)+coorl(3,idir)-coorl(4,idir)
     +         +coorl(5,idir)-coorl(6,idir)-coorl(7,idir)+coorl(8,idir))
     +         *uci*wci +
     +         (coorl(1,idir)-coorl(2,idir)-coorl(3,idir)+coorl(4,idir))
     +         *uci   +
     +         (coorl(1,idir)-coorl(3,idir)-coorl(5,idir)+coorl(7,idir))
     +         *wci   +
     +         (-coorl(1,idir)+coorl(3,idir))
            dfuvw(idir,3) =
     +         (-coorl(1,idir)+coorl(2,idir)+coorl(3,idir)-coorl(4,idir)
     +         +coorl(5,idir)-coorl(6,idir)-coorl(7,idir)+coorl(8,idir))
     +         *uci*vci +
     +         (coorl(1,idir)-coorl(2,idir)-coorl(5,idir)+coorl(6,idir))
     +         *uci   +
     +         (coorl(1,idir)-coorl(3,idir)-coorl(5,idir)+coorl(7,idir))
     +         *vci   +
     +         (-coorl(1,idir)+coorl(5,idir))

         end do

c        --- Newton Raphson

         detjac =
     +          dfuvw(1,1)*(dfuvw(2,2)*dfuvw(3,3)-dfuvw(2,3)*dfuvw(3,2))
     +         +dfuvw(1,2)*(dfuvw(2,3)*dfuvw(3,1)-dfuvw(2,1)*dfuvw(3,3))
     +         +dfuvw(1,3)*(dfuvw(2,1)*dfuvw(3,2)-dfuvw(2,2)*dfuvw(3,1))

         duci= (-fuvw(1)*(dfuvw(2,2)*dfuvw(3,3)-dfuvw(2,3)*dfuvw(3,2))
     +          -fuvw(2)*(dfuvw(3,2)*dfuvw(1,3)-dfuvw(3,3)*dfuvw(1,2))
     +          -fuvw(3)*(dfuvw(1,2)*dfuvw(2,3)-dfuvw(1,3)*dfuvw(2,2)))/
     +          detjac
         dvci= (-fuvw(1)*(dfuvw(2,3)*dfuvw(3,1)-dfuvw(2,1)*dfuvw(3,3))
     +          -fuvw(2)*(dfuvw(3,3)*dfuvw(1,1)-dfuvw(3,1)*dfuvw(1,3))
     +          -fuvw(3)*(dfuvw(1,3)*dfuvw(2,1)-dfuvw(1,1)*dfuvw(2,3)))/
     +          detjac
         dwci= (-fuvw(1)*(dfuvw(2,1)*dfuvw(3,2)-dfuvw(2,2)*dfuvw(3,1))
     +          -fuvw(2)*(dfuvw(3,1)*dfuvw(1,2)-dfuvw(3,2)*dfuvw(1,1))
     +          -fuvw(3)*(dfuvw(1,1)*dfuvw(2,2)-dfuvw(1,2)*dfuvw(2,1)))/
     +          detjac

         uci = uci + duci
         vci = vci + dvci
         wci = wci + dwci

         if ( abs(dvci).lt.1d-2 .and. abs(duci).lt.1d-2 .and.
     +        abs(dwci).lt.1d-2 ) conv = .true.
         iter = iter + 1
         go to 100

      end if
      if ( .not. conv ) then
         call iseropen('isnton')
         call iserrint(ip, 1)
         call iserrint(jp, 2)
         call iserrint(kp, 3)
         call iswarsub(1006, 3, 0, 0)
         call iserclos('isnton')

      end if

      end
