      subroutine isnton ( ni, nj, nvirtual, coor, ip, jp, ip0,  jp0,
     +                    uci, vci )
c ======================================================================
c
c        programmer   Jan Mooiman
c        version      1.3  02-04-1997 Kees K. literal constants in
c                                     double precision for f90
c        version      1.2  11-05-1994 Erik: added the very first comments
c                                     to this routine and adapted it for
c                                     the new structure of ISNaS
c        version      1.1  10-06-1993 abs for stopping criterion
c        version      1.0  06-08-1991
c
c   copyright (c) 1991-1996  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                  DESCRIPTION
c
c     Compute the inverse of the bilinear mapping by Newton Raphson iteration
c     reference    Graduation report of Margot Gerritsen
c **********************************************************************
c
c                  KEYWORDS
c
c   2d
c   inverse_mapping
c   newton
c **********************************************************************
c
c           INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, ip, jp, ip0 ,jp0
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2)
      double precision uci, vci

c     coor      i    reserved
c     ip        i    Local counting variable i-direction
c     ip0       i    Ip of point where interpolated value is needed
c     jp        i    Local counting variable j-direction
c     jp0       i    Jp of point where interpolated value is needed
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     uci       o    Local coordinate for the interpolation in the i-direction
c     vci       o    Local coordinate for the interpolation in the j-direction
c **********************************************************************
c
c                   COMMON BLOCKS
c
c **********************************************************************
c
c                    LOCAL PARAMETERS
c
c
      integer maxiter, iter
      double precision xc, x11, x21, x12, x22, duci, dvci, detjac,
     +                 yc, y11, y21, y12, y22,
     +                 f1uv, f2uv, df1du, df1dv, df2du, df2dv
      logical conv

c     conv      Logical variable indicating acquired convergence.
c     detjac    Determinant jacobian
c     df1du     dF_1 / dksi
c     df1dv     dF_1 / deta
c     df2du     dF_2 / dksi
c     df2dv     dF_2 / deta
c     duci      Finite difference of uci
c     dvci      Finite difference of vci
c     f1uv      F_1(ksi,eta)
c     f2uv      F_2(ksi,eta
c     iter      The current iteration number.
c     maxiter   Maximum number of iterations
c     x11       x co-ordinate in point 11
c     x12       x co-ordinate in point 12
c     x21       x co-ordinate in point 21
c     x22       x co-ordinate in point 22
c     xc        x co-ordinate
c     y11       y co-ordinate in point 11
c     y12       y co-ordinate in point 12
c     y21       y co-ordinate in point 21
c     y22       y co-ordinate in point 22
c     yc        y co-ordinate
c **********************************************************************
c
c                         I/O
c
c **********************************************************************
c
c                      SUBROUTINES CALLED
c
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value into error message
c     ISWARSUB  Print warnings
c **********************************************************************
c
c                     ERROR MESSAGES
c
c    1006   no convergence in computation of bilinear mapping
c **********************************************************************
c
c                     METHOD
c
c    The mathemaical description of this subroutine can be found in the ISNaS
c    mathematical manual
c
c    The purpose of this subroutine is to find the (ksi,eta) co-ordinates
c    of a point in a quadrilateral, where the xy co-ordinates of the
c    vertices as well are known as well as the xy co-ordinates of the
c    point to be found
c
c    According to the mathematical manual we have to solve:
c
c                             n          n+1         n              n
c    | dF_1/dksi  dF_1/deta |     | ksi |     | ksi |       | F_1 |
c    |                      |   ( |     |  -  |     | ) = - |     |
c    | dF_2/dksi  dF_2/deta |     | eta |     | eta |       | F_2 |
c
c    with:
c
c    F = (1-ksi)(1-eta)x_11 + ksi eta x_21 +
c        (1-ksi)(eta)x_21 + (ksi)(1-eta)x_12
c
c    dF/dksi = (x_11+x_22-x_21-x_12) eta + x_21-x_11
c    dF/deta = (x_11+x_22-x_12-x_21) ksi + x_12-x_11
c
c    The right upper point of the quadrilateral is marked ip,jp
c    The point to be interpolated ip0,jp0
c    These points have the same index for internal points,
c    but for boundary points they may be different
c
c
c                     PSEUDO CODE
c
c    Compute x,y, x_11, ...
c    ksi,eta = 0.5,0.5
c    For iter := 1 (1) max_iter while (not converged) do
c       Compute Jacobian
c       Compute right-hand side
c       Solve correction
c       Update ksi, eta
c ======================================================================
c
c     --- Compute x and y co-ordinates

      xc = coor(ip0 ,jp0 ,1)
      x11 = 25d-2*(coor(ip  ,jp  ,1)+coor(ip-1,jp  ,1)+
     +             coor(ip  ,jp-1,1)+coor(ip-1,jp-1,1))
      x21 = 25d-2*(coor(ip+1,jp  ,1)+coor(ip  ,jp  ,1)+
     +             coor(ip+1,jp-1,1)+coor(ip  ,jp-1,1))
      x12 = 25d-2*(coor(ip  ,jp+1,1)+coor(ip-1,jp+1,1)+
     +             coor(ip  ,jp  ,1)+coor(ip-1,jp  ,1))
      x22 = 25d-2*(coor(ip+1,jp+1,1)+coor(ip  ,jp+1,1)+
     +             coor(ip+1,jp  ,1)+coor(ip  ,jp  ,1))
      yc = coor(ip0 ,jp0 ,2)
      y11 = 25d-2*(coor(ip  ,jp  ,2)+coor(ip-1,jp  ,2)+
     +             coor(ip  ,jp-1,2)+coor(ip-1,jp-1,2))
      y21 = 25d-2*(coor(ip+1,jp  ,2)+coor(ip  ,jp  ,2)+
     +             coor(ip+1,jp-1,2)+coor(ip  ,jp-1,2))
      y12 = 25d-2*(coor(ip  ,jp+1,2)+coor(ip-1,jp+1,2)+
     +             coor(ip  ,jp  ,2)+coor(ip-1,jp  ,2))
      y22 = 25d-2*(coor(ip+1,jp+1,2)+coor(ip  ,jp+1,2)+
     +             coor(ip+1,jp  ,2)+coor(ip  ,jp  ,2))

c     --- set ksi and eta

      uci = 0.5d0
      vci = 0.5d0
      conv = .false.
      maxiter = 5
      iter = 1
100   if ( iter.le.maxiter .and. .not. conv ) then

c     --- Jacobian

         df1du = (x11+x22-x21-x12) * vci + x21-x11
         df1dv = (x11+x22-x21-x12) * uci + x12-x11
         df2du = (y11+y22-y21-y12) * vci + y21-y11
         df2dv = (y11+y22-y21-y12) * uci + y12-y11
         detjac= df1du * df2dv - df1dv * df2du

c        --- Newton Raphson

         f1uv  = (x11+x22-x21-x12)*uci*vci + (x12-x11)*vci +
     +           (x21-x11)*uci + x11-xc
         f2uv  = (y11+y22-y21-y12)*uci*vci + (y12-y11)*vci +
     +           (y21-y11)*uci + y11-yc
         duci    = (-f1uv*df2dv + f2uv*df1dv)/detjac
         dvci    = (-f2uv*df1du + f1uv*df2du)/detjac
         uci     = uci+duci
         vci     = vci+dvci
         if ( abs(dvci/vci).lt.1d-2 .and. abs(duci/uci).lt.1d-2 )
     +     conv = .true.
         iter = iter + 1
         go to 100

      end if
      if ( .not. conv ) then
         call iseropen('isnton')
         call iserrint(ip, 1)
         call iserrint(jp, 2)
         call iserrint(0, 3)
         call iswarsub(1006, 3, 0, 0)
         call iserclos('isnton')
      end if

      end
