      subroutine isout03a ( iblock, ni, nj, nvirtual, nblocks, nfaces,
     +                     maxnsubf, stream, order, nsubfaces, subfaces,
     +                     solut, streamfun, coor)
c ======================================================================
c
c     programmer Kees Kassels
c     version    1.0    02-02-1999  copy of routine isout03 for the
c                                   axisymmetric case
c
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     ISNAS subroutine which computes the stream function for the
c     axisymmetric case
c **********************************************************************
c
c                       KEYWORDS
c
c   axisymmetric
c   stream_function
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer  iblock, maxnsubf, nblocks, nfaces,
     +         ni, nj, nvirtual
      integer nsubfaces(1:nblocks, 1:nfaces)
      integer order(1:2*nblocks)
      double precision  streamfun(1:ni+1,1:nj+1)
      double precision  solut(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,2),
     +                   coor(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,2)
      double precision  stream(1:nblocks, 1:nfaces, 1:maxnsubf,2)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     coor      i    reserved
c     iblock    i    Block sequence number
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     nfaces    i    Number of faces
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     nvirtual  i    Number of rows of virtual cells
c     order     i    reserved
c     solut     i    reserved
c     stream   i/o   Array containing for each subface, the value of
c                    the stream function at the beginning and ending of that
c                    subface
c     streamfun o    Stream function in the vertices
c     subfaces  i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ibto, iface, ifto, iorient, ip, isfto, isubface, jp,
     +      relblckor, tsign
      double precision diff, maxdiff, mindiff, phisign, phit(6), pi

c     diff      Difference between subdomain stream functions
c     ibto      Target block
c     iface     Face number
c     ifto      Target face
c     iorient   Relative orientation of subfaces, see itopol
c     ip        Local counting variable i-direction
c     isfto     Target subface
c     isubface  Subface number
c     jp        Local counting variable j-direction
c     maxdiff   Maximal difference in stream functions found
c     mindiff   Minimal difference in stream functions found
c     phisign   Expected sign difference between the stream function
c               and its neighbour.
c     phit      The quantity  d phi/dt = phit*V_n
c               with t the tangential vector in
c               increasing co-ordinate direction (so no anti-clockwise
c               directions are used but simply the i and j-directions)
c               With V_n the outward
c     pi        the famous constant
c     relblckor Relative block orientation. If the neighbouring
c               block has sqrt(g) with opposite sign to this block
c               then relblckor = 1 otherwise it is 0
c     tsign     Inner product of tangential vector defined counter
c               clockwise with respect to the outward normal and the
c               base vector along the side
c **********************************************************************
c
c                       I/O
c     none
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERREAL  Put real value in error message file
c     ISERRINT  Put integer value in error message file
c     ISWARSUB  Print warnings
c **********************************************************************
c
c                       ERROR MESSAGES
c
c    1603   warning about inaccurate computation of the streamfunction
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Compute streamfunction for subdomain without looking at the
c     neighbouring subdomains
c
c     Fill the stream array for block iblock with the computed values.
c
c     Find a subface of this block that has a neighbour which already
c     has a computed streamfunction.
c
c     Compute the difference of the streamfunction of this block with
c     that of the found neighbour using the stream array.
c
c     Add this difference to the subdomain stream function
c
c     Add this difference to the stream array for this block
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- compute phit

      pi = 4d0 * atan(1d0)

      phit(ileft) = -1d0
      phit(iright) = 1d0
      phit(idown) = 1d0
      phit(iup) = -1d0

c     --- compute the streamfunction for this subdomain without looking at
c         the neighbours

      streamfun(1,1) = 0d0

      do jp = 1, nj+1

         if( jp.gt.1 ) then
            streamfun(1,jp) = streamfun(1, jp-1) + solut(1, jp-1, 1)*
     +                         5d-1*(coor(ip,jp-1,1)+coor(ip,jp,1))*
     +                         2d0*pi
         end if
         do ip = 2, ni+1
            streamfun(ip,jp) = streamfun(ip-1, jp) - solut(ip-1, jp, 2 )
     +                       * 5d-1*(coor(ip-1,jp,1)+coor(ip,jp,1))*
     +                         2d0*pi
         end do

      end do

c     --- store the computed values in the stream array
c         left side

      iface = ileft
      ip    = 1
      do isubface = 1, nsubfaces(iblock,iface)
         jp = subfaces(iblock,iface,isubface,1)
         stream(iblock,iface,isubface,1) = streamfun(ip,jp)
         jp = subfaces(iblock,iface,isubface,2)+1
         stream(iblock,iface,isubface,2) = streamfun(ip,jp)
      end do

c     --- right side

      iface = iright
      ip    = ni+1
      do isubface = 1, nsubfaces(iblock,iface)
         jp = subfaces(iblock,iface,isubface,1)
         stream(iblock,iface,isubface,1) = streamfun(ip,jp)
         jp = subfaces(iblock,iface,isubface,2)+1
         stream(iblock,iface,isubface,2) = streamfun(ip,jp)
      end do

c     --- down side

      iface = idown
      jp    = 1
      do isubface = 1, nsubfaces(iblock,iface)
         ip = subfaces(iblock,iface,isubface,1)
         stream(iblock,iface,isubface,1) = streamfun(ip,jp)
         ip = subfaces(iblock,iface,isubface,2)+1
         stream(iblock,iface,isubface,2) = streamfun(ip,jp)
      end do

c     --- up side

      iface = iup
      jp    = nj+1
      do isubface = 1, nsubfaces(iblock,iface)
         ip = subfaces(iblock,iface,isubface,1)
         stream(iblock,iface,isubface,1) = streamfun(ip,jp)
         ip = subfaces(iblock,iface,isubface,2)+1
         stream(iblock,iface,isubface,2) = streamfun(ip,jp)
      end do

c     --- find a subface of this block that has a neighbour with an
c         already computed streamfunction
c         diff must be initialized because of the possibility of isolated
c         structures

      diff = 0d0
      mindiff = 1d30
      maxdiff = -1d30

      do iface = 1, nfaces
         do isubface = 1, nsubfaces(iblock,iface)
            ibto = subfaces(iblock,iface,isubface,5)
            if ( ibto.ne.0 ) then
               if ( order(nblocks+ibto)
     +              .lt.
     +              order(nblocks+iblock) ) then

                  ifto = subfaces(iblock,iface,isubface,6)
                  isfto = subfaces(iblock,iface,isubface,7)

                  iorient    = subfaces(iblock,iface,isubface,8)
                  tsign      = mod(iorient,2)
                  relblckor = iorient/4

                  phisign = -phit(iface)*phit(ifto)

                  if ( relblckor.eq.1 ) phisign = -phisign

                  if ( tsign.eq.1 ) phisign = -phisign

                  if ( tsign.eq.0 ) then

                     diff = stream(ibto,ifto,isfto,1)  -
     +                   phisign*stream(iblock,iface,isubface,1)

                  else

                     diff = stream(ibto,ifto,isfto,2) -
     +                   phisign*stream(iblock,iface,isubface,1)
                  end if

                  mindiff = min(mindiff, diff)
                  maxdiff = max(maxdiff, diff)

               end if
            end if
         end do
      end do

      if ( mindiff.le.maxdiff ) then

c     --- neighbouring blocks with an already initialized streamfunction
c         were found

         if ( abs(maxdiff).gt.epsmac ) then
         if ( (maxdiff-mindiff)/abs(maxdiff).gt.0.1 ) then
            call iseropen('isout03')
            call iserrint(iblock, 1)
            call iserreal(maxdiff, 1)
            call iserreal(mindiff, 2)
            call iswarsub(1603, 1, 2, 0)
            call iserclos('isout03')
         end if
         end if

c        --- add difference to the subdomain streamfunction

         do ip = 1, ni+1
            do jp = 1, nj+1
               streamfun(ip,jp) = phisign*streamfun(ip,jp) + diff
            end do
         end do

c        --- add difference to the stream array

         do iface = 1, nfaces
            do isubface = 1, nsubfaces(iblock,iface)
               stream(iblock,iface,isubface,1) =
     +            phisign*stream(iblock,iface,isubface,1) + diff
               stream(iblock,iface,isubface,2) =
     +            phisign*stream(iblock,iface,isubface,2) + diff
            end do
         end do

      end if

      end
