      subroutine isoutfrs ( ni, nj, nvirtual, coor, vector, output )
c ======================================================================
c
c     programmer  Kees Kassels
c     version     1.0   26-04-1999
c
c   copyright (c) 1991-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Computation of the cartesian velocities in the vertices on the
c   upper boundary for freesurface computation
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2),
     +                 output(1-nvirtual:ni+1+nvirtual,2),
     +                 vector(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,2)

c     coor      i    Array containing the coordinates for one block
c                    In 2d coor(i,j,.) contains the x resp y-coordinate
c                    of point i,j
c                    in 3d: coor(i,j,k,.) corresponds to (i,j,k)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     output    i    Cartesian components of the flow field in the vertices
c     vector    i    Input flow field
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, k
      double precision a1(2), a2(2), output1, output2, sqrtg, v1h, v1l,
     +                 v2h, v2l

c     a1        Covariant xi-1 base vector
c     a2        Covariant xi-2 base vector
c     i         General loop variable
c     j         General loop variable
c     k         General loop variable
c     output1   Temporary storage for output(i,j,k,1)
c     output2   Temporary storage for output(i,j,k,2)
c     sqrtg     The Jacobian in the center of a cell
c     v1h       Second flux V1 in the 1 direction
c     v1l       First  flux V1 in the 1 direction
c     v2h       Second flux V2 in the 2 direction
c     v2l       First  flux V2 in the 2 direction
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     In order to interpolate and compute the cartesian components,
c     the contravariant components in cell centres are computed by linear
c     interpolation, these are translated to cartesian vectors and lastly
c     the cartesian vector components in the vertices are determined .
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Interpolation to vertices

      j =  nj+1
      do i = 0, ni+2
         v1l = vector(i  ,j  ,1)
         v2l = vector(i-1,j  ,2)
         v1h = vector(i  ,j-1,1)
         v2h = vector(i  ,j  ,2)

c        --- Note that to find U^alpha in the cell centre we need
c            ( u , a^(alpha) ) in the cell centre and, if x,y are
c            bilinear functions of ksi,eta, then
c            sqrt(g)*a^(alpha)(centre) =
c            (sqrt(g)*a^(alpha)(low)+sqrt(g)*a^(alpha)(high))
c            so the sqrt(g) built into the fluxes allows us to simply
c            take the average here

         output(i,1) = 0.5d0*(v1l+v1h)
         output(i,2) = 0.5d0*(v2l+v2h)
      end do

c     --- transform to cartesian components

      j = nj+1
      do i = 0, ni+2

         output1 = output(i,1)
         output2 = output(i,2)

         do k = 1, 2

            a1(k) = 0.5d0*( coor(i+1,j ,k) - coor(i,j ,k) +
     +                       coor(i ,j,k)   - coor(i-1,j,k) )
            a2(k) = 0.5d0*( coor(i ,j+1,k) - coor(i ,j,k) +
     +                       coor(i,j ,k)   - coor(i,j-1,k) )
         end do

         sqrtg = a1(1)*a2(2) - a2(1)*a1(2)

         do k = 1, 2
            output(i,k) =
     +         ( a1(k) * output1 + a2(k) * output2 ) / sqrtg
         end do

      end do

      end
