      subroutine isprar01 ( array, nvirtual, ni, nj, ndim )
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.2    date 21-02-95 (Possibilities 4 and 5 added)
c        version 1.1    date 21-02-95 (Possibility of permutation added)
c        version 1.0    date 01-06-94
c
c
c   copyright (c) 1994-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                      DESCRIPTION
c
c     Print the contents of the array array defined on a two-dimensional grid
c     Actual printing subroutine
c ********************************************************************
c
c                      KEYWORDS
c
c    2d
c    print
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer nvirtual, ni, nj, ndim
      double precision array(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ndim)

c     array     i    Double precision array
c     ndim      i    Dimension of space (2 or 3)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c ********************************************************************
c
c                      COMMON BLOCKS
c
      include 'IScommon/cisnas'
      include 'IScommon/cdebug'
c ********************************************************************
c
c                      LOCAL PARAMETERS
c
      integer i, j, l, iperm(2), ilowloop, iupploop, jlowloop, jupploop
      double precision matmax, copyar(5)
      logical null

c     copyar    Work array in which some quantities are copied
c     i         General loop variable
c     ilowloop  Lower bound for loop
c     iperm     Array of permutations of the numbers 1, 2 and 3
c               Contents: 1, 2, 3, 1, 2
c     iupploop  Upper bound for loop
c     j         General loop variable
c     jlowloop  Lower bound for loop
c     jupploop  Upper bound for loop
c     l         General loop variable
c     matmax    Maximum absolute value of matrix elements
c     null      If true the complete row of the matrix is zero
c ********************************************************************
c
c                      SUBROUTINES CALLED
c
c ********************************************************************
c
c                      I/O
c
c ********************************************************************
c
c                      ERROR MESSAGES
c
c ********************************************************************
c
c                      PSEUDO CODE
c
c        For j := 1-nvirtual (1) nj+1+nvirtual do
c           For i := 1-nvirtual (1) ni+1+nvirtual do
c              write i,j,k coor(i,j,k,l) (l=1(1)ndim)
c ======================================================================
c
      matmax = 0d0
      do i = 1-nvirtual, ni+1+nvirtual
         do j = 1-nvirtual, nj+1+nvirtual
            do l = 1, ndim
                  matmax = max ( matmax, abs(array(i,j,l)) )
            end do
         end do
      end do
      matmax = matmax*epsmac*1d2
      ilowloop = 1-nvirtual
      iupploop = ni+1+nvirtual
      jlowloop = 1-nvirtual
      jupploop = nj+1+nvirtual
      if ( ndim.eq.1 ) then
         iupploop = iupploop-1
         jupploop = jupploop-1
      end if
      if ( iprseq.eq.1 ) then

c        --- Standard sequence

         do j = jlowloop, jupploop
            do i = ilowloop, iupploop
               null = .true.
               do l = 1, ndim
                  copyar(l) = array(i,j,l)
                  if ( abs(copyar(l)).gt.matmax ) then
                     null = .false.
                  else
                     copyar(l) = 0d0
                  end if
               end do
               if ( .not. null ) then
                  write(irefwr,100) i,j,
     +            (copyar(l), l=1,ndim)
100               format(1x,ss,2i4,':',5(1pe14.6e2))
               end if
            end do
         end do

      else if ( iprseq.eq.2 ) then

c     --- Sequence j,i

         if ( ndim.eq.1 ) then
            iperm(1) = 1
         else
            iperm(1) = 2
            iperm(2) = 1
         end if
         do i = ilowloop, iupploop
            do j = jlowloop, jupploop
               null = .true.
               do l = 1, ndim
                  copyar(l) = array(i,j,l)
                  if ( abs(copyar(l)).gt.matmax ) then
                     null = .false.
                  else
                     copyar(l) = 0d0
                  end if
               end do
               if ( .not. null ) then
                  write(irefwr,100) j,i,
     +            (copyar(iperm(l)), l=1,ndim)
               end if
            end do
         end do

      else if ( iprseq.eq.4 ) then

c        --- Standard sequence, i reversed

         do j = jlowloop, jupploop
            do i = iupploop, ilowloop, -1
               null = .true.
               if ( ndim.eq.2 ) then
                  copyar(1) = -array(i,j,1)
                  if ( i.gt.1-nvirtual ) then
                     copyar(2) = array(i-1,j,2)
                  else
                     copyar(2) = 0d0
                  end if
               else
                  do l = 1, ndim
                     copyar(l) = array(i,j,l)
                  end do
               end if
               do l = 1, ndim
                  if ( abs(copyar(l)).gt.matmax ) then
                     null = .false.
                  else
                     copyar(l) = 0d0
                  end if
               end do
               if ( .not. null ) then
                  write(irefwr,100) iupploop+1-nvirtual-i,j,
     +                (copyar(l), l=1,ndim)
               end if
            end do
         end do

      else if ( iprseq.eq.5 ) then

c        --- Sequence j, j reversed

         if ( ndim.eq.1 ) then
            iperm(1) = 1
         else
            iperm(1) = 2
            iperm(2) = 1
         end if
         do i = iupploop, ilowloop, -1
            do j = jlowloop, jupploop
               null = .true.
               if ( ndim.eq.2 ) then
                  copyar(2) = -array(i,j,2)
                  if ( j.gt.1-nvirtual ) then
                     copyar(2) = array(i,j-1,2)
                  else
                     copyar(2) = 0d0
                  end if
               else
                  do l = 1, ndim
                     copyar(l) = array(i,j,l)
                  end do
               end if
               do l = 1, ndim
                  if ( abs(copyar(l)).gt.matmax ) then
                     null = .false.
                  else
                     copyar(l) = 0d0
                  end if
               end do
               if ( .not. null ) then
                  write(irefwr,100) jupploop+1-nvirtual-j,i,
     +                (copyar(iperm(l)), l=1,ndim)
               end if
            end do
         end do

      end if
      end
