      subroutine ispri107 ( iinput, coefs, text )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date 09-06-1998 Charles: nvpoints changed to npointsv
c        version 1.0    date 01-06-1994
c
c
c   copyright (c) 1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Print the contents of array coefs for one block
c     Actual printing subroutine
c **********************************************************************
c
c                       KEYWORDS
c
c    print
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinput(*)
      double precision coefs(*)
      character *(*) text

c     coefs     i    reserved
c     iinput    i    reserved
c     text      i    String in which a text is stored
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, ncoefs, ndim, ni, nj, nk, nvirtual, npointsv

c     i         General loop variable
c     ncoefs    Maximum number of different coefficients for one equation
c     ndim      dimension of the problem
c     ni        i-dimension of a block
c     nj        j-dimension of a block
c     nk        k-dimension of a block
c     npointsv  Number of points in the mesh with virtual points
c     nvirtual  number of virtual rows of coordinates
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISPRAR01  Print general field array defined in R2
c     ISPRAR02  Print general field array defined in R3
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Print heading
c    Extract ndim, ni, nj, nk, nvirtual, ncoefs
c    Print coefs
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Print heading

      write(irefwr,100) text
100   format(//1x,a)
      write(irefwr,110) iblockrf
110   format(//' Contents of array coefs for block number', i4 //)

c     --- Extract ndim, ni, nj, nk, nvirtual

      ni       = iinput(51)
      nj       = iinput(52)
      nk       = iinput(53)
      nvirtual = iinput(54)
      ndim     = iinput(57)
      ncoefs   = iinput(66)
      npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
      if ( ndim.eq.3 ) npointsv = npointsv*(nk+1+2*nvirtual)

c     --- Print coefs

      do i = 1, ncoefs

         write(irefwr,120) i
120      format(//' Coefficient ',i4//)
         if ( ndim.eq.2 ) then

c        --- ndim = 2

            call isprar01 ( coefs(1+(i-1)*npointsv), nvirtual, ni,
     +                      nj, ndim )

         else

c        --- ndim = 3

            call isprar02 ( coefs(1+(i-1)*npointsv), nvirtual, ni,
     +                      nj, nk, ndim )

         end if

      end do

      end
