      subroutine isprint ( mesnum, nints, intarr, nreal, realarr, nchar,
     +                     chararr )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.3    date 05-10-1999  Jason: added send-requests
c        version 1.2    date 23-11-1998  Jason: calls to issenmpi
c        version 1.1    date 30-01-1996  Check on length of chararr
c        version 1.0    date 19-04-1994
c
c
c     copyright (c) 1994-1999  "TU-Delft"
c     permission to copy or distribute this software or documentation
c     in hard copy or soft copy granted only by written license
c     obtained from "TU-Delft".
c     all rights reserved. no part of this publication may be reproduced,
c     stored in a retrieval system ( e.g., in memory, disk, or core)
c     or be transmitted by any means, electronic, mechanical, photocopy,
c     recording, or otherwise, without written permission from the
c     publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     General print subroutine. Prints reals integers and strings in a
c     general format or in the format provided by the user.
c **********************************************************************
c
c                       KEYWORDS
c
c    print
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer  mesnum, nchar, nints, nreal, intarr(nints)
      double precision realarr(nreal)
      character *(*) chararr(nchar)

c     chararr   i    Array of character strings containing the strings that
c                    must be substituted in the print format.
c     intarr    i    Array of integers containing the integers that
c                    must be substituted in the print format.
c     mesnum    i    Sequence number of the type of format
c                    If 1 <= mesnum <= 100 isprint calls the user print
c                    subroutine isuserpr, in which the user may define
c                    his own output formats.
c                    For mesnum > 100 the standard ISNaS print subroutine
c                    isgenpr is called, which prints according to predefined
c                    formats. The relation between mesnum and predefined
c                    format is given in isgenpr
c     nchar     i    Number of strings in a message
c     nints     i    Number of integers in a message
c     nreal     i    Number of reals in a message
c     realarr   i    Array of reals containing the reals that
c                    must be substituted in the print format.
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisparal'
      include 'IScommon/ciscom'
      include 'IScommon/cismessc'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
       integer i, ireq(4)

c     i         General loop variable
c     ireq      array of send-requests
c **********************************************************************
c
c                       I/O
c
c     If the program runs at the host the output is printed to standard
c     output, otherwise it is sent to the host
c **********************************************************************
c
c                       ERROR MESSAGES
c
c       1   mesnum has wrong value
c      22   number of strings in array chararr too large
c      23   length of strings in array chararr too large
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERRINT       Put integer value in error message file
c     ISERRSUB       Error messages
c     ISGENPR        Standard print subroutine
c     ISSENCHR       Wrapper for MPI_SEND
c     ISSENMPI       Send data to node or host
c     ISUSERPR       User print subroutine
c     ISWAITALL      Wait for all sends to complete
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Check parameters
c     if ( inodenr>0 ) then
c        send all information to host
c     else ( inodenr=0 )
c        if ( mesnum>100 ) then
c           call isgenpr
c        else ( mesnum<=100 )
c           call isuserpr
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Check parameters

      if ( mesnum.le.0 ) then

c     --- mesnum < 1

         call iserrint ( mesnum, 1 )
         call iserrsub ( 1, 1, 0, 0 )

      end if
      if ( nchar.gt.0 ) then

c     --- Check length of text

         if ( nchar.gt.1000 ) then
            call iserrint ( nchar, 1 )
            call iserrint ( 1000, 2 )
            call iserrsub ( 22, 2, 0, 0 )
         end if
         if ( len(chararr(1)).gt.80 ) then
            call iserrint ( len(chararr(1)), 1 )
            call iserrint ( 80, 2 )
            call iserrsub ( 23, 2, 0, 0 )
         end if

      end if

      if ( inodenr.ne.ISHOST ) then

c     --- Program runs at node
c         Put information of print message in array info

         info(1) = -1
         info(2) = mesnum
         info(3) = nints
         info(4) = nreal
         info(5) = nchar
         info(8) = iblockrf
         info(9) = inodenr
         info(10) = 1
         do i = 1, min(nchar,1000)
            chars(i) = chararr(i)
         end do

c        --- send information of error by issend to host

         call issenmpi (info, 10, ISINT, ISHOST, ICPRINT, ireq(1) )
         call issenmpi (intarr, nints, ISINT, ISHOST, ICPRINT,
     +                  ireq(2) )
         call issenmpi (realarr, nreal, ISDOUBLE, ISHOST, ICPRINT,
     +                  ireq(3) )
         call issenchr (chars, nchar*80, ISCHAR, ISHOST, ICPRINT,
     +                  ireq(4) )
         call iswaitall ( 4, ireq )

      else

c     --- Program runs at host

         if ( nblockrf.gt.1 .and. mesnum.ne.200 )
     +      write(irefwr,100) iblockrf
100      format(/'Block number ',i4,':'/)
         if ( mesnum.le.100 ) then

c        --- mesnum <= 100, call user print subroutine

            call isuserpr ( mesnum, nints, intarr, nreal, realarr,
     +                      nchar, chararr )

         else

c        --- mesnum > 100, call standard print subroutine

            call isgenpr ( mesnum, nints, intarr, nreal, realarr,
     +                     nchar, chararr )

         end if

      end if

      end
