      subroutine isrefine ( time, isiinput, isrinput, iscoor, issolut,
     +                      isitopol, isoutpg )
c ======================================================================
c
c        programmer     Kees  Kassels
c        version 1.2    date 09-06-1998 Charles: nvpoints changed to npointsv
c        version 1.1    date 02-04-1997 Guus: new call to isout01
c        version 1.0    date 21-03-1996
c
c   copyright (c) 1996-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Write cartesian solution to backing storage
c
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iscoor(*), isiinput(*), isrinput(*), isitopol,
     +        issolut(*), isoutpg(*)
      double precision time

c     iscoor    i    Array of sequence numbers (nblocks) of array coor
c     isiinput  i    Array of sequence numbers (nblocks) of array iinput
c     isitopol  i    Sequence number of array itopol
c     isoutpg   i    Array of sequence numbers (nblocks) of array outpg
c     isrinput  i    Array of sequence numbers (nblocks) of array rinput
c     issolut   i    Array of sequence numbers (nblocks) of array solut
c     time      i    Actual time of time step
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, iblock, idegfd, ipcoor, ipiinput, ipitopol,
     +        ipnsubf, iporder, ipoutpg, ipsolut,  ipsubf,
     +        ipvel, isorder, isvel,  lenoutpg,
     +        lenvel, maxnsubf,  maxvpnts,
     +        nblocks, ndegfd, ndim, nfaces, ni, nj, nk, npoint,
     +        ntimlv, nvirtual, npointsv, itypepost

c     i         General loop variable
c     iblock    Block sequence number
c     idegfd    Loop variable with respect to a given unknown
c     ipcoor    Starting address of array coor in buffr
c     ipiinput  Starting address of array iinput in ibuffr
c     ipitopol  Starting address of array itopol in ibuffr
c     ipnsubf   Starting address of the nsubfaces array
c     iporder   Starting address of the order array
c     ipoutpg   Starting address of a local outpg array containing quantities
c               in the vertices
c     ipsolut   Starting address of array solut in buffr
c     ipsubf    Starting address of the subfaces array
c     ipvel     Starting address of a local array for storing Cartesian velocity
c               components at the vertices
c     isorder   Sequence number of array order
c     isvel     Pointer to a local array for storing Cartesian velocity
c               components at the vertices
c     itypepost Indicates the type of postprocessing required
c               0: standard Newton    1: linear in computational domain
c     lenoutpg  Upper bound for the length of array outpg containing
c               quantities in the vertices
c     lenvel    Upper bound for the length of the local array for
c               storing Cartesian velocities at the vertices
c     maxnsubf  Maximum number of subfaces encountered on all subfaces
c     maxvpnts  Maximum number of points per block including virtual ones
c     nblocks   Number of blocks
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     nfaces    Number of faces
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npoint    Number of points for one block
c     npointsv  Number of points in the mesh with virtual points
c     ntimlv    Number of time levels
c     nvirtual  Number of rows of virtual cells
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      integer isdpgetp, isingetp

c     ISACTIVE  Activate old permanent array
c     ISDCREAT  Create permanent space in array IBUFFR (Double precision array)
c     ISDCRPRI  Decrease priorities of all arrays in ibuffr
c     ISDELARR  Delete a permanent array
c     ISDPGETP  get the pointer corresponding to a double precision array
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISINGETP  get the pointer corresponding to an integer array
c     ISORD     Compute an ordering of blocks needed for computation
c               of the streamfunction
c     ISOUT01   interpolate quantities from centroids to vertices
c     ISOUT013  interpolate quantities from centroids to vertices 3D
c     ISOUT02   determine Cartesian velocity components in the centroids, 2D
c     ISOUT04   determine Cartesian velocity components in the centroids, 3D
c     ISRESTF   Write restart file
c **********************************************************************
c
c                       DATA STATEMENTS
c
      save isorder
      data isorder /0/
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     None
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Allocate local arrays
c     DO iblock = 1, nblocks
c        determine Cartesian velocity components in the cell-centers (isout02)
c        interpolate these velocity components to the vertices (isout01)
c        Adapt results to given boundary conditions (...in later versions)
c        interpolate the pressure unknowns to the vertices (isout01)
c        DO itrnsp = ndim+2, ndegfd
c           interpolate quantity itrnsp to the vertices (isout01)
c        ENDDO
c        IF TURBULENCE
c          DO i = 1, 3*ndim
c            fill work with specified quantities in centroids (isout06)
c            interpolate these quantities to the vertices (isout01)
c          ENDDO
c        END IF
c     ENDDO
c     Delete local arrays
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen('isrefine')

c     --- compute an array containing an ordering of blocks needed for
c         computation of the streamfunction

      if ( isorder.eq.0 ) call isord(isitopol, isorder)

c     --- determine nblocks

      call isactive(isitopol)
      ipitopol     = isingetp(isitopol)
      nblocks      = ibuffr(ipitopol)
      nfaces       = ibuffr(ipitopol + 1)
      maxnsubf = ibuffr(Ipitopol + 2)

c     --- compute some general parameters

      call isactive(isiinput(1))
      ipiinput   = isingetp(isiinput(1))
      call isactive(isrinput(1))
      ndegfd     = ibuffr(ipiinput + 55 - 1)
      ndim       = ibuffr(ipiinput + 57 - 1)
      ntimlv     = ibuffr(ipiinput + 64 - 1)
      itypepost  = ibuffr(ipiinput + 67 - 1)
      maxvpnts   = ibuffr(ipiinput + 69 - 1)
      nvirtual   = ibuffr(ipiinput + 54 - 1)

c     --- compute dimensions of the local arrays

      lenvel    = maxvpnts*ndim

c     --- create arrays

      isvel    = 0
      call isdcreat(isvel   , lenvel   , 'vel'    )

c     --- write output for the different blocks
c         Note: a special ordering of blocks is used for computation of
c         the streamfunction

      do i = 1, nblocks

c     --- compute block number

         call isactive(isorder)
         iporder = isingetp(isorder)
         iblock = ibuffr(iporder + i - 1)

c        --- determine subdomain parameters

         call isactive(isiinput(iblock))
         ipiinput = isingetp(isiinput(iblock))
         ni       = ibuffr(ipiinput + 51 - 1)
         nj       = ibuffr(ipiinput + 52 - 1)
         nk       = ibuffr(ipiinput + 53 - 1)
         npoint  = (ni+1)*(nj+1)*(nk+1)
         if ( ndim.eq.2 ) then
            npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)
         else
            npointsv = (ni+1+2*nvirtual)*(nj+1+2*nvirtual)*
     +                 (nk+1+2*nvirtual)
         end if

c        --- create arrays

         isoutpg(iblock) = 0
         lenoutpg = ndegfd * npoint
         call isdcreat(isoutpg(iblock), lenoutpg, 'outpg' )

c        --- activate arrays

         call isactive(iscoor(iblock))
         call isactive(isiinput(iblock))
         call isactive(isitopol)
         call isactive(isorder)
         call isactive(issolut(iblock))
         call isactive(isvel)

c        --- determine pointers

         ipcoor   = isdpgetp(iscoor(iblock))
         ipiinput = isingetp(isiinput(iblock))
         ipitopol = isingetp(isitopol)
         ipnsubf  = ibuffr(ipitopol+5-1) + ipitopol - 1
         ipsubf   = ibuffr(ipitopol+6-1) + ipitopol - 1
         iporder  = isingetp(isorder)
         ipoutpg  = isdpgetp(isoutpg(iblock))
         ipsolut  = isdpgetp(issolut(iblock)) +
     +                 (ntimlv-2)*npointsv*ndegfd
         ipvel    = isdpgetp(isvel)

         if ( ndim .eq.2 ) then

c        --- determine Cartesian velocity components in the centroids

            call isout02( ni, nj, nvirtual,
     +               buffer(ipcoor), buffer(ipsolut), buffer(ipvel))

c           --- interpolate the Cartesian velocity components to the vertices

            call isout01 ( iblock, ni, nj, nvirtual,
     +                     nblocks, nfaces, maxnsubf,
     +                     ibuffr(ipnsubf), ibuffr(ipsubf),
     +                     buffer(ipcoor), buffer(ipvel),
     +                     buffer(ipoutpg), itypepost )

            call isout01 ( iblock, ni, nj, nvirtual,
     +                     nblocks, nfaces, maxnsubf,
     +                     ibuffr(ipnsubf), ibuffr(ipsubf),
     +                     buffer(ipcoor),
     +                     buffer(ipvel+npointsv),
     +                     buffer(ipoutpg+npoint), itypepost )

c           --- interpolate pressure to the vertices

            idegfd = ndim+1
            call isout01 ( iblock, ni, nj, nvirtual,
     +                      nblocks, nfaces, maxnsubf,
     +                      ibuffr(ipnsubf), ibuffr(ipsubf),
     +                      buffer(ipcoor),
     +                      buffer(ipsolut + npointsv*(idegfd-1)),
     +                      buffer(ipoutpg+npoint*(idegfd-1)),
     +                      itypepost )

         else

c        --- determine Cartesian velocity components in the centroids

            call isout04( ni, nj, nk, nvirtual,
     +               buffer(ipcoor), buffer(ipsolut), buffer(ipvel))

c           --- interpolate the Cartesian velocity components to the vertices

            call isout013(ni, nj, nk, nvirtual,
     +                    buffer(ipcoor), buffer(ipvel),
     +                    buffer(ipoutpg))

            call isout013(ni, nj, nk, nvirtual,
     +                    buffer(ipcoor), buffer(ipvel+npointsv),
     +                    buffer(ipoutpg+npoint))

            call isout013(ni, nj, nk, nvirtual,
     +                    buffer(ipcoor), buffer(ipvel+2*npointsv),
     +                    buffer(ipoutpg+2*npoint))

c           --- pressure 3D

            idegfd = ndim+1
            call isout013(ni, nj, nk, nvirtual,
     +                    buffer(ipcoor),
     +                    buffer(ipsolut+npointsv*(idegfd-1)),
     +                    buffer(ipoutpg+npoint*(idegfd-1)))

         end if

c        --- handle the other quantities

         if ( ndim.eq.2) then

            do idegfd = ndim+2, ndegfd

c           --- interpolate quantity to the vertices

               call isout01(iblock, ni, nj, nvirtual,
     +                      nblocks, nfaces, maxnsubf,
     +                      ibuffr(ipnsubf), ibuffr(ipsubf),
     +                      buffer(ipcoor),
     +                      buffer(ipsolut+npointsv*(idegfd-1)),
     +                      buffer(ipoutpg+npoint*(idegfd-1)),
     +                      itypepost )

            end do

         else

            do idegfd = ndim+2, ndegfd

               call isout013(ni, nj, nk, nvirtual,
     +                       buffer(ipcoor),
     +                       buffer(ipsolut+npointsv*(idegfd-1)),
     +                       buffer(ipoutpg+npoint*(idegfd-1)))

            end do

         end if

      end do

c     --- write to backing storage

      call isrestf ( time, isiinput, isrinput, isitopol, isoutpg)

c     --- delete arrays

      call isdelarr(isvel)

      call iserclos('isrefine')
      call isdcrpri

      end
