      subroutine isvelbnc ( iinput, iinbc, rinbc, output, ni, nj )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 28-01-1999
c
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Put the given essential boundary conditions for the velocity into the
c    output vector (2d case)
c    At this moment the subroutine has only been programmed for zero
c    Dirichlet boundary conditions (no-slip)
c    It must be extended in the future
c **********************************************************************
c
c                       KEYWORDS
c
c     2d
c     boundary_conditions
c     velocity
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinput(*), iinbc(*), ni, nj
      double precision output(ni+1,nj+1,2), rinbc(*)

c     iinbc     i    reserved
c     iinput    i    reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     output   i/o   Cartesian components of the flow field in the vertices
c     rinbc     i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, i1, i2, icoupled, iface, bctyp(1:2), idim, iequa,
     +        ipstart, j, k, ndegfd, iprinbc, ndegfdlc, nseg, btype
      double precision bcval(1:2)

c     bctyp     A given boundary type
c     bcval     A given boundary condition value
c     btype     Type of boundary condition
c     i         General loop variable
c     i1        Lower index number along first direction of a face or edge
c     i2        Upper index number along first direction of a face or edge
c     icoupled  If 0 the momentum equations are coupled, if 1 they are not
c     idim      Loop variable used for dimension
c     iequa     General equation sequence number
c     iface     Face number
c     iprinbc   Starting address of array rinbc in array iinput
c     ipstart   Starting address
c     j         General loop variable
c     k         General loop variable
c     ndegfd    Number of degrees of freedom
c     ndegfdlc  Local number of degrees of freedom
c     nseg      Number of segments the boundary is divided in
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    For iface := 1 (1) 4 do
c       nseg := number of segments along face iface
c       For k := 1 (1) nseg do
c          btype := type of boundary condition along segment k
c          if ( btype = 4, 10 ) then
c             Store velocity in output
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Compute ndegfdlc

      iprinbc  = iinput(27)
      ndegfd   = iinput(55)
      icoupled = iinput(76)
      iequa = 1
      if ( icoupled.eq.0 ) then

c     --- Coupled approach

         ndegfdlc = ndegfd-1

      else

c     --- Decoupled approach

         ndegfdlc = ndegfd

      end if

c     --- Loop over faces

      do iface = 1, 4

         nseg = iinbc(iinbc(iface))

c        --- Loop over subfaces

         do k = 1, nseg

c        --- btype := type of boundary condition along segment k

            ipstart = iinbc(iface)+(k-1)*(2+ndegfdlc*3)
            btype = iinbc(ipstart+3)
            do idim = 1, 2
               bctyp(idim) = iinbc( ipstart +(iequa-1)*3 +3 +idim)
               if (bctyp(idim).eq.-1) then
                  bcval(idim) = rinbc( iprinbc +(iequa-1)*2 +idim)
               end if
            end do

            if ( btype.eq.4 .and. bctyp(1).eq.-1 .and.
     +           bctyp(2).eq.-1 ) then

c           --- btype = un and ut given (constant)

               if ( abs(bcval(1)).le.0d0 .and. abs(bcval(2)).le.0d0 )
     +            then

c              --- Velocity = 0

                  i1 = iinbc(ipstart+1)
                  i2 = iinbc(ipstart+2)+1
                  if ( iface.le.2 ) then

                     if ( iface.eq.1 ) then
                        i = 1
                     else
                        i = ni+1
                     end if
                     do j = i1, i2
                        output(i,j,1) = 0d0
                        output(i,j,2) = 0d0
                     end do

                  else

                     if ( iface.eq.3 ) then
                        j = 1
                     else
                        j = nj+1
                     end if
                     do i = i1, i2
                        output(i,j,1) = 0d0
                        output(i,j,2) = 0d0
                     end do

                  end if

               end if

            end if

            iprinbc = iprinbc +ndegfdlc*2

         end do

      end do
      end
