      subroutine isintmdf ( ndim, nblocks, ncurvs, iinbc, rinbc, iuser,
     +                      user, icurvinf, kmeshq, isfread, itopol,
     +                      ineigh, iworkgen, sepran, maxtranf,
     +                      maxblockf, maxturbf, indorien, nmachines )
c ======================================================================
c
c        programmer     Guus Segal
c        version 3.22   date 05-01-2000 Guus: correction iseqeq
c        version 3.21   date 20-05-1999 Kees K: iglobstr(2) = freesurf
c        version 3.20   date 19-04-1999 Guus: Extension with main_structure and
c                                             no_time_loop
c        version 3.19   date 29-03-1999 Duncan: Extension for newton
c                                               linearisation of compressible
c                                               flow
c        version 3.18   date 23-03-1999 Guus: Extension with free surface
c        version 3.17   date 18-01-1999 Guus: change of default multiblock
c        version 3.16   date 05-01-1999 Guus: remove geotype
c        version 3.15   date 21-12-1998 Guus: change of default accuracy
c        version 3.14   date 24-08-1998 Kees K: maxdim added
c        version 3.13   date 27-05-1998 Guus: Extension of iintim/rintim
c        version 3.12   date 23-02-1998 Guus: Correction cavity
c        version 3.11   date 05-01-1998 Guus: Extend ncoefs to 10
c        version 3.10   date 26-11-1997 Guus: New call to iswrtif7
c        version 3.9    date 08-10-1997 Guus: New calls to iswrtif*
c        version 3.8    date 08-09-1997 Guus: Extra keyword cavity
c        version 3.7    date 04-09-1997 Guus: Introduction coupled
c        version 3.6    date 02-09-1997 Guus: Correction collocated
c        version 3.5    date 29-07-1997 Guus: Compressible flow without
c                                             enthalpy
c        version 3.4    date 10-07-1997 Guus: Introduction collocated approach
c        version 3.3    date 10-06-1997 Guus: More letters multi_block
c        version 3.2    date 02-04-1997 Guus: replace XXXX by POSTTYPE
c        version 3.1    date 27-01-1997 Guus: extension of iinsol/rinsol
c        version 3.0    date 17-10-1996 Guus: set iindsc(1,3)=1 if mu=0;
c                                       remove maxuser
c        version 2.1    date 10-09-1996 Marcel: added ctau1, ctau2, ctau3
c                                               default
c        version 2.0    date 25-06-1996 Guus: extra parameter indorien
c
c   copyright (c) 1994-2000 "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read user input files and store result in output arrays
c    Write output arrays to intermediate file
c **********************************************************************
c
c                       KEYWORDS
c
c       read
c       read_cards
c       write
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ndim, nblocks, icurvinf(*), kmeshq(*),
     +        ncurvs, iinbc(nblocks,*), iuser(*), isfread(*),
     +        itopol(*), ineigh(*), iworkgen(*), maxtranf, maxblockf,
     +        maxturbf, indorien(*), nmachines
      double precision user(*), rinbc(nblocks,*)
      logical sepran

c     icurvinf       i    In this array information about the SEPRAN curves with
c                         respect to array IINBC is stored in the following way:
c                         icurvinf(1,icurve) contains the block sequence number
c                            of curve icurve
c                         icurvinf(2,icurve) contains the starting address of
c                            information in array IINBC of curve icurve
c                         icurvinf(3,icurve) contains the starting address of
c                            information in array RINBC of curve icurve
c                         icurvinf(4,icurve) is used as work space in order to
c                            check if a curve has already been treated or not
c     iinbc         i/o   Array IINBC is used to indicate the type of
c                         boundary conditions for each segment at the
c                         boundary of each single block for each unknown.
c                         IINBC is defined as sub array of array IINPUT,
c                         which means that for each block there is a
c                         separate array IINBC.
c                         For a description see the programmers guide
c     indorien       i    In this array it is stored if the block orientation is
c                         positive (indorien(iblock)=1) or
c                         negative(indorien(iblock)=-1)
c     ineigh         i    Defines the connection between SEPRAN curves and the
c                         ISNaS definition of blocks, edges and sub-edges.
c                         In this array of length 10 x ncurvs for each curve the
c                         following information is stored:
c                         ineigf(1,i) first surface number curve i is lying in
c                         ineigf(2,i) corresponding edge sequence number
c                         ineigf(3,i) corresponding sub-edge sequence number
c                         ineigf(4-5,i) Parameters i1 and i2 with respect to
c                                       face
c                         ineigh(6-10,i) contain the same information for the
c                                      secondsurface
c                         A zero means no surface found
c                         A negative sign denotes the reversed direction
c     isfread        i    List of read subfaces. isfread(1-10, 1:nsfread) are
c                         valid.
c                         isfread(1, i) = block number for the i-th read subface
c                         isfread(2, i) = face number for the i-th read subface
c                         isfread(3, i) = subface number for the i-th read
c                                         subface
c                         isfread(4, i) = icurve for Sepran grid generation
c                                         0 otherwise
c                         isfread(5-10,i) = undefined
c     itopol         i    The itopol array stores the connections between
c                         the blocks. The basic idea behind the itopol
c                         data-structure is that the blocks communicate
c                         through common sub-faces.
c     iuser          i    Integer user array of length niuser
c     iworkgen       i    General integer work array.
c                         See pseudo code for the contents
c     kmeshq         i    SEPRAN array KMESH part q. Contains information about
c                         curves of curves. See the SEPRAN Programmers Guide.
c     maxblockf      i    Declared length of maxblocki in main program
c     maxtranf       i    Declared length of maxtrans in main program
c     maxturbf       i    Declared length of maxturb in main program
c     nblocks        i    Number of blocks
c     ncurvs         i    Number of curves (SEPRAN environment)
c     ndim           i    Dimension of space (2 or 3)
c     nmachines      i    Number of nodes that are used
c     rinbc         i/o   array RINBC is used to store constants for the
c                         computation of boundary conditions for each segment at
c                         the boundary of each single block for unknown.
c                         The contents of RINBC are defined by IINBC
c                         For a description see the programmers guide
c     sepran         i    If true the grid is created by SEPRAN
c     user           i    Real user array of length nuser
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cseprd'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ipnsubfaces, ipsubfaces,
     +        maxblocki, maxtrans, maxcoefs, maxdeg, maxnsubf, ndegfd,
     +        neq, nfaces, nmain, maxneq, maxturb, maxdim, maxclust,
     +        lncluster
      parameter ( maxblocki = 100, maxtrans=100, maxcoefs=25,
     +            maxturb=10, maxdeg=4+maxtrans+maxturb, nmain=28,
     +            maxneq=25, maxdim=3, maxclust=20, lncluster=300 )
      integer icommon(100), iincom(100), iincav(100),
     +        idimens(100), ichoice(100), iincof(maxcoefs,maxtrans+1),
     +        iinsol(25,maxtrans+2+maxturb), iintim(100), iintur(100),
     +        iindsc(100,maxtrans+1+maxturb), i, j, iretur, irefsv, ih,
     +        iinblk(10,maxtrans+3+maxturb), niuser,
     +        iincnd(maxdeg), ncode(1), itype(1), nout, nmcode(nmain),
     +        nncode, iretur2, ntrnsp, icoupled,
     +        nstrng, ncodmx(1), icheckeq(maxtrans+4+maxturb),
     +        iseqeq(maxtrans+2+maxturb), iinfrsur(100),
     +        iglobstr(50+20*maxclust), nclusters, icluster(lncluster),
     +        itypebc(4*(maxtrans+maxturb+maxdim)), nuser, istation,
     +        iclustern(lncluster), ifreqout(20), iinprofile(10)
      double precision dcommon(100), rincof(maxcoefs,maxtrans+1),
     +                 rinsol(25,maxtrans+2+maxturb), rintim(100),
     +                 rintur(100), rindsc(100,maxtrans+1+maxturb),
     +                 rinblk(10,maxtrans+2+maxturb), rincav(100),
     +                 rincnd(maxdeg), table(1), tabmin(1),
     +                 tabmax(1), tabout(1), rincom(100) ,
     +                 rtypebc(3*(maxtrans+maxturb+maxdim)),
     +                 rinfrsur(100), rglobstr(10+10*maxclust),
     +                 rinprofile(10)
      character * 29 codemn(nmain)
      character * 1  strout(1), code(1)
      logical transpor, timeon, output, discret, timeint, profile,
     +        bouncond, coeff, linear, initial, postfnd, mainkey,
     +        turb, station, multibl, computer, ready, compress, novisc,
     +        cavity, multiphs, freesurf, globstruc, notimloop, freqout

c     bouncond       If true boundary conditions have been read
c     cavity         If true cavity has been read
c     code           Table containing the subkeywords corresponding to this
c                    subroutine.
c     codemn         Table of main keywords
c     coeff          If true coefficients have been read
c     compress       If true compressible has been read
c     computer       If true number of computers has been read
c     dcommon        Contains all real values in the common block CISNAS
c     discret        If true information about the discretization has been
c                    read
c     freesurf       If true information about the free surface has been
c                    read
c     freqout        If true information about frequent ouput has been read
c     globstruc      If true information about the structure of the
c                    program has been read
c     i              General loop variable
c     icheckeq       Work array to indicate if an equation has been
c                    considered (icheckeq(i)=1 or not icheckeq(i)=0)
c     ichoice        Integer choice parameters for the intermediate file
c     icluster       Array in which information about the clusters is
c                    stored
c                    icluster(i), i=1(1)nclusters contains the starting
c                    position of information about cluster i
c                    In this starting position the number of equations is
c                    stored, followed by the equation sequence numbers
c     iclustern      Rearranged array icluster
c     icommon        Contains all integer values in the common block CISNAS
c     icoupled       If 0 the momentum equations are coupled,
c                    if 1 they are not
c     idimens        Integer dimension parameters for the intermediate file
c     iglobstr       Contains information of the global structure of
c                    the program
c                    See the programmers Guide for a description
c     ih             Help variable to store a constant temporarily
c     iinblk         Array IINBLK contains information about the
c                    multi-block algorithm. IINBLK is an integer array
c                    of size (3+NTRANS)10
c                    See the Programmers Guide for a description
c     iincav         Array IINCAV is part of array IINPUT and hence
c                    coupled to one block only. It contains information with
c                    respect to the cavity model
c                    See the Programmers Guide for a description
c     iincnd         Array IINCND is part of array IINPUT and hence
c                    is coupled to one block only.
c                    Contents:
c                    One dimensional array of length NDEGFD. In this
c                    array the user must put information of the initial
c                    condition of all solution vectors that are stored
c                    in array SOLUT.
c                    IINCND(i) refers to the i-th solution array in SOLUT
c                    See the Programmers Guide for a description
c     iincof         Array IINCOF consists of a series of
c                    one-dimensional arrays of length NCOEFS.
c                    For each equation NCOEFS positions are used, in
c                    the case of coupled momentum equations these
c                    equations are counted for one equation only. The
c                    sequence of the storage in IINCOF is the same as
c                    for the unknowns, i.e. first the momentum
c                    equations, then the first transport equation
c                    et-cetera.
c                    For a description see the programmers guide
c     iincom         Array IINCOM is part of array IINPUT and hence
c                    coupled to one block only. It contains information
c                    with respect to the compressible flow
c                    For a description see the programmers guide
c     iindsc         integer information about type of discretization used
c                    IINDSC may be considered as a two-dimensional array of
c                    size 10 x (NTRANS+1): IINDSC(i,j), where j refers to
c                    the equation to be solved.
c                    j=1 refers to the momentum equation, j>1 to the
c                    (j-1)-th transport equation including the
c                    turbulence equations.
c                    For a description see the programmers guide
c     iinfrsur       Contains integer input with respect to the free
c                    surface computation
c     iinprofile     ?
c     iinsol         Array IINSOL is part of array IINPUT and hence is
c                    coupled to one block only. For each equation to be
c                    solved different information may be required. For
c                    that reason IINSOL may be considered as a 25 times
c                    (2+NTRANS) array, where IINSOL(.,j) corresponds to
c                    the j-th equation. (j=1, momentum equation, j=2,
c                    pressure equation, j>2:  transport equation.
c                    See the Programmers Guide for a description
c     iintim         Contains information of the time integration
c                    See the programmers Guide for a description
c     iintur         Array IINTUR is part of array IINPUT and hence to
c                    coupled to one block only.
c                    It contains information about the turbulence modelling
c                    See the Programmers Guide for a description
c     initial        If true information about the initial conditions has
c                    been read
c     ipnsubfaces    Starting address of the nsubfaces array in IBUFFR
c     ipsubfaces     Starting address of the subfaces array in IBUFFR
c     irefsv         Help variable to save the value of a reference number
c     iretur         Return code, indicates the sequence number of the new
c                    main keyword found
c                    If -1 an error has occurred
c                    If -2 the end of file has been reached
c     iretur2        Saved value of iretur
c     iseqeq         Array with sequence numbers of equations
c     istation       Choice whether a stationary solution is needed
c     itype          Array containing the type of actions to be undertaken
c                    for each subkeyword found.
c                    See subroutine isintx02 for the possible values
c                    of itype
c     itypebc        Contains the part itype in array iinbc corresponding
c                    to one block and one segment
c     j              General loop variable
c     linear         If true information about the linear solver has been
c                    read
c     lncluster      Maximum length of array icluster
c     mainkey        If true a main keyword has been found
c     maxblocki      Maximum number of blocks
c     maxclust       Maximum number of clusters
c     maxcoefs       Maximum number of coefficients allowed
c     maxdeg         Maximum number of degrees of freedom allowed
c     maxdim         Highest physical space dimension for which ISNaS
c                    can work
c     maxneq         Maximum number of equations allowed
c     maxnsubf       Maximum number of subfaces encountered on all subfaces
c     maxtrans       Maximum number of transport equations allowed
c     maxturb        Maximum number of turbulence equations allowed
c     multibl        If true information about the multi-block process has
c                    been read
c     multiphs       If true multi phase gas flow has been found
c     nclusters      Number of clusters
c     ncode          Array containing the number of significant characters
c                    for each keyword in code
c     ncodmx         Array containing the maximum number of characters for
c                    each keyword in code
c     ndegfd         Number of degrees of freedom
c     neq            Number of equations
c     nfaces         Number of faces
c     niuser         Number of parameters stored in array iuser
c     nmain          Number of keywords in array codemn
c     nmcode         Array containing the number of significant characters
c                    for each main keyword in codemn
c     nncode         Number of subkeywords stored in array code
c     notimloop      If true he keyword no_time_loop has been read
c     nout           Length of array tabout
c     novisc         If true there is no viscosity or diffusion
c     nstrng         Length of array strout
c     ntrnsp         Number of transport equations
c     nuser          Number of parameters stored in array user
c     output         Cartesian components of the flow field in the vertices
c     postfnd        If true information about the type of post processing
c                    has been read
c     profile        If true information about the profile has been read
c     ready          Indicates if a while loop is finished or not
c     rglobstr       Contains information of the global structure of
c                    the program
c                    See the programmers Guide for a description
c     rinblk         Array RINBLK contains information about the
c                    multi-block algorithm. RINBLK is a real array of
c                    size (2+NTRANS)10
c                    See the Programmers Guide for a description
c     rincav         Array RINCAV is part of array RINPUT and hence is
c                    coupled to one block only.
c                    For a description see the programmers guide
c     rincnd         Array RINCND is part of array RINPUT and hence is
c                    coupled to one block only. It has the same
c                    structure as array IINCND. It contains information
c                    about the initial conditions
c                    See the Programmers Guide for a description
c     rincof         in rincof real information about the coefficients
c                    is stored. The contents is defined by iincof.
c                    For a description see the programmers guide
c     rincom         Array RINCOM is part of array IINPUT and hence
c                    coupled to one block only. It contains information
c                    with respect to the compressible flow
c                    For a description see the programmers guide
c     rindsc         Array RINDSC is part of array RINPUT and hence is
c                    coupled to one block only. It has the same
c                    structure as array IINDSC. It contains information
c                    about the discretization
c                    See the Programmers Guide for a description
c     rinfrsur       Contains real input with respect to the free surface
c                    computation
c     rinprofile     Contains real input with respect to the profile
c                    See the Deft programmers Guide for a description
c     rinsol         Array RINSOL is part of array RINPUT and hence
c                    is coupled to one block only. It has the same
c                    structure as array IINSOL. It contains information
c                    for the linear solver
c                    See the Programmers Guide for a description
c     rintim         Contains information of the time integration
c                    See the programmers Guide for a description
c     rintur         Array RINTUR is part of array RINPUT and hence is
c                    coupled to one block only. At this moment RINTUR
c                    is filled with fixed constants, however, since in
c                    practice these turbulence constants may depend on
c                    the type of model to be solved, it has been
c                    decided to let RINTUR be part of RINPUT. In this
c                    way it may be possible in the future to adapt the
c                    series of constants in the pre-processing phase.
c                    For a description see the programmers guide
c     rtypebc        Contains the part in array rinbc corresponding to
c                    one block and one segment
c     station        If true information whether the solution is stationary
c                    has been read
c     strout         Contains series of strings, see itype
c     table          Array containing the real or integer values
c                    corresponding to the subkeywords in code that
c                    have been read by isintx02
c     tabmax         Array containing the maximal values of the parameters
c                    read corresponding to the subkeywords in code
c     tabmin         Array containing the minimum values of the parameters
c                    read corresponding to the subkeywords in code
c     tabout         Array of output reals provided by isintx02
c     timeint        If true information about the time integration
c                    has been read
c     timeon         If true TIME ON or OFF has been read
c     transpor       If true information about the transport equations
c                    has been read
c     turb           If true information about the turbulence equations
c                    has been read
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISASKNXT       Read next item from the standard input file
c     ISCHKBND       Check if all boundary conditions have been given
c     ISERCLOS       Resets old name of previous subroutine of higher level
c     ISEROPEN       Produces concatenated name of local subroutine
c     ISERRCHR       Puts values of strings in character error array
c     ISERREAL       Put real value in error message
c     ISERRINT       Put integer value in error message
c     ISERRSUB       Error messages
c     ISFILLSTR      Fill the global and local structure arrays
c     ISINTX02       Interpret a record containing keywords followed by numbers
c     ISRDBOUN       Read information about the boundary conditions
c     ISRDCAV        Read information about the cavity model
c     ISRDCOEF       Read information about the coefficients
c     ISRDCOMP       Read information about the compressibility
c     ISRDDISC       Read information about the discretization
c     ISRDFREE       Read information about free surfaces
c     ISRDINIT       Read information about the initial condition
c     ISRDLINS       Read information about the linear solver
c     ISRDMULT       Read information about the multi-block process
c     ISRDSTRUC      Read part of input files with respect to the structure of t
c                    main program
c     ISRDTIME       Read information about the time integration
c     ISRDTURB       Read information about the turbulence equations
c     ISWARSUB       Print warnings
c     ISWRTIF0       Write first part of intermediate file
c     ISWRTIF1       Write second part of intermediate file
c     ISWRTIF2       Write third part of intermediate file
c     ISWRTIF3       Write fourth part of intermediate file
c     ISWRTIF4       Write fifth part of intermediate file
c     ISWRTIF5       Write sixth part of intermediate file
c     ISWRTIF6       Write seventh part of intermediate file
c     ISWRTIF7       Write eighth part of intermediate file
c     ISWRTIF8       Write ninth part of intermediate file
c     ISWRTIF9       Write tenth part of intermediate file
c **********************************************************************
c
c                       I/O
c
c    This subroutine reads input from the standard input file
c    The following input is recognised:
c
c      number_of_transport_equations = n
c      timeon
c      timeoff
c      output_level = n
c      discretization followed by options
c      time_integration followed by options
c      boundary_conditions followed by options
c      coefficients followed by options
c      linear_solver followed by options
c      initial_conditions followed by options
c      posttype = n
c      turbulence followed by options
c      compressible followed by options
c      cavity followed by options
c      stationary
c      multi_block followed by options
c      number_of_computers = n
c      collocated_grid
c      staggered_grid
c      multi_phase_gas_flow
c      free_surface_flow
c      no_time_loop
c      main_structure
c      end_input
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     521   Number of machines larger than number of blocks
c     528   No integer number has been found after the keyword %
c     529   Keyword found more than once
c     530   integer number read out of range
c     584   maxtrans not equal to maxtranf
c     585   maxblocki not equal to maxblockf
c     587   two keywords found that are mutually exclusive
c     589   maxturb not equal to maxturbf
c     590   mandatory keyword not found
c     597   number of equations exceeds the maximum number
c     599   The subdomain solution accuracy is too low in comparison
c           with the multi-block accuracy
c     600   ILUD preconditioning in combination with
c           'subdomain_solution = ilu'
c     609   The combination staggered approach and decoupled solution of the
c           momentum equations has not yet been implemented
c     610   The combination collocated approach and coupled solution of the
c           momentum equations has not yet been implemented
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Put default values in arrays to be written
c     main_keyword := true
c     ready := false
c     while ( not ready ) do
c        if ( main_keyword ) then
c           read next main keyword possibly followed by a number
c           main_keyword := false
c        if ( main_keyword = NUMBER_OF_TRANSPORT_EQUATIONS ) then
c           put n in idimens(3)
c           main_keyword := true
c        else if ( main_keyword = TIMEON/TIMEOFF ) then
c           set itime
c           main_keyword := true
c        else if ( main_keyword = OUTPUT_LEVEL ) then
c           set ioutpt
c           main_keyword := true
c        else if ( main_keyword = DISCRETIZATION ) then
c           read information about discretization in arrays IINDSC and RINDSC
c               until main keyword has been found
c        else if ( main_keyword = TIME_INTEGRATION ) then
c           read information about time integration in arrays IINTIM and RINTIM
c               until main keyword has been found
c        else if ( main_keyword = BOUNDARY_CONDITIONS ) then
c           read information about boundary conditions in arrays IINBC
c               and RINBC until main keyword has been found
c        else if ( main_keyword = COEFFICIENTS ) then
c           read information about coefficients in arrays IINCOF and RINCOF
c               until main keyword has been found
c        else if ( main_keyword = LINEAR_SOLVER ) then
c           read information about linear solver in arrays IINSOL and RINSOL
c               until main keyword has been found
c        else if ( main_keyword = INITIAL_CONDITIONS ) then
c           read information about initial conditions in arrays IINCND
c               and RINCND until main keyword has been found
c        else if ( main_keyword = POSTTYPE ) then
c           set ichoice(2)
c           main_keyword := true
c        else if ( main_keyword = TURBULENCE ) then
c           read information about the turbulence in arrays IINTUR and RINTUR
c               until main keyword has been found
c        else if ( main_keyword = COMPRESSIBLE ) then
c           read information about the compressibility in arrays
c               IINCOM and RINCOM until main keyword has been found
c        else if ( main_keyword = CAVITY ) then
c           read information about the cavity model in arrays IINCAV and RINCAV
c               until main keyword has been found
c        else if ( main_keyword = STATIONARY ) then
c           skip action
c           main_keyword := true
c        else if ( main_keyword = MULTI_BLOCK ) then
c           read information about the multi_block process in arrays IINBLK
c               and RINBLK until main keyword has been found
c        else if ( main_keyword = NUMBER_OF_COMPUTERS ) then
c           put n in idimens(6)
c           main_keyword := true
c        else if ( main_keyword = COLLOCATED_GRID of STAGGERED_GRID ) then
c           put n in idimens(6)
c           main_keyword := true
c        else if ( main_keyword = FREE_SURFACE_FLOW ) then
c           read information about free surface flow in arrays
c               I/RINFRSUR until main keyword has been found
c        else if ( main_keyword = NO_TIME_LOOP ) then
c           set parameter
c           main_keyword := true
c        else if ( main_keyword = main_structure ) then
c           read information about the structure of the program in arrays
c               I/RGLOBSTRUCTURE until main keyword has been found
c        else if ( main_keyword = END_INPUT ) then
c           ready := true
c        else if ( end of file ) then
c           ready := true
c        else ( keyword not recognized ) then
c           error
c           main_keyword := true
c     Check if mandatory keywords have been found
c     Extend structure arrays with information from other arrays
c     write arrays to intermediate file
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data codemn / 'NUMBER_OF_TRANSPORT_EQUATIONS', 'TIMEON',
     +              'TIMEOFF', 'OUTPUT_LEVEL', 'GEOTYPE',
     +              'DISCRETIZATION', 'TIME_INTEGRATION',
     +              'BOUNDARY_CONDITIONS','COEFFICIENTS',
     +              'LINEAR_SOLVER', 'INITIAL_CONDITIONS',
     +              'POSTTYPE','TURBULENCE', 'COMPRESSIBLE',
     +              'STATIONARY', 'MULTI_BLOCK', 'NUMBER_OF_COMPUTERS',
     +              'COLLOCATED_GRID', 'STAGGERED_GRID', 'CAVITY',
     +              'MULTI_PHASE_GAS_FLOW', 'FREE_SURFACE_FLOW',
     +              'NO_TIME_LOOP', 'MAIN_STRUCTURE', 'PROFILE_INPUT',
     +              'FREQUENT_OUTPUT', 'FULLY_COMPRESSIBLE',
     +              'END_INPUT' /
      data nmcode / 14, 6,  7,  8,  3,   7, 10,  4,  4,  4,
     +               4, 7,  4,  7,  4,  9,  14,  5,  5,  4,
     +               5, 9,  7,  8,  6,  8,  12,  3 /

c     codemn   Table containing the main keywords
c              Contents:
c              1:  NUMBER_OF_TRANSPORT_EQUATIONS
c              2:  TIMEON
c              3:  TIMEOFF
c              4:  OUTPUT_LEVEL
c              5:  GEOTYPE             superfluous
c              6:  DISCRETIZATION
c              7:  TIME_INTEGRATION
c              8:  BOUNDARY_CONDITIONS
c              9:  COEFFICIENTS
c             10:  LINEAR_SOLVER
c             11:  INITIAL_CONDITIONS
c             12:  POSTTYPE
c             13:  TURBULENCE
c             14:  COMPRESSIBLE
c             15:  STATIONARY
c             16:  MULTI_BLOCK
c             17:  NUMBER_OF_COMPUTERS
c             18:  COLLOCATED_GRID
c             19:  STAGGERED_GRID
c             20:  CAVITY
c             21:  MULTI_PHASE_GAS_FLOW
c             22:  FREE_SURFACE_FLOW
c             23:  NO_TIME_LOOP
c             24:  MAIN_STRUCTURE
c             25:  PROFILE_INPUT
c             26:  FREQUENT_OUTPUT
c             27:  FULLY_COMPRESSIBLE
c             28:  END_INPUT
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isintmdf' )

c     --- Check parameters maxtrans and maxblocki

      if ( maxtrans.ne.maxtranf ) then
         call iserrint ( maxtrans, 1 )
         call iserrint ( maxtranf, 2 )
         call iserrsub ( 584, 2, 0, 0 )
      end if
      if ( maxblocki.ne.maxblockf ) then
         call iserrint ( maxblocki, 1 )
         call iserrint ( maxblockf, 2 )
         call iserrsub ( 585, 2, 0, 0 )
      end if
      if ( maxturb.ne.maxturbf ) then
         call iserrint ( maxturb, 1 )
         call iserrint ( maxturbf, 2 )
         call iserrsub ( 589, 2, 0, 0 )
      end if

c     --- Set logicals false

      transpor = .false.
      timeon   = .false.
      output   = .false.
      discret  = .false.
      timeint  = .false.
      bouncond = .false.
      coeff    = .false.
      linear   = .false.
      initial  = .false.
      postfnd  = .false.
      turb     = .false.
      station  = .false.
      multibl  = .false.
      computer = .false.
      compress = .false.
      cavity   = .false.
      multiphs = .false.
      freesurf = .false.
      globstruc = .false.
      notimloop = .false.
      profile   = .false.
      freqout   = .false.
      istation = 0

c     --- Put default values in arrays to be written

      do i = 1, 100
         icommon(i) = 0
         idimens(i) = 0
         ichoice(i) = 0
         iincom(i) = 0
         iincav(i) = 0
         iintim(i) = 0
         iintur(i) = 0
         iinfrsur(i) = 0
         dcommon(i) = 0d0
         rincom(i) = 0d0
         rincav(i) = 0d0
         rintim(i) = 0d0
         rintur(i) = 0d0
         rinfrsur(i) = 0d0
      end do
      do i = 1, 50+20*maxclust
         iglobstr(i) = 0
      end do
      do i = 1, 10+10*maxclust
         rglobstr(i) = 0d0
      end do
      idimens(1) = 2
      idimens(2) = ndim+1
      idimens(4) = 2
      idimens(5) = max(10,3*ndim)
      idimens(7) = 1
      ichoice(1) = 2
      ichoice(3) = -1
      iintim(1) = 1
      iintim(6) = 1
      iintim(7) = 0
      iintim(8) = 1
      rintim(3) = 1d0
      rintim(4) = 0.1d0
      rintim(5) = 1d0
      rintim(6) = 1d0
      rintim(7) = 0.1d0
      do i = 19, 30
         rintim(i) = 1d-4
      end do
      rintim(41) = 0.1d0
      rintim(51) = 1d0
      rintur(1) = 0.41d0
      rintur(2) = 9d0
      rintur(3) = 0.09d0
      rintur(4) = 1d0
      rintur(5) = 1.3d0
      rintur(6) = 1.44d0
      rintur(7) = 1.92d0
      rintur(8) = 0.08d0
      rintur(9) = 2d0
      rintur(10) = 26d0
      rintur(11) = 33d0
      rintur(12) = 4.38d0
      rintur(13) = 0.012d0
      rintur(14) = 0.1d0
      rintur(15) = 0.5555556d0
      rintur(16) = 0.09d0
      rintur(17) = 0.075d0
      rintur(18) = 0.5d0
      rintur(19) = 0.5d0
      rintur(20) = 0d0
      rintur(21) = 0.05d0
      rintur(22) = 0d0
      rintur(23) = 0d0
      rintur(24) = 0d0

      do i = 1, maxtrans+1
         do j = 1, maxcoefs
            iincof(j,i) = -1
            rincof(j,i) = 0d0
         end do
      end do
      rincof(1,1) = 1d0
      rincof(2,1) = 1d0

      do i = 2, maxtrans+1
         rincof(1,i) = 1d0
         rincof(4,i) = 1d0
         rincof(6,i) = 1d0
         rincof(9,i) = 1d0
      end do

      do i = 1, maxtrans+maxturb+2
         do j = 1, 25
            iinsol(j,i) = 0
            rinsol(j,i) = 0d0
         end do
         if ( ndim.eq.2 ) then
            iinsol(2,i) = -3
         else
            iinsol(2,i) = -2
         end if
         iinsol(4,i) = 2
         iinsol(6,i) = 200
         iinsol(7,i) = 2
         iinsol(8,i) = 6
         iinsol(11,i) = 40
         iinsol(12,i) = 20
         iinsol(18,i) = 1
         iinsol(19,i) = 1
         iinsol(20,i) = 1
         iinsol(23,i) = 1
         iinsol(24,i) = 1
         rinsol(2,i) = -1d0
         rinsol(7,i) = -1d0
         rinsol(8,i) = 0.7d0
      end do
      iinsol(2,1) = 2
      iinsol(11,1) = 20
      iinsol(12,1) = 10
      iinsol(11,2) = 40
      iinsol(13,2) = 0
      rinsol(2,2) = -1d0

      do i = 1, maxdeg
         iincnd(i) = -1
         rincnd(i) = 0d0
      end do
      do i = 5+maxtrans,4+maxtrans+maxturb
         rincnd(i) = 1d-4
      end do

      do i = 1, maxtrans+maxturb+1
         do j = 1, 100
            iindsc(j,i) = 0
            rindsc(j,i) = 0d0
         end do
         iindsc(1,i) = 1
         iindsc(2,i) = 1
         iindsc(4,i) = 1
         iindsc(8,i) = 1
         iindsc(13,i) = 1
         iindsc(14,i) = 1
         iindsc(15,i) = 1
      end do
      do i = maxtrans+2, maxtrans+maxturb+1
        iindsc(5,i) = 2
      end do
      iindsc(7,1) = -1

      do j = 1, 10
         iinblk(j,1) = 0
      end do
      iinblk(2,1) = 1
      do i = 1, maxtrans+maxturb+2
         do j = 1, 10
            iinblk(j,i+1) = 0
            rinblk(j,i) = 0d0
         end do
         iinblk(1,i+1) = 0
         iinblk(2,i+1) = 100
         iinblk(3,i+1) = 20
         iinblk(4,i+1) = 1
         iinblk(6,i+1) = 1
         rinblk(1,i) = 1d-4
         rinblk(2,i) = 1d0
      end do

c     --- Initialize iseqeq

      do i = 1, maxtrans+2+maxturb
         iseqeq(i) = i
      end do

      mainkey = .true.
      ready = .false.
      istop = 2
      iretur = 0

c     --- compute quantities from itopol

      nfaces   = itopol(2)
      maxnsubf = itopol(3)
      ipnsubfaces = itopol(5)
      ipsubfaces = itopol(6)

500   if ( .not. ready ) then

c     --- while loop ready

         if ( mainkey .and. iretur.ne.-2 ) then

c        --- mainkey true
c            read next main keyword possibly followed by a number

            nncode  = 0
            nout    = 1
            nstrng  = 1
            call isintx02 ( code, ncode, nncode, itype, table, codemn,
     +                      nmcode, nmain, iretur, tabmin, tabmax,
     +                      tabout, nout, strout, nstrng, ncodmx )
            mainkey = .false.

         end if
         if ( iretur.eq.-2 ) then

c        --- end of file found

            ready = .true.
            go to 500

         else if ( iretur.eq.-1 ) then

c        --- error found

            ready = .true.
            go to 500

         end if

c        --- iretur > 0, main keyword found

         iretur2 = iretur
         go to ( 510, 520, 530, 540, 550, 560, 570, 580, 590, 600,
     +           610, 620, 630, 640, 650, 660, 670, 680, 680, 690,
     +           700, 710, 720, 730, 740, 750, 640, 800 ), iretur

c        --- NUMBER_OF_TRANSPORT_EQUATIONS found; read n and put in idimens(3)

510         call isasknxt
            if ( ityprd.ne.2 ) then
               call iserrchr ( codemn(iretur), 1 )
               call iserrsub ( 528, 0, 0, 1 )
            else
               ih = nint(realrd)
               if ( ih.lt.0 .or. ih.gt.maxtrans ) then

c              --- n out of range

                  call iserrchr ( codemn(iretur), 1 )
                  call iserrint ( ih, 1 )
                  call iserrint ( 0, 2 )
                  call iserrint ( 100, 3 )
                  call iserrsub ( 530, 3, 0, 1 )
                  ih = 0

               end if
               idimens(3) = ih
            end if
            if ( transpor ) then
               call iserrchr ( codemn(iretur), 1 )
               call iswarsub ( 529, 0, 0, 1 )
            end if
            transpor = .true.
            mainkey  = .true.
            go to 500

c           --- TIMEON found; set itime

520         icommon(2) = 1
            if ( timeon ) then
               call iserrchr ( codemn(iretur), 1 )
               call iswarsub ( 529, 0, 0, 1 )
            end if
            timeon = .true.
            mainkey = .true.
            go to 500

c           --- TIMEOFF found; set itime

530         icommon(2) = 0
            if ( timeon ) then
               call iserrchr ( codemn(iretur), 1 )
               call iswarsub ( 529, 0, 0, 1 )
            end if
            timeon = .true.
            mainkey = .true.
            go to 500

c           --- OUTPUT_LEVEL found; read n and put in ioutpt

540         call isasknxt
            if ( ityprd.ne.2 ) then
               call iserrchr ( codemn(iretur), 1 )
               call iserrsub ( 528, 0, 0, 1 )
            else
               ih = nint(realrd)
               if ( ih.lt.-1 .or. ih.gt.100 ) then

c              --- n out of range

                  call iserrchr ( codemn(iretur), 1 )
                  call iserrint ( ih, 1 )
                  call iserrint ( -1, 2 )
                  call iserrint ( 100, 3 )
                  call iserrsub ( 530, 3, 0, 1 )
                  ih = 2

               end if
               icommon(1) = ih
            end if
            if ( output ) then
               call iserrchr ( codemn(iretur), 1 )
               call iswarsub ( 529, 0, 0, 1 )
            end if
            output = .true.
550         mainkey = .true.
            go to 500

c           --- DISCRETIZATION found
c               read information about discretization in arrays IINDSC and
c               RINDSC
c               until main keyword has been found

560         call isrddisc ( nmain, codemn, nmcode, iretur, iindsc,
     +                      rindsc, maxtrans, maxturb, icheckeq,
     +                      iseqeq )
            if ( discret ) then
               call iserrchr ( codemn(iretur2), 1 )
               call iswarsub ( 529, 0, 0, 1 )
            end if
            discret = .true.
            go to 500

c           --- TIME_INTEGRATION found
c               read information about time integration in arrays IINTIM and
c               RINTIM until main keyword has been found

570         call isrdtime ( nmain, codemn, nmcode, iretur, iintim,
     +                      rintim )
            if ( timeint ) then
               call iserrchr ( codemn(iretur2), 1 )
               call iswarsub ( 529, 0, 0, 1 )
            end if
            timeint = .true.
            go to 500

c           --- BOUNDARY_CONDITIONS found
c               read information about boundary conditions in arrays IINBC
c               and RINBC until main keyword has been found

580          call isrdboun ( nblocks, nfaces, maxnsubf,
     +                       itopol(ipnsubfaces), itopol(ipsubfaces),
     +                       ineigh,
     +                       nmain, codemn, nmcode, iretur, iinbc,
     +                       rinbc, maxtrans, maxturb, itypebc,
     +                       ncurvs, icurvinf, kmeshq, iworkgen, ndim,
     +                       rtypebc, isfread, sepran, indorien )
            if ( bouncond ) then
               call iserrchr ( codemn(iretur2), 1 )
               call iswarsub ( 529, 0, 0, 1 )
            end if
            bouncond = .true.
            go to 500

c           --- COEFFICIENTS found
c               read information about coefficients in arrays IINCOF and RINCOF
c               until main keyword has been found

590         call isrdcoef ( nmain, codemn, nmcode, iretur, iincof,
     +                      rincof, maxtrans, maxcoefs, ndim, icheckeq )
            if ( coeff ) then
               call iserrchr ( codemn(iretur2), 1 )
               call iswarsub ( 529, 0, 0, 1 )
            end if
            coeff = .true.
            go to 500

c           --- LINEAR_SOLVER found
c               read information about linear solver in arrays IINSOL and RINSOL
c               until main keyword has been found

600         call isrdlins ( nmain, codemn, nmcode, iretur, iinsol,
     +                      rinsol, maxtrans, maxturb, icheckeq )
            if ( linear ) then
               call iserrchr ( codemn(iretur2), 1 )
               call iswarsub ( 529, 0, 0, 1 )
            end if
            linear = .true.
            go to 500

c           --- INITIAL_CONDITIONS found
c               read information about initial conditions in arrays IINCND
c               and RINCND until main keyword has been found

610         call isrdinit ( nmain, codemn, nmcode, iretur, iincnd,
     +                      rincnd, maxtrans, maxturb, ndim, icheckeq )
            if ( initial ) then
               call iserrchr ( codemn(iretur2), 1 )
               call iswarsub ( 529, 0, 0, 1 )
            end if
            initial = .true.
            go to 500

c           --- POSTTYPE found; read n and put in ichoice(2)

620         call isasknxt
            if ( ityprd.ne.2 ) then
               call iserrchr ( codemn(iretur), 1 )
               call iserrsub ( 528, 0, 0, 0 )
            else
               ih = nint(realrd)
               if ( ih.lt.0 .or. ih.gt.1 ) then

c              --- n out of range

                  call iserrchr ( codemn(iretur), 1 )
                  call iserrint ( ih, 1 )
                  call iserrint ( 0, 2 )
                  call iserrint ( 1, 3 )
                  call iserrsub ( 530, 3, 0, 1 )
                  ih = 0

               end if
               ichoice(2) = ih
            end if
            if ( postfnd ) then
               call iserrchr ( codemn(iretur), 1 )
               call iswarsub ( 529, 0, 0, 1 )
            end if
            postfnd = .true.
            mainkey = .true.
            go to 500

c           --- TURBULENCE found
c               read information about turbulence in arrays IINTUR and RINTUR
c               until main keyword has been found

630         call isrdturb ( nmain, codemn, nmcode, iretur, iintur,
     +                      rintur, idimens )
            if ( turb ) then
               call iserrchr ( codemn(iretur2), 1 )
               call iswarsub ( 529, 0, 0, 1 )
            end if
            turb = .true.
            go to 500

c           --- COMPRESSIBLE found
c               read information about compressible flows in arrays IINCOM and
c               RINCOM until main keyword has been found

640         call isrdcomp ( nmain, codemn, nmcode, iretur, iincom,
     +                      rincom )
            if ( compress ) then
               call iserrchr ( codemn(iretur2), 1 )
               call iswarsub ( 529, 0, 0, 1 )
            end if
            compress = .true.
            go to 500

c           --- STATIONARY found
c               skip action

650         mainkey = .true.
            istation = 1
            if ( station ) then
               call iserrchr ( codemn(iretur), 1 )
               call iswarsub ( 529, 0, 0, 1 )
            end if
            if ( notimloop ) then

c           --- The keywords no_time_loop and stationary are mutually exclusive

               call iserrchr ( codemn(iretur), 1 )
               call iserrchr ( codemn(23), 1 )
               call iserrsub ( 587, 0, 0, 2 )

            end if
            station = .true.
            go to 500

c           --- MULTI_BLOCK found
c               read information about the multi_block process in arrays IINBLK
c               and RINBLK until main keyword has been found

660         call isrdmult ( nmain, codemn, nmcode, iretur, iinblk,
     +                      rinblk, maxtrans, maxturb, icheckeq )
            if ( multibl ) then
               call iserrchr ( codemn(iretur2), 1 )
               call iswarsub ( 529, 0, 0, 1 )
            end if
            multibl = .true.
            iinblk(4,1) = 1
            go to 500

c           --- NUMBER_OF_COMPUTERS found
c               put n in idimens(7)

670         call isasknxt
            if ( ityprd.ne.2 ) then
               call iserrchr ( codemn(iretur), 1 )
               call iserrsub ( 528, 0, 0, 0 )
            else
               ih = nint(realrd)
               if ( ih.lt.1 .or. ih.gt.10000 ) then

c              --- n out of range

                  call iserrchr ( codemn(iretur), 1 )
                  call iserrint ( ih, 1 )
                  call iserrint ( 1, 2 )
                  call iserrint ( 10000, 3 )
                  call iserrsub ( 530, 3, 0, 1 )
                  ih = 2

               end if
               idimens(7) = ih
            end if
            mainkey = .true.
            if ( computer ) then
               call iserrchr ( codemn(iretur), 1 )
               call iswarsub ( 529, 0, 0, 1 )
            end if
            computer = .true.
            go to 500

c           --- STAGGERED_GRID or COLLOCATED_GRID found

680         mainkey = .true.
            if ( iretur.eq.18 ) ichoice(3) = 1
            go to 500

c           --- CAVITY found
c               read information about compressible flows in arrays IINCAV and
c               RINCAV until main keyword has been found

690         call isrdcav ( nmain, codemn, nmcode, iretur, iincav,
     +                     rincav )
            if ( cavity ) then
               call iserrchr ( codemn(iretur2), 1 )
               call iswarsub ( 529, 0, 0, 1 )
            end if
            cavity = .true.
            ichoice(5) = 1
            go to 500

c           --- MULTI_PHASE_GAS_FLOW found

700         if ( multiphs ) then
               call iserrchr ( codemn(iretur2), 1 )
               call iswarsub ( 529, 0, 0, 1 )
            end if
            multiphs = .true.
            mainkey = .true.
            iincom(3) = 1
            go to 500

c           --- FREE_SURFACE found
c               read information about free surface flows in arrays IINFRSUR and
c               RINFRSUR until main keyword has been found

710         call isrdfree ( nmain, codemn, nmcode, iretur, iinfrsur,
     +                      rinfrsur )
            if ( freesurf ) then
               call iserrchr ( codemn(iretur2), 1 )
               call iswarsub ( 529, 0, 0, 1 )
            end if
            freesurf = .true.
            ichoice(6) = 1
            go to 500

c           --- NO_TIME_LOOP found
c               set parameter

720         mainkey = .true.
            istation = 2
            if ( notimloop ) then
               call iserrchr ( codemn(iretur), 1 )
               call iswarsub ( 529, 0, 0, 1 )
            end if
            if ( station ) then

c           --- The keywords no_time_loop and stationary are mutually exclusive

               call iserrchr ( codemn(iretur), 1 )
               call iserrchr ( codemn(15), 1 )
               call iserrsub ( 587, 0, 0, 2 )

            end if
            notimloop = .true.
            go to 500

c           --- MAIN_STRUCTURE found
c               read information about the main structure of the program in
c               arrays I/RGLOBSTRU until main keyword has been found

730         call isrdstruc ( nmain, codemn, nmcode, iretur, iglobstr,
     +                       rglobstr, iglobstr(51), rglobstr(11),
     +                       maxclust, iseqeq, nclusters, icluster,
     +                       lncluster )
            idimens(8) = nclusters
            if ( globstruc ) then
               call iserrchr ( codemn(iretur2), 1 )
               call iswarsub ( 529, 0, 0, 1 )
            end if
            globstruc = .true.
            go to 500

c           --- PROFILE found
c               read information about the profile in
c               arrays I/RINPROFILE until main keyword has been found

740         if ( profile ) then
               call iserrchr ( codemn(iretur2), 1 )
               call iswarsub ( 529, 0, 0, 1 )
            end if
            profile = .true.
            go to 500

c           --- FREQUENT_OUTPUT found
c               read information about the frequent output in
c               array IFREQOUT until main keyword has been found

750        if ( freqout ) then
               call iserrchr ( codemn(iretur2), 1 )
               call iswarsub ( 529, 0, 0, 1 )
            end if
            freqout = .true.
            go to 500

c           --- END_INPUT found

800         mainkey = .true.
            ready = .true.
            go to 500

      end if

c     --- Check if mandatory keywords have been found

      if ( .not. timeint ) then
         call iserrchr ( codemn(7), 1 )
         call iserrsub ( 590, 0, 0, 1 )
      end if
      if ( .not. bouncond ) then
         call iserrchr ( codemn(8), 1 )
         call iserrsub ( 590, 0, 0, 1 )
      end if
      if ( .not. coeff ) then
         call iserrchr ( codemn(9), 1 )
         call iserrsub ( 590, 0, 0, 1 )
      end if

      idimens(2) = idimens(2)+idimens(3)+idimens(6)

c     --- check if neqs > maxneq

      ndegfd = idimens(2)
      neq    = ndegfd - ndim + 1
      if ( neq.gt.maxneq ) then

         call iserrint ( neq,    1 )
         call iserrint ( maxneq, 2 )
         call iserrsub ( 597, 2, 0, 0 )

      end if

c     --- copy SAVEPREC from iinblk to iinsol
c         and overwrite METHOD in IINSOL if ISOL = 2

      do i = 1, neq

         iinsol(15,i) = iinblk(3,1)
         if ( iinblk(2,1).eq.2 ) then

c        --- isol = 2 ---> method = 6

            iinsol(7,i) = 6

            if ( iinsol(2,i).eq.2 ) then

c           --- ILUD preconditioning used in combination with
c               subdomain_solution = ilu, change to postconditioning

               iinsol(2,i) = -iinsol(2,i)

               call iserrint ( i, 1 )
               call iswarsub ( 600, 1, 0, 0 )

            end if

         end if

         if ( iinblk(2,1).eq.0 .and. nblocks.gt.1 ) then

c        --- isol = 0, check if the subdomain solution accuracy is
c            large enough

            if ( rinsol(2,i).gt.rinblk(1,i) ) then

               call iserreal ( rinsol(2,i),  1 )
               call iserreal ( rinblk(1,i),  2 )
               call iserrint ( i,            1 )
               call iswarsub ( 599, 1, 2, 0 )

            end if

         end if

      end do

c     --- if ideal gas compressible flow, or cavitating flow with
c             density predictor, solve transport equation first

      if ( compress .and. iincom(4).eq.0 .or. compress.and.
     +            iindsc(8,1).eq.1 ) then
         if ( iseqeq(1).ne.0 ) then
            iseqeq(1) = 2
            iseqeq(2) = 3
         end if
         if ( iseqeq(3).ne.0 ) then
            iseqeq(3) = 1
         end if
      end if

c     --- Check if all boundary conditions have been prescribed

      call ischkbnd ( iinbc, maxtrans, maxturb, ndim, compress,
     +                sepran, icurvinf, ncurvs, nblocks, idimens,
     +                iindsc )

c     --- Skip stress or diffusion tensor depending on the value
c         of viscosity or diffusion

      if ( iincof(2,1).lt.0 .and. rincof(2,1).le.0d0 ) iindsc(3,1) = 1
      ntrnsp = idimens(3)
      do i = 1, ntrnsp
         novisc = .true.
         do j = 4, 3*ndim
            if ( iincof(j,i+1).ge.0 .or. rincof(j,i+1).gt.0d0 )
     +         novisc = .false.
         end do
         if ( novisc )  iindsc(3,i+1) = 1
      end do

c     --- Set defaults with respect to staggered/collocated

      if ( ichoice(3).le.0 ) then

c     --- Staggered approach

         ichoice(3) = 0
         if ( iindsc(7,1).eq.-1 ) then
            iindsc(7,1) = 0
         else if ( iindsc(7,1).eq.1 ) then

c        --- The combination staggered/decoupled approach has not yet been
c            implemented

            call iserrsub ( 609, 0, 0, 0 )

         end if

      else

c     --- Collocated approach

         if ( iindsc(7,1).eq.-1 ) then
            iindsc(7,1) = 1
         else if ( iindsc(7,1).eq.0 ) then

c        --- The combination collocated/coupled approach has not yet been
c            implemented

            call iserrsub ( 610, 0, 0, 0 )

         end if

      end if

c     --- At this moment staggered means coupled. collocated means decoupled

      icoupled = ichoice(3)
      ichoice(4) = icoupled

c     --- Set defaults with respect to accuracy of linear solver

      if ( nblocks.gt.1 ) then

c     --- Multiblock algorithm is used

         if ( iinblk(2,1).eq.0 ) then

c        --- iinblk(2,1) = 0, hence accurate domain decomposition
c            requires larger accuracy
c            If ( rinsol(2,.) < 0, the defualt value must be used

            if ( rinsol(2,1).lt.0d0 ) rinsol(2,1)=1d-6
            if ( rinsol(2,2).lt.0d0 ) rinsol(2,2)=1d-8
            do i = 3, maxtrans+maxturb+2
               if ( rinsol(2,i).lt.0d0 ) rinsol(2,i)=1d-6
            end do

         else

c        --- iinblk(2,1) # 0, hence inaccurate domain decomposition
c            standard accuracy

            if ( rinsol(2,1).lt.0d0 ) rinsol(2,1)=1d-4
            if ( rinsol(2,2).lt.0d0 ) rinsol(2,2)=1d-6
            do i = 3, maxtrans+maxturb+2
               if ( rinsol(2,i).lt.0d0 ) rinsol(2,i)=1d-4
            end do
         end if
      else

c     --- Single block algorithm is used
c         standard accuracy

         if ( rinsol(2,1).lt.0d0 ) rinsol(2,1)=1d-4
         if ( rinsol(2,2).lt.0d0 ) rinsol(2,2)=1d-6
         do i = 3, maxtrans+maxturb+2
            if ( rinsol(2,i).lt.0d0 ) rinsol(2,i)=1d-4
         end do

      end if
      nmachines = idimens(7)
      if ( nmachines.gt.nblocks ) then
         call iserrint(nmachines, 1)
         call iserrint(nblocks, 2)
         call iswarsub( 521, 2, 0, 0)
         nmachines = nblocks
         idimens(7) = nmachines
      end if

c     --- Extend structure arrays with defaults and information from other
c         arrays

      iglobstr(1) = istation
      if ( freesurf ) iglobstr(2) = 1
      call isfillstr ( iglobstr, rglobstr, iintim, rintim,
     +                 icluster, iclustern, idimens, lncluster,
     +                 iglobstr(51), rglobstr(11), maxclust, ichoice,
     +                 ndim )

      if ( nerror.eq.0 ) then

c     --- write arrays to intermediate file

         call iswrtif0 ( icommon, dcommon, idimens, ichoice, irefsv )
         call iswrtif1 ( idimens, maxcoefs, maxtrans, iincof, rincof,
     +                   ndim, icoupled )
         call iswrtif2 ( idimens, maxtrans, maxturb, iinsol, rinsol,
     +                   ndim, icoupled )
         call iswrtif3 ( idimens, iintim, rintim, maxtrans, maxturb,
     +                   iincnd, rincnd, ndim )
         call iswrtif4 ( idimens, maxtrans, maxturb, iindsc, rindsc,
     +                   ndim, icoupled )
         call iswrtif5 ( idimens, maxtrans, maxturb, iinblk, rinblk,
     +                   ndim, icoupled )
         call iswrtif6 ( iintur, rintur, iincom, rincom, iincav,
     +                   rincav, iinfrsur, rinfrsur, iglobstr,
     +                   rglobstr, iglobstr(51), rglobstr(11), maxclust,
     +                   iclustern, idimens )
         call iswrtif7 ( nblocks, ndim, idimens, iinbc, rinbc,
     +                   maxtrans, maxturb, icoupled )
         niuser = 0
         nuser  = 0
         call iswrtif8 ( niuser, nuser, iuser, user )
         call iswrtif9 ( idimens, maxtrans, maxturb, iseqeq, ndim,
     +                   icoupled )

c        --- Reset irefwr

         irefwr = irefsv

      end if

      call iserclos ( 'isintmdf' )
      end
