      function isintx00 ( codein, ncode, ncodein, ierpri, ncodmx )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date   27-09-94 (Subroutine extracted form the SEPRAN
c                                        subroutine intx00, version 1.1)
c
c
c
c   copyright (c) 1994  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                         DESCRIPTION
c
c   subroutine ISINTX00 may be used to check whether a text read by ISASKNXT
c   and stored in common CSEPCH, belongs to the table CODEIN
c   If this text belongs to CODEIN ISINTX00 returns with the seuqnce number
c   of this text in CODEIN, otherwise ISINTX00 gets the value 0
c   The number of characters that must be compared is stored in the
c   table NCODE
c ********************************************************************
c
c                      KEYWORDS
c
c  interpret
c  keyword
c  read_cards
c ********************************************************************
c
c                      INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer isintx00, ncodein, ncode(ncodein), ierpri, ncodmx(ncodein)
      character *(*) codein(ncodein)

c     codein    i    Table containing the texts to be compared
c                    ncodein texts have been stored in code
c     ierpri    i    Indicates if an error message must be printed if
c                    the text can not be found (1) or not (0)
c     isintx00  o    Gives index number of the text that matches the codein
c                    table. So if string(1:ncode(i)) = codein(1:ncode(i)),
c                    isintx00 gets the value i
c                    If none of the texts match, isintx00 gets the value 0
c     ncode     i    Array containing the number of significant characters for
c                    each keyword in code
c     ncodein   i    Number of texts stored in codein
c     ncodmx    i    Array containing the maximum number of characters for
c                    each keyword in code
c ********************************************************************
c
c                      COMMON BLOCKS
c
      include 'IScommon/csepch'
      include 'IScommon/cseprd'
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer i

c     i         General loop variable
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRCRD  Prints the last record read and underlines the errorneous part
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISWARSUB  Print warnings
c ********************************************************************
c
c                    I/O
c
c    none
c ********************************************************************
c
c                    ERROR MESSAGES
c
c     534   string could not be recognized
c     588   Keyword contains too much characters
c ********************************************************************
c
c                    PSEUDO CODE
c
c   trivial
c ======================================================================
c
      do 100 i = 1, ncodein
         if ( string(1:ncode(i)) .eq. codein(i) (1:ncode(i)) ) then

c        --- Text found, return

            isintx00 = i
            if ( nchar.gt.ncodmx(i) ) then

c           --- Too much characters in keyword

               call iseropen ( 'isintx00' )
               call iserrint ( nchar, 1 )
               call iserrint ( ncodmx(i), 2 )
               call iserrchr ( string(1:nchar), 1 )
               call iswarsub ( 588, 2, 0, 1 )
               call iserrcrd
               call iserclos ( 'isintx00' )
            end if
            return

         end if

100   continue

      isintx00 = 0
      if ( ierpri.eq.1 ) then

c     --- Print error message

         call iseropen ( 'isintx00' )
         call iserrchr ( string(1:ncode(1)), 1 )
         call iserrsub ( 534, 0, 0, 1 )
         call iserrcrd
         call iserclos ( 'isintx00' )

      end if
      end
