      subroutine isrdbou1 ( nblocks, nfaces, maxnsubf, subfaces,
     +                      nsfread, isfread, iinbc, rinbc, maxtrans,
     +                      maxturb, itypebc, rtypebc, ndim )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.2    date 08-09-1998 Kees K: indorien removed
c        version 2.1    date 17-09-1997 Guus: use ndim
c        version 2.0    date 25-06-1996 Guus: extra parameter indorien
c        version 1.2    date 03-04-1996 Kees K. dimension itypebc, rtypebc
c                                       plus 1 for the pressure, also   for
c                                       iinbc and rinbc
c        version 1.1    date 09-11-1994 Erik: adaption for subfaces instead of
c                                       curves
c        version 1.0    date 28-07-1994
c
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Put boundary conditions for one set of curves as stored in itypebc and
c    rtypebc into arrays iinbc and rinbc
c **********************************************************************
c
c                       KEYWORDS
c
c       boundary_conditions
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nblocks, nfaces, nsfread, maxnsubf, maxtrans, maxturb,
     +        ndim
      integer iinbc(nblocks,*)
      integer itypebc(maxtrans+maxturb+ndim,ndim+1)
      integer isfread(10, 1:nsfread)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)
      double precision rinbc(nblocks,*),
     +                 rtypebc(maxtrans+maxturb+ndim,ndim)

c     iinbc    i/o   reserved
c     isfread   i    reserved
c     itypebc   i    Contains the part itype in array iinbc corresponding to
c                    one block and one segment
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     maxtrans  i    Maximum number of transport equations allowed
c     maxturb   i    Maximum number of turbulence equations allowed
c     nblocks   i    Number of blocks
c     ndim      i    Dimension of space (2 or 3)
c     nfaces    i    Number of faces
c     nsfread   i    Number of subfaces read
c     rinbc    i/o   reserved
c     rtypebc   i    Contains the part in array rinbc corresponding to
c                    one block and one segment
c     subfaces  i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, iblock, iface, iseg, ipos, ipos1, j, k

c     i         General loop variable
c     iblock    Block sequence number
c     iface     Face number
c     ipos      Position in array
c     ipos1     Position in array
c     iseg      Counting variable: sequence number of segment
c     j         General loop variable
c     k         General loop variable
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     For all subfaces in isfread
c        iblock := block sequence number
c        iface  := face sequence number
c        iseq   := subface sequence number
c        ipos := starting address information in iinbc
c        ipos1 := starting address information in rinbc
c        Copy information from itypbc into iinbc
c        Copy information from rtypbc into rinbc
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      do i = 1, nsfread

         iblock = isfread(1,i)
         iface  = isfread(2,i)
         iseg   = isfread(3,i)
         ipos   = subfaces(iblock,iface,iseg,9)
         ipos1  = subfaces(iblock,iface,iseg,10)

         do j = 1, maxtrans+ndim+maxturb
            do k = 1, ndim+1
               iinbc(iblock,ipos+k-1+(j-1)*(ndim+1)) = itypebc(j,k)
            end do
         end do
         do j = 1, maxtrans+ndim+maxturb
            do k = 1, ndim
               rinbc(iblock,ipos1+k-1+(j-1)*ndim) = rtypebc(j,k)
            end do
         end do

      end do

      end
