      subroutine isrdboun ( nblocks, nfaces, maxnsubf,
     +                      nsubfaces, subfaces, ineigh,
     +                      nmain, codemn, nmcode, iretur, iinbc,
     +                      rinbc, maxtrans, maxturb, itypebc,
     +                      ncurvs, icurvinf, kmeshq, iworkgen, ndim,
     +                      rtypebc, isfread, sepran, indorien )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.7    date 08-09-1998 Kees K, vall of isrdbou1 without
c                                       indorien
c        version 2.6    date 02-02-1998 Guus, length itypbc
c        version 2.5    date 28-01-1998 Guus, variable length ineigh
c        version 2.4    date 27-01-1998 Guus, itype changed for periodic
c        version 2.3    date 28-10-1997 Kees K, itype changed for periodic
c                                       and call of isrdbou4
c        version 2.2    date 17-09-1997 Guus, u/v/w_momentum
c        version 2.1    date 11-07-1996 Marcel: eps_asymptote
c        version 2.0    date 25-06-1996 Guus: extra parameter indorien
c        version 1.7    date 04-06-1996 Marcel: wall_temperature
c        version 1.6    date 03-04-1996 Kees K. dimension itypebc, rtypebc
c                                       plus 1 for the pressure
c        version 1.5    date 20-03-1996 Layout
c        version 1.4    date 13-02-1996 Guus: test on boundary conditions
c        version 1.3    date 09-01-1996 Marcel Z. maxturb for initialize
c                                       itypebc rtypebc
c        version 1.2    date 10-01-1995 Kees K. 3D
c        version 1.1    date 30-11-1994 Introduce parallel_outflow
c
c   copyright (c) 1994-1998 "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read boundary conditions part of input files and store result in
c    arrays iinbc and rinbc
c **********************************************************************
c
c                       KEYWORDS
c
c       boundary_conditions
c       read
c       read_cards
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer maxnsubf, nblocks, nfaces, nmain, nmcode(nmain), iretur,
     +        maxtrans, maxturb, ndim,
     +        iinbc(nblocks,*), ncurvs, kmeshq(*),
     +        icurvinf(4,ncurvs), itypebc(maxtrans+maxturb+ndim,4),
     +        iworkgen(ncurvs), isfread(10,*), indorien(*)
      integer nsubfaces(1:nblocks, 1:nfaces)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)
      integer ineigh(*)
      double precision rinbc(nblocks,*),
     +                 rtypebc(maxtrans+maxturb+ndim,ndim)
      character *(*) codemn(nmain)
      logical sepran

c     codemn    i    Table of main keywords
c     icurvinf  i    reserved
c     iinbc     o    reserved
c     indorien  i    In this array it is stored if the block orientation is
c                    positive (indorien(iblock)=1) or
c                    negative(indorien(iblock)=-1)
c     ineigh    i    reserved
c     iretur    o    Return code, indicates the sequence number of the new
c                    main keyword found
c                    If -1 an error has occurred
c                    If -2 the end of file has been reached
c     isfread   o    reserved
c     itypebc        Contains the part itype in array iinbc corresponding to
c                    one block and one segment
c     iworkgen       General integer work array.
c                    See pseudo code for the contents
c     kmeshq    i    SEPRAN array KMESH part q. Contains information about
c                    curves of curves. See the SEPRAN Programmers Guide.
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     maxtrans  i    Maximum number of transport equations allowed
c     maxturb   i    Maximum number of turbulence equations allowed
c     nblocks   i    Number of blocks
c     ncurvs    i    Number of curves (SEPRAN environment)
c     ndim      i    Dimension of space (2 or 3)
c     nfaces    i    Number of faces
c     nmain     i    Number of keywords in array codemn
c     nmcode    i    Array containing the number of significant characters
c                    for each main keyword in codemn
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     rinbc     o    reserved
c     rtypebc        Contains the part in array rinbc corresponding to
c                    one block and one segment
c     sepran    i    If true the grid is created by SEPRAN
c     subfaces  i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cseprd'
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer nmainnw, newmain, lenchar, nncode, nncod2, nout
      parameter ( nmainnw=40, newmain=9, nncode=36, nncod2=2,
     +            lenchar=14, nout=10 )
      integer nmcode1(newmain), nmcoda(nmainnw), itype(nncode),
     +        nmcode2(nncod2), ncodmx(nncode), nstrng,
     +        ncode(nncode), i, nmain1, nmmaintot, numsub,
     +        itrans, j, nsfread
      double precision table(nncode), tabmin(nncode), tabmax(nncode),
     +                 tabout(nout)
      character *14 codema(nmainnw), code1(newmain), code2(nncod2)
      character *16 code(nncode)
      character *1 strout(1)
      logical uset

c     code      Table containing the subkeywords corresponding to this
c               subroutine.
c     code1     Table containing the  main keywords corresponding to this
c               subroutine.
c     code2     Table containing the  subsubkeywords corresponding to this
c               subroutine.
c     codema    Table consisting of the local main keywords followed by the
c               general main keywords possibly followed by the subsubkeywords
c     i         General loop variable
c     itrans    reserved
c     itype     Array containing the type of actions to be undertaken for
c               each subkeyword found.
c               See subroutine isintx02 for the possible values of itype
c     j         General loop variable
c     lenchar   Number of characters in main keywords
c     ncode     Array containing the number of significant characters for
c               each keyword in code
c     ncodmx    Array containing the maximum number of characters for
c               each keyword in code
c     newmain   Number of main keywords stored in array code1
c     nmain1    Total number of main keywords (newmain+nmain)
c     nmainnw   Maximal number of keywords in codema
c     nmcoda    Array containing the number of significant characters for
c               each keyword in codema
c     nmcode1   Array containing the number of significant characters for
c               each keyword in code1
c     nmcode2   Array containing the number of significant characters for
c               each keyword in code2
c     nmmaintot Total number of keywords in codema
c     nncod2    Number of subsubkeywords stored in array code2
c     nncode    Number of subkeywords stored in array code
c     nout      Length of array tabout
c     nsfread   Number of subfaces read
c     nstrng    Length of array strout
c     numsub    Number of subkeywords that may be detected by isintx02
c               In the first step numsub=0 for any other step numsub=nncode
c     strout    Contains series of strings, see itype
c     table     Array containing the real or integer values corresponding
c               to the subkeywords in code that have been read by isintx02
c     tabmax    Array containing the maximal values of the parameters read
c               corresponding to the subkeywords in code
c     tabmin    Array containing the minimum values of the parameters read
c               corresponding to the subkeywords in code
c     tabout    Array of output reals provided by isintx02
c     uset      If true a boundary conditions for the momentum equation has
c               been set for the present segment
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISASKNXT  Read next item from the standard input file
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRBND  Check boundary conditions in adjacent sides in 2D
c     ISERRCHR  Puts values of strings in character error array
c     ISERRCRD  Prints the last record read and underlines the erroneous part
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISINTX02  Interpret a record containing keywords followed by numbers
c     ISRDBOU1  Put contents of itypbc and rtypbc for a series of curves
c               into iinbc and rinbc
c     ISRDBOU2  Interpret the Sepran CURVE description
c     ISRDBOU3  Interpret subface description
c     ISRDBOU4  Interpret momentum boundary conditions
c     ISRDBOU5  Interpret transport boundary conditions
c     ISRDERR   give an error message for a set of subfaces
c     ISSTOP    Stop ISNAS
c **********************************************************************
c
c                       I/O
c
c    This subroutine reads the input after the main keyword BOUNDARY CONDITION
c    until a new main keyword has been found
c    The following input is recognized:
c    [curve description] [boundary conditions for velocity]
c    [boundary conditions for each transport equation]
c    [boundary conditions for turbulence quantities]
c    curve description, recognized is:
c    CURVE i TO j         Consider curves i , i+1, i+2, ... ,j
c    CURVE i              Consider curve i only
c    boundary conditions for velocity, recognized is:
c    NOSLip
c    INFlow umax
c    OUTFlow
c    PARALlel_outflow   (or alternatively OUTStream)
c    FREESlip
c    UN  = g
c    UT  = g
c    UX  = g
c    UY  = g
c    UZ  = g
c    SIGMANN = g
c    SIGMANT = g
c    WALL_Functions = s
c    ROUGHness = h
c    PERIODIC
c    PERIODIC_PRressjump, FLUX = g
c    ANTI_PERIODIC
c    ANTI_PERIODIC_PRressjump, FLUX = g
c    where g may take one of the forms:
c    number
c    FUNC = i
c    The parameter s in wall_functions may take one of the forms:
c       SMOOTH
c       ROUGH
c    TRANSport a = description
c    U_MOMENTUM = description
c    V_MOMENTUM = description
c    W_MOMENTUM = description
c    with description:
c    DIrichlet = g
c    NEumann = g
c    RObbins: ALPha=g, H=g
c    MIxed: ALPha=g, H=g
c    PERIODIC i or (i,j,k)
c    ANTI_PERIODIC
c    WALL_Temperature = g
c    Boundary conditions for the turbulent quantities.
c    The following boundary conditions are recognized:
c    K_DIrichlet = g
c    K_NEumann = g
c    EPS_DIrichlet = g
c    EPS_NEumann = g
c    EPS_ASYmptote
c    OMEGA_DIrichlet = g
c    OMEGA_NEumann = g
c    OMEGA_ASYmptote
c    EPS_ASYmptote
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     528   No integer number has been found after the keyword %
c     530   integer number read out of range
c     531   Internal error in subroutine; character array too small
c     568   keyword for momentum boundary conditions found in transport part
c     569   keyword for transport boundary conditions found in momentum part
c     571   The keyword TO has been found, but not straight after CURVE i
c     578   Transport number has already been used for boundary conditions
c     692   CURVE keyword found while not in Sepran mode
c     693   BLOCK, FACE or SUBFACE keyword found while in Sepran mode
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Clear array icurvinf part 4
c    Make table of new main keywords consisting of the local main keywords
c    and the global main keywords and the subsubleywords
c    read until next keyword
c    do while ( keyword = 'CURVE' or 'BLOCK' or 'FACE' or 'SUBFACE' )
c       if ( sepran ) then
c          get the curves description in the form of subfaces
c       else
c          get the subfaces description
c       end if
c       clear arrays itypebc and rtypebc
c       check for illegal transport boundary conditions
c       interpret momentum boundary conditions
c       do while ( keyword = 'TRANSPORT' )
c          get transport number
c          read boundary conditions for this transport equation
c          check for illegal momentum boundary conditions
c          interpret transport boundary conditions
c       end do
c       put boundary conditions just read into iinbc and rinbc
c    end do
c    if ( not end of file ) then
c       return number of next main keyword to
c              be interpreted by the calling subroutine
c    else
c       return -1
c    end if
c    Remark:
c      In the coupled case the first array iinbc/rinbc(.,1) corresponds
c      to the momentum equations
c      In the decoupled case the first arrys iinbc/rinbc(.,1 to ndim)
c      correspond to the u/v/w momentum equations
c      The arrays iinbc/rinbc(.,ndim+1 to maxtrans+ndim) correspond to the
c      transport equations 1 to maxtrans
c      The arrays iinbc/rinbc(maxtrans+ndim+1 to maxtrans+ndim+maxturb)
c      correspond to the turbulence equations 1 to maxturb
c      Array iworkgen positions 1 to nsfread contains the actual curve
c      numbers. If a curve contains subcurves, only the subcurve numbers are
c      stored
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data code1 / 'CURVE', 'BLOCK', 'FACE', 'SUBFACE', 'TRANSPORT' ,
     +             'SURFACE', 'U_MOMENTUM', 'V_MOMENTUM', 'W_MOMENTUM' /
      data nmcode1 / 5, 5, 4, 4, 5, 4, 5, 5, 5 /
      data code  / 'NOSLIP', 'INFLOW', 'OUTFLOW', 'OUTSTREAM',
     +             'FREESLIP', 'UN', 'UT', 'UX', 'UY', 'SIGMANN',
     +             'SIGMANT', 'WALL_FUNCTIONS', 'PERIODIC_PRESSJU',
     +             'PERIODIC', 'FLUX', 'ANTI_PERIODIC_PR',
     +             'ANTI_PERIODIC', 'DIRICHLET', 'NEUMANN',
     +             'ROBBINS', 'ALPHA', 'H', 'MIXED',
     +             'K_DIRICHLET', 'K_NEUMANN', 'EPS_DIRICHLET',
     +             'EPS_NEUMANN', 'OMEGA_DIRICHLET', 'OMEGA_NEUMANN',
     +             'OMEGA_ASYMPTOTE', 'TO', 'ROUGHNESS',
     +             'PARALLEL_OUTFLOW', 'UZ', 'WALL_TEMPERATURE',
     +             'EPS_ASYMPTOTE' /
      data ncode  / 4,  3,  4,  4,  5,    2,  2,  2,  2,  7,
     +              7,  6, 11,  8,  4,   16, 13,  2,  2,  3,
     +              3,  1,  2,  4,  4,    6,  6,  8,  8,  9,
     +              2,  5,  5,  2,  6,    7 /
      data ncodmx / 6,  6,  7,  9,  8,    2,  2,  2,  2,  7,
     +              7, 14, 18,  8,  4,   23, 13,  9,  7,  7,
     +              5,  1,  5, 11,  9,   13, 11, 15, 13, 15,
     +              2,  9, 16,  2, 16,   13 /
      data itype  / 2,  1,  2,  2,  2,   16, 16, 16, 16, 16,
     +             16,  5,  2,  0, 16,    2,  2, 16, 16,  2,
     +             16, 16,  2, 16, 16,   16, 16, 16, 16,  2,
     +              0,  1, 2 , 16, 16,    2 /
      data tabmin / nncode*0d0 /
      data tabmax / nncode*0d0 /
      data code2 / 'SMOOTH', 'ROUGH' /
      data nmcode2 / 6, 5 /

c     code      Table containing the subkeywords corresponding to this
c               subroutine. Contents:
c               1:  NOSLIP
c               2:  INFLOW
c               3:  OUTFLOW
c               4:  OUTSTREAM
c               5:  FREESLIP
c               6:  UN
c               7:  UT
c               8:  UX
c               9:  UY
c              10:  SIGMANN
c              11:  SIGMANT
c              12:  WALL_FUNCTIONS
c              13:  PERIODIC_PRESSJUMP
c              14:  PERIODIC
c              15:  FLUX
c              16:  ANTI_PERIODIC_PRESSJUMP
c              17:  ANTI_PERIODIC
c              18:  DIRICHLET
c              19:  NEUMANN
c              20:  ROBBINS
c              21:  ALPHA
c              22:  H
c              23:  MIXED
c              24:  K_DIRICHLET
c              25:  K_NEUMANN
c              26:  EPS_DIRICHLET
c              27:  EPS_NEUMANN
c              28:  OMEGA_DIRICHLET
c              29:  OMEGA_NEUMANN
c              30:  OMEGA_ASYMPTOTE
c              31:  TO
c              32:  ROUGHNESS
c              33:  PARALLEL_OUTFLOW
c              34:  UZ
c              35:  WALL_TEMPERATURE
c              36:  EPS_ASYMPTOTE
c     code1     Table containing the  main keywords corresponding to this
c               subroutine. Contents:
c               1:  CURVE
c               2:  BLOCK
c               3:  FACE
c               4:  SUBFACE
c               5:  TRANSPORT
c               6:  SURFACE
c               7:  U_MOMENTUM
c               8:  V_MOMENTUM
c               9:  W_MOMENTUM
c     code2     Table containing the  subsubkeywords corresponding to this
c               subroutine. Contents:
c               1:  SMOOTH
c               2:  ROUGH
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isrdboun' )

      if ( ndim.eq.3 ) then
         itype(7) = 26
         itype(11) = 26
      end if

c     --- Clear array icurvinf part 4

      do i = 1, ncurvs
         icurvinf(4,i) = 0
      end do

c     --- Make table of new main keywords consisting of the local main keywords
c         and the global main keywords and the subsubkeywords

      nmain1 = newmain+nmain
      nmmaintot = nmain1+nncod2
      if ( nmmaintot.gt.nmainnw ) then

c     --- Error nmainnw too small

         call iserrint ( nmainnw, 1 )
         call iserrint ( nmain, 2 )
         call iserrint ( newmain, 3 )
         call iserrsub ( 531, 3, 0, 0 )
         call isstop

      end if

      do i = 1, newmain
         codema(i)(1:lenchar) = code1(i)(1:lenchar)
         nmcoda(i) = nmcode1(i)
      end do
      do i = 1, nmain
         codema(i+newmain)(1:lenchar) = codemn(i)(1:lenchar)
         nmcoda(i+newmain) = nmcode(i)
      end do
      do i = 1, nncod2
         codema(i+nmain1)(1:lenchar) = code2(i)(1:lenchar)
         nmcoda(i+nmain1) = nmcode2(i)
      end do

c     --- Set tabmin and tabmax with references of sub subkeywords
c         stored in codema

      tabmin(7) = 1d0
      tabmax(7) = 2d0
      tabmin(11) = 1d0
      tabmax(11) = 2d0

      tabmin(12) = nmain1+1
      tabmax(12) = tabmin(12)+1
      tabmin(2)  = -1d6
      tabmax(2)  = 1d6

c     --- 'TO' keyword

      tabmin(31) = 1d0
      if ( sepran ) then
         tabmax(31) = ncurvs
         itype(14)  = 0
         tabmin(14) = 1
         tabmax(14) = ncurvs
      else
         tabmax(31) = max(nfaces, maxnsubf) + 1
         itype(14)  = 11
         tabmin(14) = 3
         tabmax(14) = max(nblocks,nfaces, maxnsubf)
      end if

      tabmin(32) = 0d0
      tabmax(32) = 1d0
      numsub     = 0
      itrans     = 0

c     --- Read first part input
c         This part recognizes all keywords in code, codemn and code1
c         The keywords in codemn and code1 are treated as main keywords
c         and produce a iretur value

      do i = 1, nncode
         table(i) = -1d20
      end do
      nstrng = 1
      call isintx02 ( code, ncode, numsub, itype, table, codema,
     +                nmcoda, nmain1, iretur, tabmin, tabmax,
     +                tabout, nout, strout, nstrng, ncodmx )

      nsfread = 0

c     --- while ( keyword == 'CURVE '  or  == 'SURFACE' )
c         do while ( iretur.le.4 .or. iretur .eq.6 )

20    if  ( iretur.le.4 .or. iretur.eq.6 ) then

         if ( iretur.eq.1 .or. iretur.eq.6 ) then

c        --- 'CURVE' or "SURFACE" found in non-sepran mode

            if ( .not. sepran ) then

               call iserrsub ( 692, 0, 0, 0 )

            end if

         else

c        --- 'BLOCK', 'FACE' or 'SUBFACE' found in sepran mode

            if ( sepran ) then

               call iserrchr ( codema(iretur), 1 )
               call iserrsub ( 693, 0, 0, 1)

            end if

         end if

c        --- reset table

         do i = 1, numsub
            table(i) = -1d20
         end do
         nstrng = 1

         numsub = nncode
         itrans = 0

c        --- get a set of subfaces

         if ( sepran ) then

            call isrdbou2 ( nsfread, isfread, iworkgen, ncurvs,
     +                      icurvinf, kmeshq, ineigh, ndim,
     +                      code, ncode, numsub, itype, table, codema,
     +                      nmcoda, nmain1, iretur, tabmin, tabmax,
     +                      tabout, nout, strout, nstrng, ncodmx )
         else

            call isrdbou3 ( nsfread, isfread,
     +                      nblocks, nfaces, maxnsubf,
     +                      nsubfaces, subfaces,
     +                      code, ncode, numsub, itype, table, codema,
     +                      nmcoda, nmain1, iretur, tabmin, tabmax,
     +                      tabout, nout, strout, nstrng, ncodmx )

         end if

c        --- Clear arrays itypebc and rtypebc

         do i = 1, maxtrans+maxturb+ndim
            do j = 1, ndim+1
               itypebc(i,j) = 0
            end do
            do j = 1, ndim
               rtypebc(i,j) = 0d0
            end do
         end do

c        --- interpret the first part of the boundary conditions
c            no transport boundary conditions are allowed since these
c            boundary conditions must be preceded by the keyword
c            TRANSPORT or U/V/W MOMENTUM

         do i = 18, 23
            if ( table(i).gt.-1d4 ) then
               call iserrchr ( code(i), 1 )
               call iserrsub ( 569, 0, 0, 1 )
               call iserrcrd
               table(i) = -1d20
            end if
         end do
         if ( table(35).gt.-1d4 ) then
           call iserrchr ( code(35), 1 )
           call iserrsub ( 569, 0, 0, 1 )
           call iserrcrd
           table(35) = -1d20
         end if

c        --- interpret momentum boundary conditions

         call isrdbou4 ( nsfread, isfread, nncode,  nout, code, table,
     +                   tabout, uset, maxtrans, maxturb, ndim, itypebc,
     +                   rtypebc, ineigh, sepran, nblocks, nsubfaces,
     +                   tabmax )

c        --- do while ( keyword == 'TRANSPORT' or U/V/W_MOMENTUM )
c            do while ( iretur.eq.5 )

30       if  ( iretur.eq.5 .or. iretur.ge.7 .and. iretur.le.9 ) then

c        --- transport or U/V/W_MOMENTUM found

            if ( iretur.eq.5 ) then

c           --- Read transport sequence number itrans

               call isasknxt
               if ( ityprd.eq.2 ) then

c              --- number found

                  itrans = nint(realrd)
                  if ( itrans.lt.1 .or. itrans.gt.maxtrans ) then

c                 --- itrans out of range

                     call iserrchr ( codema(iretur), 1 )
                     call iserrint ( itrans, 1 )
                     call iserrint ( 1, 2 )
                     call iserrint ( 1000, 3 )
                     call iserrsub ( 530, 3, 0, 1 )
                     call iserrcrd
                     itrans = 1

                  else if ( itypebc(itrans+ndim,1).ne.0 ) then

c                 --- itrans already in use

                     call isrderr( 578, ' ', itrans, nsfread, isfread )

                  end if

               else

c              --- Error not an integer number found

                  call iserrchr ( codema(iretur), 1 )
                  call iserrsub ( 528, 0, 0, 1 )
                  call iserrcrd

               end if

            else

c           --- U/V/W_MOMENTUM
c               This equation is treated as a transport equation
c               Since teh transport equations are counted from sequence number
c               ndim+1 and the u/v/w momentum equations must be counted from
c               position 1 we use the following values for itrans:
c               ndim = 2: u: -1, v:  0
c               ndim = 3: u: -2, v: -1, w: 0
c               In isrdbou5 ndim+itrans is used

               itrans = iretur-6-ndim

            end if

c           --- read boundary conditions for this transport equation

            do i = 1, numsub
               table(i) = -1d20
            end do
            nstrng = 1
            call isintx02 ( code, ncode, numsub, itype, table, codema,
     +                      nmcoda, nmain1, iretur, tabmin, tabmax,
     +                      tabout, nout, strout, nstrng, ncodmx )

c           --- 'TO' not allowed

            if ( table(31).gt.0d0 ) then

               call iserrsub ( 571, 0, 0, 0 )
               call iserrcrd
               table(31) = -1d20

            end if

c           --- no boundary conditions for momentum allowed

            do i = 1, 13
               if ( table(i).gt.-1d4 ) then
                  call iserrchr ( code(i), 1 )
                  call iserrsub ( 568, 0, 0, 1 )
                  call iserrcrd
               end if
               table(i) = -1d20
            end do
            do i = 15, 16
               if ( table(i).gt.-1d4 ) then
                  call iserrchr ( code(i), 1 )
                  call iserrsub ( 568, 0, 0, 1 )
                  call iserrcrd
                  table(i) = -1d20
               end if
            end do
            if ( table(33).gt.-1d4 ) then
               call iserrchr ( code(33), 1 )
               call iserrsub ( 568, 0, 0, 1 )
               call iserrcrd
               table(33) = -1d20
            end if

c           --- interpret boundary conditions

            call isrdbou5 ( itrans, nsfread, isfread,
     +                      nncode,  nout, code, table, tabout,
     +                      maxtrans, maxturb, ndim, itypebc, rtypebc,
     +                      ineigh )

c           --- end do while ( keyword = 'TRANSPORT' )

            goto 30

         end if

         call isrdbou1 ( nblocks, nfaces, maxnsubf, subfaces,
     +                   nsfread, isfread, iinbc, rinbc,
     +                   maxtrans, maxturb, itypebc, rtypebc,
     +                   ndim )

c        --- end do while ( keyword == 'CURVE' )

         goto 20

      end if

c     --- Check if the boundary condition SIGMAnn Ut given at two
c         adjacent boundaries is present.
c         If so: give error message

      if ( ndim.eq.2 ) then

c     --- ndim = 2

         do iblockrf = 1, nblocks

            call iserrbnd ( iinbc(iblockrf,1), maxtrans, maxturb )

         end do

      end if

      if ( iretur.ge.0 ) iretur = iretur - newmain

      call iserclos ( 'isrdboun' )
      end
