      subroutine isrderr ( mesnmseq, keyword, itrans, nsfread, isfread )
c ======================================================================
c
c
c        programmer     Erik Brakkee
c        version 1.0    date 9-11-94
c
c
c   copyright (c) 1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                      DESCRIPTION
c
c    Give an error message for a set of curves with an additional
c    keyword string given.
c    The reason for this routine is that it is the most common call
c    to the error subroutines from isrdboun
c ********************************************************************
c
c                      KEYWORDS
c
c     error_manipulation
c     print
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer itrans, nsfread, isfread(10,1:nsfread), mesnmseq

c    isfread    i  reserved
c    itrans     i  reserved
c    keyword    i  name of a keyword
c    mesnmseq  i/o Message sequence number.
c    nsfread    i  Number of subfaces read
c
c ********************************************************************
c
c                      COMMON BLOCKS
c
c    None
c ********************************************************************
c
c                      LOCAL PARAMETERS
c
      integer i, nints, nchar, iblock1, iblock2, iface1, iface2,
     +        iseg1, iseg2
c
      character * (*) keyword

c     i         General loop variable
c     iblock1   Block sequence number
c     iblock2   Block sequence number
c     iface1    Face number
c     iface2    Face number
c     iseg1     Counting variable: sequence number of segment
c     iseg2     Counting variable: sequence number of segment
c     nchar     Number of strings in a message
c     nints     Number of integers in a message
c
c
c ********************************************************************
c
c                      SUBROUTINES CALLED
c
c     ISERRCHR  Puts values of strings in character error array
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c
c ********************************************************************
c
c                      I/O
c
c ********************************************************************
c
c                      ERROR MESSAGES
c     Always
c
c ********************************************************************
c
c                      PSEUDO CODE
c
c     if ( sepran error message ) then
c
c        determine curve range
c        give error at the correct curves
c
c     else
c
c        determine new error number
c        give error at the correct subfaces
c
c     end if
c
c ********************************************************************
c
c                      DATA STATEMENTS
c
c     None
c
c ======================================================================
c

      nints = 0
      nchar = 0

      if ( isfread(4,1) .gt. 0 ) then

c        --- error message for a set of curves

         if ( itrans .ne. 0 ) then

            nints = nints + 1
            call iserrint ( itrans,             nints )

         end if

         call iserrint ( isfread(4,1),       nints+1 )
         call iserrint ( isfread(4,nsfread), nints+2 )
         nints = nints + 2

         if ( keyword .ne. ' ' ) then

            nchar = nchar + 1
            call iserrchr ( keyword,            nchar)

         end if

         call iserrsub ( mesnmseq, nints, 0, nchar )

      else

c        --- error message for a set of subfaces

c           --- determine block, face and subface range

         iblock1 = isfread(1,1)
         iblock2 = isfread(1,1)
         iface1  = isfread(2,1)
         iface2  = isfread(2,1)
         iseg1   = isfread(3,1)
         iseg2   = isfread(3,1)
         do i = 2, nsfread
            iblock1 = MIN( iblock1, isfread(1,i) )
            iblock2 = MAX( iblock2, isfread(1,i) )
            iface1  = MIN( iface1,  isfread(2,i) )
            iface2  = MAX( iface2,  isfread(2,i) )
            iseg1   = MIN( iseg1,   isfread(3,i) )
            iseg2   = MAX( iseg2,   isfread(3,i) )
         enddo

         if ( itrans .ne. 0 ) then

            nints = nints + 1
            call iserrint ( itrans,             nints )

         end if

         call iserrint ( iseg1,              nints+1 )
         call iserrint ( iseg2,              nints+2 )
         call iserrint ( iface1,             nints+3 )
         call iserrint ( iface2,             nints+4 )
         call iserrint ( iblock1,            nints+5 )
         call iserrint ( iblock2,            nints+6 )
         nints = nints + 6

         if ( keyword .ne. ' ' ) then

            nchar = nchar + 1
            call iserrchr ( keyword,            nchar)

         end if

         call iserrsub ( mesnmseq + 100, nints, 0, nchar )

      end if


      end
