      subroutine isrdlins ( nmain, codemn, nmcode, iretur, iinsol,
     +                      rinsol, maxtrans, maxturb, icheckeq )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.9    date 29-04-1999 Guus: Extension of codemn
c        version 1.8    date 13-01-1999 Guus: Correction print steps
c        version 1.7    date 14-07-1997 Guus: extension of nmainnw
c        version 1.6    date 26-05-1997 Guus: extension for multigrid
c        version 1.5    date 27-01-1997 Guus: extension for cgs/cgstab
c        version 1.4    date 12-07-1995 Guus: change minimum modify
c        version 1.3    date 10-07-1995 Guus: change sequence CGS and CGSTAB
c        version 1.2    date 08-03-1995 Kees K: bug fixed for maxt
c        version 1.1    date 21-04-1995 (Extension with print)
c        version 1.0    date 28-07-1994
c
c
c   copyright (c) 1994-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Read linear solver part of input files and store result in
c    arrays iinsol and rinsol
c **********************************************************************
c
c                       KEYWORDS
c
c       linear_solver
c       read
c       read_cards
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nmain, nmcode(nmain), iretur, maxtrans, maxturb,
     +        iinsol(25,maxtrans+2+maxturb),
     +        icheckeq(maxtrans+2+maxturb)
      double precision rinsol(25,maxtrans+2+maxturb)
      character *(*) codemn(nmain)

c     codemn    i    Table of main keywords
c     icheckeq  i    Work array to indicate if an equation has been considered
c                    (icheckeq(i)=1 or not icheckeq(i)=0)
c     iinsol    o    reserved
c     iretur    o    Return code, indicates the sequence number of the new
c                    main keyword found
c                    If -1 an error has occurred
c                    If -2 the end of file has been reached
c     maxtrans  i    Maximum number of transport equations allowed
c     maxturb   i    Maximum number of turbulence equations allowed
c     nmain     i    Number of keywords in array codemn
c     nmcode    i    Array containing the number of significant characters
c                    for each main keyword in codemn
c     rinsol    o    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/csepch'
      include 'IScommon/cseprd'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer nmainnw, newmain, lenchar, nncode, nncod2, nout
      parameter ( nmainnw=50, newmain=4, nncode=25, nncod2=18,
     +            lenchar=14, nout=3 )
      integer nmcode1(newmain), nmcoda(nmainnw), itype(nncode),
     +        nmcode2(nncod2), ncodmx(nncode), nstrng,
     +        ncode(nncode), i, iequa, ih, iseq, iseqprev, keyprev,
     +        maxt, nmain1, nmmaintot, numsub, inum
      double precision table(nncode), tabmin(nncode), tabmax(nncode),
     +                 tabout(nout)
      character *14 codema(nmainnw), code1(newmain), code2(nncod2)
      character *16 code(nncode)
      character *1 strout(1)
      logical ready

c     code      Table containing the subkeywords corresponding to this
c               subroutine.
c     code1     Table containing the  main keywords corresponding to this
c               subroutine.
c     code2     Table containing the  subsubkeywords corresponding to this
c               subroutine.
c     codema    Table consisting of the local main keywords followed by the
c               general main keywords possibly followed by the subsubkeywords
c     i         General loop variable
c     iequa     General equation sequence number
c     ih        Help variable to store a constant temporarily
c     inum      Indicates the number of numbers that have been read
c     iseq      Sequence number read after transport or turbulence equation
c     iseqprev  Previous value of iseq
c     itype     Array containing the type of actions to be undertaken for
c               each subkeyword found.
c               See subroutine isintx02 for the possible values of itype
c     keyprev   Sequence number of previous keyword found with respect to code
c     lenchar   Number of characters in main keywords
c     maxt      Maximum number of equations allowed for a specific type
c               With respect to transport: maxtrans-10
c               With respect to turbulence: 10
c     ncode     Array containing the number of significant characters for
c               each keyword in code
c     ncodmx    Array containing the maximum number of characters for
c               each keyword in code
c     newmain   Number of main keywords stored in array code1
c     nmain1    Total number of main keywords (newmain+nmain)
c     nmainnw   Maximal number of keywords in codema
c     nmcoda    Array containing the number of significant characters for
c               each keyword in codema
c     nmcode1   Array containing the number of significant characters for
c               each keyword in code1
c     nmcode2   Array containing the number of significant characters for
c               each keyword in code2
c     nmmaintot Total number of keywords in codema
c     nncod2    Number of subsubkeywords stored in array code2
c     nncode    Number of subkeywords stored in array code
c     nout      Length of array tabout
c     nstrng    Length of array strout
c     numsub    Number of subkeywords that may be detected by isintx02
c               In the first step numsub=0 for any other step numsub=nncode
c     ready     Indicates if a while loop is finished or not
c     strout    Contains series of strings, see itype
c     table     Array containing the real or integer values corresponding
c               to the subkeywords in code that have been read by isintx02
c     tabmax    Array containing the maximal values of the parameters read
c               corresponding to the subkeywords in code
c     tabmin    Array containing the minimum values of the parameters read
c               corresponding to the subkeywords in code
c     tabout    Array of output reals provided by isintx02
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISASKNXT  Read next item from the standard input file
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRCRD  Prints the last record read and underlines the errorneous part
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISINTX02  Interpret a record containing keywords followed by numbers
c     ISSTOP    Stop ISNAS
c     ISWARSUB  Print warnings
c **********************************************************************
c
c                       I/O
c
c    This subroutine reads the input after the main keyword linear_solver
c    until a new main keyword has been found
c
c    The following input is recognized:
c
c    MOMentum_equations
c    TRANSport_equation i or all
c    PRESsure_equation
c    TURBulence_equation i or all
c
c    These keywords itself may be followed by subkeywords that indicate
c     how the linear equations must be solved.
c    The following subkeywords are recognized:
c
c    SOLVER = s
c    MAXITer = n
c    ABSAccuracy = abs
c    RELAccuracy = rel
c    CONDAccuracy = con
c    DIVAccuracy = div
c    PREConditioner = pre
c    POSTConditioner = post
c    POLYPREConditioner = n
c    AMOUnt_of_output = out
c    ERRORoccurance = err
c    GMREStart = n
c    ORTHogonal = n
c    MODIFY = alpha
c    FILL_IN = n
c    STARTVector = start
c    RITZvalues = n
c    PRINT_SOLUTION
c    PRINT_STeps = (itime1,itime2,itstep)
c    CG_RELAXation = omega
c    CG_INner = n
c    NEUMayer
c    NO_NEUMayer
c    MAXLEvel = n
c    SMOOTH_steps = n
c
c    s may take one of the following values:
c       LSQR
c       CGS
c       GMRES
c       GCR
c       CGSTAB
c
c    pre may take one of the following values:
c       NONE
c       DIAGonal
c       ILUD
c       ILU_Fill
c       ILU
c       MULTIgrid
c       TEST
c
c    err may take one of the following values:
c       STOP
c       RESUME
c
c    start may take one of the following values:
c       ZERO
c       EXPLicit
c       PREC
c       LINear
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     528   No integer number has been found after the keyword %
c     530   integer number read out of range
c     531   Internal error in subroutine; character array too small
c     567   Equation has been found more than once
c     587   two keywords found that are mutually exclusive
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Make table of new main keywords consisting of the local main keywords
c    and the global main keywords and the subsubkeywords
c    ready := false
c    number_of_subkeywords := 0
c    while ( not ready ) do
c       iseq := 0
c       Read next subkeywords until a new main keyword is found
c       if ( main keyword = transport or turbulence ) then
c          Read sequence number iseq
c       if ( number_of_subkeywords=0 ) then
c          number_of_subkeywords := actual number
c       else
c          interpret subkeywords as function of keyword_previous
c       if ( keyword found = main keyword ) then
c          ready := true
c       keyword_previous := keyword found
c       iseq_previous := iseq
c
c    Remark:
c
c      The first array iinsol/rinsol(.,1) corresponds to the momentum equations
c      The arrays iinsol/rinsol(.,2 to maxtrans+1) correspond to the transport
c      equations 1 to maxtrans
c      The arrays iinsol/rinsol(maxtrans+2 to maxtrans+1+maxturb) correspond to
c      the turbulence equations 1 to maxturb
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data code1 / 'MOMENTUM_EQ', 'PRESSURE_EQ', 'TRANSPORT_EQ',
     +             'TURBULENCE_EQ' /
      data nmcode1 / 3, 4, 4, 12 /
      data code  / 'SOLVER', 'MAXITER', 'ABSACCURACY', 'RELACCURACY',
     +             'CONDACCURACY', 'DIVACCURACY', 'PRECONDITIONER',
     +             'POSTCONDITIONE', 'POLYPRECONDITI',
     +             'AMOUNT_OF_OUTP', 'ERROROCCURANCE', 'GMRESTART',
     +             'ORTHOGONAL', 'MODIFY', 'STARTVECTOR', 'RITZVALUES',
     +             'PRINT_SOLUTION', 'PRINT_STEPS', 'FILL_IN',
     +             'CG_RELAXATION', 'CG_INNER', 'NEUMAYER',
     +             'NO_NEUMAYER', 'MAXLEVEL' , 'SMOOTH_STEPS'/
      data ncode  / 6, 5,  4,  4,  5,    4,  4,  5,  8,  4,
     +              5, 5,  4,  6,  6,    4,  9,  8,  7,  6,
     +              6, 4,  7,  5,  5 /
      data ncodmx / 6, 7, 11, 11, 11,   11, 14, 15, 18, 16,
     +             14, 9, 10,  6, 11,   10, 14, 11,  7, 13,
     +              8, 8, 11,  8, 12 /
      data itype  / 5, 0,  1,  1,  1,    1,  5,  5,  0,  0,
     +              5, 0,  0,  1,  5,    0,  2, 11,  0,  1,
     +              0, 2,  2,  0,  0 /
      data tabmin / 0d0, 1d0, 0d0, 0d0,  0d0, 0d0, 0d0, 0d0, 0d0,-1d0,
     +              0d0, 1d0, 1d0, 0d0,  0d0, 0d0, 0d0, 3d0, 0d0, 0d0,
     +              1d0, 0d0, 0d0, 0d0,  0d0 /
      data tabmax / 0d5, 1d5, 1d2, 1d0, 1d-1,1d-1, 0d0, 0d0, 1d2, 2d0,
     +              0d0, 1d4, 1d4, 1d0,  0d0, 1d3, 0d0, 1d4, 1d2, 1d0,
     +             16d0, 0d0, 0d0,30d0, 10d0 /
      data code2 / 'LSQR', 'CGSTAB', 'GMRES', 'GCR', 'CGS', 'NONE',
     +             'DIAGONAL', 'ILUD', 'ILU_FILL', 'ILU', 'MULTIGRID',
     +             'TEST', 'STOP', 'RESUME', 'ZERO', 'EXPLICIT', 'PREC',
     +             'LINEAR' /
      data nmcode2 / 4, 6, 5, 3, 3,    5, 4, 4, 4, 5,
     +               3, 4, 4, 5, 4,    4, 4, 3 /

c     code      Table containing the subkeywords corresponding to this
c               subroutine. Contents:
c               1:  SOLVER
c               2:  MAXITer
c               3:  ABSAccuracy
c               4:  RELAccuracy
c               5:  CONDAccuracy
c               6:  DIVAccuracy
c               7:  PREConditioner
c               8:  POSTConditioner
c               9:  POLYPREConditioner
c              10:  AMOUnt_of_output
c              11:  ERRORoccurance
c              12:  GMREStart
c              13:  ORTHogonal
c              14:  MODIFY
c              15:  STARTVector
c              16:  RITZvalues
c              17:  PRINT_SOLUTION
c              18:  PRINT_STEPS
c              19:  FILL_IN
c              20:  CG_RELAXation
c              21:  CG_INner
c              22:  NEUMayer
c              23:  NO_NEUMayer
c              24:  MAXLEvel
c              25:  SMOOTH_steps
c     code1     Table containing the  main keywords corresponding to this
c               subroutine. Contents:
c               1:  MOMENTUM_EQUATION
c               2:  PRESSURE_EQUATION
c               3:  TRANSPORT_EQUATION
c               4:  TURBULENCE_EQUATION
c     code2     Table containing the  subsubkeywords corresponding to this
c               subroutine. Contents:
c               1:  LSQR
c               2:  CGSTAB
c               3:  GMRES
c               4:  GCR
c               5:  CGS
c               6:  NONE
c               7:  DIAGonal
c               8:  ILUD
c               9:  ILU_Fill
c              10:  ILU
c              11:  MULTIgrid
c              12:  TEST
c              13:  STOP
c              14:  RESUME
c              15:  ZERO
c              16:  EXPLicit
c              17:  PREC
c              18:  LINear
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isrdlins' )

c     --- Make table of new main keywords consisting of the local main keywords
c         and the global main keywords and the subsubkeywords

      nmain1 = newmain+nmain
      nmmaintot = nmain1+nncod2
      if ( nmmaintot.gt.nmainnw ) then

c     --- Error nmainnw too small

         call iserrint ( nmainnw, 1 )
         call iserrint ( nmain, 2 )
         call iserrint ( newmain, 3 )
         call iserrsub ( 531, 3, 0, 0 )
         call isstop

      end if

      do i = 1, newmain
         codema(i)(1:lenchar) = code1(i)(1:lenchar)
         nmcoda(i) = nmcode1(i)
      end do
      do i = 1, nmain
         codema(i+newmain)(1:lenchar) = codemn(i)(1:lenchar)
         nmcoda(i+newmain) = nmcode(i)
      end do
      do i = 1, nncod2
         codema(i+nmain1)(1:lenchar) = code2(i)(1:lenchar)
         nmcoda(i+nmain1) = nmcode2(i)
      end do

c     --- Clear array icheckeq

      do i = 1, maxtrans+2+maxturb
         icheckeq(i) = 0
      end do

c     --- Set tabmin and tabmax with references of sub subkeywords
c         stored in codema

      tabmin(1) = nmain1+1
      tabmax(1) = tabmin(1)+4
      tabmin(7) = tabmax(1)+1
      tabmax(7) = tabmin(7)+6
      tabmin(8) = tabmin(7)
      tabmax(8) = tabmax(7)
      tabmin(11) = tabmax(8)+1
      tabmax(11) = tabmin(11)+1
      tabmin(15) = tabmax(11)+1
      tabmax(15) = tabmin(15)+3
      ready = .false.
      numsub = 0

200   if ( .not. ready ) then

c     --- while loop ready

         iseq = 0

c        --- Read next subkeywords until a new main keyword is found

         do i = 1, numsub
            table(i) = -1d5
         end do
         nstrng = 1
         tabmax(18) = 1d4
         table(19)  = 8d0
         table(20)  = 0.7d0
         table(21)  = 1d0
         call isintx02 ( code, ncode, numsub, itype, table, codema,
     +                   nmcoda, nmain1, iretur, tabmin, tabmax,
     +                   tabout, nout, strout, nstrng, ncodmx )
         if ( iretur.eq.3 .or. iretur.eq.4 ) then

c        --- transport or turbulence found
c            Read sequence number iseq

            call isasknxt
            if ( ityprd.eq.2 ) then

c           --- number found

               iseq = nint(realrd)
               if ( iretur.eq.3 ) then
                  ih = maxtrans
               else
                  ih = maxturb
               end if
               if ( iseq.lt.1 .or. iseq.gt.ih ) then

c              --- iseq out of range

                  call iserrchr ( codema(iretur), 1 )
                  call iserrint ( iseq, 1 )
                  call iserrint ( 1, 2 )
                  call iserrint ( ih, 3 )
                  call iserrsub ( 530, 3, 0, 1 )
                  call iserrcrd
                  iseq = 1

               end if

            else if ( ityprd.eq.4 .and. string(1:3).eq.'ALL' ) then

c           --- text all found set iseq equal to -iretur

               iseq = -iretur

            else

c           --- Error not an integer number found

               call iserrchr ( codema(iretur), 1 )
               call iserrsub ( 528, 0, 0, 1 )
               call iserrcrd

            end if

         end if

         if ( numsub.eq.0 ) then

c        --- number_of_subkeywords = 0

            numsub = nncode

         else

c        --- interpret subkeywords as function of keyword_previous

            if ( keyprev.eq.1 .or. keyprev.eq.2 ) then
               iequa = keyprev
            else if ( keyprev.eq.3 ) then
               if ( iseqprev.gt.0 ) then
                  iequa = 2+iseqprev
               else
                  iequa = 3
               end if
            else
               if ( iseqprev.gt.0 ) then
                  iequa = iseqprev+maxtrans+2
               else
                  iequa = 3+maxtrans
               end if
            end if

c           --- Test if equation has already been treated before

            if ( icheckeq(iequa).ne.0 ) then
               call iserrint ( iseq, 1 )
               call iswarsub ( 567, 1, 0, 0 )
               call iserrcrd
            end if

c           --- Fill icheckeq

            do i = iequa, iequa+maxt-1
               icheckeq(i) = 1
            end do
            if ( table(1).ge.0 ) then

c           --- solver has been found

               ih =  nint(table(1))-1
               if ( ih.eq.1 ) then
                  ih = 4
               else if ( ih.eq.4 ) then
                  ih = 1
               end if
               do i = iequa, iequa+maxt-1
                  iinsol(7,i) = ih
               end do

            end if

            if ( table(2).ge.0 ) then

c           --- maxiter has been found

               ih =  nint(table(2))
               do i = iequa, iequa+maxt-1
                  iinsol(6,i) = ih
               end do

            end if

            if ( table(3).ge.0 ) then

c           --- absaccuracy has been found

               do i = iequa, iequa+maxt-1
                  rinsol(1,i) = table(3)
               end do

            end if

            if ( table(4).ge.0 ) then

c           --- relaccuracy has been found

               do i = iequa, iequa+maxt-1
                  rinsol(2,i) = table(4)
               end do

            end if

            if ( table(5).ge.0 ) then

c           --- condaccuracy has been found

               do i = iequa, iequa+maxt-1
                  rinsol(5,i) = table(5)
               end do

            end if

            if ( table(6).ge.0 ) then

c           --- divaccuracy has been found

               do i = iequa, iequa+maxt-1
                  rinsol(6,i) = table(6)
               end do

            end if

            if ( table(7).ge.0 ) then

c           --- preconditioner has been found

               ih =  nint(table(7))-1
               if ( ih.eq.3 ) then
                  ih = 4
               else if ( ih.eq.4 ) then
                  ih = 3
               end if
               do i = iequa, iequa+maxt-1
                  iinsol(2,i) = ih
               end do

            end if

            if ( table(8).ge.0 ) then

c           --- postconditioner has been found

               ih =  nint(table(8))-1
               if ( ih.eq.3 ) then
                  ih = 4
               else if ( ih.eq.4 ) then
                  ih = 3
               end if
               do i = iequa, iequa+maxt-1
                  iinsol(2,i) = -ih
               end do

            end if

            if ( table(9).ge.0 ) then

c           --- polypreconditioner has been found

               ih =  nint(table(9))
               do i = iequa, iequa+maxt-1
                  iinsol(13,i) = ih
               end do

            end if

            if ( table(10).ge.0 ) then

c           --- amount of output has been found

               ih =  nint(table(10))
               do i = iequa, iequa+maxt-1
                  iinsol(3,i) = ih
               end do

            end if

            if ( table(11).ge.0 ) then

c           --- error occurence has been found

               ih =  nint(table(11))-1
               do i = iequa, iequa+maxt-1
                  iinsol(5,i) = ih
               end do

            end if

            if ( table(12).ge.0 ) then

c           --- gmrestart has been found

               ih =  nint(table(12))
               do i = iequa, iequa+maxt-1
                  iinsol(11,i) = ih
               end do

            end if

            if ( table(13).ge.0 ) then

c           --- orthogonal has been found

               ih =  nint(table(13))
               do i = iequa, iequa+maxt-1
                  iinsol(12,i) = ih
               end do

            end if

            if ( table(14).ge.0 ) then

c           --- modify has been found

               do i = iequa, iequa+maxt-1
                  rinsol(7,i) = table(14)
               end do

            end if

            if ( table(15).ge.0 ) then

c           --- start vector has been found

               ih =  nint(table(15))-1
               do i = iequa, iequa+maxt-1
                  iinsol(4,i) = ih
               end do

            end if

            if ( table(16).ge.0 ) then

c           --- RITZvalues has been found

               ih =  nint(table(16))
               do i = iequa, iequa+maxt-1
                  iinsol(14,i) = ih
               end do

            end if

            if ( table(17).ge.0 ) then

c           --- PRINT_SOLUTION has been found

               ih =  nint(table(17))
               do i = iequa, iequa+maxt-1
                  iinsol(17,i) = ih
               end do

            end if

            if ( table(18).gt.0 ) then

c           --- PRINT_STEPS found

               ih = nint(table(18))
               inum = nint(tabmax(18))
               do i = iequa, iequa+maxt-1
                  iinsol(18,i) = nint(tabout(ih))
                  if ( inum.gt.1 ) then
                     iinsol(19,i) = nint(tabout(ih+1))
                  else
                     iinsol(19,i) = iinsol(13,i)
                  end if
                  if ( inum.gt.2 ) then
                     iinsol(20,i) = nint(tabout(ih+2))
                  else
                     iinsol(20,i) = 1
                  end if
               end do

            end if
            ih = nint(table(19))
            do i = iequa, iequa+maxt-1
               iinsol(22,i) = ih
            end do
            do i = iequa, iequa+maxt-1
               rinsol(8,i) = table(20)
            end do
            ih = nint(table(21))
            do i = iequa, iequa+maxt-1
               iinsol(24,i) = ih
            end do
            if ( table(22).gt.0 ) then
               if ( table(23).gt.0 ) then

c              --- Neumayer and no_neumayer are mutually exclusive

                  call iserrchr ( code(22), 1 )
                  call iserrchr ( code(23), 2 )
                  call iserrsub ( 587, 0, 0, 2 )
              end if
            else if ( table(23).gt.0 ) then
               do i = iequa, iequa+maxt-1
                  iinsol(23,i) = 0
               end do
            end if
            if ( table(24).ge.0 ) then

c           --- maxlevel has been found

               ih =  nint(table(24))
               do i = iequa, iequa+maxt-1
                  iinsol(9,i) = ih
               end do

            end if

            if ( table(25).ge.0 ) then

c           --- smooth_steps has been found

               ih =  nint(table(25))
               do i = iequa, iequa+maxt-1
                  iinsol(10,i) = ih
               end do

            end if

         end if

         if ( iretur.gt.newmain ) then

c        --- main keyword found

            ready = .true.
            iretur = iretur-newmain

         else if ( iretur.le.-1 ) then

c        --- error or end of file found

            ready = .true.

         end if
         if ( iseq.eq.-3 ) then
            maxt = maxtrans
         else if ( iseq.eq.-4 ) then
            maxt = maxturb
         else
            maxt = 1
         end if
         keyprev = iretur
         iseqprev = iseq
         go to 200

      end if

      call iserclos ( 'isrdlins' )
      end
