      subroutine iswrtif2 ( idimens, maxtrans, maxturb, iinsol,
     +                      rinsol, ndim, icoupled )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.0    date 09-10-1997 Guus: Extra parameters ndim, icoupled
c        version 1.5    date 26-05-1997 Guus: New array iinsol
c        version 1.4    date 27-01-1997 Guus: New array iinsol
c        version 1.3    date 13-09-1995 Guus: New array iinsol
c        version 1.2    date 22-02-1995 Guus: added 'print solution'
c        version 1.1    date 16-01-1995 Erik: added 'save_preconditioner'
c        version 1.0    date 27-09-1994
c
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Write third part of the intermediate file
c    It concerns the blocks 6 to 6
c **********************************************************************
c
c                       KEYWORDS
c
c    intermediate_file
c    write
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer idimens(100), maxturb, maxtrans,
     +        iinsol(25,maxtrans+2+maxturb), ndim, icoupled
      double precision rinsol(25,maxtrans+2+maxturb)

c     icoupled  i    If 0 the momentum equations are coupled, if 1 they are not
c     idimens   i    Integer dimension parameters for the intermediate file
c     iinsol    i    reserved
c     maxtrans  i    Maximum number of transport equations allowed
c     maxturb   i    Maximum number of turbulence equations allowed
c     ndim      i    Dimension of space (2 or 3)
c     rinsol    i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, itrans, ntrnsp, nturb

c     i         General loop variable
c     itrans    reserved
c     ntrnsp    Number of transport equations
c     nturb     Number of extra transport equations for turbulence
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c     This subroutine  writes the following parts to the intermediate file
c
c     Block 6: Parameters with respect to the linear solver
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Get ntrnsp and nturb
c     write coefficients for momentum equation
c     write coefficients for transport equations
c     write coefficients for turbulence equations
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Get ntrnsp and nturb

      ntrnsp = idimens(3)
      nturb  = idimens(6)

c     --- write information for momentum equation

      if ( icoupled.eq.0 ) then

c     --- Coupled equations

         write(irefwr,100)
         write(irefwr,110) (iinsol(i,1),i=1,15)
         write(irefwr,111) (iinsol(i,1),i=16,25)
         write(irefwr,120) (rinsol(i,1),i=1,25)

      else

c     --- Decoupled equations
c         First u-momentum

         write(irefwr,130)
         write(irefwr,110) (iinsol(i,1),i=1,15)
         write(irefwr,111) (iinsol(i,1),i=16,25)
         write(irefwr,120) (rinsol(i,1),i=1,25)

c        --- Next v-momentum

         write(irefwr,140)
         write(irefwr,110) (iinsol(i,1),i=1,15)
         write(irefwr,111) (iinsol(i,1),i=16,25)
         write(irefwr,120) (rinsol(i,1),i=1,25)

         if ( ndim.eq.3 ) then

c        --- Finally w-momentum

            write(irefwr,150)
            write(irefwr,110) (iinsol(i,1),i=1,15)
            write(irefwr,111) (iinsol(i,1),i=16,25)
            write(irefwr,120) (rinsol(i,1),i=1,25)

         end if

      end if
100   format ( /'**** BLOCK 6: Parameters with respect to the',
     +          ' linear solver'//
     +          '** Subblock 6.1 momentum equations'// )
110   format ( /'* iinsol:'//
     +          '                   ', i5,'     # Pos. 1 is dummy'/
     +          'ipreco            =', i5,'     # Preconditioner'/
     +          'amount_of_output  =', i5/
     +          'start_vector      =', i5/
     +          'stop              =', i5/
     +          'maxiter           =', i5/
     +          'method            =', i5/
     +          'file_nr           =', i5/
     +          'maxlevel          =', i5/
     +          'smooth_steps      =', i5/
     +          'irestart          =', i5/
     +          'iorthogonal       =', i5/
     +          'degree of polynom =', i5/
     +          'numb of Ritz vals =', i5/
     +          'save precondit.   =', i5)
111   format(   'prec. built.      =', i5/
     +          'print solution    =', i5/
     +          'itime_one         =', i5/
     +          'itime_two         =', i5/
     +          'itime_step        =', i5/
     +          'singular          =', i5/
     +          'nfill             =', i5/
     +          'Neumayer approach =', i5/
     +          'cg_inner          =', i5/
     +           i2, '  # Position 25 not yet defined' )
120   format ( /'* rinsol :'//
     +          'absaccuracy        =', d15.7/
     +          'relaccuracy        =', d15.7/
     +          '                    ', d15.7,'     # Pos. 3 is dummy'/
     +          '                    ', d15.7,'     # Pos. 4 is dummy'/
     +          'condaccuracy       =', d15.7/
     +          'divaccuracy        =', d15.7/
     +          'alpha              =', d15.7/
     +          'omega_CGStab       =', d15.7/
     +          ( 5d15.7))
130   format ( /'**** BLOCK 6: Parameters with respect to the',
     +          ' linear solver'//
     +          '** Subblock 6.1a u-momentum equations'// )
140   format ( /'****  Subblock 6.1b v-momentum equations'// )
150   format ( /'****  Subblock 6.1c w-momentum equations'// )

c     --- write information about the pressure equation

      write(irefwr,200)
200   format ( /'** Subblock 6.2 pressure equation'//)
      write(irefwr,110) (iinsol(i,2),i=1,15)
      write(irefwr,111) (iinsol(i,2),i=16,25)
      write(irefwr,120) (rinsol(i,2),i=1,25)

c     --- write information about the transport equations

      do itrans = 1, ntrnsp
         write(irefwr,300) itrans+2,itrans
300      format ( /'** Subblock 6.',i2,' transport equation',i2// )
         write(irefwr,110) (iinsol(i,2+itrans),i=1,15)
         write(irefwr,111) (iinsol(i,2+itrans),i=16,25)
         write(irefwr,120) (rinsol(i,2+itrans),i=1,25)
      end do

c     --- write information about the turbulence equations

      do itrans = 1, nturb
         write(irefwr,400) itrans+2+ntrnsp,itrans
400      format ( /'** Subblock 6.',i2,' turbulence equation',i2// )
         write(irefwr,110) (iinsol(i,2+itrans+maxtrans),i=1,15)
         write(irefwr,111) (iinsol(i,2+itrans+maxtrans),i=16,25)
         write(irefwr,120) (rinsol(i,2+itrans+maxtrans),i=1,25)
      end do

      end
