      subroutine iswrtif6 ( iintur, rintur, iincom, rincom, iincav,
     +                      rincav, iinfrsur, rinfrsur, iglobstr,
     +                      rglobstr, ilocstr, rlocstr, maxclust,
     +                      icluster, idimens )
c ======================================================================
c
c        programmer     Guus Segal
c        version 5.1    date 21-12-1999 Adapted write in case of compressible
c        version 5.0    date 29-04-1999 Extra parameters for structure
c        version 4.0    date 18-03-1999 Extra parameters iinfrsur/rinfrsur
c        version 3.8    date 04-03-1999 Extension with rincav(5)
c        version 3.7    date 17-02-1999 Adaptation texts
c        version 3.6    date 11-08-1998 Extension iincom(8)
c        version 3.5    date 29-07-1998 Extension iincav(2), iincom(7)
c        version 3.4    date 31-04-1998 Extension iincav(1)
c        version 3.3    date 02-03-1998 Extension compressible flow
c        version 3.2    date 01-12-1997 Extension compressible flow
c        version 3.1    date 15-09-1997 Introduction les-model
c        version 3.0    date 08-09-1997 New parameters for cavity
c        version 2.7    date 29-06-1997 Extension compressible flow
c        version 2.6    date 09-06-1997 Extension compressible flow
c        version 2.5    date 10-09-1996 Extension turbulence parameters
c        version 2.4    date 23-08-1996 Extension turbulence parameter
c        version 2.3    date 23-05-1996 Extension turbulence parameter
c        version 2.2    date 22-05-1996 Extension modification production
c        version 2.1    date 19-02-1996 Extension with compressible upwind
c        version 2.0    date 07-02-1995 Extension with information about
c                                       compressible flow
c        version 1.0    date 27-09-1994
c
c   copyright (c) 1994-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c    Write seventh part of the intermediate file
c    It concerns the blocks 11 to 14
c **********************************************************************
c
c                       KEYWORDS
c
c    intermediate_file
c    structure
c    write
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iintur(100), iincom(100), iincav(100), iinfrsur(100),
     +        maxclust, iglobstr(50), ilocstr(maxclust,20), icluster(*),
     +        idimens(*)
      double precision rintur(100), rincom(100), rincav(100),
     +                 rinfrsur(100), rglobstr(10), rlocstr(maxclust,10)

c     icluster  i    Array in which information about the clusters is stored
c                    icluster(i), i=1(1)nclusters contains the starting
c                    position of information about cluster i
c                    In this starting position the number of equations is
c                    stored, followed by the equation sequence numbers
c     idimens   i    Integer dimension parameters for the intermediate file
c     iglobstr  i    Contains information of the global structure of the program
c                    See the programmers Guide for a description
c     iincav    i    reserved
c     iincom    i    reserved
c     iinfrsur  i    Contains integer input with respect to the free surface
c                    computation
c     iintur    i    reserved
c     ilocstr   i    Contains information of the structure of the main program
c                    for each cluster
c                    See the programmers Guide for a description
c     maxclust  i    Maximum number of clusters
c     rglobstr  i    Contains information of the global structure of the program
c                    See the programmers Guide for a description
c     rincav    i    reserved
c     rincom    i    reserved
c     rinfrsur  i    Contains real input with respect to the free surface
c                    computation
c     rintur    i    reserved
c     rlocstr   i    Contains information of the structure of the main program
c                    for each cluster
c                    See the programmers Guide for a description
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, ipcluster, length, nclusters

c     i         General loop variable
c     ipcluster Starting address of cluster in array icluster
c     j         General loop variable
c     length    Length of the array in words
c     nclusters Number of clusters
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c     This subroutine  writes the following parts to the intermediate file
c
c     Block 11: Parameters with respect to turbulence
c     Block 12: Parameters with respect to compressible flow
c     Block 13: Parameters with respect to the cavity model
c     Block 14: Parameters with respect to the output
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    write array iintur
c    write array rintur
c    write array iincom
c    write array rincom
c    write array iincav
c    write array rincav
c    write output parameters
c    write structure arrays
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- write array iintur

      write(irefwr,100) (iintur(i),i=1,100)
100   format ( /'**** BLOCK 11: Parameters with respect to the',
     +          ' turbulence modelling'//
     +          '* iintur:'//
     +          'nturb   = ', i5/
     +          'mturb   = ', i5/
     +          'viscorr = ', i5/
     +          'maniso  = ', i5/
     +          'les_model = ', i5/
     +          'nwindow_max = ', i5/
     +          'nwinstar = ', i5/
     +          'nwinend = ', i5/
     +          'icount = ', i5/
     +          i2,8x, ' # Positions  9-10 not yet defined'/
     +          10i2, '  # Positions 11-20 not yet defined'/
     +          10i2, '  # Positions 21-30 not yet defined'/
     +          10i2, '  # Positions 31-40 not yet defined'/
     +          10i2, '  # Positions 41-50 not yet defined'/
     +          10i2, '  # Positions 51-60 not yet defined'/
     +          10i2, '  # Positions 61-70 not yet defined'/
     +          10i2, '  # Positions 71-80 not yet defined'/
     +          10i2, '  # Positions 81-90 not yet defined'/
     +          10i2, '  # Positions 91-100 not yet defined' )

c     --- write array rintur

      write(irefwr,110) (rintur(i),i=1,10)
110   format ( /'* rintur :'//
     +          'kappa      =', d15.7/
     +          'E          =', d15.7/
     +          'c_mu       =', d15.7/
     +          'sigma_k    =', d15.7/
     +          'sigma_eps  =', d15.7/
     +          'ceps_one   =', d15.7/
     +          'ceps_two   =', d15.7/
     +          'c_D        =', d15.7/
     +          'sigma_T    =', d15.7/
     +          'A          =', d15.7 )
      write(irefwr,120) (rintur(i),i=11,20)
120   format (  'E_rough    =', d15.7/
     +          'eta_zero   =', d15.7/
     +          'betarng    =', d15.7/
     +          'c_s        =', d15.7/
     +          'alpha      =', d15.7/
     +          'beta_star  =', d15.7/
     +          'beta       =', d15.7/
     +          'sigma      =', d15.7/
     +          'sigma_star =', d15.7/
     +          'mod_prod   =', d15.7 )
      write(irefwr,130) (rintur(i),i=21,30)
130   format (  'ceps_three =', d15.7/
     +          'ctau_one   =', d15.7/
     +          'ctau_two   =', d15.7/
     +          'ctau_three =', d15.7/
     +          'timwindow  =', d15.7/
     +           5f4.0, 16x, '  # Positions 26-30 not yet defined' )
      write(irefwr,140) (rintur(i),i=31,100)
140   format (  10f4.0, '  # Positions 31-40 not yet defined'/
     +          10f4.0, '  # Positions 41-50 not yet defined'/
     +          10f4.0, '  # Positions 51-60 not yet defined'/
     +          10f4.0, '  # Positions 61-70 not yet defined'/
     +          10f4.0, '  # Positions 71-80 not yet defined'/
     +          10f4.0, '  # Positions 81-90 not yet defined'/
     +          10f4.0, '  # Positions 91-100 not yet defined'/ )

c     --- write array iincom

      write(irefwr,160) (iincom(i),i=1,100)
160   format ( /'**** BLOCK 12: Parameters with respect to the',
     +          ' compressibility'//
     +          '* iincom:'//
     +          'mcom   = ', i5/
     +          'mcomupw = ', i5/
     +          'mequat = ', i5/
     +          'no_enthalpy = ', i5/
     +          'no_scaling = ', i5/
     +          'pres_corr_method = ', i5/
     +          'type of upwind = ', i5/
     +          'implicit/explicit = ', i5/
     +          'equation of state = ', i5/
     +          'energy equation = ', i5/
     +          10i2, '  # Positions 11-20 not yet defined'/
     +          10i2, '  # Positions 21-30 not yet defined'/
     +          10i2, '  # Positions 31-40 not yet defined'/
     +          10i2, '  # Positions 41-50 not yet defined'/
     +          10i2, '  # Positions 51-60 not yet defined'/
     +          10i2, '  # Positions 61-70 not yet defined'/
     +          10i2, '  # Positions 71-80 not yet defined'/
     +          10i2, '  # Positions 81-90 not yet defined'/
     +          10i2, '  # Positions 91-100 not yet defined' )

c     --- write array rincom

      write(irefwr,180) (rincom(i),i=1,20)
180   format ( /'* rincom :'//
     +          'gamma      =', d15.7/
     +          'Mach       =', d15.7/
     +          'VEL_sound  =', d15.7/
     +          'alpha     =', d15.7/
     +          'p_v       =', d15.7/
     +          'p_out     =', d15.7/
     +          'p_ref     =', d15.7/
     +          'rho_ref   =', d15.7/
     +          'vel_ref   =', d15.7/
     +          f4.0, 24x,'  # Position 10 not yet defined'/
     +          10f4.0, '  # Positions 11-20 not yet defined'/ )
      write(irefwr,140) (rincom(i),i=21,100)

c     --- write array iincav

      write(irefwr,190) (iincav(i),i=1,100)
190   format ( /'**** BLOCK 13: Parameters with respect to the',
     +          ' cavity model'//
     +          '* iincav:'//
     +          'pressure_gauss_seidel = ', i2/
     +          'start_up = ', i2/
     +           8i2, '  # Positions  3-10 not yet defined'/
     +          10i2, '  # Positions 11-20 not yet defined'/
     +          10i2, '  # Positions 21-30 not yet defined'/
     +          10i2, '  # Positions 31-40 not yet defined'/
     +          10i2, '  # Positions 41-50 not yet defined'/
     +          10i2, '  # Positions 51-60 not yet defined'/
     +          10i2, '  # Positions 61-70 not yet defined'/
     +          10i2, '  # Positions 71-80 not yet defined'/
     +          10i2, '  # Positions 81-90 not yet defined'/
     +          10i2, '  # Positions 91-100 not yet defined' )

c     --- write array rincav

      write(irefwr,200) (rincav(i),i=1,20)
200   format ( /'* rincav :'//
     +          'cliquid  =', d15.7/
     +          'cvapour  =', d15.7/
     +          'lowerpresstransit =', d15.7/
     +          'upperpresstransit =', d15.7/
     +          'delta_rho  =', d15.7/
     +          5f4.0, 24x,'  # Positions 6-10 not yet defined'/
     +          10f4.0, '  # Positions 11-20 not yet defined'/ )
      write(irefwr,140) (rincav(i),i=21,100)

c     --- write array iinfrsur

      write(irefwr,210) (iinfrsur(i),i=1,100)
210   format ( /'**** BLOCK 13: Parameters with respect to the',
     +          ' free surface model'//
     +          '* iinfrsur:'//
     +          'method_free_surface = ', i2/
     +           9i2, '  # Positions  2-10 not yet defined'/
     +          10i2, '  # Positions 11-20 not yet defined'/
     +          10i2, '  # Positions 21-30 not yet defined'/
     +          10i2, '  # Positions 31-40 not yet defined'/
     +          10i2, '  # Positions 41-50 not yet defined'/
     +          10i2, '  # Positions 51-60 not yet defined'/
     +          10i2, '  # Positions 61-70 not yet defined'/
     +          10i2, '  # Positions 71-80 not yet defined'/
     +          10i2, '  # Positions 81-90 not yet defined'/
     +          10i2, '  # Positions 91-100 not yet defined' )

c     --- write array rinfrsur

      write(irefwr,220) (rinfrsur(i),i=1,20)
220   format ( /'*  rinfrsur:'//
     +          'relaxation = ', d15.7/
     +          9f4.0, '  # Positions 2-10 not yet defined'/
     +          10f4.0, '  # Positions 11-20 not yet defined'/ )
      write(irefwr,140) (rinfrsur(i),i=21,100)

      write(irefwr,230)
230   format ( /'**** BLOCK 14: Parameters with respect to the',
     +          ' output'//
     +          ' integer array'//
     +          ' 0 0 0 0 0  0 0 0 0 0',
     +          '  # Positions  1-10 not yet defined'/
     +          ' 0 0 0 0 0  0 0 0 0 0',
     +          '  # Positions 11-20 not yet defined'/
     +          ' 0 0 0 0 0  ',
     +          10x, '  # Positions 21-25 not yet defined'//
     +          ' real array'//
     +          ' 0 0 0 0 0  0 0 0 0 0',
     +          '  # Positions  1-10 not yet defined'/
     +          ' 0 0 0 0 0  0 0 0 0 0',
     +          '  # Positions 11-20 not yet defined'/
     +          ' 0 0 0 0 0  ',
     +          12x, '  # Positions 21-25 not yet defined' )

c     --- Write structure information

      write(irefwr,600) (iglobstr(i),i=1,25)
600   format ( /'Global parameters containing the structure of the',
     +          ' main program'//
     +          'ISTATIONARY = ',i2/
     +          'FREE_SURFACE_FLOW  = ',i2/
     +          'ITER_INITIAL_CONDITION  = ',i2/
     +          'MINITER STATIONARY  = ',i5/
     +          'MAXITER STATIONARY  = ',i5/
     +          i2,7x,' # Position  6 not yet defined'/
     +          'MINITER INITIAL_CONDITION  = ',i5/
     +          'MAXITER INITIAL_CONDITION  = ',i5/
     +          2i2,7x,' # Positions  9-10 not yet defined'/
     +          10i2,7x,' # Positions 11-20 not yet defined'/
     +          10i2,7x,' # Positions 21-30 not yet defined'/
     +          10i2,7x,' # Positions 31-40 not yet defined'/
     +          10i2,7x,' # Positions 41-50 not yet defined' )

      write(irefwr,610) (rglobstr(i),i=1,10)
610   format ( /'Real global parameters'//
     +          'ABS_ACCURACY STATIONARY  = ',d15.7/
     +          'REL_ACCURACY STATIONARY  = ',d15.7/
     +          'ABS_ACCURACY INITIAL_CONDITION  = ',d15.7/
     +          'REL_ACCURACY INITIAL_CONDITION  = ',d15.7/
     +          6d12.1,7x,' # Positions  5-10 not yet defined' )

c     --- Write array icluster

      nclusters = idimens(8)
      ipcluster = icluster(nclusters)
      length = ipcluster+icluster(ipcluster)
      write(irefwr,620) (icluster(i),i=1,length)
620   format ( /'Information about the clusters'//
     +          (10i5) )

c     --- Write array ilocstr

      write(irefwr,630)
630   format ( /'The array ilocstructure'// )
      do i = 1, nclusters
         write(irefwr,640) (ilocstr(i,j),j=1,20)
640      format ( 10i5 )
      end do

c     --- Write array rlocstr

      write(irefwr,650)
650   format ( /'The array rlocstructure'// )
      do i = 1, nclusters
         write(irefwr,660) (rlocstr(i,j),j=1,10)
660      format ( 5d15.7 )
      end do

      end
