      subroutine isasknxt
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.0    date   21-06-94 (Subroutine extracted from the SEPRAN
c                                        subroutine ASKNXT version 1.5)
c
c
c
c   copyright (c) 1994  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                         DESCRIPTION
c
c    read the next item (keyword, string or number) from the input
c
c    If at the call of ISASKNXT ityprd is set equal to 1 (common cseprd), a
c    new record is read
c    If ispecl = 4 at input, also the occurrence of a - sign not followed
c    by a digit and the occurrence of a ) sign is marked
c    If ispecl = 5 at input, the occurrence of a ( sign is marked
c    If ispecl = 3 at input, the complete text is interpreted as a string
c
c ********************************************************************
c
c                      KEYWORDS
c
c   read
c   read_cards
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none

c ********************************************************************
c
c                      COMMON BLOCKS
c
      include 'IScommon/csepch'
      include 'IScommon/cseprd'
c ********************************************************************
c
c             SUBROUTINES CALLED
c
      logical isasklet

c     ISASKLET  Check if character is a capital letter
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISREADCD  Read next card
c     ISREADNM  Read number
c     ISREADQT  Read string between quotes
c     ISREADWR  Read keyword
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      logical ready

c     ready     Indicates if a while loop is finished or not
c ********************************************************************
c
c                    I/O
c
c    Possibly information is read from the standard input file
c
c ********************************************************************
c
c                    ERROR MESSAGES
c
c   -
c ********************************************************************
c
c                    PSEUDO CODE
c
c    ready := false
c    If ( ityprd=1 ) then
c       Read next record
c    while ( not ready ) do
c       Find next non-blank in record
c       if ( icolum > 80 ) then
c          if ( istop=2 ) then
c             Read next record
c             Set kcolum
c             if ( ityprd=0 ) then
c                ready := true
c          else (istop<2)
c             ityprd := 1
c             ready := true
c       else if ( ispecl=3 ) then
c          Put text in string
c          ityprd := 5
c       else if ( next character is letter ) then
c          Read string
c          ityprd := 4
c          ready := true
c       else if ( next character is digit, plus or minus followed by digit
c                 or followed by + followed by digit ) then
c          Read number
c          Set ityprd
c          ready := true
c       else if ( next character is ' ) then
c          Read string until end quote is found
c          ityprd := 5
c          ready := true
c       else if ( ispecl=4 and next character is - ) then
c          ityprd := 6
c          ready := true
c       else if ( ispecl=4 and next character is ) ) then
c          ityprd := 7
c          ready := true
c       else if ( ispecl=5 and next character is ( ) then
c          ityprd := 8
c          ready := true
c       else if ( ispecl=8 and next character is = ) then
c          ityprd := 9
c          ready := true
c       else if ( next two characters are // ) then
c          Read next record
c ======================================================================
c
      call iseropen ( 'isasknxt' )
      ready = .false.
      if ( ityprd.eq.1 ) then

c     --- ityprd = 1, i.e. end of record found, read next record

         call isreadcd

      end if

100   if ( .not. ready ) then

c     --- Find next non-blank in record

110      if ( icolum.gt.80 ) then

            if ( istop.eq.2 ) then

c           --- istop = 2, read next record

               call isreadcd
               if ( ityprd.eq.0 ) ready = .true.

            else

c           --- istop < 2, end of record reached, return

               ityprd = 1
               ready = .true.

            end if

         else if ( ispecl.eq.3 ) then

c        --- Special situation, the complete text is considered as a string

            nchar = 81-icolum
            string(1:nchar) = cards(icolum:80)
            ityprd = 5
            ready = .true.

         else if ( cards(icolum:icolum).eq.' ' ) then

c        --- non-blank not yet found, raise icolum and check again

            icolum = icolum + 1
            go to 110

         else if ( isasklet(cards(icolum:icolum)) ) then

c        --- Letter found, read keyword

            call isreadwr
            ready = .true.

         else if ( icolum.lt.80 .and. (index('.+-',cards(icolum:icolum))
     +        .gt.0 .and. cards(icolum+1:icolum+1).ge.'0' .and.
     +        cards(icolum+1:icolum+1).le.'9'
     +        .or. index('+-',cards(icolum:icolum)).gt.0 .and.
     +        index('.',cards(icolum+1:icolum+1)).gt.0
     +        .and. cards(icolum+2:icolum+2).ge.'0' .and.
     +        cards(icolum+2:icolum+2).le.'9' ) ) then

c        --- . + or - followed by digit found, read number

               call isreadnm
               ready = .true.

         else if ( cards(icolum:icolum).ge.'0' .and.
     +             cards(icolum:icolum).le.'9' ) then

c        --- Digit found, read number

            call isreadnm
            ready = .true.

         else if ( cards(icolum:icolum).eq.'''' ) then

c        --- ' found, read text till end quote has been found

            call isreadqt
            ready = .true.

         else if ( ispecl.eq.4 .and. cards(icolum:icolum).eq.'-' ) then

c        --- - found

            ityprd = 6
            icolum = icolum + 1
            ready = .true.

         else if ( ispecl.eq.4 .and. cards(icolum:icolum).eq.')' ) then

c        --- ) found

            ityprd = 7
            icolum = icolum + 1
            ready = .true.

         else if ( ispecl.eq.5 .and. cards(icolum:icolum).eq.'(' ) then

c        --- ( found

            ityprd = 8
            icolum = icolum + 1
            ready = .true.

         else if ( ispecl.eq.8 .and. cards(icolum:icolum).eq.'=' ) then

c        --- = found

            ityprd = 9
            icolum = icolum + 1
            ready = .true.

         else if ( icolum.lt.80 .and. cards(icolum:icolum+1).eq.'//' )
     +        then

c        --- // found, read next record

            call isreadcd

         else

c        --- other character found skip

            icolum = icolum + 1

         end if

         go to 100

      end if

      call iserclos ( 'isasknxt' )
      end
