      subroutine ischsort ( keysrt, kgrad, nkey, iworkch, nworkch )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date   21-06-94 (Extracted from the SEPRAN subroutine
c                                        chsort version 2.2)
c
c
c   copyright (c) 1994  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c******************************************************************************
c
c                         DESCRIPTION
c
c     ischsort sorts an integer array keysrt in ascending order.
c     nkey is the number of array entries.
c     kgrad contains on exit the permutation that sorts sortkey.
c     This implementation uses the work array iworkch as a
c     scratch pad. length nworkch. For maximum efficiency nworkch should be
c     equal to nkey, but values > .25 * nkey are acceptable. Making nworkch
c     even smaller results in poor performance.
c******************************************************************************
c
c                      KEYWORDS
c
c     sort
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer nkey, nworkch, keysrt(nkey), kgrad(nkey), iworkch(*)

c     iworkch       Work array of at least length nkey/4.
c                   For maximum efficiency length should be nkey
c     keysrt   i    Sortkey (integer) on array to be sorted.
c     kgrad    o    Contains permutation index that sorts keysrt
c                   in ascending order on output.
c     nkey     i    Number of entries in array to be sorted
c     nworkch  i    Declared dimension of iworkch
c ********************************************************************
c
c                     COMMON BLOCKS
c
c     none
c ********************************************************************
c
c                   LOCAL PARAMETERS
c
      integer i, iub, ip2, ilo, k

c     i         General loop variable
c     ilo       Lower bound of lower of two array parts to be
c               merged.
c     ip2       The total length of two array parts to be merged in
c               the k-th pass.
c     iub       Upper bound of number of passes to be performed.
c               Each pass being defined as a partial merge of the
c               whole array.
c     k         General loop variable
c ********************************************************************
c
c                            I / O
c
c     none
c ********************************************************************
c
c                      SUBROUTINES CALLED
c
      integer isinlog2

c     ISCHSRT0  to merge two partially sorted array parts
c     ISINLOG2  Compute the truncated base 2 logarithm of an integer
c ********************************************************************
c
c                    ERROR MESSAGES
c
c     -
c ********************************************************************
c
c                    PSEUDO CODE
c
c     Implementation of Quicksort. The algorithm consists of repeated
c     calls to merge on partially sorted array parts. Starting out
c     with kgrad (i) = i, i = 1...nkey, merge is called with array
c     parts of length 1, 2, 4, 8 .. until this exceeds nkey/2.
c     After that the array is sorted.
c     Number of operations : O (n log n).
c ====================================================================
c       Initialize permutation vector

      do 100 i = 1, nkey
         kgrad (i) = i
100   continue
      if ( nkey.gt.1 ) then

c        --- initialize array length of partially sorted arrays

         ip2 = 1

c        --- initialize number of passes

         iub = isinlog2 (nkey - 1) + 1
         do 200 k = 1, iub
            ip2 = ip2 * 2
            do 150 ilo = 1, nkey, ip2

c           --- Assuming adjacent array parts of length ip2/2 are already sorted
c               now sort adjacent array parts of length ip2.

                call ischsrt0 ( keysrt, ilo, ip2, nkey, kgrad, iworkch,
     +                          nworkch)
150         continue
200      continue

      end if
      end
