      subroutine isdelarr ( index )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.2    date   23-09-1999 Correction print format
c        version 1.1    date   08-03-1996 Put name of array in message
c        version 1.0    date   21-06-1994 Extracted from the SEPRAN subroutine
c                                         ini076 version 1.2
c
c   copyright (c) 1994-1999  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                         DESCRIPTION
c
c     Delete a permanent array from IBUFFR
c
c     The array is removed from array infor
c
c     If possible IBFREE and IINFOR are decreased
c     The name of the calling subroutine is extracted from common blocks
c     CISMESSC and CISMESSG, so it is supposed that the subroutine ISEROPEN has
c     been used
c
c **********************************************************************
c
c                      KEYWORDS
c
c     memory_management
c **********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS

      implicit none
      integer index

c     index  i    Integer variable containing the sequence number of the
c                 array to be processed
c **********************************************************************
c
c                    COMMON BLOCKS

      include 'IScommon/cisblank'
      include 'IScommon/cisnas'
      include 'IScommon/cdebug'
      include 'IScommon/cismessg'
      include 'IScommon/cismessc'
      include 'IScommon/cisnamtab'
c **********************************************************************
c
c                    LOCAL PARAMETERS

      integer i, last, iextra, iinfor, ibfree

c     i       General loop variable
c     ibfree  Next free position in array IBUFFR
c     iextra  Extra space needed at both sides of the array for control
c             integers
c     iinfor  Number of arrays that has been referenced
c     last    Last position used by array in IBUFFR corresponding to index
c **********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message file
c     ISERRSUB  Error messages
c     ISINICHK  Check IBUFFR
c     ISSTOP    Stop ISNAS
c     ISWARSUB  Print warnings
c **********************************************************************
c
c                    I/O
c
c     none
c **********************************************************************
c
c                    ERROR MESSAGES
c
c    1403   array corresponding to index does not exist
c    1404   array corresponding to index has already been deleted
c **********************************************************************
c
c                    PSEUDO CODE
c
c    if ( idebug>0 ) then
c       Check ibuffr
c    if ( index<0 or index>iinfor ) then
c       error index out of range
c    if ( array in IBUFFR ) then
c       if ( array is last one ) then
c          Decrease ibfree
c    else if ( array not in file 3 ) then
c       warning: array has already been deleted
c    if ( index=iinfor ) then
c       recompute iinfor
c    if ( idebug>1 ) then
c       Set new free space equal to infinity
c    Clear array from infor
c ======================================================================
c
      iinfor = ibuffr(3)
      ibfree = ibuffr(4)
      if ( idebug.eq.0 ) then
         iextra = 0
      else
         iextra = intlen
         ifirws = 0
         call isinichk ( ibuffr, ibuffr(11) )
      end if

      if ( index.lt.0 .or. index.gt.iinfor ) then

c     --- index < 0 or index > iinfor  array does not exist

         call iseropen ( 'isdelarr' )
         call iserrint ( index, 1 )
         call iserrint ( iinfor+1, 2 )
         call iserrsub ( 1403, 2, 0, 0 )
         call iserclos ( 'isdelarr' )
         call isstop

      end if
      if ( index.eq.0 ) go to 1100

      if ( ibuffr(11+3*(index-1)).gt.0 ) then

c     --- Array is positioned in IBUFFR, check if IBFREE may be decreased

         last = ibuffr(11+3*(index-1)) + ibuffr(12+3*(index-1)) +
     +          iextra*2
         if ( ibfree.eq.last .or. ibfree.eq.last+1 ) then

c        --- Recompute ibfree

            ibfree = 1
            do 100 i = 1, iinfor
               if ( i.ne.index .and. ibfree.le.ibuffr(11+3*(i-1)) )
     +              ibfree = ibuffr(11+3*(i-1)) + ibuffr(12+3*(i-1)) +
     +              iextra*2

100         continue
            if ( intlen.eq.2 .and. mod(ibfree,2).eq.0 ) ibfree=ibfree+1

         end if

      else if ( ibuffr(11+3*(index-1)).eq.0 ) then

c     --- Warning array has already been deleted

         call iseropen ( 'isdelarr' )
         call iserrint ( index, 1)
         call iswarsub ( 1404, 1, 0, 0 )
         call iserclos ( 'isdelarr' )
         go to 1000

      end if

      if ( index.eq.iinfor ) then

c     --- Recompute iinfor

         do 110 i = index-1, 1, -1
            if ( ibuffr(11+3*(i-1)).ne.0 ) then
               iinfor = i
               go to 120
            end if
110      continue

      end if

c     --- Clear array infor for this array

120   if ( idebug.gt.1 ) then

c     --- idebug > 1, set IBUFFR to IINFIN

         do 140 i = ibuffr(11+3*(index-1))-1, ibuffr(11+3*(index-1))+
     +              ibuffr(12+3*(index-1))
            ibuffr(i) = iinfin
140      continue

      end if

      do 150 i = 1, 3
         ibuffr(10+i+3*(index-1)) = 0
150   continue

      if ( ioutpt.gt.2 ) then
         write(irefwr,200) namarrays(index), index,
     +	                   namsubs(levelsub)//':isdelarr'
200      format ( ' Delete array ', a, ' with sequence number', i4,
     +            ' in subroutine ', a )
      end if

1000  index = 0
      ibuffr(3) = iinfor
      ibuffr(4) = ibfree

1100  continue

      end
