      subroutine isererrl ( errlin, ints, nints, reals, nreal, chars,
     +                      nchar, numint, numrea, numcha, ipttrg, line)
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.4    date   01-10-1997 Guus: extension with ~
c        version 1.3    date   11-12-1995 Kees K postion 80 : lnsize+1
c        version 1.2    date   13-05-1995 Correction for reals
c        version 1.1    date   30-04-1995 Improve layout
c        version 1.0    date   31-01-1994 Extracted from the SEPRAN subroutine
c                                         ererrl version 2.4
c
c     Copyright (c) 1994-1997 "Ingenieursbureau SEPRA"
c     Permission to copy or distribute this software or documentation
c     in hard copy or soft copy granted only by written license
c     obtained from "Ingenieursbureau SEPRA"
c     All rights reserved. No part of this publication may be
c     reproduced, stored in a retrieval system (e.g., in memory, disk,
c     or core) or be transmitted by any means, electronic, mechanical,
c     photocopy, recording, or otherwise, without written Permission
c     from the publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     ISERERRL prints one line of an error message, thereby substituting
c     integers, reals and character strings.
c     If $, & or % is preceded by ~, the rest of the corresponding integers
c     reals and strings is printed as a list from new lines
c **********************************************************************
c
c                       KEYWORDS
c
c     error_manipulation
c     print
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      character*(*) chars(*), errlin
      character*160 line
      integer ints(*), ipttrg, nchar, nints, nreal, numcha, numint,
     +        numrea
      double precision reals(*)

c     chars   i    Extra information as character strings.
c     errlin  i    One line of the error message
c     ints    i    Extra information as integers.
c     ipttrg  i/o  Pointer to first position not filled in LINE
c     line    i/o  Text line
c     nchar   i    Number of strings in a message
c     nints   i    Number of integers in a message
c     nreal   i    Number of reals in a message
c     numcha  i    Number of character strings in error message.
c     numint  i/o  Number of integers in error message
c     numrea  i/o  Number of reals in error message
c     reals   i    Extra information as reals.
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      character fmtcha*7, fmtint*7, fmtout*8, fmtrea*18
      logical hrdret
      integer i, iendpo, ifact, ifpt, klast, ilpt, iptend, iptsrc,
     +        istapo, leneff, length, lnsize, numdec, itype, ih
      logical nxtsgn, poinfn, tildef

c     fmtcha  Variable format to write string to line
c     fmtint  Variable format to write integer to line
c     fmtout  Variable format to write line to stdout
c     fmtrea  Variable format to write real to line
c     hrdret  Logical whether hard return is found at end of errlin
c     i       General loop variable
c     iendpo  End position of number in line
c     ifact   First 10 power greater than passed integer
c     ifpt    Position of the first non-blank character in ERRLIN
c     ih      Help variable to store a constant temporarily
c     klast   Last blank on position 80 or before, first non-blank beyond
c             found last blank
c     ilpt    Position of the last  non-blank character in ERRLIN
c     iptend  Last non blank character in line from error text file.
c     iptsrc  Pointer to current character in line from error text file.
c     istapo  Start position of number in line
c     itype   Array containing the type of actions to be undertaken for
c             each subkeyword found.
c             See subroutine isintx02 for the possible values of itype
c     leneff  Effective string length, i.e. without trailing blanks.
c     length  Length of the array in words
c     lnsize  Length of output line
c     numdec  Number of decimals of real variable
c     nxtsgn  True: next character is $- or &-sign
c     poinfn  True if period was found in series of &-signs
c     tildef  If true: a tilde has been found
c **********************************************************************
c
c                       I/O
c
c     ISERERRL  writes to the default output device.
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     None.
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     None.
c **********************************************************************
c
c                       PSEUDO CODE
c
c     tilde_found := false
c     Search last non blank in line from error text file
c     While not at last character of line from error text file do
c        itype := 0
c        Take next character
c        Case next character of
c        -Character == ~ : tilde_found := true
c        -Character == $ and tilde_found
c           Fill rest of line with spaces
c           itype := 1
c        -Character == & and tilde_found
c           Fill rest of line with spaces
c           itype := 2
c        -Character == % and tilde_found
c           Fill rest of line with spaces
c           itype := 3
c        -Character == $ : substitute with passed integer
c           tilde_found := false
c           Goto end of sequence of $-signs and count them
c           Take next passed integer
c           If there is one then
c              Determine available space and if it is not enough determine
c                  the required space
c              Make format descriptor for writing integer to line buffer
c              Write integer to line buffer
c              Set pointer to line buffer
c           Endif
c        -Character == & : substitute with passed real
c           tilde_found := false
c           Goto end of sequence of &-signs and count them
c           Take next passed real
c           If there is one then
c              Determine available space and if it is not enough determine
c                  the required space
c              If point was found then f-format
c                 Read number of &-signs after period
c                 Make format descriptor for writing real to line buffer
c              Else
c                 Make sure length is enough
c                 Make format descriptor for writing real to line buffer
c              Endif
c              Write real to line buffer
c              Set pointer to line buffer
c        -Character == % : substitute with passed character string
c           tilde_found := false
c           Goto end of sequence of %-signs and count them
c           Take next passed character string
c           If there is one then
c              Determine available space and if it is not enough determine
c                  the required space
c              Determine the effective string length
c                  (=length without trailing blanks).
c              Make format descriptor for writing character string
c                  to line buffer
c              Write character string to line buffer
c              Set pointer to line buffer
c        -Character == @ : hard return, fill with spaces
c           tilde_found := false
c        -other : ordinary character, copy it to line buffer
c           tilde_found := false
c        If a complete line is filled then
c           Search last blank on position 80 or before
c           Write line
c           Search first non-blank beyond found last blank
c           Shift not printed part of line to beginning
c           Reset remaining part of line
c           if ( tilde_found ) then
c              Print list of items depending on itype
c           tilde_found := false
c        Endif
c     Endwhile
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- use 79 columns only

      lnsize = 79

c     --- Move errlin one position to the left

      do i = 1, lnsize
         ih = i+1
         errlin(i:i) = errlin(ih:ih)
      end do

c     --- search last non blank in line from error text file

      iptsrc = 0
      iptend = len(errlin)
      hrdret = .false.
      tildef = .false.
100   if ( iptend.gt.0 ) then
         if ( errlin(iptend:iptend).eq.'@' ) then
            hrdret = .true.
            iptend = iptend - 1
            goto 100
         else if ( errlin(iptend:iptend).eq.' ' ) then
            iptend = iptend - 1
            goto 100
         end if
      end if
      if ( hrdret ) then
         iptend = iptend + 1
         errlin(iptend:iptend) = '@'
      end if
      itype  = 0

c     --- while not at last character of line from error text file

110   if ( iptsrc.le.iptend ) then

c     --- take next character

         iptsrc = iptsrc + 1
         ipttrg = ipttrg + 1

c        --- case next character of

         if ( errlin(iptsrc:iptsrc).eq.'~' ) then

c        --- tilde found

            tildef = .true.
            line(ipttrg:ipttrg) = errlin(iptsrc:iptsrc)

         else if ( tildef .and. errlin(iptsrc:iptsrc).eq.'$' ) then

c        --- ~$ found

            itype = 1
            line(ipttrg-1:160) = ' '
            ipttrg = lnsize+1

         else if ( tildef .and. errlin(iptsrc:iptsrc).eq.'&' ) then

c        --- ~& found

            itype = 2
            line(ipttrg-1:160) = ' '
            ipttrg = lnsize+1

         else if ( tildef .and. errlin(iptsrc:iptsrc).eq.'%' ) then

c        --- ~% found

            itype = 3
            line(ipttrg-1:160) = ' '
            ipttrg = lnsize+1

         else if ( errlin(iptsrc:iptsrc).eq.'$' ) then

c        --- character == $ : substitute with passed integer

            tildef = .false.
            istapo = iptsrc
            nxtsgn = .true.

c           --- goto end of sequence of $-signs

150         if (nxtsgn) then
               iptsrc = iptsrc + 1
               if (errlin(iptsrc:iptsrc).eq.'$') then
                  nxtsgn = .true.
               else
                  nxtsgn = .false.
               end if
               goto 150
            end if

c           --- take next passed integer

            iptsrc = iptsrc - 1
            iendpo = iptsrc
            numint = numint + 1
            if (numint.le.nints) then

c           --- determine available space and if it is not enough determine
c               the required space

               length = iendpo - istapo + 1
               if ( ints(numint).ge.0 ) then
                  if ( ints(numint).ge.10.0**length ) then
                     length = 0
                     ifact  = 1
160                  length = length + 1
                     ifact  = ifact * 10
                     if ( iinfin / ifact.ge.10 ) then
                        if ( ints(numint).ge.ifact ) goto 160
                     else
                        length = length + 1
                     end if
                  end if
               else
                  if ( abs(ints(numint)).ge.10.0**(length-1) ) then
                     length = 1
                     ifact  = -1
170                  length = length + 1
                     ifact  = ifact * 10
                     if ( iinfin / (-ifact).ge.10 ) then
                        if ( ints(numint).le.ifact ) goto 170
                     else
                        length = length + 1
                     end if
                  end if
               end if

c              --- make format descriptor for writing integer to line buffer

               write (fmtint,'(''(i'',i4,'')'')') length

c              --- write integer to line buffer

               istapo = ipttrg
               iendpo = istapo+length-1
               write (line(istapo:iendpo),fmtint) ints(numint)

c              --- set pointer to line buffer

               ipttrg = ipttrg + length - 1
            end if

         else if (errlin(iptsrc:iptsrc).eq.'&' ) then

c        --- character == & : substitute with passed real

            tildef = .false.
            istapo = iptsrc
            nxtsgn = .true.
            poinfn = .false.

c           --- goto end of sequence of &-signs

200         if (nxtsgn) then
               iptsrc = iptsrc + 1
               if (errlin(iptsrc:iptsrc).eq.'&') then
                  nxtsgn = .true.
               else
                  if ( errlin(iptsrc:iptsrc).eq.'.' ) then
                     if ( iptsrc.lt.iptend ) then
                        ih = iptsrc+1
                        if ( errlin(ih:ih).eq.'&' ) then
                           poinfn = .true.
                        end if
                     end if
                  end if
                  nxtsgn = .false.
               end if
               goto 200
            end if

c           --- take next passed real

            iptsrc = iptsrc - 1
            iendpo = iptsrc
            numrea = numrea + 1
            if (numrea.le.nreal) then

c           --- determine available space and if it is not enough determine
c               the required space

               length = iendpo - istapo + 1

c              --- if point was found then f-format

               if ( poinfn ) then
                  iptsrc = iptsrc + 1

c                 --- read number of &-signs after period

                  nxtsgn = .true.
210               if ( nxtsgn ) then
                     iptsrc = iptsrc + 1
                     if ( errlin(iptsrc:iptsrc).ne.'&' ) then
                        nxtsgn = .false.
                     end if
                     goto 210
                  end if

c                 --- make format descriptor for writing real to line buffer

                  numdec = length
                  iptsrc = iptsrc - 1
                  length = iptsrc - iendpo - 1
                  write ( fmtrea, '(''(f'',i3,''.'',i3,'')'')' )
     +                  numdec+length+1, length
                  length = numdec+length+1

               else

c              --- make sure length is enough

                  if ( length.eq.1 ) length = 10
                  length = max ( 7, length )+1

c                 --- make format descriptor for writing real to line buffer

                  numdec = length - 7
                  if ( reals(numrea).lt.0 ) length = length + 1
                  write (fmtrea,'(''(e'',i4,''.'',i3,'')'')')
     +                   length, numdec
               end if

c              --- write real to line buffer

               istapo = ipttrg
               iendpo = istapo+length-1
               write (line(istapo:iendpo),fmtrea) reals(numrea)

c              --- set pointer to line buffer

               ipttrg = ipttrg + length - 1
            end if

         else if (errlin(iptsrc:iptsrc).eq.'%' ) then

c        --- character == % : substitute with passed character string

            tildef = .false.
            istapo = iptsrc
            nxtsgn=.true.

c           --- goto end of sequence of %-signs

250         if (nxtsgn) then
               iptsrc = iptsrc + 1
               if (errlin(iptsrc:iptsrc).eq.'%') then
                  nxtsgn = .true.
               else
                  nxtsgn = .false.
               end if
               goto 250
            end if

c           --- take next passed character string

            iptsrc = iptsrc - 1
            iendpo = iptsrc
            numcha = numcha + 1
            if (numcha.le.nchar) then

c           --- determine available space and if it is not enough determine
c               the required space

               length = iendpo - istapo + 1

c              --- determine the effective string length
c                  (= length without trailing blanks).

               leneff = len(chars(numcha))
270            if ( leneff.gt.0 ) then
                  if ( chars(numcha)(leneff:leneff).eq.' ' ) then
                     leneff = leneff - 1
                     goto 270
                  end if
               end if
               if (length.lt.leneff) then
                  length = leneff
                  iendpo = istapo+length-1
               end if

c              --- make format descriptor for writing character string
c                  to line buffer

               write (fmtcha,'(''(a'',i4,'')'')') max(1,leneff)

c              --- write character string to line buffer

               istapo = ipttrg
               iendpo = istapo+length-1
               write (line(istapo:iendpo),fmtcha) chars(numcha)

c              --- set pointer to line buffer

               ipttrg = ipttrg + length - 1

            end if

         else if (errlin(iptsrc:iptsrc).eq.'@' ) then

c        --- character == @ : hard return, fill with spaces

            tildef = .false.
            line(ipttrg:160) = ' '
            ipttrg = lnsize

         else

c        --- other : ordinary character, copy it to line buffer

            tildef = .false.
            line(ipttrg:ipttrg) = errlin(iptsrc:iptsrc)

         end if

c        --- if one line is filled then

         if ( ipttrg.gt.lnsize ) then

c        --- search last blank on position 80 or before

            klast = lnsize+1
300         if ( line(klast:klast).ne.' ' ) then
               klast = klast - 1
               if ( klast.gt.0 ) goto 300
            end if

c           --- search last non-blank before last blank

310         if ( line(klast:klast).eq.' ' ) then
               klast = klast - 1
               if ( klast.gt.0 ) goto 310
            end if
            if ( klast.eq.0 ) klast=lnsize

c           --- write line ( if not blank )

            if ( line(1:klast).ne.' ' ) then
               if ( icarcn.eq.0 ) then
                  write (fmtout,'(''(1x,a'',i2,'')'')') max(1,klast)
                  write (irefwr,fmtout) line(1:klast)
               else
                  write (fmtout,'(''(2x,a'',i2,'')'')') max(1,klast)
                  write (irefwr,fmtout) line(1:klast)
               end if
            else
               ifpt = 1
320            if ( ( errlin(ifpt:ifpt).eq.' ' ) .and.
     +              ( ifpt.le.iptend ) ) then
                  ifpt = ifpt + 1
                  goto 320
               end if
               ilpt = iptend
               if ( errlin(ifpt:ilpt).eq.'@' ) write ( irefwr, * )
            end if

c           --- search first non-blank beyond found last blank

            klast = klast + 1
330         if ( line(klast:klast).eq.' ' ) then
               klast = klast + 1
               if ( klast.le.ipttrg ) goto 330
            end if

c           --- shift not printed part of line to beginning

            do i = klast, ipttrg
               ih = i-klast+1
               line(ih:ih) = line(i:i)
            end do

c           --- reset remaining part of line

            ipttrg = ipttrg - klast + 1
            line(ipttrg+1:160) = ' '
            if ( tildef ) then

c           --- tilde found, print list depending on itype

               if ( itype.eq.1 ) then
                  write(irefwr,400) (ints(i),i=numint+1,nints)
400               format(6x,4i8,4x,4i8)
                  numint = nints
               else if ( itype.eq.2 ) then
                  write(irefwr,410) (reals(i),i=numrea+1,nreal)
410               format((6x,ss,5(1pe14.6e2)))
                  numrea = nreal
               else if ( itype.eq.3 ) then
                  write(irefwr,420) (chars(i),i=numcha+1,nchar)
420               format(6x,a)
                  numcha = nchar
               end if
            end if
            if ( ipttrg.eq.0 ) then
               tildef = .false.
            else if ( line(ipttrg:ipttrg).ne.'~' ) then
               tildef = .false.
            end if

         end if

c        --- endwhile

         goto 110

      end if

      if ( line.eq.' ' ) ipttrg = 0

      end
