      subroutine isini100 ( index, ibuffr, infor, inback, lenbac )
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.0    date   21-06-94 (Extracted from the SEPRAN subroutine
c                                        ini100 version 1.5)
c
c
c   copyright (c) 1994  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                         DESCRIPTION
c
c   write array with reference number index from ibuffr to file 3
c   change contents of infor, inback and lenbac
c ********************************************************************
c
c                      KEYWORDS
c
c     memory_management
c     backing_storage
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer index, ibuffr(*), infor(3,*), inback(*), lenbac(*)

c     ibuffr i/o reserved
c     inback i/o Integer array of length nvectr in which of each array that
c                is referenced in array INFOR is given
c                whether the last version is also stored on backing storage
c                inback(i) > 0, or not inback(i) <= 0.
c                When inback(i) > 0, the value of inback(i) gives the record
c                number from which array i is stored,
c                if inback(i) = 0, the array has never been stored on file 3,
c                and if inback(i) < 0, then |inback(i)| gives the record
c                number from which a preceding version of array has been
c                stored
c     index   i  Integer variable containing the sequence number of the
c                array to be processed
c     infor  i/o Information concerning the storage of arrays in IBUFFR
c                is stored in the following way:
c                Pos. (1,i)  If > 0: array i is stored in IBUFFR from position
c                            stored in infor(1,i)
c                            If < 0: array i is stored in file 3 from record
c                            number -infor(1,i)
c                Pos. (2,i)  Length of the array in words
c                Pos. (3,i)  Priority number. Is used to decide which arrays
c                            are put to file 3. 0 <= prio <= 10
c     lenbac i/o Integer array of length nvectr corresponding to inback
c                For each ARRAY i with INBACK(i)#0, LENBAC(i) contains the
c                the number of records that are used by the last version of
c                array i that has been written to file 3
c ********************************************************************
c
c                      COMMON BLOCKS
c
      include 'IScommon/cisnas'
      include 'IScommon/cdebug'
      include 'IScommon/cfile3'
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer ipstar, iextra, length, irecnr, irecn, numrec, i

c     i       General loop variable
c     iextra  Extra space needed at both sides of the array for control
c             integers
c     ipstar  Computed starting position in array IBUFFR
c     irecn   Local record number
c     irecnr  Record number in direct access file
c     length  Length of the array in words
c     numrec  Number of records to be written
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISWRITFL  Write to direct access file
c ********************************************************************
c
c                    I/O
c
c    none
c ********************************************************************
c
c                    ERROR MESSAGES
c
c   -
c ********************************************************************
c
c                    PSEUDO CODE
c
c      ipstar := infor(1,index)
c      if ( ipstar>0 ) then
c         irecnr := inback(index)
c         if ( irecnr>0 ) then
c            infor(1,index) := -inback(index)
c         else if ( irecnr=0 ) then
c            write array to backing storage from position ifree3
c         else ( irecnr<0 )
c            write array to backing storage and check the start position
c ======================================================================
c
      ipstar = infor(1,index)
      if ( ipstar.gt.0 ) then

c     --- ipstar > 0, array is in IBUFFR

         if ( idebug.eq.0 ) then
            iextra = 0
         else
            iextra = intlen
         end if
         length = infor(2,index)+iextra*2
         irecnr = inback(index)
         if ( irecnr.gt.0 ) then

c        --- irecnr > 0,  no need of writing of array, since array is already
c                         on file 3
c                         positions in ibuffr are free to be used

            infor(1,index) = -inback(index)
            go to 1000

         else if ( irecnr.eq.0 ) then

c        --- irecnr  =  0   write array to backing storage from position ifree3

            irecn = ifree3
            inback(index) = irecn
            numrec = (length-1)/nrec3+1
            lenbac(index) = numrec

         else

c        --- irecnr < 0    write array to backing storage
c                          check the start position

            numrec = (length-1)/nrec3+1
            if ( numrec.le.lenbac(index) ) then

c           --- number of records is not larger than before
c               write on old positions

               irecn = -inback(index)
               inback(index) = irecn

            else

c           --- number of records is too large to store on old positions
c               store from record ifree3

               irecn = ifree3
               inback(index) = irecn
               lenbac(index) = numrec

            end if

         end if

         infor(1,index) = -irecn
         call iswritfl ( irecn, ipstar-iextra,ipstar+length-1-iextra,
     +                   3, ibuffr )
         if ( idebug.ge.2 ) then
            do 100 i = ipstar-iextra, ipstar+length-1-iextra
               ibuffr(i) = iinfin
100         continue
         end if

      end if

1000  continue
      end
