      subroutine isinichk ( ibuffr, infor )
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.1    date   03-10-94 (Use name in names table for error)
c        version 1.0    date   21-06-94 (Extracted from the SEPRAN subroutine
c                                        isinichk version 1.3)
c
c
c   copyright (c) 1994  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                         DESCRIPTION
c
c   Check all control integers in array IBUFFR
c   Set tail of IBUFFR to infinity if necessary
c ********************************************************************
c
c                      KEYWORDS
c
c   debug
c   memory_management
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ibuffr(*), infor(3,*)

c     ibuffr  i    reserved
c     infor   i    Information concerning the storage of arrays in IBUFFR
c                  is stored in the following way:
c                  Pos. (1,i)  If > 0: array i is stored in IBUFFR from position
c                              stored in infor(1,i)
c                              If < 0: array i is stored in file 3 from record
c                              number -infor(1,i)
c                  Pos. (2,i)  Length of the array in words
c                  Pos. (3,i)  Priority number. Is used to decide which arrays
c                              are put to file 3. 0 <= prio <= 10
c ********************************************************************
c
c                      COMMON BLOCKS
c
      include 'IScommon/cisnas'
      include 'IScommon/cdebug'
      include 'IScommon/cisnamtab'
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      integer i, ipoint, ihbuf, iarsv, nbuffr, nvectr, iinfor

c     i         General loop variable
c     iarsv     Help variable to compute the new value of narbuf
c     ihbuf     Help parameter to store one of the nbufw2/3 values
c     iinfor    Number of arrays that has been referenced
c     ipoint    Address in array IBUFFR
c     nbuffr    Length of buffer array ibuffr
c     nvectr    Maximum number of arrays that may be stored in IBUFFR
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERRINT  Put integer value in error message
c     ISERRSUB  Error messages
c     ISSTOP    Stop ISNAS
c ********************************************************************
c
c                    I/O
c
c    none
c ********************************************************************
c
c                    ERROR MESSAGES
c
c    1407   Leading control integer incorrect
c    1408   Trailing control integer incorrect
c    1409   Control integers of work space incorrect
c    1410   Control integers of work space created by isendspc incorrect
c ********************************************************************
c
c                    PSEUDO CODE
c
c   For i := 1 (1) iinfor
c      if ( infor(1,i) > 0 ) then
c         check if array i is preceded and followed by control integer i
c   if ( idebug>1 and nbuffr>nbufls ) then
c      Set last part of IBUFFR to iinfin
c ======================================================================
c
c     --- Check control integers

      nvectr = ibuffr(2)
      iinfor = ibuffr(3)
      nbuffr = ibuffr(7)
      nbufmx = max(nbuffr,nbufmx)
      do 100 i = 1, iinfor
         if ( infor(1,i).gt.0 ) then

c        --- Array i is stored in IBUFFR

            ipoint = infor(1,i) - 1
            if ( ibuffr(ipoint).ne.i ) then

c           --- Error 1407: leading control integer incorrect

               call iseropen ( 'isinichk' )
               call iserrchr ( namarrays(i), 1 )
               call iserrint ( ibuffr(ipoint), 1 )
               call iserrint ( i, 2 )
               call iserrsub ( 1407, 2, 0, 1 )
               call iserclos ( 'isinichk' )
               call isstop

            end if

            ipoint = infor(1,i) + infor(2,i)
            if ( ibuffr(ipoint).ne.i ) then

c           --- Error 1408  Trailing control integer incorrect

               call iseropen ( 'isinichk' )
               call iserrchr ( namarrays(i), 1 )
               call iserrint ( ibuffr(ipoint), 1 )
               call iserrint ( i, 2 )
               call iserrsub ( 1408, 2, 0, 1 )
               call iserclos ( 'isinichk' )
               call isstop

            end if

         end if

100   continue

      if ( nbufw1.gt.0 ) then

c     --- nbufw1 > 0, check work space

         if ( nbufw3.eq.0 ) then
            ihbuf = nbufw2
         else
            ihbuf = nbufw3
         end if
         if ( ibuffr(nbufw1).ne.nvectr+1 .or. ibuffr(ihbuf).ne.nvectr+1)
     +       then
            call iseropen ( 'isinichk' )
            call iserrint ( ibuffr(nbufw1), 1 )
            call iserrint ( ibuffr(ihbuf), 2 )
            call iserrint ( nvectr+1, 3 )
            call iserrsub ( 1409, 3, 0, 0 )
            call iserclos ( 'isinichk' )
            call isstop
         end if

      end if

      if ( narbuf.gt.0 .and. nbuffr.ge.iarbuf(narbuf,1) ) then

c     --- Some of the arrays have been removed, decrease narbuf

         iarsv = 0
         do 110 i = narbuf, 1, -1
            if ( nbuffr.lt.iarbuf(i,1) ) then
               iarsv = i
               go to 120
            end if
110      continue
120      narbuf = iarsv

      end if

      if ( narbuf.gt.0 ) then

c     --- narbuf > 0, check the narbuf arrays created by isendspc

         do 130 i = 1, narbuf
            if ( ibuffr(iarbuf(i,1)).ne.1600+i .or.
     +           ibuffr(iarbuf(i,2)).ne.1600+i ) then
               call iseropen ( 'isinichk' )
               call iserrint ( i, 1 )
               call iserrint ( ibuffr(iarbuf(i,1)), 2 )
               call iserrint ( ibuffr(iarbuf(i,2)), 3 )
               call iserrint ( 1600+i, 4 )
               call iserrsub ( 1410, 4, 0, 0 )
               call iserclos ( 'isinichk' )
               call isstop
            end if
130      continue

      end if

      nbufls = min(nbufls,nbuffr)

      if ( idebug.gt.1 .and. nbufls.gt.nbuffr ) then

c     --- Last part of IBUFFR has not yet been set to IINFIN

         do 140 i = nbufls+1, nbuffr
            ibuffr(i) = iinfin
140      continue
         nbufls = nbuffr

      end if

      if ( idebug.gt.0 .and. ifirws.eq.0 .and. nbufw1.gt.0 ) then

c     --- Work space of IBUFFR has not yet been set to IINFIN

         if ( idebug.gt.1 ) then
            do 150 i = nbufw1, nbufw2
               ibuffr(i) = iinfin
150         continue
         end if
         nbufw1 = 0
         nbufw2 = 0
         nbufw3 = 0

      end if
      end
