      subroutine isreadsr ( dnumbr, maxnum, inum )
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.0    date   21-06-94 (Subroutine extracted from the SEPRAN
c                                        subroutine READSR version 1.1)
c
c
c
c   copyright (c) 1994  "Ingenieursbureau SEPRA"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "Ingenieursbureau SEPRA".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                         DESCRIPTION
c
c   read a series of numbers
c   The numbers may be alternated with texts, which are skipped
c   The reading of numbers is stopped if
c
c    a ) sign is found
c    an end of record is found
c    maxnum numbers have been read
c
c   If a minus sign followed by text followed by a number is read,
c   the number is made negative
c
c   After reading the numbers, icolum points to the next position to be read
c
c ********************************************************************
c
c                      KEYWORDS
c
c   read
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer maxnum, inum
      double precision dnumbr(maxnum)

c     dnumbr  o  Work array to store a series numbers that have been read
c     inum    o  Indicates the number of numbers that have been read
c     maxnum  i  Maximum number of numbers to be read
c ********************************************************************
c
c                      COMMON BLOCKS

      include 'IScommon/cseprd'
c ********************************************************************
c
c                    LOCAL PARAMETERS
c
      logical ready, minsgn
      integer i, kcolsv

c     i         General loop variable
c     kcolsv    Help parameter to save the old value of kcolum
c     minsgn    Indicates if a minus sign has been found (true) or not (false)
c     ready     Indicates if a while loop is finished or not
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISASKNXT  Read next item from the standard input file
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c ********************************************************************
c
c                    I/O
c
c    none
c ********************************************************************
c
c                    ERROR MESSAGES
c
c   -
c ********************************************************************
c
c                    PSEUDO CODE
c
c   Clear array dnumbr
c   ready  := false
c   inum   := 0
c   ispecl := 4
c   minsgn := false
c
c   while ( not ready ) do
c      Read next item (ISASKNXT)
c      if ( ityprd = 2 or 3 ) then
c         inum := inum+1
c         if ( minsgn ) then realrd := -realrd
c         minsgn := false
c      else if ( ityprd = 6 ) then
c         minsgn := true
c      else if ( ityprd = 7 or 0 or 1 ) then
c         ready := true
c      if ( inum = maxnum ) then
c         ready := true
c ======================================================================
c
c     --- Make some initializations

      call iseropen ( 'isreadsr' )

c     --- Clear array dnumbr

      do 100 i = 1, maxnum
         dnumbr(i) = 0
100   continue

      ready  = .false.
      inum   = 0
      ispecl = 4
      minsgn = .false.
      kcolsv = kcolum

200   if ( .not. ready ) then

c     --- While loop
c         Read next item

         call isasknxt

         if ( ityprd.eq.2 .or. ityprd.eq.3 ) then

c        --- ityprd = 2 or 3, number found

            inum = inum + 1
            if ( minsgn .and. realrd.gt.0 ) realrd = - realrd
            minsgn = .false.
            dnumbr(inum) = realrd

         else if ( ityprd.eq.6 ) then

c        --- ityprd = 6, minus sign found

            minsgn = .true.

         else if ( ityprd.le.1 .or. ityprd.eq.7 ) then

c        --- ityprd = 0, 1 or 7, i.e. end of file, end of record or
c            ) sign found

            ready = .true.

         end if

         if ( inum.ge.maxnum ) then

c        --- inum = maxnum, stop reading

            ready = .true.

         end if

         go to 200

      end if
      ispecl = 0
      kcolum = min ( kcolum, kcolsv )

      call iserclos ( 'isreadsr' )

      end
