      subroutine isstarmm ( nbuffr, nvectr )
c=======================================================================
c
c     programmer      Kees Kassels
c     version  1.1    date    17-02-95 (include icommon/sinidbg)
c     version  1.0    date    21-06-94
c
c
c   copyright (c) 1994  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c***********************************************************************
c
c                   DESCRIPTION
c
c   Initialize memory management for the buffer array
c   The administration is set and if debug_level=2, the array is set to infinity
c***********************************************************************
c
c                   KEYWORDS
c
c    memory_management
c***********************************************************************
c
c                   INPUT / OUTPUT PARAMETERS

      integer nbuffr, nvectr

c     nbuffr    i    Length of buffer array ibuffr
c     nvectr    i    Maximum number of arrays that may be stored in IBUFFR
c***********************************************************************
c
c                   COMMON BLOCKS
c
c
      include 'IScommon/cisblank'
      include 'IScommon/cisnas'
      include 'IScommon/cdebug'
c***********************************************************************
c
c                   LOCAL PARAMETERS
c
      integer firstf, i

c     firstf  First free position in IBUFFR
c     i       General loop variable
c***********************************************************************
c
c                   SUBROUTINES CALLED
c
c     ISERRINT  Put integer value in error message file
c     ISERRSUB  Error messages
c     ISINIDBG  Read file isnas.dbg and initialize cdebug
c***********************************************************************
c
c                   I/O
c
c
c***********************************************************************
c
c                   ERROR MESSAGES
c
c    1421   NBUFFR out of range
c    1422   NVECTR out of range
c***********************************************************************
c
c                   PSEUDO CODE
c
c
c    Check nbuffr and nvectr
c    Initialize administration
c    if ( debug_level = 2 ) then
c       Set ibuffr equal to infinity
c=======================================================================
c
c     --- Check parameters nbuffr and nvectr

      if ( nbuffr.lt.10000 .or. nbuffr.gt.1000000000 ) then

c     --- nbuffr out of range

          call iserrint ( nbuffr, 1 )
          call iserrint ( 10000, 2 )
          call iserrint ( 1000000000, 3 )
          call iserrsub ( 1421, 3, 0, 0 )

      end if

      if ( nvectr.lt.100 .or. nvectr.gt.10000 ) then

c     --- nvectr out of range

          call iserrint ( nvectr, 1 )
          call iserrint ( 100, 2 )
          call iserrint ( 10000, 3 )
          call iserrsub ( 1422, 3, 0, 0 )

      end if

c     --- Read debug file isnas.dbg if present

      call isinidbg ( nbuffr )

c     --- Initialize the first part of array IBUFFR

      ibuffr(1) = nbuffr
      ibuffr(2) = nvectr
      ibuffr(3) = 0
      firstf = 3*nvectr+12
      if ( intlen.eq.2 .and. mod(firstf,2).eq.0 ) firstf = firstf + 1
      ibuffr(4) = firstf
      ibuffr(5) = firstf
      ibuffr(6) = firstf
      ibuffr(7) = nbuffr-6*nvectr
      nbufmx = ibuffr(7)

      do 100 i = 8, 3*nvectr+10
         ibuffr(i) = 0
100   continue

      if ( idebug.ge.2 ) then

c     --- idebug >= 2, set IBUFFR to infinity

         do 200 i = 11+3*nvectr, nbuffr-6*nvectr
            ibuffr(i) = iinfin
200      continue

      end if

      end

