      subroutine dinvu(x,b,matrix,n,ndimso,nconct,
     1                  prec,nprec,infmat)
c ==================================================================
c
c
c     programmer       Kees Vuik
c
c     version 2.0      24-01-94 Kees K. change for new structure
c                  
c     version 1.4      date 24-11-1992 Kees V. Calls tinvu if ndim=3.
c     version 1.3      date 18-05-1992
c     developed at     Convex,HP800,HP700
c
c
c ******************************************************************
c
c                       DESCRIPTION
c
c     Multiplication of x by U preconditioning matrix given
c     in matrix and prec.
c
c     In this case we obtain:
c                 -1
c            b = U  x.
c
c     The arrays prec and matrix should be filled by subroutine dmlu.f
c     This subroutine contains compiler directives to run in vector
c     speed on the Convex.
c
c ******************************************************************
c
c                       KEYWORDS
c
c     linear_solver
c     ilud
c     preconditioner
c
c ******************************************************************
c
c                       INPUT / OUTPUT   PARAMETERS
c
c
c     b       o  the result vector, which contains:
c                     -1
c                b = U  x.
c
c     infmat  i  if infmat(1) is 1 we use the momentum equations,
c                whereas if infmat(1) is larger than or equal to 4
c                we use equations, with a structure similar to the
c                pressure equation.
c
c     matrix  i  the coefficient matrix for the momentum or 
c                an equation similar to the pressure equation.
c
c     n       i  number of unknowns in the solution vector.
c
c     nconct  i  number of connections in one row of the matrix
c
c     ndimso  i  integer indicating the dimension of the space in
c                which the problem must be solved (ndimso = 1 or ndim).
c
c     nprec   i  number of diagonals, which are used in the pre-
c                conditioning. In this subroutine nprec = 1.
c
c     prec    i  the preconditioning matrix.
c
c     x       i  the original vector.
c
c ******************************************************************
c
c                       COMMON BLOCKS
c
c
c ******************************************************************
c
c                       LOCAL PARAMETERS
c
c     i       loop counter.
c
c     ii      loop counter.
c
c     j       loop counter.
c
c     jjmax   loop bound.
c
c     jjmin   loop bound.
c
c     ndim    Dimension of space.
c
c     nx      number of points in the x-direction.
c
c     ny      number of points in the y-direction.
c
c ******************************************************************
c
c                       SUBROUTINES CALLED
c
c     tinvu       Computes the multiplication of the inverse of an upper
c                 triangular matrix stored in arrays matrix and prec
c                 and a vector. Prec should be filled by tmlu.f.
c                 This subroutine works for ndim = 3.
c
c ******************************************************************
c
c                       I/O
c
c ******************************************************************
c
c                       ERROR MESSAGES
c
c ******************************************************************
c
c                       PSEUDO CODE
c
c     Multiplication of x by ILU preconditioning matrix given
c     in matrix and prec.
c
c     In this case we obtain:
c                 -1
c            b = U  x.
c
c     The arrays prec and matrix should be filled by subroutine dmlu
c
c     **************************************************************
c
c     Change1 07-02-1991 C. Vuik.
c
c     diag(L) = diag(U) = D = I.
c
c     **************************************************************
c
c     Change2 15-02-1991 C. Vuik.
c
c     The reason of this change is to solve the upper triangular system
c     in vector speed.
c
c     Literature : High performance preconditioning.
c                  H. A. van der Vorst.
c                  SIAM. J. Sci. Stat. Comput. 10 pp. 1174-1185 (1989)
c
c     Implementation : Compare subroutine precd in the program ictest
c                      written by H. A. van der Vorst.
c
c     Remark     : In this version the virtual points along the boundary
c                  are made zero. Thereafter the loop only runs over
c                  the non-boundary points.
c
c     **************************************************************
c
c     Change3 16-12-1991 C. Vuik
c
c     The virtual points are removed.
c
c     **************************************************************
c
c ==================================================================
c
c     Parameters
c
      integer          infmat(*),n,ndimso,nconct,nprec
      double precision matrix(1:n,1:ndimso,1:nconct),
     1                 prec(1:n,1:ndimso,1:nprec),
     2                 x(n,ndimso),b(n,ndimso)
c
c     local parameters
c
      integer i,j,ii,nx,ny,jjmin,jjmax,ndim
      if ( infmat(4) .eq. 0 ) then
        ndim = 2
      else
        ndim = 3
      endif
      if(ndim.eq.3) then
         call tinvu(x,b,matrix,n,ndimso,
     1              nconct,prec,nprec,infmat)
      else
      nx = infmat(2)+1
      ny = infmat(3)+1
c                             -1
c     The multiplication b = U  x
c
c     First we fill the points on the upper horizontal
c     boundary.
c
      b(n,ndimso) = x(n,ndimso)
      do i=2,nx
        b(n+1-i,ndimso)=x(n+1-i,ndimso)-
     .                matrix(n+1-i,ndimso,6)*
     .                b(n+2-i,ndimso)
      enddo
      i = (ny-1)*nx
      b(i,ndimso) = x(i,ndimso)-matrix(i,ndimso,7)*b(i+nx-1,ndimso)
     1                     -matrix(i,ndimso,8)*b(i+nx  ,ndimso)
      if(nx.eq.2*(nx/2)) then
         i = (ny-1)*nx-1
         b(i,ndimso) = x(i,ndimso)-matrix(i,ndimso,6)*b(i+1,ndimso)
     1               -matrix(i,ndimso,7)*b(i+nx-1,ndimso)
     2               -matrix(i,ndimso,8)*b(i+nx  ,ndimso)
     3               -matrix(i,ndimso,9)*b(i+nx+1,ndimso)
      endif
c
c     change2
c
      do ii = ny-2+(nx-1)/2, 1, -1
	 jjmin = max(1,ii+1-nx/2)
	 jjmax = min(ii,ny-1)
c$dir no_recurrence
         do j = jjmin, jjmax
	    i = 2*ii+2-2*j+(j-1)*nx
            b(i,ndimso) = x(i,ndimso)-matrix(i,ndimso,6)*b(i+1,ndimso)
     1                  -matrix(i,ndimso,7)*b(i+nx-1,ndimso)
     2                  -matrix(i,ndimso,8)*b(i+nx  ,ndimso)
     3                  -matrix(i,ndimso,9)*b(i+nx+1,ndimso)
         enddo
	 jjmin = max(1,ii-(nx-1)/2)
	 jjmax = min(ii,ny-1)
c$dir no_recurrence
         do j = jjmin, jjmax
	    i = 2*ii+1-2*j+(j-1)*nx
            b(i,ndimso) = x(i,ndimso)-matrix(i,ndimso,6)*b(i+1,ndimso)
     1                  -matrix(i,ndimso,7)*b(i+nx-1,ndimso)
     2                  -matrix(i,ndimso,8)*b(i+nx  ,ndimso)
     3                  -matrix(i,ndimso,9)*b(i+nx+1,ndimso)
         enddo
      enddo
c
c     if infmat(8).eq.1 we solve the momentum equations. In this case
c     b(i,2) should be calculated too.
c
      if(infmat(8).eq.1) then
c
c     i = 1, n-nx
c
c$dir no_recurrence
      do i = 1, n-nx
	 b(i,1) = x(i,1)
     5   -matrix(i,1,10)*b(i-1   ,2)
     6   -matrix(i,1,11)*b(i     ,2)
     7   -matrix(i,1,12)*b(i+nx-1,2)
     8   -matrix(i,1,13)*b(i+nx  ,2)
      enddo
c
c     i = n-nx+1
c
      i = n-nx+1
      b(i,1) = x(i,1)
     5         -matrix(i,1,10)*b(i-1   ,2)
     6         -matrix(i,1,11)*b(i     ,2)
     7         -matrix(i,1,12)*b(i+nx-1,2)
c
c     i = n-nx+2,n
c
c$dir no_recurrence
      do i = n-nx+2, n
	 b(i,1) = x(i,1)
     5   -matrix(i,1,10)*b(i-1   ,2)
     6   -matrix(i,1,11)*b(i     ,2)
      enddo
c
c     First we fill the points on the upper horizontal
c     boundary.
c
      b(n,1) = b(n,1)
      do i=2,nx
        b(n+1-i,1)=b(n+1-i,1)-
     .                matrix(n+1-i,1,6)*
     .                b(n+2-i,1)
      enddo
      i = (ny-1)*nx
      b(i,1) = b(i,1)-matrix(i,1,7)*b(i+nx-1,1)
     1               -matrix(i,1,8)*b(i+nx  ,1)
      if(nx.eq.2*(nx/2)) then
         i = (ny-1)*nx-1
         b(i,1) = b(i,1)-matrix(i,1,6)*b(i+1,1)
     1               -matrix(i,1,7)*b(i+nx-1,1)
     2               -matrix(i,1,8)*b(i+nx  ,1)
     3               -matrix(i,1,9)*b(i+nx+1,1)
      endif
c
c     change2
c
      do ii = ny-2+(nx-1)/2, 1, -1
	 jjmin = max(1,ii+1-nx/2)
	 jjmax = min(ii,ny-1)
c$dir no_recurrence
         do j = jjmin, jjmax
	    i = 2*ii+2-2*j+(j-1)*nx
	    b(i,1) = b(i,1)
     1      -matrix(i,1,6)*b(i+1   ,1)
     2      -matrix(i,1,7)*b(i+nx-1,1)
     3      -matrix(i,1,8)*b(i+nx  ,1)
     4      -matrix(i,1,9)*b(i+nx+1,1)
        enddo
	 jjmin = max(1,ii-(nx-1)/2)
	 jjmax = min(ii,ny-1)
c$dir no_recurrence
         do j = jjmin, jjmax
	    i = 2*ii+1-2*j+(j-1)*nx
	    b(i,1) = b(i,1)
     1      -matrix(i,1,6)*b(i+1   ,1)
     2      -matrix(i,1,7)*b(i+nx-1,1)
     3      -matrix(i,1,8)*b(i+nx  ,1)
     4      -matrix(i,1,9)*b(i+nx+1,1)
         enddo
      enddo
      endif
      endif
      return
      end
