      subroutine isgsckrs(pold, dpold, rinput, infmat, matrix,
     +                    work, iinput, rhs, jac, residgs, densup,
     +                    nusol, ni, nj, matdp, pinterm1, rhowork,
     +                    rhoint1, differ, dpoldv, iinmtvc,
     +                    nsubfaces, subfaces)
c ======================================================================
c
c     programmer     Duncan van der Heul
c
c     version      3.3    date 25-05-1999   Duncan: modification for low
c                                                   Mach number
c     version      3.2    date 06-05-1999   Duncan: extended for periodical
c                                                   boundary conditions
c     version      3.1    date 23-03-1999   Duncan: inclusion boundary points
c     version      3.0    date 16-11-1998   Duncan: total revision
c     version      2.0    date 22-07-1998   Duncan: extension dens upwind
c                                                   j direction
c                                                   extension for saving
c                                                  coefficients dens upwind
c     version      1.1    date 08-07-1998 Kees V: nusol, nsolut cleaned
c     version      1.0    date 01-04-1998
c
c
c   copyright (c) 1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Subroutine isgsckrs calculates the residual of the iterate by
c   substituting the solution vector into the original non-linear system for
c   the compressible pressure correction. The infinite norm of the difference
c   between the function value of the solution vector and the right hand side
c   is the residual.
c   The total pressure correction is stored in the work array which is used
c   in the non-linear Gauss-Seidel step.
c **********************************************************************
c
c                       KEYWORDS
c
c     compressible
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iinput(*), infmat(*), nusol, ni, nj, iinmtvc(*),
     +        nsubfaces(*), subfaces(*)
      double precision  jac(*),dpold(nusol),
     +                  pold(*), rinput(*),densup(nusol,3,2),
     +                  matrix(*), work(*), rhs(*),
     +                  pinterm1(*), rhoint1(*),differ(nusol),
     +                  rhowork(*),residgs, matdp(*),
     +                  dpoldv(*)

c     densup    i    coefficients of density upwind in pressure matrix
c     differ    i    Difference between Ax and b
c     dpold     i    pressure correction at previous time step
c     dpoldv    i    pressure correction at previous time step with virtual
c                    points
c     iinmtvc   i    reserved
c     iinput    i    reserved
c     infmat    i    Integer information for the matrix
c     jac       i    sqrt(g) in pnttyp
c     matrix    i    reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nsubfaces i    Array containing the number of subfaces of each face,
c     nusol     i    Number of unknowns for one degree of freedom in
c                    one block
c     pinterm1  i    Work array for pressure
c     pold      i    Pressure at the old time level
c     residgs   o    Infinite norm of the residual in the pressure correction
c                    when the non linear Gauss-Seidel algorithm is applied
c     rhoint1   i    Work array for density and density increment
c     rhowork   i    Work array for density
c     rhs       i    Right hand side  (any level)
c     rinput    i    reserved
c     subfaces  i    reserved
c     work     i/o   Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer j, mcomupw, ipiincav, istartup, ipinfmat,
     +        iprincom, iprintim, ipiincom, mscal, iprincav,
     +     idummy(29), totshift(9), periodic, iconct, nconct
      double precision  dt, drhol, drhor, drhob, drhot, dtinv,
     +                  pres1tr, pres2tr
      logical boundaryp

c     boundaryp Point is on the boundary
c     drhob     Increment in density at position 3 in pressure stencil
c     drhol     Increment in density at position 5 in pressure stencil
c     drhor     Increment in density at position 6 in pressure stencil
c     drhot     Increment in density at position 8 in pressure stencil
c     dt        Time step
c     dtinv     1/dt
c     iconct    Loop variable for loop over all connections in the matrix
c     idummy    Integer dummy
c     ipiincav  Starting address of array IINCAV in array IINPUT
c     ipiincom  Starting address of array IINCOM in array IINPUT
c     ipinfmat  Starting address in array infmat
c     iprincav  Starting address of array RINCAV in array RINPUT
c     iprincom  Starting address of array RINCOM in array RINPUT
c     iprintim  Starting address of array RINTIM in array RINPUT
c     istartup  If 1 the cavity calculation is in the startup procedure,
c               if 0 not
c     j         General loop variable
c     matdp     Contribution to the rhs due to -DGdp
c     mcomupw   If zero the rho equation for compressible flow is solved by
c               a central scheme, otherwise by a upwind scheme
c     mscal     Indicates use of non-dimensionalisation for compr. flow:
c               0:   Equations are non-dimensionalised
c               1:   Equations are in dimensional formulation
c     nconct    Number of connections in a row of the matrix
c     periodic  Periodic boundary conditions
c     pres1tr   Lower transit pressure in equation of state
c     pres2tr   Upper transit pressure in equation of state
c     totshift  total shift in the matrix
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
      double precision isdstfll, isdstlss

c     DAXPY      Vector a*X + Y
c     DCOPY      Copy double precision array into other one (BLAS subroutine)
c     ISDSTFLL   Derivative of dimensional equation of state
c     ISDSTLSS   Derivative of dimensionless equation of state
c     ISGSPER    Correct matrix shifts for periodic boundary conditions
c     ISMATVEC   Computes the product of the sparse matrix, stored in amat,
c                and a vector.
c     ISSETCON   Make array equal to a constant
c     ISSTDFL2   Dimensional equation of state
c     ISSTDLS2   Dimensionless equation of state
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   for all points do
c
c          rho(pold+dp)-rho(pold)                     *
c differ = ----------------------- -dt* DGdp+ dt * D m
c                    dt
c
c   end do
c           ||      ||
c   resid = ||differ||
c           ||      ||oo
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      ipiincav   = iinput(11)
      ipiincom   = iinput(9)
      iprincom   = iinput(34)
      iprincav   = iinput(35)
      iprintim   = iinput(29)
      ipinfmat   = infmat(13)
      dt         = rinput(iprintim -1+4)
      dtinv      = 1d0/dt
      mscal      = iinput(ipiincom -1+5)
      mcomupw    = iinput(iinput(9) + 2 - 1)
      nj         = iinput(52)
      istartup   = iinput(ipiincav -1 + 2)
      idummy(29) = 0
      pres1tr    = rinput(iprincav + 3 -1)
      pres2tr    = rinput(iprincav + 4 -1)
      periodic   = iinput(56)
      nconct     = infmat(6)

c     --- reset residual

      residgs  = 0d0
      call issetcon(differ,nusol,0d0)

c     --- calculate the new pressure pold + dpold and store in pinterm1

      call dcopy(nusol,pold, 1, pinterm1, 1)
      call daxpy(nusol,1d0, dpold, 1, pinterm1, 1)

c     --- calculate the new and the old density and store in rhowork and
c              rhoint1 respectively

      if( mscal.eq.0) then
         call isstdls2(pold, rinput(iprincom), istartup,
     +                        rhowork, nusol, rinput(iprincav))
         call isstdls2(pinterm1, rinput(iprincom), istartup,
     +                        rhoint1, nusol, rinput(iprincav))
      else if( mscal.eq.1) then
         call isstdfl2(pold, istartup, rhowork, nusol,
     +                                        rinput(iprincav))
         call isstdfl2(pinterm1, istartup, rhoint1, nusol,
     +                                        rinput(iprincav))
      end if

c     --- calculate pressure matrix times pressure correction and store
c         in matdp

      call issetcon(dpoldv,
     +              (ni+1+2*2)*(nj+1+2*2),0d0)
      call ismatvec ( matrix, dpold, dpoldv, matdp,
     +                   infmat, idummy, iinmtvc )

c     --- calculate increment in density rhonew - rhoold and store in rhoint1

      call daxpy(nusol,-1d0, rhowork, 1, rhoint1,1)

c     --- in the linear part or low Mach number part change
c         delta rho = rho+ - rho- by delta rho = 1/c^2*delta p

      if(mscal.eq.0) then

         do j=1, nusol

               if((pold(j).lt.pres1tr.and.
     +                pold(j)+
     +                   dpold(j).lt.pres1tr).or.
     +            (pold(j).gt.pres2tr.and.
     +                pold(j)+
     +                   dpold(j).gt.pres2tr)) then
                  rhoint1(j)=
     +                          isdstlss(pold(j),
     +                             rinput(iprincom),
     +                             istartup, rinput(iprincav))*
     +                                 dpold(j)
               end if
         end do

      else if(mscal.eq.1) then

         do j=1, nusol

               if((pold(j).lt.pres1tr.and.
     +                pold(j)+
     +                   dpold(j).lt.pres1tr).or.
     +            (pold(j).gt.pres2tr.and.
     +                pold(j)+
     +                   dpold(j).gt.pres2tr)) then
                  rhoint1(j)=
     +                          isdstfll(pold(j),
     +                             istartup, rinput(iprincav))*
     +                                 dpold(j)
               end if
         end do

      end if

c     --- calculate the residual

      do j=1, nj*(ni+1)
        if ( mod(j,ni+1).ne.0 ) then

           differ(j) = matdp(j)+rhoint1(j)*
     +                                jac(j)*dtinv -rhs(j)
        end if
      end do

c     --- add contributions due to density bias

      if (mcomupw.eq.1) then

         do j=1, nj*(ni+1)

            if ( mod(j,ni+1).ne.0 ) then

               do iconct = 2, nconct

                  totshift(iconct)=infmat(ipinfmat-1+iconct)

               end do

               boundaryp= ((mod(j,ni+1).eq.ni) .or.
     +                     (mod(j,ni+1).eq. 1) .or.
     +                     (         j .le.ni) .or.
     +                     (         j .gt.((ni+1)*(nj-1))))
               if( periodic.eq.1 .and. boundaryp) then

                  call isgsper(iinmtvc(1), iinmtvc(4), iinmtvc(5),
     +                iinmtvc(10), iinmtvc(11), iinmtvc(12),
     +                nsubfaces,
     +                subfaces, totshift, j)
               end if

               drhol=rhoint1(j+totshift(5))
               drhor=rhoint1(j+totshift(6))
               drhob=rhoint1(j+totshift(3))
               drhot=rhoint1(j+totshift(8))
               differ(j) = differ(j) +
     +             rhoint1(j)*
     +                  (densup(j,2,1)+densup(j,2,2))
     +           + drhob*densup(j,1,2)
     +           + drhot*densup(j,3,2)
     +           + drhol*densup(j,1,1)
     +           + drhor*densup(j,3,1)

            end if
         end do
      end if

c     --- calculate the infinite norm of the residual vector

      do j=1, nusol
        residgs=max(abs(differ(j)),residgs)
        differ(j)=abs(differ(j))
      end do

      do j=1,(ni+1)*nj
       if( mod(j,ni+1).ne.0) then
        work(j)= dpold(j)
       end if
      end do

      end
