      subroutine ismkprec ( amat, nusol, prec, initer, reiter, work )
c ======================================================================
c
c        programmer     Kees Vuik
c        version 1.5    date 07-07-1998 Kees V: nusol, nsolut cleaned
c        version 1.4    date 25-08-1997 Kees V. Multi-grid prec 3D added
c        version 1.3    date 17-04-1997 Kees V. Multi-grid prec added
c        version 1.2    date 18-03-1996 Kees V. adding ILU in 3D
c        version 1.1    date 15-09-1995 Kees V. adding ILU_Fill
c        version 1.0    date 11-08-1995
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     The subroutine ismkprec is used to build a preconditioner.
c
c **********************************************************************
c
c                       KEYWORDS
c
c     linear_solver
c     preconditioner
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer          nusol, initer(*)
      double precision amat(*), prec(*), reiter(*), work(*)

c     amat      i    The coefficients matrix used in the linear system
c     initer    i    reserved
c     nusol     i    Number of unknowns for one degree of freedom in
c                    one block
c     prec      i    The preconditioning matrix
c     reiter    i    reserved
c     work      i    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer          mkind, nconct, ndim, ndimso, nprec, maparr(9),
     +                 mapvec(19), ni, nj, nk, lprec

c     lprec     Length of array prec
c     maparr    Contains permutation for p00 ... p22 (1..9 by
c                     default)
c     mapvec    Contains permutation (1..19 by default)
c     mkind     Control parameter indicating the preconditioner.
c     nconct    Number of connections in a row of the matrix
c     ndim      Dimension of space (2 or 3)
c     ndimso    Integer indicating the dimension of the space in
c               which the problem must be solved (ndimso = 1 or ndim)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     nprec     Maximum number of diagonals in preconditioning matrix
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     AUTOSIZ3  Calculates required dimension of bufferspace (3D MG)
c     AUTOSIZE  Calculates required dimension of bufferspace (MG)
c     ISDIAG    Builds the diagonal preconditioner.
c     ISDMLU2   Builds the ILUD preconditioner for ndim = 2.
c     ISDMLU3   Builds the ILU preconditioner.
c     ISDMLU4   Builds the ILU_FILL preconditioner.
c     ISTMLU2   Builds the ILUD preconditioner for ndim = 3.
c     ISTMLU3   Builds the ILU preconditioner for ndim = 3.
c     PMAP      Remaps 9 point stencil  (MG)
c     PMAP319   Remaps 19 point stencil  (MG)
c     PRELUDE   Preliminary activities for MG
c     PRELUDE3  Preliminary activities for MG3
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     The subroutine ismkprec is used to build a preconditioner.
c     For the different choices of mkind different subroutines are
c     called.
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data maparr /2, 3, 4, 5, 1, 6, 7, 8, 9/
      data mapvec /10, 11, 12, 13, 14,
     +              2,  3,  4,  5,  1, 6, 7, 8, 9,
     +             15, 16, 17, 18, 19/
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      mkind  = initer(1)
      ndim   = initer(15)
      ndimso = initer(16)
      nprec  = initer(17)
      nconct = initer(18)

      if ( abs(mkind).eq.1 ) then

c     --- Build the diagonal scaling preconditioning matrix.

         call isdiag ( amat, nusol, ndimso, nconct,
     +                 prec, nprec )

      end if

      if(abs(mkind).eq.2) then

c     --- Build a diagonal matrix to obtain an ILU decomposition
c         for preconditioning.

         if ( ndim.eq.2 ) then
            call isdmlu2 ( amat, nusol, ndimso, nconct,
     +                     prec, nprec, initer, reiter )
         else
            call istmlu2 ( amat, nusol, ndimso, nconct,
     +                     prec, nprec, initer, reiter )
         end if
      end if

      if ( abs(mkind).eq.3 ) then

c     --- Build an incomplete LU decomposition of A such that
c         the sparseness pattern of L and U is the same as A.

         if ( ndim.eq.2 ) then
            call isdmlu3 ( amat, nusol, ndimso, nconct,
     +                     prec, nprec, initer, reiter )
         else
            call istmlu3 ( amat, nusol, ndimso, nconct,
     +                     prec, nprec, initer, reiter, work )
         end if
      end if

      if ( abs(mkind).eq.4 ) then

c     --- Build an incomplete LU decomposition of A where fill-in
c         is allowed.

         call isdmlu4 ( amat, nusol, ndimso, nconct,
     +                  prec, nprec, initer, reiter, work )

      end if

      if ( abs(mkind).eq.5 ) then

c     --- Build an multigrid preconditioner.

         ni = initer(12)
         nj = initer(13)
         nk = initer(14)

         if ( ndim.eq.2 ) then

            call pmap (maparr)
            call autosize ( ni, nj, 0, initer(26), lprec, 0, 1, 0, 1 )
            call prelude ( amat, ni, nj, 0, initer(26), prec, lprec,
     +                     0, 1, 0, 1, initer(initer(27)) )

         else

            call pmap319 (mapvec)
            call autosiz3 ( ni, nj, nk, 0, initer(26),
     +                      lprec, 0, 1, 0, 1, 0, 1 )
            call prelude3 ( amat, ni, nj, nk, 0, initer(26), prec,
     +                      lprec, 0, 1, 0, 1, 0, 1,
     +                      initer(initer(27)) )

         end if
      end if

      end
