      subroutine ispri109 ( igsinf, rgsinf )
c ======================================================================
c
c        programmer     Duncan van der Heul
c        version 1.0    date  06-04-1998
c
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Prints information from the non-linear Gauss-Seidel algorithm for
c     solving the pressure equation.
c
c **********************************************************************
c
c                       KEYWORDS
c
c    print
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer          igsinf(*)
      double precision rgsinf(*)

c     igsinf    i    Array with integer information regarding non-linear
c                    Gauss-Seidel
c     rgsinf    i    Array with real information regarding non-linear
c                    Gauss-Seidel
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c

      integer nogauss, maxiter, mitergs, itergs , method, iprings
      double precision tollbsec, tollnlgs, residgs    

c     iprings   Control parameter indicating the amount of output.
c     itergs    Number of the iteration in the mixed non linear Gaus-Seidel/
c               GMRES algorithm
c     maxiter   Maximum number of iterations
c     method    Control parameter indicating the iterative method.
c     mitergs   Maximum number of iterations in non-linear Gauss-Seidel
c     nogauss   Nogauss=1, convergence in non-linear Gauss-Seidel
c               Nogauss=0, no convergence in non-linear Gauss-Seidel
c     residgs   Infinite norm of the residual in the pressure correction
c               when the non linear Gaus-Seidel algorithm is applied
c     tollbsec  Lokal tolerance for the bisection steps
c     tollnlgs  Global tolerance for the whole non-linear Gauss-Seidel 
c               Algorithm
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     If nogauss is equal to zero convergence of the solution was achieved
c     within the desired accuracy, both the number of iterations needed and
c     the final residual of the non linear pressure eqaution are printed to 
c     the screen.
c     If nogauss is equal to one the maximum number of iterations was reached
c     in the outer loop before convergence was achieved with the required
c     accuracy. Both a message and the achieved residual are printed. 
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'ispri109' )

      nogauss   = igsinf(1)
      mitergs   = igsinf(2)
      itergs    = igsinf(3)
      maxiter   = igsinf(4)
      method    = igsinf(5)
      iprings   = igsinf(6)
      tollbsec  = rgsinf(1)
      tollnlgs  = rgsinf(2)
      residgs   = rgsinf(3)


c     --- The start of the non-linear Gauss-Seidel output

      if ( iprings.ge.1 .or. iprings.eq.0 .and.
     +     nogauss.ne.0 ) then
         if ( method.eq.0 ) write ( irefwr, 1000 )
     +                         ' '
         if ( method.eq.1 ) write ( irefwr, 1000 ) 
     +                         ' accelerated with GMRES'
      end if

c     --- Further start information is given

      if ( iprings.ge.2 .or. iprings.eq.0 .and.
     +     nogauss.ne.0 ) then
          write(irefwr, 1010 ) tollbsec
          write(irefwr, 1011 ) mitergs
          write(irefwr, 1020 ) tollnlgs
      end if

c     --- The initial and final residual are given

      if ( iprings.ge.1 ) then
         write( irefwr, 1030) itergs , residgs
      else if (iprings.eq.0.and.nogauss.ne.0 ) then
         write( irefwr, 1040) maxiter, residgs
      end if


1000  format(/, ' Non-linear Gauss-Seidel ', a, /,
     +  '------------------------------------------------------')
1010  format(/, ' The lokal tolerance in the bisection steps is :',
     +                       e15.5)
1011  format( ' The maximum number of Gauss-Seidel ', 
     +              'iterations per step is :',
     +                       i5)
1020  format( ' The global tolerance for non-linear ', 
     +             'Gauss-Seidel is :',e15.5)
1030  format(/,' The number of executed iterations is :', i5, /,
     +         ' The inf-norm of the final residual is  :', e15.5,/)
1040  format(/,' Non-Linear Gauss-Seidel failed to achieve ',
     +           'convergence to the required accuracy ', /,
     +         ' The number of executed iterations is :', i5, /,
     +         ' The inf-norm of the final residual is  :', e15.5,/)

      call iserclos ( 'ispri109' )

      end
