      subroutine isputs ( iequa, solut, nsolut, work, infmat, coupled)
c ======================================================================
c
c     programmer    Kees Kassels
c
c     version       3.3  07-07-1998  Kees V: nusol, nsolut cleaned
c     version       1.2  13-10-1997  Charles: uncoupled allowed
c     version       1.1  28-07-1994  Zijlema: correction in view of 3D
c                                  (at line 98)
c     version       1.0  25-01-1994
c
c
c   copyright (c) 1994-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     This subroutine puts the solution computed by ISLSOL
c     from array WORK into array SOLUT at the right positions
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iequa, nsolut, infmat(*)
      double precision solut(*), work(*)
      logical coupled

c     coupled i    Logical which is true if the momentum equations
c                  are coupled
c     iequa   i    General equation sequence number
c     infmat  i    Integer information for the matrix
c     nsolut  i    Number of unknowns for all degrees of freedom in one block
c     solut   i    reserved
c     work    i    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, idegfd, istart, ni, nj, nk, ndim, nusol

c     i            General loop variable
c     idegfd       Loop variable with respect to a given unknown
c     istart       Indicates how the iteration process in started
c     ndim         Dimension of space (2 or 3)
c     ni           Number of non-virtual cells in the i-direction (one block)
c     nj           Number of non-virtual cells in the j-direction (one block)
c     nk           Number of non-virtual cells in the k-direction (one block)
c     nusol        Number of unknowns for one degree of freedom in
c                  one block
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

c     --- Put solution from work into solut

      ni = infmat(2)
      nj = infmat(3)
      nk = infmat(4)

      if ( nk.eq.0 ) then
         ndim = 2
      else
         ndim = 3
      end if

      nusol = (ni+1)*(nj+1)*(nk+1)

      if ( coupled ) then

         if( iequa.eq.1)  idegfd = 1

         if( iequa.ge.2)  idegfd = iequa + ndim - 1

      else

         idegfd = iequa

      end if

c     --- the computed solution must be stored at level two

c         --- Notice: the reverse copying is in subroutine isextr

      istart =  nsolut/2 + (idegfd-1)*nusol

c     --- Copy solution into solut

      if ( infmat(8).eq.1  ) then

c     --- Coupled velocity unknown

         do 100 i=1, ndim*nusol

            solut(istart+i) = work(i)

 100     continue

      else

c     --- Other type of equations

         do 140 i = 1, nusol
            solut(istart+i) = work(i)
 140     continue

      end if

      end
