      subroutine istinvu3 ( bresul, nusol, ndimso,
     +                      prec, nprec, initer )
c ======================================================================
c
c        programmer     Kees Vuik
c        version 1.1    date 07-07-1998 Kees V: nusol, nsolut cleaned
c        version 1.0    date 11-05-1995 extracted from old ISNaS
c                       subroutine tinvu2.
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Multiplication of x by U preconditioning matrix given
c     in amat and prec.
c     In this case we obtain:
c                 -1
c            b = U  x.
c     The array amat and prec should be filled by subroutine istmlu2.f
c     This subroutine contains compiler directives to run in vector
c     speed on the Convex.
c     Warning
c     -------
c     In this subroutine it is possible that shifts correctly point
c     into another degree of freedom, when the momentum equations
c     are solved.
c **********************************************************************
c
c                       KEYWORDS
c
c     linear_solver
c     preconditioner
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer          initer(*),nusol,ndimso,nprec
      double precision prec(1:nusol,1:ndimso,1:nprec),
     +                 bresul(nusol, ndimso)

c     bresul   i/o   The result vector
c     initer    i    reserved
c     ndimso    i    Integer indicating the dimension of the space in
c                    which the problem must be solved (ndimso = 1 or ndim)
c     nprec     i    Maximum number of diagonals in preconditioning matrix
c     nusol     i    Number of unknowns for one degree of freedom in
c                    one block
c     prec      i    The preconditioning matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i,j,k,m,nx,ny,npxy,nz

c     i         General loop variable
c     j         General loop variable
c     k         General loop variable
c     m         General loop variable
c     npxy      Product of nx and ny
c     nx        Number of grid x points on finest grid
c     ny        Number of grid y points on finest grid
c     nz        Number of points in the z-direction
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      nx = initer(12)+1
      ny = initer(13)+1
      nz = initer(14)+1
      npxy = nx*ny

c     --- 1
c         --- The multiplication b = U  x

      bresul(nusol,ndimso) = bresul(nusol,ndimso)
     +                        *prec(nusol,ndimso,1)

      do i = nusol-1, nusol-nx+2, -1
         bresul(i,ndimso) = (bresul(i,ndimso)
     +                       -prec(i,ndimso,6)*bresul(i+1   ,ndimso)
     +                       )*prec(i,ndimso,1)
      end do

      i = nusol-nx+1

      bresul(i,ndimso) = (bresul(i,ndimso)
     +                    -prec(i,ndimso,6)*bresul(i+1   ,ndimso)
     +                    -prec(i,ndimso,7)*bresul(i+nx-1,ndimso)
     +                    )*prec(i,ndimso,1)

      i = nusol-nx

      bresul(i,ndimso) = (bresul(i,ndimso)
     +                    -prec(i,ndimso,6)*bresul(i+1   ,ndimso)
     +                    -prec(i,ndimso,7)*bresul(i+nx-1,ndimso)
     +                    -prec(i,ndimso,8)*bresul(i+nx  ,ndimso)
     +                    )*prec(i,ndimso,1)

      do i = nusol-nx-1, nusol-npxy+nx+1, -1

         bresul(i,ndimso) = (bresul(i,ndimso)
     +                       -prec(i,ndimso,6)*bresul(i+1   ,ndimso)
     +                       -prec(i,ndimso,7)*bresul(i+nx-1,ndimso)
     +                       -prec(i,ndimso,8)*bresul(i+nx  ,ndimso)
     +                       -prec(i,ndimso,9)*bresul(i+nx+1,ndimso)
     +                       )*prec(i,ndimso,1)
      end do

      do i = nusol-npxy+nx, nusol-npxy+2, -1

         bresul(i,ndimso) = (bresul(i,ndimso)
     +                       -prec(i,ndimso,6)*bresul(i+1   ,ndimso)
     +                       -prec(i,ndimso,7)*bresul(i+nx-1,ndimso)
     +                       -prec(i,ndimso,8)*bresul(i+nx  ,ndimso)
     +                       -prec(i,ndimso,9)*bresul(i+nx+1,ndimso)
     +                       -prec(i,ndimso,15)*bresul(i+npxy-nx,ndimso)
     +                       )*prec(i,ndimso,1)
      end do

      i = nusol-npxy+1

      bresul(i,ndimso) = (bresul(i,ndimso)
     +                    -prec(i,ndimso,6)*bresul(i+1   ,ndimso)
     +                    -prec(i,ndimso,7)*bresul(i+nx-1,ndimso)
     +                    -prec(i,ndimso,8)*bresul(i+nx  ,ndimso)
     +                    -prec(i,ndimso,9)*bresul(i+nx+1,ndimso)
     +                    -prec(i,ndimso,15)*bresul(i+npxy-nx,ndimso)
     +                    -prec(i,ndimso,16)*bresul(i+npxy-1 ,ndimso)
     +                    )*prec(i,ndimso,1)

      i = nusol-npxy

      bresul(i,ndimso) = (bresul(i,ndimso)
     +                    -prec(i,ndimso,6)*bresul(i+1   ,ndimso)
     +                    -prec(i,ndimso,7)*bresul(i+nx-1,ndimso)
     +                    -prec(i,ndimso,8)*bresul(i+nx  ,ndimso)
     +                    -prec(i,ndimso,9)*bresul(i+nx+1,ndimso)
     +                    -prec(i,ndimso,15)*bresul(i+npxy-nx,ndimso)
     +                    -prec(i,ndimso,16)*bresul(i+npxy-1 ,ndimso)
     +                    -prec(i,ndimso,17)*bresul(i+npxy   ,ndimso)
     +                    )*prec(i,ndimso,1)

      do i = nusol-npxy-1, nusol-npxy-nx+1, -1

         bresul(i,ndimso) = (bresul(i,ndimso)
     +                       -prec(i,ndimso,6)*bresul(i+1   ,ndimso)
     +                       -prec(i,ndimso,7)*bresul(i+nx-1,ndimso)
     +                       -prec(i,ndimso,8)*bresul(i+nx  ,ndimso)
     +                       -prec(i,ndimso,9)*bresul(i+nx+1,ndimso)
     +                       -prec(i,ndimso,15)*bresul(i+npxy-nx,ndimso)
     +                       -prec(i,ndimso,16)*bresul(i+npxy-1 ,ndimso)
     +                       -prec(i,ndimso,17)*bresul(i+npxy   ,ndimso)
     +                       -prec(i,ndimso,18)*bresul(i+npxy+1 ,ndimso)
     +                       )*prec(i,ndimso,1)
      end do

      do i = nusol-npxy-nx, nusol-2*npxy+1, -1

         bresul(i,ndimso) = (bresul(i,ndimso)
     +                       -prec(i,ndimso,6)*bresul(i+1   ,ndimso)
     +                       -prec(i,ndimso,7)*bresul(i+nx-1,ndimso)
     +                       -prec(i,ndimso,8)*bresul(i+nx  ,ndimso)
     +                       -prec(i,ndimso,9)*bresul(i+nx+1,ndimso)
     +                       -prec(i,ndimso,15)*bresul(i+npxy-nx,ndimso)
     +                       -prec(i,ndimso,16)*bresul(i+npxy-1 ,ndimso)
     +                       -prec(i,ndimso,17)*bresul(i+npxy   ,ndimso)
     +                       -prec(i,ndimso,18)*bresul(i+npxy+1 ,ndimso)
     +                       -prec(i,ndimso,19)*bresul(i+npxy+nx,ndimso)
     +                       )*prec(i,ndimso,1)
      end do

      do k = 3, nz

c$dir no_recurrence

       do j = 1, npxy

         i = (nz-k)*npxy+j

         bresul(i,ndimso) = bresul(i,ndimso)
     +                      -prec(i,ndimso,15)*bresul(i+npxy-nx,ndimso)
     +                      -prec(i,ndimso,16)*bresul(i+npxy-1 ,ndimso)
     +                      -prec(i,ndimso,17)*bresul(i+npxy   ,ndimso)
     +                      -prec(i,ndimso,18)*bresul(i+npxy+1 ,ndimso)
     +                      -prec(i,ndimso,19)*bresul(i+npxy+nx,ndimso)
       end do

       do j = 1, ny

c$dir no_recurrence

        do m = 1, nx

           i = (nz-k)*npxy+(ny-j)*nx+m
         bresul(i,ndimso) = bresul(i,ndimso)
     +                   -prec(i,ndimso,7)*bresul(i+nx-1,ndimso)
     +                   -prec(i,ndimso,8)*bresul(i+nx  ,ndimso)
     +                   -prec(i,ndimso,9)*bresul(i+nx+1,ndimso)
        end do

        do m = nx, 1, -1

         i = (nz-k)*npxy+(ny-j)*nx+m
         bresul(i,ndimso) = (bresul(i,ndimso)
     +                  -prec(i,ndimso,6)*bresul(i+1,ndimso)
     +                       )*prec(i,ndimso,1)
        end do
       end do
      end do

      end
