      subroutine isvcvc01 ( matrix, solut, rhside, nusol, nconct,
     +                      infmat ,irow )
c ======================================================================
c
c        programmer     Duncan van der Heul
c        version 1.1    date 08-07-1998 Kees V: nusol, nsolut cleaned
c        version 1.0    date 01-04-1998 adaptation of subroutine ismtvc01
c
c   copyright (c) 1998-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     This subroutine performs the vector vector
c     multiplication for one row of the pressure equations
c       A(j,:) x = b, where b is a scalar and x is a vector
c       Warning:  the matrix A can be extended with zero elements referring
c                 to vector elements outside the region of interest
c                 Therefore it is necessary that positions in the memory
c                 corresponding to solut -ni to solut+ni contain real
c                 values  (in 2D case and ??? in 3D case )
c **********************************************************************
c
c                       KEYWORDS
c
c      multiplication
c      matrix
c      vector
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nusol, nconct, infmat(*), irow
      double precision matrix(nusol, nconct),
     +                 rhside,  solut(*)

c     infmat    i    Integer information for the matrix
c     irow      i    Loop variable for rows
c     matrix    i    reserved
c     nconct    i    Number of connections in a row of the matrix
c     nusol     i    Number of unknowns for one degree of freedom in
c                    one block
c     rhside    o    reserved
c     solut     i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer k, ishift

c     ishift    Shift in i-direction
c     k         General loop variable
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c     Multiply the irowth row of the matrix with the vector solut
c     The vector vector multiplication is defined by:  a   x  = f
c                                                       ij  j
c
c
c                (j)    (j)   (j)
c               f    = d     u          (d is diagonal)
c                       ii    i
c
c
c            For k = 2 (1) nconct
c
c
c                  (j)     (j)    (j)   (j)
c                 f    =  f    + a     u
c                                 i l   l
c                                    k   k
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Multiply by diagonal

            rhside = matrix(irow,1) * solut(irow)

c           --- Multiply rest of matrix

c               --- From the full range i=1,nusol we should deal only the cases
c               when
c               1 <= i+ishift + (j-1)*nusol <= nusol*ndegfd

c               --- Loop over the connections

            do 210 k = 2, nconct

                ishift = infmat(k)

                if(irow+ishift.ge.1.and.irow+ishift.le.nusol) then

                    rhside = rhside + matrix(irow,k) *
     +                       solut(irow+ishift)
                end if

210         continue

      end
