      subroutine prevc(n,x,b,matrix,ndim,nconct,precon,
     1           nprec,infmat,mkind)
c ==================================================================
c
c
c     programmer       Kees Vuik
c
c     version 2.0      24-01-94 Kees K. change for new structure
c                  
c     version 1.1      date 19-05-1992
c     developed at     Convex,HP700
c
c ******************************************************************
c
c                       DESCRIPTION
c
c     Prevc multiplies the vector x with a preconditioner.
c
c ******************************************************************
c
c                       KEYWORDS
c
c     linear_solver
c     preconditioner
c
c ******************************************************************
c
c                       INPUT / OUTPUT   PARAMETERS
c
c
c     b       o   The output vector which is the preconditioner times
c                 the vector x.
c
c     infmat  i   Array which describes the structure of the matrix.
c       
c     matrix  i   Double precision array in which the matrix of the
c                 linear system of equations is stored.
c
c     mkind   i   The kind of the preconditioner required.
c
c     n       i   The length of the solution vector.
c
c     nconct  i   The number of non-zero diagonals of MATRIX.
c
c     ndim    i   The dimension of the space (ndim =2 or 3).
c
c     nprec   i   Maximum number of diagonals in PRECON.
c
c     precon  i   Double precision array in which a preconditioning
c                 matrix might be stored, of length NPREC * NUSOL. It is
c                 assumed that PRECON has a similar structure as MATRIX.
c
c     x       i   The input vector.
c
c ******************************************************************
c
c                       COMMON BLOCKS
c
c
c ******************************************************************
c
c                       LOCAL PARAMETERS
c
c     i       loop counter
c
c ******************************************************************
c
c                       SUBROUTINES CALLED
c
c     dcopy       Copy a vector in another vector.
c
c     diagmu      Computes the multiplication of a diagonal preconditioner
c                 stored in array precon and a vector.
c
c     dinvl       Computes the multiplication of the inverse of a lower
c                 triangular matrix stored in arrays matrix and precon
c                 and a vector. Precon should be filled by dmlu.f.
c
c     dinvl2      Computes the multiplication of the inverse of a lower
c                 triangular matrix stored in arrays matrix and precon
c                 and a vector. Precon should be filled by dmlu2.f.
c
c     dinvl3      Computes the multiplication of the inverse of a lower
c                 triangular matrix stored in array precon and a vector.
c                 Precon should be filled by dmlu3.f.
c
c     dinvu       Computes the multiplication of the inverse of an upper
c                 triangular matrix stored in arrays matrix and precon
c                 and a vector. Precon should be filled by dmlu.f.
c
c     dinvu2      Computes the multiplication of the inverse of an upper
c                 triangular matrix stored in arrays matrix and precon
c                 and a vector. Precon should be filled by dmlu2.f.
c
c     dinvu3      Computes the multiplication of the inverse of an upper 
c                 triangular matrix stored in array precon and a vector.
c                 Precon should be filled by dmlu3.f.
c
c ******************************************************************
c
c                       I/O
c
c ******************************************************************
c
c                       ERROR MESSAGES
c
c ******************************************************************
c
c                       PSEUDO CODE
c
c     The subroutine prevc is used to multiply x with a preconditioner
c     and store the result in b. The choice of the preconditioner is
c     given by mkind.
c
c ==================================================================
c
c     Parameters:
c
      integer          n,ndim,nconct,nprec,infmat(*),mkind,i
      double precision x(*),b(*),matrix(*), precon(*)
      if(mkind.eq.0) then
c
c     *** no preconditioner is used
c
         call dcopy(n,x,1,b,1)
      endif
      if(abs(mkind).eq.1) then
c
c     *** use a diagonal scaling preconditioning matrix
c
         call diagmu(n,x,b,precon,nprec)
      endif
      if(mkind.eq.2) then
c
c     *** use a (M)ILU preconditioner, with D = I.
c
         call dinvl(x,b,matrix,n/ndim,ndim,
     1              nconct,precon,nprec,infmat)
         call dinvu(b,b,matrix,n/ndim,ndim,
     1              nconct,precon,nprec,infmat)
      endif
      if(mkind.eq.-2) then
c
c     *** use a (M)ILU preconditioner, with D is not equal to I.
c
         call dinvl2(x,b,matrix,n/ndim,ndim,
     1               nconct,precon,nprec,infmat)
c
c        Multiplication with D
c
	 do i = 1, n
            b(i) = b(i)*precon(i)
         enddo
         call dinvu2(b,b,matrix,n/ndim,ndim,
     1               nconct,precon,nprec,infmat)
      endif
      if(abs(mkind).eq.3.and.infmat(8).ge.3) then
c
c     *** use an incomplete LU decomposition of A such that
c         the sparseness pattern of L and U is the same as A.
c
         call dinvl3(x,b,matrix,n/ndim,ndim,
     1               nconct,precon,nprec,infmat)
         call dinvu3(b,b,matrix,n/ndim,ndim,
     1               nconct,precon,nprec,infmat)
      endif
      return
      end
