      subroutine uscoefhydr(a,b,c,rk2,R,rn)
c ======================================================================
c
c        programmer     Philipe Reynier
c        version 1.0    date 06-07-1999
c
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Subroutine to compute the coefficient for the hydrazine
c   thermodynamic model
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      double precision a, b, c, rk2, R, rn

c a, b, c            coefficient use for the equation of state
c                      and the internal energy equation
c rk2                constant uses in the internal energy equation
c R                  constant for the state equation
c rn                 temperature exponent
c                         For a description see the programmers guide
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      double precision Vc, Tc, RLc, Pc, rn1, rn2

c     Pc
c     rlc
c     rn1
c     rn2
c     Tc
c     Vc
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

c     --- Coordinates at the critical point

      Vc = 3.74d-3
      Tc = 653.15d0
      Pc = 1.47d+7

c     --- Value of rn exponent for the Clausius modified equation
c         P = RT/(v-a) - C/(T^n*(v+b)^2)

      rn1 = 0.67d0
      rn2 = 0.7d0
      rn = rn2

c     --- Calculation of a, b, c, K2 and R for thermodynamic model

      R = 8.31434d0/32.04524d-3
      RLc = (Pc*Vc)/(R*Tc)
      a = Vc * (1d0 - 1d0/(4d0*RLc))
      b = Vc * (3d0/(8d0*RLc) - 1d0)
      c = (27d0/64d0)*(R**2d0)*(Tc**(rn+2d0))/Pc
      rk2 = 11d0*R*Tc

      end
