      subroutine uscomptemp(eddy,solut,ni,nj,nk,nvirtual,ndim,ndegfd,
     +                      rincom)
c ======================================================================
c
c        programmer     Philipe Reynier
c        version 1.0    date 06-07-1999
c
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Sample user subrutine to calculate the temperature. first rho is
c    expressed in function of e and T. Then T is calculated using the
c    Newton-Raphson method
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni,nj,nk,ndim,nvirtual,ndegfd
      double precision rincom(*),
     +                 solut(1-nvirtual:1+ni+nvirtual,
     +                       1-nvirtual:1+nj+nvirtual,1:ndegfd),
     +                 eddy(1-nvirtual:1+ni+nvirtual,
     +                      1-nvirtual:1+nj+nvirtual,1:7),
     +                 a, b, c, rk2, R, rn, pphi, pphiint, T

c     eddy           o    array EDDY contains the values of the
c                         eddy viscosity of that block in the centroids
c                         of the elements. The storage of such
c                         an array EDDY is identical to that of all other
c                         coefficients.
c     ndegfd         i    Number of degrees of freedom
c     ndim           i    Dimension of space (2 or 3)
c     ni             i    Number of non-virtual cells in the i-direction
c                         (one block)
c     nj             i    Number of non-virtual cells in the j-direction
c                         (one block)
c     nk             i    Number of non-virtual cells in the k-direction
c                         (one block)
c     nvirtual       i    Number of rows of virtual cells
c     rincom         i    Array RINCOM is part of array IINPUT and hence
c                         coupled to one block only. It contains information
c                         with respect to the compressible flow
c                         For a description see the programmers guide
c     solut          i    Solution array
c                         For a description see the programmers guide
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      double precision alp1, alp2, alp3, const, dalp3, den,
     +                 dsteq, dT, dv, steq, Trn, v
      integer i, j

c     i              General loop variable
c     j              General loop variable
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c         usccoefhydr
c         uspcompphiint
c         uspcompphi
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      call uscoefhydr(a,b,c,rk2,R,rn)

      const = c*(rn+1d0)

c     --- Calculation of the temperature
c         solut(i,j,3) is the pressure
c         solut(i,j,4) is the internal energy
c         eddy(i,j,7) is the temperature

      do i=1-nvirtual,1+ni+nvirtual
         do j=1-nvirtual,1+nj+nvirtual
            T = 310d0
            dT = 1d0
 100        Trn = T**rn

c Computation of the integral of phi

            call uspcompphiint(T,pphiint)

c Computation of of phi

            call uspcompphi(T,pphi)

c v (=1/rho) and dv are expressed in function of e and T

            den = (pphiint + rk2 - solut(i,j,4))*Trn
            v = const/den - b
            dv = - const*(rn*den/T + Trn*pphi)/(den**2d0)

c state equation
            alp1 = v - a
            alp2 = v + b
            alp3 = Trn*(alp2**2d0)
            dalp3 = Trn*alp2*(rn*alp2/T + 2*dv)
            steq = solut(i,j,3)*alp1*alp3 - R*T*alp3 + c*alp1
            dsteq = solut(i,j,3)*(dv*alp3 + alp1*dalp3)
     +             - R*(alp3 + T*dalp3) + c*dv
            dT = - steq/dsteq
            T = T + dT
            if (dT.gt.(1.d-3)) goto 100
            eddy(i,j,7) = T
         end do
      end do
      end

