      subroutine isrecmpi ( ibufptr, icount, itype, isource, icontext )
c ======================================================================
c
c        Programmer     Jason Frank
c
c     copyright (c) 1994-1996  "TU-Delft"
c     Permission to copy or distribute this software or documentation
c     in hard copy or soft copy granted only by written license
c     obtained from "TU-Delft"
c     All rights reserved. No part of this publication may be
c     reproduced, stored in a retrieval system (e.g., in memory, disk,
c     or core) or be transmitted by any means, electronic, mechanical,
c     photocopy, recording, or otherwise, without written Permission
c     from the publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Wrapper for MPI_RECV, checks first for special messages
c     This implementation is completely based on MPI
c **********************************************************************
c
c                       KEYWORDS
c
c     parallel
c     receive
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ibufptr, icount, itype, isource, icontext

c     ibufptr  i    pointer to first element of array to be received
c     icontext i    the communication context, (message tag)
c     icount   io   the length of array to be received
c     isource  io   rank of the source process.  NOTE: THIS VALUE WILL
c                   BE MODIFIED.  It must be a variable on input
c     itype    i    MPI datatype
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
cpar      include 'mpif.h'
cpar      integer ierr, istatus(MPI_STATUS_SIZE)

c     ierr      error status of an MPI call ( .eq.MPI_SUCCESS if ok)
c     istatus   MPI status array
c **********************************************************************
c
c                       I/O
c
c     Information is received on host or node
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message file
c     ISERRSUB  Error print routine
c     ISRCSPEC  Check for and handle special messages
c     MPI_GET_COUNT  Determine how many items were actually received
c     MPI_RECV  Immediately receives the data in the active MPI message buffer
c **********************************************************************
c
c                       ERROR MESSAGES
c
c    1750   MPI error
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
cpar      call iseropen ( 'isrecmpi' )

c     --- Wait for a message

cpar      call MPI_PROBE ( MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD,
cpar     +      istatus, ierr )

cpar      if ( ierr.ne.MPI_SUCCESS ) then
cpar         call iserrint ( ierr, 1 )
cpar         call iserrsub ( 1750, 1, 0, 0 )
cpar      end if

c     --- Check if there are any special messages
cpar      call isrcspec

c     ---  Receive regular message
cpar      call MPI_RECV ( ibufptr, icount, itype, isource, icontext,
cpar     +   MPI_COMM_WORLD, istatus, ierr )

cpar      if ( ierr.ne.MPI_SUCCESS ) then
cpar         call iserrint ( ierr, 1 )
cpar         call iserrsub ( 1750, 1, 0, 0 )
cpar      end if

cpar      call MPI_GET_COUNT ( istatus, itype, icount, ierr )

cpar      if ( ierr.ne.MPI_SUCCESS ) then
cpar         call iserrint ( ierr, 1 )
cpar         call iserrsub ( 1750, 1, 0, 0 )
cpar      end if

cpar      isource = istatus (MPI_SOURCE)

cpar      call iserclos ( 'isrecmpi' )
      end
