      subroutine iswaitall ( ireqcnt, ireq )
c ======================================================================
c
c        Programmer     Jason Frank
c
c     copyright (c) 1994-1996  "TU-Delft"
c     Permission to copy or distribute this software or documentation
c     in hard copy or soft copy granted only by written license
c     obtained from "TU-Delft"
c     All rights reserved. No part of this publication may be
c     reproduced, stored in a retrieval system (e.g., in memory, disk,
c     or core) or be transmitted by any means, electronic, mechanical,
c     photocopy, recording, or otherwise, without written Permission
c     from the publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Wait for all sends to complete
c     This implementation is completely based on MPI
c **********************************************************************
c
c                       KEYWORDS
c
c     parallel
c     send
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ireqcnt, ireq(*)

c     ireqcnt  i    number of requests in array ireq
c     ireq     i    array of MPI requests
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
cpar      include 'mpif.h'
cpar      integer ierr
cpar      integer istatus (MPI_STATUS_SIZE, ireqcnt)

c     ierr      error status of an MPI call ( .eq.MPI_SUCCESS if ok)
c     istatus   array of MPI status arrays (one per request)
c **********************************************************************
c
c                       I/O
c
c     Information is sent to host or node 
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRINT  Put integer value in error message file
c     ISERRSUB  Error message subroutine
c     MPI_SEND  Immediately sends the data in the active MPI message buffer
c **********************************************************************
c
c                       ERROR MESSAGES
c
c    1750   MPI error
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen ( 'iswaitall' )

cpar      call MPI_WAITALL ( ireqcnt, ireq, istatus, ierr)
cpar      if ( ierr.ne.MPI_SUCCESS ) then
cpar         call iserrint ( ierr, 1 )
cpar         call iserrsub ( 1750, 1, 0, 0 )
cpar      end if

      call iserclos ( 'iswaitall' )
      end
