      subroutine iserrspr ( ierrnum, nints, nreal, nchar )
c ======================================================================
c
c        Programmer     Guus Segal
c        version 1.3    date 17-08-1998 Jason:  convert to MPI
c        version 1.2    date 07-12-1994 (Use the pvm include file)
c        version 1.1    date 26-09-1994 (Correction call to pvmfperror)
c        version 1.0    date 31-01-1994 (Extracted from the SEPRAN subroutine
c                                        errsub version 2.2)
c
c
c     copyright (c) 1994  "Ingenieursbureau SEPRA"
c     Permission to copy or distribute this software or documentation
c     in hard copy or soft copy granted only by written license
c     obtained from "Ingenieursbureau SEPRA"
c     All rights reserved. No part of this publication may be
c     reproduced, stored in a retrieval system (e.g., in memory, disk,
c     or core) or be transmitted by any means, electronic, mechanical,
c     photocopy, recording, or otherwise, without written Permission
c     from the publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     print an error message in case of a call by a tool for parallel
c     computations
c **********************************************************************
c
c                       KEYWORDS
c
c     error_manipulation
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ierrnum, nints, nreal, nchar

c     ierrnum  i    Error number
c     nchar    i    Number of strings in a message
c     nints    i    Number of integers in a message
c     nreal    i    Number of reals in a message
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'
      include 'IScommon/cisnasch'
      include 'IScommon/cismessg'
      include 'IScommon/cismessc'
      include 'IScommon/ciscom'
      include 'IScommon/cisparal'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      character * 160 text
      logical errormpi
      integer ierr, msgtype, lastps, ipos
cpar      include 'mpif.h'

c     errormpi  Indicates if a fatal MPI error has occurred (true) or not
c     ierr      Return code of a subroutine (<0 is error)
c     ipos      Position in array
c     lastps    Last position in array
c     msgtype   Tag for MPI, may be used for identification
c     text      String in which a text is stored
c **********************************************************************
c
c                       I/O
c
c     iserrspr writes to the default output device if it is called at the host
c     If iserrspr is called at the node information is sent to the host
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERMESS  Prints a message, stored in the Error Text File.
c     ISPACK    "Safe" wrapper for MPI_PACK
c     MPI_SEND  Immediately sends the data in the active MPI message buffer
c     SYSTEM    Execute standard unix command
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     Number of errors = maxerr , STOP.
c **********************************************************************
c
c                       PSEUDO CODE
c
c     If ( nodenr=0 ) then
c        Increment NERROR (number of errors);
c        Call ISERMESS;
c        if (number of errors = maxerr) then
c           Print message;
c           STOP;
c     else if ( nodenr>0 and ierrnum < 1750 ) then
c        Put information of error message in array info
c        send information of error to host
c     else ( nodenr>0 and ierrnum >= 1750 ) then
c        call pvm error subroutine
c        mail information to user that a fatal error has appeared
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
cpar      errormpi = .false.
      if ( inodenr.eq.0 ) then

c     --- Program runs at host

         nerror = nerror + 1

c        --- Print the message, stored in the Error Text File.

         call isermess ( ierrnum, nints, nreal, nchar, 'ERROR' )
         ierror = ierrnum

      else if ( ierrnum.ne.1750 ) then

c     --- Program runs at node
c         Put information of error message in array info

cpar         info(1) = -1
cpar         info(2) = ierrnum
cpar         info(3) = nints
cpar         info(4) = nreal
cpar         info(5) = nchar
cpar         info(10) = 1

c        --- reset send buffer

cpar         isndpos = 1

c        --- send information of error by issend to host

cpar         call ispack ( info, 10, MPI_INTEGER, isndbuf, mxmsglen,
cpar     +        isndpos, MPI_COMM_WORLD, ierr)
cpar         if ( ierr.ne.MPI_SUCCESS ) then
cpar            errormpi = .true.
cpar            go to 100
cpar         end if
cpar         if ( nints.gt.0 ) call ispack ( ints, nints, MPI_INTEGER,
cpar     +        isndbuf, mxmsglen, isndpos, MPI_COMM_WORLD, ierr)
cpar         if ( ierr.ne.MPI_SUCCESS ) then
cpar            errormpi = .true.
cpar            go to 100
cpar         end if
cpar         if ( nreal.gt.0 ) call ispack ( reals, nreal,
cpar     +       MPI_DOUBLE_PRECISION, isndbuf, mxmsglen, isndpos,
cpar     +       MPI_COMM_WORLD, ierr)
cpar         if ( ierr.ne.MPI_SUCCESS ) then
cpar            errormpi = .true.
cpar            go to 100
cpar         end if
cpar         if ( nchar.gt.0 ) call ispack ( chars, nchar,
cpar     +         MPI_CHARACTER, isndbuf, mxmsglen, isndpos,
cpar     +         MPI_COMM_WORLD, ierr )
cpar         if ( ierr.ne.MPI_SUCCESS ) then
cpar            errormpi = .true.
cpar            go to 100
cpar         end if
cpar         msgtype = 1
cpar         call MPI_SEND ( isndbuf, isndpos, MPI_PACKED, 0, msgtype,
cpar     +        MPI_COMM_WORLD, ierr )
cpar         if ( ierr.ne.MPI_SUCCESS ) then
cpar            errormpi = .true.
cpar            go to 100
cpar         end if
      else

c     --- Error number = 1750, i.e. MPI error

cpar         errormpi = .true.
cpar         ierr = ints(1)

      end if
cpar100   if ( errormpi ) then

c     --- MPI error, fatal for process

cpar         write (6,*) 'fatal MPI error ', ierr, ' (function iserrspr)'

c        --- Mail message to user

cpar         text(1:4) = 'mail'
cpar         lastps=index(usrnam,' ')-1
cpar         if ( lastps.eq.-1 ) lastps = len(usrnam)
cpar         ipos = 4+lastps
cpar         text(5:ipos) = usrnam(1:lastps)
cpar         ipos = ipos+1
cpar         text(ipos:ipos) = '<'
cpar         ipos = ipos+1
cpar         lastps=index(sphome,' ')-1
cpar         if ( lastps.eq.-1 ) lastps = len(sphome)
cpar         text(ipos:ipos+lastps-1) = sphome
cpar         ipos = ipos+lastps
cpar         text(ipos:ipos+lastps+13) = '/instal/mess.1'
cpar         ipos = ipos+13
cpar         call system ( text )
cpar         stop

cpar      end if
      end
